
DROP SEQUENCE seq_va_admin_privileges CASCADE;
DROP TABLE va_admin_privileges;

CREATE SEQUENCE seq_va_admin_privileges START 3;
CREATE TABLE va_admin_privileges (
  privilege_id INT4 NOT NULL DEFAULT nextval('seq_va_admin_privileges'),
  is_hidden SMALLINT default '0',
  support_privilege SMALLINT default '0',
  privilege_name VARCHAR(64) NOT NULL,
  user_types_all SMALLINT default '1',
  non_logged_users SMALLINT default '1',
  user_types_ids TEXT
  ,PRIMARY KEY (privilege_id));



DROP SEQUENCE seq_va_admin_privileges_settings CASCADE;
DROP TABLE va_admin_privileges_settings;

CREATE SEQUENCE seq_va_admin_privileges_settings START 1;
CREATE TABLE va_admin_privileges_settings (
  privilege_id INT4 NOT NULL default '0',
  block_name VARCHAR(64) NOT NULL,
  permission SMALLINT
  ,PRIMARY KEY (privilege_id,block_name));



DROP SEQUENCE seq_va_admins CASCADE;
DROP TABLE va_admins;

CREATE SEQUENCE seq_va_admins START 2;
CREATE TABLE va_admins (
  admin_id INT4 NOT NULL DEFAULT nextval('seq_va_admins'),
  admin_name VARCHAR(255) NOT NULL,
  nickname VARCHAR(32),
  admin_alias VARCHAR(16),
  privilege_id INT4 NOT NULL default '0',
  is_hidden SMALLINT default '0',
  email VARCHAR(255) NOT NULL,
  cell_phone VARCHAR(32),
  login VARCHAR(64) NOT NULL,
  password VARCHAR(32) NOT NULL,
  personal_image VARCHAR(255),
  signature TEXT,
  last_order_id INT4 default '0',
  exported_order_id INT4 default '0',
  exported_order_fields TEXT,
  last_user_id INT4 default '0',
  exported_email_id VARCHAR(50),
  exported_user_id INT4 default '0',
  exported_email_fields VARCHAR(50),
  exported_user_fields TEXT,
  imported_email_fields VARCHAR(50),
  imported_user_fields TEXT,
  exported_item_fields TEXT,
  imported_item_fields TEXT,
  edited_item_fields TEXT,
  last_visit_date TIMESTAMP,
  support_online_date TIMESTAMP,
  is_generate_big_image SMALLINT default '0',
  is_generate_small_image SMALLINT default '0',
  login_attempts INT4 default '0',
  login_failed TIMESTAMP,
  access_code VARCHAR(16),
  access_added TIMESTAMP,
  access_attempts INT4 default '0',
  access_failed TIMESTAMP
  ,PRIMARY KEY (admin_id));

CREATE INDEX va_admins_login ON va_admins (login);
CREATE INDEX va_admins_password ON va_admins (password);


DROP SEQUENCE seq_va_admins_login_stats CASCADE;
DROP TABLE va_admins_login_stats;

CREATE SEQUENCE seq_va_admins_login_stats START 13;
CREATE TABLE va_admins_login_stats (
  stat_id INT4 NOT NULL DEFAULT nextval('seq_va_admins_login_stats'),
  admin_id INT4 default '0',
  login_status SMALLINT default '0',
  ip_address VARCHAR(32),
  forwarded_ips VARCHAR(255),
  date_added TIMESTAMP
  ,PRIMARY KEY (stat_id));

CREATE INDEX va_admins_login_stats_admin_id ON va_admins_login_stats (admin_id);


DROP SEQUENCE seq_va_admins_settings CASCADE;
DROP TABLE va_admins_settings;

CREATE SEQUENCE seq_va_admins_settings START 1;
CREATE TABLE va_admins_settings (
  admin_id INT4 NOT NULL default '0',
  setting_name VARCHAR(64) NOT NULL,
  setting_value TEXT
  ,PRIMARY KEY (admin_id,setting_name));



DROP SEQUENCE seq_va_ads_assigned CASCADE;
DROP TABLE va_ads_assigned;

CREATE SEQUENCE seq_va_ads_assigned START 1;
CREATE TABLE va_ads_assigned (
  item_id INT4 NOT NULL default '0',
  category_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (item_id,category_id));



DROP SEQUENCE seq_va_ads_categories CASCADE;
DROP TABLE va_ads_categories;

CREATE SEQUENCE seq_va_ads_categories START 4;
CREATE TABLE va_ads_categories (
  category_id INT4 NOT NULL DEFAULT nextval('seq_va_ads_categories'),
  language_code VARCHAR(2),
  parent_category_id INT4 default '0',
  alias_category_id INT4 default '0',
  category_path VARCHAR(255) NOT NULL,
  category_name VARCHAR(255),
  friendly_url VARCHAR(255),
  sites_all SMALLINT default '1',
  publish_price FLOAT4 default '0',
  access_level SMALLINT default '7',
  guest_access_level SMALLINT default '7',
  admin_access_level SMALLINT default '7',
  category_order INT4 default '0',
  short_description TEXT,
  full_description TEXT,
  image_small VARCHAR(255),
  image_large VARCHAR(255),
  total_views INT4 default '0',
  total_ads INT4 default '0',
  total_subcategories INT4 default '0'
  ,PRIMARY KEY (category_id));

CREATE INDEX va_ads_categories_admin_acc_1 ON va_ads_categories (admin_access_level);
CREATE INDEX va_ads_categories_category_path ON va_ads_categories (category_path);
CREATE INDEX va_ads_categories_friendly_url ON va_ads_categories (friendly_url);
CREATE INDEX va_ads_categories_guest_acc_2 ON va_ads_categories (guest_access_level);
CREATE INDEX va_ads_categories_parent_ca_3 ON va_ads_categories (parent_category_id);
CREATE INDEX va_ads_categories_sites_all ON va_ads_categories (sites_all);
CREATE INDEX va_ads_categories_subscript_4 ON va_ads_categories (access_level);


DROP SEQUENCE seq_va_ads_categories_sites CASCADE;
DROP TABLE va_ads_categories_sites;

CREATE SEQUENCE seq_va_ads_categories_sites START 1;
CREATE TABLE va_ads_categories_sites (
  category_id INT4 NOT NULL default '0',
  site_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (category_id,site_id));



DROP SEQUENCE seq_va_ads_categories_subscriptions CASCADE;
DROP TABLE va_ads_categories_subscriptions;

CREATE SEQUENCE seq_va_ads_categories_subscriptions START 1;
CREATE TABLE va_ads_categories_subscriptions (
  category_id INT4 NOT NULL default '0',
  subscription_id INT4 NOT NULL default '0',
  access_level SMALLINT default '0'
  ,PRIMARY KEY (category_id,subscription_id));



DROP SEQUENCE seq_va_ads_categories_types CASCADE;
DROP TABLE va_ads_categories_types;

CREATE SEQUENCE seq_va_ads_categories_types START 1;
CREATE TABLE va_ads_categories_types (
  category_id INT4 NOT NULL default '0',
  user_type_id INT4 NOT NULL default '0',
  access_level SMALLINT default '0'
  ,PRIMARY KEY (category_id,user_type_id));



DROP SEQUENCE seq_va_ads_days CASCADE;
DROP TABLE va_ads_days;

CREATE SEQUENCE seq_va_ads_days START 9;
CREATE TABLE va_ads_days (
  days_id INT4 NOT NULL DEFAULT nextval('seq_va_ads_days'),
  days_number INT4,
  days_title VARCHAR(64),
  publish_price FLOAT4
  ,PRIMARY KEY (days_id));



DROP SEQUENCE seq_va_ads_features CASCADE;
DROP TABLE va_ads_features;

CREATE SEQUENCE seq_va_ads_features START 1;
CREATE TABLE va_ads_features (
  feature_id INT4 NOT NULL DEFAULT nextval('seq_va_ads_features'),
  item_id INT4 default '0',
  group_id INT4 default '0',
  feature_name VARCHAR(255),
  feature_value VARCHAR(255)
  ,PRIMARY KEY (feature_id));

CREATE INDEX va_ads_features_group_id ON va_ads_features (group_id);
CREATE INDEX va_ads_features_item_id ON va_ads_features (item_id);


DROP SEQUENCE seq_va_ads_features_default CASCADE;
DROP TABLE va_ads_features_default;

CREATE SEQUENCE seq_va_ads_features_default START 1;
CREATE TABLE va_ads_features_default (
  feature_id INT4 NOT NULL DEFAULT nextval('seq_va_ads_features_default'),
  type_id INT4 default '0',
  group_id INT4 default '0',
  feature_name VARCHAR(50),
  feature_value TEXT
  ,PRIMARY KEY (feature_id));

CREATE INDEX va_ads_features_default_group_id ON va_ads_features_default (group_id);
CREATE INDEX va_ads_features_default_ite_5 ON va_ads_features_default (type_id);


DROP SEQUENCE seq_va_ads_features_groups CASCADE;
DROP TABLE va_ads_features_groups;

CREATE SEQUENCE seq_va_ads_features_groups START 4;
CREATE TABLE va_ads_features_groups (
  group_id INT4 NOT NULL DEFAULT nextval('seq_va_ads_features_groups'),
  group_order INT4 NOT NULL default '1',
  group_name VARCHAR(255)
  ,PRIMARY KEY (group_id));



DROP SEQUENCE seq_va_ads_hot_days CASCADE;
DROP TABLE va_ads_hot_days;

CREATE SEQUENCE seq_va_ads_hot_days START 6;
CREATE TABLE va_ads_hot_days (
  days_id INT4 NOT NULL DEFAULT nextval('seq_va_ads_hot_days'),
  days_number INT4,
  days_title VARCHAR(64),
  publish_price FLOAT4
  ,PRIMARY KEY (days_id));



DROP SEQUENCE seq_va_ads_images CASCADE;
DROP TABLE va_ads_images;

CREATE SEQUENCE seq_va_ads_images START 1;
CREATE TABLE va_ads_images (
  image_id INT4 NOT NULL DEFAULT nextval('seq_va_ads_images'),
  item_id INT4 NOT NULL default '0',
  image_small VARCHAR(255),
  small_width INT4,
  small_height INT4,
  image_large VARCHAR(255),
  image_title VARCHAR(255),
  image_description TEXT
  ,PRIMARY KEY (image_id));

CREATE INDEX va_ads_images_item_id ON va_ads_images (item_id);


DROP SEQUENCE seq_va_ads_items CASCADE;
DROP TABLE va_ads_items;

CREATE SEQUENCE seq_va_ads_items START 1;
CREATE TABLE va_ads_items (
  item_id INT4 NOT NULL DEFAULT nextval('seq_va_ads_items'),
  type_id INT4 NOT NULL default '0',
  is_shown SMALLINT default '1',
  is_shown_internal SMALLINT default '1',
  is_paid SMALLINT default '0',
  publish_price FLOAT4 default '0',
  language_code VARCHAR(2),
  item_title VARCHAR(255) NOT NULL,
  friendly_url VARCHAR(255),
  item_order INT4 default '0',
  user_id INT4 default '0',
  admin_id INT4 default '0',
  days_run INT4,
  date_start TIMESTAMP,
  date_end TIMESTAMP,
  date_added TIMESTAMP,
  date_updated TIMESTAMP,
  image_small VARCHAR(255),
  image_large VARCHAR(255),
  is_special SMALLINT default '0',
  special_description TEXT,
  special_days_run INT4,
  special_date_start TIMESTAMP,
  special_date_end TIMESTAMP,
  is_hot SMALLINT default '0',
  hot_description TEXT,
  hot_days_run INT4,
  hot_date_start TIMESTAMP,
  hot_date_end TIMESTAMP,
  short_description TEXT,
  full_description TEXT,
  price FLOAT4 default '0',
  currency_code VARCHAR(4),
  quantity INT4 default '0',
  availability VARCHAR(255),
  location_info TEXT,
  location_city VARCHAR(128),
  location_postcode VARCHAR(16),
  location_state_id INT4 default '0',
  location_state VARCHAR(8),
  location_country_id INT4 default '0',
  location_country VARCHAR(4),
  is_approved SMALLINT default '0',
  is_compared SMALLINT default '0',
  total_views INT4 default '0'
  ,PRIMARY KEY (item_id));

CREATE INDEX va_ads_items_currency_id ON va_ads_items (currency_code);
CREATE INDEX va_ads_items_friendly_url ON va_ads_items (friendly_url);
CREATE INDEX va_ads_items_hot_date_end ON va_ads_items (hot_date_end);
CREATE INDEX va_ads_items_hot_date_start ON va_ads_items (hot_date_start);
CREATE INDEX va_ads_items_is_hot ON va_ads_items (is_hot);
CREATE INDEX va_ads_items_is_paid ON va_ads_items (is_paid);
CREATE INDEX va_ads_items_is_shown ON va_ads_items (is_shown);
CREATE INDEX va_ads_items_item_title ON va_ads_items (item_title);
CREATE INDEX va_ads_items_location_country_id ON va_ads_items (location_country_id);
CREATE INDEX va_ads_items_location_postcode ON va_ads_items (location_postcode);
CREATE INDEX va_ads_items_location_state_id ON va_ads_items (location_state_id);
CREATE INDEX va_ads_items_special_date_end ON va_ads_items (special_date_end);
CREATE INDEX va_ads_items_special_date_start ON va_ads_items (special_date_start);
CREATE INDEX va_ads_items_type_id ON va_ads_items (type_id);


DROP SEQUENCE seq_va_ads_properties CASCADE;
DROP TABLE va_ads_properties;

CREATE SEQUENCE seq_va_ads_properties START 1;
CREATE TABLE va_ads_properties (
  property_id INT4 NOT NULL DEFAULT nextval('seq_va_ads_properties'),
  item_id INT4 NOT NULL default '0',
  property_name VARCHAR(255) NOT NULL,
  property_value TEXT
  ,PRIMARY KEY (property_id));

CREATE INDEX va_ads_properties_item_id ON va_ads_properties (item_id);


DROP SEQUENCE seq_va_ads_properties_default CASCADE;
DROP TABLE va_ads_properties_default;

CREATE SEQUENCE seq_va_ads_properties_default START 1;
CREATE TABLE va_ads_properties_default (
  property_id INT4 NOT NULL DEFAULT nextval('seq_va_ads_properties_default'),
  type_id INT4 NOT NULL default '0',
  property_name VARCHAR(255) NOT NULL,
  property_value TEXT
  ,PRIMARY KEY (property_id));

CREATE INDEX va_ads_properties_default_i_6 ON va_ads_properties_default (type_id);


DROP SEQUENCE seq_va_ads_special_days CASCADE;
DROP TABLE va_ads_special_days;

CREATE SEQUENCE seq_va_ads_special_days START 6;
CREATE TABLE va_ads_special_days (
  days_id INT4 NOT NULL DEFAULT nextval('seq_va_ads_special_days'),
  days_number INT4,
  days_title VARCHAR(64),
  publish_price FLOAT4
  ,PRIMARY KEY (days_id));



DROP SEQUENCE seq_va_ads_types CASCADE;
DROP TABLE va_ads_types;

CREATE SEQUENCE seq_va_ads_types START 3;
CREATE TABLE va_ads_types (
  type_id INT4 NOT NULL DEFAULT nextval('seq_va_ads_types'),
  type_name VARCHAR(50)
  ,PRIMARY KEY (type_id));



DROP SEQUENCE seq_va_albums CASCADE;
DROP TABLE va_albums;

CREATE SEQUENCE seq_va_albums START 1;
CREATE TABLE va_albums (
  album_id INT4 NOT NULL DEFAULT nextval('seq_va_albums'),
  album_name VARCHAR(255),
  name_first VARCHAR(2),
  album_type VARCHAR(32),
  album_date TIMESTAMP,
  friendly_url VARCHAR(255),
  image_tiny VARCHAR(255),
  image_small VARCHAR(255),
  image_large VARCHAR(255),
  image_super VARCHAR(255),
  short_description TEXT,
  full_description TEXT
  ,PRIMARY KEY (album_id));

CREATE INDEX va_albums_name_first ON va_albums (name_first);


DROP SEQUENCE seq_va_albums_authors CASCADE;
DROP TABLE va_albums_authors;

CREATE SEQUENCE seq_va_albums_authors START 1;
CREATE TABLE va_albums_authors (
  album_id INT4 NOT NULL default '0',
  author_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (album_id,author_id));



DROP SEQUENCE seq_va_allowed_cell_phones CASCADE;
DROP TABLE va_allowed_cell_phones;

CREATE SEQUENCE seq_va_allowed_cell_phones START 1;
CREATE TABLE va_allowed_cell_phones (
  cell_phone_id INT4 NOT NULL DEFAULT nextval('seq_va_allowed_cell_phones'),
  cell_phone_number VARCHAR(32)
  ,PRIMARY KEY (cell_phone_id));

CREATE INDEX va_allowed_cell_phones_cell_7 ON va_allowed_cell_phones (cell_phone_number);


DROP SEQUENCE seq_va_articles CASCADE;
DROP TABLE va_articles;

CREATE SEQUENCE seq_va_articles START 38;
CREATE TABLE va_articles (
  article_id INT4 NOT NULL DEFAULT nextval('seq_va_articles'),
  language_code VARCHAR(2),
  article_date TIMESTAMP,
  date_end TIMESTAMP,
  article_title VARCHAR(255),
  title_first VARCHAR(2),
  article_comment VARCHAR(255),
  friendly_url VARCHAR(255),
  article_template VARCHAR(255),
  article_order INT4 default '0',
  author_name VARCHAR(255),
  author_email VARCHAR(255),
  author_url VARCHAR(255),
  author_remote_address VARCHAR(255),
  created_user_id INT4 default '0',
  created_admin_id INT4 default '0',
  updated_user_id INT4 default '0',
  updated_admin_id INT4 default '0',
  status_id INT4 NOT NULL default '0',
  is_html SMALLINT default '0',
  allowed_rate SMALLINT default '0',
  is_hot SMALLINT default '0',
  hot_order INT4 default '1',
  hot_description TEXT,
  link_title VARCHAR(255),
  link_url VARCHAR(255),
  download_url VARCHAR(255),
  is_link_direct VARCHAR(255),
  is_remote_rss SMALLINT default '0',
  details_remote_url VARCHAR(255),
  date_added VARCHAR(255),
  date_updated VARCHAR(255),
  keywords TEXT,
  highlights TEXT,
  short_description TEXT,
  full_description TEXT,
  notes TEXT,
  total_views INT4 default '0',
  total_votes INT4 default '0',
  total_points INT4 default '0',
  rating FLOAT4 default '0',
  total_clicks INT4 default '0',
  image_tiny VARCHAR(255),
  image_tiny_alt VARCHAR(255),
  image_small VARCHAR(255),
  image_small_alt VARCHAR(255),
  image_large VARCHAR(255),
  image_large_alt VARCHAR(255),
  image_super VARCHAR(255),
  image_super_alt VARCHAR(255),
  stream_video VARCHAR(255),
  stream_video_width INT4,
  stream_video_height INT4,
  stream_video_preview VARCHAR(255),
  youtube_video VARCHAR(255),
  youtube_video_width INT4,
  youtube_video_height INT4,
  meta_title VARCHAR(255),
  meta_keywords VARCHAR(255),
  meta_description VARCHAR(255),
  is_draft SMALLINT default '0',
  draft_parent_id INT4 default '0'
  ,PRIMARY KEY (article_id));

CREATE INDEX va_articles_draft_parent_id ON va_articles (draft_parent_id);
CREATE INDEX va_articles_friendly_url ON va_articles (friendly_url);
CREATE INDEX va_articles_is_draft ON va_articles (is_draft);
CREATE INDEX va_articles_status_id ON va_articles (status_id);
CREATE INDEX va_articles_title_first ON va_articles (title_first);


DROP SEQUENCE seq_va_articles_albums CASCADE;
DROP TABLE va_articles_albums;

CREATE SEQUENCE seq_va_articles_albums START 1;
CREATE TABLE va_articles_albums (
  article_id INT4 NOT NULL default '0',
  album_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (article_id,album_id));



DROP SEQUENCE seq_va_articles_assigned CASCADE;
DROP TABLE va_articles_assigned;

CREATE SEQUENCE seq_va_articles_assigned START 1;
CREATE TABLE va_articles_assigned (
  article_id INT4 NOT NULL default '0',
  category_id INT4 NOT NULL default '0',
  article_order INT4 default '0'
  ,PRIMARY KEY (article_id,category_id));



DROP SEQUENCE seq_va_articles_authors CASCADE;
DROP TABLE va_articles_authors;

CREATE SEQUENCE seq_va_articles_authors START 1;
CREATE TABLE va_articles_authors (
  article_id INT4 NOT NULL default '0',
  author_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (article_id,author_id));



DROP SEQUENCE seq_va_articles_categories CASCADE;
DROP TABLE va_articles_categories;

CREATE SEQUENCE seq_va_articles_categories START 31;
CREATE TABLE va_articles_categories (
  category_id INT4 NOT NULL DEFAULT nextval('seq_va_articles_categories'),
  language_code VARCHAR(2),
  parent_category_id INT4 default '0',
  alias_category_id INT4 default '0',
  category_path VARCHAR(255) NOT NULL,
  category_name VARCHAR(255),
  friendly_url VARCHAR(255),
  category_order INT4 default '0',
  articles_list_template VARCHAR(255),
  articles_details_template VARCHAR(255),
  articles_order_column VARCHAR(50),
  articles_order_direction VARCHAR(50),
  article_edit_fields TEXT,
  article_list_fields TEXT,
  article_details_fields TEXT,
  article_required_fields TEXT,
  moderators_ids VARCHAR(255),
  is_hot SMALLINT default '0',
  sites_all SMALLINT default '1',
  user_types_all SMALLINT default '1',
  access_level SMALLINT default '7',
  guest_access_level SMALLINT default '7',
  admin_access_level SMALLINT default '7',
  allowed_rate SMALLINT default '0',
  short_description TEXT,
  full_description TEXT,
  image_small VARCHAR(255),
  image_small_alt VARCHAR(255),
  image_large VARCHAR(255),
  image_large_alt VARCHAR(255),
  is_rss SMALLINT default '0',
  rss_limit INT4,
  rss_on_breadcrumb INT4 default '0',
  rss_on_list INT4 default '0',
  is_remote_rss SMALLINT default '0',
  remote_rss_url VARCHAR(255),
  remote_rss_date_updated TIMESTAMP,
  remote_rss_ttl INT4,
  remote_rss_refresh_rate INT4,
  meta_title VARCHAR(255),
  meta_keywords VARCHAR(255),
  meta_description VARCHAR(255),
  total_views INT4 default '0',
  total_articles INT4 default '0',
  total_subcategories INT4 default '0'
  ,PRIMARY KEY (category_id));

CREATE INDEX va_articles_categories_admi_8 ON va_articles_categories (admin_access_level);
CREATE INDEX va_articles_categories_cate_9 ON va_articles_categories (category_path);
CREATE INDEX va_articles_categories_frie_10 ON va_articles_categories (friendly_url);
CREATE INDEX va_articles_categories_gues_11 ON va_articles_categories (guest_access_level);
CREATE INDEX va_articles_categories_pare_12 ON va_articles_categories (parent_category_id);
CREATE INDEX va_articles_categories_sites_all ON va_articles_categories (sites_all);
CREATE INDEX va_articles_categories_subs_13 ON va_articles_categories (access_level);
CREATE INDEX va_articles_categories_user_14 ON va_articles_categories (user_types_all);


DROP SEQUENCE seq_va_articles_categories_items CASCADE;
DROP TABLE va_articles_categories_items;

CREATE SEQUENCE seq_va_articles_categories_items START 1;
CREATE TABLE va_articles_categories_items (
  category_id INT4 default '0',
  item_id INT4 default '0',
  related_order INT4 NOT NULL default '1'
);

CREATE INDEX va_articles_categories_item_15 ON va_articles_categories_items (category_id);
CREATE INDEX va_articles_categories_item_16 ON va_articles_categories_items (item_id);


DROP SEQUENCE seq_va_articles_categories_sites CASCADE;
DROP TABLE va_articles_categories_sites;

CREATE SEQUENCE seq_va_articles_categories_sites START 1;
CREATE TABLE va_articles_categories_sites (
  category_id INT4 NOT NULL default '0',
  site_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (category_id,site_id));



DROP SEQUENCE seq_va_articles_categories_subscriptions CASCADE;
DROP TABLE va_articles_categories_subscriptions;

CREATE SEQUENCE seq_va_articles_categories_subscriptions START 1;
CREATE TABLE va_articles_categories_subscriptions (
  category_id INT4 NOT NULL default '0',
  subscription_id INT4 NOT NULL default '0',
  access_level SMALLINT default '0'
  ,PRIMARY KEY (category_id,subscription_id));



DROP SEQUENCE seq_va_articles_categories_types CASCADE;
DROP TABLE va_articles_categories_types;

CREATE SEQUENCE seq_va_articles_categories_types START 1;
CREATE TABLE va_articles_categories_types (
  category_id INT4 NOT NULL default '0',
  user_type_id INT4 NOT NULL default '0',
  access_level SMALLINT default '0'
  ,PRIMARY KEY (category_id,user_type_id));



DROP SEQUENCE seq_va_articles_forum_topics CASCADE;
DROP TABLE va_articles_forum_topics;

CREATE SEQUENCE seq_va_articles_forum_topics START 1;
CREATE TABLE va_articles_forum_topics (
  article_id INT4 NOT NULL default '0',
  thread_id INT4 NOT NULL default '0',
  article_order INT4 default '1',
  thread_order INT4 default '1'
  ,PRIMARY KEY (article_id,thread_id));



DROP SEQUENCE seq_va_articles_images CASCADE;
DROP TABLE va_articles_images;

CREATE SEQUENCE seq_va_articles_images START 1;
CREATE TABLE va_articles_images (
  image_id INT4 NOT NULL DEFAULT nextval('seq_va_articles_images'),
  article_id INT4 default '0',
  category_id INT4 default '0',
  is_default SMALLINT default '0',
  image_order INT4 default '0',
  image_position SMALLINT default '1',
  image_title VARCHAR(255),
  image_tiny VARCHAR(255),
  image_tiny_alt VARCHAR(255),
  image_small VARCHAR(255),
  image_small_alt VARCHAR(255),
  image_large VARCHAR(255),
  image_large_alt VARCHAR(255),
  image_super VARCHAR(255),
  image_super_alt VARCHAR(255),
  image_description TEXT,
  date_added TIMESTAMP,
  date_modified TIMESTAMP
  ,PRIMARY KEY (image_id));

CREATE INDEX va_articles_images_article_id ON va_articles_images (article_id);
CREATE INDEX va_articles_images_category_id ON va_articles_images (category_id);


DROP SEQUENCE seq_va_articles_items_related CASCADE;
DROP TABLE va_articles_items_related;

CREATE SEQUENCE seq_va_articles_items_related START 1;
CREATE TABLE va_articles_items_related (
  article_id INT4 default '0',
  item_id INT4 default '0',
  item_order INT4 NOT NULL default '1',
  article_order INT4 default '1'
);

CREATE INDEX va_articles_items_related_a_17 ON va_articles_items_related (article_id);
CREATE INDEX va_articles_items_related_i_18 ON va_articles_items_related (item_id);


DROP SEQUENCE seq_va_articles_links CASCADE;
DROP TABLE va_articles_links;

CREATE SEQUENCE seq_va_articles_links START 1;
CREATE TABLE va_articles_links (
  link_id INT4 NOT NULL DEFAULT nextval('seq_va_articles_links'),
  article_id INT4 default '0',
  category_id INT4 default '0',
  link_title VARCHAR(255),
  link_url VARCHAR(255),
  date_added TIMESTAMP,
  date_modified TIMESTAMP
  ,PRIMARY KEY (link_id));

CREATE INDEX va_articles_links_article_id ON va_articles_links (article_id);
CREATE INDEX va_articles_links_category_id ON va_articles_links (category_id);


DROP SEQUENCE seq_va_articles_related CASCADE;
DROP TABLE va_articles_related;

CREATE SEQUENCE seq_va_articles_related START 1;
CREATE TABLE va_articles_related (
  article_id INT4 NOT NULL default '0',
  related_id INT4 NOT NULL default '0',
  related_order INT4 NOT NULL default '1'
  ,PRIMARY KEY (article_id,related_id));



DROP SEQUENCE seq_va_articles_reviews CASCADE;
DROP TABLE va_articles_reviews;

CREATE SEQUENCE seq_va_articles_reviews START 5;
CREATE TABLE va_articles_reviews (
  review_id INT4 NOT NULL DEFAULT nextval('seq_va_articles_reviews'),
  article_id INT4 NOT NULL default '0',
  user_id INT4 default '0',
  admin_id INT4 default '0',
  recommended SMALLINT default '0',
  approved SMALLINT default '0',
  rating INT4 default '0',
  summary TEXT,
  user_name VARCHAR(64),
  user_email VARCHAR(128),
  remote_address VARCHAR(255),
  comments TEXT,
  admin_id_approved_by INT4,
  admin_id_modified_by INT4,
  date_added TIMESTAMP NOT NULL,
  date_modified TIMESTAMP,
  date_approved TIMESTAMP
  ,PRIMARY KEY (review_id));

CREATE INDEX va_articles_reviews_admin_id ON va_articles_reviews (admin_id);
CREATE INDEX va_articles_reviews_article_id ON va_articles_reviews (article_id);
CREATE INDEX va_articles_reviews_user_id ON va_articles_reviews (user_id);


DROP SEQUENCE seq_va_articles_statuses CASCADE;
DROP TABLE va_articles_statuses;

CREATE SEQUENCE seq_va_articles_statuses START 5;
CREATE TABLE va_articles_statuses (
  status_id INT4 NOT NULL DEFAULT nextval('seq_va_articles_statuses'),
  status_name VARCHAR(255),
  status_description VARCHAR(255),
  is_shown SMALLINT default '0',
  status_icon VARCHAR(255),
  status_style VARCHAR(255),
  allowed_view SMALLINT NOT NULL default '0'
  ,PRIMARY KEY (status_id));



DROP SEQUENCE seq_va_articles_tags CASCADE;
DROP TABLE va_articles_tags;

CREATE SEQUENCE seq_va_articles_tags START 1;
CREATE TABLE va_articles_tags (
  article_id INT4 NOT NULL default '0',
  tag_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (article_id,tag_id));



DROP SEQUENCE seq_va_authors CASCADE;
DROP TABLE va_authors;

CREATE SEQUENCE seq_va_authors START 1;
CREATE TABLE va_authors (
  author_id INT4 NOT NULL DEFAULT nextval('seq_va_authors'),
  author_name VARCHAR(255),
  other_name VARCHAR(255),
  name_first VARCHAR(2),
  middle_first VARCHAR(2),
  last_first VARCHAR(2),
  other_first VARCHAR(2),
  friendly_url VARCHAR(255),
  image_tiny VARCHAR(255),
  image_small VARCHAR(255),
  image_large VARCHAR(255),
  image_super VARCHAR(255),
  short_description TEXT,
  full_description TEXT,
  sites_all SMALLINT default '1',
  date_added TIMESTAMP,
  date_updated TIMESTAMP
  ,PRIMARY KEY (author_id));

CREATE INDEX va_authors_friendly_url ON va_authors (friendly_url);
CREATE INDEX va_authors_last_first ON va_authors (last_first);
CREATE INDEX va_authors_middle_first ON va_authors (middle_first);
CREATE INDEX va_authors_name_first ON va_authors (name_first);
CREATE INDEX va_authors_other_first ON va_authors (other_first);


DROP SEQUENCE seq_va_authors_sites CASCADE;
DROP TABLE va_authors_sites;

CREATE SEQUENCE seq_va_authors_sites START 1;
CREATE TABLE va_authors_sites (
  author_id INT4 NOT NULL default '0',
  site_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (author_id,site_id));



DROP SEQUENCE seq_va_banned_contents CASCADE;
DROP TABLE va_banned_contents;

CREATE SEQUENCE seq_va_banned_contents START 1;
CREATE TABLE va_banned_contents (
  content_id INT4 NOT NULL DEFAULT nextval('seq_va_banned_contents'),
  content_text VARCHAR(255) NOT NULL
  ,PRIMARY KEY (content_id));



DROP SEQUENCE seq_va_banners CASCADE;
DROP TABLE va_banners;

CREATE SEQUENCE seq_va_banners START 1;
CREATE TABLE va_banners (
  banner_id INT4 NOT NULL DEFAULT nextval('seq_va_banners'),
  banner_rank INT4 NOT NULL default '1',
  banner_title VARCHAR(255) NOT NULL,
  show_title INT4 default '0',
  image_src VARCHAR(255),
  image_alt VARCHAR(255),
  html_text TEXT,
  target_url VARCHAR(255),
  is_new_window SMALLINT default '0',
  is_active SMALLINT default '0',
  sites_all SMALLINT default '1',
  show_on_ssl SMALLINT default '0',
  max_impressions INT4 default '0',
  max_clicks INT4 default '0',
  expiry_date TIMESTAMP,
  total_impressions INT4 default '0',
  total_clicks INT4 default '0'
  ,PRIMARY KEY (banner_id));

CREATE INDEX va_banners_is_active ON va_banners (is_active);
CREATE INDEX va_banners_max_clicks ON va_banners (max_clicks);
CREATE INDEX va_banners_max_impressions ON va_banners (max_impressions);
CREATE INDEX va_banners_sites_all ON va_banners (sites_all);


DROP SEQUENCE seq_va_banners_assigned CASCADE;
DROP TABLE va_banners_assigned;

CREATE SEQUENCE seq_va_banners_assigned START 1;
CREATE TABLE va_banners_assigned (
  banner_id INT4 NOT NULL default '0',
  group_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (banner_id,group_id));



DROP SEQUENCE seq_va_banners_clicks CASCADE;
DROP TABLE va_banners_clicks;

CREATE SEQUENCE seq_va_banners_clicks START 1;
CREATE TABLE va_banners_clicks (
  click_id INT4 NOT NULL DEFAULT nextval('seq_va_banners_clicks'),
  banner_id INT4 default '0',
  user_id INT4 default '0',
  remote_address VARCHAR(50),
  click_date TIMESTAMP NOT NULL
  ,PRIMARY KEY (click_id));

CREATE INDEX va_banners_clicks_banner_id ON va_banners_clicks (banner_id);
CREATE INDEX va_banners_clicks_click_date ON va_banners_clicks (click_date);
CREATE INDEX va_banners_clicks_user_id ON va_banners_clicks (user_id);


DROP SEQUENCE seq_va_banners_groups CASCADE;
DROP TABLE va_banners_groups;

CREATE SEQUENCE seq_va_banners_groups START 1;
CREATE TABLE va_banners_groups (
  group_id INT4 NOT NULL DEFAULT nextval('seq_va_banners_groups'),
  group_name VARCHAR(128),
  group_desc TEXT,
  is_active SMALLINT default '0'
  ,PRIMARY KEY (group_id));

CREATE INDEX va_banners_groups_is_active ON va_banners_groups (is_active);


DROP SEQUENCE seq_va_banners_sites CASCADE;
DROP TABLE va_banners_sites;

CREATE SEQUENCE seq_va_banners_sites START 1;
CREATE TABLE va_banners_sites (
  banner_id INT4 NOT NULL default '0',
  site_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (banner_id,site_id));



DROP SEQUENCE seq_va_black_ips CASCADE;
DROP TABLE va_black_ips;

CREATE SEQUENCE seq_va_black_ips START 1;
CREATE TABLE va_black_ips (
  ip_address VARCHAR(32) NOT NULL,
  address_action INT4 default '0',
  address_notes VARCHAR(255)
  ,PRIMARY KEY (ip_address));



DROP SEQUENCE seq_va_bookmarks CASCADE;
DROP TABLE va_bookmarks;

CREATE SEQUENCE seq_va_bookmarks START 1;
CREATE TABLE va_bookmarks (
  bookmark_id INT4 NOT NULL DEFAULT nextval('seq_va_bookmarks'),
  title VARCHAR(50) NOT NULL,
  url VARCHAR(255) NOT NULL,
  admin_id INT4 NOT NULL default '0',
  image_path VARCHAR(255),
  is_start_page SMALLINT default '0',
  is_popup SMALLINT default '0',
  notes VARCHAR(255)
  ,PRIMARY KEY (bookmark_id));

CREATE INDEX va_bookmarks_admin_id ON va_bookmarks (admin_id);


DROP SEQUENCE seq_va_caches CASCADE;
DROP TABLE va_caches;

CREATE SEQUENCE seq_va_caches START 2712;
CREATE TABLE va_caches (
  cache_id INT4 NOT NULL DEFAULT nextval('seq_va_caches'),
  cache_type VARCHAR(32),
  cache_name VARCHAR(32),
  cache_parameter VARCHAR(32),
  cache_date TIMESTAMP NOT NULL,
  cache_data TEXT NOT NULL
  ,PRIMARY KEY (cache_id));

CREATE INDEX va_caches_cache_name ON va_caches (cache_name);
CREATE INDEX va_caches_cache_parameter ON va_caches (cache_parameter);
CREATE INDEX va_caches_cache_type ON va_caches (cache_type);


DROP SEQUENCE seq_va_categories CASCADE;
DROP TABLE va_categories;

CREATE SEQUENCE seq_va_categories START 61;
CREATE TABLE va_categories (
  category_id INT4 NOT NULL DEFAULT nextval('seq_va_categories'),
  parent_category_id INT4 NOT NULL default '0',
  alias_category_id INT4 default '0',
  redirect_category_id INT4 default '0',
  google_base_type_id INT4 default '0',
  category_path VARCHAR(255) NOT NULL,
  category_name VARCHAR(255) NOT NULL,
  friendly_url VARCHAR(255),
  category_order INT4 NOT NULL default '1',
  sites_all SMALLINT default '1',
  access_level SMALLINT default '7',
  guest_access_level SMALLINT default '7',
  admin_access_level SMALLINT default '7',
  is_showing SMALLINT NOT NULL default '0',
  allowed_post_subcategories SMALLINT default '0',
  show_sub_products SMALLINT default '0',
  short_description TEXT,
  full_description TEXT,
  image VARCHAR(255),
  image_alt VARCHAR(255),
  image_large VARCHAR(255),
  image_large_alt VARCHAR(255),
  list_template VARCHAR(128),
  details_template VARCHAR(128),
  table_view SMALLINT default '0',
  a_title VARCHAR(255),
  meta_title VARCHAR(255),
  meta_keywords VARCHAR(255),
  meta_description VARCHAR(255),
  total_views INT4 default '0',
  header_menu_show SMALLINT default '0',
  header_menu_order INT4,
  header_menu_class VARCHAR(64),
  nav_bar_show SMALLINT default '0',
  nav_bar_order INT4,
  nav_bar_class VARCHAR(64),
  admin_id_added_by INT4 default '0',
  admin_id_modified_by INT4 default '0',
  date_added TIMESTAMP,
  date_modified TIMESTAMP
  ,PRIMARY KEY (category_id));

CREATE INDEX va_categories_access_level ON va_categories (guest_access_level);
CREATE INDEX va_categories_admin_access_level ON va_categories (admin_access_level);
CREATE INDEX va_categories_category_path ON va_categories (category_path);
CREATE INDEX va_categories_friendly_url ON va_categories (friendly_url);
CREATE INDEX va_categories_google_base_t_19 ON va_categories (google_base_type_id);
CREATE INDEX va_categories_is_showing ON va_categories (is_showing);
CREATE INDEX va_categories_parent_category_id ON va_categories (parent_category_id);
CREATE INDEX va_categories_sites_all ON va_categories (sites_all);
CREATE INDEX va_categories_subscription__20 ON va_categories (access_level);


DROP SEQUENCE seq_va_categories_columns CASCADE;
DROP TABLE va_categories_columns;

CREATE SEQUENCE seq_va_categories_columns START 9;
CREATE TABLE va_categories_columns (
  column_id INT4 NOT NULL DEFAULT nextval('seq_va_categories_columns'),
  category_id INT4 NOT NULL default '0',
  column_order SMALLINT default '0',
  column_code VARCHAR(64),
  column_title VARCHAR(255),
  column_html TEXT
  ,PRIMARY KEY (column_id));

CREATE INDEX va_categories_columns_categ_21 ON va_categories_columns (category_id);


DROP SEQUENCE seq_va_categories_sites CASCADE;
DROP TABLE va_categories_sites;

CREATE SEQUENCE seq_va_categories_sites START 1;
CREATE TABLE va_categories_sites (
  category_id INT4 NOT NULL default '0',
  site_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (category_id,site_id));



DROP SEQUENCE seq_va_categories_subscriptions CASCADE;
DROP TABLE va_categories_subscriptions;

CREATE SEQUENCE seq_va_categories_subscriptions START 1;
CREATE TABLE va_categories_subscriptions (
  category_id INT4 NOT NULL default '0',
  subscription_id INT4 NOT NULL default '0',
  access_level SMALLINT default '0'
  ,PRIMARY KEY (category_id,subscription_id));



DROP SEQUENCE seq_va_categories_user_types CASCADE;
DROP TABLE va_categories_user_types;

CREATE SEQUENCE seq_va_categories_user_types START 1;
CREATE TABLE va_categories_user_types (
  category_id INT4 NOT NULL default '0',
  user_type_id INT4 NOT NULL default '0',
  access_level SMALLINT default '0'
  ,PRIMARY KEY (category_id,user_type_id));



DROP SEQUENCE seq_va_cc_expiry_years CASCADE;
DROP TABLE va_cc_expiry_years;

CREATE SEQUENCE seq_va_cc_expiry_years START 1;
CREATE TABLE va_cc_expiry_years (
  expiry_year INT4 NOT NULL default '0'
  ,PRIMARY KEY (expiry_year));



DROP SEQUENCE seq_va_cc_start_years CASCADE;
DROP TABLE va_cc_start_years;

CREATE SEQUENCE seq_va_cc_start_years START 1;
CREATE TABLE va_cc_start_years (
  start_year INT4 NOT NULL default '0'
  ,PRIMARY KEY (start_year));



DROP SEQUENCE seq_va_change_types CASCADE;
DROP TABLE va_change_types;

CREATE SEQUENCE seq_va_change_types START 3;
CREATE TABLE va_change_types (
  type_id INT4 NOT NULL DEFAULT nextval('seq_va_change_types'),
  type_name VARCHAR(255) NOT NULL
  ,PRIMARY KEY (type_id));



DROP SEQUENCE seq_va_chats CASCADE;
DROP TABLE va_chats;

CREATE SEQUENCE seq_va_chats START 1;
CREATE TABLE va_chats (
  chat_id INT4 NOT NULL DEFAULT nextval('seq_va_chats'),
  chat_status SMALLINT default '1',
  user_id INT4 default '0',
  user_name VARCHAR(128),
  user_email VARCHAR(128),
  user_message TEXT,
  admin_id INT4 default '0',
  chat_added TIMESTAMP,
  chat_started TIMESTAMP,
  chat_closed TIMESTAMP,
  user_online TIMESTAMP,
  user_last_added TIMESTAMP,
  admin_online TIMESTAMP,
  admin_last_added TIMESTAMP
  ,PRIMARY KEY (chat_id));



DROP SEQUENCE seq_va_chats_messages CASCADE;
DROP TABLE va_chats_messages;

CREATE SEQUENCE seq_va_chats_messages START 1;
CREATE TABLE va_chats_messages (
  message_id INT4 NOT NULL DEFAULT nextval('seq_va_chats_messages'),
  chat_id INT4 default '0',
  admin_id INT4 default '0',
  is_user_message SMALLINT default '0',
  message_type SMALLINT default '1',
  message_text TEXT,
  message_added TIMESTAMP
  ,PRIMARY KEY (message_id));

CREATE INDEX va_chats_messages_admin_id ON va_chats_messages (admin_id);
CREATE INDEX va_chats_messages_chat_id ON va_chats_messages (chat_id);
CREATE INDEX va_chats_messages_message_type ON va_chats_messages (message_type);


DROP SEQUENCE seq_va_cms_blocks CASCADE;
DROP TABLE va_cms_blocks;

CREATE SEQUENCE seq_va_cms_blocks START 181;
CREATE TABLE va_cms_blocks (
  block_id INT4 NOT NULL DEFAULT nextval('seq_va_cms_blocks'),
  module_id INT4 default '0',
  block_order INT4 default '1',
  block_code VARCHAR(64),
  block_name VARCHAR(128),
  php_script VARCHAR(128),
  layout_type VARCHAR(32),
  layout_template VARCHAR(64),
  html_template VARCHAR(64),
  css_class VARCHAR(128),
  block_title TEXT,
  pages_all SMALLINT default '0'
  ,PRIMARY KEY (block_id));



DROP SEQUENCE seq_va_cms_blocks_pages CASCADE;
DROP TABLE va_cms_blocks_pages;

CREATE SEQUENCE seq_va_cms_blocks_pages START 1;
CREATE TABLE va_cms_blocks_pages (
  block_id INT4 NOT NULL default '0',
  page_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (block_id,page_id));



DROP SEQUENCE seq_va_cms_blocks_periods CASCADE;
DROP TABLE va_cms_blocks_periods;

CREATE SEQUENCE seq_va_cms_blocks_periods START 1;
CREATE TABLE va_cms_blocks_periods (
  period_id INT4 NOT NULL DEFAULT nextval('seq_va_cms_blocks_periods'),
  ps_id INT4 default '0',
  pb_id INT4 default '0',
  start_date TIMESTAMP,
  end_date TIMESTAMP,
  start_time INT4 default '0',
  end_time INT4 default '0',
  week_days INT4 default '0'
  ,PRIMARY KEY (period_id));



DROP SEQUENCE seq_va_cms_blocks_properties CASCADE;
DROP TABLE va_cms_blocks_properties;

CREATE SEQUENCE seq_va_cms_blocks_properties START 287;
CREATE TABLE va_cms_blocks_properties (
  property_id INT4 NOT NULL DEFAULT nextval('seq_va_cms_blocks_properties'),
  block_id INT4 default '0',
  property_order INT4 default '1',
  property_name VARCHAR(255),
  control_type VARCHAR(16),
  parent_property_id INT4,
  parent_value_id INT4,
  variable_name VARCHAR(64),
  default_value VARCHAR(255),
  required SMALLINT default '0',
  property_class VARCHAR(128),
  property_style TEXT,
  control_style TEXT,
  start_html TEXT,
  middle_html TEXT,
  before_control_html TEXT,
  after_control_html TEXT,
  end_html TEXT,
  control_code TEXT,
  onchange_code TEXT,
  onclick_code TEXT
  ,PRIMARY KEY (property_id));



DROP SEQUENCE seq_va_cms_blocks_settings CASCADE;
DROP TABLE va_cms_blocks_settings;

CREATE SEQUENCE seq_va_cms_blocks_settings START 491;
CREATE TABLE va_cms_blocks_settings (
  bs_id INT4 NOT NULL DEFAULT nextval('seq_va_cms_blocks_settings'),
  ps_id INT4 default '0',
  pb_id INT4 default '0',
  property_id INT4 default '0',
  value_id INT4 default '0',
  variable_name VARCHAR(64),
  variable_value VARCHAR(255)
  ,PRIMARY KEY (bs_id));



DROP SEQUENCE seq_va_cms_blocks_values CASCADE;
DROP TABLE va_cms_blocks_values;

CREATE SEQUENCE seq_va_cms_blocks_values START 309;
CREATE TABLE va_cms_blocks_values (
  value_id INT4 NOT NULL DEFAULT nextval('seq_va_cms_blocks_values'),
  property_id INT4 default '0',
  value_order INT4 default '1',
  value_name VARCHAR(255),
  variable_name VARCHAR(64),
  variable_value VARCHAR(255),
  hide_value SMALLINT default '0',
  is_default_value SMALLINT default '0'
  ,PRIMARY KEY (value_id));



DROP SEQUENCE seq_va_cms_frames CASCADE;
DROP TABLE va_cms_frames;

CREATE SEQUENCE seq_va_cms_frames START 24;
CREATE TABLE va_cms_frames (
  frame_id INT4 NOT NULL DEFAULT nextval('seq_va_cms_frames'),
  layout_id INT4 default '0',
  frame_name VARCHAR(128),
  tag_name VARCHAR(128)
  ,PRIMARY KEY (frame_id));



DROP SEQUENCE seq_va_cms_frames_settings CASCADE;
DROP TABLE va_cms_frames_settings;

CREATE SEQUENCE seq_va_cms_frames_settings START 534;
CREATE TABLE va_cms_frames_settings (
  fs_id INT4 NOT NULL DEFAULT nextval('seq_va_cms_frames_settings'),
  ps_id INT4 default '0',
  frame_id INT4 default '0',
  frame_style TEXT,
  frame_class VARCHAR(64),
  frame_code TEXT,
  html_frame_start TEXT,
  html_before_block TEXT,
  html_after_block TEXT,
  html_frame_end TEXT,
  html_between_blocks TEXT
  ,PRIMARY KEY (fs_id));

CREATE INDEX va_cms_frames_settings_frame_id ON va_cms_frames_settings (frame_id);
CREATE INDEX va_cms_frames_settings_ps_id ON va_cms_frames_settings (ps_id);


DROP SEQUENCE seq_va_cms_layouts CASCADE;
DROP TABLE va_cms_layouts;

CREATE SEQUENCE seq_va_cms_layouts START 6;
CREATE TABLE va_cms_layouts (
  layout_id INT4 NOT NULL DEFAULT nextval('seq_va_cms_layouts'),
  layout_name VARCHAR(255),
  layout_order INT4 default '0',
  layout_template VARCHAR(255),
  admin_template VARCHAR(255)
  ,PRIMARY KEY (layout_id));



DROP SEQUENCE seq_va_cms_modules CASCADE;
DROP TABLE va_cms_modules;

CREATE SEQUENCE seq_va_cms_modules START 17;
CREATE TABLE va_cms_modules (
  module_id INT4 NOT NULL DEFAULT nextval('seq_va_cms_modules'),
  module_order INT4 default '0',
  module_code VARCHAR(32),
  module_name VARCHAR(255)
  ,PRIMARY KEY (module_id));



DROP SEQUENCE seq_va_cms_pages CASCADE;
DROP TABLE va_cms_pages;

CREATE SEQUENCE seq_va_cms_pages START 97;
CREATE TABLE va_cms_pages (
  page_id INT4 NOT NULL DEFAULT nextval('seq_va_cms_pages'),
  module_id INT4 default '0',
  page_order INT4 default '0',
  page_code VARCHAR(32),
  page_name VARCHAR(128)
  ,PRIMARY KEY (page_id));

CREATE INDEX va_cms_pages_page_code ON va_cms_pages (page_code);


DROP SEQUENCE seq_va_cms_pages_blocks CASCADE;
DROP TABLE va_cms_pages_blocks;

CREATE SEQUENCE seq_va_cms_pages_blocks START 651;
CREATE TABLE va_cms_pages_blocks (
  pb_id INT4 NOT NULL DEFAULT nextval('seq_va_cms_pages_blocks'),
  ps_id INT4 default '0',
  block_id INT4 default '0',
  block_key VARCHAR(32),
  frame_id INT4 default '0',
  block_order INT4 default '0',
  tag_name VARCHAR(128),
  layout_type VARCHAR(32),
  layout_template VARCHAR(64),
  html_template VARCHAR(128),
  css_class VARCHAR(128),
  block_style TEXT,
  block_title TEXT
  ,PRIMARY KEY (pb_id));



DROP SEQUENCE seq_va_cms_pages_settings CASCADE;
DROP TABLE va_cms_pages_settings;

CREATE SEQUENCE seq_va_cms_pages_settings START 117;
CREATE TABLE va_cms_pages_settings (
  ps_id INT4 NOT NULL DEFAULT nextval('seq_va_cms_pages_settings'),
  page_id INT4 default '0',
  key_code VARCHAR(32),
  key_type VARCHAR(16),
  key_rule VARCHAR(16),
  layout_id INT4 default '0',
  site_id INT4 default '0',
  page_class VARCHAR(64),
  meta_title TEXT,
  meta_keywords TEXT,
  meta_description TEXT
  ,PRIMARY KEY (ps_id));

CREATE INDEX va_cms_pages_settings_key_code ON va_cms_pages_settings (key_code);
CREATE INDEX va_cms_pages_settings_page_id ON va_cms_pages_settings (page_id);


DROP SEQUENCE seq_va_companies CASCADE;
DROP TABLE va_companies;

CREATE SEQUENCE seq_va_companies START 1;
CREATE TABLE va_companies (
  company_id INT4 NOT NULL DEFAULT nextval('seq_va_companies'),
  company_name VARCHAR(255) NOT NULL,
  image_small VARCHAR(50),
  image_large VARCHAR(50),
  address_info TEXT,
  phone_number VARCHAR(32),
  fax_number VARCHAR(32),
  site_url VARCHAR(128),
  contact_email VARCHAR(128),
  short_description TEXT,
  full_description TEXT
  ,PRIMARY KEY (company_id));



DROP SEQUENCE seq_va_countries CASCADE;
DROP TABLE va_countries;

CREATE SEQUENCE seq_va_countries START 248;
CREATE TABLE va_countries (
  country_id INT4 NOT NULL DEFAULT nextval('seq_va_countries'),
  country_code VARCHAR(4) NOT NULL,
  country_iso_number VARCHAR(4),
  country_code_alpha3 VARCHAR(4),
  country_order INT4 default '1',
  country_name VARCHAR(64) NOT NULL,
  full_name VARCHAR(64),
  state_field_name VARCHAR(64),
  phone_code VARCHAR(32),
  currency_code VARCHAR(4),
  show_for_user SMALLINT default '1',
  sites_all SMALLINT default '1'
  ,PRIMARY KEY (country_id));



DROP SEQUENCE seq_va_countries_sites CASCADE;
DROP TABLE va_countries_sites;

CREATE SEQUENCE seq_va_countries_sites START 1;
CREATE TABLE va_countries_sites (
  country_id INT4 NOT NULL default '0',
  site_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (country_id,site_id));



DROP SEQUENCE seq_va_coupons CASCADE;
DROP TABLE va_coupons;

CREATE SEQUENCE seq_va_coupons START 1;
CREATE TABLE va_coupons (
  coupon_id INT4 NOT NULL DEFAULT nextval('seq_va_coupons'),
  order_id INT4 default '0',
  order_item_id INT4 default '0',
  coupon_code VARCHAR(64),
  use_friends_code SMALLINT default '0',
  coupon_title VARCHAR(255),
  is_active SMALLINT default '1',
  is_auto_apply SMALLINT NOT NULL default '0',
  apply_order INT4 default '1',
  discount_type SMALLINT default '0',
  discount_amount FLOAT4 default '0',
  discount_quantity INT4,
  discount_tax_free INT4 default '0',
  free_postage SMALLINT default '0',
  free_postage_all SMALLINT default '1',
  free_postage_ids TEXT,
  coupon_tax_free SMALLINT default '0',
  order_tax_free SMALLINT default '0',
  order_min_goods_cost FLOAT4 default '0',
  order_max_goods_cost FLOAT4 default '0',
  order_min_weight FLOAT4 default '0',
  order_max_weight FLOAT4 default '0',
  items_all SMALLINT default '1',
  items_ids TEXT,
  items_types_ids TEXT,
  items_categories_ids TEXT,
  cart_items_all SMALLINT default '1',
  cart_items_ids TEXT,
  cart_items_types_ids TEXT,
  sites_all SMALLINT default '1',
  users_all SMALLINT default '1',
  users_ids TEXT,
  users_types_ids TEXT,
  users_use_limit INT4,
  min_quantity INT4,
  max_quantity INT4,
  minimum_amount FLOAT4 default '0',
  maximum_amount FLOAT4,
  min_cart_quantity INT4,
  max_cart_quantity INT4,
  min_cart_cost FLOAT4,
  max_cart_cost FLOAT4,
  orders_period SMALLINT,
  orders_interval INT4,
  orders_min_goods FLOAT4,
  orders_max_goods FLOAT4,
  orders_min_quantity INT4,
  orders_max_quantity INT4,
  orders_items_type SMALLINT default '1',
  orders_items_ids TEXT,
  orders_types_ids TEXT,
  friends_discount_type SMALLINT default '0',
  friends_all SMALLINT default '1',
  friends_ids TEXT,
  friends_types_ids TEXT,
  friends_period SMALLINT,
  friends_interval INT4,
  friends_min_goods FLOAT4,
  friends_max_goods FLOAT4,
  start_date TIMESTAMP,
  expiry_date TIMESTAMP,
  is_exclusive SMALLINT default '0',
  quantity_limit INT4 default '1',
  coupon_uses INT4 default '0',
  admin_id_added_by INT4 default '0',
  admin_id_modified_by INT4 default '0',
  date_added TIMESTAMP,
  date_modified TIMESTAMP
  ,PRIMARY KEY (coupon_id));

CREATE INDEX va_coupons_coupon_code ON va_coupons (coupon_code);
CREATE INDEX va_coupons_is_auto_apply ON va_coupons (is_auto_apply);
CREATE INDEX va_coupons_order_id ON va_coupons (order_id);
CREATE INDEX va_coupons_order_item_id ON va_coupons (order_item_id);
CREATE INDEX va_coupons_sites_all ON va_coupons (sites_all);
CREATE INDEX va_coupons_use_friends_code ON va_coupons (use_friends_code);


DROP SEQUENCE seq_va_coupons_sites CASCADE;
DROP TABLE va_coupons_sites;

CREATE SEQUENCE seq_va_coupons_sites START 1;
CREATE TABLE va_coupons_sites (
  coupon_id INT4 NOT NULL default '0',
  site_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (coupon_id,site_id));



DROP SEQUENCE seq_va_credit_cards CASCADE;
DROP TABLE va_credit_cards;

CREATE SEQUENCE seq_va_credit_cards START 11;
CREATE TABLE va_credit_cards (
  credit_card_id INT4 NOT NULL DEFAULT nextval('seq_va_credit_cards'),
  credit_card_code VARCHAR(32) NOT NULL,
  credit_card_name VARCHAR(64) NOT NULL
  ,PRIMARY KEY (credit_card_id));



DROP SEQUENCE seq_va_currencies CASCADE;
DROP TABLE va_currencies;

CREATE SEQUENCE seq_va_currencies START 20;
CREATE TABLE va_currencies (
  currency_id INT4 NOT NULL DEFAULT nextval('seq_va_currencies'),
  is_default SMALLINT default '0',
  is_default_show SMALLINT default '0',
  show_for_user SMALLINT default '1',
  currency_code VARCHAR(4),
  currency_value VARCHAR(16),
  currency_title VARCHAR(64),
  currency_image VARCHAR(255),
  currency_image_active VARCHAR(255),
  exchange_rate FLOAT4 NOT NULL default '1',
  symbol_left VARCHAR(32),
  symbol_right VARCHAR(32),
  decimals_number SMALLINT default '2',
  decimal_point VARCHAR(16),
  thousands_separator VARCHAR(16),
  sites_all SMALLINT default '1'
  ,PRIMARY KEY (currency_id));

CREATE INDEX va_currencies_currency_code ON va_currencies (currency_code);


DROP SEQUENCE seq_va_currencies_sites CASCADE;
DROP TABLE va_currencies_sites;

CREATE SEQUENCE seq_va_currencies_sites START 1;
CREATE TABLE va_currencies_sites (
  currency_id INT4 NOT NULL default '0',
  site_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (currency_id,site_id));



DROP SEQUENCE seq_va_custom_blocks CASCADE;
DROP TABLE va_custom_blocks;

CREATE SEQUENCE seq_va_custom_blocks START 5;
CREATE TABLE va_custom_blocks (
  block_id INT4 NOT NULL DEFAULT nextval('seq_va_custom_blocks'),
  block_name VARCHAR(128),
  block_notes TEXT,
  block_class VARCHAR(128),
  block_title VARCHAR(255),
  block_path VARCHAR(255),
  block_desc TEXT
  ,PRIMARY KEY (block_id));



DROP SEQUENCE seq_va_ethnicities CASCADE;
DROP TABLE va_ethnicities;

CREATE SEQUENCE seq_va_ethnicities START 8;
CREATE TABLE va_ethnicities (
  ethnicity_id INT4 NOT NULL DEFAULT nextval('seq_va_ethnicities'),
  ethnicity_name VARCHAR(255),
  sort_order INT4 default '1',
  show_for_user SMALLINT default '1'
  ,PRIMARY KEY (ethnicity_id));



DROP SEQUENCE seq_va_export_fields CASCADE;
DROP TABLE va_export_fields;

CREATE SEQUENCE seq_va_export_fields START 1;
CREATE TABLE va_export_fields (
  field_id INT4 NOT NULL DEFAULT nextval('seq_va_export_fields'),
  template_id INT4 default '0',
  field_order INT4 default '0',
  field_title VARCHAR(128),
  field_source TEXT
  ,PRIMARY KEY (field_id));



DROP SEQUENCE seq_va_export_filters CASCADE;
DROP TABLE va_export_filters;

CREATE SEQUENCE seq_va_export_filters START 1;
CREATE TABLE va_export_filters (
  filter_id INT4 NOT NULL DEFAULT nextval('seq_va_export_filters'),
  template_id INT4 default '0',
  filter_parameter VARCHAR(64),
  filter_value VARCHAR(255)
  ,PRIMARY KEY (filter_id));

CREATE INDEX va_export_filters_template_id ON va_export_filters (template_id);


DROP SEQUENCE seq_va_export_templates CASCADE;
DROP TABLE va_export_templates;

CREATE SEQUENCE seq_va_export_templates START 1;
CREATE TABLE va_export_templates (
  template_id INT4 NOT NULL DEFAULT nextval('seq_va_export_templates'),
  template_name VARCHAR(255),
  table_name VARCHAR(64),
  save_file SMALLINT default '0',
  file_path_mask VARCHAR(255),
  file_path_mask_copy VARCHAR(255),
  ftp_upload SMALLINT default '0',
  ftp_host VARCHAR(255),
  ftp_port VARCHAR(16),
  ftp_login VARCHAR(128),
  ftp_password VARCHAR(255),
  ftp_passive_mode SMALLINT default '0',
  ftp_transfer_mode VARCHAR(16),
  ftp_path VARCHAR(255),
  use_filter SMALLINT default '0',
  is_cronjob SMALLINT default '0',
  order_status_update INT4,
  admin_id_added_by INT4 default '0',
  admin_id_modified_by INT4 default '0',
  date_added TIMESTAMP,
  date_modified TIMESTAMP
  ,PRIMARY KEY (template_id));



DROP SEQUENCE seq_va_features CASCADE;
DROP TABLE va_features;

CREATE SEQUENCE seq_va_features START 152;
CREATE TABLE va_features (
  feature_id INT4 NOT NULL DEFAULT nextval('seq_va_features'),
  item_id INT4 default '0',
  group_id INT4 default '0',
  google_base_attribute_id INT4 default '0',
  feature_code VARCHAR(32),
  feature_name VARCHAR(255),
  feature_value VARCHAR(255),
  show_as_group SMALLINT default '1',
  show_on_table SMALLINT default '0',
  show_on_details SMALLINT default '0',
  show_on_basket SMALLINT default '0',
  show_on_checkout SMALLINT default '0',
  show_on_invoice SMALLINT default '0'
  ,PRIMARY KEY (feature_id));

CREATE INDEX va_features_feature_code ON va_features (feature_code);
CREATE INDEX va_features_google_base_att_22 ON va_features (google_base_attribute_id);
CREATE INDEX va_features_group_id ON va_features (group_id);
CREATE INDEX va_features_item_id ON va_features (item_id);


DROP SEQUENCE seq_va_features_default CASCADE;
DROP TABLE va_features_default;

CREATE SEQUENCE seq_va_features_default START 1;
CREATE TABLE va_features_default (
  feature_id INT4 NOT NULL DEFAULT nextval('seq_va_features_default'),
  item_type_id INT4 default '0',
  google_base_attribute_id INT4 default '0',
  group_id INT4 default '0',
  feature_code VARCHAR(32),
  feature_name VARCHAR(50),
  feature_value TEXT
  ,PRIMARY KEY (feature_id));

CREATE INDEX va_features_default_google__23 ON va_features_default (google_base_attribute_id);
CREATE INDEX va_features_default_group_id ON va_features_default (group_id);
CREATE INDEX va_features_default_item_type_id ON va_features_default (item_type_id);


DROP SEQUENCE seq_va_features_groups CASCADE;
DROP TABLE va_features_groups;

CREATE SEQUENCE seq_va_features_groups START 8;
CREATE TABLE va_features_groups (
  group_id INT4 NOT NULL DEFAULT nextval('seq_va_features_groups'),
  group_order INT4 NOT NULL default '1',
  group_name VARCHAR(255),
  show_on_table SMALLINT default '0',
  show_on_details SMALLINT default '1',
  show_on_basket SMALLINT default '0',
  show_on_checkout SMALLINT default '0',
  show_on_invoice SMALLINT default '0'
  ,PRIMARY KEY (group_id));



DROP SEQUENCE seq_va_file_transfers CASCADE;
DROP TABLE va_file_transfers;

CREATE SEQUENCE seq_va_file_transfers START 1;
CREATE TABLE va_file_transfers (
  transfer_id INT4 NOT NULL DEFAULT nextval('seq_va_file_transfers'),
  transfer_type SMALLINT default '0',
  transfer_status VARCHAR(16),
  transfer_date TIMESTAMP,
  ftp_host VARCHAR(255),
  ftp_port VARCHAR(16),
  ftp_login VARCHAR(128),
  ftp_password VARCHAR(255),
  ftp_passive_mode SMALLINT default '0',
  ftp_transfer_mode VARCHAR(16),
  ftp_path VARCHAR(255),
  file_path VARCHAR(255),
  date_added TIMESTAMP,
  date_transferred TIMESTAMP,
  date_failed TIMESTAMP,
  failed_attempts INT4 default '0',
  failed_errors TEXT
  ,PRIMARY KEY (transfer_id));



DROP SEQUENCE seq_va_file_types CASCADE;
DROP TABLE va_file_types;

CREATE SEQUENCE seq_va_file_types START 5;
CREATE TABLE va_file_types (
  file_type_id INT4 NOT NULL DEFAULT nextval('seq_va_file_types'),
  type_order INT4 default '0',
  type_name VARCHAR(255),
  extension_mask TEXT
  ,PRIMARY KEY (file_type_id));



DROP SEQUENCE seq_va_filters CASCADE;
DROP TABLE va_filters;

CREATE SEQUENCE seq_va_filters START 2;
CREATE TABLE va_filters (
  filter_id INT4 NOT NULL DEFAULT nextval('seq_va_filters'),
  filter_type VARCHAR(32),
  filter_name VARCHAR(255),
  filter_desc TEXT
  ,PRIMARY KEY (filter_id));



DROP SEQUENCE seq_va_filters_properties CASCADE;
DROP TABLE va_filters_properties;

CREATE SEQUENCE seq_va_filters_properties START 5;
CREATE TABLE va_filters_properties (
  property_id INT4 NOT NULL DEFAULT nextval('seq_va_filters_properties'),
  filter_id INT4 NOT NULL default '0',
  property_order INT4 NOT NULL default '1',
  property_name VARCHAR(255),
  property_type VARCHAR(64),
  property_value VARCHAR(255),
  filter_from_sql TEXT,
  filter_join_sql TEXT,
  filter_where_sql TEXT,
  list_group_fields TEXT,
  list_group_where TEXT,
  list_table VARCHAR(64),
  list_field_id VARCHAR(64),
  list_field_title VARCHAR(64),
  list_field_total VARCHAR(64),
  list_sql TEXT
  ,PRIMARY KEY (property_id));

CREATE INDEX va_filters_properties_filter_id ON va_filters_properties (filter_id);


DROP SEQUENCE seq_va_filters_properties_values CASCADE;
DROP TABLE va_filters_properties_values;

CREATE SEQUENCE seq_va_filters_properties_values START 18;
CREATE TABLE va_filters_properties_values (
  value_id INT4 NOT NULL DEFAULT nextval('seq_va_filters_properties_values'),
  property_id INT4 default '0',
  value_order INT4,
  list_value_id VARCHAR(128),
  list_value_title VARCHAR(255),
  filter_where_sql TEXT
  ,PRIMARY KEY (value_id));

CREATE INDEX va_filters_properties_value_24 ON va_filters_properties_values (property_id);


DROP SEQUENCE seq_va_forum CASCADE;
DROP TABLE va_forum;

CREATE SEQUENCE seq_va_forum START 2;
CREATE TABLE va_forum (
  thread_id INT4 NOT NULL DEFAULT nextval('seq_va_forum'),
  forum_id INT4 NOT NULL default '0',
  priority_id INT4 default '0',
  user_id INT4 NOT NULL default '0',
  user_name VARCHAR(64) NOT NULL,
  user_email VARCHAR(128) NOT NULL,
  remote_address VARCHAR(32),
  topic VARCHAR(255) NOT NULL,
  friendly_url VARCHAR(255),
  description TEXT NOT NULL,
  email_notification INT4,
  views INT4,
  replies INT4,
  date_added TIMESTAMP NOT NULL,
  date_modified TIMESTAMP,
  thread_updated TIMESTAMP NOT NULL,
  admin_id_added_by INT4 default '0',
  admin_id_modified_by INT4 default '0',
  last_post_added TIMESTAMP,
  last_post_user_id INT4 default '0',
  last_post_admin_id INT4 default '0',
  last_post_message_id INT4 default '0'
  ,PRIMARY KEY (thread_id));

CREATE INDEX va_forum_forum_id ON va_forum (forum_id);
CREATE INDEX va_forum_friendly_url ON va_forum (friendly_url);
CREATE INDEX va_forum_priority_id ON va_forum (priority_id);


DROP SEQUENCE seq_va_forum_attachments CASCADE;
DROP TABLE va_forum_attachments;

CREATE SEQUENCE seq_va_forum_attachments START 1;
CREATE TABLE va_forum_attachments (
  attachment_id INT4 NOT NULL DEFAULT nextval('seq_va_forum_attachments'),
  forum_id INT4 NOT NULL default '0',
  thread_id INT4 default '0',
  message_id INT4 default '0',
  admin_id INT4 default '0',
  user_id INT4 default '0',
  session_id VARCHAR(32),
  attachment_status INT4 default '0',
  file_name VARCHAR(255),
  file_path VARCHAR(255),
  date_added TIMESTAMP
  ,PRIMARY KEY (attachment_id));

CREATE INDEX va_forum_attachments_admin_id ON va_forum_attachments (admin_id);
CREATE INDEX va_forum_attachments_message_id ON va_forum_attachments (message_id);
CREATE INDEX va_forum_attachments_session_id ON va_forum_attachments (session_id);
CREATE INDEX va_forum_attachments_support_id ON va_forum_attachments (forum_id);
CREATE INDEX va_forum_attachments_thread_id ON va_forum_attachments (thread_id);
CREATE INDEX va_forum_attachments_user_id ON va_forum_attachments (user_id);


DROP SEQUENCE seq_va_forum_categories CASCADE;
DROP TABLE va_forum_categories;

CREATE SEQUENCE seq_va_forum_categories START 2;
CREATE TABLE va_forum_categories (
  category_id INT4 NOT NULL DEFAULT nextval('seq_va_forum_categories'),
  category_name VARCHAR(255) NOT NULL,
  friendly_url VARCHAR(255),
  category_order INT4 default '0',
  short_description TEXT,
  full_description TEXT,
  allowed_view SMALLINT NOT NULL default '0',
  sites_all SMALLINT default '1'
  ,PRIMARY KEY (category_id));

CREATE INDEX va_forum_categories_friendly_url ON va_forum_categories (friendly_url);
CREATE INDEX va_forum_categories_sites_all ON va_forum_categories (sites_all);


DROP SEQUENCE seq_va_forum_categories_sites CASCADE;
DROP TABLE va_forum_categories_sites;

CREATE SEQUENCE seq_va_forum_categories_sites START 1;
CREATE TABLE va_forum_categories_sites (
  category_id INT4 NOT NULL default '0',
  site_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (category_id,site_id));



DROP SEQUENCE seq_va_forum_list CASCADE;
DROP TABLE va_forum_list;

CREATE SEQUENCE seq_va_forum_list START 2;
CREATE TABLE va_forum_list (
  forum_id INT4 NOT NULL DEFAULT nextval('seq_va_forum_list'),
  category_id INT4 NOT NULL default '0',
  forum_name VARCHAR(255) NOT NULL,
  friendly_url VARCHAR(255),
  forum_order INT4 NOT NULL default '0',
  short_description TEXT,
  full_description TEXT,
  small_image VARCHAR(255),
  large_image VARCHAR(255),
  date_added TIMESTAMP NOT NULL,
  threads_number INT4 NOT NULL default '0',
  messages_number INT4 NOT NULL default '0',
  last_post_added TIMESTAMP,
  last_post_user_id INT4 default '0',
  last_post_admin_id INT4 default '0',
  last_post_thread_id INT4 default '0',
  last_post_message_id INT4 default '0',
  is_rss SMALLINT default '0',
  rss_limit INT4,
  rss_on_breadcrumb SMALLINT default '0',
  rss_on_list SMALLINT default '0',
  access_level SMALLINT default '7',
  guest_access_level SMALLINT default '7',
  admin_access_level SMALLINT default '7'
  ,PRIMARY KEY (forum_id));

CREATE INDEX va_forum_list_access_level ON va_forum_list (guest_access_level);
CREATE INDEX va_forum_list_admin_access_level ON va_forum_list (admin_access_level);
CREATE INDEX va_forum_list_friendly_url ON va_forum_list (friendly_url);
CREATE INDEX va_forum_list_subscription__25 ON va_forum_list (access_level);


DROP SEQUENCE seq_va_forum_messages CASCADE;
DROP TABLE va_forum_messages;

CREATE SEQUENCE seq_va_forum_messages START 3;
CREATE TABLE va_forum_messages (
  message_id INT4 NOT NULL DEFAULT nextval('seq_va_forum_messages'),
  thread_id INT4 NOT NULL default '0',
  admin_id INT4,
  user_name VARCHAR(64) NOT NULL,
  user_email VARCHAR(128) NOT NULL,
  remote_address VARCHAR(32),
  message_text TEXT NOT NULL,
  date_added TIMESTAMP NOT NULL,
  date_modified TIMESTAMP,
  user_id INT4 NOT NULL default '0',
  admin_id_modified_by INT4 default '0'
  ,PRIMARY KEY (message_id));



DROP SEQUENCE seq_va_forum_moderators CASCADE;
DROP TABLE va_forum_moderators;

CREATE SEQUENCE seq_va_forum_moderators START 1;
CREATE TABLE va_forum_moderators (
  admin_id INT4 NOT NULL default '0',
  forum_id INT4 NOT NULL default '0',
  is_default_forum SMALLINT default '0'
  ,PRIMARY KEY (admin_id,forum_id));



DROP SEQUENCE seq_va_forum_priorities CASCADE;
DROP TABLE va_forum_priorities;

CREATE SEQUENCE seq_va_forum_priorities START 3;
CREATE TABLE va_forum_priorities (
  priority_id INT4 NOT NULL DEFAULT nextval('seq_va_forum_priorities'),
  priority_name VARCHAR(255),
  priority_rank INT4 default '0',
  html_before_title TEXT,
  html_after_title TEXT,
  is_default SMALLINT default '0'
  ,PRIMARY KEY (priority_id));



DROP SEQUENCE seq_va_forum_subscriptions CASCADE;
DROP TABLE va_forum_subscriptions;

CREATE SEQUENCE seq_va_forum_subscriptions START 1;
CREATE TABLE va_forum_subscriptions (
  forum_id INT4 NOT NULL default '0',
  subscription_id INT4 NOT NULL default '0',
  access_level SMALLINT default '0'
  ,PRIMARY KEY (forum_id,subscription_id));



DROP SEQUENCE seq_va_forum_user_types CASCADE;
DROP TABLE va_forum_user_types;

CREATE SEQUENCE seq_va_forum_user_types START 1;
CREATE TABLE va_forum_user_types (
  forum_id INT4 NOT NULL default '0',
  user_type_id INT4 NOT NULL default '0',
  access_level SMALLINT default '0'
  ,PRIMARY KEY (forum_id,user_type_id));



DROP SEQUENCE seq_va_friendly_urls CASCADE;
DROP TABLE va_friendly_urls;

CREATE SEQUENCE seq_va_friendly_urls START 1;
CREATE TABLE va_friendly_urls (
  friendly_id INT4 NOT NULL DEFAULT nextval('seq_va_friendly_urls'),
  script_name VARCHAR(255),
  friendly_url VARCHAR(255),
  sites_all SMALLINT default '1'
  ,PRIMARY KEY (friendly_id));

CREATE INDEX va_friendly_urls_friendly_url ON va_friendly_urls (friendly_url);
CREATE INDEX va_friendly_urls_script_name ON va_friendly_urls (script_name);
CREATE INDEX va_friendly_urls_sites_all ON va_friendly_urls (sites_all);


DROP SEQUENCE seq_va_friendly_urls_sites CASCADE;
DROP TABLE va_friendly_urls_sites;

CREATE SEQUENCE seq_va_friendly_urls_sites START 1;
CREATE TABLE va_friendly_urls_sites (
  friendly_id INT4 NOT NULL default '0',
  site_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (friendly_id,site_id));



DROP SEQUENCE seq_va_global_settings CASCADE;
DROP TABLE va_global_settings;

CREATE SEQUENCE seq_va_global_settings START 1;
CREATE TABLE va_global_settings (
  site_id INT4 NOT NULL default '1',
  setting_type VARCHAR(32) NOT NULL,
  setting_name VARCHAR(64) NOT NULL,
  setting_value TEXT
  ,PRIMARY KEY (site_id,setting_type,setting_name));



DROP SEQUENCE seq_va_google_base_attributes CASCADE;
DROP TABLE va_google_base_attributes;

CREATE SEQUENCE seq_va_google_base_attributes START 50;
CREATE TABLE va_google_base_attributes (
  attribute_id INT4 NOT NULL DEFAULT nextval('seq_va_google_base_attributes'),
  attribute_name VARCHAR(255),
  attribute_type VARCHAR(16),
  value_type VARCHAR(32)
  ,PRIMARY KEY (attribute_id));



DROP SEQUENCE seq_va_google_base_types CASCADE;
DROP TABLE va_google_base_types;

CREATE SEQUENCE seq_va_google_base_types START 23;
CREATE TABLE va_google_base_types (
  type_id INT4 NOT NULL DEFAULT nextval('seq_va_google_base_types'),
  type_name VARCHAR(255)
  ,PRIMARY KEY (type_id));



DROP SEQUENCE seq_va_google_base_types_attributes CASCADE;
DROP TABLE va_google_base_types_attributes;

CREATE SEQUENCE seq_va_google_base_types_attributes START 1;
CREATE TABLE va_google_base_types_attributes (
  type_id INT4 NOT NULL default '0',
  attribute_id INT4 NOT NULL default '0',
  required SMALLINT default '0'
  ,PRIMARY KEY (type_id,attribute_id));



DROP SEQUENCE seq_va_icons CASCADE;
DROP TABLE va_icons;

CREATE SEQUENCE seq_va_icons START 30;
CREATE TABLE va_icons (
  icon_id INT4 NOT NULL DEFAULT nextval('seq_va_icons'),
  is_active SMALLINT default '1',
  show_for_user SMALLINT default '1',
  icon_order INT4 default '1',
  icon_code VARCHAR(32),
  icon_image VARCHAR(255),
  icon_width INT4,
  icon_height INT4,
  icon_name VARCHAR(255)
  ,PRIMARY KEY (icon_id));



DROP SEQUENCE seq_va_ips_countries CASCADE;
DROP TABLE va_ips_countries;

CREATE SEQUENCE seq_va_ips_countries START 1;
CREATE TABLE va_ips_countries (
  ip_start INT4 NOT NULL default '0',
  ip_end INT4 NOT NULL default '0',
  country_code VARCHAR(4)
  ,PRIMARY KEY (ip_start,ip_end));



DROP SEQUENCE seq_va_issue_numbers CASCADE;
DROP TABLE va_issue_numbers;

CREATE SEQUENCE seq_va_issue_numbers START 1;
CREATE TABLE va_issue_numbers (
  issue_number INT4 NOT NULL default '0'
  ,PRIMARY KEY (issue_number));



DROP SEQUENCE seq_va_item_types CASCADE;
DROP TABLE va_item_types;

CREATE SEQUENCE seq_va_item_types START 9;
CREATE TABLE va_item_types (
  item_type_id INT4 NOT NULL DEFAULT nextval('seq_va_item_types'),
  google_base_type_id INT4 default '0',
  item_type_name VARCHAR(50),
  is_gift_voucher SMALLINT default '0',
  is_bundle SMALLINT default '0',
  is_user SMALLINT default '0',
  reward_type INT4,
  reward_amount FLOAT4,
  credit_reward_type SMALLINT,
  credit_reward_amount FLOAT4,
  affiliate_commission_type INT4,
  affiliate_commission_amount FLOAT4,
  merchant_fee_type INT4,
  merchant_fee_amount FLOAT4
  ,PRIMARY KEY (item_type_id));

CREATE INDEX va_item_types_google_base_t_26 ON va_item_types (google_base_type_id);


DROP SEQUENCE seq_va_items CASCADE;
DROP TABLE va_items;

CREATE SEQUENCE seq_va_items START 53;
CREATE TABLE va_items (
  item_id INT4 NOT NULL DEFAULT nextval('seq_va_items'),
  item_code VARCHAR(64),
  item_name VARCHAR(255) NOT NULL,
  friendly_url VARCHAR(255),
  item_order INT4 NOT NULL default '1',
  item_type_id INT4 NOT NULL default '0',
  google_base_type_id INT4 default '0',
  issue_date TIMESTAMP,
  language_code VARCHAR(4),
  manufacturer_id INT4 default '0',
  manufacturer_code VARCHAR(255),
  supplier_id INT4 default '0',
  author_id INT4,
  template_name VARCHAR(255),
  hide_add_list SMALLINT default '0',
  hide_add_table SMALLINT default '0',
  hide_add_grid SMALLINT default '0',
  hide_add_details SMALLINT default '0',
  features TEXT,
  highlights TEXT,
  short_description TEXT,
  full_desc_type SMALLINT,
  full_description TEXT,
  is_special_offer SMALLINT NOT NULL default '0',
  special_order INT4 default '1',
  special_offer TEXT,
  notes TEXT,
  tiny_image VARCHAR(255),
  tiny_image_alt VARCHAR(255),
  small_image VARCHAR(255),
  small_image_alt VARCHAR(255),
  big_image VARCHAR(255),
  big_image_alt VARCHAR(255),
  super_image VARCHAR(255),
  super_image_alt VARCHAR(255),
  preview_url VARCHAR(255),
  preview_width INT4,
  preview_height INT4,
  specification VARCHAR(255),
  buying_price FLOAT4 default '0',
  buying_price_id INT4 default '0',
  price FLOAT4 NOT NULL default '0',
  price_id INT4 default '0',
  trade_price FLOAT4 NOT NULL default '0',
  trade_price_id INT4 default '0',
  is_price_edit SMALLINT default '0',
  properties_price FLOAT4 default '0',
  properties_price_id INT4 default '0',
  trade_properties_price FLOAT4 default '0',
  trade_properties_price_id INT4 default '0',
  tax_id INT4 default '0',
  tax_free SMALLINT default '0',
  width FLOAT4 default '0',
  height FLOAT4 default '0',
  length FLOAT4 default '0',
  weight FLOAT4,
  packages_number FLOAT4,
  min_quantity INT4,
  max_quantity INT4,
  quantity_increment INT4,
  stock_level INT4,
  use_stock_level INT4 NOT NULL default '0',
  hide_out_of_stock SMALLINT NOT NULL default '0',
  disable_out_of_stock SMALLINT default '0',
  is_shipping_free SMALLINT default '0',
  is_separate_shipping SMALLINT default '0',
  is_shipping_required SMALLINT default '0',
  shipping_in_stock INT4,
  shipping_out_stock INT4 default '0',
  shipping_rule_id INT4,
  shipping_cost FLOAT4,
  shipping_trade_cost FLOAT4,
  shipping_modules_default SMALLINT default '1',
  shipping_modules_ids TEXT,
  is_sales SMALLINT,
  sales_price FLOAT4 NOT NULL default '0',
  sales_price_id INT4 default '0',
  trade_sales FLOAT4 NOT NULL default '0',
  trade_sales_id INT4 default '0',
  discount_percent FLOAT4,
  is_points_price SMALLINT default '0',
  points_price FLOAT4,
  reward_type INT4,
  reward_amount FLOAT4,
  credit_reward_type SMALLINT,
  credit_reward_amount FLOAT4,
  is_recurring SMALLINT default '0',
  recurring_price FLOAT4,
  recurring_period INT4,
  recurring_interval INT4,
  recurring_payments_total INT4,
  recurring_start_date TIMESTAMP,
  recurring_end_date TIMESTAMP,
  affiliate_commission_type INT4,
  affiliate_commission_amount FLOAT4,
  merchant_fee_type INT4,
  merchant_fee_amount FLOAT4,
  downloadable SMALLINT,
  download_period INT4,
  download_path TEXT,
  download_show_terms INT4 default '0',
  download_terms_text TEXT,
  generate_serial INT4 default '0',
  serial_period INT4,
  activations_number INT4 default '0',
  is_showing SMALLINT NOT NULL default '1',
  is_approved SMALLINT NOT NULL default '1',
  is_new SMALLINT default '0',
  is_keywords SMALLINT default '0',
  sites_all SMALLINT default '1',
  access_level SMALLINT default '7',
  guest_access_level SMALLINT default '7',
  admin_access_level SMALLINT default '7',
  is_compared SMALLINT default '0',
  buy_link VARCHAR(255),
  total_views INT4 default '0',
  votes INT4,
  points INT4,
  rating FLOAT4,
  a_title VARCHAR(255),
  meta_title VARCHAR(255),
  meta_keywords VARCHAR(255),
  meta_description VARCHAR(255),
  mail_notify INT4 default '0',
  mail_to VARCHAR(255),
  mail_from VARCHAR(64),
  mail_cc VARCHAR(255),
  mail_bcc VARCHAR(255),
  mail_reply_to VARCHAR(64),
  mail_return_path VARCHAR(64),
  mail_type INT4 default '0',
  mail_subject VARCHAR(255),
  mail_body TEXT,
  sms_notify INT4 default '0',
  sms_recipient VARCHAR(255),
  sms_originator VARCHAR(64),
  sms_message TEXT,
  user_id INT4 default '0',
  is_draft SMALLINT default '0',
  draft_parent_id INT4 default '0',
  admin_id_added_by INT4 default '0',
  admin_id_modified_by INT4 default '0',
  date_added TIMESTAMP,
  date_modified TIMESTAMP
  ,PRIMARY KEY (item_id));

CREATE INDEX va_items_access_level ON va_items (access_level);
CREATE INDEX va_items_admin_access_level ON va_items (admin_access_level);
CREATE INDEX va_items_buying_price_id ON va_items (buying_price_id);
CREATE INDEX va_items_draft_parent_id ON va_items (draft_parent_id);
CREATE INDEX va_items_friendly_url ON va_items (friendly_url);
CREATE INDEX va_items_google_base_type_id ON va_items (google_base_type_id);
CREATE INDEX va_items_guest_access_level ON va_items (guest_access_level);
CREATE INDEX va_items_hide_out_of_stock ON va_items (hide_out_of_stock);
CREATE INDEX va_items_is_approved ON va_items (is_approved);
CREATE INDEX va_items_is_draft ON va_items (is_draft);
CREATE INDEX va_items_is_showing ON va_items (is_showing);
CREATE INDEX va_items_is_special_offer ON va_items (is_special_offer);
CREATE INDEX va_items_item_code ON va_items (item_code);
CREATE INDEX va_items_item_name ON va_items (item_name);
CREATE INDEX va_items_language_code ON va_items (language_code);
CREATE INDEX va_items_manufacturer_id ON va_items (manufacturer_id);
CREATE INDEX va_items_price ON va_items (price);
CREATE INDEX va_items_price_id ON va_items (price_id);
CREATE INDEX va_items_properties_price_id ON va_items (properties_price_id);
CREATE INDEX va_items_sales_price ON va_items (sales_price);
CREATE INDEX va_items_sales_price_id ON va_items (sales_price_id);
CREATE INDEX va_items_sites_all ON va_items (sites_all);
CREATE INDEX va_items_special_order ON va_items (special_order);
CREATE INDEX va_items_supplier_id ON va_items (supplier_id);
CREATE INDEX va_items_trade_price ON va_items (trade_price);
CREATE INDEX va_items_trade_price_id ON va_items (trade_price_id);
CREATE INDEX va_items_trade_properties_p_27 ON va_items (trade_properties_price_id);
CREATE INDEX va_items_trade_sales ON va_items (trade_sales);
CREATE INDEX va_items_trade_sales_id ON va_items (trade_sales_id);
CREATE INDEX va_items_user_id ON va_items (user_id);


DROP SEQUENCE seq_va_items_accessories CASCADE;
DROP TABLE va_items_accessories;

CREATE SEQUENCE seq_va_items_accessories START 1;
CREATE TABLE va_items_accessories (
  item_id INT4 NOT NULL default '0',
  accessory_id INT4 NOT NULL default '0',
  accessory_order INT4 NOT NULL default '1'
  ,PRIMARY KEY (item_id,accessory_id));



DROP SEQUENCE seq_va_items_categories CASCADE;
DROP TABLE va_items_categories;

CREATE SEQUENCE seq_va_items_categories START 1;
CREATE TABLE va_items_categories (
  item_id INT4 NOT NULL default '0',
  category_id INT4 NOT NULL default '0',
  item_order INT4 default '1'
  ,PRIMARY KEY (item_id,category_id));



DROP SEQUENCE seq_va_items_downloads CASCADE;
DROP TABLE va_items_downloads;

CREATE SEQUENCE seq_va_items_downloads START 1;
CREATE TABLE va_items_downloads (
  download_id INT4 NOT NULL DEFAULT nextval('seq_va_items_downloads'),
  order_id INT4 NOT NULL default '0',
  user_id INT4 NOT NULL default '0',
  order_item_id INT4 NOT NULL default '0',
  item_id INT4 NOT NULL default '0',
  download_notes VARCHAR(255),
  download_path VARCHAR(255),
  activated SMALLINT,
  max_downloads INT4,
  download_added TIMESTAMP NOT NULL,
  download_expiry TIMESTAMP,
  download_limit INT4
  ,PRIMARY KEY (download_id));

CREATE INDEX va_items_downloads_item_id ON va_items_downloads (item_id);
CREATE INDEX va_items_downloads_order_id ON va_items_downloads (order_id);
CREATE INDEX va_items_downloads_order_item_id ON va_items_downloads (order_item_id);
CREATE INDEX va_items_downloads_user_id ON va_items_downloads (user_id);


DROP SEQUENCE seq_va_items_downloads_statistic CASCADE;
DROP TABLE va_items_downloads_statistic;

CREATE SEQUENCE seq_va_items_downloads_statistic START 1;
CREATE TABLE va_items_downloads_statistic (
  download_id INT4 NOT NULL default '0',
  order_item_id INT4 NOT NULL default '0',
  remote_address VARCHAR(32) NOT NULL,
  downloaded_date TIMESTAMP NOT NULL
);

CREATE INDEX va_items_downloads_statisti_28 ON va_items_downloads_statistic (download_id);
CREATE INDEX va_items_downloads_statisti_29 ON va_items_downloads_statistic (downloaded_date);
CREATE INDEX va_items_downloads_statisti_30 ON va_items_downloads_statistic (order_item_id);
CREATE INDEX va_items_downloads_statisti_31 ON va_items_downloads_statistic (remote_address);


DROP SEQUENCE seq_va_items_files CASCADE;
DROP TABLE va_items_files;

CREATE SEQUENCE seq_va_items_files START 1;
CREATE TABLE va_items_files (
  file_id INT4 NOT NULL DEFAULT nextval('seq_va_items_files'),
  item_id INT4 default '0',
  item_type_id INT4 default '0',
  download_type SMALLINT default '1',
  download_title VARCHAR(255),
  download_path VARCHAR(255),
  download_period SMALLINT,
  download_interval INT4,
  download_limit INT4,
  preview_type SMALLINT default '0',
  preview_title VARCHAR(255),
  preview_path VARCHAR(255),
  preview_image VARCHAR(255),
  preview_position SMALLINT default '0'
  ,PRIMARY KEY (file_id));

CREATE INDEX va_items_files_item_id ON va_items_files (item_id);
CREATE INDEX va_items_files_item_type_id ON va_items_files (item_type_id);


DROP SEQUENCE seq_va_items_forum_topics CASCADE;
DROP TABLE va_items_forum_topics;

CREATE SEQUENCE seq_va_items_forum_topics START 1;
CREATE TABLE va_items_forum_topics (
  item_id INT4 NOT NULL default '0',
  thread_id INT4 NOT NULL default '0',
  item_order INT4 default '1',
  thread_order INT4 default '1'
  ,PRIMARY KEY (item_id,thread_id));



DROP SEQUENCE seq_va_items_images CASCADE;
DROP TABLE va_items_images;

CREATE SEQUENCE seq_va_items_images START 43;
CREATE TABLE va_items_images (
  image_id INT4 NOT NULL DEFAULT nextval('seq_va_items_images'),
  item_id INT4 NOT NULL default '0',
  is_default SMALLINT default '0',
  image_order INT4 default '0',
  image_position SMALLINT default '1',
  image_title VARCHAR(255),
  image_tiny VARCHAR(255),
  image_tiny_alt VARCHAR(255),
  image_small VARCHAR(255) NOT NULL,
  image_small_alt VARCHAR(255),
  small_width INT4,
  small_height INT4,
  image_large VARCHAR(255),
  image_large_alt VARCHAR(255),
  image_super VARCHAR(255),
  image_super_alt VARCHAR(255),
  image_description TEXT
  ,PRIMARY KEY (image_id));

CREATE INDEX va_items_images_item_id ON va_items_images (item_id);


DROP SEQUENCE seq_va_items_prices CASCADE;
DROP TABLE va_items_prices;

CREATE SEQUENCE seq_va_items_prices START 11;
CREATE TABLE va_items_prices (
  price_id INT4 NOT NULL DEFAULT nextval('seq_va_items_prices'),
  item_id INT4 default '0',
  is_active INT4 default '0',
  min_quantity INT4 default '1',
  max_quantity INT4 default '1',
  price FLOAT4 default '0',
  user_type_id INT4 default '0',
  site_id INT4 default '0',
  discount_action INT4 default '0',
  properties_discount FLOAT4 default '0'
  ,PRIMARY KEY (price_id));

CREATE INDEX va_items_prices_item_id ON va_items_prices (item_id);
CREATE INDEX va_items_prices_site_id ON va_items_prices (site_id);
CREATE INDEX va_items_prices_user_type_id ON va_items_prices (user_type_id);


DROP SEQUENCE seq_va_items_properties CASCADE;
DROP TABLE va_items_properties;

CREATE SEQUENCE seq_va_items_properties START 9;
CREATE TABLE va_items_properties (
  property_id INT4 NOT NULL DEFAULT nextval('seq_va_items_properties'),
  property_type_id INT4 default '1',
  parent_property_id INT4,
  parent_value_id INT4,
  show_for_user SMALLINT default '1',
  usage_type SMALLINT default '1',
  item_type_id INT4 default '0',
  item_id INT4 NOT NULL default '0',
  sub_item_id INT4 default '0',
  additional_price FLOAT4,
  trade_additional_price FLOAT4,
  percentage_price_type SMALLINT default '1',
  percentage_property_id INT4,
  quantity INT4,
  quantity_action SMALLINT default '1',
  property_order INT4 NOT NULL default '1',
  property_step INT4 default '1',
  property_name VARCHAR(255) NOT NULL,
  property_hint VARCHAR(255),
  hide_name SMALLINT default '0',
  property_code VARCHAR(32),
  property_description TEXT,
  property_class VARCHAR(64),
  property_style TEXT,
  property_price_type SMALLINT,
  free_price_type SMALLINT,
  free_price_amount FLOAT4,
  max_limit_type SMALLINT,
  max_limit_length INT4,
  length_units VARCHAR(8),
  use_on_list SMALLINT default '1',
  use_on_table SMALLINT default '1',
  use_on_grid SMALLINT default '0',
  use_on_details SMALLINT default '1',
  use_on_checkout SMALLINT default '0',
  use_on_second SMALLINT default '0',
  control_type VARCHAR(16) NOT NULL,
  control_style VARCHAR(255),
  control_code TEXT,
  onchange_code TEXT,
  onclick_code TEXT,
  required SMALLINT default '0',
  start_html TEXT,
  middle_html TEXT,
  before_control_html TEXT,
  after_control_html TEXT,
  end_html TEXT,
  sites_all SMALLINT default '1'
  ,PRIMARY KEY (property_id));

CREATE INDEX va_items_properties_item_id ON va_items_properties (item_id);
CREATE INDEX va_items_properties_item_type_id ON va_items_properties (item_type_id);
CREATE INDEX va_items_properties_propert_32 ON va_items_properties (property_code);
CREATE INDEX va_items_properties_propert_33 ON va_items_properties (property_type_id);
CREATE INDEX va_items_properties_sites_all ON va_items_properties (sites_all);
CREATE INDEX va_items_properties_sub_item_id ON va_items_properties (sub_item_id);


DROP SEQUENCE seq_va_items_properties_assigned CASCADE;
DROP TABLE va_items_properties_assigned;

CREATE SEQUENCE seq_va_items_properties_assigned START 1;
CREATE TABLE va_items_properties_assigned (
  item_id INT4 NOT NULL default '0',
  property_id INT4 NOT NULL default '0',
  property_description TEXT
  ,PRIMARY KEY (item_id,property_id));



DROP SEQUENCE seq_va_items_properties_sites CASCADE;
DROP TABLE va_items_properties_sites;

CREATE SEQUENCE seq_va_items_properties_sites START 1;
CREATE TABLE va_items_properties_sites (
  property_id INT4 NOT NULL default '0',
  site_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (property_id,site_id));



DROP SEQUENCE seq_va_items_properties_sizes CASCADE;
DROP TABLE va_items_properties_sizes;

CREATE SEQUENCE seq_va_items_properties_sizes START 1;
CREATE TABLE va_items_properties_sizes (
  size_id INT4 NOT NULL DEFAULT nextval('seq_va_items_properties_sizes'),
  property_id INT4 default '0',
  width FLOAT4 default '0',
  height FLOAT4 default '0',
  price FLOAT4 default '0'
  ,PRIMARY KEY (size_id));

CREATE INDEX va_items_properties_sizes_p_34 ON va_items_properties_sizes (property_id);


DROP SEQUENCE seq_va_items_properties_values CASCADE;
DROP TABLE va_items_properties_values;

CREATE SEQUENCE seq_va_items_properties_values START 16;
CREATE TABLE va_items_properties_values (
  item_property_id INT4 NOT NULL DEFAULT nextval('seq_va_items_properties_values'),
  property_id INT4 NOT NULL default '0',
  sub_item_id INT4 default '0',
  property_value VARCHAR(255) NOT NULL,
  value_order INT4 default '1',
  item_code VARCHAR(64),
  manufacturer_code VARCHAR(64),
  download_period INT4,
  download_path VARCHAR(255),
  download_files_ids TEXT,
  buying_price FLOAT4,
  additional_price FLOAT4,
  trade_additional_price FLOAT4,
  quantity INT4,
  percentage_price FLOAT4,
  additional_weight FLOAT4,
  use_stock_level SMALLINT default '0',
  stock_level INT4,
  hide_out_of_stock SMALLINT default '0',
  hide_value SMALLINT NOT NULL default '0',
  is_default_value SMALLINT default '0',
  image_tiny VARCHAR(255),
  image_small VARCHAR(255),
  image_large VARCHAR(255),
  image_super VARCHAR(255)
  ,PRIMARY KEY (item_property_id));

CREATE INDEX va_items_properties_values__35 ON va_items_properties_values (hide_value);
CREATE INDEX va_items_properties_values__36 ON va_items_properties_values (property_id);
CREATE INDEX va_items_properties_values__37 ON va_items_properties_values (sub_item_id);


DROP SEQUENCE seq_va_items_related CASCADE;
DROP TABLE va_items_related;

CREATE SEQUENCE seq_va_items_related START 1;
CREATE TABLE va_items_related (
  item_id INT4 NOT NULL default '0',
  related_id INT4 NOT NULL default '0',
  related_order INT4 NOT NULL default '1'
);

CREATE INDEX va_items_related_item_id ON va_items_related (item_id);
CREATE INDEX va_items_related_related_id ON va_items_related (related_id);


DROP SEQUENCE seq_va_items_serials CASCADE;
DROP TABLE va_items_serials;

CREATE SEQUENCE seq_va_items_serials START 1;
CREATE TABLE va_items_serials (
  serial_id INT4 NOT NULL DEFAULT nextval('seq_va_items_serials'),
  item_id INT4 default '0',
  serial_number VARCHAR(128),
  used SMALLINT default '0'
  ,PRIMARY KEY (serial_id));

CREATE INDEX va_items_serials_item_id ON va_items_serials (item_id);


DROP SEQUENCE seq_va_items_sites CASCADE;
DROP TABLE va_items_sites;

CREATE SEQUENCE seq_va_items_sites START 1;
CREATE TABLE va_items_sites (
  item_id INT4 NOT NULL default '0',
  site_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (item_id,site_id));



DROP SEQUENCE seq_va_items_subscriptions CASCADE;
DROP TABLE va_items_subscriptions;

CREATE SEQUENCE seq_va_items_subscriptions START 1;
CREATE TABLE va_items_subscriptions (
  item_id INT4 NOT NULL default '0',
  subscription_id INT4 NOT NULL default '0',
  access_level SMALLINT default '0'
  ,PRIMARY KEY (item_id,subscription_id));



DROP SEQUENCE seq_va_items_tabs CASCADE;
DROP TABLE va_items_tabs;

CREATE SEQUENCE seq_va_items_tabs START 1;
CREATE TABLE va_items_tabs (
  tab_id INT4 NOT NULL DEFAULT nextval('seq_va_items_tabs'),
  item_id INT4 default '0',
  tab_order INT4 default '1',
  tab_title VARCHAR(50),
  tab_desc TEXT,
  hide_tab SMALLINT default '0'
  ,PRIMARY KEY (tab_id));

CREATE INDEX va_items_tabs_item_id ON va_items_tabs (item_id);


DROP SEQUENCE seq_va_items_user_types CASCADE;
DROP TABLE va_items_user_types;

CREATE SEQUENCE seq_va_items_user_types START 1;
CREATE TABLE va_items_user_types (
  item_id INT4 NOT NULL default '0',
  user_type_id INT4 NOT NULL default '0',
  access_level SMALLINT default '0'
  ,PRIMARY KEY (item_id,user_type_id));



DROP SEQUENCE seq_va_items_values_assigned CASCADE;
DROP TABLE va_items_values_assigned;

CREATE SEQUENCE seq_va_items_values_assigned START 1;
CREATE TABLE va_items_values_assigned (
  item_id INT4 NOT NULL default '0',
  property_value_id INT4 NOT NULL default '0',
  is_default_value SMALLINT
  ,PRIMARY KEY (item_id,property_value_id));



DROP SEQUENCE seq_va_keywords CASCADE;
DROP TABLE va_keywords;

CREATE SEQUENCE seq_va_keywords START 1;
CREATE TABLE va_keywords (
  keyword_id INT4 NOT NULL DEFAULT nextval('seq_va_keywords'),
  keyword_name VARCHAR(255)
  ,PRIMARY KEY (keyword_id));



DROP SEQUENCE seq_va_keywords_items CASCADE;
DROP TABLE va_keywords_items;

CREATE SEQUENCE seq_va_keywords_items START 1;
CREATE TABLE va_keywords_items (
  item_id INT4 default '0',
  keyword_id INT4 default '0',
  field_id SMALLINT default '0',
  keyword_position SMALLINT default '0',
  keyword_rank SMALLINT default '0'
);

CREATE INDEX va_keywords_items_field_id ON va_keywords_items (field_id);
CREATE INDEX va_keywords_items_item_id ON va_keywords_items (item_id);
CREATE INDEX va_keywords_items_keyword_id ON va_keywords_items (keyword_id);
CREATE INDEX va_keywords_items_keyword_p_38 ON va_keywords_items (keyword_position);
CREATE INDEX va_keywords_items_keyword_rank ON va_keywords_items (keyword_rank);


DROP SEQUENCE seq_va_languages CASCADE;
DROP TABLE va_languages;

CREATE SEQUENCE seq_va_languages START 1;
CREATE TABLE va_languages (
  language_code VARCHAR(2) NOT NULL,
  language_order INT4 default '1',
  language_name VARCHAR(255) NOT NULL,
  show_for_user SMALLINT default '0',
  language_image VARCHAR(255),
  language_image_active VARCHAR(255),
  currency_code VARCHAR(4)
  ,PRIMARY KEY (language_code));



DROP SEQUENCE seq_va_layouts CASCADE;
DROP TABLE va_layouts;

CREATE SEQUENCE seq_va_layouts START 13;
CREATE TABLE va_layouts (
  layout_id INT4 NOT NULL DEFAULT nextval('seq_va_layouts'),
  sites_all SMALLINT default '1',
  show_for_user SMALLINT default '0',
  layout_name VARCHAR(64) NOT NULL,
  user_layout_name VARCHAR(255),
  top_menu_type INT4 default '1',
  style_name VARCHAR(64),
  scheme_name VARCHAR(64),
  templates_dir VARCHAR(255) NOT NULL,
  admin_templates_dir VARCHAR(255) NOT NULL,
  block_default_template VARCHAR(64),
  block_area_template VARCHAR(64),
  block_breadcrumb_template VARCHAR(64)
  ,PRIMARY KEY (layout_id));

CREATE INDEX va_layouts_sites_all ON va_layouts (sites_all);


DROP SEQUENCE seq_va_layouts_sites CASCADE;
DROP TABLE va_layouts_sites;

CREATE SEQUENCE seq_va_layouts_sites START 1;
CREATE TABLE va_layouts_sites (
  layout_id INT4 NOT NULL default '0',
  site_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (layout_id,site_id));



DROP SEQUENCE seq_va_manuals_articles CASCADE;
DROP TABLE va_manuals_articles;

CREATE SEQUENCE seq_va_manuals_articles START 1;
CREATE TABLE va_manuals_articles (
  article_id INT4 NOT NULL DEFAULT nextval('seq_va_manuals_articles'),
  manual_id INT4 default '0',
  parent_article_id INT4 default '0',
  alias_article_id INT4 default '0',
  article_path VARCHAR(255) NOT NULL,
  article_title VARCHAR(255),
  friendly_url VARCHAR(255),
  article_order INT4 default '0',
  section_number VARCHAR(255),
  image_small VARCHAR(255),
  image_small_alt VARCHAR(255),
  image_large VARCHAR(255),
  image_large_alt VARCHAR(255),
  short_description TEXT,
  full_description TEXT,
  allowed_view SMALLINT NOT NULL default '0',
  shown_in_contents SMALLINT NOT NULL default '0',
  meta_title VARCHAR(255),
  meta_keywords VARCHAR(255),
  meta_description VARCHAR(255),
  date_added TIMESTAMP,
  date_modified TIMESTAMP,
  admin_id_added_by INT4 default '0',
  admin_id_modified_by INT4 default '0'
  ,PRIMARY KEY (article_id));

CREATE INDEX va_manuals_articles_alias_a_39 ON va_manuals_articles (alias_article_id);
CREATE INDEX va_manuals_articles_article_path ON va_manuals_articles (article_path);
CREATE INDEX va_manuals_articles_friendly_url ON va_manuals_articles (friendly_url);
CREATE INDEX va_manuals_articles_manual_id ON va_manuals_articles (manual_id);
CREATE INDEX va_manuals_articles_parent__40 ON va_manuals_articles (parent_article_id);


DROP SEQUENCE seq_va_manuals_categories CASCADE;
DROP TABLE va_manuals_categories;

CREATE SEQUENCE seq_va_manuals_categories START 1;
CREATE TABLE va_manuals_categories (
  category_id INT4 NOT NULL DEFAULT nextval('seq_va_manuals_categories'),
  category_name VARCHAR(255),
  friendly_url VARCHAR(255),
  category_order INT4 default '0',
  short_description TEXT,
  full_description TEXT,
  sites_all SMALLINT default '1',
  access_level SMALLINT default '7',
  guest_access_level SMALLINT default '7',
  admin_access_level SMALLINT default '7',
  meta_title VARCHAR(255),
  meta_keywords VARCHAR(255),
  meta_description VARCHAR(255),
  date_added TIMESTAMP,
  date_modified TIMESTAMP,
  admin_id_added_by INT4 default '0',
  admin_id_modified_by INT4 default '0'
  ,PRIMARY KEY (category_id));

CREATE INDEX va_manuals_categories_admin_41 ON va_manuals_categories (admin_access_level);
CREATE INDEX va_manuals_categories_frien_42 ON va_manuals_categories (friendly_url);
CREATE INDEX va_manuals_categories_guest_43 ON va_manuals_categories (guest_access_level);
CREATE INDEX va_manuals_categories_sites_all ON va_manuals_categories (sites_all);
CREATE INDEX va_manuals_categories_subsc_44 ON va_manuals_categories (access_level);


DROP SEQUENCE seq_va_manuals_categories_sites CASCADE;
DROP TABLE va_manuals_categories_sites;

CREATE SEQUENCE seq_va_manuals_categories_sites START 1;
CREATE TABLE va_manuals_categories_sites (
  category_id INT4 NOT NULL default '0',
  site_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (category_id,site_id));



DROP SEQUENCE seq_va_manuals_categories_subscriptions CASCADE;
DROP TABLE va_manuals_categories_subscriptions;

CREATE SEQUENCE seq_va_manuals_categories_subscriptions START 1;
CREATE TABLE va_manuals_categories_subscriptions (
  category_id INT4 NOT NULL default '0',
  subscription_id INT4 NOT NULL default '0',
  access_level SMALLINT default '0'
  ,PRIMARY KEY (category_id,subscription_id));



DROP SEQUENCE seq_va_manuals_categories_types CASCADE;
DROP TABLE va_manuals_categories_types;

CREATE SEQUENCE seq_va_manuals_categories_types START 1;
CREATE TABLE va_manuals_categories_types (
  category_id INT4 NOT NULL default '0',
  user_type_id INT4 NOT NULL default '0',
  access_level SMALLINT default '0'
  ,PRIMARY KEY (category_id,user_type_id));



DROP SEQUENCE seq_va_manuals_list CASCADE;
DROP TABLE va_manuals_list;

CREATE SEQUENCE seq_va_manuals_list START 1;
CREATE TABLE va_manuals_list (
  manual_id INT4 NOT NULL DEFAULT nextval('seq_va_manuals_list'),
  category_id INT4 default '0',
  alias_manual_id INT4 default '0',
  manual_order INT4 default '0',
  manual_title VARCHAR(255),
  friendly_url VARCHAR(255),
  short_description TEXT,
  full_description TEXT,
  allowed_view SMALLINT default '0',
  meta_title VARCHAR(255),
  meta_keywords VARCHAR(255),
  meta_description VARCHAR(255),
  date_added TIMESTAMP,
  date_modified TIMESTAMP,
  admin_id_added_by INT4 default '0',
  admin_id_modified_by INT4 default '0'
  ,PRIMARY KEY (manual_id));

CREATE INDEX va_manuals_list_alias_manual_id ON va_manuals_list (alias_manual_id);
CREATE INDEX va_manuals_list_category_id ON va_manuals_list (category_id);
CREATE INDEX va_manuals_list_friendly_url ON va_manuals_list (friendly_url);


DROP SEQUENCE seq_va_manufacturers CASCADE;
DROP TABLE va_manufacturers;

CREATE SEQUENCE seq_va_manufacturers START 10;
CREATE TABLE va_manufacturers (
  manufacturer_id INT4 NOT NULL DEFAULT nextval('seq_va_manufacturers'),
  manufacturer_name VARCHAR(255) NOT NULL,
  manufacturer_order INT4 NOT NULL default '1',
  friendly_url VARCHAR(255),
  affiliate_code VARCHAR(255),
  short_description TEXT,
  full_description TEXT,
  image_small VARCHAR(255),
  image_small_alt VARCHAR(255),
  image_large VARCHAR(255),
  image_large_alt VARCHAR(255)
  ,PRIMARY KEY (manufacturer_id));

CREATE INDEX va_manufacturers_friendly_url ON va_manufacturers (friendly_url);


DROP SEQUENCE seq_va_menus CASCADE;
DROP TABLE va_menus;

CREATE SEQUENCE seq_va_menus START 5;
CREATE TABLE va_menus (
  menu_id INT4 NOT NULL DEFAULT nextval('seq_va_menus'),
  menu_type SMALLINT default '0',
  menu_title VARCHAR(255),
  menu_name VARCHAR(128),
  block_class VARCHAR(128),
  menu_class VARCHAR(128),
  menu_notes TEXT,
  show_title INT4 default '0',
  sites_all SMALLINT default '1'
  ,PRIMARY KEY (menu_id));



DROP SEQUENCE seq_va_menus_items CASCADE;
DROP TABLE va_menus_items;

CREATE SEQUENCE seq_va_menus_items START 218;
CREATE TABLE va_menus_items (
  menu_item_id INT4 NOT NULL DEFAULT nextval('seq_va_menus_items'),
  menu_id INT4 default '0',
  parent_menu_item_id INT4 default '0',
  menu_code VARCHAR(64),
  menu_title VARCHAR(255),
  menu_url VARCHAR(255),
  menu_target VARCHAR(32),
  menu_image VARCHAR(255),
  menu_image_active VARCHAR(255),
  menu_order INT4 default '0',
  menu_class VARCHAR(64),
  head_content TEXT,
  foot_content TEXT,
  guest_access SMALLINT default '1',
  user_access VARCHAR(255),
  admin_access VARCHAR(255)
  ,PRIMARY KEY (menu_item_id));

CREATE INDEX va_menus_items_menu_id ON va_menus_items (menu_id);
CREATE INDEX va_menus_items_parent_menu__45 ON va_menus_items (parent_menu_item_id);


DROP SEQUENCE seq_va_menus_sites CASCADE;
DROP TABLE va_menus_sites;

CREATE SEQUENCE seq_va_menus_sites START 1;
CREATE TABLE va_menus_sites (
  menu_id INT4 NOT NULL default '0',
  site_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (menu_id,site_id));



DROP SEQUENCE seq_va_messages CASCADE;
DROP TABLE va_messages;

CREATE SEQUENCE seq_va_messages START 1;
CREATE TABLE va_messages (
  message_id INT4 NOT NULL DEFAULT nextval('seq_va_messages'),
  parent_message_id INT4 default '0',
  reply_message_id INT4 default '0',
  forward_message_id INT4 default '0',
  admin_id INT4 default '0',
  user_id INT4 default '0',
  system_folder_id SMALLINT default '1',
  user_folder_id INT4 default '0',
  message_from VARCHAR(64),
  from_admin_id INT4 default '0',
  from_user_id INT4 default '0',
  to_admin_id INT4 default '0',
  to_user_id INT4 default '0',
  message_type SMALLINT default '0',
  message_key_id INT4 default '0',
  message_to VARCHAR(255),
  message_cc VARCHAR(255),
  message_bcc VARCHAR(255),
  message_subject VARCHAR(255),
  message_text TEXT,
  date_added TIMESTAMP,
  date_read TIMESTAMP,
  date_replied TIMESTAMP,
  date_forwarded TIMESTAMP,
  date_sent TIMESTAMP,
  date_modified TIMESTAMP,
  date_trashed TIMESTAMP,
  date_deleted TIMESTAMP
  ,PRIMARY KEY (message_id));

CREATE INDEX va_messages_admin_id ON va_messages (admin_id);
CREATE INDEX va_messages_date_added ON va_messages (date_added);
CREATE INDEX va_messages_date_deleted ON va_messages (date_deleted);
CREATE INDEX va_messages_date_trashed ON va_messages (date_trashed);
CREATE INDEX va_messages_from_admin_id ON va_messages (from_admin_id);
CREATE INDEX va_messages_from_user_id ON va_messages (from_user_id);
CREATE INDEX va_messages_message_key_id ON va_messages (message_key_id);
CREATE INDEX va_messages_message_type ON va_messages (message_type);
CREATE INDEX va_messages_parent_message_id ON va_messages (parent_message_id);
CREATE INDEX va_messages_system_folder_id ON va_messages (system_folder_id);
CREATE INDEX va_messages_user_folder_id ON va_messages (user_folder_id);
CREATE INDEX va_messages_user_id ON va_messages (user_id);


DROP SEQUENCE seq_va_newsletter_filters CASCADE;
DROP TABLE va_newsletter_filters;

CREATE SEQUENCE seq_va_newsletter_filters START 1;
CREATE TABLE va_newsletter_filters (
  filter_id INT4 NOT NULL DEFAULT nextval('seq_va_newsletter_filters'),
  newsletter_id INT4 default '0',
  filter_parameter VARCHAR(64),
  filter_value VARCHAR(255)
  ,PRIMARY KEY (filter_id));

CREATE INDEX va_newsletter_filters_newsl_46 ON va_newsletter_filters (newsletter_id);


DROP SEQUENCE seq_va_newsletters CASCADE;
DROP TABLE va_newsletters;

CREATE SEQUENCE seq_va_newsletters START 1;
CREATE TABLE va_newsletters (
  newsletter_id INT4 NOT NULL DEFAULT nextval('seq_va_newsletters'),
  campaign_id INT4 default '0',
  template_newsletter_id INT4,
  newsletter_type SMALLINT default '1',
  newsletter_date TIMESTAMP NOT NULL,
  newsletter_subject VARCHAR(255) NOT NULL,
  newsletter_body TEXT,
  mail_type SMALLINT default '0',
  mail_from VARCHAR(128),
  mail_cc VARCHAR(255),
  mail_bcc VARCHAR(50),
  mail_reply_to VARCHAR(128),
  mail_return_path VARCHAR(128),
  mailing_start TIMESTAMP,
  mailing_end TIMESTAMP,
  emails_total INT4 default '0',
  emails_left INT4 default '0',
  emails_sent INT4 default '0',
  emails_opened INT4 default '0',
  emails_clicked INT4 default '0',
  emails_bounced INT4 default '0',
  emails_unsubscribed INT4 default '0',
  is_active SMALLINT default '0',
  is_sent SMALLINT default '0',
  is_prepared SMALLINT default '0',
  users_recipients TEXT,
  admins_recipients TEXT,
  subscribed_recipients TEXT,
  orders_recipients TEXT,
  custom_recipients TEXT,
  added_by INT4 default '0',
  added_date TIMESTAMP,
  edited_by INT4 default '0',
  edited_date TIMESTAMP,
  template_active SMALLINT,
  template_period SMALLINT,
  template_interval INT4,
  template_newsletters_limit INT4,
  template_newsletters_added VARCHAR(50),
  template_start_date TIMESTAMP,
  template_end_date TIMESTAMP,
  template_next_date TIMESTAMP,
  template_last_date TIMESTAMP,
  template_filter_period INT4,
  template_filter_interval INT4
  ,PRIMARY KEY (newsletter_id));

CREATE INDEX va_newsletters_campaign_id ON va_newsletters (campaign_id);
CREATE INDEX va_newsletters_newsletter_type ON va_newsletters (newsletter_type);
CREATE INDEX va_newsletters_template_active ON va_newsletters (template_active);
CREATE INDEX va_newsletters_template_new_47 ON va_newsletters (template_newsletter_id);


DROP SEQUENCE seq_va_newsletters_campaigns CASCADE;
DROP TABLE va_newsletters_campaigns;

CREATE SEQUENCE seq_va_newsletters_campaigns START 1;
CREATE TABLE va_newsletters_campaigns (
  campaign_id INT4 NOT NULL DEFAULT nextval('seq_va_newsletters_campaigns'),
  site_id INT4 default '0',
  is_active SMALLINT default '1',
  campaign_name VARCHAR(255),
  campaign_date_start TIMESTAMP,
  campaign_date_end TIMESTAMP,
  emails_sent INT4 default '0',
  emails_opened INT4 default '0',
  emails_clicked INT4 default '0',
  emails_bounced INT4 default '0',
  emails_unsubscribed INT4 default '0',
  admin_id_added_by INT4 default '0',
  admin_id_modified_by INT4 default '0',
  date_added TIMESTAMP,
  date_modified TIMESTAMP
  ,PRIMARY KEY (campaign_id));



DROP SEQUENCE seq_va_newsletters_emails CASCADE;
DROP TABLE va_newsletters_emails;

CREATE SEQUENCE seq_va_newsletters_emails START 1;
CREATE TABLE va_newsletters_emails (
  email_id INT4 NOT NULL DEFAULT nextval('seq_va_newsletters_emails'),
  newsletter_id INT4 default '0',
  email_type SMALLINT default '0',
  user_id INT4 default '0',
  order_id INT4 default '0',
  admin_id INT4 default '0',
  user_email VARCHAR(128) NOT NULL,
  user_name VARCHAR(128),
  is_custom SMALLINT default '0',
  is_sent SMALLINT default '0',
  is_opened SMALLINT default '0',
  is_clicked SMALLINT default '0',
  is_bounced SMALLINT default '0',
  is_unsubscribed SMALLINT default '0',
  is_ordered SMALLINT default '0'
  ,PRIMARY KEY (email_id));

CREATE INDEX va_newsletters_emails_admin_id ON va_newsletters_emails (admin_id);
CREATE INDEX va_newsletters_emails_email_type ON va_newsletters_emails (email_type);
CREATE INDEX va_newsletters_emails_newsl_48 ON va_newsletters_emails (newsletter_id);
CREATE INDEX va_newsletters_emails_order_id ON va_newsletters_emails (order_id);
CREATE INDEX va_newsletters_emails_user_email ON va_newsletters_emails (user_email);
CREATE INDEX va_newsletters_emails_user_id ON va_newsletters_emails (user_id);


DROP SEQUENCE seq_va_newsletters_groups CASCADE;
DROP TABLE va_newsletters_groups;

CREATE SEQUENCE seq_va_newsletters_groups START 1;
CREATE TABLE va_newsletters_groups (
  group_id INT4 NOT NULL DEFAULT nextval('seq_va_newsletters_groups'),
  group_name VARCHAR(255),
  group_desc TEXT,
  show_for_user SMALLINT default '1',
  is_default SMALLINT default '1',
  is_hidden SMALLINT default '0',
  sites_all SMALLINT default '1'
  ,PRIMARY KEY (group_id));



DROP SEQUENCE seq_va_newsletters_groups_sites CASCADE;
DROP TABLE va_newsletters_groups_sites;

CREATE SEQUENCE seq_va_newsletters_groups_sites START 1;
CREATE TABLE va_newsletters_groups_sites (
  group_id INT4 NOT NULL default '0',
  site_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (group_id,site_id));



DROP SEQUENCE seq_va_newsletters_users CASCADE;
DROP TABLE va_newsletters_users;

CREATE SEQUENCE seq_va_newsletters_users START 1;
CREATE TABLE va_newsletters_users (
  email_id INT4 NOT NULL DEFAULT nextval('seq_va_newsletters_users'),
  user_id INT4 default '0',
  user_name VARCHAR(128),
  email VARCHAR(128),
  date_added TIMESTAMP,
  date_updated TIMESTAMP
  ,PRIMARY KEY (email_id));

CREATE INDEX va_newsletters_users_email ON va_newsletters_users (email);
CREATE INDEX va_newsletters_users_user_id ON va_newsletters_users (user_id);


DROP SEQUENCE seq_va_newsletters_users_groups CASCADE;
DROP TABLE va_newsletters_users_groups;

CREATE SEQUENCE seq_va_newsletters_users_groups START 1;
CREATE TABLE va_newsletters_users_groups (
  email_id INT4 NOT NULL default '0',
  group_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (email_id,group_id));



DROP SEQUENCE seq_va_order_custom_properties CASCADE;
DROP TABLE va_order_custom_properties;

CREATE SEQUENCE seq_va_order_custom_properties START 29;
CREATE TABLE va_order_custom_properties (
  property_id INT4 NOT NULL DEFAULT nextval('seq_va_order_custom_properties'),
  site_id INT4 default '1',
  payment_id INT4 default '0',
  property_order INT4 default '0',
  property_code VARCHAR(64),
  property_name VARCHAR(255),
  property_description TEXT,
  default_value TEXT,
  property_class VARCHAR(64),
  property_style VARCHAR(255),
  property_type INT4 default '0',
  property_show INT4 default '0',
  tax_free SMALLINT default '0',
  control_type VARCHAR(16),
  control_style VARCHAR(255),
  control_code TEXT,
  onchange_code TEXT,
  onclick_code TEXT,
  required SMALLINT default '0',
  before_name_html TEXT,
  after_name_html TEXT,
  before_control_html TEXT,
  after_control_html TEXT,
  validation_regexp TEXT,
  regexp_error TEXT,
  options_values_sql TEXT
  ,PRIMARY KEY (property_id));

CREATE INDEX va_order_custom_properties__49 ON va_order_custom_properties (payment_id);
CREATE INDEX va_order_custom_properties__50 ON va_order_custom_properties (site_id);


DROP SEQUENCE seq_va_order_custom_values CASCADE;
DROP TABLE va_order_custom_values;

CREATE SEQUENCE seq_va_order_custom_values START 145;
CREATE TABLE va_order_custom_values (
  property_value_id INT4 NOT NULL DEFAULT nextval('seq_va_order_custom_values'),
  property_id INT4 default '0',
  property_value VARCHAR(255),
  property_price FLOAT4 default '0',
  property_weight FLOAT4 default '0',
  hide_value SMALLINT default '0',
  is_default_value SMALLINT default '0'
  ,PRIMARY KEY (property_value_id));

CREATE INDEX va_order_custom_values_prop_51 ON va_order_custom_values (property_id);


DROP SEQUENCE seq_va_order_statuses CASCADE;
DROP TABLE va_order_statuses;

CREATE SEQUENCE seq_va_order_statuses START 20;
CREATE TABLE va_order_statuses (
  status_id INT4 NOT NULL DEFAULT nextval('seq_va_order_statuses'),
  status_order INT4 default '1',
  status_name VARCHAR(64) NOT NULL,
  status_type VARCHAR(16),
  admin_order_class VARCHAR(64),
  user_order_class VARCHAR(64),
  is_user_cancel SMALLINT default '0',
  allow_user_cancel SMALLINT default '0',
  generate_invoice SMALLINT default '0',
  user_invoice_activation SMALLINT default '0',
  is_dispatch SMALLINT default '0',
  is_list SMALLINT default '1',
  show_for_user SMALLINT default '0',
  paid_status SMALLINT default '0',
  credit_note_action SMALLINT default '1',
  stock_level_action INT4 default '0',
  points_action INT4 default '0',
  credit_action INT4 default '0',
  commission_action INT4 default '0',
  download_activation SMALLINT default '0',
  download_notify SMALLINT default '0',
  item_notify SMALLINT default '0',
  mail_notify SMALLINT default '0',
  mail_to VARCHAR(255),
  mail_from VARCHAR(64),
  mail_cc VARCHAR(255),
  mail_bcc VARCHAR(255),
  mail_reply_to VARCHAR(64),
  mail_return_path VARCHAR(64),
  mail_type INT4 default '0',
  mail_pdf_invoice SMALLINT default '0',
  mail_subject VARCHAR(255),
  mail_body TEXT,
  sms_notify SMALLINT default '0',
  sms_recipient VARCHAR(255),
  sms_originator VARCHAR(64),
  sms_message TEXT,
  merchant_notify SMALLINT default '0',
  merchant_to VARCHAR(255),
  merchant_from VARCHAR(64),
  merchant_cc VARCHAR(255),
  merchant_bcc VARCHAR(255),
  merchant_reply_to VARCHAR(64),
  merchant_return_path VARCHAR(64),
  merchant_mail_type SMALLINT default '0',
  merchant_subject VARCHAR(255),
  merchant_body TEXT,
  merchant_sms_notify SMALLINT default '0',
  merchant_sms_recipient VARCHAR(255),
  merchant_sms_originator VARCHAR(64),
  merchant_sms_message TEXT,
  supplier_notify SMALLINT default '0',
  supplier_to VARCHAR(255),
  supplier_from VARCHAR(64),
  supplier_cc VARCHAR(255),
  supplier_bcc VARCHAR(255),
  supplier_reply_to VARCHAR(64),
  supplier_return_path VARCHAR(64),
  supplier_mail_type SMALLINT default '0',
  supplier_subject VARCHAR(255),
  supplier_body TEXT,
  supplier_sms_notify SMALLINT default '0',
  supplier_sms_recipient VARCHAR(255),
  supplier_sms_originator VARCHAR(64),
  supplier_sms_message TEXT,
  affiliate_notify SMALLINT default '0',
  affiliate_to VARCHAR(255),
  affiliate_from VARCHAR(64),
  affiliate_cc VARCHAR(255),
  affiliate_bcc VARCHAR(255),
  affiliate_reply_to VARCHAR(64),
  affiliate_return_path VARCHAR(64),
  affiliate_mail_type SMALLINT default '0',
  affiliate_subject VARCHAR(255),
  affiliate_body TEXT,
  affiliate_sms_notify SMALLINT default '0',
  affiliate_sms_recipient VARCHAR(255),
  affiliate_sms_originator VARCHAR(64),
  affiliate_sms_message TEXT,
  admin_notify SMALLINT default '0',
  admin_to VARCHAR(255),
  admin_to_groups_ids TEXT,
  admin_from VARCHAR(64),
  admin_cc VARCHAR(255),
  admin_bcc VARCHAR(255),
  admin_reply_to VARCHAR(64),
  admin_return_path VARCHAR(64),
  admin_mail_type SMALLINT default '0',
  admin_pdf_invoice SMALLINT default '0',
  admin_subject VARCHAR(255),
  admin_body TEXT,
  admin_sms_notify SMALLINT default '0',
  admin_sms_recipient VARCHAR(255),
  admin_sms_originator VARCHAR(64),
  admin_sms_message TEXT,
  final_title VARCHAR(255),
  final_message TEXT,
  is_active SMALLINT default '1',
  view_order_groups_all SMALLINT default '1',
  view_order_groups_ids TEXT,
  set_status_groups_all SMALLINT default '1',
  set_status_groups_ids TEXT,
  update_order_groups_all SMALLINT default '1',
  update_order_groups_ids TEXT
  ,PRIMARY KEY (status_id));



DROP SEQUENCE seq_va_orders CASCADE;
DROP TABLE va_orders;

CREATE SEQUENCE seq_va_orders START 1;
CREATE TABLE va_orders (
  order_id INT4 NOT NULL DEFAULT nextval('seq_va_orders'),
  parent_order_id INT4 default '0',
  invoice_number VARCHAR(128),
  invoice_copy_number INT4 default '0',
  site_id INT4 default '1',
  session_id VARCHAR(32),
  transaction_id VARCHAR(128),
  authorization_code VARCHAR(128),
  user_id INT4 NOT NULL default '0',
  user_type_id INT4 default '0',
  newsletter_id INT4 default '0',
  newsletter_email_id INT4 default '0',
  payment_id INT4 default '0',
  order_payment_id INT4 default '0',
  payment_amount FLOAT4,
  success_message VARCHAR(255),
  pending_message VARCHAR(50),
  error_message VARCHAR(255),
  visit_id INT4 default '0',
  remote_address VARCHAR(32),
  initial_ip VARCHAR(32),
  cookie_ip VARCHAR(32),
  affiliate_code VARCHAR(64) NOT NULL,
  affiliate_user_id INT4 default '0',
  friend_code VARCHAR(64),
  friend_user_id INT4 default '0',
  keywords VARCHAR(255),
  coupons_ids TEXT,
  vouchers_ids TEXT,
  default_currency_code VARCHAR(4),
  currency_code VARCHAR(4),
  currency_rate FLOAT4 default '1',
  payment_currency_code VARCHAR(4),
  payment_currency_rate FLOAT4 default '1',
  avs_response_code VARCHAR(64),
  avs_message VARCHAR(255),
  avs_address_match VARCHAR(16),
  avs_zip_match VARCHAR(16),
  cvv2_match VARCHAR(16),
  secure_3d_check VARCHAR(16),
  secure_3d_status VARCHAR(128),
  secure_3d_md VARCHAR(255),
  secure_3d_eci VARCHAR(16),
  secure_3d_cavv VARCHAR(128),
  secure_3d_xid VARCHAR(128),
  order_status INT4 NOT NULL default '1',
  total_buying FLOAT4 NOT NULL default '0',
  total_buying_tax FLOAT4 default '0',
  total_points_amount FLOAT4 default '0',
  total_reward_points FLOAT4 default '0',
  total_reward_credits FLOAT4 default '0',
  total_merchants_commission FLOAT4 default '0',
  total_affiliate_commission FLOAT4 default '0',
  goods_total FLOAT4 NOT NULL default '0',
  goods_tax FLOAT4 default '0',
  goods_incl_tax FLOAT4 NOT NULL default '0',
  goods_points_amount FLOAT4 default '0',
  shipping_excl_tax FLOAT4 default '0',
  shipping_tax FLOAT4 default '0',
  shipping_incl_tax FLOAT4 default '0',
  shipping_points_cost FLOAT4 default '0',
  total_quantity INT4 NOT NULL default '0',
  weight_total FLOAT4 default '0',
  total_discount FLOAT4 default '0',
  total_discount_tax FLOAT4 default '0',
  shipping_type_id INT4 default '0',
  shipping_type_code VARCHAR(64),
  shipping_type_desc VARCHAR(255),
  shipping_cost FLOAT4,
  shipping_points_amount FLOAT4 default '0',
  shipping_taxable INT4 default '0',
  shipping_tracking_id VARCHAR(64),
  shipping_expecting_date TIMESTAMP,
  properties_total FLOAT4 default '0',
  properties_points_amount FLOAT4 default '0',
  properties_taxable FLOAT4 default '0',
  processing_fee FLOAT4 default '0',
  processing_tax_free SMALLINT default '0',
  processing_excl_tax FLOAT4 default '0',
  processing_tax FLOAT4 default '0',
  processing_incl_tax FLOAT4 default '0',
  tax_name VARCHAR(255),
  tax_percent FLOAT4,
  tax_total FLOAT4 default '0',
  tax_prices_type SMALLINT default '0',
  tax_round_type SMALLINT default '1',
  vouchers_amount FLOAT4 default '0',
  credit_amount FLOAT4 default '0',
  order_total FLOAT4 NOT NULL default '0',
  paid_total FLOAT4 default '0',
  name VARCHAR(128) NOT NULL,
  first_name VARCHAR(64) NOT NULL,
  last_name VARCHAR(64) NOT NULL,
  company_id INT4 default '0',
  company_name VARCHAR(128),
  email VARCHAR(128) NOT NULL,
  address1 VARCHAR(255),
  address2 VARCHAR(255),
  city VARCHAR(128),
  province VARCHAR(128),
  state_id INT4 default '0',
  state_code VARCHAR(8),
  zip VARCHAR(16),
  country_id INT4 default '0',
  country_code VARCHAR(8),
  phone VARCHAR(32),
  daytime_phone VARCHAR(32),
  evening_phone VARCHAR(32),
  cell_phone VARCHAR(32),
  fax VARCHAR(32),
  delivery_name VARCHAR(128),
  delivery_first_name VARCHAR(64),
  delivery_last_name VARCHAR(64),
  delivery_company_id INT4 default '0',
  delivery_company_name VARCHAR(128),
  delivery_email VARCHAR(128),
  delivery_address1 VARCHAR(255),
  delivery_address2 VARCHAR(255),
  delivery_city VARCHAR(128),
  delivery_province VARCHAR(128),
  delivery_state_id INT4 default '0',
  delivery_state_code VARCHAR(8),
  delivery_zip VARCHAR(16),
  delivery_country_id INT4 default '0',
  delivery_country_code VARCHAR(8),
  delivery_phone VARCHAR(32),
  delivery_daytime_phone VARCHAR(32),
  delivery_evening_phone VARCHAR(32),
  delivery_cell_phone VARCHAR(32),
  delivery_fax VARCHAR(32),
  bill_encrypt_type SMALLINT default '1',
  bill_name VARCHAR(128),
  bill_first_name VARCHAR(64),
  bill_last_name VARCHAR(64),
  bill_company_id VARCHAR(32),
  bill_company_name VARCHAR(128),
  bill_email VARCHAR(128),
  bill_address1 VARCHAR(255),
  bill_address2 VARCHAR(255),
  bill_city VARCHAR(128),
  bill_province VARCHAR(128),
  bill_state_id VARCHAR(32),
  bill_zip VARCHAR(16),
  bill_country_id VARCHAR(32),
  bill_phone VARCHAR(32),
  bill_daytime_phone VARCHAR(32),
  bill_evening_phone VARCHAR(32),
  bill_cell_phone VARCHAR(32),
  bill_fax VARCHAR(32),
  bill_cc_number VARCHAR(64),
  bill_cc_start_date VARCHAR(32),
  bill_cc_expiry_date VARCHAR(32),
  bill_cc_type VARCHAR(32),
  bill_cc_issue_number VARCHAR(16),
  bill_cc_cvv2 VARCHAR(16),
  cc_encrypt_type SMALLINT default '1',
  cc_name VARCHAR(90),
  cc_first_name VARCHAR(64),
  cc_last_name VARCHAR(64),
  cc_number VARCHAR(32),
  cc_start_date TIMESTAMP,
  cc_expiry_date TIMESTAMP,
  cc_type INT4,
  cc_issue_number INT4,
  cc_security_code VARCHAR(32),
  pay_without_cc TEXT,
  order_placed_date TIMESTAMP NOT NULL,
  order_paid_date TIMESTAMP,
  modified_date TIMESTAMP,
  is_fast_checkout SMALLINT default '0',
  is_confirmed SMALLINT default '0',
  is_placed SMALLINT default '0',
  is_paid SMALLINT default '0',
  is_exported SMALLINT default '0',
  is_call_center SMALLINT default '0',
  is_recurring SMALLINT default '0',
  is_unsubscribed SMALLINT default '0',
  is_reminder_send SMALLINT default '0',
  reminder_send_date TIMESTAMP,
  admin_id_added_by INT4 default '0',
  admin_id_modified_by INT4 default '0',
  date_modified TIMESTAMP
  ,PRIMARY KEY (order_id));

CREATE INDEX va_orders_affiliate_code ON va_orders (affiliate_code);
CREATE INDEX va_orders_affiliate_user_id ON va_orders (affiliate_user_id);
CREATE INDEX va_orders_country_id ON va_orders (country_id);
CREATE INDEX va_orders_currency_code ON va_orders (currency_code);
CREATE INDEX va_orders_delivery_country_id ON va_orders (delivery_country_id);
CREATE INDEX va_orders_delivery_state_id ON va_orders (delivery_state_id);
CREATE INDEX va_orders_email ON va_orders (email);
CREATE INDEX va_orders_first_name ON va_orders (first_name);
CREATE INDEX va_orders_friend_id ON va_orders (friend_code);
CREATE INDEX va_orders_friend_user_id ON va_orders (friend_user_id);
CREATE INDEX va_orders_goods_total ON va_orders (goods_total);
CREATE INDEX va_orders_invoice_number ON va_orders (invoice_number);
CREATE INDEX va_orders_is_paid ON va_orders (is_paid);
CREATE INDEX va_orders_keywords ON va_orders (keywords);
CREATE INDEX va_orders_last_name ON va_orders (last_name);
CREATE INDEX va_orders_name ON va_orders (name);
CREATE INDEX va_orders_newsletter_id ON va_orders (newsletter_id);
CREATE INDEX va_orders_order_paid_date ON va_orders (order_paid_date);
CREATE INDEX va_orders_order_placed_date ON va_orders (order_placed_date);
CREATE INDEX va_orders_order_status ON va_orders (order_status);
CREATE INDEX va_orders_parent_order_id ON va_orders (parent_order_id);
CREATE INDEX va_orders_payment_id ON va_orders (payment_id);
CREATE INDEX va_orders_session_id ON va_orders (session_id);
CREATE INDEX va_orders_shipping_tracking_id ON va_orders (shipping_tracking_id);
CREATE INDEX va_orders_shipping_type_id ON va_orders (shipping_type_id);
CREATE INDEX va_orders_site_id ON va_orders (site_id);
CREATE INDEX va_orders_state_id ON va_orders (state_id);
CREATE INDEX va_orders_total_buying ON va_orders (total_buying);
CREATE INDEX va_orders_user_id ON va_orders (user_id);
CREATE INDEX va_orders_user_type_id ON va_orders (user_type_id);
CREATE INDEX va_orders_visit_id ON va_orders (visit_id);


DROP SEQUENCE seq_va_orders_coupons CASCADE;
DROP TABLE va_orders_coupons;

CREATE SEQUENCE seq_va_orders_coupons START 1;
CREATE TABLE va_orders_coupons (
  order_coupon_id INT4 NOT NULL DEFAULT nextval('seq_va_orders_coupons'),
  order_id INT4 default '0',
  order_item_id INT4 default '0',
  coupon_id INT4 default '0',
  coupon_code VARCHAR(64),
  coupon_title VARCHAR(255),
  discount_type SMALLINT default '0',
  discount_amount FLOAT4 default '0',
  discount_tax_amount FLOAT4 default '0'
  ,PRIMARY KEY (order_coupon_id));

CREATE INDEX va_orders_coupons_coupon_id ON va_orders_coupons (coupon_id);
CREATE INDEX va_orders_coupons_order_id ON va_orders_coupons (order_id);
CREATE INDEX va_orders_coupons_order_item_id ON va_orders_coupons (order_item_id);


DROP SEQUENCE seq_va_orders_events CASCADE;
DROP TABLE va_orders_events;

CREATE SEQUENCE seq_va_orders_events START 1;
CREATE TABLE va_orders_events (
  event_id INT4 NOT NULL DEFAULT nextval('seq_va_orders_events'),
  order_id INT4 default '0',
  admin_id INT4 default '0',
  status_id INT4 default '0',
  order_items TEXT,
  event_date TIMESTAMP,
  event_type VARCHAR(32),
  event_name VARCHAR(255),
  event_description TEXT
  ,PRIMARY KEY (event_id));

CREATE INDEX va_orders_events_admin_id ON va_orders_events (admin_id);
CREATE INDEX va_orders_events_order_id ON va_orders_events (order_id);
CREATE INDEX va_orders_events_status_id ON va_orders_events (status_id);


DROP SEQUENCE seq_va_orders_items CASCADE;
DROP TABLE va_orders_items;

CREATE SEQUENCE seq_va_orders_items START 1;
CREATE TABLE va_orders_items (
  order_item_id INT4 NOT NULL DEFAULT nextval('seq_va_orders_items'),
  parent_order_item_id INT4 default '0',
  top_order_item_id INT4 default '0',
  order_id INT4 NOT NULL default '0',
  site_id INT4 default '1',
  user_id INT4 default '0',
  user_type_id INT4 default '0',
  item_id INT4 NOT NULL default '0',
  subscription_id INT4 default '0',
  cart_item_id INT4 default '0',
  item_user_id INT4 default '0',
  affiliate_user_id INT4 default '0',
  friend_user_id INT4 default '0',
  parent_item_id INT4 default '0',
  item_type_id INT4 NOT NULL default '0',
  item_code VARCHAR(64),
  manufacturer_code VARCHAR(255),
  supplier_id INT4 default '0',
  coupons_ids TEXT,
  item_status INT4 NOT NULL default '1',
  component_order INT4 default '1',
  component_name VARCHAR(255),
  item_name VARCHAR(255) NOT NULL,
  item_properties TEXT,
  buying_price FLOAT4 default '0',
  real_price FLOAT4 default '0',
  discount_amount FLOAT4 default '0',
  points_price FLOAT4 default '0',
  reward_points FLOAT4 default '0',
  reward_credits FLOAT4 default '0',
  affiliate_commission FLOAT4 default '0',
  merchant_commission FLOAT4 default '0',
  price FLOAT4 NOT NULL default '0',
  tax_id INT4 default '0',
  tax_free INT4 default '0',
  tax_percent FLOAT4,
  width FLOAT4 default '0',
  height FLOAT4 default '0',
  length FLOAT4 default '0',
  weight FLOAT4 default '0',
  packages_number FLOAT4,
  quantity INT4,
  downloadable INT4 default '0',
  is_shipping_free SMALLINT default '0',
  shipping_cost FLOAT4 default '0',
  order_shipping_id INT4 default '0',
  is_recurring SMALLINT default '0',
  recurring_price FLOAT4,
  recurring_period INT4,
  recurring_interval INT4,
  recurring_payments_total INT4,
  recurring_payments_made INT4,
  recurring_payments_failed INT4,
  recurring_end_date TIMESTAMP,
  recurring_last_payment TIMESTAMP,
  recurring_next_payment TIMESTAMP,
  recurring_plan_payment TIMESTAMP,
  is_subscription SMALLINT default '0',
  is_account_subscription SMALLINT default '0',
  subscription_period INT4,
  subscription_interval INT4,
  subscription_suspend INT4,
  subscription_start_date TIMESTAMP,
  subscription_expiry_date TIMESTAMP
  ,PRIMARY KEY (order_item_id));

CREATE INDEX va_orders_items_affiliate_u_52 ON va_orders_items (affiliate_user_id);
CREATE INDEX va_orders_items_friend_user_id ON va_orders_items (friend_user_id);
CREATE INDEX va_orders_items_is_subscription ON va_orders_items (is_subscription);
CREATE INDEX va_orders_items_item_code ON va_orders_items (item_code);
CREATE INDEX va_orders_items_item_id ON va_orders_items (item_id);
CREATE INDEX va_orders_items_item_status ON va_orders_items (item_status);
CREATE INDEX va_orders_items_item_type ON va_orders_items (item_type_id);
CREATE INDEX va_orders_items_item_user_id ON va_orders_items (item_user_id);
CREATE INDEX va_orders_items_order_id ON va_orders_items (order_id);
CREATE INDEX va_orders_items_order_shipp_53 ON va_orders_items (order_shipping_id);
CREATE INDEX va_orders_items_parent_item_id ON va_orders_items (parent_item_id);
CREATE INDEX va_orders_items_parent_orde_54 ON va_orders_items (parent_order_item_id);
CREATE INDEX va_orders_items_price ON va_orders_items (price);
CREATE INDEX va_orders_items_site_id ON va_orders_items (site_id);
CREATE INDEX va_orders_items_subscriptio_55 ON va_orders_items (subscription_expiry_date);
CREATE INDEX va_orders_items_subscription_id ON va_orders_items (user_type_id);
CREATE INDEX va_orders_items_subscription_id1 ON va_orders_items (subscription_id);
CREATE INDEX va_orders_items_supplier_id ON va_orders_items (supplier_id);
CREATE INDEX va_orders_items_top_order_i_56 ON va_orders_items (top_order_item_id);
CREATE INDEX va_orders_items_user_id ON va_orders_items (user_id);


DROP SEQUENCE seq_va_orders_items_properties CASCADE;
DROP TABLE va_orders_items_properties;

CREATE SEQUENCE seq_va_orders_items_properties START 1;
CREATE TABLE va_orders_items_properties (
  item_property_id INT4 NOT NULL DEFAULT nextval('seq_va_orders_items_properties'),
  order_id INT4 NOT NULL default '0',
  order_item_id INT4 NOT NULL default '0',
  property_id INT4 NOT NULL default '0',
  property_values_ids TEXT,
  property_order INT4 default '1',
  property_name VARCHAR(255) NOT NULL,
  hide_name SMALLINT default '0',
  length_units VARCHAR(8),
  property_value TEXT,
  additional_price FLOAT4 default '0',
  additional_weight FLOAT4 default '0'
  ,PRIMARY KEY (item_property_id));

CREATE INDEX va_orders_items_properties__57 ON va_orders_items_properties (order_id);
CREATE INDEX va_orders_items_properties__58 ON va_orders_items_properties (order_item_id);
CREATE INDEX va_orders_items_properties__59 ON va_orders_items_properties (property_id);


DROP SEQUENCE seq_va_orders_items_serials CASCADE;
DROP TABLE va_orders_items_serials;

CREATE SEQUENCE seq_va_orders_items_serials START 1;
CREATE TABLE va_orders_items_serials (
  serial_id INT4 NOT NULL DEFAULT nextval('seq_va_orders_items_serials'),
  order_id INT4 default '0',
  user_id INT4 default '0',
  order_item_id INT4 default '0',
  item_id INT4 default '0',
  serial_number VARCHAR(128),
  activated SMALLINT default '0',
  activations_number INT4 default '0',
  serial_added TIMESTAMP,
  serial_expiry TIMESTAMP
  ,PRIMARY KEY (serial_id));

CREATE INDEX va_orders_items_serials_item_id ON va_orders_items_serials (item_id);
CREATE INDEX va_orders_items_serials_order_id ON va_orders_items_serials (order_id);
CREATE INDEX va_orders_items_serials_ord_60 ON va_orders_items_serials (order_item_id);
CREATE INDEX va_orders_items_serials_ser_61 ON va_orders_items_serials (serial_number);
CREATE INDEX va_orders_items_serials_user_id ON va_orders_items_serials (user_id);


DROP SEQUENCE seq_va_orders_items_taxes CASCADE;
DROP TABLE va_orders_items_taxes;

CREATE SEQUENCE seq_va_orders_items_taxes START 1;
CREATE TABLE va_orders_items_taxes (
  tax_item_id INT4 NOT NULL DEFAULT nextval('seq_va_orders_items_taxes'),
  order_tax_id INT4 default '0',
  item_type_id INT4 default '0',
  tax_percent FLOAT4,
  fixed_amount FLOAT4
  ,PRIMARY KEY (tax_item_id));

CREATE INDEX va_orders_items_taxes_item__62 ON va_orders_items_taxes (item_type_id);
CREATE INDEX va_orders_items_taxes_order_63 ON va_orders_items_taxes (order_tax_id);


DROP SEQUENCE seq_va_orders_notes CASCADE;
DROP TABLE va_orders_notes;

CREATE SEQUENCE seq_va_orders_notes START 1;
CREATE TABLE va_orders_notes (
  note_id INT4 NOT NULL DEFAULT nextval('seq_va_orders_notes'),
  order_id INT4 default '0',
  note_title VARCHAR(255),
  note_details TEXT,
  show_for_user SMALLINT default '0',
  date_added TIMESTAMP,
  date_updated TIMESTAMP
  ,PRIMARY KEY (note_id));

CREATE INDEX va_orders_notes_order_id ON va_orders_notes (order_id);


DROP SEQUENCE seq_va_orders_payments CASCADE;
DROP TABLE va_orders_payments;

CREATE SEQUENCE seq_va_orders_payments START 1;
CREATE TABLE va_orders_payments (
  order_payment_id INT4 NOT NULL DEFAULT nextval('seq_va_orders_payments'),
  order_id INT4 default '0',
  payment_id INT4 default '0',
  payment_index INT4 default '1',
  payment_amount FLOAT4 default '0',
  transaction_id VARCHAR(128),
  success_message VARCHAR(255),
  pending_message VARCHAR(255),
  error_message VARCHAR(255),
  remote_ip VARCHAR(32),
  payment_currency_code VARCHAR(4),
  payment_currency_rate FLOAT4 default '1',
  payment_status INT4 default '0',
  payment_paid SMALLINT default '0'
  ,PRIMARY KEY (order_payment_id));

CREATE INDEX va_orders_payments_order_id ON va_orders_payments (order_id);
CREATE INDEX va_orders_payments_payment_id ON va_orders_payments (payment_id);
CREATE INDEX va_orders_payments_transact_64 ON va_orders_payments (transaction_id);


DROP SEQUENCE seq_va_orders_properties CASCADE;
DROP TABLE va_orders_properties;

CREATE SEQUENCE seq_va_orders_properties START 1;
CREATE TABLE va_orders_properties (
  order_property_id INT4 NOT NULL DEFAULT nextval('seq_va_orders_properties'),
  order_id INT4 default '0',
  property_id INT4 default '0',
  property_order INT4 default '1',
  property_type INT4 default '0',
  property_name VARCHAR(255) NOT NULL,
  property_value_id INT4,
  property_value TEXT,
  property_price FLOAT4 default '0',
  property_points_amount FLOAT4 default '0',
  property_weight FLOAT4 default '0',
  tax_free SMALLINT default '0'
  ,PRIMARY KEY (order_property_id));

CREATE INDEX va_orders_properties_order_id ON va_orders_properties (order_id);
CREATE INDEX va_orders_properties_property_id ON va_orders_properties (property_id);
CREATE INDEX va_orders_properties_proper_65 ON va_orders_properties (property_name);


DROP SEQUENCE seq_va_orders_serials_activations CASCADE;
DROP TABLE va_orders_serials_activations;

CREATE SEQUENCE seq_va_orders_serials_activations START 1;
CREATE TABLE va_orders_serials_activations (
  activation_id INT4 NOT NULL DEFAULT nextval('seq_va_orders_serials_activations'),
  serial_id INT4 default '0',
  order_id INT4 default '0',
  remote_address VARCHAR(32),
  generation_key VARCHAR(255),
  activation_key TEXT,
  date_added TIMESTAMP
  ,PRIMARY KEY (activation_id));

CREATE INDEX va_orders_serials_activatio_66 ON va_orders_serials_activations (order_id);
CREATE INDEX va_orders_serials_activatio_67 ON va_orders_serials_activations (serial_id);


DROP SEQUENCE seq_va_orders_shipments CASCADE;
DROP TABLE va_orders_shipments;

CREATE SEQUENCE seq_va_orders_shipments START 1;
CREATE TABLE va_orders_shipments (
  order_shipping_id INT4 NOT NULL DEFAULT nextval('seq_va_orders_shipments'),
  order_id INT4 NOT NULL default '0',
  shipping_id INT4 NOT NULL default '0',
  shipping_code VARCHAR(64),
  shipping_desc VARCHAR(255),
  shipping_cost FLOAT4 default '0',
  points_cost FLOAT4 default '0',
  tax_free SMALLINT default '0',
  tracking_id VARCHAR(64),
  expecting_date TIMESTAMP,
  goods_weight FLOAT4 default '0',
  tare_weight FLOAT4 default '0',
  order_items_ids TEXT
  ,PRIMARY KEY (order_shipping_id));

CREATE INDEX va_orders_shipments_order_id ON va_orders_shipments (order_id);
CREATE INDEX va_orders_shipments_shippin_68 ON va_orders_shipments (shipping_id);


DROP SEQUENCE seq_va_orders_taxes CASCADE;
DROP TABLE va_orders_taxes;

CREATE SEQUENCE seq_va_orders_taxes START 1;
CREATE TABLE va_orders_taxes (
  order_tax_id INT4 NOT NULL DEFAULT nextval('seq_va_orders_taxes'),
  order_id INT4 default '0',
  tax_id INT4 default '0',
  tax_type SMALLINT default '1',
  show_type SMALLINT default '0',
  tax_name VARCHAR(50),
  tax_percent FLOAT4 default '0',
  fixed_amount FLOAT4,
  shipping_tax_percent FLOAT4,
  shipping_fixed_amount FLOAT4
  ,PRIMARY KEY (order_tax_id));

CREATE INDEX va_orders_taxes_order_id ON va_orders_taxes (order_id);
CREATE INDEX va_orders_taxes_tax_id ON va_orders_taxes (tax_id);


DROP SEQUENCE seq_va_pages CASCADE;
DROP TABLE va_pages;

CREATE SEQUENCE seq_va_pages START 20;
CREATE TABLE va_pages (
  page_id INT4 NOT NULL DEFAULT nextval('seq_va_pages'),
  page_code VARCHAR(16) NOT NULL,
  page_title VARCHAR(255) NOT NULL,
  friendly_url VARCHAR(255),
  page_url VARCHAR(255),
  page_order INT4 NOT NULL default '1',
  is_site_map SMALLINT default '1',
  sites_all SMALLINT default '1',
  user_types_all SMALLINT default '1',
  is_showing SMALLINT NOT NULL default '1',
  link_in_footer SMALLINT NOT NULL default '0',
  is_html SMALLINT,
  page_type INT4,
  page_path VARCHAR(255),
  page_body TEXT,
  meta_title VARCHAR(255),
  meta_keywords VARCHAR(255),
  meta_description VARCHAR(255)
  ,PRIMARY KEY (page_id));

CREATE INDEX va_pages_friendly_url ON va_pages (friendly_url);
CREATE INDEX va_pages_is_showing ON va_pages (is_showing);
CREATE INDEX va_pages_is_site_map ON va_pages (is_site_map);
CREATE INDEX va_pages_link_in_footer ON va_pages (link_in_footer);
CREATE INDEX va_pages_page_code ON va_pages (page_code);
CREATE INDEX va_pages_sites_all ON va_pages (sites_all);
CREATE INDEX va_pages_user_types_all ON va_pages (user_types_all);


DROP SEQUENCE seq_va_pages_sites CASCADE;
DROP TABLE va_pages_sites;

CREATE SEQUENCE seq_va_pages_sites START 1;
CREATE TABLE va_pages_sites (
  page_id INT4 NOT NULL default '0',
  site_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (page_id,site_id));



DROP SEQUENCE seq_va_pages_user_types CASCADE;
DROP TABLE va_pages_user_types;

CREATE SEQUENCE seq_va_pages_user_types START 1;
CREATE TABLE va_pages_user_types (
  page_id INT4 NOT NULL default '0',
  user_type_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (page_id,user_type_id));



DROP SEQUENCE seq_va_payment_countries CASCADE;
DROP TABLE va_payment_countries;

CREATE SEQUENCE seq_va_payment_countries START 1;
CREATE TABLE va_payment_countries (
  payment_id INT4,
  country_id INT4
);



DROP SEQUENCE seq_va_payment_currencies CASCADE;
DROP TABLE va_payment_currencies;

CREATE SEQUENCE seq_va_payment_currencies START 1;
CREATE TABLE va_payment_currencies (
  payment_id INT4 NOT NULL default '0',
  currency_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (payment_id,currency_id));



DROP SEQUENCE seq_va_payment_item_types CASCADE;
DROP TABLE va_payment_item_types;

CREATE SEQUENCE seq_va_payment_item_types START 1;
CREATE TABLE va_payment_item_types (
  payment_id INT4 NOT NULL default '0',
  item_type_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (payment_id,item_type_id));



DROP SEQUENCE seq_va_payment_parameters CASCADE;
DROP TABLE va_payment_parameters;

CREATE SEQUENCE seq_va_payment_parameters START 2072;
CREATE TABLE va_payment_parameters (
  parameter_id INT4 NOT NULL DEFAULT nextval('seq_va_payment_parameters'),
  payment_id INT4 NOT NULL default '0',
  parameter_name VARCHAR(255) NOT NULL,
  parameter_type VARCHAR(16) NOT NULL,
  parameter_source TEXT,
  not_passed SMALLINT default '0'
  ,PRIMARY KEY (parameter_id));



DROP SEQUENCE seq_va_payment_systems CASCADE;
DROP TABLE va_payment_systems;

CREATE SEQUENCE seq_va_payment_systems START 105;
CREATE TABLE va_payment_systems (
  payment_id INT4 NOT NULL DEFAULT nextval('seq_va_payment_systems'),
  payment_order INT4 default '1',
  payment_name VARCHAR(255) NOT NULL,
  user_payment_name VARCHAR(255),
  payment_info TEXT,
  payment_notes TEXT,
  payment_type VARCHAR(32),
  payment_code VARCHAR(64),
  payment_php_lib VARCHAR(255),
  validation_php_lib VARCHAR(255),
  payment_url VARCHAR(255) NOT NULL,
  processing_time INT4 default '0',
  processing_fee FLOAT4,
  processing_tax_free SMALLINT default '0',
  fee_type INT4 default '0',
  fee_min_goods FLOAT4,
  fee_max_goods FLOAT4,
  submit_method VARCHAR(4),
  recurring_method INT4 default '0',
  is_advanced SMALLINT default '0',
  advanced_url VARCHAR(255),
  advanced_php_lib VARCHAR(255),
  capture_php_lib VARCHAR(255),
  refund_php_lib VARCHAR(255),
  void_php_lib VARCHAR(255),
  success_status_id INT4,
  pending_status_id INT4,
  failure_status_id INT4,
  failure_action INT4 default '0',
  is_active SMALLINT default '0',
  is_default SMALLINT default '0',
  is_call_center SMALLINT default '0',
  allowed_user_edit SMALLINT default '0',
  sites_all SMALLINT default '1',
  non_logged_users SMALLINT default '1',
  user_types_all SMALLINT default '1',
  item_types_all SMALLINT default '1',
  currencies_all SMALLINT default '1',
  image_small VARCHAR(255),
  image_small_alt VARCHAR(255),
  image_large VARCHAR(255),
  image_large_alt VARCHAR(255),
  fast_checkout_active SMALLINT default '0',
  fast_checkout_image VARCHAR(255),
  fast_checkout_width INT4,
  fast_checkout_height INT4,
  fast_checkout_alt VARCHAR(255),
  order_total_min FLOAT4 default '0',
  order_total_max FLOAT4 default '0',
  countries_all INT4 default '1'
  ,PRIMARY KEY (payment_id));



DROP SEQUENCE seq_va_payment_systems_sites CASCADE;
DROP TABLE va_payment_systems_sites;

CREATE SEQUENCE seq_va_payment_systems_sites START 1;
CREATE TABLE va_payment_systems_sites (
  payment_id INT4 NOT NULL default '0',
  site_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (payment_id,site_id));



DROP SEQUENCE seq_va_payment_user_types CASCADE;
DROP TABLE va_payment_user_types;

CREATE SEQUENCE seq_va_payment_user_types START 1;
CREATE TABLE va_payment_user_types (
  payment_id INT4 default '0',
  user_type_id INT4 default '0'
);

CREATE INDEX va_payment_user_types_payment_id ON va_payment_user_types (payment_id);
CREATE INDEX va_payment_user_types_user__69 ON va_payment_user_types (user_type_id);


DROP SEQUENCE seq_va_polls CASCADE;
DROP TABLE va_polls;

CREATE SEQUENCE seq_va_polls START 2;
CREATE TABLE va_polls (
  poll_id INT4 NOT NULL DEFAULT nextval('seq_va_polls'),
  poll_type INT4 NOT NULL default '1',
  question TEXT NOT NULL,
  is_active SMALLINT NOT NULL default '0',
  date_added TIMESTAMP NOT NULL,
  total_votes INT4 default '0'
  ,PRIMARY KEY (poll_id));



DROP SEQUENCE seq_va_polls_options CASCADE;
DROP TABLE va_polls_options;

CREATE SEQUENCE seq_va_polls_options START 7;
CREATE TABLE va_polls_options (
  poll_option_id INT4 NOT NULL DEFAULT nextval('seq_va_polls_options'),
  poll_id INT4 NOT NULL default '0',
  option_description TEXT NOT NULL,
  is_default_value SMALLINT default '0'
  ,PRIMARY KEY (poll_option_id));

CREATE INDEX va_polls_options_poll_id ON va_polls_options (poll_id);


DROP SEQUENCE seq_va_polls_votes CASCADE;
DROP TABLE va_polls_votes;

CREATE SEQUENCE seq_va_polls_votes START 1;
CREATE TABLE va_polls_votes (
  poll_id INT4 NOT NULL default '0',
  poll_option_id INT4 NOT NULL default '0',
  remote_address VARCHAR(32) NOT NULL,
  date_added TIMESTAMP NOT NULL
);

CREATE INDEX va_polls_votes_poll_id ON va_polls_votes (poll_id);
CREATE INDEX va_polls_votes_poll_option_id ON va_polls_votes (poll_option_id);
CREATE INDEX va_polls_votes_remote_address ON va_polls_votes (remote_address);


DROP SEQUENCE seq_va_prices CASCADE;
DROP TABLE va_prices;

CREATE SEQUENCE seq_va_prices START 1;
CREATE TABLE va_prices (
  price_id INT4 NOT NULL DEFAULT nextval('seq_va_prices'),
  price_title VARCHAR(64),
  price_amount FLOAT4 default '0',
  price_description TEXT
  ,PRIMARY KEY (price_id));



DROP SEQUENCE seq_va_profiles CASCADE;
DROP TABLE va_profiles;

CREATE SEQUENCE seq_va_profiles START 1;
CREATE TABLE va_profiles (
  profile_id INT4 NOT NULL DEFAULT nextval('seq_va_profiles'),
  user_id INT4 default '0',
  is_shown SMALLINT default '1',
  is_approved SMALLINT default '1',
  profile_type_id INT4 default '0',
  looking_type_id INT4 default '0',
  profile_name VARCHAR(128),
  birth_year SMALLINT default '0',
  birth_month SMALLINT,
  birth_day SMALLINT,
  birth_date TIMESTAMP,
  country_id INT4 default '0',
  state_id INT4 default '0',
  postal_code VARCHAR(16),
  city VARCHAR(128),
  photo_id INT4 default '0',
  ethnicity_id INT4 default '0',
  height SMALLINT default '0',
  weight INT4 default '0',
  profile_info TEXT,
  looking_info TEXT,
  date_added TIMESTAMP,
  date_updated TIMESTAMP,
  date_last_visit TIMESTAMP
  ,PRIMARY KEY (profile_id));

CREATE INDEX va_profiles_is_approved ON va_profiles (is_approved);
CREATE INDEX va_profiles_is_shown ON va_profiles (is_shown);
CREATE INDEX va_profiles_looking_type_id ON va_profiles (looking_type_id);
CREATE INDEX va_profiles_photo_id ON va_profiles (photo_id);
CREATE INDEX va_profiles_profile_type_id ON va_profiles (profile_type_id);
CREATE INDEX va_profiles_user_id ON va_profiles (user_id);


DROP SEQUENCE seq_va_profiles_types CASCADE;
DROP TABLE va_profiles_types;

CREATE SEQUENCE seq_va_profiles_types START 3;
CREATE TABLE va_profiles_types (
  profile_type_id INT4 NOT NULL DEFAULT nextval('seq_va_profiles_types'),
  profile_type_name VARCHAR(255),
  show_for_user SMALLINT default '1',
  is_profile_default SMALLINT default '0',
  is_looking_default SMALLINT default '0'
  ,PRIMARY KEY (profile_type_id));



DROP SEQUENCE seq_va_quotes CASCADE;
DROP TABLE va_quotes;

CREATE SEQUENCE seq_va_quotes START 1;
CREATE TABLE va_quotes (
  quote_id INT4 NOT NULL DEFAULT nextval('seq_va_quotes'),
  support_id INT4 default '0',
  user_id INT4 default '0',
  user_name VARCHAR(255),
  user_email VARCHAR(255),
  quote_price FLOAT4 default '0',
  quote_status_id INT4 default '0',
  is_paid SMALLINT default '0',
  minutes_spent INT4 default '0',
  date_due TIMESTAMP,
  summary VARCHAR(255),
  description TEXT,
  admin_id_added_by INT4 default '0',
  admin_id_modified_by INT4 default '0',
  date_added TIMESTAMP,
  date_modified TIMESTAMP
  ,PRIMARY KEY (quote_id));

CREATE INDEX va_quotes_is_paid ON va_quotes (is_paid);
CREATE INDEX va_quotes_quote_status_id ON va_quotes (quote_status_id);
CREATE INDEX va_quotes_support_id ON va_quotes (support_id);
CREATE INDEX va_quotes_user_id ON va_quotes (user_id);


DROP SEQUENCE seq_va_quotes_events CASCADE;
DROP TABLE va_quotes_events;

CREATE SEQUENCE seq_va_quotes_events START 1;
CREATE TABLE va_quotes_events (
  event_id INT4 NOT NULL DEFAULT nextval('seq_va_quotes_events'),
  quote_id INT4 default '0',
  feature_id INT4 default '0',
  admin_id INT4 default '0',
  status_id_old INT4 default '0',
  status_id_new INT4 default '0',
  event_date TIMESTAMP,
  event_name VARCHAR(255),
  event_description TEXT
  ,PRIMARY KEY (event_id));

CREATE INDEX va_quotes_events_admin_id ON va_quotes_events (admin_id);
CREATE INDEX va_quotes_events_feature_id ON va_quotes_events (feature_id);
CREATE INDEX va_quotes_events_quote_id ON va_quotes_events (quote_id);


DROP SEQUENCE seq_va_quotes_features CASCADE;
DROP TABLE va_quotes_features;

CREATE SEQUENCE seq_va_quotes_features START 1;
CREATE TABLE va_quotes_features (
  feature_id INT4 NOT NULL DEFAULT nextval('seq_va_quotes_features'),
  quote_id INT4 default '0',
  feature_description TEXT,
  feature_price FLOAT4 default '0',
  feature_status_id INT4 default '0',
  feature_paid INT4 default '0',
  minutes_spent INT4 default '0',
  date_due TIMESTAMP,
  admin_id_added_by INT4 default '0',
  admin_id_modified_by INT4 default '0',
  date_added TIMESTAMP,
  date_modified TIMESTAMP
  ,PRIMARY KEY (feature_id));

CREATE INDEX va_quotes_features_feature_paid ON va_quotes_features (feature_paid);
CREATE INDEX va_quotes_features_feature__70 ON va_quotes_features (feature_status_id);
CREATE INDEX va_quotes_features_quote_id ON va_quotes_features (quote_id);


DROP SEQUENCE seq_va_quotes_orders CASCADE;
DROP TABLE va_quotes_orders;

CREATE SEQUENCE seq_va_quotes_orders START 1;
CREATE TABLE va_quotes_orders (
  quote_order_id INT4 NOT NULL DEFAULT nextval('seq_va_quotes_orders'),
  quote_id INT4 default '0',
  feature_id INT4 default '0',
  order_id INT4 default '0',
  order_item_id INT4 default '0'
  ,PRIMARY KEY (quote_order_id));

CREATE INDEX va_quotes_orders_feature_id ON va_quotes_orders (feature_id);
CREATE INDEX va_quotes_orders_order_id ON va_quotes_orders (order_id);
CREATE INDEX va_quotes_orders_order_item_id ON va_quotes_orders (order_item_id);
CREATE INDEX va_quotes_orders_quote_id ON va_quotes_orders (quote_id);


DROP SEQUENCE seq_va_quotes_statuses CASCADE;
DROP TABLE va_quotes_statuses;

CREATE SEQUENCE seq_va_quotes_statuses START 5;
CREATE TABLE va_quotes_statuses (
  status_id INT4 NOT NULL DEFAULT nextval('seq_va_quotes_statuses'),
  status_name VARCHAR(255),
  status_icon VARCHAR(255),
  html_start VARCHAR(255),
  html_end VARCHAR(255),
  notes TEXT
  ,PRIMARY KEY (status_id));



DROP SEQUENCE seq_va_registration_categories CASCADE;
DROP TABLE va_registration_categories;

CREATE SEQUENCE seq_va_registration_categories START 1;
CREATE TABLE va_registration_categories (
  category_id INT4 NOT NULL DEFAULT nextval('seq_va_registration_categories'),
  parent_category_id INT4 default '0',
  category_order INT4 default '1',
  category_path VARCHAR(255),
  category_name VARCHAR(255),
  show_for_user SMALLINT NOT NULL default '1',
  admin_id_added_by INT4 default '0',
  admin_id_modified_by INT4 default '0',
  date_added TIMESTAMP,
  date_modified TIMESTAMP
  ,PRIMARY KEY (category_id));

CREATE INDEX va_registration_categories__71 ON va_registration_categories (category_order);
CREATE INDEX va_registration_categories__72 ON va_registration_categories (category_path);
CREATE INDEX va_registration_categories__73 ON va_registration_categories (parent_category_id);
CREATE INDEX va_registration_categories__74 ON va_registration_categories (show_for_user);


DROP SEQUENCE seq_va_registration_custom_properties CASCADE;
DROP TABLE va_registration_custom_properties;

CREATE SEQUENCE seq_va_registration_custom_properties START 1;
CREATE TABLE va_registration_custom_properties (
  property_id INT4 NOT NULL DEFAULT nextval('seq_va_registration_custom_properties'),
  site_id INT4 default '1',
  property_order INT4 default '0',
  property_name VARCHAR(255),
  property_description TEXT,
  default_value TEXT,
  property_class VARCHAR(64),
  property_style VARCHAR(255),
  section_id INT4 default '0',
  property_show SMALLINT default '0',
  control_type VARCHAR(16),
  control_style VARCHAR(255),
  control_code TEXT,
  onchange_code TEXT,
  onclick_code TEXT,
  required SMALLINT default '0',
  before_name_html TEXT,
  after_name_html TEXT,
  before_control_html TEXT,
  after_control_html TEXT,
  validation_regexp TEXT,
  regexp_error TEXT,
  options_values_sql TEXT
  ,PRIMARY KEY (property_id));

CREATE INDEX va_registration_custom_prop_75 ON va_registration_custom_properties (site_id);


DROP SEQUENCE seq_va_registration_custom_values CASCADE;
DROP TABLE va_registration_custom_values;

CREATE SEQUENCE seq_va_registration_custom_values START 1;
CREATE TABLE va_registration_custom_values (
  property_value_id INT4 NOT NULL DEFAULT nextval('seq_va_registration_custom_values'),
  property_id INT4 default '0',
  property_value VARCHAR(255),
  hide_value SMALLINT default '0',
  is_default_value SMALLINT default '0'
  ,PRIMARY KEY (property_value_id));

CREATE INDEX va_registration_custom_valu_76 ON va_registration_custom_values (property_id);


DROP SEQUENCE seq_va_registration_items CASCADE;
DROP TABLE va_registration_items;

CREATE SEQUENCE seq_va_registration_items START 1;
CREATE TABLE va_registration_items (
  item_id INT4 NOT NULL DEFAULT nextval('seq_va_registration_items'),
  item_code VARCHAR(64),
  item_name VARCHAR(255),
  item_order INT4 default '1',
  show_for_user SMALLINT default '1',
  admin_id_added_by INT4 default '0',
  admin_id_modified_by INT4 default '0',
  date_added TIMESTAMP,
  date_modified TIMESTAMP
  ,PRIMARY KEY (item_id));

CREATE INDEX va_registration_items_item_code ON va_registration_items (item_code);
CREATE INDEX va_registration_items_item_order ON va_registration_items (item_order);


DROP SEQUENCE seq_va_registration_items_assigned CASCADE;
DROP TABLE va_registration_items_assigned;

CREATE SEQUENCE seq_va_registration_items_assigned START 1;
CREATE TABLE va_registration_items_assigned (
  item_id INT4 NOT NULL default '0',
  category_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (item_id,category_id));



DROP SEQUENCE seq_va_registration_list CASCADE;
DROP TABLE va_registration_list;

CREATE SEQUENCE seq_va_registration_list START 1;
CREATE TABLE va_registration_list (
  registration_id INT4 NOT NULL DEFAULT nextval('seq_va_registration_list'),
  user_id INT4 default '0',
  is_approved SMALLINT default '0',
  category_id INT4 default '0',
  item_id INT4 default '0',
  item_code VARCHAR(64),
  item_name VARCHAR(255),
  serial_number VARCHAR(128),
  invoice_number VARCHAR(128),
  store_name VARCHAR(128),
  purchased_day SMALLINT default '0',
  purchased_month SMALLINT default '0',
  purchased_year INT4 default '0',
  admin_id_added_by INT4 default '0',
  admin_id_modified_by INT4 default '0',
  date_added TIMESTAMP,
  date_modified TIMESTAMP
  ,PRIMARY KEY (registration_id));

CREATE INDEX va_registration_list_category_id ON va_registration_list (category_id);
CREATE INDEX va_registration_list_invoic_77 ON va_registration_list (invoice_number);
CREATE INDEX va_registration_list_item_code ON va_registration_list (item_code);
CREATE INDEX va_registration_list_item_id ON va_registration_list (item_id);
CREATE INDEX va_registration_list_purcha_78 ON va_registration_list (purchased_day);
CREATE INDEX va_registration_list_purcha_79 ON va_registration_list (purchased_month);
CREATE INDEX va_registration_list_purcha_80 ON va_registration_list (purchased_year);
CREATE INDEX va_registration_list_serial_81 ON va_registration_list (serial_number);
CREATE INDEX va_registration_list_user_id ON va_registration_list (user_id);


DROP SEQUENCE seq_va_registration_properties CASCADE;
DROP TABLE va_registration_properties;

CREATE SEQUENCE seq_va_registration_properties START 1;
CREATE TABLE va_registration_properties (
  registration_property_id INT4 NOT NULL DEFAULT nextval('seq_va_registration_properties'),
  registration_id INT4 default '0',
  property_id INT4 default '0',
  property_value TEXT
  ,PRIMARY KEY (registration_property_id));

CREATE INDEX va_registration_properties__82 ON va_registration_properties (property_id);
CREATE INDEX va_registration_properties__83 ON va_registration_properties (registration_id);


DROP SEQUENCE seq_va_release_changes CASCADE;
DROP TABLE va_release_changes;

CREATE SEQUENCE seq_va_release_changes START 28;
CREATE TABLE va_release_changes (
  change_id INT4 NOT NULL DEFAULT nextval('seq_va_release_changes'),
  release_id INT4 NOT NULL default '0',
  change_type_id INT4 NOT NULL default '0',
  change_date TIMESTAMP NOT NULL,
  change_desc TEXT,
  is_showing SMALLINT NOT NULL default '1'
  ,PRIMARY KEY (change_id));

CREATE INDEX va_release_changes_release_id ON va_release_changes (release_id);


DROP SEQUENCE seq_va_release_types CASCADE;
DROP TABLE va_release_types;

CREATE SEQUENCE seq_va_release_types START 4;
CREATE TABLE va_release_types (
  type_id INT4 NOT NULL DEFAULT nextval('seq_va_release_types'),
  type_name VARCHAR(255) NOT NULL
  ,PRIMARY KEY (type_id));



DROP SEQUENCE seq_va_releases CASCADE;
DROP TABLE va_releases;

CREATE SEQUENCE seq_va_releases START 2;
CREATE TABLE va_releases (
  release_id INT4 NOT NULL DEFAULT nextval('seq_va_releases'),
  item_id INT4 NOT NULL default '0',
  release_type_id INT4 NOT NULL default '0',
  release_date TIMESTAMP NOT NULL,
  release_title VARCHAR(255) NOT NULL,
  version VARCHAR(32),
  release_desc TEXT,
  download_type INT4 NOT NULL default '0',
  path_to_file VARCHAR(255),
  is_showing SMALLINT NOT NULL default '1',
  show_on_index SMALLINT NOT NULL default '0'
  ,PRIMARY KEY (release_id));

CREATE INDEX va_releases_item_id ON va_releases (item_id);


DROP SEQUENCE seq_va_reminders CASCADE;
DROP TABLE va_reminders;

CREATE SEQUENCE seq_va_reminders START 1;
CREATE TABLE va_reminders (
  reminder_id INT4 NOT NULL DEFAULT nextval('seq_va_reminders'),
  user_id INT4 default '0',
  start_date TIMESTAMP,
  end_date TIMESTAMP,
  reminder_year INT4 default '0',
  reminder_month SMALLINT default '0',
  reminder_day SMALLINT default '0',
  reminder_weekdays SMALLINT default '0',
  reminder_title VARCHAR(255),
  reminder_notes TEXT,
  date_added TIMESTAMP,
  date_modified TIMESTAMP,
  date_sent TIMESTAMP
  ,PRIMARY KEY (reminder_id));

CREATE INDEX va_reminders_end_date ON va_reminders (end_date);
CREATE INDEX va_reminders_reminder_day ON va_reminders (reminder_day);
CREATE INDEX va_reminders_reminder_month ON va_reminders (reminder_month);
CREATE INDEX va_reminders_reminder_weekdays ON va_reminders (reminder_weekdays);
CREATE INDEX va_reminders_reminder_year ON va_reminders (reminder_year);
CREATE INDEX va_reminders_start_date ON va_reminders (start_date);
CREATE INDEX va_reminders_user_id ON va_reminders (user_id);


DROP SEQUENCE seq_va_reviews CASCADE;
DROP TABLE va_reviews;

CREATE SEQUENCE seq_va_reviews START 1;
CREATE TABLE va_reviews (
  review_id INT4 NOT NULL DEFAULT nextval('seq_va_reviews'),
  item_id INT4 NOT NULL default '0',
  user_id INT4 default '0',
  admin_id INT4 default '0',
  recommended SMALLINT NOT NULL default '0',
  approved SMALLINT NOT NULL default '0',
  rating INT4 default '0',
  summary VARCHAR(255),
  user_name VARCHAR(64),
  user_email VARCHAR(128),
  remote_address VARCHAR(32) NOT NULL,
  comments TEXT,
  admin_id_approved_by INT4,
  admin_id_modified_by INT4,
  date_added TIMESTAMP NOT NULL,
  date_modified TIMESTAMP,
  date_approved TIMESTAMP
  ,PRIMARY KEY (review_id));

CREATE INDEX va_reviews_admin_id ON va_reviews (admin_id);
CREATE INDEX va_reviews_approved ON va_reviews (approved);
CREATE INDEX va_reviews_item_id ON va_reviews (item_id);
CREATE INDEX va_reviews_recommended ON va_reviews (recommended);
CREATE INDEX va_reviews_user_id ON va_reviews (user_id);


DROP SEQUENCE seq_va_saved_carts CASCADE;
DROP TABLE va_saved_carts;

CREATE SEQUENCE seq_va_saved_carts START 1;
CREATE TABLE va_saved_carts (
  cart_id INT4 NOT NULL DEFAULT nextval('seq_va_saved_carts'),
  site_id INT4 default '1',
  user_id INT4 default '0',
  cart_name VARCHAR(255),
  cart_total FLOAT4 default '0',
  cart_added TIMESTAMP
  ,PRIMARY KEY (cart_id));

CREATE INDEX va_saved_carts_site_id ON va_saved_carts (site_id);
CREATE INDEX va_saved_carts_user_id ON va_saved_carts (user_id);


DROP SEQUENCE seq_va_saved_items CASCADE;
DROP TABLE va_saved_items;

CREATE SEQUENCE seq_va_saved_items START 1;
CREATE TABLE va_saved_items (
  cart_item_id INT4 NOT NULL DEFAULT nextval('seq_va_saved_items'),
  site_id INT4 default '1',
  item_id INT4 NOT NULL default '0',
  cart_id INT4 NOT NULL default '0',
  user_id INT4 NOT NULL default '0',
  type_id INT4 NOT NULL default '0',
  item_name VARCHAR(255),
  quantity INT4 default '0',
  quantity_bought INT4 default '0',
  price FLOAT4 default '0',
  date_added TIMESTAMP
  ,PRIMARY KEY (cart_item_id));

CREATE INDEX va_saved_items_cart_id ON va_saved_items (cart_id);
CREATE INDEX va_saved_items_item_id ON va_saved_items (item_id);
CREATE INDEX va_saved_items_site_id ON va_saved_items (site_id);
CREATE INDEX va_saved_items_type_id ON va_saved_items (type_id);
CREATE INDEX va_saved_items_user_id ON va_saved_items (user_id);


DROP SEQUENCE seq_va_saved_items_properties CASCADE;
DROP TABLE va_saved_items_properties;

CREATE SEQUENCE seq_va_saved_items_properties START 1;
CREATE TABLE va_saved_items_properties (
  item_property_id INT4 NOT NULL DEFAULT nextval('seq_va_saved_items_properties'),
  cart_item_id INT4 default '0',
  cart_id INT4 default '0',
  property_id INT4 default '0',
  property_value TEXT,
  property_values_ids TEXT
  ,PRIMARY KEY (item_property_id));

CREATE INDEX va_saved_items_properties_b_84 ON va_saved_items_properties (cart_item_id);
CREATE INDEX va_saved_items_properties_b_85 ON va_saved_items_properties (cart_id);
CREATE INDEX va_saved_items_properties_p_86 ON va_saved_items_properties (property_id);


DROP SEQUENCE seq_va_saved_types CASCADE;
DROP TABLE va_saved_types;

CREATE SEQUENCE seq_va_saved_types START 2;
CREATE TABLE va_saved_types (
  type_id INT4 NOT NULL DEFAULT nextval('seq_va_saved_types'),
  type_name VARCHAR(64),
  type_desc TEXT,
  is_active SMALLINT default '1',
  allowed_search SMALLINT default '0'
  ,PRIMARY KEY (type_id));



DROP SEQUENCE seq_va_search_engines CASCADE;
DROP TABLE va_search_engines;

CREATE SEQUENCE seq_va_search_engines START 12;
CREATE TABLE va_search_engines (
  engine_id INT4 NOT NULL DEFAULT nextval('seq_va_search_engines'),
  engine_name VARCHAR(64),
  keywords_parameter VARCHAR(32),
  referer_regexp TEXT,
  user_agent_regexp TEXT,
  ip_regexp TEXT
  ,PRIMARY KEY (engine_id));



DROP SEQUENCE seq_va_shipping_modules CASCADE;
DROP TABLE va_shipping_modules;

CREATE SEQUENCE seq_va_shipping_modules START 10;
CREATE TABLE va_shipping_modules (
  shipping_module_id INT4 NOT NULL DEFAULT nextval('seq_va_shipping_modules'),
  shipping_module_name VARCHAR(255),
  user_module_name VARCHAR(255),
  module_notes TEXT,
  admin_order_class VARCHAR(64),
  user_order_class VARCHAR(64),
  is_external SMALLINT default '1',
  php_external_lib VARCHAR(255),
  external_url VARCHAR(255),
  tracking_url VARCHAR(255),
  is_active SMALLINT default '1',
  is_default SMALLINT default '1',
  is_call_center SMALLINT default '1',
  cost_add_percent FLOAT4 default '0'
  ,PRIMARY KEY (shipping_module_id));



DROP SEQUENCE seq_va_shipping_modules_parameters CASCADE;
DROP TABLE va_shipping_modules_parameters;

CREATE SEQUENCE seq_va_shipping_modules_parameters START 158;
CREATE TABLE va_shipping_modules_parameters (
  parameter_id INT4 NOT NULL DEFAULT nextval('seq_va_shipping_modules_parameters'),
  shipping_module_id INT4 default '0',
  parameter_name VARCHAR(255),
  parameter_source TEXT,
  not_passed SMALLINT default '0'
  ,PRIMARY KEY (parameter_id));

CREATE INDEX va_shipping_modules_paramet_87 ON va_shipping_modules_parameters (shipping_module_id);


DROP SEQUENCE seq_va_shipping_rules CASCADE;
DROP TABLE va_shipping_rules;

CREATE SEQUENCE seq_va_shipping_rules START 4;
CREATE TABLE va_shipping_rules (
  shipping_rule_id INT4 NOT NULL DEFAULT nextval('seq_va_shipping_rules'),
  shipping_rule_desc VARCHAR(255) NOT NULL,
  is_country_restriction SMALLINT default '0'
  ,PRIMARY KEY (shipping_rule_id));



DROP SEQUENCE seq_va_shipping_rules_countries CASCADE;
DROP TABLE va_shipping_rules_countries;

CREATE SEQUENCE seq_va_shipping_rules_countries START 1;
CREATE TABLE va_shipping_rules_countries (
  shipping_rule_id INT4 NOT NULL default '0',
  country_id INT4 NOT NULL default '0'
);

CREATE INDEX va_shipping_rules_countries_88 ON va_shipping_rules_countries (country_id);
CREATE INDEX va_shipping_rules_countries_89 ON va_shipping_rules_countries (shipping_rule_id);


DROP SEQUENCE seq_va_shipping_times CASCADE;
DROP TABLE va_shipping_times;

CREATE SEQUENCE seq_va_shipping_times START 3;
CREATE TABLE va_shipping_times (
  shipping_time_id INT4 NOT NULL DEFAULT nextval('seq_va_shipping_times'),
  shipping_time_desc VARCHAR(255) NOT NULL,
  availability_time INT4 default '0'
  ,PRIMARY KEY (shipping_time_id));



DROP SEQUENCE seq_va_shipping_types CASCADE;
DROP TABLE va_shipping_types;

CREATE SEQUENCE seq_va_shipping_types START 163;
CREATE TABLE va_shipping_types (
  shipping_type_id INT4 NOT NULL DEFAULT nextval('seq_va_shipping_types'),
  shipping_module_id INT4 default '0',
  shipping_order INT4 default '1',
  shipping_type_code VARCHAR(128),
  shipping_type_desc VARCHAR(255) NOT NULL,
  shipping_time INT4 default '0',
  admin_order_class VARCHAR(64),
  user_order_class VARCHAR(64),
  tare_weight FLOAT4 default '0',
  min_goods_cost FLOAT4 default '0',
  max_goods_cost FLOAT4 default '0',
  min_weight FLOAT4 default '0',
  max_weight FLOAT4 default '0',
  min_quantity INT4,
  max_quantity INT4,
  cost_per_order FLOAT4 default '0',
  cost_per_product FLOAT4 default '0',
  cost_per_weight FLOAT4 default '0',
  is_taxable SMALLINT default '1',
  is_active SMALLINT default '1',
  is_call_center SMALLINT default '1',
  sites_all SMALLINT default '1',
  guest_access SMALLINT default '1',
  user_types_all SMALLINT default '1',
  countries_all SMALLINT default '1',
  states_all SMALLINT default '1',
  postal_match_type SMALLINT default '1',
  postal_codes TEXT
  ,PRIMARY KEY (shipping_type_id));

CREATE INDEX va_shipping_types_shipping__90 ON va_shipping_types (shipping_module_id);
CREATE INDEX va_shipping_types_sites_all ON va_shipping_types (sites_all);


DROP SEQUENCE seq_va_shipping_types_countries CASCADE;
DROP TABLE va_shipping_types_countries;

CREATE SEQUENCE seq_va_shipping_types_countries START 1;
CREATE TABLE va_shipping_types_countries (
  shipping_type_id INT4 NOT NULL default '0',
  country_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (shipping_type_id,country_id));



DROP SEQUENCE seq_va_shipping_types_sites CASCADE;
DROP TABLE va_shipping_types_sites;

CREATE SEQUENCE seq_va_shipping_types_sites START 1;
CREATE TABLE va_shipping_types_sites (
  shipping_type_id INT4 NOT NULL default '0',
  site_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (shipping_type_id,site_id));



DROP SEQUENCE seq_va_shipping_types_states CASCADE;
DROP TABLE va_shipping_types_states;

CREATE SEQUENCE seq_va_shipping_types_states START 1;
CREATE TABLE va_shipping_types_states (
  shipping_type_id INT4 NOT NULL default '0',
  state_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (shipping_type_id,state_id));



DROP SEQUENCE seq_va_shipping_types_users CASCADE;
DROP TABLE va_shipping_types_users;

CREATE SEQUENCE seq_va_shipping_types_users START 1;
CREATE TABLE va_shipping_types_users (
  shipping_type_id INT4 default '0',
  user_type_id INT4 default '0'
);

CREATE INDEX va_shipping_types_users_shi_91 ON va_shipping_types_users (shipping_type_id);
CREATE INDEX va_shipping_types_users_use_92 ON va_shipping_types_users (user_type_id);


DROP SEQUENCE seq_va_sites CASCADE;
DROP TABLE va_sites;

CREATE SEQUENCE seq_va_sites START 2;
CREATE TABLE va_sites (
  site_id INT4 NOT NULL DEFAULT nextval('seq_va_sites'),
  parent_site_id INT4,
  is_mobile SMALLINT default '0',
  is_mobile_redirect SMALLINT default '0',
  short_name VARCHAR(16),
  site_name VARCHAR(255),
  site_description TEXT
  ,PRIMARY KEY (site_id));



DROP SEQUENCE seq_va_sliders CASCADE;
DROP TABLE va_sliders;

CREATE SEQUENCE seq_va_sliders START 1;
CREATE TABLE va_sliders (
  slider_id INT4 NOT NULL DEFAULT nextval('seq_va_sliders'),
  slider_name VARCHAR(64),
  slider_title VARCHAR(255),
  slider_height VARCHAR(16),
  slider_width VARCHAR(16)
  ,PRIMARY KEY (slider_id));



DROP SEQUENCE seq_va_sliders_items CASCADE;
DROP TABLE va_sliders_items;

CREATE SEQUENCE seq_va_sliders_items START 1;
CREATE TABLE va_sliders_items (
  item_id INT4 NOT NULL DEFAULT nextval('seq_va_sliders_items'),
  slider_id INT4 NOT NULL default '0',
  show_for_user SMALLINT default '1',
  item_order INT4 default '0',
  item_name VARCHAR(255),
  slider_image VARCHAR(255),
  slider_link VARCHAR(255),
  slider_html TEXT
  ,PRIMARY KEY (item_id));

CREATE INDEX va_sliders_items_slider_id ON va_sliders_items (slider_id);


DROP SEQUENCE seq_va_states CASCADE;
DROP TABLE va_states;

CREATE SEQUENCE seq_va_states START 52;
CREATE TABLE va_states (
  state_id INT4 NOT NULL DEFAULT nextval('seq_va_states'),
  state_code VARCHAR(8),
  country_id INT4 default '0',
  state_name VARCHAR(64) NOT NULL,
  show_for_user SMALLINT default '1'
  ,PRIMARY KEY (state_id));

CREATE INDEX va_states_country_code ON va_states (country_id);
CREATE INDEX va_states_state_code ON va_states (state_code);


DROP SEQUENCE seq_va_subscriptions CASCADE;
DROP TABLE va_subscriptions;

CREATE SEQUENCE seq_va_subscriptions START 1;
CREATE TABLE va_subscriptions (
  subscription_id INT4 NOT NULL DEFAULT nextval('seq_va_subscriptions'),
  group_id INT4 default '0',
  user_type_id INT4 default '0',
  subscription_name VARCHAR(255),
  is_active SMALLINT default '1',
  is_default SMALLINT default '0',
  is_subscription_recurring SMALLINT default '1',
  subscription_fee FLOAT4 default '0',
  subscription_period INT4,
  subscription_interval INT4,
  subscription_suspend INT4 default '0',
  subscription_points_type SMALLINT,
  subscription_points_amount FLOAT4,
  subscription_credits_type SMALLINT,
  subscription_credits_amount FLOAT4,
  subscription_affiliate_type SMALLINT,
  subscription_affiliate_amount INT4
  ,PRIMARY KEY (subscription_id));

CREATE INDEX va_subscriptions_group_id ON va_subscriptions (group_id);
CREATE INDEX va_subscriptions_user_type_id ON va_subscriptions (user_type_id);


DROP SEQUENCE seq_va_subscriptions_groups CASCADE;
DROP TABLE va_subscriptions_groups;

CREATE SEQUENCE seq_va_subscriptions_groups START 1;
CREATE TABLE va_subscriptions_groups (
  group_id INT4 NOT NULL DEFAULT nextval('seq_va_subscriptions_groups'),
  group_name VARCHAR(255),
  is_active SMALLINT default '1'
  ,PRIMARY KEY (group_id));



DROP SEQUENCE seq_va_suppliers CASCADE;
DROP TABLE va_suppliers;

CREATE SEQUENCE seq_va_suppliers START 1;
CREATE TABLE va_suppliers (
  supplier_id INT4 NOT NULL DEFAULT nextval('seq_va_suppliers'),
  supplier_order INT4 default '1',
  supplier_name VARCHAR(255),
  supplier_email VARCHAR(255),
  short_description TEXT,
  full_description TEXT,
  image_small VARCHAR(255),
  image_small_alt VARCHAR(255),
  image_large VARCHAR(255),
  image_large_alt VARCHAR(255)
  ,PRIMARY KEY (supplier_id));



DROP SEQUENCE seq_va_support CASCADE;
DROP TABLE va_support;

CREATE SEQUENCE seq_va_support START 1;
CREATE TABLE va_support (
  support_id INT4 NOT NULL DEFAULT nextval('seq_va_support'),
  site_id INT4 NOT NULL default '1',
  user_id INT4 NOT NULL default '0',
  user_name VARCHAR(64) NOT NULL,
  user_email VARCHAR(128) NOT NULL,
  affiliate_code VARCHAR(64),
  mail_cc VARCHAR(255),
  mail_bcc VARCHAR(255),
  mail_headers TEXT,
  remote_address VARCHAR(32),
  identifier VARCHAR(128),
  environment VARCHAR(255),
  support_product_id INT4 NOT NULL default '0',
  support_type_id INT4 NOT NULL default '0',
  dep_id INT4 NOT NULL default '0',
  summary VARCHAR(255) NOT NULL,
  description TEXT,
  mail_body_text TEXT,
  mail_body_html TEXT,
  support_status_id INT4 default '0',
  support_priority_id INT4 default '0',
  admin_id_assign_to INT4 NOT NULL default '0',
  admin_id_assign_by INT4 NOT NULL default '0',
  admin_id_added_by INT4 NOT NULL default '0',
  admin_id_modified_by INT4 NOT NULL default '0',
  date_added TIMESTAMP NOT NULL,
  date_modified TIMESTAMP NOT NULL,
  date_viewed TIMESTAMP
  ,PRIMARY KEY (support_id));

CREATE INDEX va_support_admin_id_added_by ON va_support (admin_id_added_by);
CREATE INDEX va_support_admin_id_assign_by ON va_support (admin_id_assign_by);
CREATE INDEX va_support_admin_id_assign_to ON va_support (admin_id_assign_to);
CREATE INDEX va_support_admin_id_modified_by ON va_support (admin_id_modified_by);
CREATE INDEX va_support_date_added ON va_support (date_added);
CREATE INDEX va_support_dep_id ON va_support (dep_id);
CREATE INDEX va_support_site_id ON va_support (site_id);
CREATE INDEX va_support_summary ON va_support (summary);
CREATE INDEX va_support_support_product_id ON va_support (support_product_id);
CREATE INDEX va_support_support_type_id ON va_support (support_type_id);
CREATE INDEX va_support_user_email ON va_support (user_email);
CREATE INDEX va_support_user_id ON va_support (user_id);
CREATE INDEX va_support_user_name ON va_support (user_name);


DROP SEQUENCE seq_va_support_attachments CASCADE;
DROP TABLE va_support_attachments;

CREATE SEQUENCE seq_va_support_attachments START 1;
CREATE TABLE va_support_attachments (
  attachment_id INT4 NOT NULL DEFAULT nextval('seq_va_support_attachments'),
  support_id INT4 NOT NULL default '0',
  message_id INT4 default '0',
  admin_id INT4 default '0',
  user_id INT4 default '0',
  session_id VARCHAR(32),
  attachment_status INT4 default '0',
  file_name VARCHAR(255),
  file_path VARCHAR(255),
  date_added TIMESTAMP
  ,PRIMARY KEY (attachment_id));

CREATE INDEX va_support_attachments_admin_id ON va_support_attachments (admin_id);
CREATE INDEX va_support_attachments_mess_93 ON va_support_attachments (message_id);
CREATE INDEX va_support_attachments_sess_94 ON va_support_attachments (session_id);
CREATE INDEX va_support_attachments_supp_95 ON va_support_attachments (support_id);
CREATE INDEX va_support_attachments_user_id ON va_support_attachments (user_id);


DROP SEQUENCE seq_va_support_custom_properties CASCADE;
DROP TABLE va_support_custom_properties;

CREATE SEQUENCE seq_va_support_custom_properties START 1;
CREATE TABLE va_support_custom_properties (
  property_id INT4 NOT NULL DEFAULT nextval('seq_va_support_custom_properties'),
  site_id INT4 default '1',
  property_order INT4 default '0',
  property_name VARCHAR(255),
  property_description TEXT,
  default_value TEXT,
  property_class VARCHAR(64),
  property_style VARCHAR(255),
  section_id INT4 default '0',
  property_show SMALLINT default '0',
  control_type VARCHAR(16),
  control_style VARCHAR(255),
  control_code TEXT,
  onchange_code TEXT,
  onclick_code TEXT,
  required SMALLINT default '0',
  before_name_html TEXT,
  after_name_html TEXT,
  before_control_html TEXT,
  after_control_html TEXT,
  validation_regexp TEXT,
  regexp_error TEXT,
  options_values_sql TEXT
  ,PRIMARY KEY (property_id));

CREATE INDEX va_support_custom_propertie_96 ON va_support_custom_properties (site_id);


DROP SEQUENCE seq_va_support_custom_values CASCADE;
DROP TABLE va_support_custom_values;

CREATE SEQUENCE seq_va_support_custom_values START 1;
CREATE TABLE va_support_custom_values (
  property_value_id INT4 NOT NULL DEFAULT nextval('seq_va_support_custom_values'),
  property_id INT4 default '0',
  property_value VARCHAR(255),
  hide_value SMALLINT default '0',
  is_default_value SMALLINT default '0'
  ,PRIMARY KEY (property_value_id));

CREATE INDEX va_support_custom_values_pr_97 ON va_support_custom_values (property_id);


DROP SEQUENCE seq_va_support_departments CASCADE;
DROP TABLE va_support_departments;

CREATE SEQUENCE seq_va_support_departments START 2;
CREATE TABLE va_support_departments (
  dep_id INT4 NOT NULL DEFAULT nextval('seq_va_support_departments'),
  short_title VARCHAR(255),
  full_title VARCHAR(255),
  signature TEXT,
  attachments_dir VARCHAR(255),
  attachments_mask TEXT,
  incoming_account TEXT,
  incoming_type_id INT4 default '0',
  incoming_product_id INT4 default '0',
  outgoing_account VARCHAR(50),
  show_for_user SMALLINT default '1',
  sites_all SMALLINT default '1'
  ,PRIMARY KEY (dep_id));

CREATE INDEX va_support_departments_inco_98 ON va_support_departments (incoming_product_id);
CREATE INDEX va_support_departments_inco_99 ON va_support_departments (incoming_type_id);
CREATE INDEX va_support_departments_sites_all ON va_support_departments (sites_all);


DROP SEQUENCE seq_va_support_departments_sites CASCADE;
DROP TABLE va_support_departments_sites;

CREATE SEQUENCE seq_va_support_departments_sites START 1;
CREATE TABLE va_support_departments_sites (
  dep_id INT4 NOT NULL default '0',
  site_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (dep_id,site_id));



DROP SEQUENCE seq_va_support_messages CASCADE;
DROP TABLE va_support_messages;

CREATE SEQUENCE seq_va_support_messages START 1;
CREATE TABLE va_support_messages (
  message_id INT4 NOT NULL DEFAULT nextval('seq_va_support_messages'),
  support_id INT4 NOT NULL default '0',
  internal SMALLINT NOT NULL default '0',
  dep_id INT4 NOT NULL default '0',
  support_status_id INT4 NOT NULL default '0',
  admin_id_assign_by INT4 NOT NULL default '0',
  admin_id_assign_to INT4 NOT NULL default '0',
  admin_id_modified_by INT4 NOT NULL default '0',
  user_rating SMALLINT,
  user_rating_comments VARCHAR(128),
  user_rating_id INT4,
  user_rating_ip VARCHAR(32),
  admin_rating SMALLINT,
  admin_rating_comments VARCHAR(255),
  admin_rating_id INT4,
  admin_rating_ip VARCHAR(32),
  subject VARCHAR(255),
  message_text TEXT,
  mail_body_text TEXT,
  mail_body_html TEXT,
  reply_to VARCHAR(255),
  reply_from VARCHAR(255),
  mail_cc VARCHAR(255),
  mail_headers TEXT,
  date_added TIMESTAMP NOT NULL,
  date_viewed TIMESTAMP,
  date_modified TIMESTAMP,
  is_user_reply SMALLINT default '0',
  is_html SMALLINT default '0'
  ,PRIMARY KEY (message_id));

CREATE INDEX va_support_messages_admin_i_100 ON va_support_messages (admin_id_assign_by);
CREATE INDEX va_support_messages_admin_i_101 ON va_support_messages (admin_id_assign_to);
CREATE INDEX va_support_messages_admin_i_102 ON va_support_messages (admin_id_modified_by);
CREATE INDEX va_support_messages_date_added ON va_support_messages (date_added);
CREATE INDEX va_support_messages_dep_id ON va_support_messages (dep_id);
CREATE INDEX va_support_messages_support_id ON va_support_messages (support_id);
CREATE INDEX va_support_messages_support_103 ON va_support_messages (support_status_id);


DROP SEQUENCE seq_va_support_predefined CASCADE;
DROP TABLE va_support_predefined;

CREATE SEQUENCE seq_va_support_predefined START 1;
CREATE TABLE va_support_predefined (
  reply_id INT4 NOT NULL DEFAULT nextval('seq_va_support_predefined'),
  type_id INT4 default '0',
  subject VARCHAR(255),
  body TEXT,
  total_uses INT4 default '0',
  admin_id_added_by INT4 default '0',
  admin_id_modified_by INT4 default '0',
  date_added TIMESTAMP,
  date_modified TIMESTAMP,
  last_updated TIMESTAMP
  ,PRIMARY KEY (reply_id));

CREATE INDEX va_support_predefined_type_id ON va_support_predefined (type_id);


DROP SEQUENCE seq_va_support_predefined_types CASCADE;
DROP TABLE va_support_predefined_types;

CREATE SEQUENCE seq_va_support_predefined_types START 1;
CREATE TABLE va_support_predefined_types (
  type_id INT4 NOT NULL DEFAULT nextval('seq_va_support_predefined_types'),
  type_name VARCHAR(64)
  ,PRIMARY KEY (type_id));



DROP SEQUENCE seq_va_support_priorities CASCADE;
DROP TABLE va_support_priorities;

CREATE SEQUENCE seq_va_support_priorities START 4;
CREATE TABLE va_support_priorities (
  priority_id INT4 NOT NULL DEFAULT nextval('seq_va_support_priorities'),
  priority_name VARCHAR(255) NOT NULL,
  priority_rank INT4 default '0',
  admin_html TEXT,
  is_default SMALLINT default '0'
  ,PRIMARY KEY (priority_id));



DROP SEQUENCE seq_va_support_products CASCADE;
DROP TABLE va_support_products;

CREATE SEQUENCE seq_va_support_products START 1;
CREATE TABLE va_support_products (
  product_id INT4 NOT NULL DEFAULT nextval('seq_va_support_products'),
  product_name VARCHAR(255) NOT NULL,
  show_for_user SMALLINT default '1',
  sites_all INT4 default '1'
  ,PRIMARY KEY (product_id));

CREATE INDEX va_support_products_sites_all ON va_support_products (sites_all);


DROP SEQUENCE seq_va_support_products_sites CASCADE;
DROP TABLE va_support_products_sites;

CREATE SEQUENCE seq_va_support_products_sites START 1;
CREATE TABLE va_support_products_sites (
  product_id INT4 NOT NULL default '0',
  site_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (product_id,site_id));



DROP SEQUENCE seq_va_support_properties CASCADE;
DROP TABLE va_support_properties;

CREATE SEQUENCE seq_va_support_properties START 1;
CREATE TABLE va_support_properties (
  support_property_id INT4 NOT NULL DEFAULT nextval('seq_va_support_properties'),
  support_id INT4 default '0',
  property_id INT4 default '0',
  property_value TEXT
  ,PRIMARY KEY (support_property_id));

CREATE INDEX va_support_properties_prope_104 ON va_support_properties (property_id);
CREATE INDEX va_support_properties_support_id ON va_support_properties (support_id);


DROP SEQUENCE seq_va_support_statuses CASCADE;
DROP TABLE va_support_statuses;

CREATE SEQUENCE seq_va_support_statuses START 10;
CREATE TABLE va_support_statuses (
  status_id INT4 NOT NULL DEFAULT nextval('seq_va_support_statuses'),
  status_name VARCHAR(255) NOT NULL,
  show_for_user SMALLINT default '1',
  status_caption VARCHAR(255),
  status_icon VARCHAR(128),
  admin_ticket_class VARCHAR(64),
  user_ticket_class VARCHAR(64),
  html_start VARCHAR(255),
  html_end VARCHAR(255),
  is_user_new SMALLINT default '0',
  is_user_reply SMALLINT default '0',
  is_admin_reply SMALLINT default '0',
  is_operation SMALLINT default '0',
  is_reassign SMALLINT default '0',
  is_internal SMALLINT default '0',
  is_list SMALLINT default '0',
  is_closed SMALLINT default '0',
  is_add_knowledge SMALLINT default '0',
  admin_notify SMALLINT default '0',
  admin_to VARCHAR(255),
  admin_from VARCHAR(64),
  admin_cc VARCHAR(255),
  admin_bcc VARCHAR(255),
  admin_reply_to VARCHAR(64),
  admin_return_path VARCHAR(64),
  admin_mail_type SMALLINT default '0',
  admin_subject VARCHAR(255),
  admin_body TEXT,
  manager_by_notify SMALLINT default '0',
  manager_by_to VARCHAR(255),
  manager_by_from VARCHAR(64),
  manager_by_cc VARCHAR(255),
  manager_by_bcc VARCHAR(255),
  manager_by_reply_to VARCHAR(64),
  manager_by_return_path VARCHAR(64),
  manager_by_mail_type SMALLINT default '0',
  manager_by_subject VARCHAR(255),
  manager_by_body TEXT,
  manager_to_notify SMALLINT default '0',
  manager_to_to VARCHAR(255),
  manager_to_from VARCHAR(64),
  manager_to_cc VARCHAR(255),
  manager_to_bcc VARCHAR(255),
  manager_to_reply_to VARCHAR(64),
  manager_to_return_path VARCHAR(64),
  manager_to_mail_type SMALLINT default '0',
  manager_to_subject VARCHAR(255),
  manager_to_body TEXT,
  user_notify SMALLINT default '0',
  user_to VARCHAR(255),
  user_from VARCHAR(64),
  user_cc VARCHAR(255),
  user_bcc VARCHAR(255),
  user_reply_to VARCHAR(64),
  user_return_path VARCHAR(64),
  user_mail_type SMALLINT default '0',
  user_subject VARCHAR(255),
  user_body TEXT
  ,PRIMARY KEY (status_id));



DROP SEQUENCE seq_va_support_time_report CASCADE;
DROP TABLE va_support_time_report;

CREATE SEQUENCE seq_va_support_time_report START 1;
CREATE TABLE va_support_time_report (
  report_id INT4 NOT NULL DEFAULT nextval('seq_va_support_time_report'),
  admin_id INT4 NOT NULL default '0',
  support_id INT4 NOT NULL default '0',
  started_date TIMESTAMP,
  report_date TIMESTAMP,
  spent_hours INT4 default '0'
  ,PRIMARY KEY (report_id));

CREATE INDEX va_support_time_report_admin_id ON va_support_time_report (admin_id);
CREATE INDEX va_support_time_report_supp_105 ON va_support_time_report (support_id);


DROP SEQUENCE seq_va_support_types CASCADE;
DROP TABLE va_support_types;

CREATE SEQUENCE seq_va_support_types START 6;
CREATE TABLE va_support_types (
  type_id INT4 NOT NULL DEFAULT nextval('seq_va_support_types'),
  type_name VARCHAR(255) NOT NULL,
  show_for_user SMALLINT default '1'
  ,PRIMARY KEY (type_id));



DROP SEQUENCE seq_va_support_users_departments CASCADE;
DROP TABLE va_support_users_departments;

CREATE SEQUENCE seq_va_support_users_departments START 1;
CREATE TABLE va_support_users_departments (
  admin_id INT4 NOT NULL default '0',
  dep_id INT4 NOT NULL default '0',
  is_default_dep SMALLINT default '0'
  ,PRIMARY KEY (admin_id,dep_id));



DROP SEQUENCE seq_va_support_users_priorities CASCADE;
DROP TABLE va_support_users_priorities;

CREATE SEQUENCE seq_va_support_users_priorities START 1;
CREATE TABLE va_support_users_priorities (
  user_priority_id INT4 NOT NULL DEFAULT nextval('seq_va_support_users_priorities'),
  priority_id INT4 default '0',
  user_id INT4 default '0',
  user_email VARCHAR(255),
  priority_expiry TIMESTAMP,
  admin_id_added_by INT4 default '0',
  admin_id_modified_by INT4 default '0',
  date_added TIMESTAMP,
  date_modified TIMESTAMP
  ,PRIMARY KEY (user_priority_id));

CREATE INDEX va_support_users_priorities_106 ON va_support_users_priorities (priority_id);
CREATE INDEX va_support_users_priorities_107 ON va_support_users_priorities (user_id);


DROP SEQUENCE seq_va_tags CASCADE;
DROP TABLE va_tags;

CREATE SEQUENCE seq_va_tags START 1;
CREATE TABLE va_tags (
  tag_id INT4 NOT NULL DEFAULT nextval('seq_va_tags'),
  tag_name VARCHAR(128),
  name_first VARCHAR(2),
  friendly_url VARCHAR(255)
  ,PRIMARY KEY (tag_id));

CREATE INDEX va_tags_friendly_url ON va_tags (friendly_url);
CREATE INDEX va_tags_name_first ON va_tags (name_first);
CREATE INDEX va_tags_tag_name ON va_tags (tag_name);


DROP SEQUENCE seq_va_tax_rates CASCADE;
DROP TABLE va_tax_rates;

CREATE SEQUENCE seq_va_tax_rates START 1;
CREATE TABLE va_tax_rates (
  tax_id INT4 NOT NULL DEFAULT nextval('seq_va_tax_rates'),
  is_default SMALLINT default '0',
  tax_name VARCHAR(255) NOT NULL,
  tax_type SMALLINT default '1',
  show_type SMALLINT default '0',
  postal_code TEXT,
  state_id INT4 default '0',
  country_id INT4 default '0',
  tax_percent FLOAT4 NOT NULL default '0',
  fixed_amount FLOAT4,
  shipping_tax_percent FLOAT4,
  shipping_fixed_amount FLOAT4
  ,PRIMARY KEY (tax_id));

CREATE INDEX va_tax_rates_country_id ON va_tax_rates (country_id);
CREATE INDEX va_tax_rates_state_id ON va_tax_rates (state_id);


DROP SEQUENCE seq_va_tax_rates_items CASCADE;
DROP TABLE va_tax_rates_items;

CREATE SEQUENCE seq_va_tax_rates_items START 1;
CREATE TABLE va_tax_rates_items (
  tax_item_id INT4 NOT NULL DEFAULT nextval('seq_va_tax_rates_items'),
  tax_id INT4 default '0',
  item_type_id INT4 default '0',
  tax_percent FLOAT4,
  fixed_amount FLOAT4
  ,PRIMARY KEY (tax_item_id));

CREATE INDEX va_tax_rates_items_item_type_id ON va_tax_rates_items (item_type_id);
CREATE INDEX va_tax_rates_items_tax_id ON va_tax_rates_items (tax_id);


DROP SEQUENCE seq_va_tracking_pages CASCADE;
DROP TABLE va_tracking_pages;

CREATE SEQUENCE seq_va_tracking_pages START 1;
CREATE TABLE va_tracking_pages (
  page_id INT4 NOT NULL DEFAULT nextval('seq_va_tracking_pages'),
  visit_id INT4 NOT NULL default '0',
  ip_long INT4 NOT NULL default '0',
  ip_text VARCHAR(32),
  forwarded_ips VARCHAR(255),
  request_uri VARCHAR(255),
  request_page VARCHAR(255),
  date_added TIMESTAMP NOT NULL,
  year_added SMALLINT NOT NULL default '0',
  month_added SMALLINT NOT NULL default '0',
  day_added SMALLINT NOT NULL default '0',
  hour_added SMALLINT NOT NULL default '0',
  site_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (page_id));

CREATE INDEX va_tracking_pages_date_added ON va_tracking_pages (date_added);
CREATE INDEX va_tracking_pages_day_added ON va_tracking_pages (day_added);
CREATE INDEX va_tracking_pages_hour_added ON va_tracking_pages (hour_added);
CREATE INDEX va_tracking_pages_ip_long ON va_tracking_pages (ip_long);
CREATE INDEX va_tracking_pages_month_added ON va_tracking_pages (month_added);
CREATE INDEX va_tracking_pages_site_id ON va_tracking_pages (site_id);
CREATE INDEX va_tracking_pages_visit_id ON va_tracking_pages (visit_id);
CREATE INDEX va_tracking_pages_year_added ON va_tracking_pages (year_added);


DROP SEQUENCE seq_va_tracking_visits CASCADE;
DROP TABLE va_tracking_visits;

CREATE SEQUENCE seq_va_tracking_visits START 1;
CREATE TABLE va_tracking_visits (
  visit_id INT4 NOT NULL DEFAULT nextval('seq_va_tracking_visits'),
  parent_visit_id INT4 default '0',
  visit_number INT4 default '1',
  ip_long INT4 NOT NULL default '0',
  ip_text VARCHAR(32) NOT NULL,
  forwarded_ips VARCHAR(50),
  affiliate_code VARCHAR(64),
  keywords VARCHAR(255),
  user_agent VARCHAR(255),
  request_uri VARCHAR(255),
  request_page VARCHAR(255),
  referer VARCHAR(255),
  referer_host VARCHAR(255),
  referer_engine_id INT4 NOT NULL default '0',
  robot_engine_id INT4 NOT NULL default '0',
  date_added TIMESTAMP NOT NULL,
  year_added SMALLINT NOT NULL default '0',
  month_added SMALLINT NOT NULL default '0',
  week_added INT4 NOT NULL default '0',
  day_added SMALLINT NOT NULL default '0',
  hour_added SMALLINT NOT NULL default '0',
  site_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (visit_id));

CREATE INDEX va_tracking_visits_affiliat_108 ON va_tracking_visits (affiliate_code);
CREATE INDEX va_tracking_visits_date_added ON va_tracking_visits (date_added);
CREATE INDEX va_tracking_visits_day_added ON va_tracking_visits (day_added);
CREATE INDEX va_tracking_visits_hour_added ON va_tracking_visits (hour_added);
CREATE INDEX va_tracking_visits_keyword__109 ON va_tracking_visits (keywords);
CREATE INDEX va_tracking_visits_month_added ON va_tracking_visits (month_added);
CREATE INDEX va_tracking_visits_parent_v_110 ON va_tracking_visits (parent_visit_id);
CREATE INDEX va_tracking_visits_referer__111 ON va_tracking_visits (referer_engine_id);
CREATE INDEX va_tracking_visits_referer_host ON va_tracking_visits (referer_host);
CREATE INDEX va_tracking_visits_remote_i_112 ON va_tracking_visits (ip_long);
CREATE INDEX va_tracking_visits_robot_en_113 ON va_tracking_visits (robot_engine_id);
CREATE INDEX va_tracking_visits_site_id ON va_tracking_visits (site_id);
CREATE INDEX va_tracking_visits_week_added ON va_tracking_visits (week_added);
CREATE INDEX va_tracking_visits_year_added ON va_tracking_visits (year_added);


DROP SEQUENCE seq_va_user_profile_properties CASCADE;
DROP TABLE va_user_profile_properties;

CREATE SEQUENCE seq_va_user_profile_properties START 1;
CREATE TABLE va_user_profile_properties (
  property_id INT4 NOT NULL DEFAULT nextval('seq_va_user_profile_properties'),
  user_type_id INT4 default '0',
  property_order INT4 default '0',
  property_code VARCHAR(64),
  property_name VARCHAR(255),
  property_description TEXT,
  default_value TEXT,
  property_class VARCHAR(64),
  property_style VARCHAR(255),
  section_id INT4 default '0',
  property_show SMALLINT default '0',
  control_type VARCHAR(16),
  control_style VARCHAR(255),
  control_code TEXT,
  onchange_code TEXT,
  onclick_code TEXT,
  required SMALLINT default '0',
  before_name_html TEXT,
  after_name_html TEXT,
  before_control_html TEXT,
  after_control_html TEXT,
  validation_regexp TEXT,
  regexp_error TEXT,
  options_values_sql TEXT
  ,PRIMARY KEY (property_id));

CREATE INDEX va_user_profile_properties__114 ON va_user_profile_properties (user_type_id);


DROP SEQUENCE seq_va_user_profile_sections CASCADE;
DROP TABLE va_user_profile_sections;

CREATE SEQUENCE seq_va_user_profile_sections START 5;
CREATE TABLE va_user_profile_sections (
  section_id INT4 NOT NULL DEFAULT nextval('seq_va_user_profile_sections'),
  is_active SMALLINT default '1',
  step_number INT4 default '1',
  section_order INT4 default '0',
  section_code VARCHAR(16),
  section_name VARCHAR(64),
  user_types_all SMALLINT default '1'
  ,PRIMARY KEY (section_id));



DROP SEQUENCE seq_va_user_profile_sections_types CASCADE;
DROP TABLE va_user_profile_sections_types;

CREATE SEQUENCE seq_va_user_profile_sections_types START 1;
CREATE TABLE va_user_profile_sections_types (
  section_id INT4 NOT NULL default '0',
  user_type_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (section_id,user_type_id));



DROP SEQUENCE seq_va_user_profile_values CASCADE;
DROP TABLE va_user_profile_values;

CREATE SEQUENCE seq_va_user_profile_values START 1;
CREATE TABLE va_user_profile_values (
  property_value_id INT4 NOT NULL DEFAULT nextval('seq_va_user_profile_values'),
  property_id INT4 default '0',
  property_value VARCHAR(255),
  hide_value SMALLINT default '0',
  is_default_value SMALLINT default '0'
  ,PRIMARY KEY (property_value_id));

CREATE INDEX va_user_profile_values_prop_115 ON va_user_profile_values (property_id);


DROP SEQUENCE seq_va_user_types CASCADE;
DROP TABLE va_user_types;

CREATE SEQUENCE seq_va_user_types START 2;
CREATE TABLE va_user_types (
  type_id INT4 NOT NULL DEFAULT nextval('seq_va_user_types'),
  type_name VARCHAR(128),
  is_default SMALLINT default '0',
  is_active SMALLINT default '1',
  is_sms_allowed SMALLINT default '0',
  sites_all SMALLINT default '1',
  countries_all SMALLINT default '1',
  show_for_user SMALLINT default '1',
  price_type SMALLINT default '0',
  discount_type SMALLINT,
  discount_amount FLOAT4 default '0',
  reward_type SMALLINT,
  reward_amount FLOAT4,
  credit_reward_type SMALLINT,
  credit_reward_amount FLOAT4,
  affiliate_commission_type SMALLINT,
  affiliate_commission_amount FLOAT4,
  merchant_fee_type SMALLINT,
  merchant_fee_amount FLOAT4,
  order_min_goods_cost FLOAT4,
  order_max_goods_cost FLOAT4,
  tax_free SMALLINT default '0',
  coupons_ids TEXT,
  is_subscription SMALLINT default '0',
  is_subscription_recurring INT4,
  subscription_name VARCHAR(255),
  subscription_fee FLOAT4,
  subscription_period INT4,
  subscription_interval INT4,
  subscription_suspend INT4 default '0',
  subscription_points_type SMALLINT,
  subscription_points_amount FLOAT4,
  subscription_credits_type SMALLINT,
  subscription_credits_amount FLOAT4,
  subscription_affiliate_type SMALLINT,
  subscription_affiliate_amount FLOAT4
  ,PRIMARY KEY (type_id));



DROP SEQUENCE seq_va_user_types_countries CASCADE;
DROP TABLE va_user_types_countries;

CREATE SEQUENCE seq_va_user_types_countries START 1;
CREATE TABLE va_user_types_countries (
  type_id INT4 NOT NULL default '0',
  country_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (type_id,country_id));



DROP SEQUENCE seq_va_user_types_settings CASCADE;
DROP TABLE va_user_types_settings;

CREATE SEQUENCE seq_va_user_types_settings START 1;
CREATE TABLE va_user_types_settings (
  type_id INT4 NOT NULL default '0',
  setting_name VARCHAR(64) NOT NULL,
  setting_value VARCHAR(50)
  ,PRIMARY KEY (type_id,setting_name));



DROP SEQUENCE seq_va_user_types_sites CASCADE;
DROP TABLE va_user_types_sites;

CREATE SEQUENCE seq_va_user_types_sites START 1;
CREATE TABLE va_user_types_sites (
  type_id INT4 NOT NULL default '0',
  site_id INT4 NOT NULL default '0'
  ,PRIMARY KEY (type_id,site_id));



DROP SEQUENCE seq_va_users CASCADE;
DROP TABLE va_users;

CREATE SEQUENCE seq_va_users START 2;
CREATE TABLE va_users (
  user_id INT4 NOT NULL DEFAULT nextval('seq_va_users'),
  user_type_id INT4 default '0',
  subscription_id INT4 default '0',
  layout_id INT4,
  is_approved SMALLINT default '0',
  is_hidden SMALLINT default '0',
  is_unsubscribed SMALLINT default '0',
  expiry_date TIMESTAMP,
  suspend_date TIMESTAMP,
  registration_last_step INT4 default '1',
  registration_total_steps INT4 default '1',
  is_sms_allowed SMALLINT default '0',
  credit_balance FLOAT4 default '0',
  total_points FLOAT4 default '0',
  discount_type SMALLINT,
  discount_amount FLOAT4 default '0',
  reward_type SMALLINT,
  reward_amount FLOAT4,
  credit_reward_type SMALLINT,
  credit_reward_amount FLOAT4,
  affiliate_commission_type SMALLINT,
  affiliate_commission_amount FLOAT4,
  merchant_fee_type SMALLINT,
  merchant_fee_amount FLOAT4,
  tax_free SMALLINT default '0',
  order_min_goods_cost FLOAT4,
  order_max_goods_cost FLOAT4,
  coupons_ids TEXT,
  login VARCHAR(64) NOT NULL,
  password VARCHAR(32) NOT NULL,
  nickname VARCHAR(32),
  affiliate_code VARCHAR(64),
  security_question VARCHAR(255),
  security_answer VARCHAR(255),
  personal_image VARCHAR(255),
  short_description VARCHAR(255),
  full_description TEXT,
  friendly_url VARCHAR(255),
  name VARCHAR(128) NOT NULL,
  first_name VARCHAR(64) NOT NULL,
  last_name VARCHAR(64) NOT NULL,
  company_id INT4 default '0',
  company_name VARCHAR(128),
  email VARCHAR(128) NOT NULL,
  address1 VARCHAR(255),
  address2 VARCHAR(255),
  city VARCHAR(128),
  province VARCHAR(128),
  state_id INT4 default '0',
  state_code VARCHAR(8),
  zip VARCHAR(16),
  country_id INT4 default '0',
  country_code VARCHAR(8),
  phone VARCHAR(32),
  daytime_phone VARCHAR(32),
  evening_phone VARCHAR(32),
  cell_phone VARCHAR(32),
  fax VARCHAR(32),
  delivery_name VARCHAR(128),
  delivery_first_name VARCHAR(64),
  delivery_last_name VARCHAR(64),
  delivery_company_id INT4 default '0',
  delivery_company_name VARCHAR(128),
  delivery_email VARCHAR(128),
  delivery_address1 VARCHAR(255),
  delivery_address2 VARCHAR(255),
  delivery_city VARCHAR(128),
  delivery_province VARCHAR(128),
  delivery_state_id INT4 default '0',
  delivery_state_code VARCHAR(8),
  delivery_zip VARCHAR(16),
  delivery_country_id INT4 default '0',
  delivery_country_code VARCHAR(8),
  delivery_phone VARCHAR(32),
  delivery_daytime_phone VARCHAR(32),
  delivery_evening_phone VARCHAR(32),
  delivery_cell_phone VARCHAR(32),
  delivery_fax VARCHAR(32),
  bill_encrypt_type SMALLINT default '1',
  bill_name VARCHAR(128),
  bill_first_name VARCHAR(64),
  bill_last_name VARCHAR(64),
  bill_company_id VARCHAR(32),
  bill_company_name VARCHAR(128),
  bill_email VARCHAR(128),
  bill_address1 VARCHAR(255),
  bill_address2 VARCHAR(255),
  bill_city VARCHAR(128),
  bill_province VARCHAR(128),
  bill_state_id VARCHAR(32),
  bill_zip VARCHAR(32),
  bill_country_id VARCHAR(32),
  bill_phone VARCHAR(32),
  bill_daytime_phone VARCHAR(32),
  bill_evening_phone VARCHAR(32),
  bill_cell_phone VARCHAR(32),
  bill_fax VARCHAR(32),
  bill_cc_number VARCHAR(64),
  bill_cc_start_date VARCHAR(32),
  bill_cc_expiry_date VARCHAR(32),
  bill_cc_type VARCHAR(32),
  bill_cc_issue_number VARCHAR(16),
  bill_cc_cvv2 VARCHAR(16),
  paypal_account VARCHAR(128),
  msn_account VARCHAR(128),
  icq_number VARCHAR(32),
  user_site_url VARCHAR(255),
  tax_id VARCHAR(128),
  birth_year SMALLINT,
  birth_month SMALLINT,
  birth_day SMALLINT,
  user_notes TEXT,
  admin_notes TEXT,
  registration_ip VARCHAR(32),
  registration_date TIMESTAMP,
  modified_ip VARCHAR(32),
  modified_date TIMESTAMP,
  admin_modified_ip VARCHAR(32),
  admin_modified_date TIMESTAMP,
  last_logged_ip VARCHAR(32),
  last_logged_date TIMESTAMP,
  last_visit_ip VARCHAR(32),
  last_visit_page VARCHAR(255),
  last_visit_date TIMESTAMP,
  reset_password_code VARCHAR(64),
  reset_password_date TIMESTAMP
  ,PRIMARY KEY (user_id));

CREATE INDEX va_users_affiliate_code ON va_users (affiliate_code);
CREATE INDEX va_users_company_id ON va_users (company_id);
CREATE INDEX va_users_country_id ON va_users (country_id);
CREATE INDEX va_users_delivery_company_id ON va_users (delivery_company_id);
CREATE INDEX va_users_delivery_country_id ON va_users (delivery_country_id);
CREATE INDEX va_users_delivery_state_id ON va_users (delivery_state_id);
CREATE INDEX va_users_email ON va_users (email);
CREATE INDEX va_users_first_name ON va_users (first_name);
CREATE INDEX va_users_last_name ON va_users (last_name);
CREATE INDEX va_users_last_visit_date ON va_users (last_visit_date);
CREATE INDEX va_users_login ON va_users (login);
CREATE INDEX va_users_name ON va_users (name);
CREATE INDEX va_users_nickname ON va_users (nickname);
CREATE INDEX va_users_password ON va_users (password);
CREATE INDEX va_users_reset_password_code ON va_users (reset_password_code);
CREATE INDEX va_users_state_id ON va_users (state_id);
CREATE INDEX va_users_subscription_id ON va_users (subscription_id);


DROP SEQUENCE seq_va_users_addresses CASCADE;
DROP TABLE va_users_addresses;

CREATE SEQUENCE seq_va_users_addresses START 1;
CREATE TABLE va_users_addresses (
  address_id INT4 NOT NULL DEFAULT nextval('seq_va_users_addresses'),
  user_id INT4 default '0',
  address_type SMALLINT default '0',
  name VARCHAR(128),
  first_name VARCHAR(64),
  last_name VARCHAR(64),
  company_id INT4 default '0',
  company_name VARCHAR(128),
  email VARCHAR(128),
  address1 VARCHAR(255),
  address2 VARCHAR(255),
  city VARCHAR(128),
  province VARCHAR(128),
  state_id INT4 default '0',
  state_code VARCHAR(8),
  postal_code VARCHAR(16),
  country_id INT4 default '0',
  country_code VARCHAR(8),
  phone VARCHAR(32),
  daytime_phone VARCHAR(32),
  evening_phone VARCHAR(32),
  cell_phone VARCHAR(32),
  fax VARCHAR(32)
  ,PRIMARY KEY (address_id));

CREATE INDEX va_users_addresses_user_id ON va_users_addresses (user_id);


DROP SEQUENCE seq_va_users_commissions CASCADE;
DROP TABLE va_users_commissions;

CREATE SEQUENCE seq_va_users_commissions START 1;
CREATE TABLE va_users_commissions (
  commission_id INT4 NOT NULL DEFAULT nextval('seq_va_users_commissions'),
  payment_id INT4 default '0',
  user_id INT4 default '0',
  order_id INT4 default '0',
  order_item_id INT4 default '0',
  commission_amount FLOAT4 default '0',
  commission_action INT4 default '0',
  commission_type INT4 default '0',
  date_added TIMESTAMP NOT NULL
  ,PRIMARY KEY (commission_id));

CREATE INDEX va_users_commissions_date_added ON va_users_commissions (date_added);
CREATE INDEX va_users_commissions_order_id ON va_users_commissions (order_id);
CREATE INDEX va_users_commissions_order__116 ON va_users_commissions (order_item_id);
CREATE INDEX va_users_commissions_payment_id ON va_users_commissions (payment_id);
CREATE INDEX va_users_commissions_user_id ON va_users_commissions (user_id);


DROP SEQUENCE seq_va_users_credits CASCADE;
DROP TABLE va_users_credits;

CREATE SEQUENCE seq_va_users_credits START 1;
CREATE TABLE va_users_credits (
  credit_id INT4 NOT NULL DEFAULT nextval('seq_va_users_credits'),
  user_id INT4 default '0',
  order_id INT4 default '0',
  order_item_id INT4 default '0',
  credit_amount FLOAT4 default '0',
  credit_action INT4 default '0',
  credit_type SMALLINT default '0',
  admin_id_added_by INT4 default '0',
  admin_id_modified_by INT4 default '0',
  date_added TIMESTAMP,
  date_modified TIMESTAMP
  ,PRIMARY KEY (credit_id));

CREATE INDEX va_users_credits_date_added ON va_users_credits (date_added);
CREATE INDEX va_users_credits_order_id ON va_users_credits (order_id);
CREATE INDEX va_users_credits_order_item_id ON va_users_credits (order_item_id);
CREATE INDEX va_users_credits_user_id ON va_users_credits (user_id);


DROP SEQUENCE seq_va_users_files CASCADE;
DROP TABLE va_users_files;

CREATE SEQUENCE seq_va_users_files START 1;
CREATE TABLE va_users_files (
  file_id INT4 NOT NULL DEFAULT nextval('seq_va_users_files'),
  user_id INT4 NOT NULL default '0',
  file_type VARCHAR(32),
  file_name VARCHAR(255),
  file_path VARCHAR(255)
  ,PRIMARY KEY (file_id));

CREATE INDEX va_users_files_user_id ON va_users_files (user_id);


DROP SEQUENCE seq_va_users_payments CASCADE;
DROP TABLE va_users_payments;

CREATE SEQUENCE seq_va_users_payments START 1;
CREATE TABLE va_users_payments (
  payment_id INT4 NOT NULL DEFAULT nextval('seq_va_users_payments'),
  user_id INT4 default '0',
  is_paid SMALLINT default '0',
  transaction_id VARCHAR(128),
  payment_total FLOAT4 default '0',
  payment_name VARCHAR(255),
  payment_notes TEXT,
  date_added TIMESTAMP,
  date_modified TIMESTAMP,
  date_paid TIMESTAMP,
  admin_id_added_by INT4 default '0',
  admin_id_modified_by INT4 default '0'
  ,PRIMARY KEY (payment_id));

CREATE INDEX va_users_payments_date_paid ON va_users_payments (date_paid);
CREATE INDEX va_users_payments_is_paid ON va_users_payments (is_paid);
CREATE INDEX va_users_payments_user_id ON va_users_payments (user_id);


DROP SEQUENCE seq_va_users_photos CASCADE;
DROP TABLE va_users_photos;

CREATE SEQUENCE seq_va_users_photos START 1;
CREATE TABLE va_users_photos (
  photo_id INT4 NOT NULL DEFAULT nextval('seq_va_users_photos'),
  user_id INT4 default '0',
  key_id INT4 default '0',
  photo_type SMALLINT default '1',
  is_shown SMALLINT default '1',
  is_approved SMALLINT default '1',
  photo_name VARCHAR(255),
  photo_desc TEXT,
  tiny_photo VARCHAR(255),
  small_photo VARCHAR(255),
  large_photo VARCHAR(255),
  super_photo VARCHAR(255),
  date_added TIMESTAMP,
  date_updated TIMESTAMP
  ,PRIMARY KEY (photo_id));

CREATE INDEX va_users_photos_key_id ON va_users_photos (key_id);
CREATE INDEX va_users_photos_photo_type ON va_users_photos (photo_type);
CREATE INDEX va_users_photos_user_id ON va_users_photos (user_id);


DROP SEQUENCE seq_va_users_points CASCADE;
DROP TABLE va_users_points;

CREATE SEQUENCE seq_va_users_points START 1;
CREATE TABLE va_users_points (
  points_id INT4 NOT NULL DEFAULT nextval('seq_va_users_points'),
  user_id INT4 default '0',
  order_id INT4 default '0',
  order_item_id INT4 default '0',
  points_amount FLOAT4 default '0',
  points_action INT4 default '0',
  points_type INT4 default '0',
  admin_id_added_by INT4 default '0',
  admin_id_modified_by INT4 default '0',
  date_added TIMESTAMP,
  date_modified TIMESTAMP
  ,PRIMARY KEY (points_id));

CREATE INDEX va_users_points_date_added ON va_users_points (date_added);
CREATE INDEX va_users_points_order_id ON va_users_points (order_id);
CREATE INDEX va_users_points_order_item_id ON va_users_points (order_item_id);
CREATE INDEX va_users_points_user_id ON va_users_points (user_id);


DROP SEQUENCE seq_va_users_properties CASCADE;
DROP TABLE va_users_properties;

CREATE SEQUENCE seq_va_users_properties START 1;
CREATE TABLE va_users_properties (
  user_property_id INT4 NOT NULL DEFAULT nextval('seq_va_users_properties'),
  user_id INT4 default '0',
  property_id INT4 default '0',
  property_value TEXT
  ,PRIMARY KEY (user_property_id));

CREATE INDEX va_users_properties_order_id ON va_users_properties (user_id);
CREATE INDEX va_users_properties_property_id ON va_users_properties (property_id);


DROP SEQUENCE seq_va_users_ps_details CASCADE;
DROP TABLE va_users_ps_details;

CREATE SEQUENCE seq_va_users_ps_details START 1;
CREATE TABLE va_users_ps_details (
  psd_id INT4 NOT NULL DEFAULT nextval('seq_va_users_ps_details'),
  user_id INT4 default '0',
  payment_id INT4 default '0',
  is_default SMALLINT default '0',
  is_active INT4 default '0',
  cc_encrypt_type SMALLINT default '1',
  cc_name VARCHAR(128),
  cc_first_name VARCHAR(64),
  cc_last_name VARCHAR(64),
  cc_number VARCHAR(64),
  cc_start_date TIMESTAMP,
  cc_expiry_date TIMESTAMP,
  cc_type INT4,
  cc_issue_number INT4,
  cc_security_code VARCHAR(32),
  success_payments INT4 default '0',
  failed_payments INT4 default '0',
  last_payment_date TIMESTAMP,
  admin_id_added_by INT4 default '0',
  admin_id_modified_by INT4 default '0',
  date_added TIMESTAMP,
  date_modified TIMESTAMP
  ,PRIMARY KEY (psd_id));

CREATE INDEX va_users_ps_details_payment_id ON va_users_ps_details (payment_id);
CREATE INDEX va_users_ps_details_user_id ON va_users_ps_details (user_id);


DROP SEQUENCE seq_va_users_ps_properties CASCADE;
DROP TABLE va_users_ps_properties;

CREATE SEQUENCE seq_va_users_ps_properties START 1;
CREATE TABLE va_users_ps_properties (
  user_property_id INT4 NOT NULL DEFAULT nextval('seq_va_users_ps_properties'),
  psd_id INT4 default '0',
  user_id INT4 default '0',
  property_id INT4 default '0',
  property_order INT4 default '1',
  property_type INT4 default '0',
  property_name VARCHAR(255) NOT NULL,
  property_value_id INT4,
  property_value TEXT
  ,PRIMARY KEY (user_property_id));

CREATE INDEX va_users_ps_properties_prop_117 ON va_users_ps_properties (property_id);
CREATE INDEX va_users_ps_properties_prop_118 ON va_users_ps_properties (property_name);
CREATE INDEX va_users_ps_properties_psd_id ON va_users_ps_properties (psd_id);
CREATE INDEX va_users_ps_properties_user_id ON va_users_ps_properties (user_id);


INSERT INTO va_admin_privileges (privilege_id,is_hidden,support_privilege,privilege_name,user_types_all,non_logged_users,user_types_ids) VALUES (1 , 0 , 0 , 'Administrator' , 1 , 1 , NULL ) ;
INSERT INTO va_admin_privileges (privilege_id,is_hidden,support_privilege,privilege_name,user_types_all,non_logged_users,user_types_ids) VALUES (2 , 0 , 1 , 'HelpDesk Manager' , 1 , 1 , NULL ) ;


INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'add_admins' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'add_categories' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'add_duplicate_coupons' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'add_payments' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'add_products' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'add_sites' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'add_users' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'admin_registration' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'admin_sites' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'admin_users' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'admins_groups' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'admins_hidden' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'admins_login' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'ads' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'advanced_search' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'approve_products' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'articles' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'articles_lost' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'articles_reviews' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'articles_reviews_settings' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'articles_statuses' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'banned_contents' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'banners' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'black_ips' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'categories_export' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'categories_import' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'categories_order' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'cms_settings' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'coupons' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'create_orders' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'custom_blocks' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'custom_friendly_urls' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'db_management' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'downloadable_products' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'duplicate_products' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'edit_reg_categories' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'edit_reg_list' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'edit_reg_products' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'export_payments' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'export_users' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'features_groups' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'filemanager' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'filters' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'footer_links' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'forum' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'get_orders_ids' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'get_orders_statuses' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'import_export' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'import_users' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'layouts' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'manual' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'manufacturers' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'newsletter' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'order_confirmation' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'order_links' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'order_notes' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'order_payment' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'order_profile' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'order_serials' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'order_statuses' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'order_vouchers' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'orders_import' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'orders_recover' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'orders_stats' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'payment_systems' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'polls' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'product_accessories' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'product_categories' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'product_features' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'product_images' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'product_prices' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'product_properties' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'product_related' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'product_releases' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'product_types' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'products_categories' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'products_export' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'products_export_google_base' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'products_import' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'products_order' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'products_report' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'products_reviews' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'products_reviews_settings' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'products_settings' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'remove_admins' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'remove_categories' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'remove_orders' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'remove_payments' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'remove_products' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'remove_sites' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'remove_users' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'sales_orders' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'saved_types' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'shipping_methods' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'shipping_rules' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'shipping_times' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'site_navigation' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'site_settings' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'site_users' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'sliders' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'static_google_base_attributes' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'static_google_base_types' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'static_messages' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'static_prices' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'static_tables' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'subscriptions' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'subscriptions_groups' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'suppliers' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'support' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'support_departments' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'support_predefined_reply' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'support_settings' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'support_static_data' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'support_ticket_close' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'support_ticket_edit' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'support_ticket_new' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'support_ticket_reply' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'support_users' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'support_users_priorities' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'support_users_stats' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'system_upgrade' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'tax_rates' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'tell_friend' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'update_admins' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'update_categories' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'update_order_status' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'update_orders' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'update_payments' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'update_products' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'update_sites' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'update_users' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'users_forgot' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'users_groups' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'users_login' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'users_payments' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'view_categories' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'view_products' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'visits_report' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (1 , 'web_pages' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'add_admins' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'add_categories' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'add_duplicate_coupons' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'add_payments' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'add_products' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'add_sites' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'add_users' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'admin_registration' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'admin_sites' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'admin_users' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'admins_groups' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'admins_hidden' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'admins_login' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'ads' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'advanced_search' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'approve_products' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'articles' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'articles_lost' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'articles_reviews' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'articles_reviews_settings' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'articles_statuses' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'banned_contents' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'banners' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'black_ips' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'categories_export' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'categories_import' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'categories_order' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'cms_settings' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'coupons' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'create_orders' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'custom_blocks' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'custom_friendly_urls' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'db_management' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'downloadable_products' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'duplicate_products' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'edit_reg_categories' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'edit_reg_list' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'edit_reg_products' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'export_payments' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'export_users' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'features_groups' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'filemanager' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'filters' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'footer_links' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'forum' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'get_orders_ids' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'get_orders_statuses' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'import_export' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'import_users' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'layouts' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'manual' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'manufacturers' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'newsletter' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'order_confirmation' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'order_links' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'order_notes' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'order_payment' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'order_profile' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'order_serials' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'order_statuses' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'order_vouchers' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'orders_import' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'orders_recover' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'orders_stats' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'payment_systems' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'polls' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'product_accessories' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'product_categories' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'product_features' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'product_images' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'product_prices' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'product_properties' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'product_related' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'product_releases' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'product_types' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'products_categories' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'products_export' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'products_export_google_base' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'products_import' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'products_order' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'products_report' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'products_reviews' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'products_reviews_settings' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'products_settings' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'remove_admins' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'remove_categories' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'remove_orders' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'remove_payments' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'remove_products' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'remove_sites' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'remove_users' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'sales_orders' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'saved_types' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'shipping_methods' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'shipping_rules' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'shipping_times' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'site_navigation' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'site_settings' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'site_users' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'sliders' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'static_google_base_attributes' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'static_google_base_types' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'static_messages' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'static_prices' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'static_tables' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'subscriptions' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'subscriptions_groups' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'suppliers' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'support' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'support_departments' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'support_predefined_reply' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'support_settings' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'support_static_data' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'support_ticket_close' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'support_ticket_edit' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'support_ticket_new' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'support_ticket_reply' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'support_users' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'support_users_priorities' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'support_users_stats' , 1 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'system_upgrade' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'tax_rates' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'tell_friend' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'update_admins' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'update_categories' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'update_order_status' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'update_orders' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'update_payments' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'update_products' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'update_sites' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'update_users' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'users_forgot' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'users_groups' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'users_login' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'users_payments' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'view_categories' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'view_products' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'visits_report' , 0 ) ;
INSERT INTO va_admin_privileges_settings (privilege_id,block_name,permission) VALUES (2 , 'web_pages' , 0 ) ;


INSERT INTO va_admins (admin_id,admin_name,nickname,admin_alias,privilege_id,is_hidden,email,cell_phone,login,password,personal_image,signature,last_order_id,exported_order_id,exported_order_fields,last_user_id,exported_email_id,exported_user_id,exported_email_fields,exported_user_fields,imported_email_fields,imported_user_fields,exported_item_fields,imported_item_fields,edited_item_fields,last_visit_date,support_online_date,is_generate_big_image,is_generate_small_image,login_attempts,login_failed,access_code,access_added,access_attempts,access_failed) VALUES (1 , 'Site Administrator' , 'admin' , 'SA' , 1 , 0 , 'admin@yoursite.com' , NULL , 'admin' , 'admin' , NULL , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , 'i.item_id,i.item_name,i.item_order,i.buying_price,i.features,csv_delimiter,csv_delimiter,related_delimiterrelated_delimiter' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL ) ;


INSERT INTO va_admins_login_stats (stat_id,admin_id,login_status,ip_address,forwarded_ips,date_added) VALUES (1 , 1 , 1 , '127.0.0.1' , NULL , '2014-03-26 21:06:23' ) ;
INSERT INTO va_admins_login_stats (stat_id,admin_id,login_status,ip_address,forwarded_ips,date_added) VALUES (2 , 1 , 1 , '127.0.0.1' , NULL , '2015-01-27 15:56:40' ) ;
INSERT INTO va_admins_login_stats (stat_id,admin_id,login_status,ip_address,forwarded_ips,date_added) VALUES (3 , 1 , 1 , '127.0.0.1' , NULL , '2015-02-04 14:55:57' ) ;
INSERT INTO va_admins_login_stats (stat_id,admin_id,login_status,ip_address,forwarded_ips,date_added) VALUES (4 , 1 , 1 , '127.0.0.1' , NULL , '2015-02-04 14:55:57' ) ;
INSERT INTO va_admins_login_stats (stat_id,admin_id,login_status,ip_address,forwarded_ips,date_added) VALUES (5 , 1 , 1 , '127.0.0.1' , NULL , '2015-03-11 23:27:28' ) ;
INSERT INTO va_admins_login_stats (stat_id,admin_id,login_status,ip_address,forwarded_ips,date_added) VALUES (6 , 1 , 1 , '127.0.0.1' , NULL , '2015-03-11 23:27:28' ) ;
INSERT INTO va_admins_login_stats (stat_id,admin_id,login_status,ip_address,forwarded_ips,date_added) VALUES (7 , 1 , 1 , '127.0.0.1' , NULL , '2015-03-23 23:09:46' ) ;
INSERT INTO va_admins_login_stats (stat_id,admin_id,login_status,ip_address,forwarded_ips,date_added) VALUES (8 , 1 , 1 , '127.0.0.1' , NULL , '2015-03-23 23:09:46' ) ;
INSERT INTO va_admins_login_stats (stat_id,admin_id,login_status,ip_address,forwarded_ips,date_added) VALUES (9 , 1 , 1 , '127.0.0.1' , NULL , '2015-04-20 09:09:29' ) ;
INSERT INTO va_admins_login_stats (stat_id,admin_id,login_status,ip_address,forwarded_ips,date_added) VALUES (10 , 1 , 1 , '127.0.0.1' , NULL , '2015-04-20 09:09:29' ) ;
INSERT INTO va_admins_login_stats (stat_id,admin_id,login_status,ip_address,forwarded_ips,date_added) VALUES (11 , 1 , 1 , '127.0.0.1' , NULL , '2015-04-21 18:26:52' ) ;
INSERT INTO va_admins_login_stats (stat_id,admin_id,login_status,ip_address,forwarded_ips,date_added) VALUES (12 , 1 , 1 , '127.0.0.1' , NULL , '2015-04-21 18:26:52' ) ;


INSERT INTO va_admins_settings (admin_id,setting_name,setting_value) VALUES (1 , 'recent_queries' , '["update categories set header_menu_show=1,nav_bar_show=1"]' ) ;




INSERT INTO va_ads_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,sites_all,publish_price,access_level,guest_access_level,admin_access_level,category_order,short_description,full_description,image_small,image_large,total_views,total_ads,total_subcategories) VALUES (1 , NULL , 0 , 0 , '0,' , 'Books' , NULL , 1 , NULL , 7 , 7 , 7 , 1 , NULL , NULL , 'images/categories/books.gif' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_ads_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,sites_all,publish_price,access_level,guest_access_level,admin_access_level,category_order,short_description,full_description,image_small,image_large,total_views,total_ads,total_subcategories) VALUES (2 , NULL , 0 , 0 , '0,' , 'Electronics ' , NULL , 1 , NULL , 7 , 7 , 7 , 2 , NULL , NULL , 'images/categories/electronics.gif' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_ads_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,sites_all,publish_price,access_level,guest_access_level,admin_access_level,category_order,short_description,full_description,image_small,image_large,total_views,total_ads,total_subcategories) VALUES (3 , NULL , 0 , 0 , '0,' , 'Computers' , NULL , 1 , NULL , 7 , 7 , 7 , 3 , NULL , NULL , 'images/categories/computers.gif' , NULL , NULL , 0 , 0 ) ;








INSERT INTO va_ads_days (days_id,days_number,days_title,publish_price) VALUES (1 , 1 , NULL , NULL ) ;
INSERT INTO va_ads_days (days_id,days_number,days_title,publish_price) VALUES (2 , 3 , NULL , NULL ) ;
INSERT INTO va_ads_days (days_id,days_number,days_title,publish_price) VALUES (3 , 7 , NULL , NULL ) ;
INSERT INTO va_ads_days (days_id,days_number,days_title,publish_price) VALUES (4 , 14 , NULL , NULL ) ;
INSERT INTO va_ads_days (days_id,days_number,days_title,publish_price) VALUES (5 , 30 , NULL , NULL ) ;
INSERT INTO va_ads_days (days_id,days_number,days_title,publish_price) VALUES (6 , 60 , NULL , NULL ) ;
INSERT INTO va_ads_days (days_id,days_number,days_title,publish_price) VALUES (7 , 90 , NULL , NULL ) ;
INSERT INTO va_ads_days (days_id,days_number,days_title,publish_price) VALUES (8 , 180 , NULL , NULL ) ;






INSERT INTO va_ads_features_groups (group_id,group_order,group_name) VALUES (1 , 1 , 'General' ) ;
INSERT INTO va_ads_features_groups (group_id,group_order,group_name) VALUES (2 , 2 , 'Sizes' ) ;
INSERT INTO va_ads_features_groups (group_id,group_order,group_name) VALUES (3 , 3 , 'Accessories' ) ;


INSERT INTO va_ads_hot_days (days_id,days_number,days_title,publish_price) VALUES (1 , 1 , NULL , NULL ) ;
INSERT INTO va_ads_hot_days (days_id,days_number,days_title,publish_price) VALUES (2 , 3 , NULL , NULL ) ;
INSERT INTO va_ads_hot_days (days_id,days_number,days_title,publish_price) VALUES (3 , 7 , NULL , NULL ) ;
INSERT INTO va_ads_hot_days (days_id,days_number,days_title,publish_price) VALUES (4 , 14 , NULL , NULL ) ;
INSERT INTO va_ads_hot_days (days_id,days_number,days_title,publish_price) VALUES (5 , 30 , NULL , NULL ) ;










INSERT INTO va_ads_special_days (days_id,days_number,days_title,publish_price) VALUES (1 , 1 , NULL , NULL ) ;
INSERT INTO va_ads_special_days (days_id,days_number,days_title,publish_price) VALUES (2 , 3 , NULL , NULL ) ;
INSERT INTO va_ads_special_days (days_id,days_number,days_title,publish_price) VALUES (3 , 7 , NULL , NULL ) ;
INSERT INTO va_ads_special_days (days_id,days_number,days_title,publish_price) VALUES (4 , 14 , NULL , NULL ) ;
INSERT INTO va_ads_special_days (days_id,days_number,days_title,publish_price) VALUES (5 , 30 , NULL , NULL ) ;


INSERT INTO va_ads_types (type_id,type_name) VALUES (1 , 'AD_PRODUCT_TYPE_MSG' ) ;
INSERT INTO va_ads_types (type_id,type_name) VALUES (2 , 'AD_ACCESSORY_TYPE_MSG' ) ;








INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (1 , NULL , '2008-09-11 15:06:00' , NULL , '\'Mythical\' animal finally poses for camera' , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , '10.1.1.6' , NULL , 1 , NULL , 1 , 1 , 0 , 1 , 0 , NULL , 'Rare images of an African animal so elusive that it was once believed to be mythical have been released.' , NULL , NULL , NULL , NULL , 0 , NULL , NULL , '2008-09-12 14:29:25' , NULL , NULL , 'Rare images of an African animal so elusive that it was once believed to be mythical have been released.\r\n<br><br>\r\nThe okapi is a shy animal related to the giraffe, with zebra-like stripes on its rear. It is native to the tropical rainforests of Africa but is now known to live only in the Democratic Republic of the Congo.' , 'Rare images of an African animal so elusive that it was once believed to be mythical have been released.\r\n\r\nThe okapi is a shy animal related to the giraffe, with zebra-like stripes on its rear. It is native to the tropical rainforests of Africa but is now known to live only in the Democratic Republic of the Congo.\r\n\r\nOkapis were last seen in the DRC 50 years ago, and the images are the first to capture the animal in the wild.\r\n\r\n"To have captured the first-ever photographs of such a charismatic creature is amazing," said Noelle Kumpel, manager of the Bushmeat and Forests Conservation Program at the Zoological Society of London, which took the pictures. \r\n\r\n"Okapi are very shy and rare animals, which is why conventional surveys only tend to record droppings and other signs of their presence."\r\n\r\nThe okapi\'s face and long legs resemble those of the giraffe, their closest-living relative, but they look more like horses with long necks. The average height of their shoulders is 1.6 meters (5 feet, 3 inches), the ZSL says.\r\n\r\nThey have a short, dense, velvety coat and dark prehensile tongue long enough to clean their own eyelids and ears.\r\n\r\nThe new pictures show the okapi in the foliage of the DRC\'s Virunga National Park, where decades of economic collapse and armed conflict have threatened the diverse wildlife.\r\n\r\nIn one image, an okapi looks curiously toward the camera amid the green trees and groundcover. Its large ears, dark nose, and zebra stripes are clearly visible.\r\n\r\nThe next shot shows the okapi walking away from the camera, giving a clear view of its striking black-and-white stripes on its rear legs.\r\n\r\nA third image released by ZSL was taken at night and gives a vivid side view of the animal.\r\n\r\n"The photographs clearly show the stripes on their rear, which act like unique fingerprints," said Theirry Lusenge, a key member of ZSL\'s survey team in the DRC. "We have already identified three individuals, and further survey work will enable us to estimate population numbers and distribution in and around the park, which is a critical first step in targeting conservation efforts."\r\n\r\nZSL said the images prove the okapi still thrives in the park despite threats to its survival, which include poaching, deforestation, military camps inside the park, and the influx of refugees at the park\'s borders.\r\n\r\nThe animal\'s exact status is still a mystery, however, and it remains under threat, the ZSL said. Okapi meat reportedly poached from the park is now regularly on sale at the nearby town of Beni -- and if hunting continues at the same rate, okapi could become extinct within the park within a few years, the ZSL said.\r\n\r\nThe Virunga National Park is one of only three protected areas the okapi are known to inhabit, the ZSL said.\r\n\r\nThe ZSL\'s study of the okapi is part of a larger EU-funded conservation project in the park. Though focused on okapi, the study has also managed to find other species including shrews and duikers, a kind of antelope, a ZSL spokeswoman said.\r\n\r\nThe study involved 18 cameras set up around the park by the ZSL and the Congolese Institute for Nature Conservation, the spokeswoman said.\r\n\r\nThe announcement comes a month after another group of researchers reported finding thousands of previously-unknown gorillas in the neighboring Republic of Congo.\r\n\r\nThe U.S.-based Wildlife Conservation Society said in August they had found 125,000 Western lowland gorillas living in a swamp in northern Congo, adding significantly to the existing worldwide population of the threatened species.\r\n\r\nOkapi were well known by Africans for centuries but the animal remained elusive, thanks to its acute hearing and effective camouflage, according to The National Zoological Gardens of South Africa.\r\n\r\nNineteenth-century explorers sometimes caught a fleeting glimpse of the okapi\'s striped backside as it ran through the forest, leading to speculation that it was a kind of zebra, the National Zoological Gardens says. It was known to the pygmy as a sacred animal.\r\n\r\nStill, the okapi remained unknown to the Western world until the early 20th century, when British explorer Harry Johnston went to Congo and sent a complete skin and skull back to Britain, the ZSL said' , NULL , 1 , 1 , 4 , 4 , 0 , NULL , NULL , 'images/articles/small/news4.jpg' , NULL , 'images/articles/large/news4.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (2 , NULL , '2008-09-11 15:10:00' , NULL , 'Thousands stranded by Channel Tunnel fire' , NULL , NULL , NULL , NULL , 2 , NULL , NULL , NULL , '10.1.1.6' , NULL , 1 , NULL , 1 , 1 , 0 , 1 , 0 , NULL , 'Authorities on Thursday halted all traffic running through the 50-kilometer (30-mile) Chunnel after a fire broke out inside the tunnel, which runs between Britain and France underneath the English Channel, Eurotunnel and fire officials said.\r\n' , NULL , NULL , NULL , NULL , 0 , NULL , NULL , '2008-09-12 14:26:54' , NULL , NULL , 'Authorities on Thursday halted all traffic running through the 50-kilometer (30-mile) Chunnel after a fire broke out inside the tunnel, which runs between Britain and France underneath the English Channel, Eurotunnel and fire officials said.\r\n<br><br>\r\nThe fire started on a shuttle train carrying freight as it approached France, Eurotunnel said. No passenger trains were going through the tunnel at the time of the fire.' , 'Authorities on Thursday halted all traffic running through the 50-kilometer (30-mile) Chunnel after a fire broke out inside the tunnel, which runs between Britain and France underneath the English Channel, Eurotunnel and fire officials said.\r\n\r\nThe fire started on a shuttle train carrying freight as it approached France, Eurotunnel said. No passenger trains were going through the tunnel at the time of the fire.\r\n\r\nThe fire continued to burn nearly four hours later, according to Georges Bos, the prefecture of France\'s Pas de Calais region.\r\n\r\nFrench rail authorities said the tunnel will be closed at least until Friday morning.\r\n\r\nThe French Interior Ministry said some people were injured in the incident, but did not specify how many or the nature of their injuries. Bos told BFM-TV that six people suffered from smoke inhalation.\r\n\r\nThe fire broke out around 2:55 p.m. (9:55 a.m. ET) -- reportedly on one of the trucks being shuttled by the train -- about 11 kilometers (7 miles) from France, officials said. \r\n\r\nThirty-two people were evacuated through a central service tunnel that runs between the east and west tubes, Kent fire officials said. \r\n\r\nEurotunnel authorities suspended services in both directions to allow emergency vehicles to get to the fire. The tunnel closure will affect thousands of travelers, according to Simon Montague, a spokesman for the Eurostar train system. \r\n\r\n"Probably at the order of 2,000 who were actually in transit, and several more thousands who would\'ve been hoping to travel with us tonight," he said. "Obviously our highest priority tonight is to help those travelers who have not been able to complete their journeys."\r\n\r\nThe tunnel runs through England\'s Kent County and police there may have to implement "Operation Stack" -- in which trucks that are waiting to be shuttled through the tunnel would be lined up on the M20 highway. People who were planning on driving along that route are being asked to "delay or vary" their travel, according to the Kent police Web site.\r\n\r\nEurostar issued a statement Thursday night saying that service will be suspended Friday, and passengers holding tickets for Friday "are advised not to travel." Passengers can exchange their tickets for travel on a later date or get a refund. \r\n\r\nThose holding tickets for Saturday or Sunday travel should check Eurostar\'s Web site for updates, the company said.\r\n\r\nThe $15 billion undersea tunnel opened in 1994 and has drastically reduced land/sea travel time between Great Britain and mainland Europe.\r\n\r\nA massive fire broke out inside the tunnel in November 1996, causing structural damage but no serious injuries. A truck on a Eurotunnel lorry shuttle caught fire before the train entered the tunnel, and the fire spread to nearby cars.\r\n\r\nThat fire was caused by the vehicle\'s traction system and had "nothing to do with the cargo or the people," according to Colin Brown, director of engineering at the Institute of Mechanical Engineers.\r\n\r\n"It will be very interesting if we have the same issue, I doubt that we will," Brown told CNN. "My expectation is this will be a different issue."' , NULL , 2 , 1 , 5 , 5 , 0 , NULL , NULL , 'images/articles/small/news3.jpg' , NULL , 'images/articles/large/news3.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (3 , NULL , '2008-09-12 15:12:00' , NULL , 'Zimbabwe leaders said to agree to power-sharing deal' , NULL , NULL , NULL , NULL , 3 , NULL , NULL , NULL , '10.1.1.6' , NULL , 1 , NULL , 1 , 1 , 0 , 1 , 0 , NULL , 'A power-sharing deal has been reached between Zimbabwean President Robert Mugabe and the opposition Movement for Democratic Change, South African President Thabo Mbeki said.\r\n' , NULL , NULL , NULL , NULL , 0 , NULL , NULL , '2008-09-12 14:23:11' , NULL , NULL , 'A power-sharing deal has been reached between Zimbabwean President Robert Mugabe and the opposition Movement for Democratic Change, South African President Thabo Mbeki said.\r\n<br><br>\r\nMbeki, who mediated the talks in Harare for the Southern African Development Community, said the deal would be signed Monday but did not give details of the agreement.' , 'A power-sharing deal has been reached between Zimbabwean President Robert Mugabe and the opposition Movement for Democratic Change, South African President Thabo Mbeki said.\r\n\r\nMbeki, who mediated the talks in Harare for the Southern African Development Community, said the deal would be signed Monday but did not give details of the agreement.\r\n\r\nZimbabwe has had no Cabinet since the March presidential election that started the impasse.\r\n\r\nOpposition lawmakers booed and heckled Mugabe when he spoke at the opening of the country\'s parliament August 26.\r\n\r\nMDC leader Morgan Tsvangirai won the most votes in that election, but not enough to avoid a runoff, according to the government\'s official count.\r\n\r\nHe withdrew from the June 27 runoff days before the vote, saying Mugabe\'s supporters had waged a campaign of violence and intimidation against opposition supporters.\r\n\r\nThe main sticking point in the talks had been how much power Mugabe would retain.\r\n\r\nTsvangirai had said he would sign a deal only if Mugabe gave up some power and his presidency became a ceremonial position.\r\n\r\nUnited Nations Secretary-General Ban Ki-moon welcomed the agreement, according to a statement released by his spokesperson, and "hopes that this agreement will pave the way for a durable peace and recovery in the country and contribute to rapid improvement in the welfare and human rights of the people of Zimbabwe." \r\n\r\nThe country is also reeling from hyper-inflation. In August, the country\'s Central Statistical Office said inflation was at 11.2 million percent, the highest in the world.\r\n\r\nAnalysts have said the Zimbabwean government\'s official inflation rate figures are conservative.\r\n\r\nOne of Zimbabwe\'s leading banks, Kingdom Bank, said the country\'s inflation rate was more than 20 million percent.\r\n\r\nThe economic crisis has destroyed Zimbabwe\'s currency and made it difficult for Zimbabweans to buy basic commodities, electricity, fuel and medicines.' , NULL , 2 , 0 , 0 , 0 , 0 , NULL , NULL , 'images/articles/small/news2.jpg' , NULL , 'images/articles/large/news2.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (4 , NULL , '2008-09-12 15:20:00' , NULL , 'Russian bombers in Venezuela amid tension with U.S.' , NULL , NULL , NULL , NULL , 4 , NULL , NULL , NULL , '10.1.1.6' , NULL , 1 , NULL , 1 , 1 , 0 , 1 , 1 , NULL , 'Two Russian bombers have landed at a Venezuelan airfield where they will carry out training flights for several days, the Russian defense ministry said Wednesday.' , NULL , NULL , NULL , NULL , 0 , NULL , NULL , '2008-09-12 14:14:39' , NULL , NULL , 'Two Russian bombers have landed at a Venezuelan airfield where they will carry out training flights for several days, the Russian defense ministry said Wednesday.\r\n<br><br>\r\nThe Tupolev Tu-160 strategic bombers landed at Venezuela\'s Libertador military airfield and "will spend several days carrying out training flights over neutral waters, after which they will return to the base," the ministry added.\r\n\r\n' , 'Two Russian bombers have landed at a Venezuelan airfield where they will carry out training flights for several days, the Russian defense ministry said Wednesday.\r\n\r\nThe Tupolev Tu-160 strategic bombers landed at Venezuela\'s Libertador military airfield and "will spend several days carrying out training flights over neutral waters, after which they will return to the base," the ministry added.\r\n\r\nNATO fighters followed the bombers on their 13-hour flight over the Arctic Ocean and the Atlantic, the defense ministry said.\r\n\r\nIt said the Russian flights were carried out in strict accordance with international rules governing airspace above neutral waters, and that the aircraft did not violate the borders of other states.\r\n\r\nPentagon spokesman Bryan Whitman said such joint exercises between nations are not unusual. "We exercise all around the globe and have joint exercises with countries all over the world. So do many other nations."\r\n\r\nThe U.S. will monitor the Russian-Venezuelan training, said Pentagon officials who asked not to be identified as they are not authorized to speak on the information.\r\n\r\nOn Monday, Russia announced it might hold joint naval maneuvers with Venezuela in the Caribbean.\r\n\r\nThe declaration came amid increased tension between Russia and the United States over Russia\'s invasion last month of the former Soviet republic of Georgia, a U.S. ally that aspires to join NATO. \r\n\r\nOn Monday Russia denied any link between that announcement and the conflict in Georgia, although Russia has criticized U.S. support for Georgia.\r\n\r\nRussia has also objected to the missile defense system agreement, signed between the United States and Poland and the Czech Republic in August, that places a ground-based ballistic defense facility in the two eastern European nations.\r\n\r\nRussia has said the deal threatens its security, while the United States has said that the system is to guard against rogue states such as Iran.\r\n\r\nEarlier this month, Venezuelan President Hugo Chavez, whose comments have frequently antagonized Washington, said it would welcome the Russian air force, according to Russian news agency Novosti.\r\n\r\n"If Russian long-range bombers should need to land in Venezuela, we would not object to that either. We will also welcome them," Chavez said on September 1, according to Novosti. \r\n' , NULL , 1 , 1 , 5 , 5 , 0 , NULL , NULL , 'images/articles/small/news1.jpg' , NULL , 'images/articles/large/news1.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (5 , NULL , '2005-06-10 15:25:00' , NULL , 'Image 1' , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , '10.1.1.6' , NULL , 1 , NULL , 1 , 1 , 0 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , '07.09.2005 11:46:56' , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , 'images/big/story1.jpg' , NULL , 'images/big/story1.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (6 , NULL , '2005-06-10 15:26:00' , NULL , 'Image 2' , NULL , NULL , NULL , NULL , 2 , NULL , NULL , NULL , '10.1.1.6' , NULL , 1 , NULL , 1 , 1 , 0 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , '07.09.2005 11:47:13' , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , 'images/big/story2.jpg' , NULL , 'images/big/story2.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (7 , NULL , '2005-06-10 15:27:00' , NULL , 'Image 3' , NULL , NULL , NULL , NULL , 3 , NULL , NULL , NULL , '10.1.1.6' , NULL , 1 , NULL , 1 , 1 , 0 , 1 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , '10.06.2005 15:27:42' , '10.06.2005 15:27:42' , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , 'images/big/story3.jpg' , NULL , 'images/big/story3.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (8 , NULL , '2005-06-10 15:27:00' , NULL , 'Image 4' , NULL , NULL , NULL , NULL , 4 , NULL , NULL , NULL , '10.1.1.6' , NULL , 1 , NULL , 1 , 1 , 0 , 1 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , '10.06.2005 15:28:04' , '10.06.2005 15:28:04' , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , 'images/big/story4.jpg' , NULL , 'images/big/story4.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (9 , '' , NULL , NULL , 'Adele - Hello' , 'A' , NULL , '' , NULL , 1 , NULL , NULL , NULL , '10.1.1.6' , NULL , 1 , NULL , 1 , 1 , 1 , 1 , 0 , NULL , NULL , 'Lyrics Online' , 'http://www.minelyrics.com/' , NULL , NULL , 0 , NULL , NULL , '24.11.2015 1:54:43' , NULL , NULL , NULL , '<p>\r\n\r\n<span data-start="80" data-stop="85">\r\nHello, it\'s me<br>\r\n</span>\r\n<span data-start="86">\r\nI was wondering if after all these years<br>\r\nYou\'d like to meet, to go over everything<br>\r\n</span>\r\n<span data-start="97">\r\nThey say that time\'s supposed to heal ya<br>\r\nBut I ain\'t done much healing<br>\r\n</span>\r\n<span data-start="105" data-stop="110">\r\nHello, can you hear me?<br>\r\n</span>\r\n<span data-start="110" data-stop="122">\r\nI\'m in California dreaming about who we used to be<br>\r\nWhen we were younger and free<br>\r\n</span>\r\n<span data-start="122">\r\nI\'ve forgotten how it felt before the world fell at our feet<br>\r\nThere\'s such a difference between us<br>\r\n</span>\r\n<span data-start="134" data-stop="139">\r\nAnd a million miles<br>\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="141">\r\nHello from the other side<br>\r\nI must\'ve called a thousand times<br>\r\n</span>\r\n<span data-start="152">\r\nTo tell you I\'m sorry, for everything that I\'ve done<br>\r\nBut when I call you never seem to be home<br>\r\n</span>\r\n<span data-start="165">\r\nHello from the outside<br>\r\nAt least I can say that I\'ve tried<br>\r\n</span>\r\n<span data-start="177" data-stop="191">\r\nTo tell you I\'m sorry, for breaking your heart<br>\r\nBut it don\'t matter, it clearly doesn\'t tear you apart anymore<br>\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="195" data-stop="210">\r\nHello, how are you?<br>\r\n</span>\r\n<span data-start="201" data-stop="212">\r\nIt\'s so typical of me to talk about myself, I\'m sorry<br>\r\nI hope that you\'re well<br>\r\n</span>\r\n<span data-start="214">\r\nDid you ever make it out of that town<br>\r\nWhere nothing ever happened?<br>\r\n</span>\r\n<span data-start="220" data-stop="231">\r\nIt\'s no secret<br>\r\nThat the both of us are running out of time<br>\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="232">\r\nSo hello from the other side<br>\r\nI must\'ve called a thousand times<br>\r\n</span>\r\n<span data-start="243">\r\nTo tell you I\'m sorry, for everything that I\'ve done<br>\r\nBut when I call you never seem to be home<br>\r\n</span>\r\n<span data-start="257">\r\nHello from the outside<br>\r\nAt least I can say that I\'ve tried<br>\r\n</span>\r\n<span data-start="268" data-stop="282">\r\nTo tell you I\'m sorry, for breaking your heart<br>\r\nBut it don\'t matter, it clearly doesn\'t tear you apart anymore<br>\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="284" data-stop="294">\r\nOoooohh, anymore<br>\r\nOoooohh, anymore<br>\r\n</span>\r\n<span data-start="296" data-stop="304">\r\nOoooohh, anymore, anymore<br>\r\n</span>\r\n</p>\r\n\r\n\r\n<span data-start="305">\r\nHello from the other side<br>\r\nI must\'ve called a thousand times<br>\r\n</span>\r\n<span data-start="316">\r\nTo tell you I\'m sorry, for everything that I\'ve done<br>\r\nBut when I call you never seem to be home<br>\r\n</span>\r\n<span data-start="330">\r\nHello from the outside<br>\r\nAt least I can say that I\'ve tried<br>\r\n</span>\r\n<span data-start="341" data-stop="355">\r\nTo tell you I\'m sorry, for breaking your heart<br>\r\nBut it don\'t matter, it clearly doesn\'t tear you apart anymore<br><br><span><a href="http://www.minelyrics.com/adele-hello.html">Adele - Hello Lyrics</a></span><br></span>' , NULL , 1 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (10 , '' , NULL , NULL , 'Avicii - For A Better Day' , 'A' , NULL , '' , NULL , 2 , NULL , NULL , NULL , '10.1.1.6' , NULL , 1 , NULL , 1 , 1 , 1 , 1 , 0 , NULL , NULL , 'Song Texts' , 'http://www.minelyrics.com/' , NULL , NULL , 0 , NULL , '10.06.2005 15:56:05' , '24.11.2015 1:58:07' , NULL , NULL , NULL , '<div class="article-desc">\r\n\t\t\t\t\t\t<div id="youtube-lyrics" data-video-id="Xq-knHXSKYY">\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t<div class="desc desc-full"><p><span data-start="9">\r\nPale on a caution wind<br>\r\nMeet the bleeding sky<br>\r\n</span>\r\n<span data-start="19">\r\nI called your name<br>\r\nThere was no one there<br>\r\n</span>\r\n<span data-start="26">\r\nAnd in cold and snow<br>\r\nI saw your face\r\n</span>\r\n</p>\r\n\r\n\r\n<p>\r\n<span data-start="33">\r\nAnd we sang the song for the little things<br>\r\nMagic call, but the joy you bring<br>\r\n</span>\r\n<span data-start="41">\r\nRunning it down the line<br>\r\nWish you could find that love is a fragile thing<br>\r\n</span>\r\n<span data-start="49">\r\nMagic call from a pretty thing<br>\r\nMaybe it might be time<br>\r\n</span>\r\n<span data-start="58" data-stop="76">\r\nFor a better day, for a better day<br>\r\nFor a better day\r\n</span>\r\n</p>\r\n\r\n\r\n<p>\r\n<span data-start="88">\r\nStraight, from the path of love<br>\r\nIn the road of life<br>\r\n</span>\r\n<span data-start="97">\r\nI tumble forward<br>\r\nBut going on, I\'mma keep it strong<br>\r\n</span>\r\n<span data-start="108">\r\nI saw your face\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="111">\r\nAnd we sang the song for the little thing<br>\r\nMagic call, but the joy you bring<br>\r\n</span>\r\n<span data-start="120">\r\nRunning it down the line<br>\r\nWish you could find that love is a fragile thing<br>\r\n</span>\r\n<span data-start="127">\r\nMagic call from a pretty thing<br>\r\nMaybe it might be time<br>\r\n</span>\r\n<span data-start="133" data-stop="164">\r\nFor a better day, for a better day, for a better day<br>\r\nFor a better day, for a better day, for a better day\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="166">\r\nAnd we sang the song for the little thing<br>\r\nMagic call, but the joy you bring<br>\r\n</span>\r\n<span data-start="173">\r\nRunning it down the line<br>\r\nWish you could find that love is a fragile thing<br>\r\n</span>\r\n<span data-start="180">\r\nMagic come from a pretty thing<br>\r\nMaybe it might be time<br>\r\n</span>\r\n<span data-start="187" data-stop="196">\r\nFor a better day... <br></span></p><p><br><span><a href="http://www.minelyrics.com/avicii-for-a-better-day.html">Avicii - For A Better Day Lyrics</a></span><span data-start="187" data-stop="196"></span></p></div>\r\n\t\t\t\t\t\t</div>\r\n\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\r\n\r\n\r\n\t\t\t\t\t</div>' , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (11 , '' , NULL , NULL , 'Beyonce - Halo' , 'B' , NULL , '' , NULL , 3 , NULL , NULL , NULL , '10.1.1.6' , NULL , 1 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , 'English Lyrics ' , 'http://www.minelyrics.com/' , NULL , NULL , 0 , NULL , NULL , '24.11.2015 2:01:54' , NULL , NULL , NULL , '<p>\r\n<span data-start="24">\r\nRemember those walls I built?<br>\r\nOh, baby they\'re tumbling down<br>\r\n</span>\r\n<span data-start="30">\r\nAnd they didn\'t even put up a fight<br>\r\nThey didn\'t even make a sound<br>\r\n</span>\r\n<span data-start="36">\r\nI found a way to let you in<br>\r\nBut, I never really had a doubt<br>\r\n</span>\r\n<span data-start="42">\r\nStanding in the light of your halo<br>\r\nI got my angel now\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="49">\r\nIt\'s like I\'ve been awakened<br>\r\nEvery rule I had you breaking<br>\r\n</span>\r\n<span data-start="54">\r\nIt\'s the risk that I\'m taking<br>\r\nI ain\'t never gonna shut you out!\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="61">\r\nEverywhere I\'m looking now<br>\r\nI\'m surrounded by your embrace<br>\r\n</span>\r\n<span data-start="66">\r\nBaby, I can see your halo<br>\r\nYou know you\'re my saving grace<br>\r\n</span>\r\n<span data-start="72">\r\nYou\'re everything I need and more<br>\r\nIt\'s written all over your face<br>\r\n</span>\r\n<span data-start="79">\r\nBaby, I can feel your halo<br>\r\nPray it won\'t fade away\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="83">\r\nI can feel your halo, halo, halo<br>\r\nI can see your halo, halo, halo<br>\r\n</span>\r\n<span data-start="88">\r\nI can feel your halo, halo, halo<br>\r\nI can see your halo, halo, halo, ooh ooh...<br>\r\n</span>\r\n\r\n</p>\r\n\r\n<p>\r\n<span data-start="96">\r\nHit me like a ray of sun<br>\r\nBurning through my darkest night<br>\r\n</span>\r\n<span data-start="102">\r\nYou\'re the only one that I want<br>\r\nThink I\'m addicted to your light<br>\r\n</span>\r\n<span data-start="108">\r\nI swore I\'d never fall again<br>\r\nBut this don\'t even feel like falling<br>\r\n</span>\r\n<span data-start="115">\r\nGravity can\'t forget to pull me back to the ground again\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="120">\r\nAnd it\'s like I\'ve been awakened<br>\r\nEvery rule I had you breaking<br>\r\n</span>\r\n<span data-start="126">\r\nIt\'s the risk that I\'m taking<br>\r\nI\'m never gonna shut you out!\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="133">\r\nEverywhere I\'m looking now<br>\r\nI\'m surrounded by your embrace<br>\r\n</span>\r\n<span data-start="138">\r\nBaby, I can feel your halo<br>\r\nPray it won\'t fade away\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="144">\r\nI can feel your halo, halo, halo<br>\r\nI can see your halo, halo, halo<br>\r\n</span>\r\n<span data-start="149" data-stop="157">\r\nI can feel your halo, halo, halo<br>\r\nI can see your halo, halo, halo, ooh ooh...<br>\r\n</span>\r\n\r\n<span data-start="160" data-stop="180">\r\nHalo, ooh ooh...<br>\r\nHalo, ooh ooh, ooh, ooh, ooh, oh...\r\n</span></p>\r\n\r\n<p>\r\n<span data-start="183">\r\nEverywhere I\'m looking now<br>\r\nI\'m surrounded by your embrace<br>\r\n</span>\r\n<span data-start="189">\r\nBaby, I can see your halo<br>\r\nYou know you\'re my saving grace<br>\r\n</span>\r\n<span data-start="195">\r\nYou\'re everything I need and more<br>\r\nIt\'s written all over your face<br>\r\n</span>\r\n<span data-start="201">\r\nBaby, I can feel your halo<br>\r\nPray it won\'t fade away\r\n</span>\r\n</p>\r\n\r\n\r\n<span data-start="206">\r\nI can feel your halo, halo, halo<br>\r\nI can see your halo, halo, halo<br>\r\n</span>\r\n<span data-start="213">\r\nI can feel your halo, halo, halo<br>\r\nI can see your halo, halo, halo, ooh oh...<br><br><span><a href="http://www.minelyrics.com/beyonce-halo.html">Beyonce - Halo Lyrics</a></span><br></span>' , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (12 , '' , NULL , NULL , 'Coolio - Gangsta\'s Paradise' , 'C' , NULL , '' , NULL , 4 , NULL , NULL , NULL , '10.1.1.6' , NULL , 1 , NULL , 1 , 1 , 1 , 1 , 0 , NULL , NULL , 'Rap Lyrics' , 'http://www.minelyrics.com/' , NULL , NULL , 0 , NULL , '10.06.2005 15:56:48' , '24.11.2015 2:04:20' , NULL , NULL , NULL , '<p>\r\n<span data-start="11">\r\nAs I walk through the valley of the shadow of death<br>\r\nI take a look at my life and realize there\'s nothing left<br>\r\n</span>\r\n<span data-start="17.5">\r\n\'Cause I\'ve been blasting and laughing so long,<br>\r\nThat even my mama thinks that my mind is gone<br>\r\n</span>\r\n<span data-start="23">\r\nBut I ain\'t never crossed a man that didn\'t deserve it<br>\r\nMe be treated like a punk you know that\'s unheard of<br>\r\n</span>\r\n<span data-start="29">\r\nYou better watch how you\'re talking and where you\'re walking<br>\r\nOr you and your homies might be lined in chalk<br>\r\n</span>\r\n<span data-start="35">\r\nI really hate to trip but I gotta loc<br>\r\nAs they croak, I see myself in the pistol smoke, fool<br>\r\n</span>\r\n<span data-start="42">\r\nI\'m the kinda G the little homies wanna be like<br>\r\nOn my knees in the night saying prayers in the streetlight\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="48">\r\nBeen spending most their lives, living in the gangsta\'s paradise<br>\r\nBeen spending most their lives, living in the gangsta\'s paradise<br>\r\n</span>\r\n<span data-start="59">\r\nKeep spending most our lives, living in the gangsta\'s paradise<br>\r\nKeep spending most our lives, living in the gangsta\'s paradise\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="71">\r\nLook at the situation they got me facing<br>\r\nI can\'t live a normal life, I was raised by the streets<br>\r\n</span>\r\n<span data-start="77.5">\r\nSo I gotta be down with the hood team<br>\r\nToo much television watching got me chasing dreams<br>\r\n</span>\r\n<span data-start="83">\r\nI\'m an educated fool with money on my mind<br>\r\nGot my ten in my hand and a gleam in my eye<br>\r\n</span>\r\n<span data-start="89">\r\nI\'m a loc\'d out gangsta set tripping banger<br>\r\nAnd my homies is down so don\'t arouse my anger, fool<br>\r\n</span>\r\n<span data-start="95.5">\r\nDeath ain\'t nothing but a heartbeat away,<br>\r\nI\'m living life, do or die, what can I say<br>\r\n</span>\r\n<span data-start="101">\r\nI\'m twenty three now, but will I live to see twenty four<br>\r\nThe way things are going I don\'t know\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="107">\r\nTell me why are we so blind to see<br>\r\nThat the ones we hurt are you and me\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="119">\r\nBeen spending most their lives, living in the gangsta\'s paradise<br>\r\nBeen spending most their lives, living in the gangsta\'s paradise<br>\r\n</span>\r\n<span data-start="131">\r\nKeep spending most our lives, living in the gangsta\'s paradise<br>\r\nKeep spending most our lives, living in the gangsta\'s paradise\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="143">\r\nPower and the money, money and the power<br>\r\nMinute after minute, hour after hour<br>\r\n</span>\r\n<span data-start="149">\r\nEverybody\'s running, but half of them ain\'t looking<br>\r\nWhat\'s going on in the kitchen, but I don\'t know what\'s cooking<br>\r\n</span>\r\n<span data-start="155.5">\r\nThey say I gotta learn, but nobody\'s here to teach me<br>\r\nIf they can\'t understand it, how can they reach me<br>\r\n</span>\r\n<span data-start="162">\r\nI guess they can\'t, I guess they won\'t<br>\r\nI guess they front, that\'s why I know my life is out of luck, fool\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="168">\r\nBeen spending most their lives, living in the gangsta\'s paradise<br>\r\nBeen spending most their lives, living in the gangsta\'s paradise<br>\r\n</span>\r\n<span data-start="179">\r\nKeep spending most our lives, living in the gangsta\'s paradise<br>\r\nKeep spending most our lives, living in the gangsta\'s paradise\r\n</span>\r\n</p>\r\n\r\n\r\n<span data-start="190">\r\nTell me why are we so blind to see<br>\r\nThat the ones we hurt are you and me<br>\r\n</span>\r\n<span data-start="202" data-stop="215">\r\nTell me why are we so blind to see<br>\r\nThat the ones we hurt are you and me<br><br><span><a href="http://www.minelyrics.com/coolio-gangstas-paradise.html">Coolio - Gangsta\'s Paradise Lyrics</a></span><br></span>' , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (13 , '' , NULL , NULL , 'Eagles - Hotel California' , 'E' , NULL , '' , NULL , 1 , NULL , NULL , NULL , '10.1.1.6' , NULL , 1 , NULL , 1 , 1 , 1 , 1 , 0 , NULL , '<br>' , 'Online Lyrics' , 'http://www.minelyrics.com/' , NULL , NULL , 0 , NULL , '10.06.2005 15:57:49' , '24.11.2015 1:49:52' , NULL , NULL , NULL , '<div class="article-desc">\r\n\t\t\t\t\t\t<div id="youtube-lyrics" data-video-id="lrfhf1Gv4Tw">\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t<div class="desc desc-full"><p>\r\n<span data-start="90">\r\nOn a dark desert highway, cool wind in my hair<br>\r\nWarm smell of colitas, rising up through the air<br>\r\n</span>\r\n<span data-start="103">\r\nUp ahead in the distance, I saw a shimmering light<br>\r\nMy head grew heavy and my sight grew dim<br>\r\n</span>\r\n<span data-start="113">\r\nI had to stop for the night<br>\r\nThere she stood in the doorway;<br>\r\n</span>\r\n<span data-start="119">\r\nI heard the mission bell<br>\r\nAnd I was thinking to myself,<br>\r\n</span>\r\n<span data-start="124">\r\n"This could be Heaven or this could be Hell"<br>\r\nThen she lit up a candle and she showed me the way<br>\r\n</span>\r\n<span data-start="135">\r\nThere were voices down the corridor,<br>\r\nI thought I heard them say...\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="142">\r\nWelcome to the Hotel California<br>\r\nSuch a lovely place (Such a lovely place)<br>\r\nSuch a lovely face<br>\r\n</span>\r\n<span data-start="154">\r\nPlenty of room at the Hotel California<br>\r\nAny time of year (Any time of year)<br>\r\nYou can find it here\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="167">\r\nHer mind is Tiffany-twisted, she got the Mercedes bends<br>\r\nShe got a lot of pretty, pretty boys she calls friends<br>\r\n</span>\r\n<span data-start="180">\r\nHow they dance in the courtyard, sweet summer sweat.<br>\r\nSome dance to remember, some dance to forget\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="193">\r\nSo I called up the Captain,<br>\r\n"Please bring me my wine"<br>\r\n</span>\r\n<span data-start="199">\r\nHe said, "We haven\'t had that spirit here since nineteen sixty nine"<br>\r\n</span>\r\n<span data-start="206">\r\nAnd still those voices are calling from far away,<br>\r\nWake you up in the middle of the night<br>\r\nJust to hear them say...\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="219">\r\nWelcome to the Hotel California<br>\r\nSuch a lovely place (Such a lovely place)<br>\r\nSuch a lovely face<br>\r\n</span>\r\n<span data-start="231">\r\nThey livin\' it up at the Hotel California<br>\r\nWhat a nice surprise (what a nice surprise)<br>\r\nBring your alibis\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="244">\r\nMirrors on the ceiling,<br>\r\nThe pink champagne on ice<br>\r\n</span>\r\n<span data-start="250">\r\nAnd she said "We are all just prisoners here, of our own device"<br>\r\n</span>\r\n<span data-start="258">\r\nAnd in the master\'s chambers,<br>\r\nThey gathered for the feast<br>\r\n</span>\r\n<span data-start="263">\r\nThey stab it with their steely knives,<br>\r\nBut they just can\'t kill the beast\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="270">\r\nLast thing I remember, I was<br>\r\nRunning for the door<br>\r\n</span>\r\n<span data-start="276">\r\nI had to find the passage back<br>\r\nTo the place I was before<br>\r\n</span>\r\n<span data-start="283">\r\n"Relax, " said the night man,<br>\r\n"We are programmed to receive.<br>\r\n</span>\r\n<span data-start="289" data-stop="297">\r\nYou can check-out any time you like,<br>\r\nBut you can never leave! " <br></span></p><p><span><a href="http://www.minelyrics.com/eagles-hotel-california.html">Eagles - Hotel California Lyrics</a></span><br><span><a href="http://www.minelyrics.com/eagles-hotel-california.html" alt="Eagles - Hotel California Lyrics"></a></span><span data-start="289" data-stop="297"></span></p></div>\r\n\t\t\t\t\t\t</div>\r\n\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\r\n\r\n\r\n\t\t\t\t\t</div>' , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (14 , '' , NULL , NULL , 'Enrique Iglesias - Hero' , 'E' , NULL , '' , NULL , 2 , NULL , NULL , NULL , '10.1.1.6' , NULL , 1 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , 'Song Lyrics' , 'http://www.minelyrics.com/' , NULL , NULL , 0 , NULL , NULL , '24.11.2015 1:51:58' , NULL , NULL , NULL , '<div class="article-desc">\r\n\t\t\t\t\t\t<div id="youtube-lyrics" data-video-id="koJlIGDImiU">\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t<div class="desc desc-full"><p>\r\nLet me be your hero\r\n</p>\r\n\r\n<p>\r\nWould you dance if I asked you to dance?<br>\r\nWould you run and never look back?<br>\r\nWould you cry if you saw me crying?<br>\r\nWould you save my soul tonight?\r\n</p>\r\n\r\n<p>\r\nWould you tremble if I touched your lips?<br>\r\nWould you laugh? Oh, please tell me this.<br>\r\nNow would you die for the one you love?<br>\r\nHold me in your arms, tonight.\r\n</p>\r\n\r\n<p>\r\nI can be your hero, baby.<br>\r\nI can kiss away the pain.<br>\r\nI will stand by you forever.<br>\r\nYou can take my breath away.\r\n</p>\r\n\r\n<p>\r\nWould you swear that you\'ll always be mine?<br>\r\nWould you lie? Would you run and hide?<br>\r\nAm I in too deep? Have I lost my mind?<br>\r\nI don\'t care. You\'re here tonight.\r\n</p>\r\n\r\n<p>\r\nI can be your hero, baby.<br>\r\nI can kiss away the pain.<br>\r\nI will stand by you forever.<br>\r\nYou can take my breath away.\r\n</p>\r\n\r\n<p>\r\nOh, I just want to hold you.<br>\r\nI just want to hold you, oh, yeah.\r\n</p>\r\n\r\n<p>\r\nAm I in too deep? Have I lost my mind?<br>\r\nWell, I don\'t care. You\'re here tonight.\r\n</p>\r\n\r\n<p>\r\nI can be your hero, baby.<br>\r\nI can kiss away the pain, oh, yeah.<br>\r\nI will stand by you forever.<br>\r\nYou can take my breath away.\r\n</p>\r\n\r\n<p>\r\n(I can be your hero, baby)<br>\r\nI can be your hero.<br>\r\nI can kiss away the pain.<br>\r\nAnd I will stand by you forever.<br>\r\nYou can take my breath away.<br>\r\nYou can take my breath away.\r\n</p>\r\n\r\n<p>\r\nI can be your hero. <br></p><p><br><span><a href="http://www.minelyrics.com/enrique-iglesias-hero.html">Enrique Iglesias - Hero Lyrics</a></span></p></div>\r\n\t\t\t\t\t\t</div>\r\n\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\r\n\r\n\r\n\t\t\t\t\t</div>' , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (15 , '' , NULL , NULL , 'Hozier - Take Me To Church' , 'H' , NULL , '' , NULL , 3 , NULL , NULL , NULL , '10.1.1.6' , NULL , 1 , NULL , 1 , 1 , 1 , 1 , 0 , NULL , NULL , 'Online Lyrics' , 'http://www.minelyrics.com/' , NULL , NULL , 0 , NULL , '10.06.2005 15:58:40' , '24.11.2015 1:53:10' , NULL , NULL , NULL , '<span data-start="0">My lover\'s got humour<br>\r\nShe\'s the giggle at a funeral<br>\r\n</span>\r\n<span data-start="5">\r\nKnows everybody\'s disapproval<br>\r\nI should\'ve worshipped her sooner<br>\r\n</span>\r\n<br>\r\n<span data-start="11">\r\nIf the heavens ever did speak<br>\r\nShe\'s the last true mouthpiece<br>\r\n</span>\r\n<span data-start="17">\r\nEvery Sunday\'s getting more bleak<br>\r\nA fresh poison each week<br>\r\n</span>\r\n<br>\r\n<span data-start="23">\r\n"We were born sick," you heard them say it<br>\r\nMy church offers no absolutes<br>\r\n</span>\r\n<span data-start="29">\r\nShe tells me, "Worship in the bedroom."<br>\r\nThe only heaven I\'ll be sent to<br>\r\n</span>\r\n<span data-start="35">\r\nIs when I\'m alone with you<br>\r\nI was born sick<br>\r\n</span>\r\n<span data-start="40">\r\nBut I love it<br>\r\nCommand me to be well<br>\r\n</span>\r\n<span data-start="44" data-stop="55">\r\nAaay. Amen. Amen. Amen.<br>\r\n</span>\r\n\r\n<p>\r\n<span data-start="56">\r\nTake me to church<br></span>\r\n<span data-start="58">\r\nI\'ll worship like a dog at the shrine of your lies<br>\r\nI\'ll tell you my sins and you can sharpen your knife<br></span>\r\n<span data-start="65" data-stop="69.5">\r\nOffer me that deathless death<br>\r\nGood God, let me give you my life<br></span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="70">\r\nTake me to church<br></span>\r\n<span data-start="72">\r\nI\'ll worship like a dog at the shrine of your lies<br>\r\nI\'ll tell you my sins and you can sharpen your knife<br></span>\r\n<span data-start="80" data-stop="85.5">\r\nOffer me that deathless death<br>\r\nGood God, let me give you my life<br></span>\r\n</p>\r\n\r\n<br>\r\n<span data-start="87">\r\nIf I\'m a pagan of the good times<br>\r\nMy lover\'s the sunlight<br>\r\n</span>\r\n<span data-start="92">\r\nTo keep the Goddess on my side<br>\r\nShe demands a sacrifice<br>\r\n</span>\r\n<br>\r\n<span data-start="98">\r\nDrain the whole sea<br>\r\nGet something shiny<br>\r\n</span>\r\n<span data-start="102">\r\nSomething meaty for the main course<br>\r\nThat\'s a fine-looking high horse<br>\r\n</span>\r\n<span data-start="107">\r\nWhat you got in the stable?<br>\r\nWe\'ve a lot of starving faithful<br>\r\n</span>\r\n\r\n<p>\r\n<span data-start="113">\r\nThat looks tasty<br>\r\nThat looks plenty<br>\r\nThis is hungry work<br>\r\n</span>\r\n</p>\r\n\r\n\r\n<p>\r\n<span data-start="119">\r\nTake me to church<br>\r\n</span>\r\n<span data-start="121">\r\nI\'ll worship like a dog at the shrine of your lies<br>\r\nI\'ll tell you my sins so you can sharpen your knife<br>\r\n</span>\r\n<span data-start="128" data-stop="133.8">\r\nOffer me my deathless death<br>\r\nGood God, let me give you my life<br>\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="134">\r\nTake me to church<br>\r\n</span>\r\n<span data-start="136">\r\nI\'ll worship like a dog at the shrine of your lies<br>\r\nI\'ll tell you my sins so you can sharpen your knife<br>\r\n</span>\r\n<span data-start="143" data-stop="149">\r\nOffer me my deathless death<br>\r\nGood God, let me give you my life<br>\r\n</span>\r\n</p>\r\n\r\n<br>\r\n<span data-start="150">\r\nNo Masters or Kings<br>\r\nWhen the Ritual begins<br>\r\n</span>\r\n<span data-start="157">\r\nThere is no sweeter innocence than our gentle sin<br>\r\nIn the madness and soil of that sad earthly scene<br>\r\n</span>\r\n<span data-start="172">\r\nOnly then I am human<br>\r\nOnly then I am clean<br>\r\n</span>\r\n<span data-start="180">\r\nOoh oh. Amen. Amen. Amen.<br>\r\n</span>\r\n\r\n<p>\r\n<span data-start="196">\r\nTake me to church<br>\r\n</span>\r\n<span data-start="198">\r\nI\'ll worship like a dog at the shrine of your lies<br>\r\nI\'ll tell you my sins and you can sharpen your knife<br>\r\n</span>\r\n<span data-start="206" data-stop="210.9">\r\nOffer me that deathless death<br>\r\nGood God, let me give you my life\r\n</span>\r\n</p>\r\n\r\n\r\n\r\n<p>\r\n<span data-start="211">\r\nTake me to church<br>\r\n</span>\r\n<span data-start="213">\r\nI\'ll worship like a dog at the shrine of your lies<br>\r\nI\'ll tell you my sins and you can sharpen your knife<br>\r\n</span>\r\n<span data-start="220" data-stop="227">\r\nOffer me that deathless death<br>\r\nGood God, let me give you my life <br></span></p><p><span data-start="220" data-stop="227"><span><a href="http://www.minelyrics.com/hozier-take-me-to-church.html">Hozier - Take Me To Church Lyrics</a></span><br></span>\r\n</p>' , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (16 , '' , NULL , NULL , 'Justin Timberlake - Mirrors' , 'J' , NULL , '' , NULL , 1 , NULL , NULL , NULL , '10.1.1.6' , NULL , 1 , NULL , 1 , 1 , 1 , 1 , 0 , NULL , NULL , 'Pop Lyrics' , 'http://www.minelyrics.com/' , NULL , NULL , 0 , NULL , '10.06.2005 15:59:41' , '24.11.2015 2:11:53' , NULL , NULL , NULL , '<div class="article-desc">\r\n\t\t\t\t\t\t<div id="youtube-lyrics" data-video-id="uuZE_IRwLNI">\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t<div class="desc desc-full"><p>\r\n<span data-start="29">\r\nAren\'t you something to admire?<br>\r\nCause your shine is something like a mirror<br>\r\n</span>\r\n<span data-start="35">\r\nAnd I can\'t help but notice<br>\r\nYou reflect in this heart of mine<br>\r\n</span>\r\n<span data-start="39">\r\nIf you ever feel alone and<br>\r\nThe glare makes me hard to find<br>\r\n</span>\r\n<span data-start="47">\r\nJust know that I\'m always parallel on the other side\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="53">\r\nCause with your hand in my hand and a pocket full of soul<br>\r\nI can tell you there\'s no place we couldn\'t go<br>\r\n</span>\r\n<span data-start="59">\r\nJust put your hand on the glass<br>\r\nI\'ll be trying to pull you through you just gotta be strong<br>\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="65">\r\nCause I don\'t wanna lose you now<br>\r\nI\'m looking right at the other half of me<br>\r\n</span>\r\n<span data-start="71">\r\nThe vacancy that sat in my heart<br>\r\nIs a space that now you hold<br>\r\n</span>\r\n<span data-start="77">\r\nShow me how to fight for now<br>\r\nAnd I\'ll tell you, baby, it was easy<br>\r\n</span>\r\n<span data-start="83">\r\nComing back here to you once I figured it out<br>\r\nYou were right here all along\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="90">\r\nIt\'s like you\'re my mirror<br>\r\nMy mirror staring back at me<br>\r\n</span>\r\n<span data-start="96">\r\nI couldn\'t get any bigger<br>\r\nWith anyone else beside of me<br>\r\n</span>\r\n<span data-start="102">\r\nAnd now it\'s clear as this promise<br>\r\nThat we\'re making two reflections into one<br>\r\n</span>\r\n<span data-start="110">\r\nCause it\'s like you\'re my mirror<br>\r\nMy mirror staring back at me, staring back at me\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="116">\r\nAren\'t you something, an original<br>\r\nCause it doesn\'t seem merely assembled<br>\r\n</span>\r\n<span data-start="122">\r\nAnd I can\'t help but stare, cause<br>\r\nI see truth somewhere in your eyes<br>\r\n</span>\r\n<span data-start="127">\r\nI can\'t ever change without you<br>\r\nYou reflect me, I love that about you<br>\r\n</span>\r\n<span data-start="135">\r\nAnd if I could, I would look at us all the time\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="140">\r\nCause with your hand in my hand and a pocket full of soul<br>\r\nI can tell you there\'s no place we couldn\'t go<br>\r\n</span>\r\n<span data-start="147">\r\nJust put your hand on the glass<br>\r\nI\'ll be trying to pull you through<br>\r\nYou just gotta be strong\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="153">\r\nCause I don\'t wanna lose you now<br>\r\nI\'m looking right at the other half of me<br>\r\n</span>\r\n<span data-start="159">\r\nThe vacancy that sat in my heart<br>\r\nIs a space that now you hold<br>\r\n</span>\r\n<span data-start="165">\r\nShow me how to fight for now<br>\r\nAnd I\'ll tell you, baby, it was easy<br>\r\n</span>\r\n<span data-start="171">\r\nComing back into you once I figured it out<br>\r\nYou were right here all along\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="178">\r\nIt\'s like you\'re my mirror<br>\r\nMy mirror staring back at me<br>\r\n</span>\r\n<span data-start="184">\r\nI couldn\'t get any bigger<br>\r\nWith anyone else beside of me<br>\r\n</span>\r\n<span data-start="190">\r\nAnd now it\'s clear as this promise<br>\r\nThat we\'re making two reflections into one<br>\r\n</span>\r\n<span data-start="196">\r\nCause it\'s like you\'re my mirror<br>\r\nMy mirror staring back at me, staring back at me\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="204">\r\nYesterday is history<br>\r\nTomorrow\'s a mystery<br>\r\n</span>\r\n<span data-start="217">\r\nI can see you looking back at me<br>\r\nKeep your eyes on me<br>\r\nBaby, keep your eyes on me\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="228">\r\nCause I don\'t wanna lose you now<br>\r\nI\'m looking right at the other half of me<br>\r\n</span>\r\n<span data-start="234">\r\nThe vacancy that sat in my heart<br>\r\nIs a space that now you hold<br>\r\n</span>\r\n<span data-start="240">\r\nShow me how to fight for now (please show me, baby)<br>\r\nI\'ll tell you, baby, it was easy<br>\r\n</span>\r\n<span data-start="245">\r\nComing back into you once I figured it out<br>\r\nYou were right here all along\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="253">\r\nIt\'s like you\'re my mirror<br>\r\nMy mirror staring back at me<br>\r\n</span>\r\n<span data-start="259">\r\nI couldn\'t get any bigger<br>\r\nWith anyone else beside of me<br>\r\n</span>\r\n<span data-start="265">\r\nAnd now it\'s clear as this promise<br>\r\nThat we\'re making two reflections into one<br>\r\n</span>\r\n<span data-start="271" data-stop="280">\r\nCause it\'s like you\'re my mirror<br>\r\nMy mirror staring back at me, staring back at me\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="325">\r\nYou are, you are the love of my life <i>[x10]</i>\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="359">\r\nNow you\'re the inspiration of this precious song<br>\r\nAnd I just wanna see your face light up since you put me on<br>\r\n</span>\r\n<span data-start="371">\r\nSo now I say goodbye to the old me, it\'s already gone<br>\r\nAnd I can\'t wait wait wait wait wait to get you home<br>\r\n</span>\r\n<span data-start="382">\r\nJust to let you know, you are\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="387">\r\nYou are, you are the love of my life <i>[x8]</i>\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="409">\r\nGirl you\'re my reflection, all I see is you<br>\r\nMy reflection, in everything I do<br>\r\n</span>\r\n<span data-start="421">\r\nYou\'re my reflection and all I see is you<br>\r\nMy reflection, in everything I do\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="435" data-stop="489">\r\nYou are, you are the love of my life <i>[x16]</i></span></p><p><span data-start="435" data-stop="489"><i><span><a href="http://www.minelyrics.com/justin-timberlake-mirrors.html">Justin Timberlake - Mirrors Lyrics</a></span><br></i>\r\n</span>\r\n</p></div>\r\n\t\t\t\t\t\t</div>\r\n\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\r\n\r\n\r\n\t\t\t\t\t</div>' , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (17 , '' , NULL , NULL , 'Katy Perry - Dark Horse' , 'K' , NULL , '' , NULL , 2 , NULL , NULL , NULL , '10.1.1.6' , NULL , 1 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , 'Lyrics English' , 'http://www.minelyrics.com/' , NULL , NULL , 0 , NULL , NULL , '24.11.2015 2:16:29' , NULL , NULL , NULL , '<div class="article-desc">\r\n\t\t\t\t\t\t<div id="youtube-lyrics" data-video-id="0KSOMA3QBU0">\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t<div class="desc desc-full"><p>\r\n<span data-start="4" data-stop="7">\r\nOh, no.\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<i>[Juicy J]</i><br>\r\n<span data-start="9">\r\nYeah<br>\r\nYa\'ll know what it is<br>\r\n</span>\r\n<span data-start="12">\r\nKaty Perry<br>\r\nJuicy J, aha.<br>\r\nLet\'s rage\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<i>[Katy Perry]</i><br>\r\n<span data-start="19">\r\nI knew you were<br>\r\nYou were gonna come to me<br>\r\n</span>\r\n<span data-start="22">\r\nAnd here you are<br>\r\nBut you better choose carefully<br>\r\n</span>\r\n<span data-start="26">\r\nCause I, Im capable of anything<br>\r\nOf anything and everything\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="33">\r\nMake me your Aphrodite<br>\r\nMake me your one and only<br>\r\n</span>\r\n<span data-start="40">\r\nBut dont make me your enemy, your enemy, your enemy\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="49">\r\nSo you wanna play with magic<br>\r\nBoy, you should know what you\'re falling for<br>\r\n</span>\r\n<span data-start="56">\r\nBaby do you dare to do this?<br>\r\nCause Im coming at you like a dark horse<br>\r\n</span>\r\n<span data-start="64">\r\nAre you ready for, ready for<br>\r\nA perfect storm, perfect storm<br>\r\n</span>\r\n<span data-start="71" data-stop="77">\r\nCause once youre mine, once youre mine<br>\r\nTheres no going back\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="84">\r\nMark my words<br>\r\nThis love will make you levitate<br>\r\n</span>\r\n<span data-start="88">\r\nLike a bird<br>\r\nLike a bird without a cage<br>\r\n</span>\r\n<span data-start="91">\r\nBut down to earth<br>\r\nIf you choose to walk away, dont walk away\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="98">\r\nIts in the palm of your hand now baby<br>\r\nIts a yes or no, no maybe<br>\r\n</span>\r\n<span data-start="106">\r\nSo just be sure before you give it all to me<br>\r\nAll to me, give it all to me\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="114">\r\nSo you wanna play with magic<br>\r\nBoy, you should know what you\'re falling for<br>\r\n</span>\r\n<span data-start="121">\r\nBaby do you dare to do this?<br>\r\nCause Im coming at you like a dark horse<br>\r\n</span>\r\n<span data-start="130">\r\nAre you ready for, ready for<br>\r\nA perfect storm, perfect storm<br>\r\n</span>\r\n<span data-start="136">\r\nCause once youre mine, once youre mine (love trippin\')<br>\r\nTheres no going back\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<i>[Juicy J]</i><br>\r\n<span data-start="143">\r\nUh<br>\r\nShes a beast<br>\r\nI call her Karma (come back)<br>\r\n</span>\r\n<span data-start="147">\r\nShe eats your heart out<br>\r\nLike Jeffrey Dahmer (woo)<br>\r\nBe careful<br>\r\n</span>\r\n<span data-start="151">\r\nTry not to lead her on<br>\r\nShortys heart is on steroids<br>\r\nCause her love is so strong<br>\r\n</span>\r\n<span data-start="157">\r\nYou may fall in love<br>\r\nWhen you meet her<br>\r\n</span>\r\n<span data-start="158">\r\nIf you get the chance you better keep her<br>\r\nShe\'s sweet as pie but if you break her heart<br>\r\n</span>\r\n<span data-start="162">\r\nShe\'ll turn cold as a freezer<br>\r\nThat fairy tale ending with a knight in shining armor<br>\r\n</span>\r\n<span data-start="167">\r\nShe can be my Sleeping Beauty<br>\r\nIm gon put her in a coma<br>\r\nWoo!<br>\r\n</span>\r\n<span data-start="171">\r\nDamn I think I love her<br>\r\nShorty so bad, Im sprung and I dont care<br>\r\n</span>\r\n<span data-start="174">\r\nShe ride me like a roller coaster<br>\r\nTurned the bedroom into a fair (a fair!)<br>\r\n</span>\r\n<span data-start="178">\r\nHer love is like a drug<br>\r\nI was tryna hit it and quit it<br>\r\n</span>\r\n<span data-start="182">\r\nBut lil\' mama so dope<br>\r\nI messed around and got addicted\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<i>[Katy Perry]</i><br>\r\n<span data-start="187">\r\nSo you wanna play with magic<br>\r\nBoy, you should know what you\'re falling for (you should know)<br>\r\n</span>\r\n<span data-start="194">\r\nBaby do you dare to do this?<br>\r\nCause Im coming at you like a dark horse (like a dark horse)<br>\r\n</span>\r\n<span data-start="202">\r\nAre you ready for, ready for (ready for)<br>\r\nA perfect storm, perfect storm (a perfect storm)<br>\r\n</span>\r\n<span data-start="209" data-stop="216">\r\nCause once youre mine, once youre mine (mine)<br>\r\nTheres no going back <br></span></p><p><br><span><a href="http://www.minelyrics.com/katy-perry-dark-horse.html">Katy Perry - Dark Horse Lyrics</a></span><span data-start="209" data-stop="216"></span></p></div>\r\n\t\t\t\t\t\t</div>\r\n\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\r\n\r\n\r\n\t\t\t\t\t</div>' , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (18 , '' , NULL , NULL , 'Lana Del Rey - High By The Beach' , 'L' , NULL , '' , NULL , 3 , NULL , NULL , NULL , '10.1.1.6' , NULL , 1 , NULL , 1 , 1 , 1 , 1 , 0 , NULL , NULL , 'Lyrics' , 'http://www.minelyrics.com/' , NULL , NULL , 0 , NULL , '10.06.2005 16:00:27' , '24.11.2015 2:17:48' , NULL , NULL , NULL , '<div class="article-desc">\r\n\t\t\t\t\t\t<div id="youtube-lyrics" data-video-id="QnxpHIl5Ynw">\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t<div class="desc desc-full"><p>\r\n<span data-start="40">\r\nBoy, look at you, looking at me<br>\r\nI know you know how I feel<br>\r\n</span>\r\n<span data-start="49">\r\nLoving you is hard, being here is harder<br>\r\nYou take the wheel<br>\r\n</span>\r\n<span data-start="56">\r\nI don\'t wanna do this anymore<br>\r\nIt\'s so surreal, I can\'t survive<br>\r\n</span>\r\n<span data-start="67">\r\nIf this is all that\'s real\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="70">\r\nAll I wanna do is get high by the beach<br>\r\nGet high by the beach, get high<br>\r\n</span>\r\n<span data-start="76">\r\nAll I wanna do is get by by the beach<br>\r\nGet by baby, baby, bye, bye<br>\r\n</span>\r\n<span data-start="84">\r\nThe truth is I never<br>\r\nBought into your bullshit<br>\r\n</span>\r\n<span data-start="88">\r\nWhen you would pay tribute to me<br>\r\nCause I know that<br>\r\n</span>\r\n<span data-start="92">\r\nAll I wanted to do is get high by the beach<br>\r\nGet high baby, baby, bye, bye\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="98">\r\nBoy look at you, looking at me<br>\r\nI know you don\'t understand<br>\r\n</span>\r\n<span data-start="106">\r\nYou could be a bad motherfucker<br>\r\nBut that don\'t make you a man<br>\r\n</span>\r\n<span data-start="114">\r\nNow you\'re just another one of my problems<br>\r\nBecause you got out of hand<br>\r\n</span>\r\n<span data-start="122">\r\nWe won\'t survive<br>\r\nWe\'re sinking into the sand\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="128">\r\nAll I wanna do is get high by the beach<br>\r\nGet high by the beach, get high<br>\r\n</span>\r\n<span data-start="134">\r\nAll I wanna do is get by by the beach<br>\r\nGet by baby, baby, bye, bye<br>\r\n</span>\r\n<span data-start="142">\r\nThe truth is I never<br>\r\nBought into your bullshit<br>\r\n</span>\r\n<span data-start="146">\r\nWhen you would pay tribute to me<br>\r\nCause I know that<br>\r\n</span>\r\n<span data-start="150">\r\nAll I wanted to do is get high by the beach<br>\r\nGet high baby, baby, bye, bye\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="156">\r\nLights, camera, accin<br>\r\nI\'ll do it on my own<br>\r\n</span>\r\n<span data-start="165">\r\nDon\'t need your money, money<br>\r\nTo get me what I want<br>\r\n</span>\r\n<span data-start="171">\r\nLights camera accin<br>\r\nI\'ll do it on my own<br>\r\n</span>\r\n<span data-start="180" data-stop="185">\r\nDon\'t need your money, money<br>\r\nTo get me what I want\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="206">\r\nAll I wanna do is get high by the beach<br>\r\nGet high by the beach, get high<br>\r\n</span>\r\n<span data-start="212">\r\nAll I wanna do is get by by the beach<br>\r\nGet by baby, baby, bye, bye<br>\r\n</span>\r\n<span data-start="219">\r\nThe truth is I never<br>\r\nBought into your bullshit<br>\r\n</span>\r\n<span data-start="224">\r\nWhen you would pay tribute to me<br>\r\nCause I know that<br>\r\n</span>\r\n<span data-start="228" data-stop="235">\r\nAll I wanted to do is get high by the beach<br>\r\nGet high baby, baby, bye, bye\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="235" data-stop="262">\r\nHigh-igh-igh-igh, high-igh-igh-igh<br>\r\nHigh-igh-igh-igh, high-igh-igh-igh\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="264">\r\nEveryone can start again<br>\r\nNot through love but through revenge<br>\r\n</span>\r\n<span data-start="271" data-stop="279">\r\nThrough the fire, we\'re born again<br>\r\nPeace by vengeance brings the end <br></span></p><p><span data-start="271" data-stop="279"><span><a href="http://www.minelyrics.com/lana-del-rey-high-by-the-beach.html">Lana Del Rey - High By The Beach Lyrics</a></span><br></span>\r\n\r\n</p></div>\r\n\t\t\t\t\t\t</div>\r\n\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\r\n\r\n\r\n\t\t\t\t\t</div>' , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (19 , NULL , '2005-06-10 17:18:00' , NULL , 'PHP Applications gathering ' , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , '10.1.1.6' , NULL , 1 , NULL , 1 , 1 , 0 , 1 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , '10.06.2005 17:18:26' , '10.06.2005 17:18:26' , NULL , NULL , 'The PHP Quebec Conference team is looking for the best software developed with PHP, MySQL and Apache. Each proposal received by April 29th 29 will be will be reviewed by an experts and members of the PHP Quebec community. ' , 'The PHP Quebec Conference team is looking for the best software developed with PHP, MySQL and Apache. Each proposal received by April 29th 29 will be will be reviewed by an experts and members of the PHP Quebec community. \r\n\r\nThe most useful software will be included on the 2005 PHP Quebec Conference CDROM along with over 25 hours of conference which where given by the leaders behind PHP, MySQL and Apache, as well as regional and international experts at the 2005 PHP Quebec conference. ' , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (20 , NULL , '2005-06-10 17:18:00' , NULL , 'MySQL, RedHat and other partners started a new campaign against software patents' , NULL , NULL , NULL , NULL , 2 , NULL , NULL , NULL , '10.1.1.6' , NULL , 1 , NULL , 1 , 1 , 0 , 1 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , '10.06.2005 17:19:00' , '10.06.2005 17:19:00' , NULL , NULL , 'In order to boost more awareness of the pending software patent issues in the European UnionMySQL, ISP 1&1, RedHat and other partners started a new portal "http://nosoftwarepatents.com". The site is available in 10 different european languages and explains why software patents are bad, and how they might affect you. ' , 'Under the influence of the patent system and big industry lobbyists, the European Union is on the verge of making a huge mistake: to pass a law that would legalize software patents. \r\n\r\nIf that happens, you will pay dearly. Europe\'s software industry will fall victim to unscrupulous extortioners. A cartel of large corporations will crush smaller competitors. Consequently, we will all pay more money for less good and less secure software. You personally, your household, your company, your government, all of us. \r\n\r\nYou\'ll know when you get the bill. When someone breaks into your computer, reads your E-mails, and steals the password of your bank account. When your computer crashes every day. When spam doesn\'t stop. When prices go up and companies shut down. When people lose their jobs. \r\n\r\n"Chances are that patents on software, common practice in the US and on the brink of being legalised in Europe, in fact stifle innovation. Europe could still alter course." \r\nDeutsche Bank Research \r\n\r\nWithout software patents, Europe could save costs, foster innovation, enhance security, and create jobs. Thanks to Linux and other open-source software, Europe has the chance to gain independence from Microsoft and other large American companies. However, if the EU allows software patents, then that\'s the beginning of the end for Linux. Not only for Linux. It\'s just a prominent example' , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (21 , NULL , '2005-06-10 17:19:00' , NULL , 'PHP Quebec 2005: PHP - MySQL - Apache ' , NULL , NULL , NULL , NULL , 3 , NULL , NULL , NULL , '10.1.1.6' , NULL , 1 , NULL , 1 , 1 , 0 , 1 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , '10.06.2005 17:19:19' , '10.06.2005 17:19:19' , NULL , NULL , 'PHP Quebec is proud to present an extended conference in Montreal, Quebec, covering the latest innovations in PHP, MySQL and Apache.' , 'PHP Quebec is proud to present an extended conference in Montreal, Quebec, covering the latest innovations in PHP, MySQL and Apache. From March 30th until April 1st, major AMP contributors will talk about Security, Web Services, PHP 5 and great achievements with PHP in North America. The 4 tracks are available at the PHP Quebec site. \r\n\r\nPHP Certification exams by Zend are included with every admission ticket to the conference. The certification center will also provide MySQL and Linux certification from MySQL AB, LPI and Savoir Faire Linux. The examination center is open to everyone. ' , NULL , 1 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (22 , NULL , '2005-06-10 17:27:28' , NULL , 'What is eShopping? ' , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , '10.1.1.6' , NULL , 1 , NULL , 1 , 1 , 0 , 1 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , '10.06.2005 17:27:28' , '10.06.2005 17:27:28' , NULL , NULL , NULL , 'eShopping contains sites of which the primary focus is to allow the consumer to select and obtain goods and services over the Web. Common examples include: \r\nIntegrated online shopping-cart systems that allow the user to order directly over the Web \r\nOnline shopping-cart systems that allow the user to generate an order form to be sent to the merchant via fax or snail-mail \r\nSimple directories of products and prices that the user can order via mail or phone. \r\nThe key is that the site contains three elements: lists of products for sale, prices, and information on how the consumer can obtain the product from their own h\r\n' , NULL , 4 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (23 , NULL , '2005-06-10 17:28:21' , NULL , 'How to Shop at Our Online Store?' , NULL , NULL , NULL , NULL , 2 , NULL , NULL , NULL , '10.1.1.6' , NULL , 1 , NULL , 1 , 1 , 0 , 1 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , '10.06.2005 17:28:21' , '10.06.2005 17:28:21' , NULL , NULL , NULL , 'Read online help please. See Help Section' , NULL , 3 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (24 , NULL , '2005-06-10 17:29:09' , NULL , 'I\'ve received an error message stating that my credit card is invalid. What should I do?' , NULL , NULL , NULL , NULL , 3 , NULL , NULL , NULL , '10.1.1.6' , NULL , 1 , NULL , 1 , 1 , 0 , 1 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , '10.06.2005 17:29:09' , '10.06.2005 17:29:09' , NULL , NULL , NULL , 'If you see an error message stating that your credit card is invalid, this has most likely happened for one of the following reasons: \r\n1. You have incorrectly entered your credit card number. \r\n2. You have incorrectly entered your CVV code. \r\n3. You have incorrectly entered your expiration date. \r\n4. You have used a cancelled credit card to place your order. \r\n5. Your card is restricted to point-of-sale transactions only (sales where your card is physically scanned). \r\n\r\nPlease double-check your credit card and/or with your bank to ensure that none of these problems have occurred. If you realize that you\'ve made a mistake, simply return to the payment option page and enter in the correct information. Continue checking out as normal. \r\n\r\nIf you\'re still encountering problems, please contact us. However, at NO point should you ever e-mail us your full credit card number. Your e-mail does not have the same security features in place as the website. If you wish to reference a specific credit card that you already have on file with us, please reference by the order number.\r\n\r\nIf you believe that you have provided us with an incorrect CVV code or expiration date you may send us that corrected information via e-mail referencing by the order number.\r\n\r\nIf you believe that your card was declined due to insufficient funds, but now have those funds available, you may ask that we attempt to charge your credit card again.\r\n\r\nIf you believe that your card is restricted to point-of-sale transactions, you must either contact your bank about removing such restrictions, provide us with a different credit card, or choose a new payment method.\r\n' , NULL , 2 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (25 , NULL , '2005-06-10 17:29:38' , NULL , 'Why is my online confirmation number different from my order number?' , NULL , NULL , NULL , NULL , 4 , NULL , NULL , NULL , '10.1.1.6' , NULL , 1 , NULL , 1 , 1 , 0 , 1 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , '10.06.2005 17:29:38' , '10.06.2005 17:29:38' , NULL , NULL , NULL , 'This is due to the internal process we must maintain to properly track your order through the system. Once an order number is assigned you may use it for all future references to that purchase.' , NULL , 1 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (26 , '' , NULL , NULL , 'Maroon 5 - One More Night' , 'M' , NULL , '' , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 0 , NULL , NULL , 'Online Lyrics' , 'http://www.minelyrics.com/' , NULL , NULL , 0 , NULL , NULL , '24.11.2015 2:21:39' , NULL , NULL , NULL , '<div class="article-desc">\r\n\t\t\t\t\t\t<div id="youtube-lyrics" data-video-id="fwK7ggA3-bU">\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t<div class="desc desc-full"><p>\r\n<span data-start="8">\r\nYou and I go hard at each other like we\'re going to war.<br>\r\nYou and I go rough, we keep throwing things and slamming the door.<br>\r\n</span>\r\n<span data-start="18">\r\nYou and I get so damn dysfunctional, we stopped keeping score.<br>\r\nYou and I get sick, yeah, I know that we can\'t do this no more.\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="29">\r\nYeah, but baby there you go again, there you go again, making me love you.<br>\r\nYeah, I stopped using my head, using my head, let it all go.\r\n</span><br>\r\n<span data-start="40">\r\nGot you stuck on my body, on my body, like a tattoo.<br>\r\nAnd now I\'m feeling stupid, feeling stupid, crawling back to you.\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="50">\r\nSo I cross my heart and I hope to die<br>\r\nThat I\'ll only stay with you one more night<br>\r\n</span>\r\n<span data-start="60">\r\nAnd I know I said it a million times<br>\r\nBut I\'ll only stay with you one more night\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="70">\r\nTry to tell you "no" but my body keeps on telling you "yes".<br>\r\nTry to tell you "stop", but your lipstick got me so out of breath.<br>\r\n</span>\r\n<span data-start="80">\r\nI\'ll be waking up in the morning, probably hating myself.<br>\r\nAnd I\'ll be waking up, feeling satisfied but guilty as hell.\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="91">\r\nYeah, but baby there you go again, there you go again, making me love you.<br>\r\n(Making me love you)<br>\r\n</span>\r\n<span data-start="97">\r\nYeah, I stopped using my head, using my head, let it all go.<br>\r\n(I let it all go)<br>\r\n</span>\r\n<span data-start="102">\r\nGot you stuck on my body, on my body, like a tattoo.<br>\r\n(Like a tattoo, yeah)<br>\r\n</span>\r\n<span data-start="107">\r\nAnd now I\'m feeling stupid, feeling stupid, crawling back to you.\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="112">\r\nSo I cross my heart and I hope to die<br>\r\n(Yeah)<br>\r\n</span>\r\n<span data-start="118">\r\nThat I\'ll only stay with you one more night<br>\r\n(Oh)<br>\r\n</span>\r\n<span data-start="122">\r\nAnd I know I said it a million times<br>\r\n(Yeah)<br>\r\n</span>\r\n<span data-start="128">\r\nBut I\'ll only stay with you one more night<br>\r\n(Yeah)\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="136">\r\nYeah, baby, give me one more night<br>\r\n</span>\r\n<span data-start="141">\r\nYeah, baby, give me one more night (whoa, yeah)<br>\r\nYeah, baby, give me one more night (oh, yeah, yeah)\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="154">\r\nYeah, but baby there you go again, there you go again making me love you.<br>\r\nYeah, I stopped using my head, using my head, let it all go.\r\n</span>\r\n<br>\r\n<span data-start="164">\r\nGot you stuck on my body, on my body like a tattoo.<br>\r\nYeah, yeah, yeah, yeah\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="174">\r\nSo I cross my heart and I hope to die<br>\r\n(Oh oh oh oh, oh oh oh oh)<br>\r\n</span>\r\n<span data-start="180">\r\nThat I\'ll only stay with you one more night<br>\r\n(Oh oh oh oh oh oh)<br>\r\n</span>\r\n<span data-start="184">\r\nAnd I know I said it a million times<br>\r\n(Oh, I said it a million times)<br>\r\n</span>\r\n<span data-start="190">\r\nBut I\'ll only stay with you one more night<br>\r\n(Yeah, baby give me one more night)\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="195">\r\nSo I cross my heart and I hope to die (yeah, yeah)<br>\r\nThat I\'ll only stay with you one more night (yeah, yeah)<br>\r\n</span>\r\n<span data-start="206">\r\nAnd I know I said it a million times (yeah, yeah)<br>\r\nBut I\'ll only stay with you one more night (yeah, yeah)\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="217">\r\nI don\'t know, whatever.\r\n</span>\r\n</p></div>\r\n\t\t\t\t\t\t</div>\r\n\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\r\n\r\n\r\n\t\t\t\t\t<span><a href="http://www.minelyrics.com/maroon-5-one-more-night.html">Maroon 5 - One More Night Lyrics</a></span><br></div>' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (27 , '' , NULL , NULL , 'Michael Jackson - They Don\'t Care About Us' , 'M' , NULL , '' , NULL , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 0 , NULL , NULL , 'Mine Lyrics' , 'http://www.minelyrics.com/' , NULL , NULL , 0 , NULL , NULL , '24.11.2015 2:25:00' , NULL , NULL , NULL , '<div class="article-desc">\r\n\t\t\t\t\t\t<div id="youtube-lyrics" data-video-id="t1pqi8vjTLY">\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t<div class="desc desc-full"><p>\r\n<span data-start="17">\r\nAll I wanna say is that<br>\r\nThey don\'t really care about us<br>\r\n</span>\r\n<span data-start="23" data-stop="30">\r\nAll I wanna say is that<br>\r\nThey don\'t really care about us\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="54">\r\nSkin head, dead head<br>\r\nEverybody gone bad<br>\r\n</span>\r\n<span data-start="58">\r\nSituation, aggravation<br>\r\nEverybody allegation<br>\r\n</span>\r\n<span data-start="60">\r\nIn the suite, on the news<br>\r\nEverybody dog food<br>\r\n</span>\r\n<span data-start="63">\r\nBang bang, shot dead<br>\r\nEverybody\'s gone mad\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="66">\r\nAll I wanna say is that<br>\r\nThey don\'t really care about us<br>\r\n</span>\r\n<span data-start="71">\r\nAll I wanna say is that<br>\r\nThey don\'t really care about us\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="76">\r\nBeat me, hate me<br>\r\nYou can never break me<br>\r\n</span>\r\n<span data-start="78">\r\nWill me, thrill me<br>\r\nYou can never kill me<br>\r\n</span>\r\n<span data-start="82">\r\nJew me, sue me<br>\r\nEverybody do me<br>\r\n</span>\r\n<span data-start="84">\r\nKick me, kike me<br>\r\nDon\'t you black or white me\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="87">\r\nAll I wanna say is that<br>\r\nThey don\'t really care about us<br>\r\n</span>\r\n<span data-start="92">\r\nAll I wanna say is that<br>\r\nThey don\'t really care about us\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="98">\r\nTell me what has become of my life<br>\r\nI have a wife and two children who love me<br>\r\n</span>\r\n<span data-start="103">\r\nI am the victim of police brutality, now<br>\r\nI\'m tired of bein\' the victim of hate<br>\r\n</span>\r\n<span data-start="111">\r\nYou\'re rapin\' me off my pride<br>\r\nOh, for God\'s sake<br>\r\n</span>\r\n<span data-start="114">\r\nI look to heaven to fulfill its prophecy...<br>\r\nSet me free\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="119">\r\nSkin head, dead head<br>\r\nEverybody gone bad<br>\r\n</span>\r\n<span data-start="122">\r\nTrepidation, speculation<br>\r\nEverybody allegation<br>\r\n</span>\r\n<span data-start="124">\r\nIn the suite, on the news<br>\r\nEverybody dog food<br>\r\n</span>\r\n<span data-start="127">\r\nBlack male, black mail<br>\r\nThrow your brother in jail\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="130">\r\nAll I wanna say is that<br>\r\nThey don\'t really care about us<br>\r\n</span>\r\n<span data-start="134">\r\nAll I wanna say is that<br>\r\nThey don\'t really care about us\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="141">\r\nTell me what has become of my rights<br>\r\nAm I invisible because you ignore me?<br>\r\n</span>\r\n<span data-start="146">\r\nYour proclamation promised me free liberty, now<br>\r\nI\'m tired of bein\' the victim of shame<br>\r\n</span>\r\n<span data-start="154">\r\nThey\'re throwing me in a class with a bad name<br>\r\nI can\'t believe this is the land from which I came<br>\r\n</span>\r\n<span data-start="160">\r\nYou know I really do hate to say it<br>\r\nThe government don\'t wanna see<br>\r\n</span>\r\n<span data-start="166">\r\nBut if Roosevelt was livin\'<br>\r\nHe wouldn\'t let this be, no, no\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="172">\r\nSkin head, dead head<br>\r\nEverybody gone bad<br>\r\n</span>\r\n<span data-start="175">\r\nSituation, speculation<br>\r\nEverybody litigation<br>\r\n</span>\r\n<span data-start="178">\r\nBeat me, bash me<br>\r\nYou can never trash me<br>\r\n</span>\r\n<span data-start="181">\r\nHit me, kick me<br>\r\nYou can never get me\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="183">\r\nAll I wanna say is that<br>\r\nThey don\'t really care about us<br>\r\n</span>\r\n<span data-start="188" data-stop="192">\r\nAll I wanna say is that<br>\r\nThey don\'t really care about us\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="215">\r\nSome things in life they just don\'t wanna see<br>\r\nBut if Martin Luther was livin\'<br>\r\n</span>\r\n<span data-start="223">\r\nHe wouldn\'t let this be, no, no\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="226">\r\nSkin head, dead head<br>\r\nEverybody gone bad<br>\r\n</span>\r\n<span data-start="229">\r\nSituation, segregation<br>\r\nEverybody allegation<br>\r\n</span>\r\n<span data-start="231">\r\nIn the suite, on the news<br>\r\nEverybody dog food<br>\r\n</span>\r\n<span data-start="234">\r\nKick me, kike me<br>\r\nDon\'t you wrong or right me\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="236">\r\nAll I wanna say is that<br>\r\nThey don\'t really care about us<br>\r\n</span>\r\n<span data-start="241">\r\nAll I wanna say is that<br>\r\nThey don\'t really care about us\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="246">\r\nAll I wanna say is that<br>\r\nThey don\'t really care about us<br>\r\n</span>\r\n<span data-start="252">\r\nAll I wanna say is that<br>\r\nThey don\'t really care about<br>\r\n</span>\r\n<span data-start="256">\r\nAll I wanna say is that<br>\r\nthey don\'t really care about<br>\r\n</span>\r\n<span data-start="258" data-stop="265">\r\nAll I wanna say is that<br>\r\nThey don\'t really care about us\r\n</span>\r\n</p></div>\r\n\t\t\t\t\t\t</div>\r\n\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\r\n\r\n\r\n\t\t\t\t\t<span><a href="http://www.minelyrics.com/michael-jackson-they-dont-care-about-us.html">Michael Jackson - They Don\'t Care About Us Lyrics</a></span><br></div>' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (28 , '' , NULL , NULL , 'Pink - Try' , 'P' , NULL , '' , NULL , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 0 , NULL , NULL , 'Online Lyrics' , 'http://www.minelyrics.com/' , NULL , NULL , 0 , NULL , NULL , '24.11.2015 2:27:04' , NULL , NULL , NULL , '<div class="article-desc">\r\n\t\t\t\t\t\t<div id="youtube-lyrics" data-video-id="yTCDVfMz15M">\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t<div class="desc desc-full"><p>\r\n<span data-start="14">\r\nOh, oh, oh, ohhhh\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="22">\r\nEver wonder about what he\'s doing?<br>\r\nHow it all turned to lies?<br>\r\n</span>\r\n<span data-start="30" data-stop="40">\r\nSometimes I think that it\'s better to never ask why\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="49">\r\nWhere there is desire<br>\r\nThere is gonna be a flame<br>\r\n</span>\r\n<span data-start="54">\r\nWhere there is a flame<br>\r\nSomeone\'s bound to get burned<br>\r\n</span>\r\n<span data-start="59">\r\nBut just because it burns<br>\r\nDoesn\'t mean you\'re gonna die<br>\r\n</span>\r\n<span data-start="63">\r\nYou\'ve gotta get up and try, try, try<br>\r\nGotta get up and try, try, try<br>\r\n</span>\r\n<span data-start="73" data-stop="77">\r\nGotta get up and try, try, try\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="82">\r\nEh, eh, eh\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="85">\r\nFunny how the heart can be deceiving<br>\r\nMore than just a couple times<br>\r\n</span>\r\n<span data-start="93" data-stop="102">\r\nWhy do we fall in love so easy?<br>\r\nEven when it\'s not right\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="108">\r\nEh, eh, eh, eh, ohhh\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="112">\r\nWhere there is desire<br>\r\nThere is gonna be a flame<br>\r\n</span>\r\n<span data-start="116">\r\nWhere there is a flame<br>\r\nSomeone\'s bound to get burned<br>\r\n</span>\r\n<span data-start="121">\r\nBut just because it burns<br>\r\nDoesn\'t mean you\'re gonna die<br>\r\n</span>\r\n<span data-start="125">\r\nYou\'ve gotta get up and try, try, try<br>\r\nGotta get up and try, try, try<br>\r\n</span>\r\n<span data-start="135" data-stop="140">\r\nYou gotta get up and try, try, try\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="147">\r\nEver worry that it might be ruined<br>\r\nAnd does it make you wanna cry?<br>\r\n</span>\r\n<span data-start="156">\r\nWhen you\'re out there doing what you\'re doing<br>\r\nAre you just getting by?<br>\r\n</span>\r\n<span data-start="164">\r\nTell me are you just getting by, by, by?\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="170">\r\nWhere there is desire<br>\r\nThere is gonna be a flame<br>\r\n</span>\r\n<span data-start="174">\r\nWhere there is a flame<br>\r\nSomeone\'s bound to get burned<br>\r\n</span>\r\n<span data-start="179">\r\nBut just because it burns<br>\r\nDoesn\'t mean you\'re gonna die<br>\r\n</span>\r\n<span data-start="183">\r\nYou\'ve gotta get up and try, try, try<br>\r\nGotta get up and try, try, try<br>\r\n</span>\r\n<span data-start="192">\r\nYou gotta get up and try, try, try<br>\r\nGotta get up and try, try, try<br>\r\n</span>\r\n<span data-start="201">\r\nGotta get up and try, try, try<br>\r\nYou gotta get up and try, try, try<br>\r\n</span>\r\n<span data-start="211" data-stop="216">\r\nGotta get up and try, try, try\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="222" data-stop="233">\r\nYou gotta get up and try, try, try<br>\r\nGotta get up and try, try, try\r\n</span>\r\n</p></div>\r\n\t\t\t\t\t\t</div>\r\n\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\r\n\r\n\r\n\t\t\t\t\t<span><a href="http://www.minelyrics.com/pink-try.html">Pink - Try Lyrics</a></span><br></div>' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (29 , '' , NULL , NULL , 'Queen - We Are The Champions' , 'Q' , NULL , '' , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 0 , NULL , NULL , 'Rock Lyrics' , 'http://www.minelyrics.com/' , NULL , NULL , 0 , NULL , NULL , '24.11.2015 2:29:18' , NULL , NULL , NULL , '<div class="article-desc">\r\n\t\t\t\t\t\t<div id="youtube-lyrics" data-video-id="04854XqcfCY">\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t<div class="desc desc-full"><p>\r\n<span data-start="4">\r\nI\'ve paid my dues<br>\r\nTime after time.<br>\r\n</span>\r\n<span data-start="11">\r\nI\'ve done my sentence<br>\r\nBut committed no crime.<br>\r\n</span>\r\n<span data-start="19">\r\nAnd bad mistakes <br>\r\nI\'ve made a few.<br>\r\n</span>\r\n<span data-start="26">\r\nI\'ve had my share of sand kicked in my face<br>\r\nBut I\'ve come through.\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="34">\r\n<i>(And I need just go on and on, and on, and on)</i>\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="38">\r\nWe are the champions, my friends,<br>\r\nAnd we\'ll keep on fighting \'til the end.<br>\r\n</span>\r\n<span data-start="52">\r\nWe are the champions.<br>\r\nWe are the champions.<br>\r\n</span>\r\n<span data-start="60" data-stop="75">\r\nNo time for losers<br>\r\nCause we are the champions of the world.\r\n</span>\r\n</p>\r\n\r\n<p>\r\n\r\n<span data-start="80">\r\nI\'ve taken my bows<br>\r\nAnd my curtain calls<br>\r\n</span>\r\n<span data-start="87">\r\nYou brought me fame and fortune and everything that goes with it<br>\r\nI thank you all\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="95">\r\nBut it\'s been no bed of roses,<br>\r\nNo pleasure cruise.<br>\r\n</span>\r\n<span data-start="103">\r\nI consider it a challenge before the whole human race<br>\r\nAnd I ain\'t gonna lose.\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="109">\r\n<i>(And I need just go on and on, and on, and on)</i>\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="113">\r\nWe are the champions, my friends,<br>\r\nAnd we\'ll keep on fighting \'til the end.<br>\r\n</span>\r\n<span data-start="128">\r\nWe are the champions.<br>\r\nWe are the champions.<br>\r\n</span>\r\n<span data-start="136">\r\nNo time for losers<br>\r\nCause we are the champions of the world.\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="147">\r\nWe are the champions, my friends,<br>\r\nAnd we\'ll keep on fighting \'til the end.<br>\r\n</span>\r\n<span data-start="159">\r\nWe are the champions.<br>\r\nWe are the champions.<br>\r\n</span>\r\n<span data-start="168" data-stop="182">\r\nNo time for losers<br>\r\nCause we are the champions. <br></span></p><p><span data-start="168" data-stop="182"><span><a href="http://www.minelyrics.com/queen-we-are-the-champions.html">Queen - We Are The Champions Lyrics</a></span><br></span>\r\n</p></div>\r\n\t\t\t\t\t\t</div>\r\n\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\r\n\r\n\r\n\t\t\t\t\t</div>' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (30 , '' , NULL , NULL , 'Rihanna - Stay' , 'R' , NULL , '' , NULL , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 0 , NULL , NULL , 'Online Lyrics' , 'http://www.minelyrics.com/' , NULL , NULL , 0 , NULL , NULL , '24.11.2015 2:33:43' , NULL , NULL , NULL , '<div class="article-desc">\r\n\t\t\t\t\t\t<div id="youtube-lyrics" data-video-id="JF8BRvqGCNs">\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t<div class="desc desc-full"><p>\r\n<i>[Rihanna]</i><br>\r\n<span data-start="17" data-stop="21">\r\nAll along it was a fever<br>\r\n</span>\r\n<span data-start="24" data-stop="29">\r\nA cold sweat hot-headed believer<br>\r\n</span>\r\n<span data-start="32" data-stop="38">\r\nI threw my hands in the air, said, "Show me something"<br>\r\n</span>\r\n<span data-start="41" data-stop="47">\r\nHe said, "If you dare, come a little closer"\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="50" data-stop="56">\r\nRound and around and around and around we go<br>\r\n</span>\r\n<span data-start="58" data-stop="65">\r\nOh now, tell me now, tell me now, tell me now you know.\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="68">\r\nNot really sure how to feel about it.<br>\r\nSomething in the way you move<br>\r\n</span>\r\n<span data-start="76">\r\nMakes me feel like I can\'t live without you.<br>\r\nIt takes me all the way.<br>\r\n</span>\r\n<span data-start="84" data-stop="90">\r\nI want you to stay\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<i>[Mikky]</i><br>\r\n<span data-start="94" data-stop="98">\r\nIt\'s not much of a life you\'re living<br>\r\n</span>\r\n<span data-start="101" data-stop="107">\r\nIt\'s not just something you take, it\'s given\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="110" data-stop="116">\r\nRound and around and around and around we go<br>\r\n</span>\r\n<span data-start="118" data-stop="125">\r\nOh now, tell me now, tell me now, tell me now you know.\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="128">\r\nNot really sure how to feel about it.<br>\r\nSomething in the way you move<br>\r\n</span>\r\n<span data-start="136">\r\nMakes me feel like I can\'t live without you.<br>\r\nYeah, it takes me all the way.<br>\r\n</span>\r\n<span data-start="144" data-stop="150">\r\nI want you to stay.\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<i>[RIhanna &amp; Mikky]</i><br>\r\n<span data-start="153" data-stop="160">\r\nOoh, ooh, ooh, the reason I hold on<br>\r\n</span>\r\n<span data-start="162" data-stop="169">\r\nOoh, ooh, ooh, \'cause I need this hole gone<br>\r\n</span>\r\n<span data-start="170" data-stop="178">\r\nFunny you\'re the broken one but I\'m the only one who needed saving<br>\r\n</span>\r\n<span data-start="179" data-stop="186">\r\n\'Cause when you never see the light it\'s hard to know which one of us is caving.\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<i>[Rihanna]</i><br>\r\n<span data-start="188">\r\nNot really sure how to feel about it.<br>\r\nSomething in the way you move<br>\r\n</span>\r\n<span data-start="196">\r\nMakes me feel like I can\'t live without you.<br>\r\nYeah, it takes me all the way.<br>\r\n</span>\r\n<span data-start="204" data-stop="233">\r\nI want you to sta-a-ay, sta-a-ay.<br>\r\nI want you to stay, oooh. <br></span></p><p><span data-start="204" data-stop="233"><span><a href="http://www.minelyrics.com/rihanna-stay.html">Rihanna - Stay Lyrics</a></span><br></span>\r\n</p></div>\r\n\t\t\t\t\t\t</div>\r\n\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\r\n\r\n\r\n\t\t\t\t\t</div>' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (31 , '' , NULL , NULL , 'Stevie Wonder - I Just Called to Say I Love You' , 'S' , NULL , '' , NULL , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 0 , NULL , NULL , 'Retro Lyrics' , 'http://www.minelyrics.com/' , NULL , NULL , 0 , NULL , NULL , '24.11.2015 2:37:18' , NULL , NULL , NULL , '<p>\r\n<span data-start="7" data-stop="23">\r\nNo New Year\'s Day to celebrate<br>\r\nNo chocolate covered candy hearts to give away<br>\r\n</span>\r\n<span data-start="24" data-stop="31.5">\r\nNo first of spring<br>\r\nNo song to sing<br>\r\n</span>\r\n<span data-start="32" data-stop="40">\r\nIn fact here\'s just another ordinary day\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="41" data-stop="48">\r\nNo April rain<br>\r\nNo flowers bloom<br>\r\n</span>\r\n<span data-start="49" data-stop="65">\r\nNo wedding Saturday within the month of June<br>\r\nBut what it is, is something true<br>\r\n</span>\r\n<span data-start="66" data-stop="74">\r\nMade up of these three words that I must say to you\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="75" data-stop="90">\r\nI just called to say - I love you<br>\r\nI just called to say how much I care<br>\r\n</span>\r\n<span data-start="92" data-stop="107">\r\nI just called to say - I love you<br>\r\nAnd I mean it from the bottom of my heart\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="108" data-stop="116">\r\nNo summer\'s high<br>\r\nNo warm July<br>\r\n</span>\r\n<span data-start="116">\r\nNo harvest moon to light one tender August night<br>\r\nNo autumn breeze<br>\r\n</span>\r\n<span data-start="130" data-stop="141">\r\nNo falling leaves<br>\r\nNot even time for birds to fly to southern skies\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="142">\r\nNo Libra sun<br>\r\nNo Halloween<br>\r\n</span>\r\n<span data-start="150" data-stop="166">\r\nNo giving thanks to all the Christmas joy you bring<br>\r\nBut what it is, though old so new<br>\r\n</span>\r\n<span data-start="167" data-stop="175">\r\nTo fill your heart like no three words could ever do\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="176">\r\nI just called to say - I love you<br>\r\nI just called to say how much I care, I do<br>\r\n</span>\r\n<span data-start="193" data-stop="209">\r\nI just called to say - I love you<br>\r\nAnd I mean it from the bottom of my heart\r\n</span>\r\n</p>\r\n\r\n\r\n<span data-start="210">\r\nI just called to say - I love you<br>\r\nI just called to say how much I care, I do<br>\r\n</span>\r\n<span data-start="227" data-stop="255">\r\nI just called to say - I love you<br>\r\nAnd I mean it from the bottom of my heart, of my heart, of my heart<br><br><span><a href="http://www.minelyrics.com/stevie-wonder-i-just-called-to-say-i-love-you.html">Stevie Wonder - I Just Called to Say I Love You Lyrics</a></span><br></span>' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (32 , '' , NULL , NULL , 'The Beatles - Let It Be' , 'T' , NULL , '' , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 0 , NULL , NULL , 'All Lyrics' , 'http://www.minelyrics.com/' , NULL , NULL , 0 , NULL , NULL , '24.11.2015 2:39:37' , NULL , NULL , NULL , '<div class="article-desc">\r\n\t\t\t\t\t\t<div id="youtube-lyrics" data-video-id="ajCYQL8ouqw">\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t<div class="desc desc-full"><p>\r\n<span data-start="12">\r\nWhen I find myself in times of trouble<br>\r\nMother Mary comes to me<br>\r\n</span>\r\n<span data-start="19">\r\nSpeaking words of wisdom, let it be<br>\r\nAnd in my hour of darkness<br>\r\n</span>\r\n<span data-start="29">\r\nShe is standing right in front of me<br>\r\nSpeaking words of wisdom, let it be\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="38">\r\nLet it be, let it be<br>\r\nLet it be, let it be<br>\r\n</span>\r\n<span data-start="45">\r\nWhisper words of wisdom, let it be\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="52">\r\nAnd when the broken-hearted people<br>\r\nLiving in the world agree<br>\r\n</span>\r\n<span data-start="58">\r\nThere will be an answer, let it be<br>\r\nFor though they may be parted<br>\r\n</span>\r\n<span data-start="67">\r\nThere is still a chance that they will see<br>\r\nThere will be an answer, let it be\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="77">\r\nLet it be, let it be<br>\r\nLet it be, let it be<br>\r\n</span>\r\n<span data-start="85">\r\nYeah, there will be an answer, let it be<br>\r\nLet it be, let it be<br>\r\n</span>\r\n<span data-start="94" data-stop="104">\r\nLet it be, let it be<br>\r\nWhisper words of wisdom, let it be\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="145">\r\nLet it be, let it be<br>\r\nAh, let it be, yeah, let it be<br>\r\n</span>\r\n<span data-start="153">\r\nWhisper words of wisdom, let it be\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="160">\r\nAnd when the night is cloudy<br>\r\nThere is still a light that shines on me<br>\r\n</span>\r\n<span data-start="168">\r\nShine on until tomorrow, let it be<br>\r\nI wake up to the sound of music,<br>\r\n</span>\r\n<span data-start="178">\r\nMother Mary comes to me<br>\r\nSpeaking words of wisdom, let it be\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="188">\r\nLet it be, let it be<br>\r\nLet it be, yeah, let it be<br>\r\n</span>\r\n<span data-start="195">\r\nOh, there will be an answer, let it be<br>\r\nLet it be, let it be<br>\r\n</span>\r\n<span data-start="205" data-stop="217">\r\nLet it be, yeah, let it be<br>\r\nWhisper words of wisdom, let it be <br></span></p><p><span data-start="205" data-stop="217"><span><a href="http://www.minelyrics.com/the-beatles-let-it-be.html">The Beatles - Let It Be Lyrics</a></span><br></span>\r\n</p></div>\r\n\t\t\t\t\t\t</div>\r\n\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\r\n\r\n\r\n\t\t\t\t\t</div>' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (33 , '' , NULL , NULL , 'Toni Braxton - Un-Break My Heart' , 'T' , NULL , '' , NULL , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 0 , NULL , NULL , 'Youtube Lyrics' , 'http://www.minelyrics.com/' , NULL , NULL , 0 , NULL , NULL , '24.11.2015 2:46:04' , NULL , NULL , NULL , '<div class="article-desc">\r\n\t\t\t\t\t\t<div id="youtube-lyrics" data-video-id="p2Rch6WvPJE">\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t<div class="desc desc-full"><p>\r\n<span data-start="20">\r\nDon\'t leave me in all this pain<br>\r\nDon\'t leave me out in the rain<br>\r\n</span>\r\n<span data-start="29">\r\nCome back and bring back my smile<br>\r\nCome and take these tears away<br>\r\n</span>\r\n<span data-start="36">\r\nI need your arms to hold me now<br>\r\nThe nights are so unkind<br>\r\n</span>\r\n<span data-start="45">\r\nBring back those nights when I held you beside me\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="52">\r\nUn-break my heart<br>\r\nSay you\'ll love me again<br>\r\n</span>\r\n<span data-start="60">\r\nUndo this hurt you caused<br>\r\nWhen you walked out the door<br>\r\n</span>\r\n<span data-start="66">\r\nAnd walked out of my life<br>\r\nUn-cry these tears<br>\r\n</span>\r\n<span data-start="73">\r\nI cried so many nights<br>\r\nUn-break my heart<br>\r\nMy heart\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="90">\r\nTake back that sad word good-bye<br>\r\nBring back the joy to my life<br>\r\n</span>\r\n<span data-start="99">\r\nDon\'t leave me here with these tears<br>\r\nCome and kiss this pain away<br>\r\n</span>\r\n<span data-start="106">\r\nI can\'t forget the day you left<br>\r\nTime is so unkind<br>\r\n</span>\r\n<span data-start="115">\r\nAnd life is so cruel without you here beside me\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="122">\r\nUn-break my heart<br>\r\nSay you\'ll love me again<br>\r\n</span>\r\n<span data-start="130">\r\nUndo this hurt you caused<br>\r\nWhen you walked out the door<br>\r\n</span>\r\n<span data-start="136">\r\nAnd walked out of my life<br>\r\nUn-cry these tears<br>\r\n</span>\r\n<span data-start="143" data-stop="161">\r\nI cried so many nights<br>\r\nUn-break my heart<br>\r\nMy heart\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="175">\r\nDon\'t leave me in all this pain<br>\r\nDon\'t leave me out in the rain<br>\r\n</span>\r\n<span data-start="183">\r\nBring back the nights when I held you beside me\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="189">\r\nUn-break my heart<br>\r\nSay you\'ll love me again<br>\r\n</span>\r\n<span data-start="198">\r\nUndo this hurt you caused<br>\r\nWhen you walked out the door<br>\r\n</span>\r\n<span data-start="204">\r\nAnd walked out of my life<br>\r\nUn-cry this tears<br>\r\n</span>\r\n<span data-start="210">\r\nI cried so many, many nights<br>\r\nUn-break my\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="218">\r\nUn-break my heart oh baby<br>\r\nCome back and say you love me<br>\r\n</span>\r\n<span data-start="227">\r\nUn-break my heart<br>\r\nSweet darlin\'<br>\r\n</span>\r\n<span data-start="231">\r\nWithout you I just can\'t go on<br>\r\nCan\'t go on.... <br></span></p><p><span data-start="231"><span><a href="http://www.minelyrics.com/toni-braxton-un-break-my-heart.html">Toni Braxton - Un-Break My Heart Lyrics</a></span><br></span>\r\n</p></div>\r\n\t\t\t\t\t\t</div>\r\n\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\r\n\r\n\r\n\t\t\t\t\t</div>' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (34 , '' , NULL , NULL , 'Whitney Houston - I Will Always Love You' , 'W' , NULL , '' , NULL , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 0 , NULL , NULL , 'Karaoke Lyrics' , 'http://www.minelyrics.com/' , NULL , NULL , 0 , NULL , NULL , '24.11.2015 2:48:53' , NULL , NULL , NULL , '<span data-start="0">If I should stay<br>\r\nI would only be in your way<br>\r\n</span>\r\n<span data-start="20">\r\nSo I\'ll go but I know<br>\r\nI\'ll think of you every step of the way<br>\r\n</span>\r\n<br>\r\n<span data-start="43">\r\nAnd I... will always love you, ooh<br>\r\nWill always love you<br>\r\nYou<br>\r\n</span>\r\n<span data-start="70">\r\nMy darling, you...<br>\r\nMmm-mm<br>\r\n</span>\r\n<br>\r\n<span data-start="78">\r\nBittersweet memories<br>\r\nThat is all I\'m taking with me.<br>\r\n</span>\r\n<span data-start="92">\r\nSo good-bye.<br>\r\nPlease don\'t cry:<br>\r\nWe both know I\'m not what you, you need<br>\r\n</span>\r\n<br>\r\n<span data-start="106" data-stop="130">\r\nAnd I... will always love you<br>\r\nI... will always love you<br>\r\nYou, ooh<br>\r\n</span>\r\n<br>\r\n<span data-start="156">\r\nI hope life treats you kind<br>\r\nAnd I hope you have all you\'ve dreamed of<br>\r\n</span>\r\n<span data-start="171">\r\nAnd I wish you joy and happiness<br>\r\nBut above all this I wish you love<br>\r\n</span>\r\n<br>\r\n<span data-start="189">\r\nAnd I... will always love you<br>\r\nI will always love you<br>\r\n</span>\r\n<span data-start="207" data-stop="221">\r\nI will always love you<br>\r\nI will always love you<br>\r\n</span>\r\n<br>\r\n<span data-start="223" data-stop="244">\r\nI will always love you<br>\r\nI, I will always love you.<br>\r\n</span>\r\n<br>\r\n<span data-start="245">\r\nYou.<br>\r\nDarling, I love you.<br>\r\nI\'ll always...<br>\r\n</span>\r\n<span data-start="253">\r\nI\'ll always love you.<br>\r\nOoh<br>\r\nOoh<br><br><span><a href="http://www.minelyrics.com/whitney-houston-i-will-always-love-you.html">Whitney Houston - I Will Always Love You Lyrics</a></span><br></span>' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (35 , '' , NULL , NULL , 'Zedd - Clarity' , 'Z' , NULL , '' , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 0 , NULL , NULL , 'Best Lyrics' , 'http://www.minelyrics.com/' , NULL , NULL , 0 , NULL , NULL , '24.11.2015 2:52:43' , NULL , NULL , NULL , '<div class="article-desc">\r\n\t\t\t\t\t\t<div id="youtube-lyrics" data-video-id="IxxstCcJlsc">\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t<div class="desc desc-full"><p>\r\nHigh dive into frozen waves where the past comes back to life<br>\r\nFight fear for the selfish pain, it was worth it every time<br>\r\nHold still right before we crash \'cause we both know how this ends<br>\r\nA clock ticks \'til it breaks your glass and I drown in you again\r\n</p>\r\n\r\n<p>\r\n\'Cause you are the piece of me I wish I didn\'t need<br>\r\nChasing relentlessly, still fight and I don\'t know why\r\n</p>\r\n\r\n<p>\r\nIf our love is tragedy, why are you my remedy?<br>\r\nIf our love\'s insanity, why are you my clarity?\r\n</p>\r\n\r\n<p>\r\n(Hey-ay, hey-ay-ay. Hey-ay, hey-ay-ay. Hey-ay, hey-ay-ay. Hey-ay, hey)\r\n</p>\r\n\r\n<p>\r\nIf our love is tragedy, why are you my remedy?<br>\r\nIf our love\'s insanity, why are you my clarity?\r\n</p>\r\n\r\n<p>\r\nWalk on through a red parade and refuse to make amends<br>\r\nIt cuts deep through our ground and makes us forget all common sense<br>\r\nDon\'t speak as I try to leave \'cause we both know what we\'ll choose<br>\r\nIf you pull then I\'ll push too deep and I\'ll fall right back to you\r\n</p>\r\n\r\n<p>\r\n\'Cause you are the piece of me I wish I didn\'t need<br>\r\nChasing relentlessly, still fight and I don\'t know why\r\n</p>\r\n\r\n<p>\r\nIf our love is tragedy, why are you my remedy?<br>\r\nIf our love\'s insanity, why are you my clarity?\r\n</p>\r\n\r\n<p>\r\n(Hey-ay, hey-ay-ay. Hey-ay, hey-ay-ay. Hey-ay, hey-ay-ay. Hey-ay, hey)\r\n</p>\r\n\r\n<p>\r\nWhy are you my clarity?<br>\r\nWhy are you my remedy?<br>\r\nWhy are you my clarity?<br>\r\nWhy are you my remedy?\r\n</p>\r\n\r\n<p>\r\nIf our love is tragedy, why are you my remedy?<br>\r\nIf our love\'s insanity, why are you my clarity?\r\n</p></div>\r\n\t\t\t\t\t\t</div>\r\n\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\r\n\r\n\r\n\t\t\t\t\t<span><a href="http://www.minelyrics.com/zedd-clarity.html">Zedd - Clarity Lyrics</a></span><br></div>' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (36 , '' , NULL , NULL , '2Pac - Dear Mama' , '2' , NULL , '' , NULL , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 0 , NULL , NULL , 'Lyrics Hits' , 'http://www.minelyrics.com/' , NULL , NULL , 0 , NULL , NULL , '24.11.2015 2:55:36' , NULL , NULL , NULL , '<div class="article-desc">\r\n\t\t\t\t\t\t<div id="youtube-lyrics" data-video-id="Mb1ZvUDvLDY">\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t<div class="desc desc-full"><p>\r\n<span data-start="5" data-stop="7">\r\n<i>You are appreciated</i>\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="11">\r\nWhen I was young me and my mama had beef<br>\r\nSeventeen years old kicked out on the streets<br>\r\n</span>\r\n<span data-start="16">\r\nThough back at the time, I never thought I\'d see her face<br>\r\nAin\'t a woman alive that could take my mama\'s place<br>\r\n</span>\r\n<span data-start="22">\r\nSuspended from school and scared to go home, I was a fool<br>\r\nwith the big boys, breaking all the rules<br>\r\n</span>\r\n<span data-start="27">\r\nI shed tears with my baby sister<br>\r\nOver the years we was poorer than the other little kids<br>\r\n</span>\r\n<span data-start="33">\r\nAnd even though we had different daddy\'s, the same drama<br>\r\nWhen things went wrong we\'d blame mama<br>\r\n</span>\r\n<span data-start="39">\r\nI reminisce on the stress I caused, it was hell<br>\r\nHuggin on my mama from a jail cell<br>\r\n</span>\r\n<span data-start="45">\r\nAnd who\'d think in elementary?<br>\r\nHeeey! I see the penitentiary, one day<br>\r\n</span>\r\n<span data-start="51">\r\nAnd running from the police, that\'s right<br>\r\nMama catch me, put a whoopin to my backside<br>\r\n</span>\r\n<span data-start="57">\r\nAnd even as a crack fiend, mama<br>\r\nYou always was a black queen, mama<br>\r\n</span>\r\n<span data-start="63">\r\nI finally understand<br>\r\nfor a woman it ain\'t easy tryin to raise a man<br>\r\n</span>\r\n<span data-start="68">\r\nYou always was committed<br>\r\nA poor single mother on welfare, tell me how ya did it<br>\r\n</span>\r\n<span data-start="73">\r\nThere\'s no way I can pay you back<br>\r\nBut the plan is to show you that I understand<br>\r\n</span>\r\n<span data-start="78">\r\nYou are appreciated\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="80">\r\n<i>Lady, don\'t cha know we love ya? </i><br>\r\n</span>\r\n<span data-start="85">\r\nDear mama<br>\r\n<i>Sweet lady, place no one above ya</i><br>\r\n</span>\r\n<span data-start="90">\r\nYou are appreciated<br>\r\n<i>Sweet lady, don\'t cha know we love ya?</i>\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="97">\r\nNow ain\'t nobody tell us it was fair<br>\r\nNo love from my daddy cause the coward wasn\'t there<br>\r\n</span>\r\n<span data-start="102">\r\nHe passed away and I didn\'t cry, cause my anger<br>\r\nwouldn\'t let me feel for a stranger<br>\r\n</span>\r\n<span data-start="107">\r\nThey say I\'m wrong and I\'m heartless, but all along<br>\r\nI was lookin for a father he was gone<br>\r\n</span>\r\n<span data-start="113">\r\nI hung around with the Thugs, and even though they sold drugs<br>\r\nThey showed a young brother love<br>\r\n</span>\r\n<span data-start="119">\r\nI moved out and started really hangin<br>\r\nI needed money of my own so I started slangin<br>\r\n</span>\r\n<span data-start="124">\r\nI ain\'t guilty cause, even though I sell rocks<br>\r\nIt feels good puttin money in your mailbox<br>\r\n</span>\r\n<span data-start="129">\r\nI love payin rent when the rent\'s due<br>\r\nI hope ya got the diamond necklace that I sent to you<br>\r\n</span>\r\n<span data-start="136">\r\nCause when I was low you was there for me<br>\r\nAnd never left me alone because you cared for me<br>\r\n</span>\r\n<span data-start="141">\r\nAnd I could see you comin home after work late<br>\r\nYou\'re in the kitchen tryin to fix us a hot plate<br>\r\n</span>\r\n<span data-start="147">\r\nYa just workin with the scraps you was given<br>\r\nAnd mama made miracles every Thanksgivin<br>\r\n</span>\r\n<span data-start="153">\r\nBut now the road got rough, you\'re alone<br>\r\nYou\'re tryin to raise two bad kids on your own<br>\r\n</span>\r\n<span data-start="159">\r\nAnd there\'s no way I can pay you back<br>\r\nBut my plan is to show you that I understand<br>\r\n</span>\r\n<span data-start="164">\r\nYou are appreciated\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="166">\r\n<i>Lady, don\'t cha know we love ya? </i><br>\r\n</span>\r\n<span data-start="170">\r\nDear mama<br>\r\n<i>Sweet lady, place no one above ya</i><br>\r\n</span>\r\n<span data-start="175">\r\n<i>You are appreciated</i><br>\r\nSweet lady, don\'t cha know we love ya?\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="182">\r\nPour out some liquor and I reminisce, cause through the drama<br>\r\nI can always depend on my mama<br>\r\n</span>\r\n<span data-start="187">\r\nAnd when it seems that I\'m hopeless<br>\r\nYou say the words that can get me back in focus<br>\r\n</span>\r\n<span data-start="192">\r\nWhen I was sick as a little kid<br>\r\nTo keep me happy there\'s no limit to the things you did<br>\r\n</span>\r\n<span data-start="198">\r\nAnd all my childhood memories<br>\r\nAre full of all the sweet things you did for me<br>\r\n</span>\r\n<span data-start="204">\r\nAnd even though I act craaazy<br>\r\nI gotta thank the Lord that you made me<br>\r\n</span>\r\n<span data-start="210">\r\nThere are no words that can express how I feel<br>\r\nYou never kept a secret, always stayed real<br>\r\n</span>\r\n<span data-start="216">\r\nAnd I appreciate, how you raised me<br>\r\nAnd all the extra love that you gave me<br>\r\n</span>\r\n<span data-start="221">\r\nI wish I could take the pain away<br>\r\nIf you can make it through the night there\'s a brighter day<br>\r\n</span>\r\n<span data-start="227">\r\nEverything will be alright if ya hold on<br>\r\nIt\'s a struggle everyday, gotta roll on<br>\r\n</span>\r\n<span data-start="233">\r\nAnd there\'s no way I can pay you back<br>\r\nBut my plan is to show you that I understand<br>\r\n</span>\r\n<span data-start="238">\r\nYou are appreciated\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="240">\r\n<i>Lady, don\'t cha know we love ya? </i><br>\r\n</span>\r\n<span data-start="244">\r\nDear mama<br>\r\n<i>Sweet lady, place no one above ya</i><br>\r\n</span>\r\n<span data-start="249">\r\nYou are appreciated<br>\r\n<i>Sweet lady, don\'t cha know we love ya?</i>\r\n</span>\r\n</p>\r\n\r\n\r\n<p>\r\n<span data-start="255">\r\n<i>Sweet lady</i><br>\r\nAnd dear mama<br>\r\n</span>\r\n<span data-start="261" data-stop="275">\r\nDear mama<br>\r\n<i>Lady, Lady, Lady.</i>\r\n</span>\r\n</p></div>\r\n\t\t\t\t\t\t</div>\r\n\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\r\n\r\n\r\n\t\t\t\t\t<span><a href="http://www.minelyrics.com/2pac-dear-mama.html">2Pac - Dear Mama Lyrics</a></span><br></div>' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 ) ;
INSERT INTO va_articles (article_id,language_code,article_date,date_end,article_title,title_first,article_comment,friendly_url,article_template,article_order,author_name,author_email,author_url,author_remote_address,created_user_id,created_admin_id,updated_user_id,updated_admin_id,status_id,is_html,allowed_rate,is_hot,hot_order,hot_description,link_title,link_url,download_url,is_link_direct,is_remote_rss,details_remote_url,date_added,date_updated,keywords,highlights,short_description,full_description,notes,total_views,total_votes,total_points,rating,total_clicks,image_tiny,image_tiny_alt,image_small,image_small_alt,image_large,image_large_alt,image_super,image_super_alt,stream_video,stream_video_width,stream_video_height,stream_video_preview,youtube_video,youtube_video_width,youtube_video_height,meta_title,meta_keywords,meta_description,is_draft,draft_parent_id) VALUES (37 , '' , NULL , NULL , '3 Doors Down - Here Without You' , '3' , NULL , '' , NULL , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 0 , NULL , NULL , 'Sing Lyrics' , 'http://www.minelyrics.com/' , NULL , NULL , 0 , NULL , NULL , '24.11.2015 2:57:41' , NULL , NULL , NULL , '<div class="article-desc">\r\n\t\t\t\t\t\t<div id="youtube-lyrics" data-video-id="kPBzTxZQG5Q">\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t<div class="desc desc-full"><p>\r\n<span data-start="5" data-stop="14">\r\nA hundred days have made me older<br>\r\nSince the last time that I saw your pretty face<br>\r\n</span>\r\n<span data-start="18.5" data-stop="28">\r\nA thousand lies have made me colder<br>\r\nAnd I don\'t think I can look at this the same\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="33.5" data-stop="46.5">\r\nBut all the miles that separate<br>\r\nDisappear now when I\'m dreaming of your face\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="47">\r\nI\'m here without you, baby<br>\r\nBut you\'re still on my lonely mind<br>\r\n</span>\r\n<span data-start="54">\r\nI think about you, baby<br>\r\nAnd I dream about you all the time<br>\r\n</span>\r\n<span data-start="61" data-stop="67.5">\r\nI\'m here without you, baby<br>\r\nBut you\'re still with me in my dreams<br>\r\n</span>\r\n<span data-start="69" data-stop="79">\r\nAnd tonight it\'s only you and me, yeah.\r\n</span>\r\n</p>\r\n\r\n\r\n<p>\r\n<span data-start="86" data-stop="94">\r\nThe miles just keep rolling<br>\r\nAs the people leave their way to say hello<br>\r\n</span>\r\n<span data-start="98" data-stop="113">\r\nI\'ve heard this life is overrated<br>\r\nBut I hope that it gets better as we go, oh, yeah, yeah\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="114">\r\nI\'m here without you, baby<br>\r\nBut you\'re still on my lonely mind<br>\r\n</span>\r\n<span data-start="120">\r\nI think about you, baby<br>\r\nAnd I dream about you all the time<br>\r\n</span>\r\n<span data-start="127" data-stop="134">\r\nI\'m here without you, baby<br>\r\nBut you\'re still with me in my dreams<br>\r\n</span>\r\n<span data-start="136">\r\nAnd tonight, girl, its only you and me\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="141">\r\nEverything I know, and anywhere I go (<i>oh whoa</i>)<br>\r\nIt gets hard but it won\'t take away my love (<i>oh whoa</i>)<br>\r\n</span>\r\n<span data-start="154">\r\nAnd when the last one falls<br>\r\nWhen it\'s all said and done<br>\r\n</span>\r\n<span data-start="161" data-stop="178">\r\nIt gets hard but it wont take away my love, whoa, oh, oh\r\n</span>\r\n</p>\r\n\r\n<p>\r\n<span data-start="183">\r\nI\'m here without you, baby<br>\r\nBut you\'re still on my lonely mind<br>\r\n</span>\r\n<span data-start="190">\r\nI think about you, baby<br>\r\nAnd I dream about you all the time<br>\r\n</span>\r\n<span data-start="197" data-stop="204">\r\nI\'m here without you, baby<br>\r\nBut you\'re still with me in my dreams<br>\r\n</span>\r\n<span data-start="206" data-stop="228">\r\nAnd tonight, girl, it\'s only you and me<br>\r\nYeah, oh yeah, oh, oh, oh, oh, oh...\r\n</span>\r\n</p></div>\r\n\t\t\t\t\t\t</div>\r\n\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t\r\n\r\n\r\n\r\n\t\t\t\t\t<span><a href="http://www.minelyrics.com/3-doors-down-here-without-you.html">3 Doors Down - Here Without You Lyrics</a></span><br></div>' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 ) ;




INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (1 , 1 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (1 , 15 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (1 , 16 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (1 , 17 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (1 , 18 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (1 , 19 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (1 , 20 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (1 , 21 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (1 , 22 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (1 , 23 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (1 , 24 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (1 , 25 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (1 , 26 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (1 , 27 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (1 , 28 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (1 , 29 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (2 , 1 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (2 , 18 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (2 , 19 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (2 , 21 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (2 , 23 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (2 , 29 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (3 , 1 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (3 , 20 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (3 , 28 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (4 , 1 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (4 , 16 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (4 , 23 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (5 , 5 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (6 , 5 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (7 , 5 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (8 , 5 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (9 , 7 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (10 , 7 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (11 , 7 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (12 , 7 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (13 , 8 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (14 , 8 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (15 , 8 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (16 , 10 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (17 , 10 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (18 , 10 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (19 , 6 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (20 , 6 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (21 , 6 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (22 , 4 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (23 , 4 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (24 , 4 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (25 , 4 , 0 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (26 , 11 , 1 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (27 , 11 , 2 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (28 , 11 , 3 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (29 , 12 , 1 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (30 , 12 , 2 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (31 , 12 , 3 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (32 , 13 , 1 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (33 , 13 , 2 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (34 , 13 , 3 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (35 , 14 , 1 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (36 , 14 , 2 ) ;
INSERT INTO va_articles_assigned (article_id,category_id,article_order) VALUES (37 , 14 , 3 ) ;




INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (1 , '' , 0 , 0 , '0,' , 'News' , '' , 10 , NULL , NULL , 'article_date' , 'DESC' , NULL , 'article_date,article_title,short_description,image_small' , 'article_date,article_title,full_description,image_large' , 'article_date,article_title' , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , '<br>' , '<br>' , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 4 , 4 , 15 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (3 , '' , 0 , NULL , '0,' , 'Lyrics' , '' , 7 , NULL , NULL , NULL , NULL , '{"article_title":1,"link_url":1,"full_description":1,"article_date_format":"datetime","date_end_format":"datetime"}' , 'article_title,short_description' , 'article_title,link_url,full_description' , 'article_title,full_description' , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , '<br>' , '<br>' , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 2 , 0 , 7 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (4 , '' , 0 , 0 , '0,' , 'FAQ' , '' , 9 , NULL , NULL , NULL , NULL , NULL , 'article_title,full_description,image_small' , 'article_title,full_description,image_large' , 'article_title' , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , '<br>' , '<br>' , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 4 , 0 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (5 , '' , 0 , 0 , '0,' , 'Image Gallery' , '' , 5 , NULL , NULL , NULL , NULL , NULL , 'article_date,article_title,short_description,image_small' , 'article_date,article_title,full_description,image_large' , 'article_date,article_title' , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , '<br>' , '<br>' , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 4 , 0 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (6 , '' , 0 , 0 , '0,' , 'Events' , '' , 6 , NULL , NULL , NULL , NULL , NULL , 'article_date,article_title,short_description,image_small' , 'article_date,article_title,full_description,image_large' , 'article_date,article_title' , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , '<br>' , '<br>' , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 3 , 0 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (7 , '' , 3 , 0 , '0,3,' , 'A, B, C, D' , '' , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , '<br>' , '<br>' , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 4 , 0 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (8 , '' , 3 , 0 , '0,3,' , 'E, F, G, H' , '' , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , '<br>' , '<br>' , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 3 , 0 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (10 , '' , 3 , 0 , '0,3,' , 'I, J, K, L' , '' , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , '<br>' , '<br>' , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 3 , 0 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (11 , '' , 3 , 0 , '0,3,' , 'M, N, O, P' , '' , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , '<br>' , '<br>' , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 3 , 0 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (12 , '' , 3 , 0 , '0,3,' , 'Q, R, S' , '' , 6 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , '<br>' , '<br>' , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 3 , 0 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (13 , '' , 3 , 0 , '0,3,' , 'T, U, V, W' , '' , 7 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , '<br>' , '<br>' , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 3 , 0 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (14 , '' , 3 , 0 , '0,3,' , 'X, Y, Z, 0-9' , '' , 8 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , '<br>' , '<br>' , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 3 , 0 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (15 , NULL , 1 , 0 , '0,1,' , 'Asia' , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 0 , 0 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (16 , NULL , 1 , 0 , '0,1,' , 'Europe' , NULL , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (17 , NULL , 1 , 0 , '0,1,' , 'U.S.' , NULL , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (18 , NULL , 1 , 0 , '0,1,' , 'WorldWorld' , NULL , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (19 , NULL , 1 , 0 , '0,1,' , 'Business' , NULL , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (20 , NULL , 1 , 0 , '0,1,' , 'Technology' , NULL , 6 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (21 , NULL , 1 , 0 , '0,1,' , 'Science & Space' , NULL , 7 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (22 , NULL , 1 , 0 , '0,1,' , 'Entertainment' , NULL , 8 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (23 , NULL , 1 , 0 , '0,1,' , 'World Sport' , NULL , 9 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (24 , NULL , 1 , 0 , '0,1,' , 'Travel' , NULL , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (25 , NULL , 1 , 0 , '0,1,' , 'Executive' , NULL , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (26 , NULL , 1 , 0 , '0,1,' , 'Education' , NULL , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (27 , NULL , 1 , 0 , '0,1,' , 'Weather' , NULL , 13 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (28 , NULL , 1 , 0 , '0,1,' , 'Special Reports' , NULL , 14 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (29 , NULL , 1 , 0 , '0,1,' , 'Video' , NULL , 15 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_articles_categories (category_id,language_code,parent_category_id,alias_category_id,category_path,category_name,friendly_url,category_order,articles_list_template,articles_details_template,articles_order_column,articles_order_direction,article_edit_fields,article_list_fields,article_details_fields,article_required_fields,moderators_ids,is_hot,sites_all,user_types_all,access_level,guest_access_level,admin_access_level,allowed_rate,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,is_remote_rss,remote_rss_url,remote_rss_date_updated,remote_rss_ttl,remote_rss_refresh_rate,meta_title,meta_keywords,meta_description,total_views,total_articles,total_subcategories) VALUES (30 , '' , 0 , 0 , '0,' , 'Knowledge Base' , '' , 11 , NULL , NULL , 'article_order' , 'ASC' , NULL , 'article_date,article_title,short_description' , 'article_date,article_title,full_description' , 'article_date,article_title,full_description' , NULL , 0 , 1 , 1 , 7 , 7 , 7 , 1 , '<br>' , '<br>' , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 ) ;


















INSERT INTO va_articles_related (article_id,related_id,related_order) VALUES (1 , 2 , 1 ) ;
INSERT INTO va_articles_related (article_id,related_id,related_order) VALUES (1 , 3 , 2 ) ;
INSERT INTO va_articles_related (article_id,related_id,related_order) VALUES (2 , 1 , 2 ) ;
INSERT INTO va_articles_related (article_id,related_id,related_order) VALUES (2 , 3 , 1 ) ;
INSERT INTO va_articles_related (article_id,related_id,related_order) VALUES (3 , 1 , 2 ) ;
INSERT INTO va_articles_related (article_id,related_id,related_order) VALUES (3 , 2 , 1 ) ;


INSERT INTO va_articles_reviews (review_id,article_id,user_id,admin_id,recommended,approved,rating,summary,user_name,user_email,remote_address,comments,admin_id_approved_by,admin_id_modified_by,date_added,date_modified,date_approved) VALUES (1 , 1 , NULL , NULL , 1 , 1 , 4 , NULL , 'Author' , NULL , '10.1.1.6' , NULL , NULL , NULL , '2008-09-08 15:53:50' , NULL , NULL ) ;
INSERT INTO va_articles_reviews (review_id,article_id,user_id,admin_id,recommended,approved,rating,summary,user_name,user_email,remote_address,comments,admin_id_approved_by,admin_id_modified_by,date_added,date_modified,date_approved) VALUES (2 , 4 , NULL , NULL , 1 , 1 , 5 , NULL , 'test' , NULL , '127.0.0.1' , NULL , NULL , NULL , '2008-09-12 08:44:13' , NULL , NULL ) ;
INSERT INTO va_articles_reviews (review_id,article_id,user_id,admin_id,recommended,approved,rating,summary,user_name,user_email,remote_address,comments,admin_id_approved_by,admin_id_modified_by,date_added,date_modified,date_approved) VALUES (3 , 2 , NULL , NULL , 1 , 1 , 5 , NULL , 'test3' , NULL , '127.0.0.1' , NULL , NULL , NULL , '2008-09-09 13:11:48' , NULL , NULL ) ;
INSERT INTO va_articles_reviews (review_id,article_id,user_id,admin_id,recommended,approved,rating,summary,user_name,user_email,remote_address,comments,admin_id_approved_by,admin_id_modified_by,date_added,date_modified,date_approved) VALUES (4 , 3 , NULL , NULL , 1 , 1 , 0 , NULL , 'test2' , NULL , '127.0.0.1' , NULL , NULL , NULL , '2008-09-09 18:05:32' , NULL , NULL ) ;


INSERT INTO va_articles_statuses (status_id,status_name,status_description,is_shown,status_icon,status_style,allowed_view) VALUES (1 , 'ARTICLE_NEW_MSG' , NULL , 1 , NULL , NULL , 1 ) ;
INSERT INTO va_articles_statuses (status_id,status_name,status_description,is_shown,status_icon,status_style,allowed_view) VALUES (2 , 'ARTICLE_PUBLISHED_MSG' , NULL , 1 , NULL , NULL , 1 ) ;
INSERT INTO va_articles_statuses (status_id,status_name,status_description,is_shown,status_icon,status_style,allowed_view) VALUES (3 , 'ARTICLE_PENDING_MSG' , NULL , 1 , NULL , NULL , 0 ) ;
INSERT INTO va_articles_statuses (status_id,status_name,status_description,is_shown,status_icon,status_style,allowed_view) VALUES (4 , 'ARTICLE_HIDDEN_MSG' , NULL , 1 , NULL , NULL , 0 ) ;
























INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (16 , 'admin.php' , 'visits' , 'yesterday' , '2012-12-06 14:55:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (17 , 'admin.php' , 'visits' , 'this_week' , '2012-12-06 14:55:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (18 , 'admin.php' , 'visits' , 'this_month' , '2012-12-06 14:55:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (19 , 'admin.php' , 'visits' , 'last_month' , '2012-12-06 14:55:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (20 , 'admin.php' , 'visits' , 'total_year' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (26 , 'counts_users' , 'date' , '2010-06-01' , '2010-07-01 17:08:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (27 , 'counts_users' , 'date' , '2010-06-02' , '2010-07-01 17:08:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (28 , 'counts_users' , 'date' , '2010-06-03' , '2010-07-01 17:08:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (29 , 'counts_users' , 'date' , '2010-06-04' , '2010-07-01 17:08:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (30 , 'counts_users' , 'date' , '2010-06-05' , '2010-07-01 17:08:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (31 , 'counts_users' , 'date' , '2010-06-06' , '2010-07-06 03:20:58' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (32 , 'counts_users' , 'date' , '2010-06-07' , '2010-07-06 03:20:58' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (33 , 'counts_users' , 'date' , '2010-06-08' , '2010-07-06 03:20:58' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (34 , 'counts_users' , 'date' , '2010-06-09' , '2010-07-06 03:20:58' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (35 , 'counts_users' , 'date' , '2010-06-10' , '2010-07-06 03:20:58' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (36 , 'counts_users' , 'date' , '2010-06-11' , '2010-07-06 03:20:58' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (37 , 'counts_users' , 'date' , '2010-06-12' , '2010-07-06 03:20:58' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (38 , 'counts_users' , 'date' , '2010-06-13' , '2010-07-06 03:20:58' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (39 , 'counts_users' , 'date' , '2010-06-14' , '2010-07-06 03:20:58' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (40 , 'counts_users' , 'date' , '2010-06-15' , '2010-07-06 03:20:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (41 , 'counts_users' , 'date' , '2010-06-16' , '2010-07-06 03:20:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (42 , 'counts_users' , 'date' , '2010-06-17' , '2010-07-06 03:20:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (43 , 'counts_users' , 'date' , '2010-06-18' , '2010-07-06 03:20:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (44 , 'counts_users' , 'date' , '2010-06-19' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (45 , 'counts_users' , 'date' , '2010-06-20' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (46 , 'counts_users' , 'date' , '2010-06-21' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (47 , 'counts_users' , 'date' , '2010-06-22' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (48 , 'counts_users' , 'date' , '2010-06-23' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (49 , 'counts_users' , 'date' , '2010-06-24' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (50 , 'counts_users' , 'date' , '2010-06-25' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (51 , 'counts_users' , 'date' , '2010-06-26' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (52 , 'counts_users' , 'date' , '2010-06-27' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (53 , 'counts_users' , 'date' , '2010-06-28' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (54 , 'counts_users' , 'date' , '2010-06-29' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (55 , 'counts_users' , 'date' , '2010-06-30' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (56 , 'counts_orders' , 'date' , '2010-06-01' , '2010-07-01 17:08:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (57 , 'counts_orders' , 'date' , '2010-06-02' , '2010-07-01 17:08:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (58 , 'counts_orders' , 'date' , '2010-06-03' , '2010-07-01 17:08:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (59 , 'counts_orders' , 'date' , '2010-06-04' , '2010-07-01 17:08:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (60 , 'counts_orders' , 'date' , '2010-06-05' , '2010-07-01 17:08:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (61 , 'counts_orders' , 'date' , '2010-06-06' , '2010-07-06 03:20:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (62 , 'counts_orders' , 'date' , '2010-06-07' , '2010-07-06 03:20:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (63 , 'counts_orders' , 'date' , '2010-06-08' , '2010-07-06 03:20:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (64 , 'counts_orders' , 'date' , '2010-06-09' , '2010-07-06 03:20:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (65 , 'counts_orders' , 'date' , '2010-06-10' , '2010-07-06 03:20:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (66 , 'counts_orders' , 'date' , '2010-06-11' , '2010-07-06 03:20:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (67 , 'counts_orders' , 'date' , '2010-06-12' , '2010-07-06 03:20:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (68 , 'counts_orders' , 'date' , '2010-06-13' , '2010-07-06 03:20:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (69 , 'counts_orders' , 'date' , '2010-06-14' , '2010-07-06 03:20:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (70 , 'counts_orders' , 'date' , '2010-06-15' , '2010-07-06 03:20:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (71 , 'counts_orders' , 'date' , '2010-06-16' , '2010-07-06 03:20:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (72 , 'counts_orders' , 'date' , '2010-06-17' , '2010-07-06 03:20:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (73 , 'counts_orders' , 'date' , '2010-06-18' , '2010-07-06 03:20:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (74 , 'counts_orders' , 'date' , '2010-06-19' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (75 , 'counts_orders' , 'date' , '2010-06-20' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (76 , 'counts_orders' , 'date' , '2010-06-21' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (77 , 'counts_orders' , 'date' , '2010-06-22' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (78 , 'counts_orders' , 'date' , '2010-06-23' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (79 , 'counts_orders' , 'date' , '2010-06-24' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (80 , 'counts_orders' , 'date' , '2010-06-25' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (81 , 'counts_orders' , 'date' , '2010-06-26' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (82 , 'counts_orders' , 'date' , '2010-06-27' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (83 , 'counts_orders' , 'date' , '2010-06-28' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (84 , 'counts_orders' , 'date' , '2010-06-29' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (85 , 'counts_orders' , 'date' , '2010-06-30' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (86 , 'counts_sales' , 'date' , '2010-06-01' , '2010-07-01 17:08:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (87 , 'counts_sales' , 'date' , '2010-06-02' , '2010-07-01 17:08:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (88 , 'counts_sales' , 'date' , '2010-06-03' , '2010-07-01 17:08:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (89 , 'counts_sales' , 'date' , '2010-06-04' , '2010-07-01 17:08:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (90 , 'counts_sales' , 'date' , '2010-06-05' , '2010-07-01 17:08:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (91 , 'counts_sales' , 'date' , '2010-06-06' , '2010-07-06 03:20:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (92 , 'counts_sales' , 'date' , '2010-06-07' , '2010-07-06 03:20:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (93 , 'counts_sales' , 'date' , '2010-06-08' , '2010-07-06 03:20:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (94 , 'counts_sales' , 'date' , '2010-06-09' , '2010-07-06 03:20:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (95 , 'counts_sales' , 'date' , '2010-06-10' , '2010-07-06 03:21:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (96 , 'counts_sales' , 'date' , '2010-06-11' , '2010-07-06 03:21:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (97 , 'counts_sales' , 'date' , '2010-06-12' , '2010-07-06 03:21:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (98 , 'counts_sales' , 'date' , '2010-06-13' , '2010-07-06 03:21:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (99 , 'counts_sales' , 'date' , '2010-06-14' , '2010-07-06 03:21:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (100 , 'counts_sales' , 'date' , '2010-06-15' , '2010-07-06 03:21:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (101 , 'counts_sales' , 'date' , '2010-06-16' , '2010-07-06 03:21:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (102 , 'counts_sales' , 'date' , '2010-06-17' , '2010-07-06 03:21:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (103 , 'counts_sales' , 'date' , '2010-06-18' , '2010-07-06 03:21:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (104 , 'counts_sales' , 'date' , '2010-06-19' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (105 , 'counts_sales' , 'date' , '2010-06-20' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (106 , 'counts_sales' , 'date' , '2010-06-21' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (107 , 'counts_sales' , 'date' , '2010-06-22' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (108 , 'counts_sales' , 'date' , '2010-06-23' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (109 , 'counts_sales' , 'date' , '2010-06-24' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (110 , 'counts_sales' , 'date' , '2010-06-25' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (111 , 'counts_sales' , 'date' , '2010-06-26' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (112 , 'counts_sales' , 'date' , '2010-06-27' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (113 , 'counts_sales' , 'date' , '2010-06-28' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (114 , 'counts_sales' , 'date' , '2010-06-29' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (115 , 'counts_sales' , 'date' , '2010-06-30' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (116 , 'counts_visits' , 'date' , '2010-06-01' , '2010-07-01 17:08:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (117 , 'counts_visits' , 'date' , '2010-06-02' , '2010-07-01 17:08:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (118 , 'counts_visits' , 'date' , '2010-06-03' , '2010-07-01 17:08:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (119 , 'counts_visits' , 'date' , '2010-06-04' , '2010-07-01 17:08:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (120 , 'counts_visits' , 'date' , '2010-06-05' , '2010-07-01 17:08:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (121 , 'counts_visits' , 'date' , '2010-06-06' , '2010-07-06 03:21:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (122 , 'counts_visits' , 'date' , '2010-06-07' , '2010-07-06 03:21:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (123 , 'counts_visits' , 'date' , '2010-06-08' , '2010-07-06 03:21:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (124 , 'counts_visits' , 'date' , '2010-06-09' , '2010-07-06 03:21:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (125 , 'counts_visits' , 'date' , '2010-06-10' , '2010-07-06 03:21:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (126 , 'counts_visits' , 'date' , '2010-06-11' , '2010-07-06 03:21:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (127 , 'counts_visits' , 'date' , '2010-06-12' , '2010-07-06 03:21:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (128 , 'counts_visits' , 'date' , '2010-06-13' , '2010-07-06 03:21:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (129 , 'counts_visits' , 'date' , '2010-06-14' , '2010-07-06 03:21:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (130 , 'counts_visits' , 'date' , '2010-06-15' , '2010-07-06 03:21:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (131 , 'counts_visits' , 'date' , '2010-06-16' , '2010-07-06 03:21:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (132 , 'counts_visits' , 'date' , '2010-06-17' , '2010-07-06 03:21:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (133 , 'counts_visits' , 'date' , '2010-06-18' , '2010-07-06 03:21:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (134 , 'counts_visits' , 'date' , '2010-06-19' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (135 , 'counts_visits' , 'date' , '2010-06-20' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (136 , 'counts_visits' , 'date' , '2010-06-21' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (137 , 'counts_visits' , 'date' , '2010-06-22' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (138 , 'counts_visits' , 'date' , '2010-06-23' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (139 , 'counts_visits' , 'date' , '2010-06-24' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (140 , 'counts_visits' , 'date' , '2010-06-25' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (141 , 'counts_visits' , 'date' , '2010-06-26' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (142 , 'counts_visits' , 'date' , '2010-06-27' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (143 , 'counts_visits' , 'date' , '2010-06-28' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (144 , 'counts_visits' , 'date' , '2010-06-29' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (145 , 'counts_visits' , 'date' , '2010-06-30' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (146 , 'counts_users' , 'date' , '2010-07-01' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (147 , 'counts_users' , 'date' , '2010-07-02' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (148 , 'counts_users' , 'date' , '2010-07-03' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (149 , 'counts_users' , 'date' , '2010-07-04' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (150 , 'counts_users' , 'date' , '2010-07-05' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (151 , 'counts_orders' , 'date' , '2010-07-01' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (152 , 'counts_orders' , 'date' , '2010-07-02' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (153 , 'counts_orders' , 'date' , '2010-07-03' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (154 , 'counts_orders' , 'date' , '2010-07-04' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (155 , 'counts_orders' , 'date' , '2010-07-05' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (156 , 'counts_sales' , 'date' , '2010-07-01' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (157 , 'counts_sales' , 'date' , '2010-07-02' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (158 , 'counts_sales' , 'date' , '2010-07-03' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (159 , 'counts_sales' , 'date' , '2010-07-04' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (160 , 'counts_sales' , 'date' , '2010-07-05' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (161 , 'counts_visits' , 'date' , '2010-07-01' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (162 , 'counts_visits' , 'date' , '2010-07-02' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (163 , 'counts_visits' , 'date' , '2010-07-03' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (164 , 'counts_visits' , 'date' , '2010-07-04' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (165 , 'counts_visits' , 'date' , '2010-07-05' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (166 , 'counts_users' , 'date' , '2010-07-06' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (167 , 'counts_users' , 'date' , '2010-07-07' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (168 , 'counts_users' , 'date' , '2010-07-08' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (169 , 'counts_users' , 'date' , '2010-07-09' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (170 , 'counts_users' , 'date' , '2010-07-10' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (171 , 'counts_users' , 'date' , '2010-07-11' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (172 , 'counts_users' , 'date' , '2010-07-12' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (173 , 'counts_users' , 'date' , '2010-07-13' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (174 , 'counts_users' , 'date' , '2010-07-14' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (175 , 'counts_users' , 'date' , '2010-07-15' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (176 , 'counts_users' , 'date' , '2010-07-16' , '2010-07-19 15:58:52' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (177 , 'counts_users' , 'date' , '2010-07-17' , '2010-08-16 20:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (178 , 'counts_users' , 'date' , '2010-07-18' , '2010-08-16 20:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (179 , 'counts_orders' , 'date' , '2010-07-06' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (180 , 'counts_orders' , 'date' , '2010-07-07' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (181 , 'counts_orders' , 'date' , '2010-07-08' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (182 , 'counts_orders' , 'date' , '2010-07-09' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (183 , 'counts_orders' , 'date' , '2010-07-10' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (184 , 'counts_orders' , 'date' , '2010-07-11' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (185 , 'counts_orders' , 'date' , '2010-07-12' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (186 , 'counts_orders' , 'date' , '2010-07-13' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (187 , 'counts_orders' , 'date' , '2010-07-14' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (188 , 'counts_orders' , 'date' , '2010-07-15' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (189 , 'counts_orders' , 'date' , '2010-07-16' , '2010-07-19 15:58:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (190 , 'counts_orders' , 'date' , '2010-07-17' , '2010-08-16 20:47:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (191 , 'counts_orders' , 'date' , '2010-07-18' , '2010-08-16 20:47:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (192 , 'counts_sales' , 'date' , '2010-07-06' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (193 , 'counts_sales' , 'date' , '2010-07-07' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (194 , 'counts_sales' , 'date' , '2010-07-08' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (195 , 'counts_sales' , 'date' , '2010-07-09' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (196 , 'counts_sales' , 'date' , '2010-07-10' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (197 , 'counts_sales' , 'date' , '2010-07-11' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (198 , 'counts_sales' , 'date' , '2010-07-12' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (199 , 'counts_sales' , 'date' , '2010-07-13' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (200 , 'counts_sales' , 'date' , '2010-07-14' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (201 , 'counts_sales' , 'date' , '2010-07-15' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (202 , 'counts_sales' , 'date' , '2010-07-16' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (203 , 'counts_sales' , 'date' , '2010-07-17' , '2010-08-16 20:47:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (204 , 'counts_sales' , 'date' , '2010-07-18' , '2010-08-16 20:47:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (205 , 'counts_visits' , 'date' , '2010-07-06' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (206 , 'counts_visits' , 'date' , '2010-07-07' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (207 , 'counts_visits' , 'date' , '2010-07-08' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (208 , 'counts_visits' , 'date' , '2010-07-09' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (209 , 'counts_visits' , 'date' , '2010-07-10' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (210 , 'counts_visits' , 'date' , '2010-07-11' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (211 , 'counts_visits' , 'date' , '2010-07-12' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (212 , 'counts_visits' , 'date' , '2010-07-13' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (213 , 'counts_visits' , 'date' , '2010-07-14' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (214 , 'counts_visits' , 'date' , '2010-07-15' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (215 , 'counts_visits' , 'date' , '2010-07-16' , '2010-07-19 15:58:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (216 , 'counts_visits' , 'date' , '2010-07-17' , '2010-08-16 20:47:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (217 , 'counts_visits' , 'date' , '2010-07-18' , '2010-08-16 20:47:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (218 , 'counts_users' , 'date' , '2010-07-19' , '2010-08-16 20:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (219 , 'counts_users' , 'date' , '2010-07-20' , '2010-08-16 20:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (220 , 'counts_users' , 'date' , '2010-07-21' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (221 , 'counts_users' , 'date' , '2010-07-22' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (222 , 'counts_users' , 'date' , '2010-07-23' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (223 , 'counts_users' , 'date' , '2010-07-24' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (224 , 'counts_users' , 'date' , '2010-07-25' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (225 , 'counts_users' , 'date' , '2010-07-26' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (226 , 'counts_users' , 'date' , '2010-07-27' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (227 , 'counts_users' , 'date' , '2010-07-28' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (228 , 'counts_users' , 'date' , '2010-07-29' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (229 , 'counts_users' , 'date' , '2010-07-30' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (230 , 'counts_users' , 'date' , '2010-07-31' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (231 , 'counts_users' , 'date' , '2010-08-01' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (232 , 'counts_users' , 'date' , '2010-08-02' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (233 , 'counts_users' , 'date' , '2010-08-03' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (234 , 'counts_users' , 'date' , '2010-08-04' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (235 , 'counts_users' , 'date' , '2010-08-05' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (236 , 'counts_users' , 'date' , '2010-08-06' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (237 , 'counts_users' , 'date' , '2010-08-07' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (238 , 'counts_users' , 'date' , '2010-08-08' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (239 , 'counts_users' , 'date' , '2010-08-09' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (240 , 'counts_users' , 'date' , '2010-08-10' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (241 , 'counts_users' , 'date' , '2010-08-11' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (242 , 'counts_users' , 'date' , '2010-08-12' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (243 , 'counts_users' , 'date' , '2010-08-13' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (244 , 'counts_users' , 'date' , '2010-08-14' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (245 , 'counts_users' , 'date' , '2010-08-15' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (246 , 'counts_orders' , 'date' , '2010-07-19' , '2010-08-16 20:47:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (247 , 'counts_orders' , 'date' , '2010-07-20' , '2010-08-16 20:47:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (248 , 'counts_orders' , 'date' , '2010-07-21' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (249 , 'counts_orders' , 'date' , '2010-07-22' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (250 , 'counts_orders' , 'date' , '2010-07-23' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (251 , 'counts_orders' , 'date' , '2010-07-24' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (252 , 'counts_orders' , 'date' , '2010-07-25' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (253 , 'counts_orders' , 'date' , '2010-07-26' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (254 , 'counts_orders' , 'date' , '2010-07-27' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (255 , 'counts_orders' , 'date' , '2010-07-28' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (256 , 'counts_orders' , 'date' , '2010-07-29' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (257 , 'counts_orders' , 'date' , '2010-07-30' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (258 , 'counts_orders' , 'date' , '2010-07-31' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (259 , 'counts_orders' , 'date' , '2010-08-01' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (260 , 'counts_orders' , 'date' , '2010-08-02' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (261 , 'counts_orders' , 'date' , '2010-08-03' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (262 , 'counts_orders' , 'date' , '2010-08-04' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (263 , 'counts_orders' , 'date' , '2010-08-05' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (264 , 'counts_orders' , 'date' , '2010-08-06' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (265 , 'counts_orders' , 'date' , '2010-08-07' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (266 , 'counts_orders' , 'date' , '2010-08-08' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (267 , 'counts_orders' , 'date' , '2010-08-09' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (268 , 'counts_orders' , 'date' , '2010-08-10' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (269 , 'counts_orders' , 'date' , '2010-08-11' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (270 , 'counts_orders' , 'date' , '2010-08-12' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (271 , 'counts_orders' , 'date' , '2010-08-13' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (272 , 'counts_orders' , 'date' , '2010-08-14' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (273 , 'counts_orders' , 'date' , '2010-08-15' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (274 , 'counts_sales' , 'date' , '2010-07-19' , '2010-08-16 20:47:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (275 , 'counts_sales' , 'date' , '2010-07-20' , '2010-08-16 20:47:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (276 , 'counts_sales' , 'date' , '2010-07-21' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (277 , 'counts_sales' , 'date' , '2010-07-22' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (278 , 'counts_sales' , 'date' , '2010-07-23' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (279 , 'counts_sales' , 'date' , '2010-07-24' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (280 , 'counts_sales' , 'date' , '2010-07-25' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (281 , 'counts_sales' , 'date' , '2010-07-26' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (282 , 'counts_sales' , 'date' , '2010-07-27' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (283 , 'counts_sales' , 'date' , '2010-07-28' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (284 , 'counts_sales' , 'date' , '2010-07-29' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (285 , 'counts_sales' , 'date' , '2010-07-30' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (286 , 'counts_sales' , 'date' , '2010-07-31' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (287 , 'counts_sales' , 'date' , '2010-08-01' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (288 , 'counts_sales' , 'date' , '2010-08-02' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (289 , 'counts_sales' , 'date' , '2010-08-03' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (290 , 'counts_sales' , 'date' , '2010-08-04' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (291 , 'counts_sales' , 'date' , '2010-08-05' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (292 , 'counts_sales' , 'date' , '2010-08-06' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (293 , 'counts_sales' , 'date' , '2010-08-07' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (294 , 'counts_sales' , 'date' , '2010-08-08' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (295 , 'counts_sales' , 'date' , '2010-08-09' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (296 , 'counts_sales' , 'date' , '2010-08-10' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (297 , 'counts_sales' , 'date' , '2010-08-11' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (298 , 'counts_sales' , 'date' , '2010-08-12' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (299 , 'counts_sales' , 'date' , '2010-08-13' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (300 , 'counts_sales' , 'date' , '2010-08-14' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (301 , 'counts_sales' , 'date' , '2010-08-15' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (302 , 'counts_visits' , 'date' , '2010-07-19' , '2010-08-16 20:47:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (303 , 'counts_visits' , 'date' , '2010-07-20' , '2010-08-16 20:47:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (304 , 'counts_visits' , 'date' , '2010-07-21' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (305 , 'counts_visits' , 'date' , '2010-07-22' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (306 , 'counts_visits' , 'date' , '2010-07-23' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (307 , 'counts_visits' , 'date' , '2010-07-24' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (308 , 'counts_visits' , 'date' , '2010-07-25' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (309 , 'counts_visits' , 'date' , '2010-07-26' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (310 , 'counts_visits' , 'date' , '2010-07-27' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (311 , 'counts_visits' , 'date' , '2010-07-28' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (312 , 'counts_visits' , 'date' , '2010-07-29' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (313 , 'counts_visits' , 'date' , '2010-07-30' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (314 , 'counts_visits' , 'date' , '2010-07-31' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (315 , 'counts_visits' , 'date' , '2010-08-01' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (316 , 'counts_visits' , 'date' , '2010-08-02' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (317 , 'counts_visits' , 'date' , '2010-08-03' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (318 , 'counts_visits' , 'date' , '2010-08-04' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (319 , 'counts_visits' , 'date' , '2010-08-05' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (320 , 'counts_visits' , 'date' , '2010-08-06' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (321 , 'counts_visits' , 'date' , '2010-08-07' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (322 , 'counts_visits' , 'date' , '2010-08-08' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (323 , 'counts_visits' , 'date' , '2010-08-09' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (324 , 'counts_visits' , 'date' , '2010-08-10' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (325 , 'counts_visits' , 'date' , '2010-08-11' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (326 , 'counts_visits' , 'date' , '2010-08-12' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (327 , 'counts_visits' , 'date' , '2010-08-13' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (328 , 'counts_visits' , 'date' , '2010-08-14' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (329 , 'counts_visits' , 'date' , '2010-08-15' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (330 , 'counts_users' , 'date' , '2010-08-16' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (331 , 'counts_users' , 'date' , '2010-08-17' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (332 , 'counts_users' , 'date' , '2010-08-18' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (333 , 'counts_users' , 'date' , '2010-08-19' , '2010-08-20 20:50:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (334 , 'counts_sales' , 'date' , '2010-08-16' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (335 , 'counts_sales' , 'date' , '2010-08-17' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (336 , 'counts_sales' , 'date' , '2010-08-18' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (337 , 'counts_sales' , 'date' , '2010-08-19' , '2010-08-20 20:50:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (338 , 'counts_visits' , 'date' , '2010-08-16' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (339 , 'counts_visits' , 'date' , '2010-08-17' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (340 , 'counts_visits' , 'date' , '2010-08-18' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (341 , 'counts_visits' , 'date' , '2010-08-19' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (342 , 'counts_orders' , 'date' , '2010-08-16' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (343 , 'counts_orders' , 'date' , '2010-08-17' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (344 , 'counts_orders' , 'date' , '2010-08-18' , '2010-08-20 20:50:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (345 , 'counts_orders' , 'date' , '2010-08-19' , '2010-08-20 20:50:35' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (346 , 'counts_users' , 'date' , '2010-08-20' , '2010-09-19 03:29:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (347 , 'counts_users' , 'date' , '2010-08-21' , '2010-09-19 03:29:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (348 , 'counts_users' , 'date' , '2010-08-22' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (349 , 'counts_users' , 'date' , '2010-08-23' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (350 , 'counts_users' , 'date' , '2010-08-24' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (351 , 'counts_users' , 'date' , '2010-08-25' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (352 , 'counts_users' , 'date' , '2010-08-26' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (353 , 'counts_users' , 'date' , '2010-08-27' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (354 , 'counts_users' , 'date' , '2010-08-28' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (355 , 'counts_users' , 'date' , '2010-08-29' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (356 , 'counts_users' , 'date' , '2010-08-30' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (357 , 'counts_users' , 'date' , '2010-08-31' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (358 , 'counts_users' , 'date' , '2010-09-01' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (359 , 'counts_users' , 'date' , '2010-09-02' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (360 , 'counts_users' , 'date' , '2010-09-03' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (361 , 'counts_users' , 'date' , '2010-09-04' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (362 , 'counts_users' , 'date' , '2010-09-05' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (363 , 'counts_users' , 'date' , '2010-09-06' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (364 , 'counts_users' , 'date' , '2010-09-07' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (365 , 'counts_users' , 'date' , '2010-09-08' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (366 , 'counts_users' , 'date' , '2010-09-09' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (367 , 'counts_users' , 'date' , '2010-09-10' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (368 , 'counts_users' , 'date' , '2010-09-11' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (369 , 'counts_users' , 'date' , '2010-09-12' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (370 , 'counts_users' , 'date' , '2010-09-13' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (371 , 'counts_users' , 'date' , '2010-09-14' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (372 , 'counts_users' , 'date' , '2010-09-15' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (373 , 'counts_users' , 'date' , '2010-09-16' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (374 , 'counts_users' , 'date' , '2010-09-17' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (375 , 'counts_users' , 'date' , '2010-09-18' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (376 , 'counts_sales' , 'date' , '2010-08-20' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (377 , 'counts_sales' , 'date' , '2010-08-21' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (378 , 'counts_sales' , 'date' , '2010-08-22' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (379 , 'counts_sales' , 'date' , '2010-08-23' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (380 , 'counts_sales' , 'date' , '2010-08-24' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (381 , 'counts_sales' , 'date' , '2010-08-25' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (382 , 'counts_sales' , 'date' , '2010-08-26' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (383 , 'counts_sales' , 'date' , '2010-08-27' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (384 , 'counts_sales' , 'date' , '2010-08-28' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (385 , 'counts_sales' , 'date' , '2010-08-29' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (386 , 'counts_sales' , 'date' , '2010-08-30' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (387 , 'counts_sales' , 'date' , '2010-08-31' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (388 , 'counts_sales' , 'date' , '2010-09-01' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (389 , 'counts_sales' , 'date' , '2010-09-02' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (390 , 'counts_sales' , 'date' , '2010-09-03' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (391 , 'counts_sales' , 'date' , '2010-09-04' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (392 , 'counts_sales' , 'date' , '2010-09-05' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (393 , 'counts_sales' , 'date' , '2010-09-06' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (394 , 'counts_sales' , 'date' , '2010-09-07' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (395 , 'counts_sales' , 'date' , '2010-09-08' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (396 , 'counts_sales' , 'date' , '2010-09-09' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (397 , 'counts_sales' , 'date' , '2010-09-10' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (398 , 'counts_sales' , 'date' , '2010-09-11' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (399 , 'counts_sales' , 'date' , '2010-09-12' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (400 , 'counts_sales' , 'date' , '2010-09-13' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (401 , 'counts_sales' , 'date' , '2010-09-14' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (402 , 'counts_sales' , 'date' , '2010-09-15' , '2010-09-19 03:29:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (403 , 'counts_sales' , 'date' , '2010-09-16' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (404 , 'counts_sales' , 'date' , '2010-09-17' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (405 , 'counts_sales' , 'date' , '2010-09-18' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (406 , 'counts_visits' , 'date' , '2010-08-20' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (407 , 'counts_visits' , 'date' , '2010-08-21' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (408 , 'counts_visits' , 'date' , '2010-08-22' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (409 , 'counts_visits' , 'date' , '2010-08-23' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (410 , 'counts_visits' , 'date' , '2010-08-24' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (411 , 'counts_visits' , 'date' , '2010-08-25' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (412 , 'counts_visits' , 'date' , '2010-08-26' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (413 , 'counts_visits' , 'date' , '2010-08-27' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (414 , 'counts_visits' , 'date' , '2010-08-28' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (415 , 'counts_visits' , 'date' , '2010-08-29' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (416 , 'counts_visits' , 'date' , '2010-08-30' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (417 , 'counts_visits' , 'date' , '2010-08-31' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (418 , 'counts_visits' , 'date' , '2010-09-01' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (419 , 'counts_visits' , 'date' , '2010-09-02' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (420 , 'counts_visits' , 'date' , '2010-09-03' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (421 , 'counts_visits' , 'date' , '2010-09-04' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (422 , 'counts_visits' , 'date' , '2010-09-05' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (423 , 'counts_visits' , 'date' , '2010-09-06' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (424 , 'counts_visits' , 'date' , '2010-09-07' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (425 , 'counts_visits' , 'date' , '2010-09-08' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (426 , 'counts_visits' , 'date' , '2010-09-09' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (427 , 'counts_visits' , 'date' , '2010-09-10' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (428 , 'counts_visits' , 'date' , '2010-09-11' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (429 , 'counts_visits' , 'date' , '2010-09-12' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (430 , 'counts_visits' , 'date' , '2010-09-13' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (431 , 'counts_visits' , 'date' , '2010-09-14' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (432 , 'counts_visits' , 'date' , '2010-09-15' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (433 , 'counts_visits' , 'date' , '2010-09-16' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (434 , 'counts_visits' , 'date' , '2010-09-17' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (435 , 'counts_visits' , 'date' , '2010-09-18' , '2010-09-19 03:29:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (436 , 'counts_visits' , 'date' , '2010-09-27' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (437 , 'counts_visits' , 'date' , '2010-09-28' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (438 , 'counts_visits' , 'date' , '2010-09-29' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (439 , 'counts_visits' , 'date' , '2010-09-30' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (440 , 'counts_visits' , 'date' , '2010-10-01' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (441 , 'counts_visits' , 'date' , '2010-10-02' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (442 , 'counts_visits' , 'date' , '2010-10-03' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (443 , 'counts_visits' , 'date' , '2010-10-04' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (444 , 'counts_visits' , 'date' , '2010-10-05' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (445 , 'counts_visits' , 'date' , '2010-10-06' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (446 , 'counts_visits' , 'date' , '2010-10-07' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (447 , 'counts_visits' , 'date' , '2010-10-08' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (448 , 'counts_visits' , 'date' , '2010-10-09' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (449 , 'counts_visits' , 'date' , '2010-10-10' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (450 , 'counts_visits' , 'date' , '2010-10-11' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (451 , 'counts_visits' , 'date' , '2010-10-12' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (452 , 'counts_visits' , 'date' , '2010-10-13' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (453 , 'counts_visits' , 'date' , '2010-10-14' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (454 , 'counts_visits' , 'date' , '2010-10-15' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (455 , 'counts_visits' , 'date' , '2010-10-16' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (456 , 'counts_visits' , 'date' , '2010-10-17' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (457 , 'counts_visits' , 'date' , '2010-10-18' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (458 , 'counts_visits' , 'date' , '2010-10-19' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (459 , 'counts_visits' , 'date' , '2010-10-20' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (460 , 'counts_visits' , 'date' , '2010-10-21' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (461 , 'counts_visits' , 'date' , '2010-10-22' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (462 , 'counts_visits' , 'date' , '2010-10-23' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (463 , 'counts_visits' , 'date' , '2010-10-24' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (464 , 'counts_visits' , 'date' , '2010-10-25' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (465 , 'counts_visits' , 'date' , '2010-10-26' , '2010-10-27 20:42:07' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (466 , 'counts_users' , 'date' , '2010-09-27' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (467 , 'counts_users' , 'date' , '2010-09-28' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (468 , 'counts_users' , 'date' , '2010-09-29' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (469 , 'counts_users' , 'date' , '2010-09-30' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (470 , 'counts_users' , 'date' , '2010-10-01' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (471 , 'counts_users' , 'date' , '2010-10-02' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (472 , 'counts_users' , 'date' , '2010-10-03' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (473 , 'counts_users' , 'date' , '2010-10-04' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (474 , 'counts_users' , 'date' , '2010-10-05' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (475 , 'counts_users' , 'date' , '2010-10-06' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (476 , 'counts_users' , 'date' , '2010-10-07' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (477 , 'counts_users' , 'date' , '2010-10-08' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (478 , 'counts_users' , 'date' , '2010-10-09' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (479 , 'counts_users' , 'date' , '2010-10-10' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (480 , 'counts_users' , 'date' , '2010-10-11' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (481 , 'counts_users' , 'date' , '2010-10-12' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (482 , 'counts_users' , 'date' , '2010-10-13' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (483 , 'counts_users' , 'date' , '2010-10-14' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (484 , 'counts_users' , 'date' , '2010-10-15' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (485 , 'counts_users' , 'date' , '2010-10-16' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (486 , 'counts_users' , 'date' , '2010-10-17' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (487 , 'counts_users' , 'date' , '2010-10-18' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (488 , 'counts_users' , 'date' , '2010-10-19' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (489 , 'counts_users' , 'date' , '2010-10-20' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (490 , 'counts_users' , 'date' , '2010-10-21' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (491 , 'counts_users' , 'date' , '2010-10-22' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (492 , 'counts_users' , 'date' , '2010-10-23' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (493 , 'counts_users' , 'date' , '2010-10-24' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (494 , 'counts_users' , 'date' , '2010-10-25' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (495 , 'counts_users' , 'date' , '2010-10-26' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (496 , 'counts_sales' , 'date' , '2010-09-27' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (497 , 'counts_sales' , 'date' , '2010-09-28' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (498 , 'counts_sales' , 'date' , '2010-09-29' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (499 , 'counts_sales' , 'date' , '2010-09-30' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (500 , 'counts_sales' , 'date' , '2010-10-01' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (501 , 'counts_sales' , 'date' , '2010-10-02' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (502 , 'counts_sales' , 'date' , '2010-10-03' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (503 , 'counts_sales' , 'date' , '2010-10-04' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (504 , 'counts_sales' , 'date' , '2010-10-05' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (505 , 'counts_sales' , 'date' , '2010-10-06' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (506 , 'counts_sales' , 'date' , '2010-10-07' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (507 , 'counts_sales' , 'date' , '2010-10-08' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (508 , 'counts_sales' , 'date' , '2010-10-09' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (509 , 'counts_sales' , 'date' , '2010-10-10' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (510 , 'counts_sales' , 'date' , '2010-10-11' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (511 , 'counts_sales' , 'date' , '2010-10-12' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (512 , 'counts_sales' , 'date' , '2010-10-13' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (513 , 'counts_sales' , 'date' , '2010-10-14' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (514 , 'counts_sales' , 'date' , '2010-10-15' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (515 , 'counts_sales' , 'date' , '2010-10-16' , '2010-10-27 20:42:08' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (516 , 'counts_sales' , 'date' , '2010-10-17' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (517 , 'counts_sales' , 'date' , '2010-10-18' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (518 , 'counts_sales' , 'date' , '2010-10-19' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (519 , 'counts_sales' , 'date' , '2010-10-20' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (520 , 'counts_sales' , 'date' , '2010-10-21' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (521 , 'counts_sales' , 'date' , '2010-10-22' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (522 , 'counts_sales' , 'date' , '2010-10-23' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (523 , 'counts_sales' , 'date' , '2010-10-24' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (524 , 'counts_sales' , 'date' , '2010-10-25' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (525 , 'counts_sales' , 'date' , '2010-10-26' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (526 , 'counts_orders' , 'date' , '2010-09-27' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (527 , 'counts_orders' , 'date' , '2010-09-28' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (528 , 'counts_orders' , 'date' , '2010-09-29' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (529 , 'counts_orders' , 'date' , '2010-09-30' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (530 , 'counts_orders' , 'date' , '2010-10-01' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (531 , 'counts_orders' , 'date' , '2010-10-02' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (532 , 'counts_orders' , 'date' , '2010-10-03' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (533 , 'counts_orders' , 'date' , '2010-10-04' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (534 , 'counts_orders' , 'date' , '2010-10-05' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (535 , 'counts_orders' , 'date' , '2010-10-06' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (536 , 'counts_orders' , 'date' , '2010-10-07' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (537 , 'counts_orders' , 'date' , '2010-10-08' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (538 , 'counts_orders' , 'date' , '2010-10-09' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (539 , 'counts_orders' , 'date' , '2010-10-10' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (540 , 'counts_orders' , 'date' , '2010-10-11' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (541 , 'counts_orders' , 'date' , '2010-10-12' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (542 , 'counts_orders' , 'date' , '2010-10-13' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (543 , 'counts_orders' , 'date' , '2010-10-14' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (544 , 'counts_orders' , 'date' , '2010-10-15' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (545 , 'counts_orders' , 'date' , '2010-10-16' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (546 , 'counts_orders' , 'date' , '2010-10-17' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (547 , 'counts_orders' , 'date' , '2010-10-18' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (548 , 'counts_orders' , 'date' , '2010-10-19' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (549 , 'counts_orders' , 'date' , '2010-10-20' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (550 , 'counts_orders' , 'date' , '2010-10-21' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (551 , 'counts_orders' , 'date' , '2010-10-22' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (552 , 'counts_orders' , 'date' , '2010-10-23' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (553 , 'counts_orders' , 'date' , '2010-10-24' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (554 , 'counts_orders' , 'date' , '2010-10-25' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (555 , 'counts_orders' , 'date' , '2010-10-26' , '2010-10-27 20:42:09' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (556 , 'counts_users' , 'date' , '2010-10-27' , '2010-10-28 04:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (557 , 'counts_orders' , 'date' , '2010-10-27' , '2010-10-28 04:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (558 , 'counts_sales' , 'date' , '2010-10-27' , '2010-10-28 04:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (559 , 'counts_visits' , 'date' , '2010-10-27' , '2010-10-28 04:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (560 , 'counts_orders' , 'date' , '2010-11-01' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (561 , 'counts_orders' , 'date' , '2010-11-02' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (562 , 'counts_orders' , 'date' , '2010-11-03' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (563 , 'counts_orders' , 'date' , '2010-11-04' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (564 , 'counts_orders' , 'date' , '2010-11-05' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (565 , 'counts_orders' , 'date' , '2010-11-06' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (566 , 'counts_orders' , 'date' , '2010-11-07' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (567 , 'counts_orders' , 'date' , '2010-11-08' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (568 , 'counts_orders' , 'date' , '2010-11-09' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (569 , 'counts_orders' , 'date' , '2010-11-10' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (570 , 'counts_orders' , 'date' , '2010-11-11' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (571 , 'counts_orders' , 'date' , '2010-11-12' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (572 , 'counts_orders' , 'date' , '2010-11-13' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (573 , 'counts_orders' , 'date' , '2010-11-14' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (574 , 'counts_orders' , 'date' , '2010-11-15' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (575 , 'counts_orders' , 'date' , '2010-11-16' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (576 , 'counts_orders' , 'date' , '2010-11-17' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (577 , 'counts_orders' , 'date' , '2010-11-18' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (578 , 'counts_orders' , 'date' , '2010-11-19' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (579 , 'counts_orders' , 'date' , '2010-11-20' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (580 , 'counts_orders' , 'date' , '2010-11-21' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (581 , 'counts_orders' , 'date' , '2010-11-22' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (582 , 'counts_orders' , 'date' , '2010-11-23' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (583 , 'counts_orders' , 'date' , '2010-11-24' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (584 , 'counts_orders' , 'date' , '2010-11-25' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (585 , 'counts_orders' , 'date' , '2010-11-26' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (586 , 'counts_orders' , 'date' , '2010-11-27' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (587 , 'counts_orders' , 'date' , '2010-11-28' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (588 , 'counts_orders' , 'date' , '2010-11-29' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (589 , 'counts_orders' , 'date' , '2010-11-30' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (590 , 'counts_visits' , 'date' , '2010-11-01' , '2010-12-01 17:47:27' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (591 , 'counts_visits' , 'date' , '2010-11-02' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (592 , 'counts_visits' , 'date' , '2010-11-03' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (593 , 'counts_visits' , 'date' , '2010-11-04' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (594 , 'counts_visits' , 'date' , '2010-11-05' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (595 , 'counts_visits' , 'date' , '2010-11-06' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (596 , 'counts_visits' , 'date' , '2010-11-07' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (597 , 'counts_visits' , 'date' , '2010-11-08' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (598 , 'counts_visits' , 'date' , '2010-11-09' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (599 , 'counts_visits' , 'date' , '2010-11-10' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (600 , 'counts_visits' , 'date' , '2010-11-11' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (601 , 'counts_visits' , 'date' , '2010-11-12' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (602 , 'counts_visits' , 'date' , '2010-11-13' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (603 , 'counts_visits' , 'date' , '2010-11-14' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (604 , 'counts_visits' , 'date' , '2010-11-15' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (605 , 'counts_visits' , 'date' , '2010-11-16' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (606 , 'counts_visits' , 'date' , '2010-11-17' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (607 , 'counts_visits' , 'date' , '2010-11-18' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (608 , 'counts_visits' , 'date' , '2010-11-19' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (609 , 'counts_visits' , 'date' , '2010-11-20' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (610 , 'counts_visits' , 'date' , '2010-11-21' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (611 , 'counts_visits' , 'date' , '2010-11-22' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (612 , 'counts_visits' , 'date' , '2010-11-23' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (613 , 'counts_visits' , 'date' , '2010-11-24' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (614 , 'counts_visits' , 'date' , '2010-11-25' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (615 , 'counts_visits' , 'date' , '2010-11-26' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (616 , 'counts_visits' , 'date' , '2010-11-27' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (617 , 'counts_visits' , 'date' , '2010-11-28' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (618 , 'counts_visits' , 'date' , '2010-11-29' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (619 , 'counts_visits' , 'date' , '2010-11-30' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (620 , 'counts_sales' , 'date' , '2010-11-01' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (621 , 'counts_sales' , 'date' , '2010-11-02' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (622 , 'counts_sales' , 'date' , '2010-11-03' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (623 , 'counts_sales' , 'date' , '2010-11-04' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (624 , 'counts_sales' , 'date' , '2010-11-05' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (625 , 'counts_sales' , 'date' , '2010-11-06' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (626 , 'counts_sales' , 'date' , '2010-11-07' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (627 , 'counts_sales' , 'date' , '2010-11-08' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (628 , 'counts_sales' , 'date' , '2010-11-09' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (629 , 'counts_sales' , 'date' , '2010-11-10' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (630 , 'counts_sales' , 'date' , '2010-11-11' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (631 , 'counts_sales' , 'date' , '2010-11-12' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (632 , 'counts_sales' , 'date' , '2010-11-13' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (633 , 'counts_sales' , 'date' , '2010-11-14' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (634 , 'counts_sales' , 'date' , '2010-11-15' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (635 , 'counts_sales' , 'date' , '2010-11-16' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (636 , 'counts_sales' , 'date' , '2010-11-17' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (637 , 'counts_sales' , 'date' , '2010-11-18' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (638 , 'counts_sales' , 'date' , '2010-11-19' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (639 , 'counts_sales' , 'date' , '2010-11-20' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (640 , 'counts_sales' , 'date' , '2010-11-21' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (641 , 'counts_sales' , 'date' , '2010-11-22' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (642 , 'counts_sales' , 'date' , '2010-11-23' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (643 , 'counts_sales' , 'date' , '2010-11-24' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (644 , 'counts_sales' , 'date' , '2010-11-25' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (645 , 'counts_sales' , 'date' , '2010-11-26' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (646 , 'counts_sales' , 'date' , '2010-11-27' , '2010-12-01 17:47:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (647 , 'counts_sales' , 'date' , '2010-11-28' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (648 , 'counts_sales' , 'date' , '2010-11-29' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (649 , 'counts_sales' , 'date' , '2010-11-30' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (650 , 'counts_users' , 'date' , '2010-11-01' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (651 , 'counts_users' , 'date' , '2010-11-02' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (652 , 'counts_users' , 'date' , '2010-11-03' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (653 , 'counts_users' , 'date' , '2010-11-04' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (654 , 'counts_users' , 'date' , '2010-11-05' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (655 , 'counts_users' , 'date' , '2010-11-06' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (656 , 'counts_users' , 'date' , '2010-11-07' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (657 , 'counts_users' , 'date' , '2010-11-08' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (658 , 'counts_users' , 'date' , '2010-11-09' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (659 , 'counts_users' , 'date' , '2010-11-10' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (660 , 'counts_users' , 'date' , '2010-11-11' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (661 , 'counts_users' , 'date' , '2010-11-12' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (662 , 'counts_users' , 'date' , '2010-11-13' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (663 , 'counts_users' , 'date' , '2010-11-14' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (664 , 'counts_users' , 'date' , '2010-11-15' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (665 , 'counts_users' , 'date' , '2010-11-16' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (666 , 'counts_users' , 'date' , '2010-11-17' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (667 , 'counts_users' , 'date' , '2010-11-18' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (668 , 'counts_users' , 'date' , '2010-11-19' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (669 , 'counts_users' , 'date' , '2010-11-20' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (670 , 'counts_users' , 'date' , '2010-11-21' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (671 , 'counts_users' , 'date' , '2010-11-22' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (672 , 'counts_users' , 'date' , '2010-11-23' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (673 , 'counts_users' , 'date' , '2010-11-24' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (674 , 'counts_users' , 'date' , '2010-11-25' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (675 , 'counts_users' , 'date' , '2010-11-26' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (676 , 'counts_users' , 'date' , '2010-11-27' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (677 , 'counts_users' , 'date' , '2010-11-28' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (678 , 'counts_users' , 'date' , '2010-11-29' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (679 , 'counts_users' , 'date' , '2010-11-30' , '2010-12-01 17:47:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (680 , 'counts_visits' , 'date' , '2011-03-23' , '2011-04-22 20:03:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (681 , 'counts_visits' , 'date' , '2011-03-24' , '2011-04-22 20:03:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (682 , 'counts_visits' , 'date' , '2011-03-25' , '2011-04-22 20:03:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (683 , 'counts_visits' , 'date' , '2011-03-26' , '2011-04-22 20:03:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (684 , 'counts_visits' , 'date' , '2011-03-27' , '2011-04-22 20:03:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (685 , 'counts_visits' , 'date' , '2011-03-28' , '2011-04-22 20:03:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (686 , 'counts_visits' , 'date' , '2011-03-29' , '2011-04-22 20:03:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (687 , 'counts_visits' , 'date' , '2011-03-30' , '2011-04-22 20:03:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (688 , 'counts_visits' , 'date' , '2011-03-31' , '2011-04-22 20:03:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (689 , 'counts_visits' , 'date' , '2011-04-01' , '2011-04-22 20:03:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (690 , 'counts_visits' , 'date' , '2011-04-02' , '2011-04-22 20:03:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (691 , 'counts_visits' , 'date' , '2011-04-03' , '2011-04-22 20:03:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (692 , 'counts_visits' , 'date' , '2011-04-04' , '2011-04-22 20:03:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (693 , 'counts_visits' , 'date' , '2011-04-05' , '2011-04-22 20:03:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (694 , 'counts_visits' , 'date' , '2011-04-06' , '2011-04-22 20:03:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (695 , 'counts_visits' , 'date' , '2011-04-07' , '2011-04-22 20:03:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (696 , 'counts_visits' , 'date' , '2011-04-08' , '2011-04-22 20:03:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (697 , 'counts_visits' , 'date' , '2011-04-09' , '2011-04-22 20:03:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (698 , 'counts_visits' , 'date' , '2011-04-10' , '2011-04-22 20:03:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (699 , 'counts_visits' , 'date' , '2011-04-11' , '2011-04-22 20:03:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (700 , 'counts_visits' , 'date' , '2011-04-12' , '2011-04-22 20:03:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (701 , 'counts_visits' , 'date' , '2011-04-13' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (702 , 'counts_visits' , 'date' , '2011-04-14' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (703 , 'counts_visits' , 'date' , '2011-04-15' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (704 , 'counts_visits' , 'date' , '2011-04-16' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (705 , 'counts_visits' , 'date' , '2011-04-17' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (706 , 'counts_visits' , 'date' , '2011-04-18' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (707 , 'counts_visits' , 'date' , '2011-04-19' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (708 , 'counts_visits' , 'date' , '2011-04-20' , '2011-05-20 12:36:02' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (709 , 'counts_visits' , 'date' , '2011-04-21' , '2011-05-20 12:36:03' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (710 , 'counts_users' , 'date' , '2011-03-23' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (711 , 'counts_users' , 'date' , '2011-03-24' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (712 , 'counts_users' , 'date' , '2011-03-25' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (713 , 'counts_users' , 'date' , '2011-03-26' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (714 , 'counts_users' , 'date' , '2011-03-27' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (715 , 'counts_users' , 'date' , '2011-03-28' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (716 , 'counts_users' , 'date' , '2011-03-29' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (717 , 'counts_users' , 'date' , '2011-03-30' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (718 , 'counts_users' , 'date' , '2011-03-31' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (719 , 'counts_users' , 'date' , '2011-04-01' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (720 , 'counts_users' , 'date' , '2011-04-02' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (721 , 'counts_users' , 'date' , '2011-04-03' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (722 , 'counts_users' , 'date' , '2011-04-04' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (723 , 'counts_users' , 'date' , '2011-04-05' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (724 , 'counts_users' , 'date' , '2011-04-06' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (725 , 'counts_users' , 'date' , '2011-04-07' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (726 , 'counts_users' , 'date' , '2011-04-08' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (727 , 'counts_users' , 'date' , '2011-04-09' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (728 , 'counts_users' , 'date' , '2011-04-10' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (729 , 'counts_users' , 'date' , '2011-04-11' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (730 , 'counts_users' , 'date' , '2011-04-12' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (731 , 'counts_users' , 'date' , '2011-04-13' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (732 , 'counts_users' , 'date' , '2011-04-14' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (733 , 'counts_users' , 'date' , '2011-04-15' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (734 , 'counts_users' , 'date' , '2011-04-16' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (735 , 'counts_users' , 'date' , '2011-04-17' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (736 , 'counts_users' , 'date' , '2011-04-18' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (737 , 'counts_users' , 'date' , '2011-04-19' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (738 , 'counts_users' , 'date' , '2011-04-20' , '2011-05-20 12:36:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (739 , 'counts_users' , 'date' , '2011-04-21' , '2011-05-20 12:36:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (740 , 'counts_orders' , 'date' , '2011-03-23' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (741 , 'counts_orders' , 'date' , '2011-03-24' , '2011-04-22 20:03:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (742 , 'counts_orders' , 'date' , '2011-03-25' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (743 , 'counts_orders' , 'date' , '2011-03-26' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (744 , 'counts_orders' , 'date' , '2011-03-27' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (745 , 'counts_orders' , 'date' , '2011-03-28' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (746 , 'counts_orders' , 'date' , '2011-03-29' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (747 , 'counts_orders' , 'date' , '2011-03-30' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (748 , 'counts_orders' , 'date' , '2011-03-31' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (749 , 'counts_orders' , 'date' , '2011-04-01' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (750 , 'counts_orders' , 'date' , '2011-04-02' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (751 , 'counts_orders' , 'date' , '2011-04-03' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (752 , 'counts_orders' , 'date' , '2011-04-04' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (753 , 'counts_orders' , 'date' , '2011-04-05' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (754 , 'counts_orders' , 'date' , '2011-04-06' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (755 , 'counts_orders' , 'date' , '2011-04-07' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (756 , 'counts_orders' , 'date' , '2011-04-08' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (757 , 'counts_orders' , 'date' , '2011-04-09' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (758 , 'counts_orders' , 'date' , '2011-04-10' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (759 , 'counts_orders' , 'date' , '2011-04-11' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (760 , 'counts_orders' , 'date' , '2011-04-12' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (761 , 'counts_orders' , 'date' , '2011-04-13' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (762 , 'counts_orders' , 'date' , '2011-04-14' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (763 , 'counts_orders' , 'date' , '2011-04-15' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (764 , 'counts_orders' , 'date' , '2011-04-16' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (765 , 'counts_orders' , 'date' , '2011-04-17' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (766 , 'counts_orders' , 'date' , '2011-04-18' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (767 , 'counts_orders' , 'date' , '2011-04-19' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (768 , 'counts_orders' , 'date' , '2011-04-20' , '2011-05-20 12:35:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (769 , 'counts_orders' , 'date' , '2011-04-21' , '2011-05-20 12:35:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (770 , 'counts_sales' , 'date' , '2011-03-23' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (771 , 'counts_sales' , 'date' , '2011-03-24' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (772 , 'counts_sales' , 'date' , '2011-03-25' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (773 , 'counts_sales' , 'date' , '2011-03-26' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (774 , 'counts_sales' , 'date' , '2011-03-27' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (775 , 'counts_sales' , 'date' , '2011-03-28' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (776 , 'counts_sales' , 'date' , '2011-03-29' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (777 , 'counts_sales' , 'date' , '2011-03-30' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (778 , 'counts_sales' , 'date' , '2011-03-31' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (779 , 'counts_sales' , 'date' , '2011-04-01' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (780 , 'counts_sales' , 'date' , '2011-04-02' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (781 , 'counts_sales' , 'date' , '2011-04-03' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (782 , 'counts_sales' , 'date' , '2011-04-04' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (783 , 'counts_sales' , 'date' , '2011-04-05' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (784 , 'counts_sales' , 'date' , '2011-04-06' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (785 , 'counts_sales' , 'date' , '2011-04-07' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (786 , 'counts_sales' , 'date' , '2011-04-08' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (787 , 'counts_sales' , 'date' , '2011-04-09' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (788 , 'counts_sales' , 'date' , '2011-04-10' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (789 , 'counts_sales' , 'date' , '2011-04-11' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (790 , 'counts_sales' , 'date' , '2011-04-12' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (791 , 'counts_sales' , 'date' , '2011-04-13' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (792 , 'counts_sales' , 'date' , '2011-04-14' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (793 , 'counts_sales' , 'date' , '2011-04-15' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (794 , 'counts_sales' , 'date' , '2011-04-16' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (795 , 'counts_sales' , 'date' , '2011-04-17' , '2011-04-22 20:03:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (796 , 'counts_sales' , 'date' , '2011-04-18' , '2011-04-22 20:03:56' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (797 , 'counts_sales' , 'date' , '2011-04-19' , '2011-04-22 20:03:56' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (798 , 'counts_sales' , 'date' , '2011-04-20' , '2011-05-20 12:36:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (799 , 'counts_sales' , 'date' , '2011-04-21' , '2011-05-20 12:36:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (800 , 'counts_orders' , 'date' , '2011-04-22' , '2011-05-20 12:35:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (801 , 'counts_orders' , 'date' , '2011-04-23' , '2011-05-20 12:35:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (802 , 'counts_orders' , 'date' , '2011-04-24' , '2011-05-20 12:35:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (803 , 'counts_orders' , 'date' , '2011-04-25' , '2011-05-20 12:35:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (804 , 'counts_orders' , 'date' , '2011-04-26' , '2011-05-20 12:35:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (805 , 'counts_orders' , 'date' , '2011-04-27' , '2011-05-27 14:52:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (806 , 'counts_orders' , 'date' , '2011-04-28' , '2011-05-27 14:52:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (807 , 'counts_orders' , 'date' , '2011-04-29' , '2011-05-27 14:52:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (808 , 'counts_orders' , 'date' , '2011-04-30' , '2011-05-27 14:52:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (809 , 'counts_orders' , 'date' , '2011-05-01' , '2011-05-27 14:52:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (810 , 'counts_orders' , 'date' , '2011-05-02' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (811 , 'counts_orders' , 'date' , '2011-05-03' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (812 , 'counts_orders' , 'date' , '2011-05-04' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (813 , 'counts_orders' , 'date' , '2011-05-05' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (814 , 'counts_orders' , 'date' , '2011-05-06' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (815 , 'counts_orders' , 'date' , '2011-05-07' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (816 , 'counts_orders' , 'date' , '2011-05-08' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (817 , 'counts_orders' , 'date' , '2011-05-09' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (818 , 'counts_orders' , 'date' , '2011-05-10' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (819 , 'counts_orders' , 'date' , '2011-05-11' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (820 , 'counts_orders' , 'date' , '2011-05-12' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (821 , 'counts_orders' , 'date' , '2011-05-13' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (822 , 'counts_orders' , 'date' , '2011-05-14' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (823 , 'counts_orders' , 'date' , '2011-05-15' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (824 , 'counts_orders' , 'date' , '2011-05-16' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (825 , 'counts_orders' , 'date' , '2011-05-17' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (826 , 'counts_orders' , 'date' , '2011-05-18' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (827 , 'counts_orders' , 'date' , '2011-05-19' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (828 , 'counts_sales' , 'date' , '2011-04-22' , '2011-05-20 12:36:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (829 , 'counts_sales' , 'date' , '2011-04-23' , '2011-05-20 12:36:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (830 , 'counts_sales' , 'date' , '2011-04-24' , '2011-05-20 12:36:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (831 , 'counts_sales' , 'date' , '2011-04-25' , '2011-05-20 12:36:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (832 , 'counts_sales' , 'date' , '2011-04-26' , '2011-05-20 12:36:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (833 , 'counts_sales' , 'date' , '2011-04-27' , '2011-05-27 14:52:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (834 , 'counts_sales' , 'date' , '2011-04-28' , '2011-05-27 14:52:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (835 , 'counts_sales' , 'date' , '2011-04-29' , '2011-05-27 14:52:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (836 , 'counts_sales' , 'date' , '2011-04-30' , '2011-05-27 14:52:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (837 , 'counts_sales' , 'date' , '2011-05-01' , '2011-05-27 14:52:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (838 , 'counts_sales' , 'date' , '2011-05-02' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (839 , 'counts_sales' , 'date' , '2011-05-03' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (840 , 'counts_sales' , 'date' , '2011-05-04' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (841 , 'counts_sales' , 'date' , '2011-05-05' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (842 , 'counts_sales' , 'date' , '2011-05-06' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (843 , 'counts_sales' , 'date' , '2011-05-07' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (844 , 'counts_sales' , 'date' , '2011-05-08' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (845 , 'counts_sales' , 'date' , '2011-05-09' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (846 , 'counts_sales' , 'date' , '2011-05-10' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (847 , 'counts_sales' , 'date' , '2011-05-11' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (848 , 'counts_sales' , 'date' , '2011-05-12' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (849 , 'counts_sales' , 'date' , '2011-05-13' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (850 , 'counts_sales' , 'date' , '2011-05-14' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (851 , 'counts_sales' , 'date' , '2011-05-15' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (852 , 'counts_sales' , 'date' , '2011-05-16' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (853 , 'counts_sales' , 'date' , '2011-05-17' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (854 , 'counts_sales' , 'date' , '2011-05-18' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (855 , 'counts_sales' , 'date' , '2011-05-19' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (856 , 'counts_users' , 'date' , '2011-04-22' , '2011-05-20 12:36:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (857 , 'counts_users' , 'date' , '2011-04-23' , '2011-05-20 12:36:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (858 , 'counts_users' , 'date' , '2011-04-24' , '2011-05-20 12:36:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (859 , 'counts_users' , 'date' , '2011-04-25' , '2011-05-20 12:36:02' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (860 , 'counts_users' , 'date' , '2011-04-26' , '2011-05-20 12:36:02' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (861 , 'counts_users' , 'date' , '2011-04-27' , '2011-05-27 14:52:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (862 , 'counts_users' , 'date' , '2011-04-28' , '2011-05-27 14:52:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (863 , 'counts_users' , 'date' , '2011-04-29' , '2011-05-27 14:52:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (864 , 'counts_users' , 'date' , '2011-04-30' , '2011-05-27 14:52:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (865 , 'counts_users' , 'date' , '2011-05-01' , '2011-05-27 14:52:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (866 , 'counts_users' , 'date' , '2011-05-02' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (867 , 'counts_users' , 'date' , '2011-05-03' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (868 , 'counts_users' , 'date' , '2011-05-04' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (869 , 'counts_users' , 'date' , '2011-05-05' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (870 , 'counts_users' , 'date' , '2011-05-06' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (871 , 'counts_users' , 'date' , '2011-05-07' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (872 , 'counts_users' , 'date' , '2011-05-08' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (873 , 'counts_users' , 'date' , '2011-05-09' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (874 , 'counts_users' , 'date' , '2011-05-10' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (875 , 'counts_users' , 'date' , '2011-05-11' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (876 , 'counts_users' , 'date' , '2011-05-12' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (877 , 'counts_users' , 'date' , '2011-05-13' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (878 , 'counts_users' , 'date' , '2011-05-14' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (879 , 'counts_users' , 'date' , '2011-05-15' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (880 , 'counts_users' , 'date' , '2011-05-16' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (881 , 'counts_users' , 'date' , '2011-05-17' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (882 , 'counts_users' , 'date' , '2011-05-18' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (883 , 'counts_users' , 'date' , '2011-05-19' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (884 , 'counts_visits' , 'date' , '2011-04-22' , '2011-05-20 12:36:03' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (885 , 'counts_visits' , 'date' , '2011-04-23' , '2011-05-20 12:36:03' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (886 , 'counts_visits' , 'date' , '2011-04-24' , '2011-05-20 12:36:03' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (887 , 'counts_visits' , 'date' , '2011-04-25' , '2011-05-20 12:36:03' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (888 , 'counts_visits' , 'date' , '2011-04-26' , '2011-05-20 12:36:03' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (889 , 'counts_visits' , 'date' , '2011-04-27' , '2011-05-27 14:52:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (890 , 'counts_visits' , 'date' , '2011-04-28' , '2011-05-27 14:52:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (891 , 'counts_visits' , 'date' , '2011-04-29' , '2011-05-27 14:52:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (892 , 'counts_visits' , 'date' , '2011-04-30' , '2011-05-27 14:52:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (893 , 'counts_visits' , 'date' , '2011-05-01' , '2011-05-27 14:52:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (894 , 'counts_visits' , 'date' , '2011-05-02' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (895 , 'counts_visits' , 'date' , '2011-05-03' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (896 , 'counts_visits' , 'date' , '2011-05-04' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (897 , 'counts_visits' , 'date' , '2011-05-05' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (898 , 'counts_visits' , 'date' , '2011-05-06' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (899 , 'counts_visits' , 'date' , '2011-05-07' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (900 , 'counts_visits' , 'date' , '2011-05-08' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (901 , 'counts_visits' , 'date' , '2011-05-09' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (902 , 'counts_visits' , 'date' , '2011-05-10' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (903 , 'counts_visits' , 'date' , '2011-05-11' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (904 , 'counts_visits' , 'date' , '2011-05-12' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (905 , 'counts_visits' , 'date' , '2011-05-13' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (906 , 'counts_visits' , 'date' , '2011-05-14' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (907 , 'counts_visits' , 'date' , '2011-05-15' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (908 , 'counts_visits' , 'date' , '2011-05-16' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (909 , 'counts_visits' , 'date' , '2011-05-17' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (910 , 'counts_visits' , 'date' , '2011-05-18' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (911 , 'counts_visits' , 'date' , '2011-05-19' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (912 , 'counts_users' , 'date' , '2011-05-20' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (913 , 'counts_users' , 'date' , '2011-05-21' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (914 , 'counts_users' , 'date' , '2011-05-22' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (915 , 'counts_users' , 'date' , '2011-05-23' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (916 , 'counts_users' , 'date' , '2011-05-24' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (917 , 'counts_users' , 'date' , '2011-05-25' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (918 , 'counts_users' , 'date' , '2011-05-26' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (919 , 'counts_orders' , 'date' , '2011-05-20' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (920 , 'counts_orders' , 'date' , '2011-05-21' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (921 , 'counts_orders' , 'date' , '2011-05-22' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (922 , 'counts_orders' , 'date' , '2011-05-23' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (923 , 'counts_orders' , 'date' , '2011-05-24' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (924 , 'counts_orders' , 'date' , '2011-05-25' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (925 , 'counts_orders' , 'date' , '2011-05-26' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (926 , 'counts_sales' , 'date' , '2011-05-20' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (927 , 'counts_sales' , 'date' , '2011-05-21' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (928 , 'counts_sales' , 'date' , '2011-05-22' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (929 , 'counts_sales' , 'date' , '2011-05-23' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (930 , 'counts_sales' , 'date' , '2011-05-24' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (931 , 'counts_sales' , 'date' , '2011-05-25' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (932 , 'counts_sales' , 'date' , '2011-05-26' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (933 , 'counts_visits' , 'date' , '2011-05-20' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (934 , 'counts_visits' , 'date' , '2011-05-21' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (935 , 'counts_visits' , 'date' , '2011-05-22' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (936 , 'counts_visits' , 'date' , '2011-05-23' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (937 , 'counts_visits' , 'date' , '2011-05-24' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (938 , 'counts_visits' , 'date' , '2011-05-25' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (939 , 'counts_visits' , 'date' , '2011-05-26' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (940 , 'counts_visits' , 'date' , '2011-05-27' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (941 , 'counts_visits' , 'date' , '2011-05-28' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (942 , 'counts_visits' , 'date' , '2011-05-29' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (943 , 'counts_visits' , 'date' , '2011-05-30' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (944 , 'counts_visits' , 'date' , '2011-05-31' , '2011-06-01 17:54:28' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (945 , 'counts_sales' , 'date' , '2011-05-27' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (946 , 'counts_sales' , 'date' , '2011-05-28' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (947 , 'counts_sales' , 'date' , '2011-05-29' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (948 , 'counts_sales' , 'date' , '2011-05-30' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (949 , 'counts_sales' , 'date' , '2011-05-31' , '2011-06-01 17:54:29' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (950 , 'counts_users' , 'date' , '2011-05-27' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (951 , 'counts_users' , 'date' , '2011-05-28' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (952 , 'counts_users' , 'date' , '2011-05-29' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (953 , 'counts_users' , 'date' , '2011-05-30' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (954 , 'counts_users' , 'date' , '2011-05-31' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (955 , 'counts_orders' , 'date' , '2011-05-27' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (956 , 'counts_orders' , 'date' , '2011-05-28' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (957 , 'counts_orders' , 'date' , '2011-05-29' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (958 , 'counts_orders' , 'date' , '2011-05-30' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (959 , 'counts_orders' , 'date' , '2011-05-31' , '2011-06-01 17:54:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (960 , 'counts_users' , 'date' , '2011-06-01' , '2011-06-02 00:14:58' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (961 , 'counts_visits' , 'date' , '2011-06-01' , '2011-06-02 00:14:58' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (962 , 'counts_orders' , 'date' , '2011-06-01' , '2011-06-02 00:14:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (963 , 'counts_sales' , 'date' , '2011-06-01' , '2011-06-02 00:14:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (964 , 'counts_sales' , 'date' , '2011-07-20' , '2011-08-19 00:28:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (965 , 'counts_sales' , 'date' , '2011-07-21' , '2011-08-19 00:28:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (966 , 'counts_sales' , 'date' , '2011-07-22' , '2011-08-19 00:28:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (967 , 'counts_sales' , 'date' , '2011-07-23' , '2011-08-19 00:28:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (968 , 'counts_sales' , 'date' , '2011-07-24' , '2011-08-19 00:28:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (969 , 'counts_sales' , 'date' , '2011-07-25' , '2011-08-19 00:28:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (970 , 'counts_sales' , 'date' , '2011-07-26' , '2011-08-19 00:28:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (971 , 'counts_sales' , 'date' , '2011-07-27' , '2011-08-19 00:28:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (972 , 'counts_sales' , 'date' , '2011-07-28' , '2011-08-19 00:28:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (973 , 'counts_sales' , 'date' , '2011-07-29' , '2011-08-19 00:28:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (974 , 'counts_sales' , 'date' , '2011-07-30' , '2011-08-19 00:28:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (975 , 'counts_sales' , 'date' , '2011-07-31' , '2011-08-19 00:28:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (976 , 'counts_sales' , 'date' , '2011-08-01' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (977 , 'counts_sales' , 'date' , '2011-08-02' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (978 , 'counts_sales' , 'date' , '2011-08-03' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (979 , 'counts_sales' , 'date' , '2011-08-04' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (980 , 'counts_sales' , 'date' , '2011-08-05' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (981 , 'counts_sales' , 'date' , '2011-08-06' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (982 , 'counts_sales' , 'date' , '2011-08-07' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (983 , 'counts_sales' , 'date' , '2011-08-08' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (984 , 'counts_sales' , 'date' , '2011-08-09' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (985 , 'counts_sales' , 'date' , '2011-08-10' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (986 , 'counts_sales' , 'date' , '2011-08-11' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (987 , 'counts_sales' , 'date' , '2011-08-12' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (988 , 'counts_sales' , 'date' , '2011-08-13' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (989 , 'counts_sales' , 'date' , '2011-08-14' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (990 , 'counts_sales' , 'date' , '2011-08-15' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (991 , 'counts_sales' , 'date' , '2011-08-16' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (992 , 'counts_sales' , 'date' , '2011-08-17' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (993 , 'counts_sales' , 'date' , '2011-08-18' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (994 , 'counts_users' , 'date' , '2011-07-20' , '2011-08-19 00:28:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (995 , 'counts_users' , 'date' , '2011-07-21' , '2011-08-19 00:28:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (996 , 'counts_users' , 'date' , '2011-07-22' , '2011-08-19 00:28:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (997 , 'counts_users' , 'date' , '2011-07-23' , '2011-08-19 00:28:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (998 , 'counts_users' , 'date' , '2011-07-24' , '2011-08-19 00:28:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (999 , 'counts_users' , 'date' , '2011-07-25' , '2011-08-19 00:28:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1000 , 'counts_users' , 'date' , '2011-07-26' , '2011-08-19 00:28:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1001 , 'counts_users' , 'date' , '2011-07-27' , '2011-08-19 00:28:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1002 , 'counts_users' , 'date' , '2011-07-28' , '2011-08-19 00:28:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1003 , 'counts_users' , 'date' , '2011-07-29' , '2011-08-19 00:28:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1004 , 'counts_users' , 'date' , '2011-07-30' , '2011-08-19 00:28:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1005 , 'counts_users' , 'date' , '2011-07-31' , '2011-08-19 00:28:34' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1006 , 'counts_users' , 'date' , '2011-08-01' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1007 , 'counts_users' , 'date' , '2011-08-02' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1008 , 'counts_users' , 'date' , '2011-08-03' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1009 , 'counts_users' , 'date' , '2011-08-04' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1010 , 'counts_users' , 'date' , '2011-08-05' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1011 , 'counts_users' , 'date' , '2011-08-06' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1012 , 'counts_users' , 'date' , '2011-08-07' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1013 , 'counts_users' , 'date' , '2011-08-08' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1014 , 'counts_users' , 'date' , '2011-08-09' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1015 , 'counts_users' , 'date' , '2011-08-10' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1016 , 'counts_users' , 'date' , '2011-08-11' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1017 , 'counts_users' , 'date' , '2011-08-12' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1018 , 'counts_users' , 'date' , '2011-08-13' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1019 , 'counts_users' , 'date' , '2011-08-14' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1020 , 'counts_users' , 'date' , '2011-08-15' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1021 , 'counts_users' , 'date' , '2011-08-16' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1022 , 'counts_users' , 'date' , '2011-08-17' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1023 , 'counts_users' , 'date' , '2011-08-18' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1024 , 'counts_visits' , 'date' , '2011-07-20' , '2011-08-19 00:28:35' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1025 , 'counts_visits' , 'date' , '2011-07-21' , '2011-08-19 00:28:35' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1026 , 'counts_visits' , 'date' , '2011-07-22' , '2011-08-19 00:28:35' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1027 , 'counts_visits' , 'date' , '2011-07-23' , '2011-08-19 00:28:35' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1028 , 'counts_visits' , 'date' , '2011-07-24' , '2011-08-19 00:28:35' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1029 , 'counts_visits' , 'date' , '2011-07-25' , '2011-08-19 00:28:35' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1030 , 'counts_visits' , 'date' , '2011-07-26' , '2011-08-19 00:28:35' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1031 , 'counts_visits' , 'date' , '2011-07-27' , '2011-08-19 00:28:35' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1032 , 'counts_visits' , 'date' , '2011-07-28' , '2011-08-19 00:28:35' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1033 , 'counts_visits' , 'date' , '2011-07-29' , '2011-08-19 00:28:35' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1034 , 'counts_visits' , 'date' , '2011-07-30' , '2011-08-19 00:28:35' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1035 , 'counts_visits' , 'date' , '2011-07-31' , '2011-08-19 00:28:35' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1036 , 'counts_visits' , 'date' , '2011-08-01' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1037 , 'counts_visits' , 'date' , '2011-08-02' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1038 , 'counts_visits' , 'date' , '2011-08-03' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1039 , 'counts_visits' , 'date' , '2011-08-04' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1040 , 'counts_visits' , 'date' , '2011-08-05' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1041 , 'counts_visits' , 'date' , '2011-08-06' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1042 , 'counts_visits' , 'date' , '2011-08-07' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1043 , 'counts_visits' , 'date' , '2011-08-08' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1044 , 'counts_visits' , 'date' , '2011-08-09' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1045 , 'counts_visits' , 'date' , '2011-08-10' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1046 , 'counts_visits' , 'date' , '2011-08-11' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1047 , 'counts_visits' , 'date' , '2011-08-12' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1048 , 'counts_visits' , 'date' , '2011-08-13' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1049 , 'counts_visits' , 'date' , '2011-08-14' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1050 , 'counts_visits' , 'date' , '2011-08-15' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1051 , 'counts_visits' , 'date' , '2011-08-16' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1052 , 'counts_visits' , 'date' , '2011-08-17' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1053 , 'counts_visits' , 'date' , '2011-08-18' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1054 , 'counts_orders' , 'date' , '2011-07-20' , '2011-08-19 00:28:35' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1055 , 'counts_orders' , 'date' , '2011-07-21' , '2011-08-19 00:28:35' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1056 , 'counts_orders' , 'date' , '2011-07-22' , '2011-08-19 00:28:35' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1057 , 'counts_orders' , 'date' , '2011-07-23' , '2011-08-19 00:28:35' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1058 , 'counts_orders' , 'date' , '2011-07-24' , '2011-08-19 00:28:35' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1059 , 'counts_orders' , 'date' , '2011-07-25' , '2011-08-19 00:28:35' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1060 , 'counts_orders' , 'date' , '2011-07-26' , '2011-08-19 00:28:35' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1061 , 'counts_orders' , 'date' , '2011-07-27' , '2011-08-19 00:28:35' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1062 , 'counts_orders' , 'date' , '2011-07-28' , '2011-08-19 00:28:35' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1063 , 'counts_orders' , 'date' , '2011-07-29' , '2011-08-19 00:28:36' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1064 , 'counts_orders' , 'date' , '2011-07-30' , '2011-08-19 00:28:36' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1065 , 'counts_orders' , 'date' , '2011-07-31' , '2011-08-19 00:28:36' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1066 , 'counts_orders' , 'date' , '2011-08-01' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1067 , 'counts_orders' , 'date' , '2011-08-02' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1068 , 'counts_orders' , 'date' , '2011-08-03' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1069 , 'counts_orders' , 'date' , '2011-08-04' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1070 , 'counts_orders' , 'date' , '2011-08-05' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1071 , 'counts_orders' , 'date' , '2011-08-06' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1072 , 'counts_orders' , 'date' , '2011-08-07' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1073 , 'counts_orders' , 'date' , '2011-08-08' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1074 , 'counts_orders' , 'date' , '2011-08-09' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1075 , 'counts_orders' , 'date' , '2011-08-10' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1076 , 'counts_orders' , 'date' , '2011-08-11' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1077 , 'counts_orders' , 'date' , '2011-08-12' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1078 , 'counts_orders' , 'date' , '2011-08-13' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1079 , 'counts_orders' , 'date' , '2011-08-14' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1080 , 'counts_orders' , 'date' , '2011-08-15' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1081 , 'counts_orders' , 'date' , '2011-08-16' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1082 , 'counts_orders' , 'date' , '2011-08-17' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1083 , 'counts_orders' , 'date' , '2011-08-18' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1084 , 'counts_users' , 'date' , '2011-08-19' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1085 , 'counts_users' , 'date' , '2011-08-20' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1086 , 'counts_users' , 'date' , '2011-08-21' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1087 , 'counts_users' , 'date' , '2011-08-22' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1088 , 'counts_users' , 'date' , '2011-08-23' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1089 , 'counts_users' , 'date' , '2011-08-24' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1090 , 'counts_users' , 'date' , '2011-08-25' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1091 , 'counts_users' , 'date' , '2011-08-26' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1092 , 'counts_users' , 'date' , '2011-08-27' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1093 , 'counts_users' , 'date' , '2011-08-28' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1094 , 'counts_users' , 'date' , '2011-08-29' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1095 , 'counts_users' , 'date' , '2011-08-30' , '2011-08-31 13:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1096 , 'counts_orders' , 'date' , '2011-08-19' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1097 , 'counts_orders' , 'date' , '2011-08-20' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1098 , 'counts_orders' , 'date' , '2011-08-21' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1099 , 'counts_orders' , 'date' , '2011-08-22' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1100 , 'counts_orders' , 'date' , '2011-08-23' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1101 , 'counts_orders' , 'date' , '2011-08-24' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1102 , 'counts_orders' , 'date' , '2011-08-25' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1103 , 'counts_orders' , 'date' , '2011-08-26' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1104 , 'counts_orders' , 'date' , '2011-08-27' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1105 , 'counts_orders' , 'date' , '2011-08-28' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1106 , 'counts_orders' , 'date' , '2011-08-29' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1107 , 'counts_orders' , 'date' , '2011-08-30' , '2011-08-31 13:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1108 , 'counts_visits' , 'date' , '2011-08-19' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1109 , 'counts_visits' , 'date' , '2011-08-20' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1110 , 'counts_visits' , 'date' , '2011-08-21' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1111 , 'counts_visits' , 'date' , '2011-08-22' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1112 , 'counts_visits' , 'date' , '2011-08-23' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1113 , 'counts_visits' , 'date' , '2011-08-24' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1114 , 'counts_visits' , 'date' , '2011-08-25' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1115 , 'counts_visits' , 'date' , '2011-08-26' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1116 , 'counts_visits' , 'date' , '2011-08-27' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1117 , 'counts_visits' , 'date' , '2011-08-28' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1118 , 'counts_visits' , 'date' , '2011-08-29' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1119 , 'counts_visits' , 'date' , '2011-08-30' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1120 , 'counts_sales' , 'date' , '2011-08-19' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1121 , 'counts_sales' , 'date' , '2011-08-20' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1122 , 'counts_sales' , 'date' , '2011-08-21' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1123 , 'counts_sales' , 'date' , '2011-08-22' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1124 , 'counts_sales' , 'date' , '2011-08-23' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1125 , 'counts_sales' , 'date' , '2011-08-24' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1126 , 'counts_sales' , 'date' , '2011-08-25' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1127 , 'counts_sales' , 'date' , '2011-08-26' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1128 , 'counts_sales' , 'date' , '2011-08-27' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1129 , 'counts_sales' , 'date' , '2011-08-28' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1130 , 'counts_sales' , 'date' , '2011-08-29' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1131 , 'counts_sales' , 'date' , '2011-08-30' , '2011-08-31 13:26:55' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1132 , 'counts_users' , 'date' , '2012-04-15' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1133 , 'counts_users' , 'date' , '2012-04-16' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1134 , 'counts_users' , 'date' , '2012-04-17' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1135 , 'counts_users' , 'date' , '2012-04-18' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1136 , 'counts_users' , 'date' , '2012-04-19' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1137 , 'counts_users' , 'date' , '2012-04-20' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1138 , 'counts_users' , 'date' , '2012-04-21' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1139 , 'counts_users' , 'date' , '2012-04-22' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1140 , 'counts_users' , 'date' , '2012-04-23' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1141 , 'counts_users' , 'date' , '2012-04-24' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1142 , 'counts_users' , 'date' , '2012-04-25' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1143 , 'counts_users' , 'date' , '2012-04-26' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1144 , 'counts_users' , 'date' , '2012-04-27' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1145 , 'counts_users' , 'date' , '2012-04-28' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1146 , 'counts_users' , 'date' , '2012-04-29' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1147 , 'counts_users' , 'date' , '2012-04-30' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1148 , 'counts_users' , 'date' , '2012-05-01' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1149 , 'counts_users' , 'date' , '2012-05-02' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1150 , 'counts_users' , 'date' , '2012-05-03' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1151 , 'counts_users' , 'date' , '2012-05-04' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1152 , 'counts_users' , 'date' , '2012-05-05' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1153 , 'counts_users' , 'date' , '2012-05-06' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1154 , 'counts_users' , 'date' , '2012-05-07' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1155 , 'counts_users' , 'date' , '2012-05-08' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1156 , 'counts_users' , 'date' , '2012-05-09' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1157 , 'counts_users' , 'date' , '2012-05-10' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1158 , 'counts_users' , 'date' , '2012-05-11' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1159 , 'counts_users' , 'date' , '2012-05-12' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1160 , 'counts_users' , 'date' , '2012-05-13' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1161 , 'counts_users' , 'date' , '2012-05-14' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1162 , 'counts_orders' , 'date' , '2012-04-15' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1163 , 'counts_orders' , 'date' , '2012-04-16' , '2012-05-15 16:00:37' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1164 , 'counts_orders' , 'date' , '2012-04-17' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1165 , 'counts_orders' , 'date' , '2012-04-18' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1166 , 'counts_orders' , 'date' , '2012-04-19' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1167 , 'counts_orders' , 'date' , '2012-04-20' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1168 , 'counts_orders' , 'date' , '2012-04-21' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1169 , 'counts_orders' , 'date' , '2012-04-22' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1170 , 'counts_orders' , 'date' , '2012-04-23' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1171 , 'counts_orders' , 'date' , '2012-04-24' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1172 , 'counts_orders' , 'date' , '2012-04-25' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1173 , 'counts_orders' , 'date' , '2012-04-26' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1174 , 'counts_orders' , 'date' , '2012-04-27' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1175 , 'counts_orders' , 'date' , '2012-04-28' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1176 , 'counts_orders' , 'date' , '2012-04-29' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1177 , 'counts_orders' , 'date' , '2012-04-30' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1178 , 'counts_orders' , 'date' , '2012-05-01' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1179 , 'counts_orders' , 'date' , '2012-05-02' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1180 , 'counts_orders' , 'date' , '2012-05-03' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1181 , 'counts_orders' , 'date' , '2012-05-04' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1182 , 'counts_orders' , 'date' , '2012-05-05' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1183 , 'counts_orders' , 'date' , '2012-05-06' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1184 , 'counts_orders' , 'date' , '2012-05-07' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1185 , 'counts_orders' , 'date' , '2012-05-08' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1186 , 'counts_orders' , 'date' , '2012-05-09' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1187 , 'counts_orders' , 'date' , '2012-05-10' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1188 , 'counts_orders' , 'date' , '2012-05-11' , '2012-06-10 22:57:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1189 , 'counts_orders' , 'date' , '2012-05-12' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1190 , 'counts_orders' , 'date' , '2012-05-13' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1191 , 'counts_orders' , 'date' , '2012-05-14' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1192 , 'counts_sales' , 'date' , '2012-04-15' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1193 , 'counts_sales' , 'date' , '2012-04-16' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1194 , 'counts_sales' , 'date' , '2012-04-17' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1195 , 'counts_sales' , 'date' , '2012-04-18' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1196 , 'counts_sales' , 'date' , '2012-04-19' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1197 , 'counts_sales' , 'date' , '2012-04-20' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1198 , 'counts_sales' , 'date' , '2012-04-21' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1199 , 'counts_sales' , 'date' , '2012-04-22' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1200 , 'counts_sales' , 'date' , '2012-04-23' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1201 , 'counts_sales' , 'date' , '2012-04-24' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1202 , 'counts_sales' , 'date' , '2012-04-25' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1203 , 'counts_sales' , 'date' , '2012-04-26' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1204 , 'counts_sales' , 'date' , '2012-04-27' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1205 , 'counts_sales' , 'date' , '2012-04-28' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1206 , 'counts_sales' , 'date' , '2012-04-29' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1207 , 'counts_sales' , 'date' , '2012-04-30' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1208 , 'counts_sales' , 'date' , '2012-05-01' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1209 , 'counts_sales' , 'date' , '2012-05-02' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1210 , 'counts_sales' , 'date' , '2012-05-03' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1211 , 'counts_sales' , 'date' , '2012-05-04' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1212 , 'counts_sales' , 'date' , '2012-05-05' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1213 , 'counts_sales' , 'date' , '2012-05-06' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1214 , 'counts_sales' , 'date' , '2012-05-07' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1215 , 'counts_sales' , 'date' , '2012-05-08' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1216 , 'counts_sales' , 'date' , '2012-05-09' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1217 , 'counts_sales' , 'date' , '2012-05-10' , '2012-05-15 16:00:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1218 , 'counts_sales' , 'date' , '2012-05-11' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1219 , 'counts_sales' , 'date' , '2012-05-12' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1220 , 'counts_sales' , 'date' , '2012-05-13' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1221 , 'counts_sales' , 'date' , '2012-05-14' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1222 , 'counts_visits' , 'date' , '2012-04-15' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1223 , 'counts_visits' , 'date' , '2012-04-16' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1224 , 'counts_visits' , 'date' , '2012-04-17' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1225 , 'counts_visits' , 'date' , '2012-04-18' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1226 , 'counts_visits' , 'date' , '2012-04-19' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1227 , 'counts_visits' , 'date' , '2012-04-20' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1228 , 'counts_visits' , 'date' , '2012-04-21' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1229 , 'counts_visits' , 'date' , '2012-04-22' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1230 , 'counts_visits' , 'date' , '2012-04-23' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1231 , 'counts_visits' , 'date' , '2012-04-24' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1232 , 'counts_visits' , 'date' , '2012-04-25' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1233 , 'counts_visits' , 'date' , '2012-04-26' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1234 , 'counts_visits' , 'date' , '2012-04-27' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1235 , 'counts_visits' , 'date' , '2012-04-28' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1236 , 'counts_visits' , 'date' , '2012-04-29' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1237 , 'counts_visits' , 'date' , '2012-04-30' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1238 , 'counts_visits' , 'date' , '2012-05-01' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1239 , 'counts_visits' , 'date' , '2012-05-02' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1240 , 'counts_visits' , 'date' , '2012-05-03' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1241 , 'counts_visits' , 'date' , '2012-05-04' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1242 , 'counts_visits' , 'date' , '2012-05-05' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1243 , 'counts_visits' , 'date' , '2012-05-06' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1244 , 'counts_visits' , 'date' , '2012-05-07' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1245 , 'counts_visits' , 'date' , '2012-05-08' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1246 , 'counts_visits' , 'date' , '2012-05-09' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1247 , 'counts_visits' , 'date' , '2012-05-10' , '2012-05-15 16:00:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1248 , 'counts_visits' , 'date' , '2012-05-11' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1249 , 'counts_visits' , 'date' , '2012-05-12' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1250 , 'counts_visits' , 'date' , '2012-05-13' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1251 , 'counts_visits' , 'date' , '2012-05-14' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1252 , 'counts_orders' , 'date' , '2012-05-15' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1253 , 'counts_orders' , 'date' , '2012-05-16' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1254 , 'counts_orders' , 'date' , '2012-05-17' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1255 , 'counts_orders' , 'date' , '2012-05-18' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1256 , 'counts_orders' , 'date' , '2012-05-19' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1257 , 'counts_orders' , 'date' , '2012-05-20' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1258 , 'counts_orders' , 'date' , '2012-05-21' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1259 , 'counts_orders' , 'date' , '2012-05-22' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1260 , 'counts_orders' , 'date' , '2012-05-23' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1261 , 'counts_orders' , 'date' , '2012-05-24' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1262 , 'counts_orders' , 'date' , '2012-05-25' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1263 , 'counts_orders' , 'date' , '2012-05-26' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1264 , 'counts_orders' , 'date' , '2012-05-27' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1265 , 'counts_orders' , 'date' , '2012-05-28' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1266 , 'counts_orders' , 'date' , '2012-05-29' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1267 , 'counts_orders' , 'date' , '2012-05-30' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1268 , 'counts_orders' , 'date' , '2012-05-31' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1269 , 'counts_orders' , 'date' , '2012-06-01' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1270 , 'counts_orders' , 'date' , '2012-06-02' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1271 , 'counts_orders' , 'date' , '2012-06-03' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1272 , 'counts_orders' , 'date' , '2012-06-04' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1273 , 'counts_orders' , 'date' , '2012-06-05' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1274 , 'counts_orders' , 'date' , '2012-06-06' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1275 , 'counts_orders' , 'date' , '2012-06-07' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1276 , 'counts_orders' , 'date' , '2012-06-08' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1277 , 'counts_orders' , 'date' , '2012-06-09' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1278 , 'counts_users' , 'date' , '2012-05-15' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1279 , 'counts_users' , 'date' , '2012-05-16' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1280 , 'counts_users' , 'date' , '2012-05-17' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1281 , 'counts_users' , 'date' , '2012-05-18' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1282 , 'counts_users' , 'date' , '2012-05-19' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1283 , 'counts_users' , 'date' , '2012-05-20' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1284 , 'counts_users' , 'date' , '2012-05-21' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1285 , 'counts_users' , 'date' , '2012-05-22' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1286 , 'counts_users' , 'date' , '2012-05-23' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1287 , 'counts_users' , 'date' , '2012-05-24' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1288 , 'counts_users' , 'date' , '2012-05-25' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1289 , 'counts_users' , 'date' , '2012-05-26' , '2012-06-10 22:57:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1290 , 'counts_users' , 'date' , '2012-05-27' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1291 , 'counts_users' , 'date' , '2012-05-28' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1292 , 'counts_users' , 'date' , '2012-05-29' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1293 , 'counts_users' , 'date' , '2012-05-30' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1294 , 'counts_users' , 'date' , '2012-05-31' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1295 , 'counts_users' , 'date' , '2012-06-01' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1296 , 'counts_users' , 'date' , '2012-06-02' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1297 , 'counts_users' , 'date' , '2012-06-03' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1298 , 'counts_users' , 'date' , '2012-06-04' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1299 , 'counts_users' , 'date' , '2012-06-05' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1300 , 'counts_users' , 'date' , '2012-06-06' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1301 , 'counts_users' , 'date' , '2012-06-07' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1302 , 'counts_users' , 'date' , '2012-06-08' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1303 , 'counts_users' , 'date' , '2012-06-09' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1304 , 'counts_visits' , 'date' , '2012-05-15' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1305 , 'counts_visits' , 'date' , '2012-05-16' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1306 , 'counts_visits' , 'date' , '2012-05-17' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1307 , 'counts_visits' , 'date' , '2012-05-18' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1308 , 'counts_visits' , 'date' , '2012-05-19' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1309 , 'counts_visits' , 'date' , '2012-05-20' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1310 , 'counts_visits' , 'date' , '2012-05-21' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1311 , 'counts_visits' , 'date' , '2012-05-22' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1312 , 'counts_visits' , 'date' , '2012-05-23' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1313 , 'counts_visits' , 'date' , '2012-05-24' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1314 , 'counts_visits' , 'date' , '2012-05-25' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1315 , 'counts_visits' , 'date' , '2012-05-26' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1316 , 'counts_visits' , 'date' , '2012-05-27' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1317 , 'counts_visits' , 'date' , '2012-05-28' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1318 , 'counts_visits' , 'date' , '2012-05-29' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1319 , 'counts_visits' , 'date' , '2012-05-30' , '2012-06-29 14:11:18' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1320 , 'counts_visits' , 'date' , '2012-05-31' , '2012-06-29 14:11:18' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1321 , 'counts_visits' , 'date' , '2012-06-01' , '2012-06-29 14:11:18' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1322 , 'counts_visits' , 'date' , '2012-06-02' , '2012-06-29 14:11:18' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1323 , 'counts_visits' , 'date' , '2012-06-03' , '2012-06-29 14:11:18' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1324 , 'counts_visits' , 'date' , '2012-06-04' , '2012-06-29 14:11:18' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1325 , 'counts_visits' , 'date' , '2012-06-05' , '2012-06-29 14:11:18' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1326 , 'counts_visits' , 'date' , '2012-06-06' , '2012-06-29 14:11:18' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1327 , 'counts_visits' , 'date' , '2012-06-07' , '2012-06-29 14:11:18' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1328 , 'counts_visits' , 'date' , '2012-06-08' , '2012-06-29 14:11:18' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1329 , 'counts_visits' , 'date' , '2012-06-09' , '2012-06-29 14:11:18' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1330 , 'counts_sales' , 'date' , '2012-05-15' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1331 , 'counts_sales' , 'date' , '2012-05-16' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1332 , 'counts_sales' , 'date' , '2012-05-17' , '2012-06-10 22:57:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1333 , 'counts_sales' , 'date' , '2012-05-18' , '2012-06-10 22:57:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1334 , 'counts_sales' , 'date' , '2012-05-19' , '2012-06-10 22:57:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1335 , 'counts_sales' , 'date' , '2012-05-20' , '2012-06-10 22:57:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1336 , 'counts_sales' , 'date' , '2012-05-21' , '2012-06-10 22:57:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1337 , 'counts_sales' , 'date' , '2012-05-22' , '2012-06-10 22:57:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1338 , 'counts_sales' , 'date' , '2012-05-23' , '2012-06-10 22:57:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1339 , 'counts_sales' , 'date' , '2012-05-24' , '2012-06-10 22:57:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1340 , 'counts_sales' , 'date' , '2012-05-25' , '2012-06-10 22:57:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1341 , 'counts_sales' , 'date' , '2012-05-26' , '2012-06-10 22:57:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1342 , 'counts_sales' , 'date' , '2012-05-27' , '2012-06-10 22:57:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1343 , 'counts_sales' , 'date' , '2012-05-28' , '2012-06-10 22:57:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1344 , 'counts_sales' , 'date' , '2012-05-29' , '2012-06-10 22:57:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1345 , 'counts_sales' , 'date' , '2012-05-30' , '2012-06-29 14:11:20' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1346 , 'counts_sales' , 'date' , '2012-05-31' , '2012-06-29 14:11:20' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1347 , 'counts_sales' , 'date' , '2012-06-01' , '2012-06-29 14:11:20' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1348 , 'counts_sales' , 'date' , '2012-06-02' , '2012-06-29 14:11:20' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1349 , 'counts_sales' , 'date' , '2012-06-03' , '2012-06-29 14:11:20' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1350 , 'counts_sales' , 'date' , '2012-06-04' , '2012-06-29 14:11:20' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1351 , 'counts_sales' , 'date' , '2012-06-05' , '2012-06-29 14:11:20' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1352 , 'counts_sales' , 'date' , '2012-06-06' , '2012-06-29 14:11:20' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1353 , 'counts_sales' , 'date' , '2012-06-07' , '2012-06-29 14:11:20' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1354 , 'counts_sales' , 'date' , '2012-06-08' , '2012-06-29 14:11:20' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1355 , 'counts_sales' , 'date' , '2012-06-09' , '2012-06-29 14:11:20' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1356 , 'counts_visits' , 'date' , '2012-06-10' , '2012-06-29 14:11:18' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1357 , 'counts_visits' , 'date' , '2012-06-11' , '2012-06-29 14:11:18' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1358 , 'counts_visits' , 'date' , '2012-06-12' , '2012-06-29 14:11:18' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1359 , 'counts_visits' , 'date' , '2012-06-13' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1360 , 'counts_visits' , 'date' , '2012-06-14' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1361 , 'counts_visits' , 'date' , '2012-06-15' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1362 , 'counts_visits' , 'date' , '2012-06-16' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1363 , 'counts_visits' , 'date' , '2012-06-17' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1364 , 'counts_visits' , 'date' , '2012-06-18' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1365 , 'counts_visits' , 'date' , '2012-06-19' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1366 , 'counts_visits' , 'date' , '2012-06-20' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1367 , 'counts_visits' , 'date' , '2012-06-21' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1368 , 'counts_visits' , 'date' , '2012-06-22' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1369 , 'counts_visits' , 'date' , '2012-06-23' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1370 , 'counts_visits' , 'date' , '2012-06-24' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1371 , 'counts_visits' , 'date' , '2012-06-25' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1372 , 'counts_visits' , 'date' , '2012-06-26' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1373 , 'counts_visits' , 'date' , '2012-06-27' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1374 , 'counts_visits' , 'date' , '2012-06-28' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1375 , 'counts_orders' , 'date' , '2012-06-10' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1376 , 'counts_orders' , 'date' , '2012-06-11' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1377 , 'counts_orders' , 'date' , '2012-06-12' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1378 , 'counts_orders' , 'date' , '2012-06-13' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1379 , 'counts_orders' , 'date' , '2012-06-14' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1380 , 'counts_orders' , 'date' , '2012-06-15' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1381 , 'counts_orders' , 'date' , '2012-06-16' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1382 , 'counts_orders' , 'date' , '2012-06-17' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1383 , 'counts_orders' , 'date' , '2012-06-18' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1384 , 'counts_orders' , 'date' , '2012-06-19' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1385 , 'counts_orders' , 'date' , '2012-06-20' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1386 , 'counts_orders' , 'date' , '2012-06-21' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1387 , 'counts_orders' , 'date' , '2012-06-22' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1388 , 'counts_orders' , 'date' , '2012-06-23' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1389 , 'counts_orders' , 'date' , '2012-06-24' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1390 , 'counts_orders' , 'date' , '2012-06-25' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1391 , 'counts_orders' , 'date' , '2012-06-26' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1392 , 'counts_orders' , 'date' , '2012-06-27' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1393 , 'counts_orders' , 'date' , '2012-06-28' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1394 , 'counts_users' , 'date' , '2012-06-10' , '2012-06-29 14:11:19' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1395 , 'counts_users' , 'date' , '2012-06-11' , '2012-06-29 14:11:20' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1396 , 'counts_users' , 'date' , '2012-06-12' , '2012-06-29 14:11:20' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1397 , 'counts_users' , 'date' , '2012-06-13' , '2012-07-13 16:44:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1398 , 'counts_users' , 'date' , '2012-06-14' , '2012-07-13 16:44:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1399 , 'counts_users' , 'date' , '2012-06-15' , '2012-07-13 16:44:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1400 , 'counts_users' , 'date' , '2012-06-16' , '2012-07-13 16:44:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1401 , 'counts_users' , 'date' , '2012-06-17' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1402 , 'counts_users' , 'date' , '2012-06-18' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1403 , 'counts_users' , 'date' , '2012-06-19' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1404 , 'counts_users' , 'date' , '2012-06-20' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1405 , 'counts_users' , 'date' , '2012-06-21' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1406 , 'counts_users' , 'date' , '2012-06-22' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1407 , 'counts_users' , 'date' , '2012-06-23' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1408 , 'counts_users' , 'date' , '2012-06-24' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1409 , 'counts_users' , 'date' , '2012-06-25' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1410 , 'counts_users' , 'date' , '2012-06-26' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1411 , 'counts_users' , 'date' , '2012-06-27' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1412 , 'counts_users' , 'date' , '2012-06-28' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1413 , 'counts_sales' , 'date' , '2012-06-10' , '2012-06-29 14:11:20' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1414 , 'counts_sales' , 'date' , '2012-06-11' , '2012-06-29 14:11:20' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1415 , 'counts_sales' , 'date' , '2012-06-12' , '2012-06-29 14:11:20' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1416 , 'counts_sales' , 'date' , '2012-06-13' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1417 , 'counts_sales' , 'date' , '2012-06-14' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1418 , 'counts_sales' , 'date' , '2012-06-15' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1419 , 'counts_sales' , 'date' , '2012-06-16' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1420 , 'counts_sales' , 'date' , '2012-06-17' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1421 , 'counts_sales' , 'date' , '2012-06-18' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1422 , 'counts_sales' , 'date' , '2012-06-19' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1423 , 'counts_sales' , 'date' , '2012-06-20' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1424 , 'counts_sales' , 'date' , '2012-06-21' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1425 , 'counts_sales' , 'date' , '2012-06-22' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1426 , 'counts_sales' , 'date' , '2012-06-23' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1427 , 'counts_sales' , 'date' , '2012-06-24' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1428 , 'counts_sales' , 'date' , '2012-06-25' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1429 , 'counts_sales' , 'date' , '2012-06-26' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1430 , 'counts_sales' , 'date' , '2012-06-27' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1431 , 'counts_sales' , 'date' , '2012-06-28' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1432 , 'counts_users' , 'date' , '2012-06-29' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1433 , 'counts_users' , 'date' , '2012-06-30' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1434 , 'counts_users' , 'date' , '2012-07-01' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1435 , 'counts_users' , 'date' , '2012-07-02' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1436 , 'counts_users' , 'date' , '2012-07-03' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1437 , 'counts_users' , 'date' , '2012-07-04' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1438 , 'counts_users' , 'date' , '2012-07-05' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1439 , 'counts_users' , 'date' , '2012-07-06' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1440 , 'counts_users' , 'date' , '2012-07-07' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1441 , 'counts_users' , 'date' , '2012-07-08' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1442 , 'counts_users' , 'date' , '2012-07-09' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1443 , 'counts_users' , 'date' , '2012-07-10' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1444 , 'counts_users' , 'date' , '2012-07-11' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1445 , 'counts_users' , 'date' , '2012-07-12' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1446 , 'counts_visits' , 'date' , '2012-06-29' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1447 , 'counts_visits' , 'date' , '2012-06-30' , '2012-07-13 16:44:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1448 , 'counts_visits' , 'date' , '2012-07-01' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1449 , 'counts_visits' , 'date' , '2012-07-02' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1450 , 'counts_visits' , 'date' , '2012-07-03' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1451 , 'counts_visits' , 'date' , '2012-07-04' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1452 , 'counts_visits' , 'date' , '2012-07-05' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1453 , 'counts_visits' , 'date' , '2012-07-06' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1454 , 'counts_visits' , 'date' , '2012-07-07' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1455 , 'counts_visits' , 'date' , '2012-07-08' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1456 , 'counts_visits' , 'date' , '2012-07-09' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1457 , 'counts_visits' , 'date' , '2012-07-10' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1458 , 'counts_visits' , 'date' , '2012-07-11' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1459 , 'counts_visits' , 'date' , '2012-07-12' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1460 , 'counts_sales' , 'date' , '2012-06-29' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1461 , 'counts_sales' , 'date' , '2012-06-30' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1462 , 'counts_sales' , 'date' , '2012-07-01' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1463 , 'counts_sales' , 'date' , '2012-07-02' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1464 , 'counts_sales' , 'date' , '2012-07-03' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1465 , 'counts_sales' , 'date' , '2012-07-04' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1466 , 'counts_sales' , 'date' , '2012-07-05' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1467 , 'counts_sales' , 'date' , '2012-07-06' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1468 , 'counts_sales' , 'date' , '2012-07-07' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1469 , 'counts_sales' , 'date' , '2012-07-08' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1470 , 'counts_sales' , 'date' , '2012-07-09' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1471 , 'counts_sales' , 'date' , '2012-07-10' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1472 , 'counts_sales' , 'date' , '2012-07-11' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1473 , 'counts_sales' , 'date' , '2012-07-12' , '2012-07-13 16:44:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1474 , 'counts_orders' , 'date' , '2012-06-29' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1475 , 'counts_orders' , 'date' , '2012-06-30' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1476 , 'counts_orders' , 'date' , '2012-07-01' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1477 , 'counts_orders' , 'date' , '2012-07-02' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1478 , 'counts_orders' , 'date' , '2012-07-03' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1479 , 'counts_orders' , 'date' , '2012-07-04' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1480 , 'counts_orders' , 'date' , '2012-07-05' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1481 , 'counts_orders' , 'date' , '2012-07-06' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1482 , 'counts_orders' , 'date' , '2012-07-07' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1483 , 'counts_orders' , 'date' , '2012-07-08' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1484 , 'counts_orders' , 'date' , '2012-07-09' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1485 , 'counts_orders' , 'date' , '2012-07-10' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1486 , 'counts_orders' , 'date' , '2012-07-11' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1487 , 'counts_orders' , 'date' , '2012-07-12' , '2012-07-13 16:44:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1488 , 'counts_users' , 'date' , '2012-08-02' , '2012-09-01 02:17:12' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1489 , 'counts_users' , 'date' , '2012-08-03' , '2012-09-01 02:17:12' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1490 , 'counts_users' , 'date' , '2012-08-04' , '2012-09-01 02:17:12' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1491 , 'counts_users' , 'date' , '2012-08-05' , '2012-09-01 02:17:12' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1492 , 'counts_users' , 'date' , '2012-08-06' , '2012-09-01 02:17:12' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1493 , 'counts_users' , 'date' , '2012-08-07' , '2012-09-01 02:17:12' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1494 , 'counts_users' , 'date' , '2012-08-08' , '2012-09-01 02:17:12' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1495 , 'counts_users' , 'date' , '2012-08-09' , '2012-09-01 02:17:12' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1496 , 'counts_users' , 'date' , '2012-08-10' , '2012-09-01 02:17:12' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1497 , 'counts_users' , 'date' , '2012-08-11' , '2012-09-01 02:17:12' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1498 , 'counts_users' , 'date' , '2012-08-12' , '2012-09-01 02:17:12' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1499 , 'counts_users' , 'date' , '2012-08-13' , '2012-09-01 02:17:12' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1500 , 'counts_users' , 'date' , '2012-08-14' , '2012-09-01 02:17:12' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1501 , 'counts_users' , 'date' , '2012-08-15' , '2012-09-01 02:17:12' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1502 , 'counts_users' , 'date' , '2012-08-16' , '2012-09-01 02:17:12' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1503 , 'counts_users' , 'date' , '2012-08-17' , '2012-09-01 02:17:12' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1504 , 'counts_users' , 'date' , '2012-08-18' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1505 , 'counts_users' , 'date' , '2012-08-19' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1506 , 'counts_users' , 'date' , '2012-08-20' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1507 , 'counts_users' , 'date' , '2012-08-21' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1508 , 'counts_users' , 'date' , '2012-08-22' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1509 , 'counts_users' , 'date' , '2012-08-23' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1510 , 'counts_users' , 'date' , '2012-08-24' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1511 , 'counts_users' , 'date' , '2012-08-25' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1512 , 'counts_users' , 'date' , '2012-08-26' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1513 , 'counts_users' , 'date' , '2012-08-27' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1514 , 'counts_users' , 'date' , '2012-08-28' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1515 , 'counts_users' , 'date' , '2012-08-29' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1516 , 'counts_users' , 'date' , '2012-08-30' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1517 , 'counts_users' , 'date' , '2012-08-31' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1518 , 'counts_orders' , 'date' , '2012-08-02' , '2012-09-01 02:17:12' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1519 , 'counts_orders' , 'date' , '2012-08-03' , '2012-09-01 02:17:12' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1520 , 'counts_orders' , 'date' , '2012-08-04' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1521 , 'counts_orders' , 'date' , '2012-08-05' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1522 , 'counts_orders' , 'date' , '2012-08-06' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1523 , 'counts_orders' , 'date' , '2012-08-07' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1524 , 'counts_orders' , 'date' , '2012-08-08' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1525 , 'counts_orders' , 'date' , '2012-08-09' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1526 , 'counts_orders' , 'date' , '2012-08-10' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1527 , 'counts_orders' , 'date' , '2012-08-11' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1528 , 'counts_orders' , 'date' , '2012-08-12' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1529 , 'counts_orders' , 'date' , '2012-08-13' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1530 , 'counts_orders' , 'date' , '2012-08-14' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1531 , 'counts_orders' , 'date' , '2012-08-15' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1532 , 'counts_orders' , 'date' , '2012-08-16' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1533 , 'counts_orders' , 'date' , '2012-08-17' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1534 , 'counts_orders' , 'date' , '2012-08-18' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1535 , 'counts_orders' , 'date' , '2012-08-19' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1536 , 'counts_orders' , 'date' , '2012-08-20' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1537 , 'counts_orders' , 'date' , '2012-08-21' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1538 , 'counts_orders' , 'date' , '2012-08-22' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1539 , 'counts_orders' , 'date' , '2012-08-23' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1540 , 'counts_orders' , 'date' , '2012-08-24' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1541 , 'counts_orders' , 'date' , '2012-08-25' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1542 , 'counts_orders' , 'date' , '2012-08-26' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1543 , 'counts_orders' , 'date' , '2012-08-27' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1544 , 'counts_orders' , 'date' , '2012-08-28' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1545 , 'counts_orders' , 'date' , '2012-08-29' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1546 , 'counts_orders' , 'date' , '2012-08-30' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1547 , 'counts_orders' , 'date' , '2012-08-31' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1548 , 'counts_sales' , 'date' , '2012-08-02' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1549 , 'counts_sales' , 'date' , '2012-08-03' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1550 , 'counts_sales' , 'date' , '2012-08-04' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1551 , 'counts_sales' , 'date' , '2012-08-05' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1552 , 'counts_sales' , 'date' , '2012-08-06' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1553 , 'counts_sales' , 'date' , '2012-08-07' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1554 , 'counts_sales' , 'date' , '2012-08-08' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1555 , 'counts_sales' , 'date' , '2012-08-09' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1556 , 'counts_sales' , 'date' , '2012-08-10' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1557 , 'counts_sales' , 'date' , '2012-08-11' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1558 , 'counts_sales' , 'date' , '2012-08-12' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1559 , 'counts_sales' , 'date' , '2012-08-13' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1560 , 'counts_sales' , 'date' , '2012-08-14' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1561 , 'counts_sales' , 'date' , '2012-08-15' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1562 , 'counts_sales' , 'date' , '2012-08-16' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1563 , 'counts_sales' , 'date' , '2012-08-17' , '2012-09-01 02:17:13' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1564 , 'counts_sales' , 'date' , '2012-08-18' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1565 , 'counts_sales' , 'date' , '2012-08-19' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1566 , 'counts_sales' , 'date' , '2012-08-20' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1567 , 'counts_sales' , 'date' , '2012-08-21' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1568 , 'counts_sales' , 'date' , '2012-08-22' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1569 , 'counts_sales' , 'date' , '2012-08-23' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1570 , 'counts_sales' , 'date' , '2012-08-24' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1571 , 'counts_sales' , 'date' , '2012-08-25' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1572 , 'counts_sales' , 'date' , '2012-08-26' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1573 , 'counts_sales' , 'date' , '2012-08-27' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1574 , 'counts_sales' , 'date' , '2012-08-28' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1575 , 'counts_sales' , 'date' , '2012-08-29' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1576 , 'counts_sales' , 'date' , '2012-08-30' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1577 , 'counts_sales' , 'date' , '2012-08-31' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1578 , 'counts_visits' , 'date' , '2012-08-02' , '2012-09-01 02:17:14' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1579 , 'counts_visits' , 'date' , '2012-08-03' , '2012-09-01 02:17:14' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1580 , 'counts_visits' , 'date' , '2012-08-04' , '2012-09-01 02:17:14' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1581 , 'counts_visits' , 'date' , '2012-08-05' , '2012-09-01 02:17:14' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1582 , 'counts_visits' , 'date' , '2012-08-06' , '2012-09-01 02:17:14' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1583 , 'counts_visits' , 'date' , '2012-08-07' , '2012-09-01 02:17:14' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1584 , 'counts_visits' , 'date' , '2012-08-08' , '2012-09-01 02:17:14' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1585 , 'counts_visits' , 'date' , '2012-08-09' , '2012-09-01 02:17:14' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1586 , 'counts_visits' , 'date' , '2012-08-10' , '2012-09-01 02:17:14' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1587 , 'counts_visits' , 'date' , '2012-08-11' , '2012-09-01 02:17:14' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1588 , 'counts_visits' , 'date' , '2012-08-12' , '2012-09-01 02:17:14' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1589 , 'counts_visits' , 'date' , '2012-08-13' , '2012-09-01 02:17:14' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1590 , 'counts_visits' , 'date' , '2012-08-14' , '2012-09-01 02:17:14' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1591 , 'counts_visits' , 'date' , '2012-08-15' , '2012-09-01 02:17:14' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1592 , 'counts_visits' , 'date' , '2012-08-16' , '2012-09-01 02:17:14' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1593 , 'counts_visits' , 'date' , '2012-08-17' , '2012-09-01 02:17:14' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1594 , 'counts_visits' , 'date' , '2012-08-18' , '2012-09-17 04:28:03' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1595 , 'counts_visits' , 'date' , '2012-08-19' , '2012-09-17 04:28:03' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1596 , 'counts_visits' , 'date' , '2012-08-20' , '2012-09-17 04:28:03' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1597 , 'counts_visits' , 'date' , '2012-08-21' , '2012-09-17 04:28:03' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1598 , 'counts_visits' , 'date' , '2012-08-22' , '2012-09-17 04:28:03' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1599 , 'counts_visits' , 'date' , '2012-08-23' , '2012-09-17 04:28:03' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1600 , 'counts_visits' , 'date' , '2012-08-24' , '2012-09-17 04:28:03' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1601 , 'counts_visits' , 'date' , '2012-08-25' , '2012-09-17 04:28:03' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1602 , 'counts_visits' , 'date' , '2012-08-26' , '2012-09-17 04:28:03' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1603 , 'counts_visits' , 'date' , '2012-08-27' , '2012-09-17 04:28:03' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1604 , 'counts_visits' , 'date' , '2012-08-28' , '2012-09-17 04:28:03' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1605 , 'counts_visits' , 'date' , '2012-08-29' , '2012-09-17 04:28:03' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1606 , 'counts_visits' , 'date' , '2012-08-30' , '2012-09-17 04:28:03' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1607 , 'counts_visits' , 'date' , '2012-08-31' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1608 , 'counts_visits' , 'date' , '2012-09-01' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1609 , 'counts_visits' , 'date' , '2012-09-02' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1610 , 'counts_visits' , 'date' , '2012-09-03' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1611 , 'counts_visits' , 'date' , '2012-09-04' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1612 , 'counts_visits' , 'date' , '2012-09-05' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1613 , 'counts_visits' , 'date' , '2012-09-06' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1614 , 'counts_visits' , 'date' , '2012-09-07' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1615 , 'counts_visits' , 'date' , '2012-09-08' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1616 , 'counts_visits' , 'date' , '2012-09-09' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1617 , 'counts_visits' , 'date' , '2012-09-10' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1618 , 'counts_visits' , 'date' , '2012-09-11' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1619 , 'counts_visits' , 'date' , '2012-09-12' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1620 , 'counts_visits' , 'date' , '2012-09-13' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1621 , 'counts_visits' , 'date' , '2012-09-14' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1622 , 'counts_visits' , 'date' , '2012-09-15' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1623 , 'counts_visits' , 'date' , '2012-09-16' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1624 , 'counts_users' , 'date' , '2012-09-01' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1625 , 'counts_users' , 'date' , '2012-09-02' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1626 , 'counts_users' , 'date' , '2012-09-03' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1627 , 'counts_users' , 'date' , '2012-09-04' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1628 , 'counts_users' , 'date' , '2012-09-05' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1629 , 'counts_users' , 'date' , '2012-09-06' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1630 , 'counts_users' , 'date' , '2012-09-07' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1631 , 'counts_users' , 'date' , '2012-09-08' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1632 , 'counts_users' , 'date' , '2012-09-09' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1633 , 'counts_users' , 'date' , '2012-09-10' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1634 , 'counts_users' , 'date' , '2012-09-11' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1635 , 'counts_users' , 'date' , '2012-09-12' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1636 , 'counts_users' , 'date' , '2012-09-13' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1637 , 'counts_users' , 'date' , '2012-09-14' , '2012-09-17 04:28:04' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1638 , 'counts_users' , 'date' , '2012-09-15' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1639 , 'counts_users' , 'date' , '2012-09-16' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1640 , 'counts_sales' , 'date' , '2012-09-01' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1641 , 'counts_sales' , 'date' , '2012-09-02' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1642 , 'counts_sales' , 'date' , '2012-09-03' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1643 , 'counts_sales' , 'date' , '2012-09-04' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1644 , 'counts_sales' , 'date' , '2012-09-05' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1645 , 'counts_sales' , 'date' , '2012-09-06' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1646 , 'counts_sales' , 'date' , '2012-09-07' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1647 , 'counts_sales' , 'date' , '2012-09-08' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1648 , 'counts_sales' , 'date' , '2012-09-09' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1649 , 'counts_sales' , 'date' , '2012-09-10' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1650 , 'counts_sales' , 'date' , '2012-09-11' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1651 , 'counts_sales' , 'date' , '2012-09-12' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1652 , 'counts_sales' , 'date' , '2012-09-13' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1653 , 'counts_sales' , 'date' , '2012-09-14' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1654 , 'counts_sales' , 'date' , '2012-09-15' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1655 , 'counts_sales' , 'date' , '2012-09-16' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1656 , 'counts_orders' , 'date' , '2012-09-01' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1657 , 'counts_orders' , 'date' , '2012-09-02' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1658 , 'counts_orders' , 'date' , '2012-09-03' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1659 , 'counts_orders' , 'date' , '2012-09-04' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1660 , 'counts_orders' , 'date' , '2012-09-05' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1661 , 'counts_orders' , 'date' , '2012-09-06' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1662 , 'counts_orders' , 'date' , '2012-09-07' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1663 , 'counts_orders' , 'date' , '2012-09-08' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1664 , 'counts_orders' , 'date' , '2012-09-09' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1665 , 'counts_orders' , 'date' , '2012-09-10' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1666 , 'counts_orders' , 'date' , '2012-09-11' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1667 , 'counts_orders' , 'date' , '2012-09-12' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1668 , 'counts_orders' , 'date' , '2012-09-13' , '2012-09-17 04:28:05' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1669 , 'counts_orders' , 'date' , '2012-09-14' , '2012-09-17 04:28:06' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1670 , 'counts_orders' , 'date' , '2012-09-15' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1671 , 'counts_orders' , 'date' , '2012-09-16' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1672 , 'counts_users' , 'date' , '2012-09-17' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1673 , 'counts_visits' , 'date' , '2012-09-17' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1674 , 'counts_sales' , 'date' , '2012-09-17' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1675 , 'counts_orders' , 'date' , '2012-09-17' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1676 , 'counts_sales' , 'date' , '2012-09-18' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1677 , 'counts_sales' , 'date' , '2012-09-19' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1678 , 'counts_sales' , 'date' , '2012-09-20' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1679 , 'counts_sales' , 'date' , '2012-09-21' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1680 , 'counts_sales' , 'date' , '2012-09-22' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1681 , 'counts_sales' , 'date' , '2012-09-23' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1682 , 'counts_sales' , 'date' , '2012-09-24' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1683 , 'counts_sales' , 'date' , '2012-09-25' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1684 , 'counts_sales' , 'date' , '2012-09-26' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1685 , 'counts_sales' , 'date' , '2012-09-27' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1686 , 'counts_sales' , 'date' , '2012-09-28' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1687 , 'counts_sales' , 'date' , '2012-09-29' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1688 , 'counts_sales' , 'date' , '2012-09-30' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1689 , 'counts_sales' , 'date' , '2012-10-01' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1690 , 'counts_sales' , 'date' , '2012-10-02' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1691 , 'counts_sales' , 'date' , '2012-10-03' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1692 , 'counts_sales' , 'date' , '2012-10-04' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1693 , 'counts_sales' , 'date' , '2012-10-05' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1694 , 'counts_sales' , 'date' , '2012-10-06' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1695 , 'counts_sales' , 'date' , '2012-10-07' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1696 , 'counts_sales' , 'date' , '2012-10-08' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1697 , 'counts_sales' , 'date' , '2012-10-09' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1698 , 'counts_sales' , 'date' , '2012-10-10' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1699 , 'counts_sales' , 'date' , '2012-10-11' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1700 , 'counts_sales' , 'date' , '2012-10-12' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1701 , 'counts_sales' , 'date' , '2012-10-13' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1702 , 'counts_sales' , 'date' , '2012-10-14' , '2012-10-15 18:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1703 , 'counts_visits' , 'date' , '2012-09-18' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1704 , 'counts_visits' , 'date' , '2012-09-19' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1705 , 'counts_visits' , 'date' , '2012-09-20' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1706 , 'counts_visits' , 'date' , '2012-09-21' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1707 , 'counts_visits' , 'date' , '2012-09-22' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1708 , 'counts_visits' , 'date' , '2012-09-23' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1709 , 'counts_visits' , 'date' , '2012-09-24' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1710 , 'counts_visits' , 'date' , '2012-09-25' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1711 , 'counts_visits' , 'date' , '2012-09-26' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1712 , 'counts_visits' , 'date' , '2012-09-27' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1713 , 'counts_visits' , 'date' , '2012-09-28' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1714 , 'counts_visits' , 'date' , '2012-09-29' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1715 , 'counts_visits' , 'date' , '2012-09-30' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1716 , 'counts_visits' , 'date' , '2012-10-01' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1717 , 'counts_visits' , 'date' , '2012-10-02' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1718 , 'counts_visits' , 'date' , '2012-10-03' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1719 , 'counts_visits' , 'date' , '2012-10-04' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1720 , 'counts_visits' , 'date' , '2012-10-05' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1721 , 'counts_visits' , 'date' , '2012-10-06' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1722 , 'counts_visits' , 'date' , '2012-10-07' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1723 , 'counts_visits' , 'date' , '2012-10-08' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1724 , 'counts_visits' , 'date' , '2012-10-09' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1725 , 'counts_visits' , 'date' , '2012-10-10' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1726 , 'counts_visits' , 'date' , '2012-10-11' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1727 , 'counts_visits' , 'date' , '2012-10-12' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1728 , 'counts_visits' , 'date' , '2012-10-13' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1729 , 'counts_visits' , 'date' , '2012-10-14' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1730 , 'counts_users' , 'date' , '2012-09-18' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1731 , 'counts_users' , 'date' , '2012-09-19' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1732 , 'counts_users' , 'date' , '2012-09-20' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1733 , 'counts_users' , 'date' , '2012-09-21' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1734 , 'counts_users' , 'date' , '2012-09-22' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1735 , 'counts_users' , 'date' , '2012-09-23' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1736 , 'counts_users' , 'date' , '2012-09-24' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1737 , 'counts_users' , 'date' , '2012-09-25' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1738 , 'counts_users' , 'date' , '2012-09-26' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1739 , 'counts_users' , 'date' , '2012-09-27' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1740 , 'counts_users' , 'date' , '2012-09-28' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1741 , 'counts_users' , 'date' , '2012-09-29' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1742 , 'counts_users' , 'date' , '2012-09-30' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1743 , 'counts_users' , 'date' , '2012-10-01' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1744 , 'counts_users' , 'date' , '2012-10-02' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1745 , 'counts_users' , 'date' , '2012-10-03' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1746 , 'counts_users' , 'date' , '2012-10-04' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1747 , 'counts_users' , 'date' , '2012-10-05' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1748 , 'counts_users' , 'date' , '2012-10-06' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1749 , 'counts_users' , 'date' , '2012-10-07' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1750 , 'counts_users' , 'date' , '2012-10-08' , '2012-10-15 18:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1751 , 'counts_users' , 'date' , '2012-10-09' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1752 , 'counts_users' , 'date' , '2012-10-10' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1753 , 'counts_users' , 'date' , '2012-10-11' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1754 , 'counts_users' , 'date' , '2012-10-12' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1755 , 'counts_users' , 'date' , '2012-10-13' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1756 , 'counts_users' , 'date' , '2012-10-14' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1757 , 'counts_orders' , 'date' , '2012-09-18' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1758 , 'counts_orders' , 'date' , '2012-09-19' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1759 , 'counts_orders' , 'date' , '2012-09-20' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1760 , 'counts_orders' , 'date' , '2012-09-21' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1761 , 'counts_orders' , 'date' , '2012-09-22' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1762 , 'counts_orders' , 'date' , '2012-09-23' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1763 , 'counts_orders' , 'date' , '2012-09-24' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1764 , 'counts_orders' , 'date' , '2012-09-25' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1765 , 'counts_orders' , 'date' , '2012-09-26' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1766 , 'counts_orders' , 'date' , '2012-09-27' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1767 , 'counts_orders' , 'date' , '2012-09-28' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1768 , 'counts_orders' , 'date' , '2012-09-29' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1769 , 'counts_orders' , 'date' , '2012-09-30' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1770 , 'counts_orders' , 'date' , '2012-10-01' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1771 , 'counts_orders' , 'date' , '2012-10-02' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1772 , 'counts_orders' , 'date' , '2012-10-03' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1773 , 'counts_orders' , 'date' , '2012-10-04' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1774 , 'counts_orders' , 'date' , '2012-10-05' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1775 , 'counts_orders' , 'date' , '2012-10-06' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1776 , 'counts_orders' , 'date' , '2012-10-07' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1777 , 'counts_orders' , 'date' , '2012-10-08' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1778 , 'counts_orders' , 'date' , '2012-10-09' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1779 , 'counts_orders' , 'date' , '2012-10-10' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1780 , 'counts_orders' , 'date' , '2012-10-11' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1781 , 'counts_orders' , 'date' , '2012-10-12' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1782 , 'counts_orders' , 'date' , '2012-10-13' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1783 , 'counts_orders' , 'date' , '2012-10-14' , '2012-10-15 18:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1784 , 'counts_users' , 'date' , '2012-11-06' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1785 , 'counts_users' , 'date' , '2012-11-07' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1786 , 'counts_users' , 'date' , '2012-11-08' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1787 , 'counts_users' , 'date' , '2012-11-09' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1788 , 'counts_users' , 'date' , '2012-11-10' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1789 , 'counts_users' , 'date' , '2012-11-11' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1790 , 'counts_users' , 'date' , '2012-11-12' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1791 , 'counts_users' , 'date' , '2012-11-13' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1792 , 'counts_users' , 'date' , '2012-11-14' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1793 , 'counts_users' , 'date' , '2012-11-15' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1794 , 'counts_users' , 'date' , '2012-11-16' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1795 , 'counts_users' , 'date' , '2012-11-17' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1796 , 'counts_users' , 'date' , '2012-11-18' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1797 , 'counts_users' , 'date' , '2012-11-19' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1798 , 'counts_users' , 'date' , '2012-11-20' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1799 , 'counts_users' , 'date' , '2012-11-21' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1800 , 'counts_users' , 'date' , '2012-11-22' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1801 , 'counts_users' , 'date' , '2012-11-23' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1802 , 'counts_users' , 'date' , '2012-11-24' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1803 , 'counts_users' , 'date' , '2012-11-25' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1804 , 'counts_users' , 'date' , '2012-11-26' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1805 , 'counts_users' , 'date' , '2012-11-27' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1806 , 'counts_users' , 'date' , '2012-11-28' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1807 , 'counts_users' , 'date' , '2012-11-29' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1808 , 'counts_users' , 'date' , '2012-11-30' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1809 , 'counts_users' , 'date' , '2012-12-01' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1810 , 'counts_users' , 'date' , '2012-12-02' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1811 , 'counts_users' , 'date' , '2012-12-03' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1812 , 'counts_users' , 'date' , '2012-12-04' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1813 , 'counts_users' , 'date' , '2012-12-05' , '2012-12-06 14:55:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1814 , 'counts_orders' , 'date' , '2012-11-06' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1815 , 'counts_orders' , 'date' , '2012-11-07' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1816 , 'counts_orders' , 'date' , '2012-11-08' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1817 , 'counts_orders' , 'date' , '2012-11-09' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1818 , 'counts_orders' , 'date' , '2012-11-10' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1819 , 'counts_orders' , 'date' , '2012-11-11' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1820 , 'counts_orders' , 'date' , '2012-11-12' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1821 , 'counts_orders' , 'date' , '2012-11-13' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1822 , 'counts_orders' , 'date' , '2012-11-14' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1823 , 'counts_orders' , 'date' , '2012-11-15' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1824 , 'counts_orders' , 'date' , '2012-11-16' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1825 , 'counts_orders' , 'date' , '2012-11-17' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1826 , 'counts_orders' , 'date' , '2012-11-18' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1827 , 'counts_orders' , 'date' , '2012-11-19' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1828 , 'counts_orders' , 'date' , '2012-11-20' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1829 , 'counts_orders' , 'date' , '2012-11-21' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1830 , 'counts_orders' , 'date' , '2012-11-22' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1831 , 'counts_orders' , 'date' , '2012-11-23' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1832 , 'counts_orders' , 'date' , '2012-11-24' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1833 , 'counts_orders' , 'date' , '2012-11-25' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1834 , 'counts_orders' , 'date' , '2012-11-26' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1835 , 'counts_orders' , 'date' , '2012-11-27' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1836 , 'counts_orders' , 'date' , '2012-11-28' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1837 , 'counts_orders' , 'date' , '2012-11-29' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1838 , 'counts_orders' , 'date' , '2012-11-30' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1839 , 'counts_orders' , 'date' , '2012-12-01' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1840 , 'counts_orders' , 'date' , '2012-12-02' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1841 , 'counts_orders' , 'date' , '2012-12-03' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1842 , 'counts_orders' , 'date' , '2012-12-04' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1843 , 'counts_orders' , 'date' , '2012-12-05' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1844 , 'counts_sales' , 'date' , '2012-11-06' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1845 , 'counts_sales' , 'date' , '2012-11-07' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1846 , 'counts_sales' , 'date' , '2012-11-08' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1847 , 'counts_sales' , 'date' , '2012-11-09' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1848 , 'counts_sales' , 'date' , '2012-11-10' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1849 , 'counts_sales' , 'date' , '2012-11-11' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1850 , 'counts_sales' , 'date' , '2012-11-12' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1851 , 'counts_sales' , 'date' , '2012-11-13' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1852 , 'counts_sales' , 'date' , '2012-11-14' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1853 , 'counts_sales' , 'date' , '2012-11-15' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1854 , 'counts_sales' , 'date' , '2012-11-16' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1855 , 'counts_sales' , 'date' , '2012-11-17' , '2012-12-06 14:55:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1856 , 'counts_sales' , 'date' , '2012-11-18' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1857 , 'counts_sales' , 'date' , '2012-11-19' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1858 , 'counts_sales' , 'date' , '2012-11-20' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1859 , 'counts_sales' , 'date' , '2012-11-21' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1860 , 'counts_sales' , 'date' , '2012-11-22' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1861 , 'counts_sales' , 'date' , '2012-11-23' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1862 , 'counts_sales' , 'date' , '2012-11-24' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1863 , 'counts_sales' , 'date' , '2012-11-25' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1864 , 'counts_sales' , 'date' , '2012-11-26' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1865 , 'counts_sales' , 'date' , '2012-11-27' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1866 , 'counts_sales' , 'date' , '2012-11-28' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1867 , 'counts_sales' , 'date' , '2012-11-29' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1868 , 'counts_sales' , 'date' , '2012-11-30' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1869 , 'counts_sales' , 'date' , '2012-12-01' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1870 , 'counts_sales' , 'date' , '2012-12-02' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1871 , 'counts_sales' , 'date' , '2012-12-03' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1872 , 'counts_sales' , 'date' , '2012-12-04' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1873 , 'counts_sales' , 'date' , '2012-12-05' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1874 , 'counts_visits' , 'date' , '2012-11-06' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1875 , 'counts_visits' , 'date' , '2012-11-07' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1876 , 'counts_visits' , 'date' , '2012-11-08' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1877 , 'counts_visits' , 'date' , '2012-11-09' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1878 , 'counts_visits' , 'date' , '2012-11-10' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1879 , 'counts_visits' , 'date' , '2012-11-11' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1880 , 'counts_visits' , 'date' , '2012-11-12' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1881 , 'counts_visits' , 'date' , '2012-11-13' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1882 , 'counts_visits' , 'date' , '2012-11-14' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1883 , 'counts_visits' , 'date' , '2012-11-15' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1884 , 'counts_visits' , 'date' , '2012-11-16' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1885 , 'counts_visits' , 'date' , '2012-11-17' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1886 , 'counts_visits' , 'date' , '2012-11-18' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1887 , 'counts_visits' , 'date' , '2012-11-19' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1888 , 'counts_visits' , 'date' , '2012-11-20' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1889 , 'counts_visits' , 'date' , '2012-11-21' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1890 , 'counts_visits' , 'date' , '2012-11-22' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1891 , 'counts_visits' , 'date' , '2012-11-23' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1892 , 'counts_visits' , 'date' , '2012-11-24' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1893 , 'counts_visits' , 'date' , '2012-11-25' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1894 , 'counts_visits' , 'date' , '2012-11-26' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1895 , 'counts_visits' , 'date' , '2012-11-27' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1896 , 'counts_visits' , 'date' , '2012-11-28' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1897 , 'counts_visits' , 'date' , '2012-11-29' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1898 , 'counts_visits' , 'date' , '2012-11-30' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1899 , 'counts_visits' , 'date' , '2012-12-01' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1900 , 'counts_visits' , 'date' , '2012-12-02' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1901 , 'counts_visits' , 'date' , '2012-12-03' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1902 , 'counts_visits' , 'date' , '2012-12-04' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1903 , 'counts_visits' , 'date' , '2012-12-05' , '2012-12-06 14:55:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1904 , 'counts_orders' , 'date' , '2013-12-28' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1905 , 'counts_orders' , 'date' , '2013-12-29' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1906 , 'counts_orders' , 'date' , '2013-12-30' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1907 , 'counts_orders' , 'date' , '2013-12-31' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1908 , 'counts_orders' , 'date' , '2014-01-01' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1909 , 'counts_orders' , 'date' , '2014-01-02' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1910 , 'counts_orders' , 'date' , '2014-01-03' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1911 , 'counts_orders' , 'date' , '2014-01-04' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1912 , 'counts_orders' , 'date' , '2014-01-05' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1913 , 'counts_orders' , 'date' , '2014-01-06' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1914 , 'counts_orders' , 'date' , '2014-01-07' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1915 , 'counts_orders' , 'date' , '2014-01-08' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1916 , 'counts_orders' , 'date' , '2014-01-09' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1917 , 'counts_orders' , 'date' , '2014-01-10' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1918 , 'counts_orders' , 'date' , '2014-01-11' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1919 , 'counts_orders' , 'date' , '2014-01-12' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1920 , 'counts_orders' , 'date' , '2014-01-13' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1921 , 'counts_orders' , 'date' , '2014-01-14' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1922 , 'counts_orders' , 'date' , '2014-01-15' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1923 , 'counts_orders' , 'date' , '2014-01-16' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1924 , 'counts_orders' , 'date' , '2014-01-17' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1925 , 'counts_orders' , 'date' , '2014-01-18' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1926 , 'counts_orders' , 'date' , '2014-01-19' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1927 , 'counts_orders' , 'date' , '2014-01-20' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1928 , 'counts_orders' , 'date' , '2014-01-21' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1929 , 'counts_orders' , 'date' , '2014-01-22' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1930 , 'counts_orders' , 'date' , '2014-01-23' , '2014-01-27 23:36:45' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1931 , 'counts_orders' , 'date' , '2014-01-24' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1932 , 'counts_orders' , 'date' , '2014-01-25' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1933 , 'counts_orders' , 'date' , '2014-01-26' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1934 , 'counts_users' , 'date' , '2013-12-28' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1935 , 'counts_users' , 'date' , '2013-12-29' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1936 , 'counts_users' , 'date' , '2013-12-30' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1937 , 'counts_users' , 'date' , '2013-12-31' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1938 , 'counts_users' , 'date' , '2014-01-01' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1939 , 'counts_users' , 'date' , '2014-01-02' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1940 , 'counts_users' , 'date' , '2014-01-03' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1941 , 'counts_users' , 'date' , '2014-01-04' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1942 , 'counts_users' , 'date' , '2014-01-05' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1943 , 'counts_users' , 'date' , '2014-01-06' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1944 , 'counts_users' , 'date' , '2014-01-07' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1945 , 'counts_users' , 'date' , '2014-01-08' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1946 , 'counts_users' , 'date' , '2014-01-09' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1947 , 'counts_users' , 'date' , '2014-01-10' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1948 , 'counts_users' , 'date' , '2014-01-11' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1949 , 'counts_users' , 'date' , '2014-01-12' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1950 , 'counts_users' , 'date' , '2014-01-13' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1951 , 'counts_users' , 'date' , '2014-01-14' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1952 , 'counts_users' , 'date' , '2014-01-15' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1953 , 'counts_users' , 'date' , '2014-01-16' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1954 , 'counts_users' , 'date' , '2014-01-17' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1955 , 'counts_users' , 'date' , '2014-01-18' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1956 , 'counts_users' , 'date' , '2014-01-19' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1957 , 'counts_users' , 'date' , '2014-01-20' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1958 , 'counts_users' , 'date' , '2014-01-21' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1959 , 'counts_users' , 'date' , '2014-01-22' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1960 , 'counts_users' , 'date' , '2014-01-23' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1961 , 'counts_users' , 'date' , '2014-01-24' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1962 , 'counts_users' , 'date' , '2014-01-25' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1963 , 'counts_users' , 'date' , '2014-01-26' , '2014-01-27 23:36:46' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1964 , 'counts_sales' , 'date' , '2013-12-28' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1965 , 'counts_sales' , 'date' , '2013-12-29' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1966 , 'counts_sales' , 'date' , '2013-12-30' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1967 , 'counts_sales' , 'date' , '2013-12-31' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1968 , 'counts_sales' , 'date' , '2014-01-01' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1969 , 'counts_sales' , 'date' , '2014-01-02' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1970 , 'counts_sales' , 'date' , '2014-01-03' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1971 , 'counts_sales' , 'date' , '2014-01-04' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1972 , 'counts_sales' , 'date' , '2014-01-05' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1973 , 'counts_sales' , 'date' , '2014-01-06' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1974 , 'counts_sales' , 'date' , '2014-01-07' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1975 , 'counts_sales' , 'date' , '2014-01-08' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1976 , 'counts_sales' , 'date' , '2014-01-09' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1977 , 'counts_sales' , 'date' , '2014-01-10' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1978 , 'counts_sales' , 'date' , '2014-01-11' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1979 , 'counts_sales' , 'date' , '2014-01-12' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1980 , 'counts_sales' , 'date' , '2014-01-13' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1981 , 'counts_sales' , 'date' , '2014-01-14' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1982 , 'counts_sales' , 'date' , '2014-01-15' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1983 , 'counts_sales' , 'date' , '2014-01-16' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1984 , 'counts_sales' , 'date' , '2014-01-17' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1985 , 'counts_sales' , 'date' , '2014-01-18' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1986 , 'counts_sales' , 'date' , '2014-01-19' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1987 , 'counts_sales' , 'date' , '2014-01-20' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1988 , 'counts_sales' , 'date' , '2014-01-21' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1989 , 'counts_sales' , 'date' , '2014-01-22' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1990 , 'counts_sales' , 'date' , '2014-01-23' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1991 , 'counts_sales' , 'date' , '2014-01-24' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1992 , 'counts_sales' , 'date' , '2014-01-25' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1993 , 'counts_sales' , 'date' , '2014-01-26' , '2014-01-27 23:36:47' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1994 , 'counts_users' , 'date' , '2014-02-24' , '2014-03-26 21:06:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1995 , 'counts_users' , 'date' , '2014-02-25' , '2014-03-26 21:06:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1996 , 'counts_users' , 'date' , '2014-02-26' , '2014-03-26 21:06:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1997 , 'counts_users' , 'date' , '2014-02-27' , '2014-03-26 21:06:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1998 , 'counts_users' , 'date' , '2014-02-28' , '2014-03-26 21:06:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (1999 , 'counts_users' , 'date' , '2014-03-01' , '2014-03-26 21:06:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2000 , 'counts_users' , 'date' , '2014-03-02' , '2014-03-26 21:06:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2001 , 'counts_users' , 'date' , '2014-03-03' , '2014-03-26 21:06:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2002 , 'counts_users' , 'date' , '2014-03-04' , '2014-03-26 21:06:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2003 , 'counts_users' , 'date' , '2014-03-05' , '2014-03-26 21:06:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2004 , 'counts_users' , 'date' , '2014-03-06' , '2014-03-26 21:06:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2005 , 'counts_users' , 'date' , '2014-03-07' , '2014-03-26 21:06:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2006 , 'counts_users' , 'date' , '2014-03-08' , '2014-03-26 21:06:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2007 , 'counts_users' , 'date' , '2014-03-09' , '2014-03-26 21:06:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2008 , 'counts_users' , 'date' , '2014-03-10' , '2014-03-26 21:06:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2009 , 'counts_users' , 'date' , '2014-03-11' , '2014-03-26 21:06:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2010 , 'counts_users' , 'date' , '2014-03-12' , '2014-03-26 21:06:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2011 , 'counts_users' , 'date' , '2014-03-13' , '2014-03-26 21:06:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2012 , 'counts_users' , 'date' , '2014-03-14' , '2014-03-26 21:06:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2013 , 'counts_users' , 'date' , '2014-03-15' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2014 , 'counts_users' , 'date' , '2014-03-16' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2015 , 'counts_users' , 'date' , '2014-03-17' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2016 , 'counts_users' , 'date' , '2014-03-18' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2017 , 'counts_users' , 'date' , '2014-03-19' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2018 , 'counts_users' , 'date' , '2014-03-20' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2019 , 'counts_users' , 'date' , '2014-03-21' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2020 , 'counts_users' , 'date' , '2014-03-22' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2021 , 'counts_users' , 'date' , '2014-03-23' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2022 , 'counts_users' , 'date' , '2014-03-24' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2023 , 'counts_users' , 'date' , '2014-03-25' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2024 , 'counts_sales' , 'date' , '2014-02-24' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2025 , 'counts_sales' , 'date' , '2014-02-25' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2026 , 'counts_sales' , 'date' , '2014-02-26' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2027 , 'counts_sales' , 'date' , '2014-02-27' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2028 , 'counts_sales' , 'date' , '2014-02-28' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2029 , 'counts_sales' , 'date' , '2014-03-01' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2030 , 'counts_sales' , 'date' , '2014-03-02' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2031 , 'counts_sales' , 'date' , '2014-03-03' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2032 , 'counts_sales' , 'date' , '2014-03-04' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2033 , 'counts_sales' , 'date' , '2014-03-05' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2034 , 'counts_sales' , 'date' , '2014-03-06' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2035 , 'counts_sales' , 'date' , '2014-03-07' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2036 , 'counts_sales' , 'date' , '2014-03-08' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2037 , 'counts_sales' , 'date' , '2014-03-09' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2038 , 'counts_sales' , 'date' , '2014-03-10' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2039 , 'counts_sales' , 'date' , '2014-03-11' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2040 , 'counts_sales' , 'date' , '2014-03-12' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2041 , 'counts_sales' , 'date' , '2014-03-13' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2042 , 'counts_sales' , 'date' , '2014-03-14' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2043 , 'counts_sales' , 'date' , '2014-03-15' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2044 , 'counts_sales' , 'date' , '2014-03-16' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2045 , 'counts_sales' , 'date' , '2014-03-17' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2046 , 'counts_sales' , 'date' , '2014-03-18' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2047 , 'counts_sales' , 'date' , '2014-03-19' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2048 , 'counts_sales' , 'date' , '2014-03-20' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2049 , 'counts_sales' , 'date' , '2014-03-21' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2050 , 'counts_sales' , 'date' , '2014-03-22' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2051 , 'counts_sales' , 'date' , '2014-03-23' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2052 , 'counts_sales' , 'date' , '2014-03-24' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2053 , 'counts_sales' , 'date' , '2014-03-25' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2054 , 'counts_orders' , 'date' , '2014-02-24' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2055 , 'counts_orders' , 'date' , '2014-02-25' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2056 , 'counts_orders' , 'date' , '2014-02-26' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2057 , 'counts_orders' , 'date' , '2014-02-27' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2058 , 'counts_orders' , 'date' , '2014-02-28' , '2014-03-26 21:06:25' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2059 , 'counts_orders' , 'date' , '2014-03-01' , '2014-03-26 21:06:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2060 , 'counts_orders' , 'date' , '2014-03-02' , '2014-03-26 21:06:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2061 , 'counts_orders' , 'date' , '2014-03-03' , '2014-03-26 21:06:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2062 , 'counts_orders' , 'date' , '2014-03-04' , '2014-03-26 21:06:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2063 , 'counts_orders' , 'date' , '2014-03-05' , '2014-03-26 21:06:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2064 , 'counts_orders' , 'date' , '2014-03-06' , '2014-03-26 21:06:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2065 , 'counts_orders' , 'date' , '2014-03-07' , '2014-03-26 21:06:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2066 , 'counts_orders' , 'date' , '2014-03-08' , '2014-03-26 21:06:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2067 , 'counts_orders' , 'date' , '2014-03-09' , '2014-03-26 21:06:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2068 , 'counts_orders' , 'date' , '2014-03-10' , '2014-03-26 21:06:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2069 , 'counts_orders' , 'date' , '2014-03-11' , '2014-03-26 21:06:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2070 , 'counts_orders' , 'date' , '2014-03-12' , '2014-03-26 21:06:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2071 , 'counts_orders' , 'date' , '2014-03-13' , '2014-03-26 21:06:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2072 , 'counts_orders' , 'date' , '2014-03-14' , '2014-03-26 21:06:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2073 , 'counts_orders' , 'date' , '2014-03-15' , '2014-03-26 21:06:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2074 , 'counts_orders' , 'date' , '2014-03-16' , '2014-03-26 21:06:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2075 , 'counts_orders' , 'date' , '2014-03-17' , '2014-03-26 21:06:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2076 , 'counts_orders' , 'date' , '2014-03-18' , '2014-03-26 21:06:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2077 , 'counts_orders' , 'date' , '2014-03-19' , '2014-03-26 21:06:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2078 , 'counts_orders' , 'date' , '2014-03-20' , '2014-03-26 21:06:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2079 , 'counts_orders' , 'date' , '2014-03-21' , '2014-03-26 21:06:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2080 , 'counts_orders' , 'date' , '2014-03-22' , '2014-03-26 21:06:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2081 , 'counts_orders' , 'date' , '2014-03-23' , '2014-03-26 21:06:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2082 , 'counts_orders' , 'date' , '2014-03-24' , '2014-03-26 21:06:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2083 , 'counts_orders' , 'date' , '2014-03-25' , '2014-03-26 21:06:26' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2084 , 'counts_users' , 'date' , '2014-12-28' , '2015-01-27 15:56:41' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2085 , 'counts_users' , 'date' , '2014-12-29' , '2015-01-27 15:56:41' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2086 , 'counts_users' , 'date' , '2014-12-30' , '2015-01-27 15:56:41' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2087 , 'counts_users' , 'date' , '2014-12-31' , '2015-01-27 15:56:41' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2088 , 'counts_users' , 'date' , '2015-01-01' , '2015-01-27 15:56:41' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2089 , 'counts_users' , 'date' , '2015-01-02' , '2015-01-27 15:56:42' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2090 , 'counts_users' , 'date' , '2015-01-03' , '2015-01-27 15:56:42' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2091 , 'counts_users' , 'date' , '2015-01-04' , '2015-01-27 15:56:42' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2092 , 'counts_users' , 'date' , '2015-01-05' , '2015-02-04 14:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2093 , 'counts_users' , 'date' , '2015-01-06' , '2015-02-04 14:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2094 , 'counts_users' , 'date' , '2015-01-07' , '2015-02-04 14:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2095 , 'counts_users' , 'date' , '2015-01-08' , '2015-02-04 14:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2096 , 'counts_users' , 'date' , '2015-01-09' , '2015-02-04 14:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2097 , 'counts_users' , 'date' , '2015-01-10' , '2015-02-04 14:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2098 , 'counts_users' , 'date' , '2015-01-11' , '2015-02-04 14:55:59' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2099 , 'counts_users' , 'date' , '2015-01-12' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2100 , 'counts_users' , 'date' , '2015-01-13' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2101 , 'counts_users' , 'date' , '2015-01-14' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2102 , 'counts_users' , 'date' , '2015-01-15' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2103 , 'counts_users' , 'date' , '2015-01-16' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2104 , 'counts_users' , 'date' , '2015-01-17' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2105 , 'counts_users' , 'date' , '2015-01-18' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2106 , 'counts_users' , 'date' , '2015-01-19' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2107 , 'counts_users' , 'date' , '2015-01-20' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2108 , 'counts_users' , 'date' , '2015-01-21' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2109 , 'counts_users' , 'date' , '2015-01-22' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2110 , 'counts_users' , 'date' , '2015-01-23' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2111 , 'counts_users' , 'date' , '2015-01-24' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2112 , 'counts_users' , 'date' , '2015-01-25' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2113 , 'counts_users' , 'date' , '2015-01-26' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2114 , 'counts_orders' , 'date' , '2014-12-28' , '2015-01-27 15:56:42' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2115 , 'counts_orders' , 'date' , '2014-12-29' , '2015-01-27 15:56:42' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2116 , 'counts_orders' , 'date' , '2014-12-30' , '2015-01-27 15:56:42' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2117 , 'counts_orders' , 'date' , '2014-12-31' , '2015-01-27 15:56:42' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2118 , 'counts_orders' , 'date' , '2015-01-01' , '2015-01-27 15:56:42' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2119 , 'counts_orders' , 'date' , '2015-01-02' , '2015-01-27 15:56:42' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2120 , 'counts_orders' , 'date' , '2015-01-03' , '2015-01-27 15:56:42' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2121 , 'counts_orders' , 'date' , '2015-01-04' , '2015-01-27 15:56:42' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2122 , 'counts_orders' , 'date' , '2015-01-05' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2123 , 'counts_orders' , 'date' , '2015-01-06' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2124 , 'counts_orders' , 'date' , '2015-01-07' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2125 , 'counts_orders' , 'date' , '2015-01-08' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2126 , 'counts_orders' , 'date' , '2015-01-09' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2127 , 'counts_orders' , 'date' , '2015-01-10' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2128 , 'counts_orders' , 'date' , '2015-01-11' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2129 , 'counts_orders' , 'date' , '2015-01-12' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2130 , 'counts_orders' , 'date' , '2015-01-13' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2131 , 'counts_orders' , 'date' , '2015-01-14' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2132 , 'counts_orders' , 'date' , '2015-01-15' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2133 , 'counts_orders' , 'date' , '2015-01-16' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2134 , 'counts_orders' , 'date' , '2015-01-17' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2135 , 'counts_orders' , 'date' , '2015-01-18' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2136 , 'counts_orders' , 'date' , '2015-01-19' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2137 , 'counts_orders' , 'date' , '2015-01-20' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2138 , 'counts_orders' , 'date' , '2015-01-21' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2139 , 'counts_orders' , 'date' , '2015-01-22' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2140 , 'counts_orders' , 'date' , '2015-01-23' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2141 , 'counts_orders' , 'date' , '2015-01-24' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2142 , 'counts_orders' , 'date' , '2015-01-25' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2143 , 'counts_orders' , 'date' , '2015-01-26' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2144 , 'counts_sales' , 'date' , '2014-12-28' , '2015-01-27 15:56:43' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2145 , 'counts_sales' , 'date' , '2014-12-29' , '2015-01-27 15:56:43' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2146 , 'counts_sales' , 'date' , '2014-12-30' , '2015-01-27 15:56:43' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2147 , 'counts_sales' , 'date' , '2014-12-31' , '2015-01-27 15:56:43' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2148 , 'counts_sales' , 'date' , '2015-01-01' , '2015-01-27 15:56:43' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2149 , 'counts_sales' , 'date' , '2015-01-02' , '2015-01-27 15:56:43' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2150 , 'counts_sales' , 'date' , '2015-01-03' , '2015-01-27 15:56:43' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2151 , 'counts_sales' , 'date' , '2015-01-04' , '2015-01-27 15:56:43' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2152 , 'counts_sales' , 'date' , '2015-01-05' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2153 , 'counts_sales' , 'date' , '2015-01-06' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2154 , 'counts_sales' , 'date' , '2015-01-07' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2155 , 'counts_sales' , 'date' , '2015-01-08' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2156 , 'counts_sales' , 'date' , '2015-01-09' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2157 , 'counts_sales' , 'date' , '2015-01-10' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2158 , 'counts_sales' , 'date' , '2015-01-11' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2159 , 'counts_sales' , 'date' , '2015-01-12' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2160 , 'counts_sales' , 'date' , '2015-01-13' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2161 , 'counts_sales' , 'date' , '2015-01-14' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2162 , 'counts_sales' , 'date' , '2015-01-15' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2163 , 'counts_sales' , 'date' , '2015-01-16' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2164 , 'counts_sales' , 'date' , '2015-01-17' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2165 , 'counts_sales' , 'date' , '2015-01-18' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2166 , 'counts_sales' , 'date' , '2015-01-19' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2167 , 'counts_sales' , 'date' , '2015-01-20' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2168 , 'counts_sales' , 'date' , '2015-01-21' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2169 , 'counts_sales' , 'date' , '2015-01-22' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2170 , 'counts_sales' , 'date' , '2015-01-23' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2171 , 'counts_sales' , 'date' , '2015-01-24' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2172 , 'counts_sales' , 'date' , '2015-01-25' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2173 , 'counts_sales' , 'date' , '2015-01-26' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2174 , 'counts_users' , 'date' , '2015-01-27' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2175 , 'counts_users' , 'date' , '2015-01-28' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2176 , 'counts_users' , 'date' , '2015-01-29' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2177 , 'counts_users' , 'date' , '2015-01-30' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2178 , 'counts_users' , 'date' , '2015-01-31' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2179 , 'counts_users' , 'date' , '2015-02-01' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2180 , 'counts_users' , 'date' , '2015-02-02' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2181 , 'counts_users' , 'date' , '2015-02-03' , '2015-02-04 14:56:00' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2182 , 'counts_orders' , 'date' , '2015-01-27' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2183 , 'counts_orders' , 'date' , '2015-01-28' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2184 , 'counts_orders' , 'date' , '2015-01-29' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2185 , 'counts_orders' , 'date' , '2015-01-30' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2186 , 'counts_orders' , 'date' , '2015-01-31' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2187 , 'counts_orders' , 'date' , '2015-02-01' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2188 , 'counts_orders' , 'date' , '2015-02-02' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2189 , 'counts_orders' , 'date' , '2015-02-03' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2190 , 'counts_sales' , 'date' , '2015-01-27' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2191 , 'counts_sales' , 'date' , '2015-01-28' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2192 , 'counts_sales' , 'date' , '2015-01-29' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2193 , 'counts_sales' , 'date' , '2015-01-30' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2194 , 'counts_sales' , 'date' , '2015-01-31' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2195 , 'counts_sales' , 'date' , '2015-02-01' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2196 , 'counts_sales' , 'date' , '2015-02-02' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2197 , 'counts_sales' , 'date' , '2015-02-03' , '2015-02-04 14:56:01' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2198 , 'counts_users' , 'date' , '2015-02-09' , '2015-03-11 23:27:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2199 , 'counts_users' , 'date' , '2015-02-10' , '2015-03-11 23:27:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2200 , 'counts_users' , 'date' , '2015-02-11' , '2015-03-11 23:27:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2201 , 'counts_users' , 'date' , '2015-02-12' , '2015-03-11 23:27:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2202 , 'counts_users' , 'date' , '2015-02-13' , '2015-03-11 23:27:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2203 , 'counts_users' , 'date' , '2015-02-14' , '2015-03-11 23:27:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2204 , 'counts_users' , 'date' , '2015-02-15' , '2015-03-11 23:27:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2205 , 'counts_users' , 'date' , '2015-02-16' , '2015-03-11 23:27:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2206 , 'counts_users' , 'date' , '2015-02-17' , '2015-03-11 23:27:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2207 , 'counts_users' , 'date' , '2015-02-18' , '2015-03-11 23:27:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2208 , 'counts_users' , 'date' , '2015-02-19' , '2015-03-11 23:27:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2209 , 'counts_users' , 'date' , '2015-02-20' , '2015-03-11 23:27:30' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2210 , 'counts_users' , 'date' , '2015-02-21' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2211 , 'counts_users' , 'date' , '2015-02-22' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2212 , 'counts_users' , 'date' , '2015-02-23' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2213 , 'counts_users' , 'date' , '2015-02-24' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2214 , 'counts_users' , 'date' , '2015-02-25' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2215 , 'counts_users' , 'date' , '2015-02-26' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2216 , 'counts_users' , 'date' , '2015-02-27' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2217 , 'counts_users' , 'date' , '2015-02-28' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2218 , 'counts_users' , 'date' , '2015-03-01' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2219 , 'counts_users' , 'date' , '2015-03-02' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2220 , 'counts_users' , 'date' , '2015-03-03' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2221 , 'counts_users' , 'date' , '2015-03-04' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2222 , 'counts_users' , 'date' , '2015-03-05' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2223 , 'counts_users' , 'date' , '2015-03-06' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2224 , 'counts_users' , 'date' , '2015-03-07' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2225 , 'counts_users' , 'date' , '2015-03-08' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2226 , 'counts_users' , 'date' , '2015-03-09' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2227 , 'counts_users' , 'date' , '2015-03-10' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2228 , 'counts_sales' , 'date' , '2015-02-09' , '2015-03-11 23:27:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2229 , 'counts_sales' , 'date' , '2015-02-10' , '2015-03-11 23:27:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2230 , 'counts_sales' , 'date' , '2015-02-11' , '2015-03-11 23:27:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2231 , 'counts_sales' , 'date' , '2015-02-12' , '2015-03-11 23:27:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2232 , 'counts_sales' , 'date' , '2015-02-13' , '2015-03-11 23:27:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2233 , 'counts_sales' , 'date' , '2015-02-14' , '2015-03-11 23:27:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2234 , 'counts_sales' , 'date' , '2015-02-15' , '2015-03-11 23:27:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2235 , 'counts_sales' , 'date' , '2015-02-16' , '2015-03-11 23:27:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2236 , 'counts_sales' , 'date' , '2015-02-17' , '2015-03-11 23:27:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2237 , 'counts_sales' , 'date' , '2015-02-18' , '2015-03-11 23:27:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2238 , 'counts_sales' , 'date' , '2015-02-19' , '2015-03-11 23:27:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2239 , 'counts_sales' , 'date' , '2015-02-20' , '2015-03-11 23:27:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2240 , 'counts_sales' , 'date' , '2015-02-21' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2241 , 'counts_sales' , 'date' , '2015-02-22' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2242 , 'counts_sales' , 'date' , '2015-02-23' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2243 , 'counts_sales' , 'date' , '2015-02-24' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2244 , 'counts_sales' , 'date' , '2015-02-25' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2245 , 'counts_sales' , 'date' , '2015-02-26' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2246 , 'counts_sales' , 'date' , '2015-02-27' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2247 , 'counts_sales' , 'date' , '2015-02-28' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2248 , 'counts_sales' , 'date' , '2015-03-01' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2249 , 'counts_sales' , 'date' , '2015-03-02' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2250 , 'counts_sales' , 'date' , '2015-03-03' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2251 , 'counts_sales' , 'date' , '2015-03-04' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2252 , 'counts_sales' , 'date' , '2015-03-05' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2253 , 'counts_sales' , 'date' , '2015-03-06' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2254 , 'counts_sales' , 'date' , '2015-03-07' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2255 , 'counts_sales' , 'date' , '2015-03-08' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2256 , 'counts_sales' , 'date' , '2015-03-09' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2257 , 'counts_sales' , 'date' , '2015-03-10' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2258 , 'counts_orders' , 'date' , '2015-02-09' , '2015-03-11 23:27:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2259 , 'counts_orders' , 'date' , '2015-02-10' , '2015-03-11 23:27:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2260 , 'counts_orders' , 'date' , '2015-02-11' , '2015-03-11 23:27:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2261 , 'counts_orders' , 'date' , '2015-02-12' , '2015-03-11 23:27:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2262 , 'counts_orders' , 'date' , '2015-02-13' , '2015-03-11 23:27:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2263 , 'counts_orders' , 'date' , '2015-02-14' , '2015-03-11 23:27:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2264 , 'counts_orders' , 'date' , '2015-02-15' , '2015-03-11 23:27:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2265 , 'counts_orders' , 'date' , '2015-02-16' , '2015-03-11 23:27:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2266 , 'counts_orders' , 'date' , '2015-02-17' , '2015-03-11 23:27:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2267 , 'counts_orders' , 'date' , '2015-02-18' , '2015-03-11 23:27:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2268 , 'counts_orders' , 'date' , '2015-02-19' , '2015-03-11 23:27:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2269 , 'counts_orders' , 'date' , '2015-02-20' , '2015-03-11 23:27:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2270 , 'counts_orders' , 'date' , '2015-02-21' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2271 , 'counts_orders' , 'date' , '2015-02-22' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2272 , 'counts_orders' , 'date' , '2015-02-23' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2273 , 'counts_orders' , 'date' , '2015-02-24' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2274 , 'counts_orders' , 'date' , '2015-02-25' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2275 , 'counts_orders' , 'date' , '2015-02-26' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2276 , 'counts_orders' , 'date' , '2015-02-27' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2277 , 'counts_orders' , 'date' , '2015-02-28' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2278 , 'counts_orders' , 'date' , '2015-03-01' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2279 , 'counts_orders' , 'date' , '2015-03-02' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2280 , 'counts_orders' , 'date' , '2015-03-03' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2281 , 'counts_orders' , 'date' , '2015-03-04' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2282 , 'counts_orders' , 'date' , '2015-03-05' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2283 , 'counts_orders' , 'date' , '2015-03-06' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2284 , 'counts_orders' , 'date' , '2015-03-07' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2285 , 'counts_orders' , 'date' , '2015-03-08' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2286 , 'counts_orders' , 'date' , '2015-03-09' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2287 , 'counts_orders' , 'date' , '2015-03-10' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2288 , 'counts_orders' , 'date' , '2015-03-11' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2289 , 'counts_orders' , 'date' , '2015-03-12' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2290 , 'counts_orders' , 'date' , '2015-03-13' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2291 , 'counts_orders' , 'date' , '2015-03-14' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2292 , 'counts_orders' , 'date' , '2015-03-15' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2293 , 'counts_orders' , 'date' , '2015-03-16' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2294 , 'counts_orders' , 'date' , '2015-03-17' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2295 , 'counts_orders' , 'date' , '2015-03-18' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2296 , 'counts_orders' , 'date' , '2015-03-19' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2297 , 'counts_orders' , 'date' , '2015-03-20' , '2015-03-23 23:09:48' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2298 , 'counts_orders' , 'date' , '2015-03-21' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2299 , 'counts_orders' , 'date' , '2015-03-22' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2300 , 'counts_sales' , 'date' , '2015-03-11' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2301 , 'counts_sales' , 'date' , '2015-03-12' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2302 , 'counts_sales' , 'date' , '2015-03-13' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2303 , 'counts_sales' , 'date' , '2015-03-14' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2304 , 'counts_sales' , 'date' , '2015-03-15' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2305 , 'counts_sales' , 'date' , '2015-03-16' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2306 , 'counts_sales' , 'date' , '2015-03-17' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2307 , 'counts_sales' , 'date' , '2015-03-18' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2308 , 'counts_sales' , 'date' , '2015-03-19' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2309 , 'counts_sales' , 'date' , '2015-03-20' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2310 , 'counts_sales' , 'date' , '2015-03-21' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2311 , 'counts_sales' , 'date' , '2015-03-22' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2312 , 'counts_users' , 'date' , '2015-03-11' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2313 , 'counts_users' , 'date' , '2015-03-12' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2314 , 'counts_users' , 'date' , '2015-03-13' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2315 , 'counts_users' , 'date' , '2015-03-14' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2316 , 'counts_users' , 'date' , '2015-03-15' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2317 , 'counts_users' , 'date' , '2015-03-16' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2318 , 'counts_users' , 'date' , '2015-03-17' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2319 , 'counts_users' , 'date' , '2015-03-18' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2320 , 'counts_users' , 'date' , '2015-03-19' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2321 , 'counts_users' , 'date' , '2015-03-20' , '2015-03-23 23:09:49' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2322 , 'counts_users' , 'date' , '2015-03-21' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2323 , 'counts_users' , 'date' , '2015-03-22' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2324 , 'counts_orders' , 'date' , '2015-03-23' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2325 , 'counts_orders' , 'date' , '2015-03-24' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2326 , 'counts_orders' , 'date' , '2015-03-25' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2327 , 'counts_orders' , 'date' , '2015-03-26' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2328 , 'counts_orders' , 'date' , '2015-03-27' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2329 , 'counts_orders' , 'date' , '2015-03-28' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2330 , 'counts_orders' , 'date' , '2015-03-29' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2331 , 'counts_orders' , 'date' , '2015-03-30' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2332 , 'counts_orders' , 'date' , '2015-03-31' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2333 , 'counts_orders' , 'date' , '2015-04-01' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2334 , 'counts_orders' , 'date' , '2015-04-02' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2335 , 'counts_orders' , 'date' , '2015-04-03' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2336 , 'counts_orders' , 'date' , '2015-04-04' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2337 , 'counts_orders' , 'date' , '2015-04-05' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2338 , 'counts_orders' , 'date' , '2015-04-06' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2339 , 'counts_orders' , 'date' , '2015-04-07' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2340 , 'counts_orders' , 'date' , '2015-04-08' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2341 , 'counts_orders' , 'date' , '2015-04-09' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2342 , 'counts_orders' , 'date' , '2015-04-10' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2343 , 'counts_orders' , 'date' , '2015-04-11' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2344 , 'counts_orders' , 'date' , '2015-04-12' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2345 , 'counts_orders' , 'date' , '2015-04-13' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2346 , 'counts_orders' , 'date' , '2015-04-14' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2347 , 'counts_orders' , 'date' , '2015-04-15' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2348 , 'counts_orders' , 'date' , '2015-04-16' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2349 , 'counts_orders' , 'date' , '2015-04-17' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2350 , 'counts_orders' , 'date' , '2015-04-18' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2351 , 'counts_orders' , 'date' , '2015-04-19' , '2015-04-20 09:09:31' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2352 , 'counts_users' , 'date' , '2015-03-23' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2353 , 'counts_users' , 'date' , '2015-03-24' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2354 , 'counts_users' , 'date' , '2015-03-25' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2355 , 'counts_users' , 'date' , '2015-03-26' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2356 , 'counts_users' , 'date' , '2015-03-27' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2357 , 'counts_users' , 'date' , '2015-03-28' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2358 , 'counts_users' , 'date' , '2015-03-29' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2359 , 'counts_users' , 'date' , '2015-03-30' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2360 , 'counts_users' , 'date' , '2015-03-31' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2361 , 'counts_users' , 'date' , '2015-04-01' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2362 , 'counts_users' , 'date' , '2015-04-02' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2363 , 'counts_users' , 'date' , '2015-04-03' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2364 , 'counts_users' , 'date' , '2015-04-04' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2365 , 'counts_users' , 'date' , '2015-04-05' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2366 , 'counts_users' , 'date' , '2015-04-06' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2367 , 'counts_users' , 'date' , '2015-04-07' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2368 , 'counts_users' , 'date' , '2015-04-08' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2369 , 'counts_users' , 'date' , '2015-04-09' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2370 , 'counts_users' , 'date' , '2015-04-10' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2371 , 'counts_users' , 'date' , '2015-04-11' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2372 , 'counts_users' , 'date' , '2015-04-12' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2373 , 'counts_users' , 'date' , '2015-04-13' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2374 , 'counts_users' , 'date' , '2015-04-14' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2375 , 'counts_users' , 'date' , '2015-04-15' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2376 , 'counts_users' , 'date' , '2015-04-16' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2377 , 'counts_users' , 'date' , '2015-04-17' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2378 , 'counts_users' , 'date' , '2015-04-18' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2379 , 'counts_users' , 'date' , '2015-04-19' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2380 , 'counts_sales' , 'date' , '2015-03-23' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2381 , 'counts_sales' , 'date' , '2015-03-24' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2382 , 'counts_sales' , 'date' , '2015-03-25' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2383 , 'counts_sales' , 'date' , '2015-03-26' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2384 , 'counts_sales' , 'date' , '2015-03-27' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2385 , 'counts_sales' , 'date' , '2015-03-28' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2386 , 'counts_sales' , 'date' , '2015-03-29' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2387 , 'counts_sales' , 'date' , '2015-03-30' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2388 , 'counts_sales' , 'date' , '2015-03-31' , '2015-04-20 09:09:32' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2389 , 'counts_sales' , 'date' , '2015-04-01' , '2015-04-20 09:09:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2390 , 'counts_sales' , 'date' , '2015-04-02' , '2015-04-20 09:09:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2391 , 'counts_sales' , 'date' , '2015-04-03' , '2015-04-20 09:09:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2392 , 'counts_sales' , 'date' , '2015-04-04' , '2015-04-20 09:09:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2393 , 'counts_sales' , 'date' , '2015-04-05' , '2015-04-20 09:09:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2394 , 'counts_sales' , 'date' , '2015-04-06' , '2015-04-20 09:09:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2395 , 'counts_sales' , 'date' , '2015-04-07' , '2015-04-20 09:09:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2396 , 'counts_sales' , 'date' , '2015-04-08' , '2015-04-20 09:09:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2397 , 'counts_sales' , 'date' , '2015-04-09' , '2015-04-20 09:09:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2398 , 'counts_sales' , 'date' , '2015-04-10' , '2015-04-20 09:09:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2399 , 'counts_sales' , 'date' , '2015-04-11' , '2015-04-20 09:09:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2400 , 'counts_sales' , 'date' , '2015-04-12' , '2015-04-20 09:09:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2401 , 'counts_sales' , 'date' , '2015-04-13' , '2015-04-20 09:09:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2402 , 'counts_sales' , 'date' , '2015-04-14' , '2015-04-20 09:09:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2403 , 'counts_sales' , 'date' , '2015-04-15' , '2015-04-20 09:09:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2404 , 'counts_sales' , 'date' , '2015-04-16' , '2015-04-20 09:09:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2405 , 'counts_sales' , 'date' , '2015-04-17' , '2015-04-20 09:09:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2406 , 'counts_sales' , 'date' , '2015-04-18' , '2015-04-20 09:09:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2407 , 'counts_sales' , 'date' , '2015-04-19' , '2015-04-20 09:09:33' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2408 , 'counts_orders' , 'date' , '2015-04-20' , '2015-04-21 18:26:53' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2409 , 'counts_users' , 'date' , '2015-04-20' , '2015-04-21 18:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2410 , 'counts_sales' , 'date' , '2015-04-20' , '2015-04-21 18:26:54' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2411 , 'counts_orders' , 'date' , '2015-07-07' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2412 , 'counts_orders' , 'date' , '2015-07-08' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2413 , 'counts_orders' , 'date' , '2015-07-09' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2414 , 'counts_orders' , 'date' , '2015-07-10' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2415 , 'counts_orders' , 'date' , '2015-07-11' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2416 , 'counts_orders' , 'date' , '2015-07-12' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2417 , 'counts_orders' , 'date' , '2015-07-13' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2418 , 'counts_orders' , 'date' , '2015-07-14' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2419 , 'counts_orders' , 'date' , '2015-07-15' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2420 , 'counts_orders' , 'date' , '2015-07-16' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2421 , 'counts_orders' , 'date' , '2015-07-17' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2422 , 'counts_orders' , 'date' , '2015-07-18' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2423 , 'counts_orders' , 'date' , '2015-07-19' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2424 , 'counts_orders' , 'date' , '2015-07-20' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2425 , 'counts_orders' , 'date' , '2015-07-21' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2426 , 'counts_orders' , 'date' , '2015-07-22' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2427 , 'counts_orders' , 'date' , '2015-07-23' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2428 , 'counts_orders' , 'date' , '2015-07-24' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2429 , 'counts_orders' , 'date' , '2015-07-25' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2430 , 'counts_orders' , 'date' , '2015-07-26' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2431 , 'counts_orders' , 'date' , '2015-07-27' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2432 , 'counts_orders' , 'date' , '2015-07-28' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2433 , 'counts_orders' , 'date' , '2015-07-29' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2434 , 'counts_orders' , 'date' , '2015-07-30' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2435 , 'counts_orders' , 'date' , '2015-07-31' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2436 , 'counts_orders' , 'date' , '2015-08-01' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2437 , 'counts_orders' , 'date' , '2015-08-02' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2438 , 'counts_orders' , 'date' , '2015-08-03' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2439 , 'counts_orders' , 'date' , '2015-08-04' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2440 , 'counts_orders' , 'date' , '2015-08-05' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2441 , 'counts_sales' , 'date' , '2015-07-07' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2442 , 'counts_sales' , 'date' , '2015-07-08' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2443 , 'counts_sales' , 'date' , '2015-07-09' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2444 , 'counts_sales' , 'date' , '2015-07-10' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2445 , 'counts_sales' , 'date' , '2015-07-11' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2446 , 'counts_sales' , 'date' , '2015-07-12' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2447 , 'counts_sales' , 'date' , '2015-07-13' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2448 , 'counts_sales' , 'date' , '2015-07-14' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2449 , 'counts_sales' , 'date' , '2015-07-15' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2450 , 'counts_sales' , 'date' , '2015-07-16' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2451 , 'counts_sales' , 'date' , '2015-07-17' , '2015-08-06 17:13:23' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2452 , 'counts_sales' , 'date' , '2015-07-18' , '2015-08-06 17:13:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2453 , 'counts_sales' , 'date' , '2015-07-19' , '2015-08-06 17:13:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2454 , 'counts_sales' , 'date' , '2015-07-20' , '2015-08-06 17:13:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2455 , 'counts_sales' , 'date' , '2015-07-21' , '2015-08-06 17:13:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2456 , 'counts_sales' , 'date' , '2015-07-22' , '2015-08-06 17:13:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2457 , 'counts_sales' , 'date' , '2015-07-23' , '2015-08-06 17:13:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2458 , 'counts_sales' , 'date' , '2015-07-24' , '2015-08-06 17:13:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2459 , 'counts_sales' , 'date' , '2015-07-25' , '2015-08-06 17:13:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2460 , 'counts_sales' , 'date' , '2015-07-26' , '2015-08-06 17:13:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2461 , 'counts_sales' , 'date' , '2015-07-27' , '2015-08-06 17:13:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2462 , 'counts_sales' , 'date' , '2015-07-28' , '2015-08-06 17:13:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2463 , 'counts_sales' , 'date' , '2015-07-29' , '2015-08-06 17:13:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2464 , 'counts_sales' , 'date' , '2015-07-30' , '2015-08-06 17:13:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2465 , 'counts_sales' , 'date' , '2015-07-31' , '2015-08-06 17:13:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2466 , 'counts_sales' , 'date' , '2015-08-01' , '2015-08-06 17:13:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2467 , 'counts_sales' , 'date' , '2015-08-02' , '2015-08-06 17:13:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2468 , 'counts_sales' , 'date' , '2015-08-03' , '2015-08-06 17:13:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2469 , 'counts_sales' , 'date' , '2015-08-04' , '2015-08-06 17:13:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2470 , 'counts_sales' , 'date' , '2015-08-05' , '2015-08-06 17:13:24' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2471 , 'counts_users' , 'date' , '2015-07-07' , '2015-08-06 20:09:43' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2472 , 'counts_users' , 'date' , '2015-07-08' , '2015-08-06 20:09:43' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2473 , 'counts_users' , 'date' , '2015-07-09' , '2015-08-06 20:09:43' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2474 , 'counts_users' , 'date' , '2015-07-10' , '2015-08-06 20:09:43' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2475 , 'counts_users' , 'date' , '2015-07-11' , '2015-08-06 20:09:43' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2476 , 'counts_users' , 'date' , '2015-07-12' , '2015-08-06 20:09:43' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2477 , 'counts_users' , 'date' , '2015-07-13' , '2015-08-06 20:09:43' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2478 , 'counts_users' , 'date' , '2015-07-14' , '2015-08-06 20:09:43' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2479 , 'counts_users' , 'date' , '2015-07-15' , '2015-08-06 20:09:44' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2480 , 'counts_users' , 'date' , '2015-07-16' , '2015-08-06 20:09:44' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2481 , 'counts_users' , 'date' , '2015-07-17' , '2015-08-06 20:09:44' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2482 , 'counts_users' , 'date' , '2015-07-18' , '2015-08-06 20:09:44' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2483 , 'counts_users' , 'date' , '2015-07-19' , '2015-08-06 20:09:44' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2484 , 'counts_users' , 'date' , '2015-07-20' , '2015-08-06 20:09:44' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2485 , 'counts_users' , 'date' , '2015-07-21' , '2015-08-06 20:09:44' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2486 , 'counts_users' , 'date' , '2015-07-22' , '2015-08-06 20:09:44' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2487 , 'counts_users' , 'date' , '2015-07-23' , '2015-08-06 20:09:44' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2488 , 'counts_users' , 'date' , '2015-07-24' , '2015-08-06 20:09:44' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2489 , 'counts_users' , 'date' , '2015-07-25' , '2015-08-06 20:09:44' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2490 , 'counts_users' , 'date' , '2015-07-26' , '2015-08-06 20:09:44' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2491 , 'counts_users' , 'date' , '2015-07-27' , '2015-08-06 20:09:44' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2492 , 'counts_users' , 'date' , '2015-07-28' , '2015-08-06 20:09:44' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2493 , 'counts_users' , 'date' , '2015-07-29' , '2015-08-06 20:09:44' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2494 , 'counts_users' , 'date' , '2015-07-30' , '2015-08-06 20:09:44' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2495 , 'counts_users' , 'date' , '2015-07-31' , '2015-08-06 20:09:44' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2496 , 'counts_users' , 'date' , '2015-08-01' , '2015-08-06 20:09:44' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2497 , 'counts_users' , 'date' , '2015-08-02' , '2015-08-06 20:09:44' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2498 , 'counts_users' , 'date' , '2015-08-03' , '2015-08-06 20:09:44' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2499 , 'counts_users' , 'date' , '2015-08-04' , '2015-08-06 20:09:44' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2500 , 'counts_users' , 'date' , '2015-08-05' , '2015-08-06 20:09:44' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2501 , 'counts_users' , 'date' , '2015-08-06' , '2015-08-07 00:08:43' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2502 , 'counts_sales' , 'date' , '2015-08-06' , '2015-08-07 00:08:43' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2503 , 'counts_orders' , 'date' , '2015-08-06' , '2015-08-07 00:08:43' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2608 , 'admin.php' , 'orders' , 'yesterday' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2609 , 'admin.php' , 'orders' , 'this_week' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2610 , 'admin.php' , 'orders' , 'this_month' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2611 , 'admin.php' , 'orders' , 'last_month' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2612 , 'admin.php' , 'orders' , 'total_year' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2613 , 'admin.php' , 'sales' , 'yesterday' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2614 , 'admin.php' , 'sales' , 'this_week' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2615 , 'admin.php' , 'sales' , 'this_month' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2616 , 'admin.php' , 'sales' , 'last_month' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2617 , 'admin.php' , 'sales' , 'total_year' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2618 , 'admin.php' , 'users' , 'yesterday' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2619 , 'admin.php' , 'users' , 'this_week' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2620 , 'admin.php' , 'users' , 'this_month' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2621 , 'admin.php' , 'users' , 'last_month' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2622 , 'admin.php' , 'users' , 'total_year' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2623 , 'admin.php' , 'tickets' , 'yesterday' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2624 , 'admin.php' , 'tickets' , 'this_week' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2625 , 'admin.php' , 'tickets' , 'this_month' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2626 , 'admin.php' , 'tickets' , 'last_month' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2627 , 'admin.php' , 'tickets' , 'total_year' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2628 , 'counts_users' , 'date' , '2015-10-27' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2629 , 'counts_users' , 'date' , '2015-10-28' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2630 , 'counts_users' , 'date' , '2015-10-29' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2631 , 'counts_users' , 'date' , '2015-10-30' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2632 , 'counts_users' , 'date' , '2015-10-31' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2633 , 'counts_users' , 'date' , '2015-11-01' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2634 , 'counts_users' , 'date' , '2015-11-02' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2635 , 'counts_users' , 'date' , '2015-11-03' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2636 , 'counts_users' , 'date' , '2015-11-04' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2637 , 'counts_users' , 'date' , '2015-11-05' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2638 , 'counts_users' , 'date' , '2015-11-06' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2639 , 'counts_users' , 'date' , '2015-11-07' , '2015-11-24 03:15:38' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2640 , 'counts_users' , 'date' , '2015-11-08' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2641 , 'counts_users' , 'date' , '2015-11-09' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2642 , 'counts_users' , 'date' , '2015-11-10' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2643 , 'counts_users' , 'date' , '2015-11-11' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2644 , 'counts_users' , 'date' , '2015-11-12' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2645 , 'counts_users' , 'date' , '2015-11-13' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2646 , 'counts_users' , 'date' , '2015-11-14' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2647 , 'counts_users' , 'date' , '2015-11-15' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2648 , 'counts_users' , 'date' , '2015-11-16' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2649 , 'counts_users' , 'date' , '2015-11-17' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2650 , 'counts_users' , 'date' , '2015-11-18' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2651 , 'counts_users' , 'date' , '2015-11-19' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2652 , 'counts_users' , 'date' , '2015-11-20' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2653 , 'counts_users' , 'date' , '2015-11-21' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2654 , 'counts_users' , 'date' , '2015-11-22' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2655 , 'counts_users' , 'date' , '2015-11-23' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2656 , 'counts_orders' , 'date' , '2015-10-27' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2657 , 'counts_orders' , 'date' , '2015-10-28' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2658 , 'counts_orders' , 'date' , '2015-10-29' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2659 , 'counts_orders' , 'date' , '2015-10-30' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2660 , 'counts_orders' , 'date' , '2015-10-31' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2661 , 'counts_orders' , 'date' , '2015-11-01' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2662 , 'counts_orders' , 'date' , '2015-11-02' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2663 , 'counts_orders' , 'date' , '2015-11-03' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2664 , 'counts_orders' , 'date' , '2015-11-04' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2665 , 'counts_orders' , 'date' , '2015-11-05' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2666 , 'counts_orders' , 'date' , '2015-11-06' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2667 , 'counts_orders' , 'date' , '2015-11-07' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2668 , 'counts_orders' , 'date' , '2015-11-08' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2669 , 'counts_orders' , 'date' , '2015-11-09' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2670 , 'counts_orders' , 'date' , '2015-11-10' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2671 , 'counts_orders' , 'date' , '2015-11-11' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2672 , 'counts_orders' , 'date' , '2015-11-12' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2673 , 'counts_orders' , 'date' , '2015-11-13' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2674 , 'counts_orders' , 'date' , '2015-11-14' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2675 , 'counts_orders' , 'date' , '2015-11-15' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2676 , 'counts_orders' , 'date' , '2015-11-16' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2677 , 'counts_orders' , 'date' , '2015-11-17' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2678 , 'counts_orders' , 'date' , '2015-11-18' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2679 , 'counts_orders' , 'date' , '2015-11-19' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2680 , 'counts_orders' , 'date' , '2015-11-20' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2681 , 'counts_orders' , 'date' , '2015-11-21' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2682 , 'counts_orders' , 'date' , '2015-11-22' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2683 , 'counts_orders' , 'date' , '2015-11-23' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2684 , 'counts_sales' , 'date' , '2015-10-27' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2685 , 'counts_sales' , 'date' , '2015-10-28' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2686 , 'counts_sales' , 'date' , '2015-10-29' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2687 , 'counts_sales' , 'date' , '2015-10-30' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2688 , 'counts_sales' , 'date' , '2015-10-31' , '2015-11-24 03:15:39' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2689 , 'counts_sales' , 'date' , '2015-11-01' , '2015-11-24 03:15:40' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2690 , 'counts_sales' , 'date' , '2015-11-02' , '2015-11-24 03:15:40' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2691 , 'counts_sales' , 'date' , '2015-11-03' , '2015-11-24 03:15:40' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2692 , 'counts_sales' , 'date' , '2015-11-04' , '2015-11-24 03:15:40' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2693 , 'counts_sales' , 'date' , '2015-11-05' , '2015-11-24 03:15:40' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2694 , 'counts_sales' , 'date' , '2015-11-06' , '2015-11-24 03:15:40' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2695 , 'counts_sales' , 'date' , '2015-11-07' , '2015-11-24 03:15:40' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2696 , 'counts_sales' , 'date' , '2015-11-08' , '2015-11-24 03:15:40' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2697 , 'counts_sales' , 'date' , '2015-11-09' , '2015-11-24 03:15:40' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2698 , 'counts_sales' , 'date' , '2015-11-10' , '2015-11-24 03:15:40' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2699 , 'counts_sales' , 'date' , '2015-11-11' , '2015-11-24 03:15:40' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2700 , 'counts_sales' , 'date' , '2015-11-12' , '2015-11-24 03:15:40' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2701 , 'counts_sales' , 'date' , '2015-11-13' , '2015-11-24 03:15:40' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2702 , 'counts_sales' , 'date' , '2015-11-14' , '2015-11-24 03:15:40' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2703 , 'counts_sales' , 'date' , '2015-11-15' , '2015-11-24 03:15:40' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2704 , 'counts_sales' , 'date' , '2015-11-16' , '2015-11-24 03:15:40' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2705 , 'counts_sales' , 'date' , '2015-11-17' , '2015-11-24 03:15:40' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2706 , 'counts_sales' , 'date' , '2015-11-18' , '2015-11-24 03:15:40' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2707 , 'counts_sales' , 'date' , '2015-11-19' , '2015-11-24 03:15:40' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2708 , 'counts_sales' , 'date' , '2015-11-20' , '2015-11-24 03:15:40' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2709 , 'counts_sales' , 'date' , '2015-11-21' , '2015-11-24 03:15:40' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2710 , 'counts_sales' , 'date' , '2015-11-22' , '2015-11-24 03:15:40' , '0' ) ;
INSERT INTO va_caches (cache_id,cache_type,cache_name,cache_parameter,cache_date,cache_data) VALUES (2711 , 'counts_sales' , 'date' , '2015-11-23' , '2015-11-24 03:15:40' , '0' ) ;


INSERT INTO va_categories (category_id,parent_category_id,alias_category_id,redirect_category_id,google_base_type_id,category_path,category_name,friendly_url,category_order,sites_all,access_level,guest_access_level,admin_access_level,is_showing,allowed_post_subcategories,show_sub_products,short_description,full_description,image,image_alt,image_large,image_large_alt,list_template,details_template,table_view,a_title,meta_title,meta_keywords,meta_description,total_views,header_menu_show,header_menu_order,header_menu_class,nav_bar_show,nav_bar_order,nav_bar_class,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (48 , 0 , NULL , NULL , -1 , '0,' , 'Computers' , '' , 1 , 1 , 7 , 7 , 7 , 1 , 0 , 1 , '<br>' , '<br>' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 'Demo Site | Computers' , 'Demo Site | Computers: Notebooks, Desctops, ...' , 'Demo Site | Computers: Notebooks, Desctops, ...' , 6 , 1 , NULL , NULL , 1 , NULL , NULL , 1 , 1 , '2008-08-29 08:08:34' , '2008-12-05 17:01:21' ) ;
INSERT INTO va_categories (category_id,parent_category_id,alias_category_id,redirect_category_id,google_base_type_id,category_path,category_name,friendly_url,category_order,sites_all,access_level,guest_access_level,admin_access_level,is_showing,allowed_post_subcategories,show_sub_products,short_description,full_description,image,image_alt,image_large,image_large_alt,list_template,details_template,table_view,a_title,meta_title,meta_keywords,meta_description,total_views,header_menu_show,header_menu_order,header_menu_class,nav_bar_show,nav_bar_order,nav_bar_class,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (49 , 0 , NULL , NULL , -1 , '0,' , 'Cameras' , '' , 2 , 1 , 7 , 7 , 7 , 1 , 0 , 1 , '<br>' , '<br>' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 'Demo Site | Cameras' , 'Demo Site | Cameras: Digital Camera, Camcorders, ...' , 'Demo Site | Cameras: Digital Camera, Camcorders, ...' , 2 , 1 , NULL , NULL , 1 , NULL , NULL , 1 , 1 , '2008-08-29 11:25:21' , '2008-12-05 17:01:31' ) ;
INSERT INTO va_categories (category_id,parent_category_id,alias_category_id,redirect_category_id,google_base_type_id,category_path,category_name,friendly_url,category_order,sites_all,access_level,guest_access_level,admin_access_level,is_showing,allowed_post_subcategories,show_sub_products,short_description,full_description,image,image_alt,image_large,image_large_alt,list_template,details_template,table_view,a_title,meta_title,meta_keywords,meta_description,total_views,header_menu_show,header_menu_order,header_menu_class,nav_bar_show,nav_bar_order,nav_bar_class,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (50 , 0 , NULL , NULL , -1 , '0,' , 'TV & Home' , '' , 3 , 1 , 7 , 7 , 7 , 1 , 0 , 1 , '<br>' , '<br>' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 'Demo Site | TV & Home' , 'Demo Site | TV & Home: Televisions, Home Theater, ...' , 'Demo Site | TV & Home: Televisions, Home Theater, ...' , 1 , 1 , NULL , NULL , 1 , NULL , NULL , 1 , 1 , '2008-08-29 11:28:23' , '2008-12-05 17:01:40' ) ;
INSERT INTO va_categories (category_id,parent_category_id,alias_category_id,redirect_category_id,google_base_type_id,category_path,category_name,friendly_url,category_order,sites_all,access_level,guest_access_level,admin_access_level,is_showing,allowed_post_subcategories,show_sub_products,short_description,full_description,image,image_alt,image_large,image_large_alt,list_template,details_template,table_view,a_title,meta_title,meta_keywords,meta_description,total_views,header_menu_show,header_menu_order,header_menu_class,nav_bar_show,nav_bar_order,nav_bar_class,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (51 , 0 , NULL , NULL , -1 , '0,' , 'MP3' , '' , 4 , 1 , 7 , 7 , 7 , 1 , 0 , 1 , '<br>' , '<br>' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 'Demo Site | MP3' , 'Demo Site | MP3: Players, Headphones, ...' , 'Demo Site | MP3: Players, Headphones, ...' , 1 , 1 , NULL , NULL , 1 , NULL , NULL , 1 , 1 , '2008-08-29 11:29:33' , '2008-12-05 17:01:50' ) ;
INSERT INTO va_categories (category_id,parent_category_id,alias_category_id,redirect_category_id,google_base_type_id,category_path,category_name,friendly_url,category_order,sites_all,access_level,guest_access_level,admin_access_level,is_showing,allowed_post_subcategories,show_sub_products,short_description,full_description,image,image_alt,image_large,image_large_alt,list_template,details_template,table_view,a_title,meta_title,meta_keywords,meta_description,total_views,header_menu_show,header_menu_order,header_menu_class,nav_bar_show,nav_bar_order,nav_bar_class,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (52 , 0 , NULL , NULL , -1 , '0,' , 'Accessories' , '' , 5 , 1 , 7 , 7 , 7 , 1 , 0 , 0 , '<br>' , '<br>' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 'Demo Site | Accessories' , 'Demo Site | Accessories: Computers, Notebooks, Desctops, Cameras, Digital Camera, Camcorders, TV & Home, Televisions, Home Theater, ...' , 'Demo Site | Accessories: Computers, Notebooks, Desctops, Cameras, Digital Camera, Camcorders, TV & Home, Televisions, Home Theater, ...' , 2 , 1 , NULL , NULL , 1 , NULL , NULL , 1 , 1 , '2008-08-29 11:30:20' , '2008-12-05 17:02:01' ) ;
INSERT INTO va_categories (category_id,parent_category_id,alias_category_id,redirect_category_id,google_base_type_id,category_path,category_name,friendly_url,category_order,sites_all,access_level,guest_access_level,admin_access_level,is_showing,allowed_post_subcategories,show_sub_products,short_description,full_description,image,image_alt,image_large,image_large_alt,list_template,details_template,table_view,a_title,meta_title,meta_keywords,meta_description,total_views,header_menu_show,header_menu_order,header_menu_class,nav_bar_show,nav_bar_order,nav_bar_class,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (53 , 48 , NULL , NULL , -1 , '0,48,' , 'Notebooks' , NULL , 1 , 1 , 7 , 7 , 7 , 1 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 'Demo Site | Computers | Notebooks' , 'Demo Site | Computers | Notebooks' , 'Demo Site | Computers | Notebooks' , 4 , 1 , NULL , NULL , 1 , NULL , NULL , 1 , 1 , '2008-09-08 10:55:41' , '2008-09-08 10:55:41' ) ;
INSERT INTO va_categories (category_id,parent_category_id,alias_category_id,redirect_category_id,google_base_type_id,category_path,category_name,friendly_url,category_order,sites_all,access_level,guest_access_level,admin_access_level,is_showing,allowed_post_subcategories,show_sub_products,short_description,full_description,image,image_alt,image_large,image_large_alt,list_template,details_template,table_view,a_title,meta_title,meta_keywords,meta_description,total_views,header_menu_show,header_menu_order,header_menu_class,nav_bar_show,nav_bar_order,nav_bar_class,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (54 , 48 , NULL , NULL , -1 , '0,48,' , 'Desktops' , NULL , 2 , 1 , 7 , 7 , 7 , 1 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 'Demo Site | Computers | Desctops' , 'Demo Site | Computers | Desctops' , 'Demo Site | Computers | Desctops' , 3 , 1 , NULL , NULL , 1 , NULL , NULL , 1 , 1 , '2008-09-08 10:56:32' , '2008-09-22 17:11:43' ) ;
INSERT INTO va_categories (category_id,parent_category_id,alias_category_id,redirect_category_id,google_base_type_id,category_path,category_name,friendly_url,category_order,sites_all,access_level,guest_access_level,admin_access_level,is_showing,allowed_post_subcategories,show_sub_products,short_description,full_description,image,image_alt,image_large,image_large_alt,list_template,details_template,table_view,a_title,meta_title,meta_keywords,meta_description,total_views,header_menu_show,header_menu_order,header_menu_class,nav_bar_show,nav_bar_order,nav_bar_class,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (55 , 49 , NULL , NULL , -1 , '0,49,' , 'Digital Camera' , NULL , 1 , 1 , 7 , 7 , 7 , 1 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 'Demo Site | Cameras | Digital Camera' , 'Demo Site | Cameras | Digital Camera' , 'Demo Site | Cameras | Digital Camera' , 2 , 1 , NULL , NULL , 1 , NULL , NULL , 1 , 1 , '2008-09-09 11:32:55' , '2008-09-09 11:32:55' ) ;
INSERT INTO va_categories (category_id,parent_category_id,alias_category_id,redirect_category_id,google_base_type_id,category_path,category_name,friendly_url,category_order,sites_all,access_level,guest_access_level,admin_access_level,is_showing,allowed_post_subcategories,show_sub_products,short_description,full_description,image,image_alt,image_large,image_large_alt,list_template,details_template,table_view,a_title,meta_title,meta_keywords,meta_description,total_views,header_menu_show,header_menu_order,header_menu_class,nav_bar_show,nav_bar_order,nav_bar_class,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (56 , 49 , NULL , NULL , -1 , '0,49,' , 'Camcorders' , NULL , 2 , 1 , 7 , 7 , 7 , 1 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 'Demo Site | Cameras | Camcorders' , 'Demo Site | Cameras | Camcorders' , 'Demo Site | Cameras | Camcorders' , 2 , 1 , NULL , NULL , 1 , NULL , NULL , 1 , 1 , '2008-09-09 11:33:28' , '2008-09-09 11:33:28' ) ;
INSERT INTO va_categories (category_id,parent_category_id,alias_category_id,redirect_category_id,google_base_type_id,category_path,category_name,friendly_url,category_order,sites_all,access_level,guest_access_level,admin_access_level,is_showing,allowed_post_subcategories,show_sub_products,short_description,full_description,image,image_alt,image_large,image_large_alt,list_template,details_template,table_view,a_title,meta_title,meta_keywords,meta_description,total_views,header_menu_show,header_menu_order,header_menu_class,nav_bar_show,nav_bar_order,nav_bar_class,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (57 , 50 , NULL , NULL , -1 , '0,50,' , 'Televisions' , NULL , 1 , 1 , 7 , 7 , 7 , 1 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 'Demo Site | TV & Home | Televisions' , 'Demo Site | TV & Home | Televisions' , 'Demo Site | TV & Home | Televisions' , 3 , 1 , NULL , NULL , 1 , NULL , NULL , 1 , 1 , '2008-09-09 17:41:29' , '2008-09-09 17:41:29' ) ;
INSERT INTO va_categories (category_id,parent_category_id,alias_category_id,redirect_category_id,google_base_type_id,category_path,category_name,friendly_url,category_order,sites_all,access_level,guest_access_level,admin_access_level,is_showing,allowed_post_subcategories,show_sub_products,short_description,full_description,image,image_alt,image_large,image_large_alt,list_template,details_template,table_view,a_title,meta_title,meta_keywords,meta_description,total_views,header_menu_show,header_menu_order,header_menu_class,nav_bar_show,nav_bar_order,nav_bar_class,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (58 , 50 , NULL , NULL , -1 , '0,50,' , 'Home Theater' , NULL , 2 , 1 , 7 , 7 , 7 , 1 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 'Demo Site | TV & Home | Home Theater' , 'Demo Site | TV & Home | Home Theater' , 'Demo Site | TV & Home | Home Theater' , 2 , 1 , NULL , NULL , 1 , NULL , NULL , 1 , 1 , '2008-09-09 17:41:39' , '2008-09-09 17:42:00' ) ;
INSERT INTO va_categories (category_id,parent_category_id,alias_category_id,redirect_category_id,google_base_type_id,category_path,category_name,friendly_url,category_order,sites_all,access_level,guest_access_level,admin_access_level,is_showing,allowed_post_subcategories,show_sub_products,short_description,full_description,image,image_alt,image_large,image_large_alt,list_template,details_template,table_view,a_title,meta_title,meta_keywords,meta_description,total_views,header_menu_show,header_menu_order,header_menu_class,nav_bar_show,nav_bar_order,nav_bar_class,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (59 , 51 , NULL , NULL , -1 , '0,51,' , 'Players' , NULL , 1 , 1 , 7 , 7 , 7 , 1 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 'Demo Site | MP3 | Players' , 'Demo Site | MP3 | Players' , 'Demo Site | MP3 | Players' , 1 , 1 , NULL , NULL , 1 , NULL , NULL , 1 , 1 , '2008-09-11 12:44:30' , '2008-09-11 12:44:30' ) ;
INSERT INTO va_categories (category_id,parent_category_id,alias_category_id,redirect_category_id,google_base_type_id,category_path,category_name,friendly_url,category_order,sites_all,access_level,guest_access_level,admin_access_level,is_showing,allowed_post_subcategories,show_sub_products,short_description,full_description,image,image_alt,image_large,image_large_alt,list_template,details_template,table_view,a_title,meta_title,meta_keywords,meta_description,total_views,header_menu_show,header_menu_order,header_menu_class,nav_bar_show,nav_bar_order,nav_bar_class,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (60 , 51 , NULL , NULL , -1 , '0,51,' , 'Headphones' , NULL , 2 , 1 , 7 , 7 , 7 , 1 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 'Demo Site | MP3 | Headphones' , 'Demo Site | MP3 | Headphones' , 'Demo Site | MP3 | Headphones' , 1 , 1 , NULL , NULL , 1 , NULL , NULL , 1 , 1 , '2008-09-11 12:45:09' , '2008-09-11 12:45:09' ) ;


INSERT INTO va_categories_columns (column_id,category_id,column_order,column_code,column_title,column_html) VALUES (1 , 0 , 1 , 'image|compare' , 'IMAGE_MSG' , NULL ) ;
INSERT INTO va_categories_columns (column_id,category_id,column_order,column_code,column_title,column_html) VALUES (2 , 0 , 2 , 'item_name|found_in_category|description' , 'PROD_NAME_MSG' , NULL ) ;
INSERT INTO va_categories_columns (column_id,category_id,column_order,column_code,column_title,column_html) VALUES (3 , 0 , 3 , 'options' , 'OPTIONS_MSG' , NULL ) ;
INSERT INTO va_categories_columns (column_id,category_id,column_order,column_code,column_title,column_html) VALUES (4 , 0 , 4 , 'price' , 'PRICE_MSG' , NULL ) ;
INSERT INTO va_categories_columns (column_id,category_id,column_order,column_code,column_title,column_html) VALUES (5 , 0 , 5 , 'sales_price' , 'OUR_PRICE_MSG' , NULL ) ;
INSERT INTO va_categories_columns (column_id,category_id,column_order,column_code,column_title,column_html) VALUES (6 , 0 , 6 , 'save' , 'YOU_SAVE_MSG' , NULL ) ;
INSERT INTO va_categories_columns (column_id,category_id,column_order,column_code,column_title,column_html) VALUES (7 , 0 , 7 , 'quantity' , 'QTY_MSG' , NULL ) ;
INSERT INTO va_categories_columns (column_id,category_id,column_order,column_code,column_title,column_html) VALUES (8 , 0 , 8 , 'add_button' , '&nbsp;' , NULL ) ;












INSERT INTO va_change_types (type_id,type_name) VALUES (1 , 'Bug' ) ;
INSERT INTO va_change_types (type_id,type_name) VALUES (2 , 'New Feature' ) ;






INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (1 , 1 , 1 , 'language' , 'LANGUAGE_TITLE' , 'block_language.php' , NULL , NULL , NULL , 'bk-language' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (2 , 1 , 1 , 'user_profile' , 'PROFILE_TITLE' , 'block_user_profile.php' , NULL , NULL , NULL , 'bk-user-profile' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (3 , 1 , 1 , 'subscribe' , 'NEWSLETTER_SUBSRIPTION_MSG' , 'block_subscribe.php' , NULL , NULL , NULL , 'bk-subscribe' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (4 , 1 , 1 , 'login' , 'LOGIN_TITLE' , 'block_login.php' , NULL , NULL , NULL , 'bk-login' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (5 , 1 , 1 , 'login_advanced' , '{LOGIN_TITLE}: {ADVANCED_MSG}' , 'block_login_advanced.php' , NULL , NULL , NULL , 'bk-login-advanced' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (6 , 1 , 1 , 'forgot_password' , 'FORGOTTEN_PASSWORD_SETTINGS_MSG' , 'block_password_forgot.php' , NULL , NULL , NULL , 'bk-forgot-password' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (7 , 1 , 1 , 'reset_password' , 'RESET_PASSWORD_INFO_MSG' , 'block_password_reset.php' , NULL , NULL , NULL , 'bk-reset-password' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (8 , 1 , 1 , 'custom_page_body' , 'CUSTOM_PAGE_BODY_MSG' , 'block_custom_page_body.php' , NULL , NULL , NULL , 'bk-custom-page-body' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (9 , 1 , 1 , 'designs' , 'DESIGNS_MSG' , 'block_designs.php' , NULL , NULL , NULL , 'bk-designs' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (10 , 1 , 1 , 'poll' , 'POLL_TITLE' , 'block_poll.php' , NULL , NULL , NULL , 'bk-poll' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (11 , 1 , 1 , 'polls_previous_list' , 'PREVIOUS_POLLS_MSG' , 'block_polls_previous_list.php' , NULL , NULL , NULL , 'bk-polls-previous-list' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (12 , 1 , 1 , 'site_map' , 'SITE_MAP_TITLE' , 'block_site_map.php' , NULL , NULL , NULL , 'bk-site-map' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (13 , 1 , 1 , 'sms_test' , 'SMS_TEST_TITLE' , 'block_sms_test.php' , NULL , NULL , NULL , 'bk-sms-test' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (14 , 12 , 1 , 'user_account_breadcrumb' , '{USER_ACCOUNT_MSG}: {BREADCRUMB_MSG}' , 'block_user_account_breadcrumb.php' , NULL , NULL , NULL , 'bk-user-account-breadcrumb' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (15 , 12 , 1 , 'user_home' , '{USER_ACCOUNT_MSG}: {USER_HOME_TITLE}' , 'block_user_home.php' , NULL , NULL , NULL , 'bk-user-home' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (16 , 1 , 1 , 'site_search_form' , '{ADMIN_SITE_MSG}: {SEARCH_TITLE}' , 'block_site_search_form.php' , NULL , NULL , NULL , 'bk-site-search-form' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (17 , 1 , 1 , 'site_search_results' , '{ADMIN_SITE_MSG}: {SEARCH_RESULTS_MSG}' , 'block_site_search_results.php' , NULL , NULL , NULL , 'bk-site-search-results' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (18 , 2 , 1 , 'products_list' , 'PRODUCTS_LIST_MSG' , 'block_products_list.php' , NULL , NULL , NULL , 'bk-products-list' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (19 , 2 , 1 , 'product_details' , 'PRODUCT_DETAILS_MSG' , 'block_product_details.php' , NULL , NULL , NULL , 'bk-product-details' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (20 , 2 , 1 , 'categories_list' , 'CATEGORIES_LIST_MSG' , 'block_categories_list.php' , NULL , NULL , NULL , 'bk-categories-list' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (21 , 2 , 1 , 'category_description' , 'CATEGORY_INFO_MSG' , 'block_category_description.php' , NULL , NULL , NULL , 'bk-category-description' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (22 , 2 , 1 , 'shopping_cart' , 'CART_TITLE' , 'block_basket.php' , NULL , NULL , NULL , 'bk-shopping-cart' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (23 , 2 , 1 , 'cart_recommended_products' , '{CART_TITLE}: {PRODUCTS_RECOMMENDED_TITLE}' , 'block_basket_recommended.php' , NULL , NULL , NULL , 'bk-cart-recommended-products' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (24 , 2 , 1 , 'mini_cart' , 'SMALL_CART_MSG' , 'block_cart.php' , NULL , NULL , NULL , 'bk-mini-cart' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (25 , 2 , 1 , 'cart_retrieve' , 'RETRIEVE_CART_BUTTON' , 'block_cart_retrieve.php' , NULL , NULL , NULL , 'bk-cart-retrieve' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (26 , 2 , 1 , 'cart_save.php' , 'SAVE_CART_BUTTON' , 'block_cart_save.php' , NULL , NULL , NULL , 'bk-cart-save.php' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (27 , 2 , 1 , 'products_search' , '{PRODUCTS_TITLE} {SEARCH_FORM_MSG}' , 'block_search.php' , NULL , NULL , NULL , 'bk-products-search' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (28 , 2 , 1 , 'products_advanced_search' , 'PRODUCTS_SEARCH_ADVANCED_MSG' , 'block_products_search_advanced.php' , NULL , NULL , NULL , 'bk-products-advanced-search' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (29 , 2 , 1 , 'manufacturers_list' , 'MANUFACTURERS_TITLE' , 'block_manufacturers.php' , NULL , NULL , NULL , 'bk-manufacturers-list' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (30 , 2 , 1 , 'manufacturer_info' , 'MANUFACTURER_INFO_MSG' , 'block_manufacturer_info.php' , NULL , NULL , NULL , 'bk-manufacturer-info' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (31 , 2 , 1 , 'merchants_list' , 'MERCHANTS_TITLE' , 'block_merchants.php' , NULL , NULL , NULL , 'bk-merchants-list' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (32 , 2 , 1 , 'merchant_info' , 'MERCHANT_INFO_MSG' , 'block_merchant_info.php' , NULL , NULL , NULL , 'bk-merchant-info' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (33 , 2 , 1 , 'merchant_contact' , 'CONTACT_MERCHANT_TITLE' , 'block_merchant_contact.php' , NULL , NULL , NULL , 'bk-merchant-contact' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (34 , 2 , 1 , 'products_offers' , 'SPECIAL_OFFER_MSG' , 'block_offers.php' , NULL , NULL , NULL , 'bk-products-offers' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (35 , 2 , 1 , 'products_breadcrumb' , '{PRODUCTS_TITLE} {BREADCRUMB_MSG}' , 'block_products_breadcrumb.php' , NULL , NULL , NULL , 'bk-products-breadcrumb' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (36 , 2 , 1 , 'products_changes_log' , 'CHANGES_LOG_TITLE' , 'block_products_changes_log.php' , NULL , NULL , NULL , 'bk-products-changes-log' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (37 , 2 , 1 , 'products_compare' , 'PRODUCTS_COMPARE_RESULTS_MSG' , 'block_products_compare.php' , NULL , NULL , NULL , 'bk-products-compare' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (38 , 2 , 1 , 'products_fast_add' , 'FAST_PRODUCT_ADDING_MSG' , 'block_products_fast_add.php' , NULL , NULL , NULL , 'bk-products-fast-add' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (39 , 2 , 1 , 'products_latest' , '{PRODUCTS_TITLE} {LATEST_TITLE}' , 'block_products_latest.php' , NULL , NULL , NULL , 'bk-products-latest' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (40 , 2 , 1 , 'products_options' , 'PRODUCT_OPTIONS_MSG' , 'block_product_options.php' , NULL , NULL , NULL , 'bk-products-options' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (41 , 2 , 1 , 'products_recently_viewed' , '{PRODUCTS_TITLE} {RECENTLY_VIEWED_MSG}' , 'block_products_recently.php' , NULL , NULL , NULL , 'bk-products-recently-viewed' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (42 , 2 , 1 , 'products_recommended' , '{REGISTERED_CUSTOMERS_MSG}: {PRODUCTS_RECOMMENDED_TITLE}' , 'block_products_recommended.php' , NULL , NULL , NULL , 'bk-products-recommended' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (43 , 2 , 1 , 'products_releases' , '{PRODUCTS_TITLE} {RELEASES_TITLE}' , 'block_products_releases.php' , NULL , NULL , NULL , 'bk-products-releases' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (44 , 2 , 1 , 'products_releases_hot' , '{PRODUCTS_TITLE} {HOT_RELEASES_MSG}' , 'block_products_releases_hot.php' , NULL , NULL , NULL , 'bk-products-releases-hot' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (45 , 2 , 1 , 'products_top_sellers' , 'TOP_SELLERS_TITLE' , 'block_products_top_sellers.php' , NULL , NULL , NULL , 'bk-products-top-sellers' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (46 , 2 , 1 , 'products_top_viewed' , 'TOP_VIEWED_TITLE' , 'block_products_top_viewed.php' , NULL , NULL , NULL , 'bk-products-top-viewed' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (47 , 2 , 1 , 'products_top_rated' , 'TOP_RATED_TITLE' , 'block_products_top_rated.php' , NULL , NULL , NULL , 'bk-products-top-rated' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (48 , 2 , 1 , 'products_related' , '{PRODUCTS_TITLE} {RELATED_MSG}' , 'block_products_related.php' , NULL , NULL , NULL , 'bk-products-related' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (49 , 2 , 1 , 'products_related_purchase' , 'WHO_BOUGHT_THIS_SHORT_MSG' , 'block_products_related_purchase.php' , NULL , NULL , NULL , 'bk-products-related-purchase' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (50 , 2 , 1 , 'checkout_breadcrumb' , '{ADMIN_CHECKOUT_MSG} {BREADCRUMB_MSG}' , 'block_checkout_breadcrumb.php' , NULL , NULL , NULL , 'bk-checkout-breadcrumb' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (51 , 2 , 1 , 'checkout_final' , 'FINAL_CHECKOUT_MSG' , 'block_checkout_final.php' , NULL , NULL , NULL , 'bk-checkout-final' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (52 , 2 , 1 , 'checkout_login' , 'CHECKOUT_LOGIN_TITLE' , 'block_checkout_login.php' , NULL , NULL , NULL , 'bk-checkout-login' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (53 , 2 , 1 , 'order_cart' , 'ORDER_CART_MSG' , 'block_order_cart.php' , NULL , NULL , NULL , 'bk-order-cart' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (54 , 2 , 1 , 'order_form' , 'ORDER_FORM_MSG' , 'block_order_info.php' , NULL , NULL , NULL , 'bk-order-form' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (55 , 2 , 1 , 'order_preview' , 'ORDER_PREVIEW_MSG' , 'block_order_data_preview.php' , NULL , NULL , NULL , 'bk-order-preview' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (56 , 2 , 1 , 'order_payment_details' , 'PAYMENT_DETAILS_MSG' , 'block_order_payment_form.php' , NULL , NULL , NULL , 'bk-order-payment-details' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (57 , 2 , 1 , 'coupon_form' , 'COUPON_INFO_MSG' , 'block_coupon_form.php' , NULL , NULL , NULL , 'bk-coupon-form' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (58 , 2 , 1 , 'currencies_list' , 'CURRENCIES_MSG' , 'block_currency.php' , NULL , NULL , NULL , 'bk-currencies-list' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (59 , 2 , 1 , 'subscriptions' , 'SUBSCRIPTIONS_MSG' , 'block_subscriptions.php' , NULL , NULL , NULL , 'bk-subscriptions' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (60 , 2 , 1 , 'subscriptions_breadcrumb' , '{SUBSCRIPTIONS_MSG}: {BREADCRUMB_MSG}' , 'block_subscriptions_breadcrumb.php' , NULL , NULL , NULL , 'bk-subscriptions-breadcrumb' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (61 , 2 , 1 , 'users_bought_item' , 'CUSTOMERS_LIST_BOUGHT_ITEM_TITLE' , 'block_products_users_bought.php' , NULL , NULL , NULL , 'bk-users-bought-item' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (62 , 2 , 1 , 'wishlist_items' , 'WISHLIST_ITEMS_SETTINGS_MSG' , 'block_wishlist_items.php' , NULL , NULL , NULL , 'bk-wishlist-items' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (63 , 2 , 1 , 'wishlist_search' , '{WISHLIST_MSG}: {SEARCH_TITLE}' , 'block_wishlist_search.php' , NULL , NULL , NULL , 'bk-wishlist-search' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (64 , 2 , 1 , 'product_related_articles' , 'RELATED_ARTICLES_MSG' , 'block_articles_related.php' , NULL , NULL , NULL , 'bk-product-related-articles' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (65 , 2 , 1 , 'product_related_forums' , 'RELATED_FORUMS_MSG' , 'block_forums_related.php' , NULL , NULL , NULL , 'bk-product-related-forums' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (66 , 3 , 1 , 'articles_breadcrumb' , 'BREADCRUMB_MSG' , 'block_articles_breadcrumb.php' , NULL , NULL , NULL , 'bk-articles-breadcrumb' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (67 , 3 , 1 , 'articles_list' , 'ARTICLES_LIST_MSG' , 'block_articles_list.php' , NULL , NULL , NULL , 'bk-articles-list' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (68 , 3 , 1 , 'article_details' , 'ARTICLE_MSG' , 'block_articles_details.php' , NULL , NULL , NULL , 'bk-article-details' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (69 , 3 , 1 , 'articles_categories' , 'CATEGORIES_LIST_MSG' , 'block_articles_categories.php' , NULL , NULL , NULL , 'bk-articles-categories' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (70 , 3 , 1 , 'articles_category_description' , 'CATEGORY_INFO_MSG' , 'block_articles_category.php' , NULL , NULL , NULL , 'bk-articles-category-description' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (71 , 3 , 1 , 'articles_content' , 'CONTENT_TITLE' , 'block_articles_content.php' , NULL , NULL , NULL , 'bk-articles-content' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (72 , 3 , 1 , 'articles_hot' , 'HOT_TITLE' , 'block_articles_hot.php' , NULL , NULL , NULL , 'bk-articles-hot' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (73 , 3 , 1 , 'articles_related' , 'RELATED_ARTICLES_MSG' , 'block_articles_related.php' , NULL , NULL , NULL , 'bk-articles-related' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (74 , 3 , 1 , 'articles_related_forums' , 'RELATED_FORUMS_MSG' , 'block_forums_related.php' , NULL , NULL , NULL , 'bk-articles-related-forums' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (75 , 3 , 1 , 'articles_related_products' , 'ARTICLE_RELATED_PRODUCTS_TITLE' , 'block_products_related.php' , NULL , NULL , NULL , 'bk-articles-related-products' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (76 , 3 , 1 , 'articles_category_products_relat' , 'CATEGORY_RELATED_PRODUCTS_TITLE' , 'block_products_related.php' , NULL , NULL , NULL , 'bk-articles-category-products-relat' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (77 , 3 , 1 , 'articles_latest' , 'LATEST_TITLE' , 'block_articles_latest.php' , NULL , NULL , NULL , 'bk-articles-latest' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (78 , 3 , 1 , 'articles_search' , 'SEARCH_TITLE' , 'block_articles_search.php' , NULL , NULL , NULL , 'bk-articles-search' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (79 , 3 , 1 , 'articles_top_rated' , 'TOP_RATED_TITLE' , 'block_articles_top_rated.php' , NULL , NULL , NULL , 'bk-articles-top-rated' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (80 , 3 , 1 , 'articles_top_viewed' , 'TOP_VIEWED_TITLE' , 'block_articles_top_viewed.php' , NULL , NULL , NULL , 'bk-articles-top-viewed' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (81 , 4 , 1 , 'ticket_new' , 'SUBMIT_TICKET_MSG' , 'block_support.php' , NULL , NULL , NULL , 'bk-ticket-new' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (82 , 4 , 1 , 'ticket_reply' , 'REPLYING_TICKETS_MSG' , 'block_support_reply.php' , NULL , NULL , NULL , 'bk-ticket-reply' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (83 , 5 , 1 , 'forum_breadcrumb' , 'BREADCRUMB_MSG' , 'block_forum_breadcrumb.php' , NULL , NULL , NULL , 'bk-forum-breadcrumb' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (84 , 5 , 1 , 'forum_list' , 'FORUMS_LIST_PAGE_MSG' , 'block_forum_list.php' , NULL , NULL , NULL , 'bk-forum-list' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (85 , 5 , 1 , 'forum_description' , 'DESCRIPTION_MSG' , 'block_forum_description.php' , NULL , NULL , NULL , 'bk-forum-description' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (86 , 5 , 1 , 'forum_topics' , 'TOPICS_LIST_MSG' , 'block_forum_topics.php' , NULL , NULL , NULL , 'bk-forum-topics' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (87 , 5 , 1 , 'forum_topic' , 'FORUM_TOPICS_THREAD_MSG' , 'block_forum_topic.php' , NULL , NULL , NULL , 'bk-forum-topic' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (88 , 5 , 1 , 'forum_top_viewed' , 'TOP_VIEWED_TITLE' , 'block_forum_top_viewed.php' , NULL , NULL , NULL , 'bk-forum-top-viewed' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (89 , 5 , 1 , 'forum_latest' , 'LATEST_TITLE' , 'block_forum_latest.php' , NULL , NULL , NULL , 'bk-forum-latest' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (90 , 5 , 1 , 'forum_related_articles' , 'RELATED_ARTICLES_MSG' , 'block_articles_related.php' , NULL , NULL , NULL , 'bk-forum-related-articles' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (91 , 5 , 1 , 'forum_related_products' , 'RELATED_PRODUCTS_TITLE' , 'block_products_related.php' , NULL , NULL , NULL , 'bk-forum-related-products' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (92 , 5 , 1 , 'forum_search' , 'SEARCH_TITLE' , 'block_forum_search.php' , NULL , NULL , NULL , 'bk-forum-search' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (93 , 6 , 1 , 'ads_breadcrumb' , 'BREADCRUMB_MSG' , 'block_ads_breadcrumb.php' , NULL , NULL , NULL , 'bk-ads-breadcrumb' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (94 , 6 , 1 , 'ads_categories' , 'CATEGORIES_LIST_MSG' , 'block_ads_categories.php' , NULL , NULL , NULL , 'bk-ads-categories' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (95 , 6 , 1 , 'ads_category_info' , 'CATEGORY_INFO_MSG' , 'block_ads_category.php' , NULL , NULL , NULL , 'bk-ads-category-info' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (96 , 6 , 1 , 'ads_list' , 'ADS_LISTING_MSG' , 'block_ads_list.php' , NULL , NULL , NULL , 'bk-ads-list' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (97 , 6 , 1 , 'ads_details' , 'ADS_DETAILS_MSG' , 'block_ads_details.php' , NULL , NULL , NULL , 'bk-ads-details' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (98 , 6 , 1 , 'ads_compare' , 'PRODUCTS_COMPARE_RESULTS_MSG' , 'block_ads_compare.php' , NULL , NULL , NULL , 'bk-ads-compare' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (99 , 6 , 1 , 'ads_add' , 'NEW_AD_MSG' , 'block_ads_add.php' , NULL , NULL , NULL , 'bk-ads-add' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (100 , 6 , 1 , 'ads_hot' , 'HOT_OFFERS_MSG' , 'block_ads_hot.php' , NULL , NULL , NULL , 'bk-ads-hot' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (101 , 6 , 1 , 'ads_special' , 'SPECIAL_OFFER_MSG' , 'block_ads_special.php' , NULL , NULL , NULL , 'bk-ads-special' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (102 , 6 , 1 , 'ads_latest' , 'LATEST_TITLE' , 'block_ads_latest.php' , NULL , NULL , NULL , 'bk-ads-latest' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (103 , 6 , 1 , 'ads_top_viewed' , 'TOP_VIEWED_TITLE' , 'block_ads_top_viewed.php' , NULL , NULL , NULL , 'bk-ads-top-viewed' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (104 , 6 , 1 , 'ads_recently_viewed' , 'RECENTLY_VIEWED_MSG' , 'block_ads_recently.php' , NULL , NULL , NULL , 'bk-ads-recently-viewed' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (105 , 6 , 1 , 'ads_sellers' , 'SELLERS_MSG' , 'block_ads_sellers.php' , NULL , NULL , NULL , 'bk-ads-sellers' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (106 , 6 , 1 , 'ads_search' , 'SEARCH_FORM_MSG' , 'block_ads_search.php' , NULL , NULL , NULL , 'bk-ads-search' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (107 , 6 , 1 , 'ads_search_advanced' , 'ADVANCED_SEARCH_TITLE' , 'block_ads_search_advanced.php' , NULL , NULL , NULL , 'bk-ads-search-advanced' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (108 , 7 , 1 , 'manuals_breadcrumb' , 'BREADCRUMB_MSG' , 'block_manuals_breadcrumb.php' , NULL , NULL , NULL , 'bk-manuals-breadcrumb' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (109 , 7 , 1 , 'manuals_list' , 'MANUAL_LIST_MSG' , 'block_manuals_list.php' , NULL , NULL , NULL , 'bk-manuals-list' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (110 , 7 , 1 , 'manuals_articles' , 'MANUAL_ARTICLES_MSG' , 'block_manuals_articles.php' , NULL , NULL , NULL , 'bk-manuals-articles' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (111 , 7 , 1 , 'manuals_article_details' , 'MANUAL_ARTICLE_DETAILS_MSG' , 'block_manuals_article_details.php' , NULL , NULL , NULL , 'bk-manuals-article-details' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (112 , 7 , 1 , 'manuals_search' , 'SEARCH_TITLE' , 'block_manuals_search.php' , NULL , NULL , NULL , 'bk-manuals-search' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (113 , 7 , 1 , 'manuals_search_results' , 'MANUAL_SEARCH_RESULTS_MSG' , 'block_manuals_search_results.php' , NULL , NULL , NULL , 'bk-manuals-search-results' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (114 , 8 , 1 , 'filter' , '{filter_name}' , 'block_filter.php' , NULL , NULL , NULL , 'bk-filter' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (115 , 9 , 1 , 'custom_block' , '{block_name}' , 'block_custom.php' , NULL , NULL , NULL , 'bk-custom-block' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (116 , 10 , 1 , 'navigation' , '{menu_title}' , 'block_navigation.php' , NULL , NULL , NULL , 'bk-navigation' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (117 , 11 , 1 , 'banners' , '{group_name}' , 'block_banners.php' , NULL , NULL , NULL , 'bk-banners' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (118 , 1 , 1 , 'header' , 'HEADER_MSG' , 'header.php' , NULL , NULL , NULL , 'bk-header' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (119 , 1 , 1 , 'footer' , 'FOOTER_MSG' , 'footer.php' , NULL , NULL , NULL , 'bk-footer' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (120 , 2 , 1 , 'product_reviews' , 'PRODUCTS_REVIEWS_MSG' , 'block_reviews.php' , NULL , NULL , NULL , 'bk-product-reviews' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (121 , 3 , 1 , 'article_reviews' , 'ARTICLES_REVIEWS_MSG' , 'block_articles_reviews.php' , NULL , NULL , NULL , 'bk-article-reviews' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (122 , 1 , 1 , 'download' , 'DOWNLOAD_TITLE' , 'block_download.php' , NULL , NULL , NULL , 'bk-download' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (123 , 5 , 1 , 'forum_topic_new' , 'NEW_TOPIC_MSG' , 'block_forum_topic_new.php' , NULL , NULL , NULL , 'bk-forum-topic-new' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (124 , 12 , 1 , 'user_account_profile' , 'PROFILE_TITLE' , 'block_user_profile.php' , NULL , NULL , NULL , 'bk-user-account-profile' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (125 , 12 , 1 , 'user_change_password' , 'CHANGE_PASSWORD_MSG' , 'block_user_change_password.php' , NULL , NULL , NULL , 'bk-user-change-password' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (126 , 12 , 1 , 'user_change_type' , 'UPGRADE_DOWNGRADE_MSG' , 'block_user_change_type.php' , NULL , NULL , NULL , 'bk-user-change-type' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (127 , 12 , 1 , 'user_carts' , 'MY_SAVED_CARTS_MSG' , 'block_user_carts.php' , NULL , NULL , NULL , 'bk-user-carts' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (128 , 12 , 1 , 'user_wishlist' , 'MY_WISHLIST_MSG' , 'block_user_wishlist.php' , NULL , NULL , NULL , 'bk-user-wishlist' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (129 , 12 , 1 , 'user_orders' , '{MY_ORDERS_MSG}: {LIST_MSG}' , 'block_user_orders.php' , NULL , NULL , NULL , 'bk-user-orders' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (130 , 12 , 1 , 'user_order' , '{MY_ORDERS_MSG}: {VIEW_DETAILS_MSG}' , 'block_user_order.php' , NULL , NULL , NULL , 'bk-user-order' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (131 , 12 , 1 , 'user_order_update' , '{MY_ORDERS_MSG}:  {EDIT_MSG}' , 'block_user_order_update.php' , NULL , NULL , NULL , 'bk-user-order-update' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (132 , 12 , 1 , 'user_support' , 'MY_SUPPORT_ISSUES_MSG' , 'block_user_support.php' , NULL , NULL , NULL , 'bk-user-support' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (133 , 12 , 1 , 'user_ads' , '{MY_ADS_MSG}: {LIST_MSG}' , 'block_user_ads.php' , NULL , NULL , NULL , 'bk-user-ads' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (134 , 12 , 1 , 'user_ad' , '{MY_ADS_MSG}: {EDIT_MSG}' , 'block_user_ad.php' , NULL , NULL , NULL , 'bk-user-ad' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (135 , 12 , 1 , 'user_affiliate_sales' , 'AFFILIATE_SALES_MSG' , 'block_user_affiliate_sales.php' , NULL , NULL , NULL , 'bk-user-affiliate-sales' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (136 , 12 , 1 , 'user_affiliate_items' , '{AFFILIATE_SALES_MSG}: {PRODUCTS_LIST_MSG}' , 'block_user_affiliate_items.php' , NULL , NULL , NULL , 'bk-user-affiliate-items' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (137 , 12 , 1 , 'user_products' , '{MY_PRODUCTS_MSG}: {LIST_MSG}' , 'block_user_products.php' , NULL , NULL , NULL , 'bk-user-products' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (138 , 12 , 1 , 'user_product' , '{MY_PRODUCTS_MSG}: {EDIT_MSG}' , 'block_user_product.php' , NULL , NULL , NULL , 'bk-user-product' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (139 , 12 , 1 , 'user_product_options' , '{OPTIONS_AND_COMPONENTS_MSG}: {LIST_MSG}' , 'block_user_product_options.php' , NULL , NULL , NULL , 'bk-user-product-options' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (140 , 12 , 1 , 'user_product_option' , '{OPTIONS_AND_COMPONENTS_MSG}: {EDIT_OPTION_MSG}' , 'block_user_product_option.php' , NULL , NULL , NULL , 'bk-user-product-option' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (141 , 12 , 1 , 'user_product_subcomponent' , '{OPTIONS_AND_COMPONENTS_MSG}: {EDIT_SUBCOMP_MSG}' , 'block_user_product_subcomponent.php' , NULL , NULL , NULL , 'bk-user-product-subcomponent' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (142 , 12 , 1 , 'user_product_subcomponents' , '{OPTIONS_AND_COMPONENTS_MSG}: {EDIT_SUBCOMP_SELECTION_MSG}' , 'block_user_product_subcomponents.php' , NULL , NULL , NULL , 'bk-user-product-subcomponents' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (143 , 12 , 1 , 'user_payments' , '{COMMISSION_PAYMENTS_MSG}: {LIST_MSG}' , 'block_user_payments.php' , NULL , NULL , NULL , 'bk-user-payments' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (144 , 12 , 1 , 'user_payment' , '{COMMISSION_PAYMENTS_MSG}: {VIEW_DETAILS_MSG}' , 'block_user_payment.php' , NULL , NULL , NULL , 'bk-user-payment' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (145 , 12 , 1 , 'user_merchant_orders' , '{MY_SALES_ORDERS_MSG}: {LIST_MSG}' , 'block_user_merchant_orders.php' , NULL , NULL , NULL , 'bk-user-merchant-orders' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (146 , 12 , 1 , 'user_merchant_order' , '{MY_SALES_ORDERS_MSG}: {VIEW_DETAILS_MSG}' , 'block_user_merchant_order.php' , NULL , NULL , NULL , 'bk-user-merchant-order' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (147 , 12 , 1 , 'user_merchant_sales' , 'MERCHANT_SALES_MSG' , 'block_user_merchant_sales.php' , NULL , NULL , NULL , 'bk-user-merchant-sales' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (148 , 12 , 1 , 'user_merchant_items' , '{MERCHANT_SALES_MSG}: {PRODUCTS_LIST_MSG}' , 'block_user_merchant_items.php' , NULL , NULL , NULL , 'bk-user-merchant-items' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (149 , 12 , 1 , 'user_reminders' , '{MY_REMINDERS_MSG}: {LIST_MSG}' , 'block_user_reminders.php' , NULL , NULL , NULL , 'bk-user-reminders' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (150 , 12 , 1 , 'user_reminder' , '{MY_REMINDERS_MSG}: {EDIT_MSG}' , 'block_user_reminder.php' , NULL , NULL , NULL , 'bk-user-reminder' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (151 , 12 , 1 , 'user_psd_list' , '{PAYMENT_DETAILS_MSG}: {LIST_MSG}' , 'block_user_psd_list.php' , NULL , NULL , NULL , 'bk-user-psd-list' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (152 , 12 , 1 , 'user_psd_update' , '{PAYMENT_DETAILS_MSG}: {EDIT_MSG}' , 'block_user_psd_update.php' , NULL , NULL , NULL , 'bk-user-psd-update' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (153 , 12 , 1 , 'user_product_registrations' , 'MY_PRODUCT_REGISTRATIONS_MSG' , 'block_user_product_registrations.php' , NULL , NULL , NULL , 'bk-user-product-registrations' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (154 , 12 , 1 , 'user_product_registration' , 'REGISTER_PRODUCT_MSG' , 'block_user_product_registration.php' , NULL , NULL , NULL , 'bk-user-product-registration' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (155 , 1 , 1 , 'contact_us' , 'CONTACT_US_TITLE' , 'block_contact_us.php' , NULL , NULL , NULL , 'bk-contact-us' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (156 , 13 , 1 , 'sliders' , '{slider_name}' , 'block_sliders.php' , NULL , NULL , NULL , 'bk-sliders' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (157 , 1 , 1 , 'unsubscribe' , 'UNSUBSCRIBE_TITLE' , 'block_unsubscribe.php' , NULL , NULL , NULL , 'bk-unsubscribe' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (158 , 14 , 1 , 'facebook_feed' , 'Facebook Feed' , 'block_facebook_feed.php' , NULL , NULL , NULL , 'bk-facebook-feed' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (159 , 14 , 1 , 'twitter_feed' , 'Twitter Feed' , 'block_twitter_feed.php' , NULL , NULL , NULL , 'bk-twitter-feed' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (160 , 12 , 1 , 'user_addresses' , '{MY_ADDRESSES_MSG}: {LIST_MSG}' , 'block_user_addresses.php' , NULL , NULL , NULL , 'bk-user-addresses' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (161 , 12 , 1 , 'user_address' , '{MY_ADDRESSES_MSG}: {EDIT_MSG}' , 'block_user_address.php' , NULL , NULL , NULL , 'bk-user-address' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (162 , 1 , 1 , 'site_navigation' , 'SITE_NAVIGATION_MSG' , 'block_site_navigation.php' , NULL , NULL , NULL , 'bk-site-navigation' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (163 , 1 , 1 , 'cookies_control' , 'COOKIE_CONTROL_MSG' , 'block_cookies_control.php' , NULL , NULL , NULL , 'bk-cookies-control' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (164 , 2 , 1 , 'friend_form' , 'FRIEND_AFFILIATE_FORM_MSG' , 'block_friend_form.php' , NULL , NULL , NULL , 'bk-friend-form' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (165 , 4 , 1 , 'support_live' , 'SUPPORT_LIVE_MSG' , 'block_support_live.php' , NULL , NULL , NULL , 'bk-support-live' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (166 , 12 , 1 , 'user_messages' , 'MY_MESSAGES_MSG' , 'block_user_messages.php' , NULL , NULL , NULL , 'bk-user-messages' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (167 , 15 , 1 , 'profiles_search' , 'SEARCH_FORM_MSG' , 'block_profiles_search.php' , NULL , NULL , NULL , 'bk-profiles-search' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (168 , 15 , 2 , 'profiles_list' , '{PROFILES_TITLE}: {LIST_MSG}' , 'block_profiles_list.php' , NULL , NULL , NULL , 'bk-profiles-list' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (169 , 15 , 3 , 'profiles_view' , '{PROFILES_TITLE}: {VIEW_MSG}' , 'block_profiles_details.php' , NULL , NULL , NULL , 'bk-profiles-view' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (170 , 15 , 4 , 'profiles_user_list' , '{MY_PROFILES_MSG}: {LIST_MSG}' , 'block_profiles_user_list.php' , NULL , NULL , NULL , 'bk-profiles-user-list' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (171 , 15 , 5 , 'profiles_user_edit' , '{MY_PROFILES_MSG}: {EDIT_MSG}' , 'block_profiles_user_edit.php' , NULL , NULL , NULL , 'bk-profiles-user-edit' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (172 , 1 , 1 , 'navigation_bar' , 'NAVIGATION_BAR_MSG' , 'block_navigation_bar.php' , NULL , NULL , NULL , 'bk-navigation-bar' , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (173 , 3 , 1 , 'articles_random' , 'RANDOM_TITLE' , 'block_articles_random.php' , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (174 , 16 , 1 , 'authors_list' , '{AUTHORS_MSG}: {LIST_MSG}' , 'block_authors_list.php' , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (175 , 16 , 2 , 'author_details' , '{AUTHOR_MSG}: {DETAILED_DESCRIPTION_MSG}' , 'block_author_details.php' , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (176 , 16 , 3 , 'author_articles' , '{AUTHOR_MSG}: {ARTICLES_TITLE}' , 'block_author_articles.php' , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (177 , 16 , 1 , 'albums_list' , '{ALBUMS_MSG}: {LIST_MSG}' , 'block_albums_list.php' , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (178 , 16 , 2 , 'album_details' , '{ALBUM_MSG}: {DETAILED_DESCRIPTION_MSG}' , 'block_album_details.php' , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (179 , 16 , 3 , 'album_articles' , '{ALBUM_MSG}: {ARTICLES_TITLE}' , 'block_album_articles.php' , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_cms_blocks (block_id,module_id,block_order,block_code,block_name,php_script,layout_type,layout_template,html_template,css_class,block_title,pages_all) VALUES (180 , 16 , 1 , 'author_breadcrumb' , '{AUTHOR_MSG}: {BREADCRUMB_MSG}' , 'block_author_breadcrumb.php' , NULL , NULL , NULL , NULL , NULL , 1 ) ;






INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (1 , 1 , 1 , 'SELECT_TYPE_MSG' , 'RADIOBUTTON' , NULL , NULL , 'language_selection' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , '<br>' , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (2 , 9 , 1 , 'SELECT_TYPE_MSG' , 'RADIOBUTTON' , NULL , NULL , 'design_selection' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , '<br>' , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (3 , 13 , 1 , 'ORIGINATOR_MSG' , 'TEXTBOX' , NULL , NULL , 'sms_originator' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (4 , 13 , 2 , 'SMS_TEST_MESSAGE_MSG' , 'TEXTAREA' , NULL , NULL , 'sms_test_message' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (5 , 18 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'products_per_page' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (6 , 18 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'products_columns' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (7 , 18 , 3 , 'NAVIGATOR_SORTING_SETTINGS_MSG' , 'LISTBOX' , NULL , NULL , 'products_nav_type' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (8 , 18 , 4 , 'NAVIGATOR_FIRST_LAST_PAGES_MSG' , 'CHECKBOX' , NULL , NULL , 'products_nav_first_last' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (9 , 18 , 5 , 'NAVIGATOR_PREV_NEXT_PAGES_MSG' , 'CHECKBOX' , NULL , NULL , 'products_nav_prev_next' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (10 , 18 , 6 , 'NUMBER_OF_PAGES_NAVIGATOR_MSG' , 'TEXTBOX' , NULL , NULL , 'products_nav_pages' , '5' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (11 , 18 , 7 , 'SHOW_PRODUCTS_SORTING_MSG' , 'CHECKBOX' , NULL , NULL , 'products_sortings' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (12 , 18 , 8 , 'GROUP_PRODUCTS_BY_CATEGORIES_MSG' , 'CHECKBOX' , NULL , NULL , 'products_group_by_cats' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (13 , 18 , 9 , 'DEFAULT_VIEW_TYPE_MSG' , 'LISTBOX' , NULL , NULL , 'products_default_view' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (15 , 19 , 2 , 'SHOW_SUPER_IMAGE_MSG' , 'LISTBOX' , NULL , NULL , 'show_super_image' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (16 , 19 , 3 , 'MANUFACTURER_IMAGE_MSG' , 'LISTBOX' , NULL , NULL , 'show_manufacturer_image' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (17 , 20 , 1 , 'CATEGORIES_TYPE_VIEW_MSG' , 'LISTBOX' , NULL , NULL , 'categories_type' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (18 , 20 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'categories_columns' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (19 , 20 , 3 , 'NUMBER_OF_SHOWN_SUBS_MSG' , 'TEXTBOX' , 17 , 20 , 'categories_subs' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (20 , 20 , 4 , 'IMAGE_TYPE_FOR_TOP_CAT_MSG' , 'LISTBOX' , NULL , NULL , 'categories_image' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (21 , 21 , 1 , 'IMAGE_MSG' , 'LISTBOX' , NULL , NULL , 'category_description_image' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (22 , 21 , 2 , 'DESCRIPTION_MSG' , 'LISTBOX' , NULL , NULL , 'category_description_type' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (23 , 22 , 1 , 'IMAGE_TYPE_MSG' , 'LISTBOX' , NULL , NULL , 'image_type' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (25 , 29 , 1 , 'SELECT_TYPE_MSG' , 'RADIOBUTTON' , NULL , NULL , 'manufacturers_selection' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (26 , 29 , 2 , 'IMAGE_TYPE_MSG' , 'LISTBOX' , NULL , NULL , 'manufacturers_image' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (27 , 29 , 3 , 'DESCRIPTION_MSG' , 'LISTBOX' , NULL , NULL , 'manufacturers_desc' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (28 , 29 , 4 , 'SORT_ORDER_MSG' , 'LISTBOX' , NULL , NULL , 'manufacturers_order' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (29 , 29 , 5 , 'SORT_DIRECTION_MSG' , 'LISTBOX' , NULL , NULL , 'manufacturers_direction' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (30 , 30 , 1 , 'IMAGE_TYPE_MSG' , 'LISTBOX' , NULL , NULL , 'manufacturer_info_image' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (31 , 30 , 2 , 'DESCRIPTION_MSG' , 'LISTBOX' , NULL , NULL , 'manufacturer_info_type' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (32 , 31 , 1 , 'SELECT_TYPE_MSG' , 'RADIOBUTTON' , NULL , NULL , 'merchants_selection' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (33 , 32 , 1 , 'PERSONAL_IMAGE_FIELD' , 'CHECKBOX' , NULL , NULL , 'merchant_info_image' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (34 , 32 , 2 , 'NICKNAME_FIELD' , 'CHECKBOX' , NULL , NULL , 'merchant_info_nick' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (35 , 32 , 3 , 'MERCHANT_COUNTRY_FLAG_MSG' , 'CHECKBOX' , NULL , NULL , 'merchant_info_country_flag' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (36 , 32 , 4 , '{ONLINE_MSG} / {OFFLINE_MSG}' , 'CHECKBOX' , NULL , NULL , 'merchant_info_online' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (37 , 32 , 5 , 'MEMBER_SINCE_MSG' , 'CHECKBOX' , NULL , NULL , 'merchant_info_member' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (38 , 32 , 6 , 'MERCHANT_PRODUCTS_LINK_MSG' , 'CHECKBOX' , NULL , NULL , 'merchant_info_prod_link' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (39 , 32 , 7 , 'DESCRIPTION_MSG' , 'LISTBOX' , NULL , NULL , 'merchant_info_desc_value' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (40 , 34 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'prod_offers_recs' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (41 , 34 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'prod_offers_cols' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (42 , 34 , 3 , 'SHOW_POINTS_PRICE_MSG' , 'CHECKBOX' , NULL , NULL , 'prod_offers_points_price' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (43 , 34 , 4 , 'SHOW_REWARD_POINTS_MSG' , 'CHECKBOX' , NULL , NULL , 'prod_offers_reward_points' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (44 , 34 , 5 , 'SHOW_REWARD_CREDITS_MSG' , 'CHECKBOX' , NULL , NULL , 'prod_offers_reward_credits' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (45 , 34 , 6 , 'ADD_TO_CART_MSG' , 'CHECKBOX' , NULL , NULL , 'prod_offers_add_button' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (46 , 34 , 7 , 'VIEW_CART_MSG' , 'CHECKBOX' , NULL , NULL , 'prod_offers_view_button' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (47 , 34 , 8 , 'GOTO_CHECKOUT_MSG' , 'CHECKBOX' , NULL , NULL , 'prod_offers_goto_button' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (48 , 34 , 9 , 'ADD_TO_WISHLIST_MSG' , 'CHECKBOX' , NULL , NULL , 'prod_offers_wish_button' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (49 , 34 , 10 , 'QUANTITY_CONTROL_MSG' , 'LISTBOX' , NULL , NULL , 'prod_offers_quantity_control' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (50 , 34 , 11 , 'SLIDER_TYPE_MSG' , 'LISTBOX' , NULL , NULL , 'prod_slider_type' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (51 , 34 , 12 , 'SLIDER_WIDTH_MSG' , 'TEXTBOX' , NULL , NULL , 'prod_slider_width' , '100%' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (52 , 34 , 13 , 'SLIDER_HEIGHT_MSG' , 'TEXTBOX' , NULL , NULL , 'prod_slider_height' , '300' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (53 , 34 , 14 , 'SLIDER_STYLE_MSG' , 'TEXTAREA' , NULL , NULL , 'prod_slider_style' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (61 , 39 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'products_latest_recs' , '10' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (62 , 39 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'products_latest_cols' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (63 , 39 , 3 , 'IMAGE_TYPE_MSG' , 'LISTBOX' , NULL , NULL , 'prod_latest_image' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (64 , 39 , 4 , 'DESCRIPTION_MSG' , 'LISTBOX' , NULL , NULL , 'prod_latest_desc' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (65 , 39 , 5 , 'SORT_ORDER_MSG' , 'LISTBOX' , NULL , NULL , 'prod_latest_order' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (66 , 41 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'products_recent_records' , '5' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (67 , 41 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'products_recent_cols' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (68 , 41 , 3 , 'IMAGE_TYPE_MSG' , 'LISTBOX' , NULL , NULL , 'recent_image' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (69 , 41 , 4 , 'DESCRIPTION_MSG' , 'LISTBOX' , NULL , NULL , 'recent_desc' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (70 , 42 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'prod_recom_per_page' , '10' , 0 , NULL , NULL , NULL , '{RECOMMENDED_PRODUCTS_DESC}\r\n<br><br>' , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (71 , 42 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'prod_recom_cols' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (72 , 45 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'bestsellers_records' , '10' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (73 , 45 , 2 , 'RECENT_PERIOD_PRODUCTS_BOUGHT_MSG' , 'TEXTBOX' , NULL , NULL , 'bestsellers_days' , '90' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 'DAYS_MSG' , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (74 , 45 , 3 , 'ORDER_STATUS_MSG' , 'LISTBOX' , NULL , NULL , 'bestsellers_status' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (75 , 45 , 4 , 'IMAGE_TYPE_MSG' , 'LISTBOX' , NULL , NULL , 'bestsellers_image' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (76 , 45 , 5 , 'DESCRIPTION_MSG' , 'LISTBOX' , NULL , NULL , 'bestsellers_desc' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (77 , 46 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'prod_top_viewed_recs' , '10' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (78 , 46 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'prod_top_viewed_cols' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (79 , 46 , 3 , 'IMAGE_TYPE_MSG' , 'LISTBOX' , NULL , NULL , 'top_viewed_image' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (80 , 46 , 4 , 'DESCRIPTION_MSG' , 'LISTBOX' , NULL , NULL , 'top_viewed_desc' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (81 , 47 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'top_rates_recs' , '10' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (82 , 47 , 2 , 'IMAGE_TYPE_MSG' , 'LISTBOX' , NULL , NULL , 'top_rated_image' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (83 , 47 , 3 , 'DESCRIPTION_MSG' , 'LISTBOX' , NULL , NULL , 'top_rated_desc' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (84 , 48 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'related_per_page' , '4' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (85 , 48 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'related_columns' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (86 , 49 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'related_purchase_recs' , '10' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (87 , 49 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'related_purchase_cols' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (88 , 49 , 3 , 'RECENT_PERIOD_PRODUCTS_BOUGHT_MSG' , 'TEXTBOX' , NULL , NULL , 'related_purchase_days' , '90' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (89 , 49 , 4 , 'ORDER_STATUS_MSG' , 'LISTBOX' , NULL , NULL , 'related_purchase_status' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (90 , 49 , 5 , 'IMAGE_TYPE_MSG' , 'LISTBOX' , NULL , NULL , 'related_purchase_image' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (91 , 49 , 6 , 'DESCRIPTION_MSG' , 'LISTBOX' , NULL , NULL , 'related_purchase_desc' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (92 , 58 , 1 , 'SELECT_TYPE_MSG' , 'RADIOBUTTON' , NULL , NULL , 'currency_selection' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (93 , 61 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'users_bought_item_recs' , '10' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (94 , 61 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'users_bought_item_cols' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (95 , 61 , 3 , 'USERS_BOUGHT_DURING_MSG' , 'TEXTBOX' , NULL , NULL , 'users_bought_item_days' , '90' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (96 , 61 , 4 , 'ORDER_STATUS_MSG' , 'LISTBOX' , NULL , NULL , 'users_bought_status' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (97 , 61 , 5 , 'USER_TYPE_MSG' , 'LISTBOX' , NULL , NULL , 'users_bought_type' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (98 , 61 , 6 , 'SELECT_FIELDS_TO_SHOW_MSG' , 'CHECKBOXLIST' , NULL , NULL , 'users_bought_item_fields' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (99 , 62 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'wl_recs' , '20' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (100 , 62 , 2 , 'IMAGE_TYPE_MSG' , 'LISTBOX' , NULL , NULL , 'wl_image' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (101 , 64 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'articles_related_per_page' , '4' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (102 , 64 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'articles_related_columns' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (103 , 64 , 3 , 'IMAGE_TYPE_MSG' , 'LISTBOX' , NULL , NULL , 'articles_related_image' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (104 , 64 , 4 , 'DESCRIPTION_MSG' , 'LISTBOX' , NULL , NULL , 'articles_related_desc' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (105 , 64 , 5 , 'DATE_MSG' , 'LISTBOX' , NULL , NULL , 'articles_related_date' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (106 , 65 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'forums_related_per_page' , '4' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (107 , 65 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'forums_related_columns' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (108 , 65 , 3 , 'DESCRIPTION_MSG' , 'LISTBOX' , NULL , NULL , 'forums_related_desc' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (109 , 65 , 4 , 'AUTHOR_INFO_MSG' , 'CHECKBOX' , NULL , NULL , 'forums_related_user_info' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (110 , 67 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'articles_recs' , '10' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (111 , 67 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'articles_cols' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (112 , 69 , 1 , 'CATEGORIES_TYPE_VIEW_MSG' , 'LISTBOX' , NULL , NULL , 'articles_categories_type' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (113 , 69 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'articles_categories_cols' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (114 , 69 , 3 , 'NUMBER_OF_SHOWN_SUBS_MSG' , 'TEXTBOX' , NULL , NULL , 'articles_categories_subs' , '0' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (115 , 69 , 4 , 'IMAGE_TYPE_FOR_TOP_CAT_MSG' , 'LISTBOX' , NULL , NULL , 'articles_categories_image' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (116 , 70 , 1 , 'IMAGE_MSG' , 'LISTBOX' , NULL , NULL , 'articles_category_image' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (117 , 70 , 2 , 'DESCRIPTION_MSG' , 'LISTBOX' , NULL , NULL , 'articles_category_desc_type' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (118 , 72 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'articles_hot_recs' , '10' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (119 , 72 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'articles_hot_cols' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (120 , 73 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'articles_related_recs' , '4' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (121 , 73 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'articles_related_cols' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (122 , 73 , 3 , 'IMAGE_TYPE_MSG' , 'LISTBOX' , NULL , NULL , 'articles_related_image' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (123 , 73 , 4 , 'DESCRIPTION_MSG' , 'LISTBOX' , NULL , NULL , 'articles_related_desc' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (124 , 73 , 5 , 'DATE_MSG' , 'LISTBOX' , NULL , NULL , 'articles_related_date' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (125 , 74 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'articles_related_forums_recs' , '4' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (126 , 74 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'articles_related_forums_cols' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (127 , 74 , 3 , 'DESCRIPTION_MSG' , 'LISTBOX' , NULL , NULL , 'articles_related_forums_desc' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (128 , 74 , 4 , 'AUTHOR_INFO_MSG' , 'CHECKBOX' , NULL , NULL , 'articles_related_forums_author' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (129 , 75 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'articles_related_products_recs' , '5' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (130 , 75 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'articles_related_products_cols' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (131 , 76 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'articles_products_cats_recs' , '5' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (132 , 76 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'articles_products_cats_cols' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (133 , 77 , 1 , 'GROUP_BY_MSG' , 'LISTBOX' , NULL , NULL , 'articles_latest_group_by' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (134 , 77 , 2 , 'CATEGORIES_IDS_MSG' , 'TEXTBOX' , 133 , 196 , 'articles_latest_cats' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (135 , 77 , 3 , 'INCLUDE_ARTICLES_FROM_SUBCAT_MSG' , 'CHECKBOX' , NULL , NULL , 'articles_latest_subcats' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (136 , 77 , 4 , 'TOP_ARTICLES_MSG' , 'TEXTBOX' , NULL , NULL , 'articles_latest_recs' , '10' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (137 , 77 , 5 , 'ARTICLES_LINKS_MSG' , 'TEXTBOX' , NULL , NULL , 'articles_latest_subrecs' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (138 , 77 , 6 , 'TOP_ARTICLE_IMAGE_TYPE_MSG' , 'LISTBOX' , NULL , NULL , 'articles_latest_image' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (139 , 77 , 7 , 'TOP_ARTICLE_DESCRIPTION_MSG' , 'LISTBOX' , NULL , NULL , 'articles_latest_desc' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (140 , 77 , 8 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'articles_latest_cols' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (141 , 80 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'articles_top_viewed_recs' , '10' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (142 , 80 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'articles_top_viewed_cols' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (143 , 85 , 1 , 'IMAGE_MSG' , 'LISTBOX' , NULL , NULL , 'forum_description_image' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (144 , 85 , 2 , 'DESCRIPTION_MSG' , 'LISTBOX' , NULL , NULL , 'forum_description_type' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (145 , 88 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'forum_top_viewed_recs' , '10' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (146 , 88 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'forum_top_viewed_cols' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (147 , 89 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'forum_latest_recs' , '10' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (148 , 89 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'forum_latest_cols' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (149 , 90 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'forum_articles_related_recs' , '4' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (150 , 90 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'forum_articles_related_cols' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (151 , 90 , 3 , 'IMAGE_TYPE_MSG' , 'LISTBOX' , NULL , NULL , 'forum_articles_related_image' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (152 , 90 , 4 , 'DESCRIPTION_MSG' , 'LISTBOX' , NULL , NULL , 'forum_articles_related_desc' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (153 , 90 , 5 , 'DATE_MSG' , 'LISTBOX' , NULL , NULL , 'forum_articles_related_date' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (154 , 91 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'forum_item_related_recs' , '4' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (155 , 91 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'forum_item_related_cols' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (156 , 94 , 1 , 'CATEGORIES_TYPE_VIEW_MSG' , 'LISTBOX' , NULL , NULL , 'ads_categories_type' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (157 , 94 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'ads_categories_cols' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (158 , 94 , 3 , 'NUMBER_OF_SHOWN_SUBS_MSG' , 'TEXTBOX' , NULL , NULL , 'ads_categories_subs' , '0' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (159 , 94 , 4 , 'IMAGE_TYPE_FOR_TOP_CAT_MSG' , 'LISTBOX' , NULL , NULL , 'ads_categories_image' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (160 , 95 , 1 , 'IMAGE_MSG' , 'LISTBOX' , NULL , NULL , 'ads_cat_desc_image' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (161 , 95 , 2 , 'DESCRIPTION_MSG' , 'LISTBOX' , NULL , NULL , 'ads_cat_desc_type' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (162 , 96 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'ads_list_recs' , '10' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (163 , 96 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'ads_list_cols' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (164 , 97 , 1 , 'SHOW_PRODUCTS_SECTION_MSG' , 'RADIOBUTTON' , NULL , NULL , 'ads_details_tabs' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (165 , 100 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'ads_hot_recs' , '10' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (166 , 100 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'ads_hot_cols' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (167 , 101 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'ads_special_recs' , '10' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (168 , 101 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'ads_special_cols' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (169 , 102 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'ads_latest_recs' , '10' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (170 , 102 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'ads_latest_cols' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (171 , 103 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'ads_top_viewed_recs' , '10' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (172 , 103 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'ads_top_viewed_cols' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (173 , 104 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'ads_recent_recs' , '5' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (174 , 114 , 1 , 'NUMBER_DEFAULT_VALUES_MSG' , 'TEXTBOX' , NULL , NULL , 'filter_values_limit' , '10' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (176 , 115 , 2 , 'CUSTOMERS_MSG' , 'LISTBOX' , NULL , NULL , 'cb_user_type' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (177 , 115 , 3 , 'ADMINISTRATORS_MSG' , 'LISTBOX' , NULL , NULL , 'cb_admin_type' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (178 , 115 , 4 , 'PARAMETERS_MSG' , 'TEXTBOX' , NULL , NULL , 'cb_params' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (179 , 117 , 1 , 'NUMBER_OF_SHOWN_BANNERS_MSG' , 'TEXTBOX' , NULL , NULL , 'bg_limit' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (180 , 117 , 2 , 'PARAMETERS_MSG' , 'TEXTBOX' , NULL , NULL , 'bg_params' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (181 , 116 , 1 , 'NAVIGATION_VISIBLE_DEPTH_MSG' , 'LISTBOX' , NULL , NULL , 'visible_depth_level' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (182 , 23 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'basket_prod_recom_recs' , '10' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (183 , 23 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'basket_prod_recom_cols' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (184 , 17 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'ss_recs' , '10' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (185 , 18 , 10 , 'CATEGORY_DESC_MSG' , 'LISTBOX' , 12 , NULL , 'category_desc' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (186 , 34 , 15 , 'POPUP_BOX_MSG' , 'CHECKBOX' , NULL , NULL , 'popup_box' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 'SHOW_BOX_MOUSE_OVER_IMAGE_MSG' , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (187 , 34 , 16 , 'BOX_IMAGE_TYPE_MSG' , 'LISTBOX' , 186 , NULL , 'box_image_type' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (188 , 34 , 17 , 'BOX_DESC_TYPE_MSG' , 'LISTBOX' , 186 , NULL , 'box_desc_type' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (189 , 19 , 4 , 'ZOOM_WIDTH_MSG' , 'TEXTBOX' , 15 , 259 , 'zoom_width' , '200' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (190 , 19 , 5 , 'ZOOM_HEIGHT_MSG' , 'TEXTBOX' , 15 , 259 , 'zoom_height' , '200' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (191 , 18 , 11 , 'MULTI_ADD_MSG' , 'CHECKBOX' , NULL , NULL , 'multi_add' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 'MULTI_ADD_DESC' , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (192 , 72 , 3 , 'SLIDER_TYPE_MSG' , 'LISTBOX' , NULL , NULL , 'slider_type' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (193 , 72 , 4 , 'SLIDER_WIDTH_MSG' , 'TEXTBOX' , NULL , NULL , 'slider_width' , '100%' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (194 , 72 , 5 , 'SLIDER_HEIGHT_MSG' , 'TEXTBOX' , NULL , NULL , 'slider_height' , '300' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (195 , 156 , 1 , 'SLIDER_TYPE_MSG' , 'LISTBOX' , NULL , NULL , 'slider_type' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (196 , 156 , 2 , 'SLIDER_WIDTH_MSG' , 'TEXTBOX' , NULL , NULL , 'slider_width' , '100%' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (197 , 156 , 3 , 'SLIDER_HEIGHT_MSG' , 'TEXTBOX' , NULL , NULL , 'slider_height' , '300' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (198 , 156 , 4 , 'BLOCK_VIEW_TYPE_MSG' , 'RADIOBUTTON' , NULL , NULL , 'block_view_type' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (199 , 158 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'recs' , '5' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (200 , 158 , 2 , 'USERNAME_FIELD' , 'TEXTBOX' , NULL , NULL , 'username' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (201 , 158 , 3 , 'ACCESS_TOKEN_MSG' , 'TEXTBOX' , NULL , NULL , 'access_token' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (202 , 159 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'recs' , '5' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (203 , 159 , 2 , 'USERNAME_FIELD' , 'TEXTBOX' , NULL , NULL , 'username' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (204 , 129 , 1 , 'SHOW_ORDER_RESTORE_MSG' , 'CHECKBOX' , NULL , NULL , 'show_restore' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (205 , 130 , 1 , 'SHOW_ORDER_RESTORE_MSG' , 'CHECKBOX' , NULL , NULL , 'show_restore' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (206 , 71 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'articles_content_recs' , '10' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (207 , 159 , 3 , 'TWITTER_SHOW_ICON_MSG' , 'CHECKBOX' , NULL , NULL , 'tf_show_icon' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (208 , 159 , 4 , 'TWITTER_CONSUMER_KEY_MSG' , 'TEXTBOX' , NULL , NULL , 'consumer_key' , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (209 , 159 , 5 , 'TWITTER_CONSUMER_SECRET_MSG' , 'TEXTBOX' , NULL , NULL , 'consumer_secret' , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (210 , 159 , 6 , 'TWITTER_ACCESS_TOKEN_MSG' , 'TEXTBOX' , NULL , NULL , 'oauth_access_token' , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (211 , 159 , 7 , 'TWITTER_ACCESS_TOKEN_SECRET_MSG' , 'TEXTBOX' , NULL , NULL , 'oauth_access_token_secret' , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (212 , 172 , 1 , 'BLOCK_POSITION_MSG' , 'LISTBOX' , NULL , NULL , 'block_position' , NULL , 0 , 'property-bottom' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (213 , 172 , 2 , 'NAME_MSG' , 'LABEL' , NULL , NULL , 'column_name' , NULL , 0 , 'cell-name' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (214 , 172 , 3 , 'ADMIN_SHOW_MSG' , 'LABEL' , NULL , NULL , 'column_show' , 'ADMIN_SHOW_MSG' , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (215 , 172 , 4 , 'ADMIN_ORDER_MSG' , 'LABEL' , NULL , NULL , 'column_order' , 'ADMIN_ORDER_MSG' , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (216 , 172 , 5 , 'POSITION_MSG' , 'LABEL' , NULL , NULL , 'column_pos' , 'POSITION_MSG' , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (217 , 172 , 6 , 'MENU_HOME' , 'LABEL' , NULL , NULL , 'home_name' , NULL , 0 , 'cell-name' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (218 , 172 , 7 , 'ADMIN_SHOW_MSG' , 'CHECKBOX' , NULL , NULL , 'home_show' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (219 , 172 , 8 , 'ADMIN_ORDER_MSG' , 'TEXTBOX' , NULL , NULL , 'home_order' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (220 , 172 , 9 , 'POSITION_MSG' , 'LISTBOX' , NULL , NULL , 'home_pos' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (221 , 172 , 10 , 'LANGUAGE_TITLE' , 'LABEL' , NULL , NULL , 'language_name' , NULL , 0 , 'cell-name' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (222 , 172 , 11 , 'ADMIN_SHOW_MSG' , 'CHECKBOX' , NULL , NULL , 'language_show' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (223 , 172 , 12 , 'ADMIN_ORDER_MSG' , 'TEXTBOX' , NULL , NULL , 'language_order' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (224 , 172 , 13 , 'POSITION_MSG' , 'LISTBOX' , NULL , NULL , 'language_pos' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (225 , 172 , 14 , 'CURRENCY_TITLE' , 'LABEL' , NULL , NULL , 'currency_name' , NULL , 0 , 'cell-name' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (226 , 172 , 15 , 'ADMIN_SHOW_MSG' , 'CHECKBOX' , NULL , NULL , 'currency_show' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (227 , 172 , 16 , 'ADMIN_ORDER_MSG' , 'TEXTBOX' , NULL , NULL , 'currency_order' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (228 , 172 , 17 , 'POSITION_MSG' , 'LISTBOX' , NULL , NULL , 'currency_pos' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (229 , 172 , 18 , 'MY_ACCOUNT_MSG' , 'LABEL' , NULL , NULL , 'account_name' , NULL , 0 , 'cell-name' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (230 , 172 , 19 , 'ADMIN_SHOW_MSG' , 'CHECKBOX' , NULL , NULL , 'account_show' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (231 , 172 , 20 , 'ADMIN_ORDER_MSG' , 'TEXTBOX' , NULL , NULL , 'account_order' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (232 , 172 , 21 , 'POSITION_MSG' , 'LISTBOX' , NULL , NULL , 'account_pos' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (233 , 172 , 22 , 'MY_WISHLIST_MSG' , 'LABEL' , NULL , NULL , 'wishlist_name' , NULL , 0 , 'cell-name' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (234 , 172 , 23 , 'ADMIN_SHOW_MSG' , 'CHECKBOX' , NULL , NULL , 'wishlist_show' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (235 , 172 , 24 , 'ADMIN_ORDER_MSG' , 'TEXTBOX' , NULL , NULL , 'wishlist_order' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (236 , 172 , 25 , 'POSITION_MSG' , 'LISTBOX' , NULL , NULL , 'wishlist_pos' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (237 , 172 , 26 , 'COMPARE_MSG' , 'LABEL' , NULL , NULL , 'compare_name' , NULL , 0 , 'cell-name' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (238 , 172 , 27 , 'ADMIN_SHOW_MSG' , 'CHECKBOX' , NULL , NULL , 'compare_show' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (239 , 172 , 28 , 'ADMIN_ORDER_MSG' , 'TEXTBOX' , NULL , NULL , 'compare_order' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (240 , 172 , 29 , 'POSITION_MSG' , 'LISTBOX' , NULL , NULL , 'compare_pos' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (241 , 172 , 30 , '{PRODUCTS_TITLE} ({CATEGORIES_TITLE})' , 'LABEL' , NULL , NULL , 'products_name' , NULL , 0 , 'cell-name' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (242 , 172 , 31 , 'ADMIN_SHOW_MSG' , 'CHECKBOX' , NULL , NULL , 'products_show' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (243 , 172 , 32 , 'ADMIN_ORDER_MSG' , 'TEXTBOX' , NULL , NULL , 'products_order' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (244 , 172 , 33 , 'POSITION_MSG' , 'LISTBOX' , NULL , NULL , 'products_pos' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (245 , 172 , 34 , '{PRODUCTS_TITLE} ({SEARCH_TITLE})' , 'LABEL' , NULL , NULL , 'products_search_name' , NULL , 0 , 'cell-name' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (246 , 172 , 35 , 'ADMIN_SHOW_MSG' , 'CHECKBOX' , NULL , NULL , 'products_search_show' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (247 , 172 , 36 , 'ADMIN_ORDER_MSG' , 'TEXTBOX' , NULL , NULL , 'products_search_order' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (248 , 172 , 37 , 'POSITION_MSG' , 'LISTBOX' , NULL , NULL , 'products_search_pos' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (249 , 172 , 38 , 'FULL_SITE_SEARCH_MSG' , 'LABEL' , NULL , NULL , 'site_search_name' , NULL , 0 , 'cell-name' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (250 , 172 , 39 , 'ADMIN_SHOW_MSG' , 'CHECKBOX' , NULL , NULL , 'site_search_show' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (251 , 172 , 40 , 'ADMIN_ORDER_MSG' , 'TEXTBOX' , NULL , NULL , 'site_search_order' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (252 , 172 , 41 , 'POSITION_MSG' , 'LISTBOX' , NULL , NULL , 'site_search_pos' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (253 , 172 , 42 , 'CART_TITLE' , 'LABEL' , NULL , NULL , 'cart_name' , NULL , 0 , 'cell-name' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (254 , 172 , 43 , 'ADMIN_SHOW_MSG' , 'CHECKBOX' , NULL , NULL , 'cart_show' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (255 , 172 , 44 , 'ADMIN_ORDER_MSG' , 'TEXTBOX' , NULL , NULL , 'cart_order' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (256 , 172 , 45 , 'POSITION_MSG' , 'LISTBOX' , NULL , NULL , 'cart_pos' , NULL , 0 , 'cell-short' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (257 , 173 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'recs' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (258 , 173 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'cols' , '1' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (259 , 173 , 3 , 'RANDOM_IMAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'random_image' , '' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (260 , 173 , 4 , 'ADMIN_SHOW_MSG' , 'CHECKBOXLIST' , NULL , NULL , 'show' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , '<br/>' , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (261 , 156 , 46 , 'TRANSITION_DELAY_MSG' , 'TEXTBOX' , NULL , NULL , 'transition_delay' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , '<br/>{TRANSITION_DELAY_DESC}' , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (262 , 156 , 47 , 'TRANSITION_DURATION_MSG' , 'TEXTBOX' , NULL , NULL , 'transition_duration' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , '<br/>{TRANSITION_DURATION_DESC}' , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (263 , 174 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'recs' , '50' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (264 , 174 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'cols' , '2' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (265 , 174 , 3 , 'IMAGE_TINY_MSG' , 'CHECKBOX' , NULL , NULL , 'image_tiny' , '0' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (266 , 174 , 4 , 'IMAGE_SMALL_MSG' , 'CHECKBOX' , NULL , NULL , 'image_small' , '0' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (267 , 174 , 5 , 'IMAGE_LARGE_MSG' , 'CHECKBOX' , NULL , NULL , 'image_large' , '0' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (268 , 174 , 6 , 'IMAGE_SUPER_MSG' , 'CHECKBOX' , NULL , NULL , 'image_super' , '0' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (269 , 174 , 7 , 'SHORT_DESCRIPTION_MSG' , 'CHECKBOX' , NULL , NULL , 'short_description' , '0' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (270 , 174 , 8 , 'FULL_DESCRIPTION_MSG' , 'CHECKBOX' , NULL , NULL , 'full_description' , '0' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (271 , 176 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'recs' , '50' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (272 , 176 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'cols' , '2' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (273 , 176 , 3 , 'IMAGE_TINY_MSG' , 'CHECKBOX' , NULL , NULL , 'image_tiny' , '0' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (274 , 176 , 4 , 'IMAGE_SMALL_MSG' , 'CHECKBOX' , NULL , NULL , 'image_small' , '0' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (275 , 176 , 5 , 'IMAGE_LARGE_MSG' , 'CHECKBOX' , NULL , NULL , 'image_large' , '0' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (276 , 176 , 6 , 'IMAGE_SUPER_MSG' , 'CHECKBOX' , NULL , NULL , 'image_super' , '0' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (277 , 176 , 7 , 'SHORT_DESCRIPTION_MSG' , 'CHECKBOX' , NULL , NULL , 'short_description' , '0' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (278 , 176 , 8 , 'FULL_DESCRIPTION_MSG' , 'CHECKBOX' , NULL , NULL , 'full_description' , '0' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (279 , 179 , 1 , 'RECORDS_PER_PAGE_MSG' , 'TEXTBOX' , NULL , NULL , 'recs' , '50' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (280 , 179 , 2 , 'NUMBER_OF_COLUMNS_MSG' , 'TEXTBOX' , NULL , NULL , 'cols' , '2' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (281 , 179 , 3 , 'IMAGE_TINY_MSG' , 'CHECKBOX' , NULL , NULL , 'image_tiny' , '0' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (282 , 179 , 4 , 'IMAGE_SMALL_MSG' , 'CHECKBOX' , NULL , NULL , 'image_small' , '0' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (283 , 179 , 5 , 'IMAGE_LARGE_MSG' , 'CHECKBOX' , NULL , NULL , 'image_large' , '0' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (284 , 179 , 6 , 'IMAGE_SUPER_MSG' , 'CHECKBOX' , NULL , NULL , 'image_super' , '0' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (285 , 179 , 7 , 'SHORT_DESCRIPTION_MSG' , 'CHECKBOX' , NULL , NULL , 'short_description' , '0' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_blocks_properties (property_id,block_id,property_order,property_name,control_type,parent_property_id,parent_value_id,variable_name,default_value,required,property_class,property_style,control_style,start_html,middle_html,before_control_html,after_control_html,end_html,control_code,onchange_code,onclick_code) VALUES (286 , 179 , 8 , 'FULL_DESCRIPTION_MSG' , 'CHECKBOX' , NULL , NULL , 'full_description' , '0' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;


INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (1 , 7 , 27 , 111 , NULL , 'articles_cols' , '2' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (2 , 7 , 27 , 110 , NULL , 'articles_recs' , '10' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (3 , 8 , 31 , 113 , NULL , 'articles_categories_cols' , '2' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (4 , 8 , 31 , 114 , NULL , 'articles_categories_subs' , '0' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (5 , 8 , 31 , 112 , 169 , 'articles_categories_type' , '3' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (6 , 8 , 32 , 111 , NULL , 'articles_cols' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (7 , 8 , 32 , 110 , NULL , 'articles_recs' , '20' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (8 , 9 , 36 , 111 , NULL , 'articles_cols' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (9 , 9 , 36 , 110 , NULL , 'articles_recs' , '10' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (10 , 10 , 40 , 111 , NULL , 'articles_cols' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (11 , 10 , 40 , 110 , NULL , 'articles_recs' , '20' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (12 , 11 , 43 , 111 , NULL , 'articles_cols' , '3' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (13 , 11 , 43 , 110 , NULL , 'articles_recs' , '10' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (14 , 12 , 47 , 111 , NULL , 'articles_cols' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (15 , 12 , 47 , 110 , NULL , 'articles_recs' , '10' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (16 , 14 , 54 , 157 , NULL , 'ads_categories_cols' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (17 , 14 , 54 , 158 , NULL , 'ads_categories_subs' , '0' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (18 , 14 , 54 , 156 , 222 , 'ads_categories_type' , '3' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (19 , 14 , 56 , 164 , 234 , 'ads_details_tabs' , '0' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (20 , 14 , 57 , 173 , NULL , 'ads_recent_recs' , '5' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (21 , 15 , 62 , 157 , NULL , 'ads_categories_cols' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (22 , 15 , 62 , 158 , NULL , 'ads_categories_subs' , '0' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (23 , 15 , 62 , 156 , 222 , 'ads_categories_type' , '3' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (24 , 15 , 64 , 163 , NULL , 'ads_list_cols' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (25 , 15 , 64 , 162 , NULL , 'ads_list_recs' , '10' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (108 , 48 , 203 , 35 , NULL , 'merchant_info_country_flag' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (109 , 48 , 203 , NULL , NULL , 'merchant_info_desc' , '2' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (110 , 48 , 203 , 33 , NULL , 'merchant_info_image' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (111 , 48 , 203 , 37 , NULL , 'merchant_info_member' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (112 , 48 , 203 , 34 , NULL , 'merchant_info_nick' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (113 , 48 , 203 , 36 , NULL , 'merchant_info_online' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (114 , 48 , 203 , 38 , NULL , 'merchant_info_prod_link' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (115 , 48 , 204 , 32 , 61 , 'merchants_selection' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (116 , 48 , 205 , 6 , NULL , 'products_columns' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (117 , 48 , 205 , 13 , 9 , 'products_default_view' , 'list' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (118 , 48 , 205 , 12 , NULL , 'products_group_by_cats' , '0' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (119 , 48 , 205 , 8 , NULL , 'products_nav_first_last' , '0' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (120 , 48 , 205 , 10 , NULL , 'products_nav_pages' , '5' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (121 , 48 , 205 , 9 , NULL , 'products_nav_prev_next' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (122 , 48 , 205 , 7 , 5 , 'products_nav_type' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (123 , 48 , 205 , 5 , NULL , 'products_per_page' , '10' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (124 , 48 , 205 , 11 , NULL , 'products_sortings' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (125 , 52 , 221 , 100 , 151 , 'wl_image' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (126 , 52 , 221 , 99 , NULL , 'wl_recs' , '10' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (275 , 53 , 591 , 218 , NULL , 'home_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (276 , 53 , 591 , 219 , NULL , 'home_order' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (277 , 53 , 591 , 222 , NULL , 'language_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (278 , 53 , 591 , 223 , NULL , 'language_order' , '4' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (279 , 53 , 591 , 226 , NULL , 'currency_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (280 , 53 , 591 , 227 , NULL , 'currency_order' , '5' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (281 , 53 , 591 , 230 , NULL , 'account_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (282 , 53 , 591 , 231 , NULL , 'account_order' , '2' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (283 , 53 , 591 , 234 , NULL , 'wishlist_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (284 , 53 , 591 , 235 , NULL , 'wishlist_order' , '3' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (285 , 53 , 591 , 239 , NULL , 'compare_order' , '6' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (286 , 53 , 591 , 242 , NULL , 'products_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (287 , 53 , 591 , 243 , NULL , 'products_order' , '7' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (288 , 53 , 591 , 244 , 291 , 'products_pos' , 'right' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (289 , 53 , 591 , 246 , NULL , 'products_search_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (290 , 53 , 591 , 247 , NULL , 'products_search_order' , '8' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (291 , 53 , 591 , 248 , 293 , 'products_search_pos' , 'right' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (292 , 53 , 591 , 254 , NULL , 'cart_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (293 , 53 , 591 , 255 , NULL , 'cart_order' , '9' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (294 , 53 , 591 , 256 , 297 , 'cart_pos' , 'right' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (295 , 53 , 595 , 1 , 1 , 'language_selection' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (296 , 53 , 599 , 40 , NULL , 'prod_offers_recs' , '10' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (297 , 53 , 599 , 41 , NULL , 'prod_offers_cols' , '2' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (298 , 53 , 600 , 18 , NULL , 'categories_columns' , '2' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (299 , 53 , 600 , 19 , NULL , 'categories_subs' , '6' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (300 , 53 , 600 , 17 , 20 , 'categories_type' , '2' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (301 , 53 , 601 , 119 , NULL , 'articles_hot_cols' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (302 , 53 , 601 , 118 , NULL , 'articles_hot_recs' , '2' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (303 , 39 , 603 , 17 , 275 , 'categories_type' , '6' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (304 , 39 , 603 , 20 , 23 , 'categories_image' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (305 , 39 , 604 , 27 , 48 , 'manufacturers_desc' , '0' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (306 , 39 , 604 , 29 , 53 , 'manufacturers_direction' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (307 , 39 , 604 , 26 , 45 , 'manufacturers_image' , '0' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (308 , 39 , 604 , 28 , 51 , 'manufacturers_order' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (309 , 39 , 604 , 25 , 43 , 'manufacturers_selection' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (310 , 39 , 605 , 64 , 77 , 'prod_latest_desc' , '0' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (311 , 39 , 605 , 63 , 74 , 'prod_latest_image' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (312 , 39 , 605 , 65 , 83 , 'prod_latest_order' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (313 , 39 , 605 , 62 , NULL , 'products_latest_cols' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (314 , 39 , 605 , 61 , NULL , 'products_latest_recs' , '2' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (315 , 39 , 606 , 218 , NULL , 'home_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (316 , 39 , 606 , 219 , NULL , 'home_order' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (317 , 39 , 606 , 222 , NULL , 'language_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (318 , 39 , 606 , 223 , NULL , 'language_order' , '4' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (319 , 39 , 606 , 226 , NULL , 'currency_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (320 , 39 , 606 , 227 , NULL , 'currency_order' , '5' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (321 , 39 , 606 , 230 , NULL , 'account_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (322 , 39 , 606 , 231 , NULL , 'account_order' , '2' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (323 , 39 , 606 , 234 , NULL , 'wishlist_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (324 , 39 , 606 , 235 , NULL , 'wishlist_order' , '3' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (325 , 39 , 606 , 239 , NULL , 'compare_order' , '6' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (326 , 39 , 606 , 242 , NULL , 'products_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (327 , 39 , 606 , 243 , NULL , 'products_order' , '7' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (328 , 39 , 606 , 244 , 291 , 'products_pos' , 'right' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (329 , 39 , 606 , 246 , NULL , 'products_search_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (330 , 39 , 606 , 247 , NULL , 'products_search_order' , '8' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (331 , 39 , 606 , 248 , 293 , 'products_search_pos' , 'right' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (332 , 39 , 606 , 254 , NULL , 'cart_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (333 , 39 , 606 , 255 , NULL , 'cart_order' , '9' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (334 , 39 , 606 , 256 , 297 , 'cart_pos' , 'right' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (335 , 39 , 608 , 174 , NULL , 'filter_values_limit' , '10' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (336 , 39 , 609 , 6 , NULL , 'products_columns' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (337 , 39 , 609 , 13 , 9 , 'products_default_view' , 'list' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (338 , 39 , 609 , 12 , NULL , 'products_group_by_cats' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (339 , 39 , 609 , 8 , NULL , 'products_nav_first_last' , '0' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (340 , 39 , 609 , 10 , NULL , 'products_nav_pages' , '5' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (341 , 39 , 609 , 9 , NULL , 'products_nav_prev_next' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (342 , 39 , 609 , 7 , 6 , 'products_nav_type' , '2' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (343 , 39 , 609 , 5 , NULL , 'products_per_page' , '10' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (344 , 39 , 609 , 11 , NULL , 'products_sortings' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (345 , 39 , 612 , 67 , NULL , 'products_recent_cols' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (346 , 39 , 612 , 66 , NULL , 'products_recent_records' , '2' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (347 , 39 , 612 , 69 , 89 , 'recent_desc' , '0' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (348 , 39 , 612 , 68 , 86 , 'recent_image' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (349 , 38 , 615 , 18 , NULL , 'categories_columns' , NULL ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (350 , 38 , 615 , 20 , 23 , 'categories_image' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (351 , 38 , 615 , 19 , NULL , 'categories_subs' , NULL ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (352 , 38 , 615 , 17 , 21 , 'categories_type' , '3' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (353 , 38 , 616 , 64 , 77 , 'prod_latest_desc' , '0' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (354 , 38 , 616 , 63 , 74 , 'prod_latest_image' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (355 , 38 , 616 , 65 , 83 , 'prod_latest_order' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (356 , 38 , 616 , 62 , NULL , 'products_latest_cols' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (357 , 38 , 616 , 61 , NULL , 'products_latest_recs' , '2' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (358 , 38 , 617 , 218 , NULL , 'home_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (359 , 38 , 617 , 219 , NULL , 'home_order' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (360 , 38 , 617 , 222 , NULL , 'language_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (361 , 38 , 617 , 223 , NULL , 'language_order' , '4' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (362 , 38 , 617 , 226 , NULL , 'currency_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (363 , 38 , 617 , 227 , NULL , 'currency_order' , '5' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (364 , 38 , 617 , 230 , NULL , 'account_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (365 , 38 , 617 , 231 , NULL , 'account_order' , '2' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (366 , 38 , 617 , 234 , NULL , 'wishlist_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (367 , 38 , 617 , 235 , NULL , 'wishlist_order' , '3' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (368 , 38 , 617 , 239 , NULL , 'compare_order' , '6' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (369 , 38 , 617 , 242 , NULL , 'products_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (370 , 38 , 617 , 243 , NULL , 'products_order' , '7' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (371 , 38 , 617 , 244 , 291 , 'products_pos' , 'right' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (372 , 38 , 617 , 246 , NULL , 'products_search_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (373 , 38 , 617 , 247 , NULL , 'products_search_order' , '8' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (374 , 38 , 617 , 248 , 293 , 'products_search_pos' , 'right' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (375 , 38 , 617 , 254 , NULL , 'cart_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (376 , 38 , 617 , 255 , NULL , 'cart_order' , '9' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (377 , 38 , 617 , 256 , 297 , 'cart_pos' , 'right' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (378 , 38 , 619 , 16 , 17 , 'show_manufacturer_image' , '2' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (379 , 38 , 619 , 14 , 12 , 'use_tabs' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (380 , 38 , 620 , 85 , NULL , 'related_columns' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (381 , 38 , 620 , 84 , NULL , 'related_per_page' , '2' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (382 , 38 , 623 , 87 , NULL , 'related_purchase_cols' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (383 , 38 , 623 , 88 , NULL , 'related_purchase_days' , '30' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (384 , 38 , 623 , 91 , 129 , 'related_purchase_desc' , '0' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (385 , 38 , 623 , 90 , 126 , 'related_purchase_image' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (386 , 38 , 623 , 86 , NULL , 'related_purchase_recs' , '2' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (387 , 38 , 623 , 89 , 123 , 'related_purchase_status' , 'ANY' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (388 , 41 , 626 , 18 , NULL , 'categories_columns' , NULL ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (389 , 41 , 626 , 20 , 23 , 'categories_image' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (390 , 41 , 626 , 19 , NULL , 'categories_subs' , NULL ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (391 , 41 , 626 , 17 , 21 , 'categories_type' , '3' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (392 , 41 , 627 , 27 , 48 , 'manufacturers_desc' , '0' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (393 , 41 , 627 , 29 , 53 , 'manufacturers_direction' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (394 , 41 , 627 , 26 , 45 , 'manufacturers_image' , '0' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (395 , 41 , 627 , 28 , 51 , 'manufacturers_order' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (396 , 41 , 627 , 25 , 43 , 'manufacturers_selection' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (397 , 41 , 628 , 64 , 77 , 'prod_latest_desc' , '0' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (398 , 41 , 628 , 63 , 74 , 'prod_latest_image' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (399 , 41 , 628 , 65 , 82 , 'prod_latest_order' , '0' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (400 , 41 , 628 , 62 , NULL , 'products_latest_cols' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (401 , 41 , 628 , 61 , NULL , 'products_latest_recs' , '2' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (402 , 41 , 629 , 218 , NULL , 'home_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (403 , 41 , 629 , 219 , NULL , 'home_order' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (404 , 41 , 629 , 222 , NULL , 'language_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (405 , 41 , 629 , 223 , NULL , 'language_order' , '4' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (406 , 41 , 629 , 226 , NULL , 'currency_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (407 , 41 , 629 , 227 , NULL , 'currency_order' , '5' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (408 , 41 , 629 , 230 , NULL , 'account_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (409 , 41 , 629 , 231 , NULL , 'account_order' , '2' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (410 , 41 , 629 , 234 , NULL , 'wishlist_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (411 , 41 , 629 , 235 , NULL , 'wishlist_order' , '3' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (412 , 41 , 629 , 239 , NULL , 'compare_order' , '6' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (413 , 41 , 629 , 242 , NULL , 'products_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (414 , 41 , 629 , 243 , NULL , 'products_order' , '7' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (415 , 41 , 629 , 244 , 291 , 'products_pos' , 'right' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (416 , 41 , 629 , 246 , NULL , 'products_search_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (417 , 41 , 629 , 247 , NULL , 'products_search_order' , '8' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (418 , 41 , 629 , 248 , 293 , 'products_search_pos' , 'right' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (419 , 41 , 629 , 254 , NULL , 'cart_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (420 , 41 , 629 , 255 , NULL , 'cart_order' , '9' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (421 , 41 , 629 , 256 , 297 , 'cart_pos' , 'right' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (422 , 41 , 631 , 6 , NULL , 'products_columns' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (423 , 41 , 631 , 13 , 10 , 'products_default_view' , 'table' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (424 , 41 , 631 , 12 , NULL , 'products_group_by_cats' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (425 , 41 , 631 , 8 , NULL , 'products_nav_first_last' , '0' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (426 , 41 , 631 , 10 , NULL , 'products_nav_pages' , '5' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (427 , 41 , 631 , 9 , NULL , 'products_nav_prev_next' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (428 , 41 , 631 , 7 , 5 , 'products_nav_type' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (429 , 41 , 631 , 5 , NULL , 'products_per_page' , '10' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (430 , 41 , 631 , 11 , NULL , 'products_sortings' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (431 , 41 , 634 , 67 , NULL , 'products_recent_cols' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (432 , 41 , 634 , 66 , NULL , 'products_recent_records' , '2' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (433 , 41 , 634 , 69 , 89 , 'recent_desc' , '0' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (434 , 41 , 634 , 68 , 86 , 'recent_image' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (463 , 17 , 644 , 18 , NULL , 'categories_columns' , NULL ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (464 , 17 , 644 , 20 , 23 , 'categories_image' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (465 , 17 , 644 , 19 , NULL , 'categories_subs' , NULL ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (466 , 17 , 644 , 17 , 21 , 'categories_type' , '3' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (467 , 17 , 645 , 92 , 134 , 'currency_selection' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (468 , 17 , 646 , 218 , NULL , 'home_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (469 , 17 , 646 , 219 , NULL , 'home_order' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (470 , 17 , 646 , 222 , NULL , 'language_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (471 , 17 , 646 , 223 , NULL , 'language_order' , '4' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (472 , 17 , 646 , 226 , NULL , 'currency_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (473 , 17 , 646 , 227 , NULL , 'currency_order' , '5' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (474 , 17 , 646 , 230 , NULL , 'account_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (475 , 17 , 646 , 231 , NULL , 'account_order' , '2' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (476 , 17 , 646 , 234 , NULL , 'wishlist_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (477 , 17 , 646 , 235 , NULL , 'wishlist_order' , '3' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (478 , 17 , 646 , 239 , NULL , 'compare_order' , '6' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (479 , 17 , 646 , 242 , NULL , 'products_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (480 , 17 , 646 , 243 , NULL , 'products_order' , '7' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (481 , 17 , 646 , 244 , 291 , 'products_pos' , 'right' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (482 , 17 , 646 , 246 , NULL , 'products_search_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (483 , 17 , 646 , 247 , NULL , 'products_search_order' , '8' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (484 , 17 , 646 , 248 , 293 , 'products_search_pos' , 'right' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (485 , 17 , 646 , 254 , NULL , 'cart_show' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (486 , 17 , 646 , 255 , NULL , 'cart_order' , '9' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (487 , 17 , 646 , 256 , 297 , 'cart_pos' , 'right' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (488 , 17 , 647 , 23 , 34 , 'image_type' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (489 , 17 , 648 , 183 , NULL , 'basket_prod_recom_cols' , '1' ) ;
INSERT INTO va_cms_blocks_settings (bs_id,ps_id,pb_id,property_id,value_id,variable_name,variable_value) VALUES (490 , 17 , 648 , 182 , NULL , 'basket_prod_recom_recs' , '10' ) ;


INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (1 , 1 , 1 , 'SHOW_IMAGES_LANGUAGE_MSG' , NULL , '1' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (2 , 1 , 2 , 'SHOW_LISTBOX_LANGUAGE_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (3 , 2 , 1 , 'SHOW_DESIGNS_PER_ROW_MSG' , NULL , '1' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (4 , 2 , 2 , 'USE_LISTBOX_DESIGNS_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (5 , 7 , 1 , 'SIMPLE_CURRENT_PAGE_ONLY_MSG' , NULL , '1' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (6 , 7 , 2 , 'CURRENT_PAGE_SHOWN_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (7 , 7 , 3 , 'NAVIGATOR_SPLITS_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (8 , 7 , 4 , 'SHOWS_LINKS_PAGES_MSG' , NULL , '4' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (9 , 13 , 1 , 'DETAILED_LISTING_MSG' , NULL , 'list' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (10 , 13 , 2 , 'TABLE_VIEW_MSG' , NULL , 'table' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (11 , 13 , 3 , 'GRID_MSG' , NULL , 'grid' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (12 , 14 , 1 , 'ON_DIFFERENT_PAGES_MSG' , NULL , '1' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (13 , 14 , 2 , 'ON_ONE_PAGE_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (14 , 15 , 1 , 'SUPER_IMAGE_ON_CLICK_MSG' , NULL , '0' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (15 , 15 , 2 , 'SUPER_IMAGE_ON_MOVE_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (16 , 16 , 1 , 'DONT_SHOW_IMAGE_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (17 , 16 , 2 , 'IMAGE_SMALL_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (18 , 16 , 3 , 'IMAGE_LARGE_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (19 , 17 , 1 , 'ONELEVEL_LIST_MSG' , NULL , '1' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (20 , 17 , 2 , 'TWOLEVEL_LIST_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (21 , 17 , 3 , 'MULTILEVEL_LIST_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (22 , 17 , 4 , 'TREETYPE_STRUCTURE_MSG' , NULL , '4' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (23 , 20 , 1 , 'DEFAULT_IMAGE_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (24 , 20 , 2 , 'IMAGE_SMALL_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (25 , 20 , 3 , 'IMAGE_LARGE_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (26 , 21 , 1 , 'DONT_SHOW_IMAGE_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (27 , 21 , 2 , 'IMAGE_SMALL_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (28 , 21 , 3 , 'IMAGE_LARGE_MSG' , NULL , '3' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (29 , 22 , 1 , 'DONT_SHOW_DESC_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (30 , 22 , 2 , 'SHORT_DESCRIPTION_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (31 , 22 , 3 , 'FULL_DESCRIPTION_MSG' , NULL , '2' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (32 , 17 , 5 , 'CHAINED_MENU_TITLE' , NULL , '5' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (33 , 23 , 1 , 'DONT_SHOW_IMAGE_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (34 , 23 , 2 , 'IMAGE_TINY_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (35 , 23 , 3 , 'IMAGE_SMALL_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (36 , 23 , 4 , 'IMAGE_LARGE_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (43 , 25 , 1 , 'SHOW_MANUFACTURERS_MSG' , NULL , '1' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (44 , 25 , 2 , 'USE_LISTBOX_MANUFACTURERS_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (45 , 26 , 1 , 'DONT_SHOW_IMAGE_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (46 , 26 , 2 , 'IMAGE_SMALL_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (47 , 26 , 3 , 'IMAGE_LARGE_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (48 , 27 , 1 , 'DONT_SHOW_DESC_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (49 , 27 , 2 , 'SHORT_DESCRIPTION_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (50 , 27 , 3 , 'FULL_DESCRIPTION_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (51 , 28 , 1 , 'MANUFACTURER_NAME_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (52 , 28 , 2 , 'MANUFACTURER_ORDER_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (53 , 29 , 1 , 'ASC_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (54 , 29 , 2 , 'DESC_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (55 , 30 , 1 , 'DONT_SHOW_IMAGE_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (56 , 30 , 2 , 'IMAGE_SMALL_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (57 , 30 , 3 , 'IMAGE_LARGE_MSG' , NULL , '3' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (58 , 31 , 1 , 'DONT_SHOW_DESC_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (59 , 31 , 2 , 'SHORT_DESCRIPTION_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (60 , 31 , 3 , 'FULL_DESCRIPTION_MSG' , NULL , '2' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (61 , 32 , 1 , 'SHOW_MERCHANTS_PER_ROW_MSG' , NULL , '1' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (62 , 32 , 2 , 'USE_LISTBOX_MERCHANTS_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (63 , 39 , 1 , 'DONT_SHOW_DESC_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (64 , 39 , 2 , 'SHORT_DESCRIPTION_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (65 , 39 , 3 , 'FULL_DESCRIPTION_MSG' , NULL , '2' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (66 , 49 , 1 , 'NONE_MSG' , NULL , 'NONE' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (67 , 49 , 2 , 'LABEL_MSG' , NULL , 'LABEL' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (68 , 49 , 3 , 'LISTBOX_MSG' , NULL , 'LISTBOX' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (69 , 49 , 4 , 'TEXTBOX_MSG' , NULL , 'TEXTBOX' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (70 , 50 , 1 , 'NONE_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (71 , 50 , 2 , '{VERTICAL_SLIDER_MSG} ({UP_MSG})' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (72 , 50 , 4 , '{HORIZONTAL_SLIDER_MSG} ({LEFT_MSG})' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (73 , 63 , 1 , 'DONT_SHOW_IMAGE_MSG' , NULL , '0' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (74 , 63 , 2 , 'IMAGE_TINY_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (75 , 63 , 3 , 'IMAGE_SMALL_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (76 , 63 , 4 , 'IMAGE_LARGE_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (77 , 64 , 1 , 'DONT_SHOW_DESC_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (78 , 64 , 2 , 'SHORT_DESCRIPTION_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (79 , 64 , 3 , 'FULL_DESCRIPTION_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (80 , 64 , 4 , 'HIGHLIGHTS_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (81 , 64 , 5 , 'SPECIAL_OFFER_MSG' , NULL , '4' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (82 , 65 , 1 , 'PROD_ISSUE_DATE_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (83 , 65 , 2 , 'DATE_ADDED_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (84 , 65 , 3 , 'DATE_MODIFIED_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (85 , 68 , 1 , 'DONT_SHOW_IMAGE_MSG' , NULL , '0' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (86 , 68 , 2 , 'IMAGE_TINY_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (87 , 68 , 3 , 'IMAGE_SMALL_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (88 , 68 , 4 , 'IMAGE_LARGE_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (89 , 69 , 1 , 'DONT_SHOW_DESC_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (90 , 69 , 2 , 'SHORT_DESCRIPTION_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (91 , 69 , 3 , 'FULL_DESCRIPTION_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (92 , 69 , 4 , 'HIGHLIGHTS_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (93 , 69 , 5 , 'SPECIAL_OFFER_MSG' , NULL , '4' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (94 , 74 , 1 , 'ANY_STATUS_MSG' , NULL , 'ANY' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (95 , 74 , 2 , 'ANY_PAID_STATUS_MSG' , NULL , 'PAID' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (96 , 75 , 1 , 'DONT_SHOW_IMAGE_MSG' , NULL , '0' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (97 , 75 , 2 , 'IMAGE_TINY_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (98 , 75 , 3 , 'IMAGE_SMALL_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (99 , 75 , 4 , 'IMAGE_LARGE_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (100 , 76 , 1 , 'DONT_SHOW_DESC_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (101 , 76 , 2 , 'SHORT_DESCRIPTION_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (102 , 76 , 3 , 'FULL_DESCRIPTION_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (103 , 76 , 4 , 'HIGHLIGHTS_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (104 , 76 , 5 , 'SPECIAL_OFFER_MSG' , NULL , '4' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (105 , 79 , 1 , 'DONT_SHOW_IMAGE_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (106 , 79 , 2 , 'IMAGE_TINY_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (107 , 79 , 3 , 'IMAGE_SMALL_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (108 , 79 , 4 , 'IMAGE_LARGE_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (109 , 80 , 1 , 'DONT_SHOW_DESC_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (110 , 80 , 2 , 'SHORT_DESCRIPTION_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (111 , 80 , 3 , 'FULL_DESCRIPTION_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (112 , 80 , 4 , 'HIGHLIGHTS_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (113 , 80 , 5 , 'SPECIAL_OFFER_MSG' , NULL , '4' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (114 , 82 , 1 , 'DONT_SHOW_IMAGE_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (115 , 82 , 2 , 'IMAGE_TINY_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (116 , 82 , 3 , 'IMAGE_SMALL_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (117 , 82 , 4 , 'IMAGE_LARGE_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (118 , 83 , 1 , 'DONT_SHOW_DESC_MSG' , NULL , '0' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (119 , 83 , 2 , 'SHORT_DESCRIPTION_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (120 , 83 , 3 , 'FULL_DESCRIPTION_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (121 , 83 , 4 , 'HIGHLIGHTS_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (122 , 83 , 5 , 'SPECIAL_OFFER_MSG' , NULL , '4' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (123 , 89 , 1 , 'ANY_STATUS_MSG' , NULL , 'ANY' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (124 , 89 , 2 , 'ANY_PAID_STATUS_MSG' , NULL , 'PAID' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (125 , 90 , 1 , 'DONT_SHOW_IMAGE_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (126 , 90 , 2 , 'IMAGE_TINY_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (127 , 90 , 3 , 'IMAGE_SMALL_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (128 , 90 , 4 , 'IMAGE_LARGE_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (129 , 91 , 1 , 'DONT_SHOW_DESC_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (130 , 91 , 2 , 'SHORT_DESCRIPTION_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (131 , 91 , 3 , 'FULL_DESCRIPTION_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (132 , 91 , 4 , 'HIGHLIGHTS_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (133 , 91 , 5 , 'SPECIAL_OFFER_MSG' , NULL , '4' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (134 , 92 , 1 , 'SHOW_IMAGES_CURRENCY_MSG' , NULL , '1' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (135 , 92 , 2 , 'SHOW_LISTBOX_CURRENCY_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (136 , 96 , 1 , 'ANY_STATUS_MSG' , NULL , 'ANY' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (137 , 96 , 2 , 'ANY_PAID_STATUS_MSG' , NULL , 'PAID' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (138 , 97 , 1 , 'ALL_USERS_INCLUDING_UNREGISTERED_MSG' , NULL , 'ALL' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (139 , 97 , 2 , 'UNREGISTERED_USER_ONLY_MSG' , NULL , 'NON' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (140 , 97 , 3 , 'ANY_REGISTERED_USERS_MSG' , NULL , 'REG' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (141 , 98 , 1 , 'FIRST_NAME_FIELD' , 'users_bought_item_fn' , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (142 , 98 , 2 , 'LAST_NAME_FIELD' , 'users_bought_item_ln' , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (143 , 98 , 3 , 'NAME_MSG' , 'users_bought_item_name' , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (144 , 98 , 4 , 'NICKNAME_FIELD' , 'users_bought_item_nickname' , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (145 , 98 , 5 , 'COMPANY_NAME_FIELD' , 'users_bought_item_cn' , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (146 , 98 , 6 , 'EMAIL_FIELD' , 'users_bought_item_email' , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (147 , 98 , 7 , 'COUNTRY_FIELD' , 'users_bought_item_country' , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (148 , 98 , 8 , 'STATE_FIELD' , 'users_bought_item_state' , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (149 , 98 , 9 , 'DATE_OF_PURCHASE_MSG' , 'users_bought_item_od' , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (150 , 100 , 1 , 'DONT_SHOW_IMAGE_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (151 , 100 , 2 , 'IMAGE_TINY_MSG' , NULL , '1' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (152 , 100 , 3 , 'IMAGE_SMALL_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (153 , 100 , 4 , 'IMAGE_LARGE_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (154 , 103 , 1 , 'DONT_SHOW_IMAGE_MSG' , NULL , '0' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (155 , 103 , 2 , 'IMAGE_SMALL_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (156 , 103 , 3 , 'IMAGE_LARGE_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (157 , 104 , 1 , 'DONT_SHOW_DESC_MSG' , NULL , '0' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (158 , 104 , 2 , 'SHORT_DESCRIPTION_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (159 , 104 , 3 , 'FULL_DESCRIPTION_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (160 , 104 , 4 , 'HOT_DESCRIPTION_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (161 , 105 , 1 , 'DONT_SHOW_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (162 , 105 , 2 , 'DATE_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (163 , 105 , 3 , 'DATE_END_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (164 , 108 , 1 , 'DONT_SHOW_DESC_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (165 , 108 , 2 , 'FULL_DESCRIPTION_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (166 , 108 , 3 , 'LAST_POST_ADDED_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (167 , 112 , 1 , 'ONELEVEL_LIST_MSG' , NULL , '1' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (168 , 112 , 2 , 'TWOLEVEL_LIST_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (169 , 112 , 3 , 'MULTILEVEL_LIST_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (170 , 112 , 4 , 'TREETYPE_STRUCTURE_MSG' , NULL , '4' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (171 , 115 , 1 , 'DEFAULT_IMAGE_MSG' , NULL , '1' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (172 , 115 , 2 , 'IMAGE_SMALL_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (173 , 115 , 3 , 'IMAGE_LARGE_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (174 , 116 , 1 , 'DONT_SHOW_IMAGE_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (175 , 116 , 2 , 'IMAGE_SMALL_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (176 , 116 , 3 , 'IMAGE_LARGE_MSG' , NULL , '3' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (177 , 117 , 1 , 'DONT_SHOW_DESC_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (178 , 117 , 2 , 'SHORT_DESCRIPTION_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (179 , 117 , 3 , 'FULL_DESCRIPTION_MSG' , NULL , '2' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (180 , 122 , 1 , 'DONT_SHOW_IMAGE_MSG' , NULL , '0' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (181 , 122 , 2 , 'IMAGE_SMALL_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (182 , 122 , 3 , 'IMAGE_LARGE_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (183 , 123 , 1 , 'DONT_SHOW_DESC_MSG' , NULL , '0' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (184 , 123 , 2 , 'SHORT_DESCRIPTION_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (185 , 123 , 3 , 'FULL_DESCRIPTION_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (186 , 123 , 4 , 'HOT_DESCRIPTION_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (187 , 124 , 1 , 'DONT_SHOW_MSG' , NULL , '0' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (188 , 124 , 2 , 'DATE_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (189 , 124 , 3 , 'DATE_END_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (190 , 127 , 1 , 'DONT_SHOW_DESC_MSG' , NULL , '0' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (191 , 127 , 2 , 'FULL_DESCRIPTION_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (192 , 127 , 3 , 'LAST_POST_ADDED_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (193 , 133 , 1 , 'NO_GROUPING_MSG' , NULL , '0' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (194 , 133 , 2 , 'TOP_CATEGORIES_ONLY_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (195 , 133 , 3 , 'ALL_AVAILABLE_CATEGORIES_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (196 , 133 , 4 , 'SELECTED_CATEGORIES_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (197 , 138 , 1 , 'DONT_SHOW_IMAGE_MSG' , NULL , '0' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (198 , 138 , 2 , 'IMAGE_SMALL_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (199 , 138 , 3 , 'IMAGE_LARGE_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (200 , 139 , 1 , 'DONT_SHOW_DESC_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (201 , 139 , 2 , 'SHORT_DESCRIPTION_MSG' , NULL , '1' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (202 , 139 , 3 , 'FULL_DESCRIPTION_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (203 , 139 , 4 , 'HOT_DESCRIPTION_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (204 , 143 , 1 , 'DONT_SHOW_IMAGE_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (205 , 143 , 2 , 'IMAGE_SMALL_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (206 , 143 , 3 , 'IMAGE_LARGE_MSG' , NULL , '3' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (207 , 144 , 1 , 'DONT_SHOW_DESC_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (208 , 144 , 2 , 'FULL_DESCRIPTION_MSG' , NULL , '2' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (209 , 144 , 3 , 'LAST_POST_ADDED_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (210 , 151 , 1 , 'DONT_SHOW_IMAGE_MSG' , NULL , '0' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (211 , 151 , 2 , 'IMAGE_SMALL_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (212 , 151 , 3 , 'IMAGE_LARGE_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (213 , 152 , 1 , 'DONT_SHOW_DESC_MSG' , NULL , '0' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (214 , 152 , 2 , 'SHORT_DESCRIPTION_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (215 , 152 , 3 , 'FULL_DESCRIPTION_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (216 , 152 , 4 , 'HOT_DESCRIPTION_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (217 , 153 , 1 , 'DONT_SHOW_MSG' , NULL , '0' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (218 , 153 , 2 , 'DATE_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (219 , 153 , 3 , 'DATE_END_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (220 , 156 , 1 , 'ONELEVEL_LIST_MSG' , NULL , '1' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (221 , 156 , 2 , 'TWOLEVEL_LIST_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (222 , 156 , 3 , 'MULTILEVEL_LIST_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (223 , 156 , 4 , 'TREETYPE_STRUCTURE_MSG' , NULL , '4' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (224 , 159 , 1 , 'DEFAULT_IMAGE_MSG' , NULL , '1' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (225 , 159 , 2 , 'IMAGE_SMALL_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (226 , 159 , 3 , 'IMAGE_LARGE_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (227 , 160 , 1 , 'DONT_SHOW_IMAGE_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (228 , 160 , 2 , 'IMAGE_SMALL_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (229 , 160 , 3 , 'IMAGE_LARGE_MSG' , NULL , '3' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (230 , 161 , 1 , 'DONT_SHOW_DESC_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (231 , 161 , 2 , 'SHORT_DESCRIPTION_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (232 , 161 , 3 , 'FULL_DESCRIPTION_MSG' , NULL , '2' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (233 , 164 , 1 , 'ON_DIFFERENT_PAGES_MSG' , NULL , '1' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (234 , 164 , 2 , 'ON_ONE_PAGE_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (235 , 176 , 1 , 'ALL_USERS_INCLUDING_UNREGISTERED_MSG' , NULL , 'ALL' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (236 , 176 , 2 , 'NON_REGISTERED_USERS_MSG' , NULL , 'NON' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (237 , 176 , 3 , 'ANY_REGISTERED_USERS_MSG' , NULL , 'ANY' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (238 , 177 , 1 , '---' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (239 , 177 , 2 , 'ANY_REGISTERED_ADMIN_MSG' , NULL , 'ANY' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (240 , 181 , 1 , 'ALL_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (241 , 181 , 2 , '1' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (242 , 181 , 3 , '2' , NULL , '2' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (243 , 181 , 4 , '3' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (244 , 181 , 5 , '4' , NULL , '4' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (245 , 181 , 6 , '5' , NULL , '5' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (246 , 181 , 7 , '6' , NULL , '6' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (247 , 181 , 8 , '7' , NULL , '7' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (248 , 185 , 1 , 'DONT_SHOW_DESC_MSG' , NULL , '0' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (249 , 185 , 2 , 'SHORT_DESCRIPTION_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (250 , 185 , 3 , 'FULL_DESCRIPTION_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (251 , 187 , 1 , 'DONT_SHOW_IMAGE_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (252 , 187 , 2 , 'IMAGE_SMALL_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (253 , 187 , 3 , 'IMAGE_LARGE_MSG' , NULL , '3' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (254 , 188 , 1 , 'DONT_SHOW_DESC_MSG' , NULL , '0' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (255 , 188 , 2 , 'SHORT_DESCRIPTION_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (256 , 188 , 3 , 'FULL_DESCRIPTION_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (257 , 188 , 4 , 'HIGHLIGHTS_MSG' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (258 , 188 , 5 , 'SPECIAL_OFFER_MSG' , NULL , '4' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (259 , 15 , 3 , 'SUPER_IMAGE_IN_ZOOM_BOX_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (260 , 192 , 1 , 'NONE_MSG' , NULL , '0' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (261 , 192 , 2 , '{VERTICAL_SLIDER_MSG} ({UP_MSG})' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (262 , 192 , 4 , '{HORIZONTAL_SLIDER_MSG} ({LEFT_MSG})' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (263 , 195 , 2 , '{VERTICAL_SLIDER_MSG} ({UP_MSG})' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (264 , 195 , 4 , '{HORIZONTAL_SLIDER_MSG} ({LEFT_MSG})' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (265 , 195 , 3 , '{VERTICAL_SLIDER_MSG} ({DOWN_MSG})' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (266 , 195 , 5 , '{HORIZONTAL_SLIDER_MSG} ({RIGHT_MSG})' , NULL , '4' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (267 , 195 , 1 , 'SLIDESHOW_MSG' , NULL , '5' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (268 , 198 , 1 , 'DEFAULT_VIEW_MSG' , NULL , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (269 , 198 , 2 , 'CONTENT_AND_BORDERS_MSG' , NULL , '2' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (270 , 198 , 3 , 'CONTENT_ONLY_MSG' , NULL , '3' , 0 , 1 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (271 , 50 , 3 , '{VERTICAL_SLIDER_MSG} ({DOWN_MSG})' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (272 , 50 , 5 , '{HORIZONTAL_SLIDER_MSG} ({RIGHT_MSG})' , NULL , '4' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (273 , 192 , 3 , '{VERTICAL_SLIDER_MSG} ({DOWN_MSG})' , NULL , '3' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (274 , 192 , 5 , '{HORIZONTAL_SLIDER_MSG} ({RIGHT_MSG})' , NULL , '4' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (275 , 17 , 6 , 'AJAX_TREE_TYPE_MSG' , NULL , '6' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (276 , 212 , 1 , 'DEFAULT_POSITION_MSG' , NULL , 'default' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (277 , 212 , 2 , 'FIXED_POSITION_MSG' , NULL , 'fixed' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (278 , 220 , 1 , 'LEFT_MSG' , NULL , 'left' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (279 , 220 , 2 , 'RIGHT_MSG' , NULL , 'right' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (280 , 224 , 1 , 'LEFT_MSG' , NULL , 'left' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (281 , 224 , 2 , 'RIGHT_MSG' , NULL , 'right' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (282 , 228 , 1 , 'LEFT_MSG' , NULL , 'left' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (283 , 228 , 2 , 'RIGHT_MSG' , NULL , 'right' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (284 , 232 , 1 , 'LEFT_MSG' , NULL , 'left' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (285 , 232 , 2 , 'RIGHT_MSG' , NULL , 'right' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (286 , 236 , 1 , 'LEFT_MSG' , NULL , 'left' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (287 , 236 , 2 , 'RIGHT_MSG' , NULL , 'right' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (288 , 240 , 1 , 'LEFT_MSG' , NULL , 'left' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (289 , 240 , 2 , 'RIGHT_MSG' , NULL , 'right' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (290 , 244 , 1 , 'LEFT_MSG' , NULL , 'left' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (291 , 244 , 2 , 'RIGHT_MSG' , NULL , 'right' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (292 , 248 , 1 , 'LEFT_MSG' , NULL , 'left' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (293 , 248 , 2 , 'RIGHT_MSG' , NULL , 'right' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (294 , 252 , 1 , 'LEFT_MSG' , NULL , 'left' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (295 , 252 , 2 , 'RIGHT_MSG' , NULL , 'right' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (296 , 256 , 1 , 'LEFT_MSG' , NULL , 'left' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (297 , 256 , 2 , 'RIGHT_MSG' , NULL , 'right' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (298 , 260 , 1 , 'ARTICLE_TITLE_MSG' , 'article_title' , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (299 , 260 , 1 , 'AUTHOR_INFO_MSG' , 'author' , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (300 , 260 , 1 , 'ARTICLE_DATE_MSG' , 'article_date' , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (301 , 260 , 1 , 'IMAGE_TINY_MSG' , 'image_tiny' , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (302 , 260 , 1 , 'IMAGE_SMALL_MSG' , 'image_small' , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (303 , 260 , 1 , 'IMAGE_LARGE_MSG' , 'image_large' , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (304 , 260 , 1 , 'IMAGE_SUPER_MSG' , 'image_super' , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (305 , 260 , 1 , 'HOT_DESCRIPTION_MSG' , 'hot_description' , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (306 , 260 , 1 , 'HIGHLIGHTS_MSG' , 'highlights' , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (307 , 260 , 1 , 'SHORT_DESCRIPTION_MSG' , 'short_description' , '1' , 0 , 0 ) ;
INSERT INTO va_cms_blocks_values (value_id,property_id,value_order,value_name,variable_name,variable_value,hide_value,is_default_value) VALUES (308 , 260 , 1 , 'FULL_DESCRIPTION_MSG' , 'full_description' , '1' , 0 , 0 ) ;


INSERT INTO va_cms_frames (frame_id,layout_id,frame_name,tag_name) VALUES (1 , 1 , 'HEADER_MSG' , 'header' ) ;
INSERT INTO va_cms_frames (frame_id,layout_id,frame_name,tag_name) VALUES (2 , 1 , 'LEFT_COLUMN_MSG' , 'left' ) ;
INSERT INTO va_cms_frames (frame_id,layout_id,frame_name,tag_name) VALUES (3 , 1 , 'MIDDLE_COLUMN_MSG' , 'middle' ) ;
INSERT INTO va_cms_frames (frame_id,layout_id,frame_name,tag_name) VALUES (4 , 1 , 'RIGHT_COLUMN_MSG' , 'right' ) ;
INSERT INTO va_cms_frames (frame_id,layout_id,frame_name,tag_name) VALUES (5 , 1 , 'FOOTER_MSG' , 'footer' ) ;
INSERT INTO va_cms_frames (frame_id,layout_id,frame_name,tag_name) VALUES (6 , 2 , 'HEADER_MSG' , 'header' ) ;
INSERT INTO va_cms_frames (frame_id,layout_id,frame_name,tag_name) VALUES (7 , 2 , 'LEFT_COLUMN_MSG' , 'left' ) ;
INSERT INTO va_cms_frames (frame_id,layout_id,frame_name,tag_name) VALUES (8 , 2 , 'MIDDLE_COLUMN_MSG' , 'middle' ) ;
INSERT INTO va_cms_frames (frame_id,layout_id,frame_name,tag_name) VALUES (9 , 2 , 'FOOTER_MSG' , 'footer' ) ;
INSERT INTO va_cms_frames (frame_id,layout_id,frame_name,tag_name) VALUES (10 , 3 , 'HEADER_MSG' , 'header' ) ;
INSERT INTO va_cms_frames (frame_id,layout_id,frame_name,tag_name) VALUES (11 , 3 , 'MIDDLE_COLUMN_MSG' , 'middle' ) ;
INSERT INTO va_cms_frames (frame_id,layout_id,frame_name,tag_name) VALUES (12 , 3 , 'FOOTER_MSG' , 'footer' ) ;
INSERT INTO va_cms_frames (frame_id,layout_id,frame_name,tag_name) VALUES (13 , 4 , 'HEADER_MSG' , 'header' ) ;
INSERT INTO va_cms_frames (frame_id,layout_id,frame_name,tag_name) VALUES (14 , 4 , 'HOT_TITLE' , 'hot' ) ;
INSERT INTO va_cms_frames (frame_id,layout_id,frame_name,tag_name) VALUES (15 , 4 , 'LEFT_COLUMN_MSG' , 'left' ) ;
INSERT INTO va_cms_frames (frame_id,layout_id,frame_name,tag_name) VALUES (16 , 4 , 'MIDDLE_COLUMN_MSG' , 'middle' ) ;
INSERT INTO va_cms_frames (frame_id,layout_id,frame_name,tag_name) VALUES (17 , 4 , 'RIGHT_COLUMN_MSG' , 'right' ) ;
INSERT INTO va_cms_frames (frame_id,layout_id,frame_name,tag_name) VALUES (18 , 4 , 'FOOTER_MSG' , 'footer' ) ;
INSERT INTO va_cms_frames (frame_id,layout_id,frame_name,tag_name) VALUES (19 , 5 , 'MIDDLE_COLUMN_MSG' , 'middle' ) ;
INSERT INTO va_cms_frames (frame_id,layout_id,frame_name,tag_name) VALUES (20 , 1 , 'NAVIGATION_BAR_MSG' , 'frame_bar' ) ;
INSERT INTO va_cms_frames (frame_id,layout_id,frame_name,tag_name) VALUES (21 , 2 , 'NAVIGATION_BAR_MSG' , 'frame_bar' ) ;
INSERT INTO va_cms_frames (frame_id,layout_id,frame_name,tag_name) VALUES (22 , 3 , 'NAVIGATION_BAR_MSG' , 'frame_bar' ) ;
INSERT INTO va_cms_frames (frame_id,layout_id,frame_name,tag_name) VALUES (23 , 4 , 'NAVIGATION_BAR_MSG' , 'frame_bar' ) ;


INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (2 , 1 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (3 , 1 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (4 , 1 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (5 , 1 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (6 , 2 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (7 , 2 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (8 , 2 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (9 , 2 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (10 , 2 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (11 , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (12 , 3 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (13 , 3 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (14 , 3 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (15 , 3 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (16 , 4 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (17 , 4 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (18 , 4 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (19 , 4 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (20 , 4 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (21 , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (22 , 5 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (23 , 5 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (24 , 5 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (25 , 5 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (26 , 6 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (27 , 6 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (28 , 6 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (29 , 6 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (30 , 6 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (31 , 7 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (32 , 7 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (33 , 7 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (34 , 7 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (35 , 7 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (36 , 8 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (37 , 8 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (38 , 8 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (39 , 8 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (40 , 8 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (41 , 9 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (42 , 9 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (43 , 9 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (44 , 9 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (45 , 9 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (46 , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (47 , 10 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (48 , 10 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (49 , 10 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (50 , 10 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (51 , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (52 , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (53 , 11 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (54 , 11 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (55 , 11 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (56 , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (57 , 12 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (58 , 12 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (59 , 12 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (60 , 12 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (61 , 13 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (62 , 13 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (63 , 13 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (64 , 13 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (65 , 13 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (66 , 14 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (67 , 14 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (68 , 14 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (69 , 14 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (70 , 14 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (71 , 15 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (72 , 15 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (73 , 15 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (74 , 15 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (75 , 15 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (76 , 16 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (77 , 16 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (78 , 16 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (79 , 16 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (80 , 16 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (86 , 18 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (87 , 18 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (88 , 18 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (89 , 18 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (90 , 18 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (91 , 19 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (92 , 19 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (93 , 19 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (94 , 19 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (95 , 19 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (101 , 21 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (102 , 21 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (103 , 21 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (104 , 21 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (105 , 21 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (106 , 22 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (107 , 22 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (108 , 22 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (109 , 22 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (110 , 22 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (111 , 23 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (112 , 23 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (113 , 23 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (114 , 23 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (115 , 23 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (116 , 24 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (117 , 24 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (118 , 24 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (119 , 24 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (120 , 24 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (121 , 25 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (122 , 25 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (123 , 25 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (124 , 25 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (125 , 25 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (126 , 26 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (127 , 26 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (128 , 26 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (129 , 26 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (130 , 26 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (131 , 27 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (132 , 27 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (133 , 27 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (134 , 27 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (135 , 27 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (136 , 28 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (137 , 28 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (138 , 28 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (139 , 28 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (140 , 28 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (141 , 29 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (142 , 29 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (143 , 29 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (144 , 29 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (145 , 29 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (166 , 34 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (167 , 34 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (168 , 34 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (169 , 34 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (170 , 34 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (171 , 35 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (172 , 35 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (173 , 35 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (174 , 35 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (175 , 35 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (176 , 36 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (177 , 36 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (178 , 36 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (179 , 36 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (180 , 36 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (181 , 37 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (182 , 37 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (183 , 37 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (184 , 37 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (185 , 37 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (196 , 40 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (197 , 40 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (198 , 40 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (199 , 40 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (200 , 40 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (206 , 42 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (207 , 42 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (208 , 42 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (209 , 42 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (210 , 42 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (211 , 43 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (212 , 43 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (213 , 43 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (214 , 43 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (215 , 43 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (216 , 44 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (217 , 44 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (218 , 44 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (219 , 44 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (220 , 44 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (221 , 45 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (222 , 45 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (223 , 45 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (224 , 45 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (225 , 45 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (226 , 46 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (227 , 46 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (228 , 46 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (229 , 46 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (230 , 46 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (231 , 47 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (232 , 47 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (233 , 47 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (234 , 47 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (235 , 47 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (236 , 48 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (237 , 48 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (238 , 48 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (239 , 48 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (240 , 48 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (241 , 49 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (242 , 49 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (243 , 49 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (244 , 49 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (245 , 49 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (246 , 50 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (247 , 50 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (248 , 50 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (249 , 50 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (250 , 50 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (256 , 52 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (257 , 52 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (258 , 52 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (259 , 52 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (260 , 52 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (286 , 58 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (287 , 58 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (288 , 58 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (289 , 59 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (290 , 59 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (291 , 59 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (292 , 60 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (293 , 60 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (294 , 60 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (295 , 61 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (296 , 61 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (297 , 61 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (298 , 62 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (299 , 62 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (300 , 62 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (301 , 63 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (302 , 63 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (303 , 63 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (304 , 64 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (305 , 64 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (306 , 64 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (307 , 65 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (308 , 65 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (309 , 65 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (310 , 66 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (311 , 66 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (312 , 66 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (313 , 67 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (314 , 67 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (315 , 67 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (316 , 68 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (317 , 68 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (318 , 68 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (319 , 69 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (320 , 69 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (321 , 69 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (322 , 70 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (323 , 70 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (324 , 70 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (325 , 71 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (326 , 71 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (327 , 71 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (328 , 72 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (329 , 72 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (330 , 72 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (331 , 73 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (332 , 73 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (333 , 73 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (334 , 74 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (335 , 74 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (336 , 74 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (337 , 75 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (338 , 75 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (339 , 75 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (340 , 76 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (341 , 76 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (342 , 76 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (343 , 77 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (344 , 77 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (345 , 77 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (346 , 78 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (347 , 78 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (348 , 78 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (349 , 79 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (350 , 79 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (351 , 79 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (352 , 80 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (353 , 80 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (354 , 80 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (355 , 81 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (356 , 81 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (357 , 81 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (358 , 82 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (359 , 82 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (360 , 82 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (361 , 83 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (362 , 83 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (363 , 83 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (364 , 84 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (365 , 84 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (366 , 84 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (367 , 85 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (368 , 85 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (369 , 85 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (370 , 86 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (371 , 86 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (372 , 86 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (373 , 87 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (374 , 87 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (375 , 87 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (376 , 88 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (377 , 88 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (378 , 88 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (379 , 89 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (380 , 89 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (381 , 89 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (382 , 90 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (383 , 90 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (384 , 90 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (385 , 91 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (386 , 91 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (387 , 91 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (388 , 92 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (389 , 92 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (390 , 92 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (391 , 93 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (392 , 93 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (393 , 93 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (394 , 94 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (395 , 94 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (396 , 94 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (397 , 95 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (398 , 95 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (399 , 95 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (400 , 96 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (401 , 96 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (402 , 96 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (403 , 97 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (404 , 97 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (405 , 97 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (406 , 98 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (407 , 98 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (408 , 98 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (409 , 99 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (410 , 99 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (411 , 99 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (412 , 100 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (413 , 100 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (414 , 100 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (415 , 101 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (416 , 101 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (417 , 101 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (418 , 102 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (419 , 102 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (420 , 102 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (421 , 20 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (422 , 20 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (423 , 20 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (424 , 32 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (425 , 32 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (426 , 32 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (427 , 33 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (428 , 33 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (429 , 33 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (430 , 30 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (431 , 30 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (432 , 30 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (433 , 31 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (434 , 31 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (435 , 31 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (451 , 103 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (452 , 103 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (453 , 103 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (454 , 104 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (455 , 104 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (456 , 104 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (457 , 105 , 19 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (458 , 51 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (459 , 51 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (460 , 51 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (461 , 106 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (462 , 106 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (463 , 106 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (464 , 107 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (465 , 107 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (466 , 107 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (467 , 108 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (468 , 108 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (469 , 108 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (470 , 109 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (471 , 109 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (472 , 109 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (476 , 111 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (477 , 111 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (478 , 111 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (479 , 110 , 11 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (480 , 110 , 12 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (481 , 110 , 10 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (500 , 53 , 21 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (501 , 53 , 6 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (502 , 53 , 7 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (503 , 53 , 8 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (504 , 53 , 9 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (505 , 39 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (506 , 39 , 20 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (507 , 39 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (508 , 39 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (509 , 39 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (510 , 39 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (511 , 38 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (512 , 38 , 20 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (513 , 38 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (514 , 38 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (515 , 38 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (516 , 38 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (517 , 41 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (518 , 41 , 20 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (519 , 41 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (520 , 41 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (521 , 41 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (522 , 41 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (529 , 17 , 7 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (530 , 17 , 21 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (531 , 17 , 8 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (532 , 17 , 9 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_frames_settings (fs_id,ps_id,frame_id,frame_style,frame_class,frame_code,html_frame_start,html_before_block,html_after_block,html_frame_end,html_between_blocks) VALUES (533 , 17 , 6 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;


INSERT INTO va_cms_layouts (layout_id,layout_name,layout_order,layout_template,admin_template) VALUES (1 , 'THREE_COLUMNS_MSG' , 1 , 'layout_three_columns.html' , 'admin_cms_columns3.html' ) ;
INSERT INTO va_cms_layouts (layout_id,layout_name,layout_order,layout_template,admin_template) VALUES (2 , 'TWO_COLUMNS_MSG' , 2 , 'layout_two_columns.html' , 'admin_cms_columns2.html' ) ;
INSERT INTO va_cms_layouts (layout_id,layout_name,layout_order,layout_template,admin_template) VALUES (3 , 'ONE_COLUMN_MSG' , 3 , 'layout_one_column.html' , 'admin_cms_column1.html' ) ;
INSERT INTO va_cms_layouts (layout_id,layout_name,layout_order,layout_template,admin_template) VALUES (4 , 'HOT_LAYOUT_MSG' , 4 , 'layout_hot.html' , 'admin_cms_hot_layout.html' ) ;
INSERT INTO va_cms_layouts (layout_id,layout_name,layout_order,layout_template,admin_template) VALUES (5 , 'POPUP_FRAME_LAYOUT_MSG' , 5 , 'layout_popup_frame.html' , 'admin_layout_popup_frame.html' ) ;


INSERT INTO va_cms_modules (module_id,module_order,module_code,module_name) VALUES (1 , 1 , 'global' , 'GLOBAL_MSG' ) ;
INSERT INTO va_cms_modules (module_id,module_order,module_code,module_name) VALUES (2 , 2 , 'cart' , 'PRODUCTS_TITLE' ) ;
INSERT INTO va_cms_modules (module_id,module_order,module_code,module_name) VALUES (3 , 3 , 'articles' , '{ARTICLES_TITLE}: {category_name}' ) ;
INSERT INTO va_cms_modules (module_id,module_order,module_code,module_name) VALUES (4 , 4 , 'helpdesk' , 'HELPDESK_MSG' ) ;
INSERT INTO va_cms_modules (module_id,module_order,module_code,module_name) VALUES (5 , 5 , 'forum' , 'FORUM_TITLE' ) ;
INSERT INTO va_cms_modules (module_id,module_order,module_code,module_name) VALUES (6 , 6 , 'ads' , 'ADS_TITLE' ) ;
INSERT INTO va_cms_modules (module_id,module_order,module_code,module_name) VALUES (7 , 7 , 'manuals' , 'MANUALS_TITLE' ) ;
INSERT INTO va_cms_modules (module_id,module_order,module_code,module_name) VALUES (8 , 8 , 'filters' , 'FILTERS_MSG' ) ;
INSERT INTO va_cms_modules (module_id,module_order,module_code,module_name) VALUES (9 , 9 , 'custom_blocks' , 'CUSTOM_BLOCKS_MSG' ) ;
INSERT INTO va_cms_modules (module_id,module_order,module_code,module_name) VALUES (10 , 10 , 'custom_menus' , 'CUSTOM_MENUS_MSG' ) ;
INSERT INTO va_cms_modules (module_id,module_order,module_code,module_name) VALUES (11 , 11 , 'banners' , 'BANNERS_MSG' ) ;
INSERT INTO va_cms_modules (module_id,module_order,module_code,module_name) VALUES (12 , 12 , 'user_account' , 'USER_ACCOUNT_MSG' ) ;
INSERT INTO va_cms_modules (module_id,module_order,module_code,module_name) VALUES (13 , 13 , 'sliders' , 'SLIDERS_MSG' ) ;
INSERT INTO va_cms_modules (module_id,module_order,module_code,module_name) VALUES (14 , 14 , 'social_media' , 'SOCIAL_MEDIA_MSG' ) ;
INSERT INTO va_cms_modules (module_id,module_order,module_code,module_name) VALUES (15 , 15 , 'profiles' , 'PROFILES_TITLE' ) ;
INSERT INTO va_cms_modules (module_id,module_order,module_code,module_name) VALUES (16 , 16 , 'authors' , 'AUTHORS_MSG' ) ;


INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (1 , 1 , 1 , 'index' , 'HOME_PAGE_LAYOUT_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (2 , 1 , 2 , 'site_map' , 'SITE_MAP_TITLE' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (3 , 1 , 3 , 'site_search' , 'FULL_SITE_SEARCH_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (4 , 1 , 4 , 'custom_page' , 'CUSTOM_PAGES_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (5 , 1 , 5 , 'forgot_password' , 'FORGOTTEN_PASSWORD_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (6 , 1 , 6 , 'reset_password' , 'RESET_PASSWORD_INFO_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (7 , 1 , 7 , 'polls' , 'PREVIOUS_POLLS_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (8 , 2 , 1 , 'products_list' , 'PRODUCTS_LISTING_PAGE_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (9 , 2 , 2 , 'products_search_results' , 'PRODUCTS_SEARCH_RESULTS_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (10 , 2 , 3 , 'product_details' , 'PRODUCTS_DETAILS_PAGE_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (11 , 2 , 4 , 'product_options' , 'PRODUCT_OPTIONS_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (12 , 2 , 5 , 'products_compare' , 'PRODUCTS_COMPARE_RESULTS_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (13 , 2 , 6 , 'products_search_advanced' , 'ADVANCED_SEARCH_TITLE' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (14 , 2 , 7 , 'wishlist' , 'WISHLIST_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (15 , 2 , 8 , 'products_releases' , 'RELEASES_TITLE' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (16 , 2 , 9 , 'products_changes_log' , 'CHANGES_LOG_TITLE' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (17 , 2 , 10 , 'cart' , 'ADMIN_BASKET_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (18 , 2 , 11 , 'cart_save' , 'SAVE_CART_BUTTON' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (19 , 2 , 12 , 'cart_retrieve' , 'RETRIEVE_CART_BUTTON' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (20 , 2 , 13 , 'subscriptions' , 'SUBSCRIPTIONS_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (21 , 2 , 14 , 'checkout' , 'CHECKOUT_LOGIN_TITLE' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (22 , 2 , 15 , 'order_info' , 'ONE_PAGE_CHECKOUT_TITLE' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (23 , 2 , 16 , 'order_payment_details' , 'CHECKOUT_PAYMENT_TITLE' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (24 , 2 , 17 , 'order_confirmation' , 'CHECKOUT_CONFIRM_TITLE' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (25 , 2 , 18 , 'order_final' , 'FINAL_CHECKOUT_PAGE_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (26 , 1 , 9 , 'user_login' , 'LOGIN_TITLE' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (27 , 1 , 10 , 'user_profile' , 'PROFILE_SETTINGS_INFO_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (28 , 12 , 1 , 'user_home' , 'DEFAULT_USERHOME_PAGE_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (29 , 1 , 11 , 'user_products_list' , 'USER_LISTING_PAGE_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (30 , 3 , 1 , 'articles_list' , 'LISTING_PAGE_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (31 , 3 , 2 , 'article_details' , 'DETAILS_PAGE_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (32 , 4 , 1 , 'ticket_new' , 'NEW_TICKET_PAGE_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (33 , 4 , 3 , 'ticket_reply' , '{USER_ACCOUNT_MSG}: {REQUEST_DETAILS_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (34 , 5 , 1 , 'forum_list' , 'FORUMS_LIST_PAGE_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (35 , 5 , 2 , 'forum_topics' , 'FORUM_TOPICS_PAGE_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (36 , 5 , 3 , 'forum_topic' , 'FORUM_TOPICS_THREAD_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (37 , 6 , 1 , 'ads_list' , 'LISTING_PAGE_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (38 , 6 , 2 , 'ad_details' , 'DETAILS_PAGE_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (39 , 6 , 3 , 'ads_compare' , 'ADS_COMPARE_TITLE' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (40 , 6 , 4 , 'ads_search_advanced' , 'ADS_ADVANCED_SEARCH_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (41 , 7 , 1 , 'manuals_list' , 'MANUALS_LIST_PAGE_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (42 , 7 , 2 , 'manual_articles' , 'MANUAL_ARTICLES_PAGE_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (43 , 7 , 3 , 'manual_article_details' , 'MANUAL_ARTICLE_DETAILS_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (44 , 7 , 4 , 'manuals_search_results' , 'MANUAL_SEARCH_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (45 , 2 , 5 , 'product_reviews' , 'PRODUCTS_REVIEWS_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (46 , 3 , 3 , 'article_reviews' , 'ARTICLES_REVIEWS_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (47 , 1 , 8 , 'download' , 'DOWNLOAD_TITLE' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (48 , 5 , 4 , 'forum_topic_new' , 'NEW_TOPIC_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (49 , 12 , 2 , 'user_account_profile' , 'EDIT_PROFILE_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (50 , 12 , 3 , 'user_change_password' , 'CHANGE_PASSWORD_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (51 , 12 , 4 , 'user_change_type' , 'UPGRADE_DOWNGRADE_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (52 , 12 , 5 , 'user_carts' , 'MY_SAVED_CARTS_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (53 , 12 , 6 , 'user_wishlist' , 'MY_WISHLIST_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (54 , 12 , 7 , 'user_orders' , '{MY_ORDERS_MSG}: {LIST_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (55 , 12 , 8 , 'user_order' , '{MY_ORDERS_MSG}: {VIEW_DETAILS_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (56 , 12 , 9 , 'user_order_update' , '{MY_ORDERS_MSG}:  {EDIT_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (57 , 4 , 2 , 'user_support' , '{USER_ACCOUNT_MSG}: {MY_SUPPORT_ISSUES_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (58 , 12 , 11 , 'user_ads' , '{MY_ADS_MSG}: {LIST_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (59 , 12 , 12 , 'user_ad' , '{MY_ADS_MSG}: {EDIT_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (60 , 12 , 13 , 'user_affiliate_sales' , 'AFFILIATE_SALES_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (61 , 12 , 14 , 'user_affiliate_items' , '{AFFILIATE_SALES_MSG}: {PRODUCTS_LIST_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (62 , 12 , 15 , 'user_products' , '{MY_PRODUCTS_MSG}: {LIST_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (63 , 12 , 16 , 'user_product' , '{MY_PRODUCTS_MSG}: {EDIT_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (64 , 12 , 17 , 'user_product_options' , '{OPTIONS_AND_COMPONENTS_MSG}: {LIST_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (65 , 12 , 18 , 'user_product_option' , '{OPTIONS_AND_COMPONENTS_MSG}: {EDIT_OPTION_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (66 , 12 , 19 , 'user_product_subcomponent' , '{OPTIONS_AND_COMPONENTS_MSG}: {EDIT_SUBCOMP_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (67 , 12 , 20 , 'user_product_subcomponents' , '{OPTIONS_AND_COMPONENTS_MSG}: {EDIT_SUBCOMP_SELECTION_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (68 , 12 , 21 , 'user_payments' , '{COMMISSION_PAYMENTS_MSG}: {LIST_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (69 , 12 , 22 , 'user_payment' , '{COMMISSION_PAYMENTS_MSG}: {VIEW_DETAILS_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (70 , 12 , 23 , 'user_merchant_orders' , '{MY_SALES_ORDERS_MSG}: {LIST_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (71 , 12 , 24 , 'user_merchant_order' , '{MY_SALES_ORDERS_MSG}: {VIEW_DETAILS_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (72 , 12 , 25 , 'user_merchant_sales' , 'MERCHANT_SALES_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (73 , 12 , 26 , 'user_merchant_items' , '{MERCHANT_SALES_MSG}: {PRODUCTS_LIST_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (74 , 12 , 27 , 'user_reminders' , '{MY_REMINDERS_MSG}: {LIST_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (75 , 12 , 28 , 'user_reminder' , '{MY_REMINDERS_MSG}: {EDIT_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (76 , 12 , 29 , 'user_psd_list' , '{PAYMENT_DETAILS_MSG}: {LIST_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (77 , 12 , 30 , 'user_psd_update' , '{PAYMENT_DETAILS_MSG}: {EDIT_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (78 , 12 , 31 , 'user_product_registrations' , 'MY_PRODUCT_REGISTRATIONS_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (79 , 12 , 32 , 'user_product_registration' , 'REGISTER_PRODUCT_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (80 , 1 , 12 , 'contact_us' , 'CONTACT_US_TITLE' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (81 , 12 , 33 , 'user_addresses' , '{MY_ADDRESSES_MSG}: {LIST_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (82 , 12 , 34 , 'user_address' , '{MY_ADDRESSES_MSG}: {EDIT_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (83 , 1 , 13 , 'subscribe' , 'SUBSCRIBE_TITLE' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (84 , 1 , 14 , 'unsubscribe' , 'UNSUBSCRIBE_TITLE' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (85 , 2 , 19 , 'add_to_cart_frame' , 'ADD_TO_CART_FRAME_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (86 , 12 , 35 , 'user_messages' , 'MY_MESSAGES_MSG' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (87 , 15 , 1 , 'profiles_home' , '{PROFILES_TITLE}: {HOME_PAGE_TITLE}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (88 , 15 , 2 , 'profiles_list' , '{PROFILES_TITLE}: {LIST_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (89 , 15 , 3 , 'profiles_view' , '{PROFILES_TITLE}: {VIEW_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (90 , 15 , 4 , 'profiles_user_list' , '{MY_PROFILES_MSG}: {LIST_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (91 , 15 , 5 , 'profiles_user_edit' , '{MY_PROFILES_MSG}: {EDIT_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (92 , 16 , 1 , 'authors_list' , '{AUTHORS_MSG}: {LISTING_PAGE_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (93 , 16 , 2 , 'author_details' , '{AUTHORS_MSG}: {DETAILS_PAGE_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (94 , 16 , 3 , 'author_articles' , '{AUTHOR_MSG}: {ARTICLES_TITLE}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (95 , 16 , 4 , 'albums_list' , '{ALBUMS_MSG}: {LISTING_PAGE_MSG}' ) ;
INSERT INTO va_cms_pages (page_id,module_id,page_order,page_code,page_name) VALUES (96 , 16 , 5 , 'album_details' , '{ALBUMS_MSG}: {DETAILS_PAGE_MSG}' ) ;


INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (1 , 1 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (2 , 1 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (3 , 1 , 66 , '1' , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (4 , 1 , 68 , '1' , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (5 , 1 , 73 , '1' , 3 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (6 , 2 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (7 , 2 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (8 , 2 , 66 , '3' , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (9 , 2 , 68 , '3' , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (10 , 3 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (11 , 3 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (12 , 3 , 66 , '30' , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (13 , 3 , 68 , '30' , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (14 , 4 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (15 , 4 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (16 , 4 , 68 , '4' , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (17 , 5 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (18 , 5 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (19 , 5 , 68 , '5' , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (20 , 6 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (21 , 6 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (22 , 6 , 66 , '6' , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (23 , 6 , 68 , '6' , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (24 , 7 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (25 , 7 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (26 , 7 , 66 , '1' , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (27 , 7 , 67 , '1' , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (28 , 8 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (29 , 8 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (30 , 8 , 66 , '3' , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (31 , 8 , 69 , '3' , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (32 , 8 , 67 , '3' , 3 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (33 , 9 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (34 , 9 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (35 , 9 , 66 , '30' , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (36 , 9 , 67 , '30' , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (37 , 10 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (38 , 10 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (39 , 10 , 71 , '4' , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (40 , 10 , 67 , '4' , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (41 , 11 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (42 , 11 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (43 , 11 , 67 , '5' , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (44 , 12 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (45 , 12 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (46 , 12 , 66 , '6' , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (47 , 12 , 67 , '6' , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (48 , 13 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (49 , 13 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (50 , 13 , 98 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (51 , 14 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (52 , 14 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (53 , 14 , 93 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (54 , 14 , 94 , NULL , 2 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (55 , 14 , 95 , NULL , 2 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (56 , 14 , 97 , NULL , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (57 , 14 , 104 , NULL , 2 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (58 , 14 , 106 , NULL , 2 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (59 , 15 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (60 , 15 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (61 , 15 , 93 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (62 , 15 , 94 , NULL , 2 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (63 , 15 , 95 , NULL , 2 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (64 , 15 , 96 , NULL , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (65 , 15 , 106 , NULL , 2 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (66 , 16 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (67 , 16 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (68 , 16 , 107 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (75 , 18 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (76 , 18 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (77 , 18 , 25 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (78 , 19 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (79 , 19 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (80 , 19 , 26 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (84 , 21 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (85 , 21 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (86 , 21 , 8 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (87 , 22 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (88 , 22 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (89 , 22 , 6 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (90 , 23 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (91 , 23 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (92 , 23 , 83 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (93 , 23 , 84 , NULL , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (94 , 24 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (95 , 24 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (96 , 24 , 83 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (97 , 24 , 87 , NULL , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (98 , 25 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (99 , 25 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (100 , 25 , 83 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (101 , 25 , 86 , NULL , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (102 , 26 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (103 , 26 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (104 , 26 , 111 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (105 , 26 , 112 , NULL , 2 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (106 , 27 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (107 , 27 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (108 , 27 , 110 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (109 , 27 , 112 , NULL , 2 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (110 , 28 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (111 , 28 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (112 , 28 , 109 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (113 , 28 , 112 , NULL , 2 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (114 , 29 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (115 , 29 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (116 , 29 , 112 , NULL , 2 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (117 , 29 , 113 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (135 , 34 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (136 , 34 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (137 , 34 , 11 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (138 , 35 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (139 , 35 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (140 , 35 , 28 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (141 , 36 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (142 , 36 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (143 , 36 , 36 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (144 , 37 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (145 , 37 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (146 , 37 , 37 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (168 , 40 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (169 , 40 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (170 , 40 , 43 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (181 , 42 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (182 , 42 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (183 , 42 , 7 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (184 , 43 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (185 , 43 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (186 , 43 , 12 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (187 , 44 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (188 , 44 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (189 , 44 , 16 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (190 , 44 , 17 , NULL , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (191 , 45 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (192 , 45 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (193 , 45 , 59 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (194 , 46 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (195 , 46 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (196 , 46 , 81 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (197 , 47 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (198 , 47 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (199 , 47 , 82 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (200 , 48 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (201 , 48 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (202 , 48 , 24 , NULL , 2 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (203 , 48 , 32 , NULL , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (204 , 48 , 31 , NULL , 2 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (205 , 48 , 18 , NULL , 3 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (206 , 48 , 35 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (207 , 48 , 27 , NULL , 2 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (208 , 49 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (209 , 49 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (210 , 49 , 5 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (211 , 50 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (212 , 50 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (213 , 50 , 2 , NULL , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (214 , 50 , 14 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (219 , 52 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (220 , 52 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (221 , 52 , 62 , NULL , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (222 , 52 , 63 , NULL , 3 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (272 , 58 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (273 , 58 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (274 , 58 , 120 , NULL , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (275 , 59 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (276 , 59 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (277 , 59 , 121 , '1' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (278 , 60 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (279 , 60 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (280 , 60 , 121 , '3' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (281 , 61 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (282 , 61 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (283 , 61 , 121 , '4' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (284 , 62 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (285 , 62 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (286 , 62 , 121 , '5' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (287 , 63 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (288 , 63 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (289 , 63 , 121 , '6' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (290 , 64 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (291 , 64 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (292 , 64 , 121 , '30' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (293 , 65 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (294 , 65 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (295 , 65 , 122 , NULL , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (296 , 66 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (297 , 66 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (298 , 66 , 123 , NULL , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (299 , 67 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (300 , 67 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (301 , 67 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (302 , 67 , 124 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (303 , 68 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (304 , 68 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (305 , 68 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (306 , 68 , 125 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (307 , 69 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (308 , 69 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (309 , 69 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (310 , 69 , 126 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (311 , 70 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (312 , 70 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (313 , 70 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (314 , 70 , 127 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (315 , 71 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (316 , 71 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (317 , 71 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (318 , 71 , 128 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (319 , 72 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (320 , 72 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (321 , 72 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (322 , 72 , 129 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (323 , 73 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (324 , 73 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (325 , 73 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (326 , 73 , 130 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (327 , 74 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (328 , 74 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (329 , 74 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (330 , 74 , 131 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (331 , 75 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (332 , 75 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (333 , 75 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (334 , 75 , 132 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (335 , 76 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (336 , 76 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (337 , 76 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (338 , 76 , 133 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (339 , 77 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (340 , 77 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (341 , 77 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (342 , 77 , 134 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (343 , 78 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (344 , 78 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (345 , 78 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (346 , 78 , 135 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (347 , 79 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (348 , 79 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (349 , 79 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (350 , 79 , 136 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (351 , 80 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (352 , 80 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (353 , 80 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (354 , 80 , 137 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (355 , 81 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (356 , 81 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (357 , 81 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (358 , 81 , 138 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (359 , 82 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (360 , 82 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (361 , 82 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (362 , 82 , 139 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (363 , 83 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (364 , 83 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (365 , 83 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (366 , 83 , 140 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (367 , 84 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (368 , 84 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (369 , 84 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (370 , 84 , 141 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (371 , 85 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (372 , 85 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (373 , 85 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (374 , 85 , 142 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (375 , 86 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (376 , 86 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (377 , 86 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (378 , 86 , 143 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (379 , 87 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (380 , 87 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (381 , 87 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (382 , 87 , 144 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (383 , 88 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (384 , 88 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (385 , 88 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (386 , 88 , 145 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (387 , 89 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (388 , 89 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (389 , 89 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (390 , 89 , 146 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (391 , 90 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (392 , 90 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (393 , 90 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (394 , 90 , 147 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (395 , 91 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (396 , 91 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (397 , 91 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (398 , 91 , 148 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (399 , 92 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (400 , 92 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (401 , 92 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (402 , 92 , 149 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (403 , 93 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (404 , 93 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (405 , 93 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (406 , 93 , 150 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (407 , 94 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (408 , 94 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (409 , 94 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (410 , 94 , 151 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (411 , 95 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (412 , 95 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (413 , 95 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (414 , 95 , 152 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (415 , 96 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (416 , 96 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (417 , 96 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (418 , 96 , 153 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (419 , 97 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (420 , 97 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (421 , 97 , 14 , 'userhome_breadcrumb' , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (422 , 97 , 154 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (423 , 98 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (424 , 98 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (425 , 98 , 155 , NULL , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (429 , 99 , 160 , NULL , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (430 , 99 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (431 , 99 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (432 , 100 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (433 , 100 , 161 , NULL , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (434 , 100 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (435 , 101 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (436 , 101 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (437 , 101 , 40 , NULL , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (438 , 102 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (439 , 102 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (440 , 102 , 40 , NULL , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (441 , 20 , 52 , NULL , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (442 , 20 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (443 , 20 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (444 , 32 , 50 , NULL , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (445 , 32 , 54 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (446 , 32 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (447 , 32 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (448 , 33 , 50 , NULL , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (449 , 33 , 53 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (450 , 33 , 56 , NULL , 11 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (451 , 33 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (452 , 33 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (453 , 30 , 50 , NULL , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (454 , 30 , 53 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (455 , 30 , 55 , NULL , 11 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (456 , 30 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (457 , 30 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (458 , 31 , 51 , NULL , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (459 , 31 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (460 , 31 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (497 , 103 , 3 , NULL , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (498 , 103 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (499 , 103 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (500 , 104 , 157 , NULL , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (501 , 104 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (502 , 104 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (503 , 105 , 22 , NULL , 19 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (504 , 51 , 14 , NULL , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (505 , 51 , 15 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (506 , 51 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (507 , 51 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (508 , 106 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (509 , 106 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (510 , 106 , 14 , NULL , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (511 , 106 , 166 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (512 , 107 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (513 , 107 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (514 , 107 , 167 , NULL , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (515 , 108 , 168 , NULL , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (516 , 108 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (517 , 108 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (518 , 109 , 169 , NULL , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (519 , 109 , 119 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (520 , 109 , 118 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (524 , 111 , 14 , NULL , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (525 , 111 , 171 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (526 , 111 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (527 , 111 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (528 , 110 , 14 , NULL , 11 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (529 , 110 , 170 , NULL , 11 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (530 , 110 , 119 , NULL , 12 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (531 , 110 , 118 , NULL , 10 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (551 , 112 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (552 , 112 , 174 , NULL , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (553 , 112 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (554 , 113 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (555 , 113 , 175 , NULL , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (556 , 113 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (557 , 114 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (558 , 114 , 176 , NULL , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (559 , 114 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (560 , 115 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (561 , 115 , 177 , NULL , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (562 , 115 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (563 , 116 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (564 , 116 , 178 , NULL , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (565 , 116 , 179 , NULL , 3 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (566 , 116 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (591 , 53 , 172 , NULL , 21 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (592 , 53 , 118 , NULL , 6 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (593 , 53 , 24 , NULL , 7 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (594 , 53 , 27 , NULL , 7 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (595 , 53 , 1 , NULL , 7 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (596 , 53 , 47 , NULL , 7 , 4 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (597 , 53 , 115 , '4' , 7 , 5 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (598 , 53 , 4 , NULL , 7 , 6 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (599 , 53 , 34 , NULL , 8 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (600 , 53 , 20 , NULL , 8 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (601 , 53 , 72 , '1' , 8 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (602 , 53 , 119 , NULL , 9 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (603 , 39 , 20 , NULL , 2 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (604 , 39 , 29 , NULL , 2 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (605 , 39 , 39 , NULL , 2 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (606 , 39 , 172 , NULL , 20 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (607 , 39 , 35 , NULL , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (608 , 39 , 114 , '1' , 3 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (609 , 39 , 18 , NULL , 3 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (610 , 39 , 27 , NULL , 4 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (611 , 39 , 24 , NULL , 4 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (612 , 39 , 41 , NULL , 4 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (613 , 39 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (614 , 39 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (615 , 38 , 20 , NULL , 2 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (616 , 38 , 39 , NULL , 2 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (617 , 38 , 172 , NULL , 20 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (618 , 38 , 35 , NULL , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (619 , 38 , 19 , NULL , 3 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (620 , 38 , 48 , NULL , 3 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (621 , 38 , 27 , NULL , 4 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (622 , 38 , 24 , NULL , 4 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (623 , 38 , 49 , NULL , 4 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (624 , 38 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (625 , 38 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (626 , 41 , 20 , NULL , 2 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (627 , 41 , 29 , NULL , 2 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (628 , 41 , 39 , NULL , 2 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (629 , 41 , 172 , NULL , 20 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (630 , 41 , 35 , NULL , 3 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (631 , 41 , 18 , NULL , 3 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (632 , 41 , 27 , NULL , 4 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (633 , 41 , 24 , NULL , 4 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (634 , 41 , 41 , NULL , 4 , 3 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (635 , 41 , 119 , NULL , 5 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (636 , 41 , 118 , NULL , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (644 , 17 , 20 , NULL , 7 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (645 , 17 , 58 , NULL , 7 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (646 , 17 , 172 , NULL , 21 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (647 , 17 , 22 , NULL , 8 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (648 , 17 , 23 , NULL , 8 , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (649 , 17 , 119 , NULL , 9 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_blocks (pb_id,ps_id,block_id,block_key,frame_id,block_order,tag_name,layout_type,layout_template,html_template,css_class,block_style,block_title) VALUES (650 , 17 , 118 , NULL , 6 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;


INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (1 , 31 , '1' , 'category' , 'all' , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (2 , 31 , '3' , 'category' , 'all' , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (3 , 31 , '30' , 'category' , 'all' , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (4 , 31 , '4' , 'category' , 'all' , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (5 , 31 , '5' , 'category' , 'all' , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (6 , 31 , '6' , 'category' , 'all' , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (7 , 30 , '1' , 'category' , 'all' , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (8 , 30 , '3' , 'category' , 'all' , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (9 , 30 , '30' , 'category' , 'all' , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (10 , 30 , '4' , 'category' , 'all' , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (11 , 30 , '5' , 'category' , 'all' , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (12 , 30 , '6' , 'category' , 'all' , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (13 , 39 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (14 , 38 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (15 , 37 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (16 , 40 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (17 , 17 , '' , '' , '' , 2 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (18 , 19 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (19 , 18 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (20 , 21 , '' , '' , '' , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (21 , 4 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (22 , 5 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (23 , 34 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (24 , 36 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (25 , 35 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (26 , 43 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (27 , 42 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (28 , 41 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (29 , 44 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (30 , 24 , '' , '' , '' , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (31 , 25 , '' , '' , '' , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (32 , 22 , '' , '' , '' , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (33 , 23 , '' , '' , '' , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (34 , 7 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (35 , 13 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (36 , 16 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (37 , 12 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (38 , 10 , '' , '' , '' , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (39 , 8 , '' , '' , '' , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (40 , 15 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (41 , 9 , '' , '' , '' , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (42 , 6 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (43 , 2 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (44 , 3 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (45 , 20 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (46 , 32 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (47 , 33 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (48 , 29 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (49 , 26 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (50 , 27 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (51 , 28 , '' , '' , '' , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (52 , 14 , '' , '' , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (53 , 1 , '' , '' , '' , 2 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (58 , 45 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (59 , 46 , '1' , 'category' , 'all' , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (60 , 46 , '3' , 'category' , 'all' , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (61 , 46 , '4' , 'category' , 'all' , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (62 , 46 , '5' , 'category' , 'all' , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (63 , 46 , '6' , 'category' , 'all' , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (64 , 46 , '30' , 'category' , 'all' , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (65 , 47 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (66 , 48 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (67 , 49 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (68 , 50 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (69 , 51 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (70 , 52 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (71 , 53 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (72 , 54 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (73 , 55 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (74 , 56 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (75 , 57 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (76 , 58 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (77 , 59 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (78 , 60 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (79 , 61 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (80 , 62 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (81 , 63 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (82 , 64 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (83 , 65 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (84 , 66 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (85 , 67 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (86 , 68 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (87 , 69 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (88 , 70 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (89 , 71 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (90 , 72 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (91 , 73 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (92 , 74 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (93 , 75 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (94 , 76 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (95 , 77 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (96 , 78 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (97 , 79 , '' , '' , NULL , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (98 , 80 , '' , '' , '' , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (99 , 81 , '' , '' , '' , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (100 , 82 , '' , '' , '' , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (101 , 11 , '' , '' , '' , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (102 , 11 , '' , '' , '' , 3 , 2 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (103 , 83 , '' , '' , '' , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (104 , 84 , '' , '' , '' , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (105 , 85 , '' , '' , '' , 5 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (106 , 86 , '' , '' , '' , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (107 , 87 , '' , '' , '' , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (108 , 88 , '' , '' , '' , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (109 , 89 , '' , '' , '' , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (110 , 90 , '' , '' , '' , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (111 , 91 , '' , '' , '' , 3 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (112 , 92 , NULL , NULL , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (113 , 93 , NULL , NULL , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (114 , 94 , NULL , NULL , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (115 , 95 , NULL , NULL , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_cms_pages_settings (ps_id,page_id,key_code,key_type,key_rule,layout_id,site_id,page_class,meta_title,meta_keywords,meta_description) VALUES (116 , 96 , NULL , NULL , NULL , 1 , 1 , NULL , NULL , NULL , NULL ) ;




INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (1 , 'AD' , '020' , 'AND' , 3 , 'Andorra' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (2 , 'AE' , '784' , 'ARE' , 3 , 'United Arab Emirates' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (3 , 'AF' , '004' , 'AFG' , 3 , 'Afghanistan' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (4 , 'AG' , '028' , 'ATG' , 3 , 'Antigua and Barbuda' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (5 , 'AI' , '660' , 'AIA' , 3 , 'Anguilla' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (6 , 'AL' , '008' , 'ALB' , 3 , 'Albania' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (7 , 'AM' , '051' , 'ARM' , 3 , 'Armenia' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (8 , 'AN' , '530' , 'ANT' , 2 , 'Netherlands Antilles' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (9 , 'AO' , '024' , 'AGO' , 3 , 'Angola' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (10 , 'AQ' , '010' , 'ATA' , 3 , 'Antarctica' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (11 , 'AR' , '032' , 'ARG' , 3 , 'Argentina' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (12 , 'AS' , '016' , 'ASM' , 3 , 'American Samoa' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (13 , 'AT' , '040' , 'AUT' , 2 , 'Austria' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (14 , 'AU' , '036' , 'AUS' , 3 , 'Australia' , NULL , NULL , NULL , 'AUD' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (15 , 'AW' , '533' , 'ABW' , 3 , 'Aruba' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (16 , 'AX' , '248' , 'ALA' , 3 , 'Aland Islands' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (17 , 'AZ' , '031' , 'AZE' , 3 , 'Azerbaijan' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (18 , 'BA' , '070' , 'BIH' , 3 , 'Bosnia and Herzegowina' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (19 , 'BB' , '052' , 'BRB' , 3 , 'Barbados' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (20 , 'BD' , '050' , 'BGD' , 3 , 'Bangladesh' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (21 , 'BE' , '056' , 'BEL' , 2 , 'Belgium' , NULL , NULL , NULL , 'DKK' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (22 , 'BF' , '854' , 'BFA' , 3 , 'Burkina Faso' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (23 , 'BG' , '100' , 'BGR' , 3 , 'Bulgaria' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (24 , 'BH' , '048' , 'BHR' , 3 , 'Bahrain' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (25 , 'BI' , '108' , 'BDI' , 3 , 'Burundi' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (26 , 'BJ' , '204' , 'BEN' , 3 , 'Benin' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (27 , 'BM' , '060' , 'BMU' , 3 , 'Bermuda' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (28 , 'BN' , '096' , 'BRN' , 3 , 'Brunei Darussalam' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (29 , 'BO' , '068' , 'BOL' , 3 , 'Bolivia' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (30 , 'BR' , '076' , 'BRA' , 3 , 'Brazil' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (31 , 'BS' , '044' , 'BHS' , 3 , 'Bahamas' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (32 , 'BT' , '064' , 'BTN' , 3 , 'Bhutan' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (33 , 'BV' , '074' , 'BVT' , 3 , 'Bouvet Island' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (34 , 'BW' , '072' , 'BWA' , 3 , 'Botswana' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (35 , 'BY' , '112' , 'BLR' , 3 , 'Belarus' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (36 , 'BZ' , '084' , 'BLZ' , 3 , 'Belize' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (37 , 'CA' , '124' , 'CAN' , 3 , 'Canada' , NULL , NULL , NULL , 'CAD' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (38 , 'CC' , '166' , 'CCK' , 3 , 'Cocos (Keeling) Islands' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (39 , 'CD' , '180' , 'COD' , 3 , 'Congo' , 'Democratic Republic of the Congo' , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (40 , 'CF' , '140' , 'CAF' , 3 , 'Central African Republic' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (41 , 'CG' , '178' , 'COG' , 3 , 'Congo' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (42 , 'CH' , '756' , 'CHE' , 3 , 'Switzerland' , NULL , NULL , NULL , 'CHF' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (43 , 'CI' , '384' , 'CIV' , 3 , 'Cote d\'Ivoire' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (44 , 'CK' , '184' , 'COK' , 3 , 'Cook Islands' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (45 , 'CL' , '152' , 'CHL' , 3 , 'Chile' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (46 , 'CM' , '120' , 'CMR' , 3 , 'Cameroon' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (47 , 'CN' , '156' , 'CHN' , 3 , 'China' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (48 , 'CO' , '170' , 'COL' , 3 , 'Colombia' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (49 , 'CR' , '188' , 'CRI' , 3 , 'Costa Rica' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (50 , 'RS' , '88' , 'SRB' , 3 , 'Serbia' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (51 , 'CU' , '192' , 'CUB' , 3 , 'Cuba' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (52 , 'CV' , '132' , 'CPV' , 3 , 'Cape Verde' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (53 , 'CX' , '162' , 'CXR' , 3 , 'Christmas Island' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (54 , 'CY' , '196' , 'CYP' , 2 , 'Cyprus' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (55 , 'CZ' , '203' , 'CZE' , 2 , 'Czech Republic' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (56 , 'DE' , '276' , 'DEU' , 2 , 'Germany' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (57 , 'DJ' , '262' , 'DJI' , 3 , 'Djibouti' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (58 , 'DK' , '208' , 'DNK' , 2 , 'Denmark' , NULL , NULL , NULL , 'DKK' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (59 , 'DM' , '212' , 'DMA' , 3 , 'Dominica' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (60 , 'DO' , '214' , 'DOM' , 3 , 'Dominican Republic' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (61 , 'DZ' , '012' , 'DZA' , 3 , 'Algeria' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (62 , 'EC' , '218' , 'ECU' , 3 , 'Ecuador' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (63 , 'EE' , '233' , 'EST' , 2 , 'Estonia' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (64 , 'EG' , '818' , 'EGY' , 3 , 'Egypt' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (65 , 'EH' , '732' , 'ESH' , 3 , 'Western Sahara' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (66 , 'ER' , '232' , 'ERI' , 3 , 'Eritrea' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (67 , 'ES' , '724' , 'ESP' , 2 , 'Spain' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (68 , 'ET' , '231' , 'ETH' , 3 , 'Ethiopia' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (69 , 'FI' , '246' , 'FIN' , 2 , 'Finland' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (70 , 'FJ' , '242' , 'FJI' , 3 , 'Fiji' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (71 , 'FK' , '238' , 'FLK' , 3 , 'Falkland Islands' , 'Falkland Islands (Malvinas)' , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (72 , 'FM' , '583' , 'FSM' , 3 , 'Micronesia' , 'Federated States of Micronesia' , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (73 , 'FO' , '234' , 'FRO' , 3 , 'Faroe Islands' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (74 , 'FR' , '250' , 'FRA' , 2 , 'France' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (76 , 'GA' , '266' , 'GAB' , 3 , 'Gabon' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (77 , 'GB' , '826' , 'GBR' , 1 , 'United Kingdom' , NULL , NULL , NULL , 'GBP' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (78 , 'GD' , '308' , 'GRD' , 3 , 'Grenada' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (79 , 'GE' , '268' , 'GEO' , 3 , 'Georgia' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (80 , 'GF' , '254' , 'GUF' , 3 , 'French Guiana' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (81 , 'GG' , '831' , 'GGY' , 3 , 'Guernsey' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (82 , 'GH' , '288' , 'GHA' , 3 , 'Ghana' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (83 , 'GI' , '292' , 'GIB' , 3 , 'Gibraltar' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (84 , 'GL' , '304' , 'GRL' , 3 , 'Greenland' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (85 , 'GM' , '270' , 'GMB' , 3 , 'Gambia' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (86 , 'GN' , '324' , 'GIN' , 3 , 'Guinea' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (87 , 'GP' , '312' , 'GLP' , 3 , 'Guadeloupe' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (88 , 'GQ' , '226' , 'GNQ' , 3 , 'Equatorial Guinea' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (89 , 'GR' , '300' , 'GRC' , 2 , 'Greece' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (90 , 'GS' , '239' , 'SGS' , 3 , 'South Georgia' , 'South Georgia and the South Sandwich Islands' , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (91 , 'GT' , '320' , 'GTM' , 3 , 'Guatemala' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (92 , 'GU' , '316' , 'GUM' , 3 , 'Guam' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (93 , 'GW' , '624' , 'GNB' , 3 , 'Guinea-Bissau' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (94 , 'GY' , '328' , 'GUY' , 3 , 'Guyana' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (95 , 'HK' , '344' , 'HKG' , 3 , 'Hong Kong' , NULL , NULL , NULL , 'HKD' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (96 , 'HM' , '334' , 'HMD' , 3 , 'HIMI' , 'Heard Island and McDonald Islands' , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (97 , 'HN' , '340' , '340' , 3 , 'Honduras' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (98 , 'HR' , '191' , 'HRV' , 3 , 'Croatia' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (99 , 'HT' , '332' , 'HTI' , 3 , 'Haiti' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (100 , 'HU' , '348' , 'HUN' , 2 , 'Hungary' , NULL , NULL , NULL , 'HUF' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (101 , 'ID' , '360' , 'IDN' , 3 , 'Indonesia' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (102 , 'IE' , '372' , 'IRL' , 2 , 'Ireland' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (103 , 'IL' , '376' , 'ISR' , 3 , 'Israel' , NULL , NULL , NULL , 'ILS' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (104 , 'IM' , '833' , 'IMN' , 3 , 'Isle of Man' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (105 , 'IN' , '356' , 'IND' , 3 , 'India' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (106 , 'IO' , '086' , 'IOT' , 3 , 'BIOT' , 'British Indian Ocean Territory' , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (107 , 'IQ' , '368' , 'IRQ' , 3 , 'Iraq' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (108 , 'IR' , '364' , 'IRN' , 3 , 'Iran' , 'Islamic Republic of Iran' , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (109 , 'IS' , '352' , 'ISL' , 3 , 'Iceland' , NULL , NULL , NULL , 'ISK' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (110 , 'IT' , '380' , 'ITA' , 2 , 'Italy' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (111 , 'JE' , '832' , 'JEY' , 3 , 'Jersey' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (112 , 'JM' , '388' , 'JAM' , 3 , 'Jamaica' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (113 , 'JO' , '400' , 'JOR' , 3 , 'Jordan' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (114 , 'JP' , '392' , 'JPN' , 3 , 'Japan' , NULL , NULL , NULL , 'JPY' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (115 , 'KE' , '404' , 'KEN' , 3 , 'Kenya' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (116 , 'KG' , '417' , 'KGZ' , 3 , 'Kyrgyzstan' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (117 , 'KH' , '116' , 'KHM' , 3 , 'Cambodia' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (118 , 'KI' , '296' , 'KIR' , 3 , 'Kiribati' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (119 , 'KM' , '174' , 'COM' , 3 , 'Comoros' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (120 , 'KN' , '659' , 'KNA' , 3 , 'Saint Kitts and Nevis' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (121 , 'KP' , '408' , 'PRK' , 3 , 'Norh Korea' , 'Democratic People\'s Republic of Korea' , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (122 , 'KR' , '410' , 'KOR' , 3 , 'South Korea' , 'Republic of Korea' , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (123 , 'KW' , '414' , 'KWT' , 3 , 'Kuwait' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (124 , 'KY' , '136' , 'CYM' , 3 , 'Cayman Islands' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (125 , 'KZ' , '398' , 'KAZ' , 3 , 'Kazakhstan' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (126 , 'LA' , '418' , 'LAO' , 3 , 'Laos' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (127 , 'LB' , '422' , 'LBN' , 3 , 'Lebanon' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (128 , 'LC' , '662' , 'LCA' , 3 , 'Saint Lucia' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (129 , 'LI' , '438' , 'LIE' , 3 , 'Liechtenstein' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (130 , 'LK' , '144' , 'LKA' , 3 , 'Sri Lanka' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (131 , 'LR' , '430' , 'LBR' , 3 , 'Liberia' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (132 , 'LS' , '426' , 'LSO' , 3 , 'Lesotho' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (133 , 'LT' , '440' , 'LTU' , 2 , 'Lithuania' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (134 , 'LU' , '442' , 'LUX' , 2 , 'Luxembourg' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (135 , 'LV' , '428' , 'LVA' , 2 , 'Latvia' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (136 , 'LY' , '434' , 'LBY' , 3 , 'Libya' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (137 , 'MA' , '504' , '504' , 3 , 'Morocco' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (138 , 'MC' , '492' , 'MCO' , 2 , 'Monaco' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (139 , 'MD' , '498' , 'MDA' , 3 , 'Moldova' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (140 , 'MG' , '450' , 'MDG' , 3 , 'Madagascar' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (141 , 'MH' , '584' , 'MHL' , 3 , 'Marshall Islands' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (142 , 'MK' , '807' , 'MKD' , 3 , 'Macedonia, the FYROM' , 'The former Yugoslav Republic of Macedonia' , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (143 , 'ML' , '466' , 'MLI' , 3 , 'Mali' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (144 , 'MM' , '104' , 'MMR' , 3 , 'Myanmar' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (145 , 'MN' , '496' , 'MNG' , 3 , 'Mongolia' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (146 , 'MO' , '446' , 'MAC' , 3 , 'Macau' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (147 , 'MP' , '580' , 'MNP' , 3 , 'Northern Mariana Islands' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (148 , 'MQ' , '474' , 'MTQ' , 3 , 'Martinique' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (149 , 'MR' , '478' , 'MRT' , 3 , 'Mauritania' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (150 , 'MS' , '500' , '500' , 3 , 'Montserrat' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (151 , 'MT' , '470' , 'MLT' , 2 , 'Malta' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (152 , 'MU' , '480' , 'MUS' , 3 , 'Mauritius' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (153 , 'MV' , '462' , 'MDV' , 3 , 'Maldives' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (154 , 'MW' , '454' , 'MWI' , 3 , 'Malawi' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (155 , 'MX' , '484' , 'MEX' , 3 , 'Mexico' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (156 , 'MY' , '458' , 'MYS' , 3 , 'Malaysia' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (157 , 'MZ' , '508' , '508' , 3 , 'Mozambique' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (158 , 'NA' , '516' , 'NAM' , 3 , 'Namibia' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (159 , 'NC' , '540' , 'NCL' , 3 , 'New Caledonia' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (160 , 'NE' , '562' , 'NER' , 3 , 'Niger' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (161 , 'NF' , '574' , 'NFK' , 3 , 'Norfolk Island' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (162 , 'NG' , '566' , 'NGA' , 3 , 'Nigeria' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (163 , 'NI' , '558' , 'NIC' , 3 , 'Nicaragua' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (164 , 'NL' , '528' , 'NLD' , 2 , 'Netherlands' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (165 , 'NO' , '578' , 'NOR' , 3 , 'Norway' , NULL , NULL , NULL , 'NOK' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (166 , 'NP' , '524' , 'NPL' , 3 , 'Nepal' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (167 , 'NR' , '520' , 'NRU' , 3 , 'Nauru' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (168 , 'NU' , '570' , 'NIU' , 3 , 'Niue' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (169 , 'NZ' , '554' , 'NZL' , 3 , 'New Zealand' , NULL , NULL , NULL , 'NZD' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (170 , 'OM' , '512' , 'OMN' , 3 , 'Oman' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (171 , 'PA' , '591' , 'PAN' , 3 , 'Panama' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (172 , 'PE' , '604' , 'PER' , 3 , 'Peru' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (173 , 'PF' , '258' , 'PYF' , 3 , 'French Polynesia' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (174 , 'PG' , '598' , 'PNG' , 3 , 'Papua New Guinea' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (175 , 'PH' , '608' , 'PHL' , 3 , 'Philippines' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (176 , 'PK' , '586' , 'PAK' , 3 , 'Pakistan' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (177 , 'PL' , '616' , 'POL' , 2 , 'Poland' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (178 , 'PM' , '666' , 'SPM' , 3 , 'Saint Pierre and Miquelon' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (179 , 'PN' , '612' , 'PCN' , 3 , 'Pitcairn' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (180 , 'PR' , '630' , 'PRI' , 3 , 'Puerto Rico' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (181 , 'PS' , '275' , 'PSE' , 3 , 'Palestinian territories' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (182 , 'PT' , '620' , 'PRT' , 2 , 'Portugal' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (183 , 'PW' , '585' , 'PLW' , 3 , 'Palau' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (184 , 'PY' , '600' , 'PRY' , 3 , 'Paraguay' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (185 , 'QA' , '634' , 'QAT' , 3 , 'Qatar' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (186 , 'RE' , '638' , 'REU' , 3 , 'Reunion' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (187 , 'RO' , '642' , 'ROU' , 3 , 'Romania' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (188 , 'RU' , '643' , 'RUS' , 3 , 'Russian Federation' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (189 , 'RW' , '646' , 'RWA' , 3 , 'Rwanda' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (190 , 'SA' , '682' , 'SAU' , 3 , 'Saudi Arabia' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (191 , 'SB' , '090' , 'SLB' , 3 , 'Solomon Islands' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (192 , 'SC' , '690' , 'SYC' , 3 , 'Seychelles' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (193 , 'SD' , '736' , 'SDN' , 3 , 'Sudan' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (194 , 'SE' , '752' , 'SWE' , 2 , 'Sweden' , NULL , NULL , NULL , 'SEK' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (195 , 'SG' , '702' , 'SGP' , 3 , 'Singapore' , NULL , NULL , NULL , 'SGD' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (196 , 'SH' , '654' , 'SHN' , 3 , 'Saint Helena' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (197 , 'SI' , '705' , 'SVN' , 2 , 'Slovenia' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (198 , 'SJ' , '744' , 'SJM' , 3 , 'Svalbard and Jan Mayen' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (199 , 'SK' , '703' , 'SVK' , 2 , 'Slovakia' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (200 , 'SL' , '694' , 'SLE' , 3 , 'Sierra Leone' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (201 , 'SM' , '674' , 'SMR' , 3 , 'San Marino' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (202 , 'SN' , '686' , 'SEN' , 3 , 'Senegal' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (203 , 'SO' , '706' , 'SOM' , 3 , 'Somalia' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (204 , 'SR' , '740' , 'SUR' , 3 , 'Suriname' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (205 , 'ST' , '678' , 'STP' , 3 , 'Sao Tome and Principe' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (206 , 'SV' , '222' , 'SLV' , 3 , 'El Salvador' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (207 , 'SY' , '760' , 'SYR' , 3 , 'Syrian Arab Republic' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (208 , 'SZ' , '748' , 'SWZ' , 3 , 'Swaziland' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (209 , 'TC' , '796' , 'TCA' , 3 , 'Turks and Caicos Islands' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (210 , 'TD' , '148' , 'TCD' , 3 , 'Chad' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (211 , 'TF' , '260' , 'ATF' , 3 , 'French Southern Territories' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (212 , 'TG' , '768' , 'TGO' , 3 , 'Togo' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (213 , 'TH' , '764' , 'THA' , 3 , 'Thailand' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (214 , 'TJ' , '762' , 'TJK' , 3 , 'Tajikistan' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (215 , 'TK' , '772' , 'TKL' , 3 , 'Tokelau' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (216 , 'TL' , '626' , 'TLS' , 3 , 'Timor-Leste' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (217 , 'TM' , '795' , 'TKM' , 3 , 'Turkmenistan' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (218 , 'TN' , '788' , 'TUN' , 3 , 'Tunisia' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (219 , 'TO' , '776' , 'TON' , 3 , 'Tonga' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (220 , 'TR' , '792' , 'TUR' , 3 , 'Turkey' , NULL , NULL , NULL , 'YTL' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (221 , 'TT' , '780' , 'TTO' , 3 , 'Trinidad and Tobago' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (222 , 'TV' , '798' , 'TUV' , 3 , 'Tuvalu' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (223 , 'TW' , '158' , 'TWN' , 3 , 'Taiwan' , 'Province of China Taiwan' , NULL , NULL , 'TWD' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (224 , 'TZ' , '834' , 'TZA' , 3 , 'Tanzania' , 'United Republic of Tanzania' , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (225 , 'UA' , '804' , 'UKR' , 3 , 'Ukraine' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (226 , 'UG' , '800' , 'UGA' , 3 , 'Uganda' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (227 , 'UM' , '581' , 'UMI' , 3 , 'US Minor Outlying Islands' , 'United States Minor Outlying Islands' , NULL , NULL , 'USD' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (228 , 'US' , '840' , 'USA' , 1 , 'United States' , NULL , NULL , NULL , 'USD' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (229 , 'UY' , '858' , 'URY' , 3 , 'Uruguay' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (230 , 'UZ' , '860' , 'UZB' , 3 , 'Uzbekistan' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (231 , 'VA' , '336' , 'VAT' , 3 , 'Vatican City State' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (232 , 'VC' , '670' , 'VCT' , 3 , 'Saint Vincent' , 'Saint Vincent and the Grenadines' , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (233 , 'VE' , '862' , 'VEN' , 3 , 'Venezuela' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (234 , 'VG' , '092' , 'VGB' , 3 , 'Virgin Islands, British' , NULL , NULL , NULL , 'GBP' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (235 , 'VI' , '850' , 'VIR' , 3 , 'Virgin Islands, U.S.' , NULL , NULL , NULL , 'USD' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (236 , 'VN' , '704' , 'VNM' , 3 , 'Vietnam' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (237 , 'VU' , '548' , 'VUT' , 3 , 'Vanuatu' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (238 , 'WF' , '876' , 'WLF' , 3 , 'Wallis And Futuna' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (239 , 'WS' , '882' , 'WSM' , 3 , 'Samoa' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (240 , 'YE' , '887' , 'YEM' , 3 , 'Yemen' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (241 , 'YT' , '175' , 'MYT' , 3 , 'Mayotte' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (242 , 'ZA' , '710' , 'ZAF' , 3 , 'South Africa' , NULL , NULL , NULL , 'ZAR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (243 , 'ZM' , '894' , 'ZMB' , 3 , 'Zambia' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (244 , 'ZW' , '716' , 'ZWE' , 3 , 'Zimbabwe' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (245 , 'ME' , '499' , 'MNE' , 3 , 'Montenegro' , NULL , NULL , NULL , NULL , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (246 , 'BL' , '652' , 'BLM' , 3 , 'Saint Barthelemy' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;
INSERT INTO va_countries (country_id,country_code,country_iso_number,country_code_alpha3,country_order,country_name,full_name,state_field_name,phone_code,currency_code,show_for_user,sites_all) VALUES (247 , 'MF' , '663' , 'MAF' , 3 , 'Saint Martin (French part)' , NULL , NULL , NULL , 'EUR' , 1 , 1 ) ;








INSERT INTO va_credit_cards (credit_card_id,credit_card_code,credit_card_name) VALUES (1 , 'Visa' , 'VISA' ) ;
INSERT INTO va_credit_cards (credit_card_id,credit_card_code,credit_card_name) VALUES (2 , 'Visa' , 'VISA Electron' ) ;
INSERT INTO va_credit_cards (credit_card_id,credit_card_code,credit_card_name) VALUES (3 , 'MC' , 'Mastercard' ) ;
INSERT INTO va_credit_cards (credit_card_id,credit_card_code,credit_card_name) VALUES (4 , 'AMEX' , 'American Express' ) ;
INSERT INTO va_credit_cards (credit_card_id,credit_card_code,credit_card_name) VALUES (5 , 'Switch' , 'Switch' ) ;
INSERT INTO va_credit_cards (credit_card_id,credit_card_code,credit_card_name) VALUES (6 , 'Solo' , 'Solo' ) ;
INSERT INTO va_credit_cards (credit_card_id,credit_card_code,credit_card_name) VALUES (7 , 'JCB' , 'JCB' ) ;
INSERT INTO va_credit_cards (credit_card_id,credit_card_code,credit_card_name) VALUES (8 , 'Delta' , 'Delta' ) ;
INSERT INTO va_credit_cards (credit_card_id,credit_card_code,credit_card_name) VALUES (9 , 'Eurocard' , 'Eurocard' ) ;
INSERT INTO va_credit_cards (credit_card_id,credit_card_code,credit_card_name) VALUES (10 , 'discover' , 'Discover' ) ;


INSERT INTO va_currencies (currency_id,is_default,is_default_show,show_for_user,currency_code,currency_value,currency_title,currency_image,currency_image_active,exchange_rate,symbol_left,symbol_right,decimals_number,decimal_point,thousands_separator,sites_all) VALUES (1 , 1 , 1 , 1 , 'USD' , '840' , 'American Dollar' , 'images/flags/us.gif' , NULL , 1 , '$' , NULL , 2 , '.' , ',' , 1 ) ;
INSERT INTO va_currencies (currency_id,is_default,is_default_show,show_for_user,currency_code,currency_value,currency_title,currency_image,currency_image_active,exchange_rate,symbol_left,symbol_right,decimals_number,decimal_point,thousands_separator,sites_all) VALUES (2 , 0 , 0 , 1 , 'AUD' , '036' , 'Australian Dollar' , 'images/flags/au.gif' , NULL , 1.3245 , '$' , NULL , 2 , '.' , ',' , 1 ) ;
INSERT INTO va_currencies (currency_id,is_default,is_default_show,show_for_user,currency_code,currency_value,currency_title,currency_image,currency_image_active,exchange_rate,symbol_left,symbol_right,decimals_number,decimal_point,thousands_separator,sites_all) VALUES (3 , 0 , 0 , 1 , 'GBP' , '826' , 'British Pound' , 'images/flags/uk.gif' , NULL , 0.548065 , '' , NULL , 2 , '.' , ',' , 1 ) ;
INSERT INTO va_currencies (currency_id,is_default,is_default_show,show_for_user,currency_code,currency_value,currency_title,currency_image,currency_image_active,exchange_rate,symbol_left,symbol_right,decimals_number,decimal_point,thousands_separator,sites_all) VALUES (4 , 0 , 0 , 1 , 'CAD' , '124' , 'Canadian Dollar' , 'images/flags/ca.gif' , NULL , 1.2627 , '$' , NULL , 2 , '.' , ',' , 1 ) ;
INSERT INTO va_currencies (currency_id,is_default,is_default_show,show_for_user,currency_code,currency_value,currency_title,currency_image,currency_image_active,exchange_rate,symbol_left,symbol_right,decimals_number,decimal_point,thousands_separator,sites_all) VALUES (5 , 0 , 0 , 0 , 'DKK' , '208' , 'Danish Krone' , 'images/flags/dk.gif' , NULL , 5.9324 , 'DKK' , NULL , 2 , '.' , ',' , 1 ) ;
INSERT INTO va_currencies (currency_id,is_default,is_default_show,show_for_user,currency_code,currency_value,currency_title,currency_image,currency_image_active,exchange_rate,symbol_left,symbol_right,decimals_number,decimal_point,thousands_separator,sites_all) VALUES (6 , 0 , 0 , 1 , 'EUR' , '978' , 'Euro' , 'images/flags/eu.gif' , NULL , 0.796622 , '' , NULL , 2 , '.' , ',' , 1 ) ;
INSERT INTO va_currencies (currency_id,is_default,is_default_show,show_for_user,currency_code,currency_value,currency_title,currency_image,currency_image_active,exchange_rate,symbol_left,symbol_right,decimals_number,decimal_point,thousands_separator,sites_all) VALUES (7 , 0 , 0 , 0 , 'HKD' , '344' , 'Hong Kong Dollar' , 'images/flags/hk.gif' , NULL , 7.7905 , 'HK$' , NULL , 2 , '.' , ',' , 1 ) ;
INSERT INTO va_currencies (currency_id,is_default,is_default_show,show_for_user,currency_code,currency_value,currency_title,currency_image,currency_image_active,exchange_rate,symbol_left,symbol_right,decimals_number,decimal_point,thousands_separator,sites_all) VALUES (8 , 0 , 0 , 0 , 'HUF' , '348' , 'Hungarian Forint' , 'images/flags/hu.gif' , NULL , 190.54 , 'HUF' , NULL , 2 , '.' , ',' , 1 ) ;
INSERT INTO va_currencies (currency_id,is_default,is_default_show,show_for_user,currency_code,currency_value,currency_title,currency_image,currency_image_active,exchange_rate,symbol_left,symbol_right,decimals_number,decimal_point,thousands_separator,sites_all) VALUES (9 , 0 , 0 , 0 , 'ISK' , '352' , 'Iceland Krona' , 'images/flags/is.gif' , NULL , 61.15 , 'ISK' , NULL , 2 , '.' , ',' , 1 ) ;
INSERT INTO va_currencies (currency_id,is_default,is_default_show,show_for_user,currency_code,currency_value,currency_title,currency_image,currency_image_active,exchange_rate,symbol_left,symbol_right,decimals_number,decimal_point,thousands_separator,sites_all) VALUES (10 , 0 , 0 , 0 , 'ILS' , '376' , 'Israeli Shekel' , 'images/flags/il.gif' , NULL , 4.366 , 'ILS' , NULL , 2 , '.' , ',' , 1 ) ;
INSERT INTO va_currencies (currency_id,is_default,is_default_show,show_for_user,currency_code,currency_value,currency_title,currency_image,currency_image_active,exchange_rate,symbol_left,symbol_right,decimals_number,decimal_point,thousands_separator,sites_all) VALUES (11 , 0 , 0 , 0 , 'JPY' , '392' , 'Japanese Yen' , 'images/flags/jp.gif' , NULL , 108.17 , 'JPY' , NULL , 2 , '.' , ',' , 1 ) ;
INSERT INTO va_currencies (currency_id,is_default,is_default_show,show_for_user,currency_code,currency_value,currency_title,currency_image,currency_image_active,exchange_rate,symbol_left,symbol_right,decimals_number,decimal_point,thousands_separator,sites_all) VALUES (12 , 0 , 0 , 0 , 'NZD' , '554' , 'New Zealand Dollar' , 'images/flags/nz.gif' , NULL , 1.41203 , '$' , NULL , 2 , '.' , ',' , 1 ) ;
INSERT INTO va_currencies (currency_id,is_default,is_default_show,show_for_user,currency_code,currency_value,currency_title,currency_image,currency_image_active,exchange_rate,symbol_left,symbol_right,decimals_number,decimal_point,thousands_separator,sites_all) VALUES (13 , 0 , 0 , 0 , 'NOK' , '578' , 'Norwegian Kroner' , 'images/flags/no.gif' , NULL , 6.4656 , 'NOK' , NULL , 2 , '.' , ',' , 1 ) ;
INSERT INTO va_currencies (currency_id,is_default,is_default_show,show_for_user,currency_code,currency_value,currency_title,currency_image,currency_image_active,exchange_rate,symbol_left,symbol_right,decimals_number,decimal_point,thousands_separator,sites_all) VALUES (14 , 0 , 0 , 0 , 'SGD' , '702' , 'Singapore Dollar' , 'images/flags/sg.gif' , NULL , 1.6635 , 'SG$' , NULL , 2 , '.' , ',' , 1 ) ;
INSERT INTO va_currencies (currency_id,is_default,is_default_show,show_for_user,currency_code,currency_value,currency_title,currency_image,currency_image_active,exchange_rate,symbol_left,symbol_right,decimals_number,decimal_point,thousands_separator,sites_all) VALUES (15 , 0 , 0 , 0 , 'ZAR' , '710' , 'South African Rand' , 'images/flags/za.gif' , NULL , 6.5415 , 'ZAR' , NULL , 2 , '.' , ',' , 1 ) ;
INSERT INTO va_currencies (currency_id,is_default,is_default_show,show_for_user,currency_code,currency_value,currency_title,currency_image,currency_image_active,exchange_rate,symbol_left,symbol_right,decimals_number,decimal_point,thousands_separator,sites_all) VALUES (16 , 0 , 0 , 0 , 'SEK' , '752' , 'Swedish Krona SEK' , 'images/flags/se.gif' , NULL , 7.3069 , 'SEK' , NULL , 2 , '.' , ',' , 1 ) ;
INSERT INTO va_currencies (currency_id,is_default,is_default_show,show_for_user,currency_code,currency_value,currency_title,currency_image,currency_image_active,exchange_rate,symbol_left,symbol_right,decimals_number,decimal_point,thousands_separator,sites_all) VALUES (17 , 0 , 0 , 0 , 'CHF' , '756' , 'Swiss Franc' , 'images/flags/ch.gif' , NULL , 1.2335 , 'CHF' , NULL , 2 , '.' , ',' , 1 ) ;
INSERT INTO va_currencies (currency_id,is_default,is_default_show,show_for_user,currency_code,currency_value,currency_title,currency_image,currency_image_active,exchange_rate,symbol_left,symbol_right,decimals_number,decimal_point,thousands_separator,sites_all) VALUES (18 , 0 , 0 , 0 , 'TWD' , '901' , 'Taiwan Dollar' , 'images/flags/tw.gif' , NULL , 31.47 , 'TW$' , NULL , 2 , '.' , ',' , 1 ) ;
INSERT INTO va_currencies (currency_id,is_default,is_default_show,show_for_user,currency_code,currency_value,currency_title,currency_image,currency_image_active,exchange_rate,symbol_left,symbol_right,decimals_number,decimal_point,thousands_separator,sites_all) VALUES (19 , 0 , 0 , 0 , 'TRY' , '949' , 'Turkish Lira' , 'images/flags/tr.gif' , NULL , 1.2935 , NULL , 'YTL' , 2 , '.' , ',' , 1 ) ;




INSERT INTO va_custom_blocks (block_id,block_name,block_notes,block_class,block_title,block_path,block_desc) VALUES (2 , 'Custom' , NULL , NULL , 'Custom Block' , NULL , 'Feel free to add any text here' ) ;
INSERT INTO va_custom_blocks (block_id,block_name,block_notes,block_class,block_title,block_path,block_desc) VALUES (3 , 'Categories' , NULL , NULL , 'All categories' , NULL , '<br>Asia\r\n<br>Europe\r\n<br>U.S.\r\n<br>WorldWorld \r\n<br>Business\r\n<br>Technology\r\n<br>Science & Space\r\n<br>Entertainment\r\n<br>World Sport\r\n<br>Travel\r\n<br>Executive\r\n<br>Education\r\n<br>Weather\r\n<br>Special Reports\r\n<br>Video\r\n\r\n' ) ;
INSERT INTO va_custom_blocks (block_id,block_name,block_notes,block_class,block_title,block_path,block_desc) VALUES (4 , 'Currency Converter' , NULL , NULL , 'Currency Converter' , NULL , '<form id="frConverter">\r\n<table border="0" cellpadding="3" cellspacing="0">\r\n<tr>\r\n  <td><b>I Have</b></td>\r\n</tr>\r\n<tr>\r\n  <td>\r\n    <select name="base_currency" onKeyUp="frExCalc();" onChange="frExCalc();" style="width: 180px;"></select>\r\n    <input type="hidden" name="default_base" value="usd"></select>\r\n  </td>\r\n</tr>\r\n<tr>\r\n  <td><input onKeyUp="frExCalc();" type="text" name="base_value" style="width: 175px;" value="1"></td>\r\n</tr>\r\n<tr>\r\n  <td><br><b>I Want</b></td>\r\n</tr>\r\n<tr>\r\n  <td>\r\n    <select name="target_currency" onKeyUp="frExCalc();" onChange="frExCalc();" style="width: 180px;"></select>\r\n    <input type="hidden" name="default_target" value="eur"></select>\r\n  </td>\r\n</tr>\r\n<tr>\r\n  <td><input onKeyUp="frExCalc(true);" type="text" name="target_value" style="width: 175px;" value=""></td>\r\n</tr>\r\n</table>\r\n\r\n<br><a id="frLink" href="http://www.floatrates.com/">Currency Converter</a>\r\n</form>\r\n<script type="text/javascript" src="http://www.floatrates.com/scripts/converter.js"></script>' ) ;


INSERT INTO va_ethnicities (ethnicity_id,ethnicity_name,sort_order,show_for_user) VALUES (1 , 'Caucasian/White' , 1 , 1 ) ;
INSERT INTO va_ethnicities (ethnicity_id,ethnicity_name,sort_order,show_for_user) VALUES (2 , 'Black' , 2 , 1 ) ;
INSERT INTO va_ethnicities (ethnicity_id,ethnicity_name,sort_order,show_for_user) VALUES (3 , 'Hispanic' , 3 , 1 ) ;
INSERT INTO va_ethnicities (ethnicity_id,ethnicity_name,sort_order,show_for_user) VALUES (4 , 'Middle Eastern' , 4 , 1 ) ;
INSERT INTO va_ethnicities (ethnicity_id,ethnicity_name,sort_order,show_for_user) VALUES (5 , 'Asian' , 5 , 1 ) ;
INSERT INTO va_ethnicities (ethnicity_id,ethnicity_name,sort_order,show_for_user) VALUES (6 , 'Indian' , 6 , 1 ) ;
INSERT INTO va_ethnicities (ethnicity_id,ethnicity_name,sort_order,show_for_user) VALUES (7 , 'Other Ethnicity' , 7 , 1 ) ;








INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (1 , 1 , 4 , 0 , NULL , 'Front Side Bus Speed' , '533MHz ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (2 , 1 , 4 , 0 , NULL , 'L2 Cache ' , '2MB ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (3 , 1 , 4 , 0 , NULL , 'Speed ' , '1.33GHz' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (4 , 1 , 4 , 0 , NULL , 'Technology ' , 'Intel Centrino Duo Processor Technology ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (5 , 1 , 4 , 0 , NULL , 'Type ' , 'Intel Core 2 Duo Processor U7700 ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (6 , 1 , 5 , 0 , NULL , 'Screen Size ' , '11.1' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (7 , 1 , 5 , 0 , NULL , 'XBRITE-HiColor Technology' , 'Yes ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (8 , 1 , 5 , 0 , NULL , 'Technology ' , 'WXGA ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (9 , 1 , 5 , 0 , NULL , 'Resolution ' , '1366 x 768 ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (10 , 1 , 6 , 0 , NULL , 'Interface ' , 'Serial ATA ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (11 , 1 , 6 , 0 , NULL , 'Speed ' , '5400rpm ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (12 , 1 , 6 , 0 , NULL , 'Capacity ' , '320GB' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (13 , 1 , 7 , 0 , NULL , 'Maximum ' , '2GB ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (14 , 1 , 7 , 0 , NULL , 'Speed ' , '533MHz ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (15 , 1 , 7 , 0 , NULL , 'Type ' , 'DDR2 ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (16 , 1 , 7 , 0 , NULL , 'Installed ' , '2GB PC2-4200 (2GBx1)' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (17 , 3 , 4 , 0 , NULL , 'Front Side Bus Speed' , '1066MHz ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (18 , 3 , 4 , 0 , NULL , 'L2 Cache ' , '3MB ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (19 , 3 , 4 , 0 , NULL , 'Speed ' , '2.26GHz' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (20 , 3 , 4 , 0 , NULL , 'Technology ' , 'Intel Centrino 2 processor technology  ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (21 , 3 , 4 , 0 , NULL , 'Type ' , 'Intel Core 2 Duo Processor P8400  ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (22 , 3 , 5 , 0 , NULL , 'Screen Size ' , '16.4' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (23 , 3 , 5 , 0 , NULL , 'XBRITE-HiColor Technology' , 'Yes ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (25 , 3 , 5 , 0 , NULL , 'Resolution ' , '1600 x 900 ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (26 , 3 , 6 , 0 , NULL , 'Interface ' , 'Serial ATA ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (27 , 3 , 6 , 0 , NULL , 'Speed ' , '5400rpm ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (28 , 3 , 6 , 0 , NULL , 'Capacity ' , '320GB' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (29 , 3 , 7 , 0 , NULL , 'Maximum ' , '4GB' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (30 , 3 , 7 , 0 , NULL , 'Speed ' , '800MHz ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (31 , 3 , 7 , 0 , NULL , 'Type ' , 'DDR2 SDRAM  ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (32 , 3 , 7 , 0 , NULL , 'Installed ' , '3GB PC2-6400 (2GBx1 + 1GBx1)   ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (33 , 4 , 4 , 0 , NULL , 'Front Side Bus Speed' , '1066MHz ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (34 , 4 , 4 , 0 , NULL , 'L2 Cache ' , '6MB ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (35 , 4 , 4 , 0 , NULL , 'Speed ' , '2.53GHz' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (36 , 4 , 4 , 0 , NULL , 'Technology ' , 'Intel Centrino 2 processor technology ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (37 , 4 , 4 , 0 , NULL , 'Type ' , 'Intel Core 2 Duo Processor P9500 ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (38 , 4 , 5 , 0 , NULL , 'Screen Size ' , '13.1' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (39 , 4 , 5 , 0 , NULL , 'XBRITE-HiColor Technology' , 'Yes ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (40 , 4 , 5 , 0 , NULL , 'Technology ' , 'LED ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (41 , 4 , 5 , 0 , NULL , 'Resolution ' , '1600 x 900 ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (42 , 4 , 6 , 0 , NULL , 'Interface ' , 'Ultra ATA with RAID technology ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (44 , 4 , 6 , 0 , NULL , 'Capacity ' , '128GB (64GBx2)' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (45 , 4 , 7 , 0 , NULL , 'Maximum ' , '4GB' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (46 , 4 , 7 , 0 , NULL , 'Speed ' , '1066MHz ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (47 , 4 , 7 , 0 , NULL , 'Type ' , 'DDR3 SDRAM ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (48 , 4 , 7 , 0 , NULL , 'Installed ' , '4GB PC3-8500 (2GBx2) ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (49 , 5 , 4 , 0 , NULL , 'Front Side Bus Speed' , '800MHz ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (50 , 5 , 4 , 0 , NULL , 'L2 Cache ' , '2MB ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (51 , 5 , 4 , 0 , NULL , 'Speed ' , '2.50GHz' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (52 , 5 , 4 , 0 , NULL , 'Technology ' , 'Intel Centrino 2 processor technology ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (53 , 5 , 4 , 0 , NULL , 'Type ' , 'Intel Pentium Dual-Core Processor E5200 ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (54 , 5 , 5 , 0 , NULL , 'Screen Size ' , '20.1' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (55 , 5 , 5 , 0 , NULL , 'XBRITE-HiColor Technology' , 'Yes ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (56 , 5 , 5 , 0 , NULL , 'Technology ' , 'WSXGA+' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (57 , 5 , 5 , 0 , NULL , 'Resolution ' , '1680 x 1050 ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (58 , 5 , 6 , 0 , NULL , 'Interface ' , 'Serial ATA  ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (59 , 5 , 6 , 0 , NULL , 'Capacity ' , '500GB' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (60 , 5 , 6 , 0 , NULL , 'Speed ' , '7200rpm ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (61 , 5 , 7 , 0 , NULL , 'Maximum ' , '4GB ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (62 , 5 , 7 , 0 , NULL , 'Speed ' , '800MHz ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (63 , 5 , 7 , 0 , NULL , 'Type ' , 'DDR2 ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (64 , 5 , 7 , 0 , NULL , 'Installed ' , '4GB PC2-6400 (2GBx2)   ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (65 , 6 , 4 , 0 , NULL , 'Front Side Bus Speed' , '1066MHz ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (66 , 6 , 4 , 0 , NULL , 'L2 Cache ' , '3MB ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (67 , 6 , 4 , 0 , NULL , 'Speed ' , ' 2.53GHz' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (69 , 6 , 4 , 0 , NULL , 'Type ' , 'Intel Core 2 Duo Processor E7200   ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (70 , 6 , 5 , 0 , NULL , 'Screen Size ' , '24.0' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (71 , 6 , 5 , 0 , NULL , 'XBRITE-HiColor Technology' , 'Yes ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (72 , 6 , 5 , 0 , NULL , 'Technology ' , 'WUXGA ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (73 , 6 , 5 , 0 , NULL , 'Resolution ' , '1920 x 1200 ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (74 , 6 , 6 , 0 , NULL , 'Interface ' , 'Serial ATA  ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (75 , 6 , 6 , 0 , NULL , 'Capacity ' , '500GB' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (76 , 6 , 6 , 0 , NULL , 'Speed ' , '7200rpm ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (77 , 6 , 7 , 0 , NULL , 'Maximum ' , '4GB ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (78 , 6 , 7 , 0 , NULL , 'Speed ' , '800MHz ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (79 , 6 , 7 , 0 , NULL , 'Type ' , 'DDR2 ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (80 , 6 , 7 , 0 , NULL , 'Installed ' , '4GB PC2-6400 (2GBx2)   ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (81 , 7 , 4 , 0 , NULL , 'Front Side Bus Speed' , '800MHz ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (82 , 7 , 4 , 0 , NULL , 'L2 Cache ' , '3MB ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (83 , 7 , 4 , 0 , NULL , 'Speed ' , '2.40GHz' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (84 , 7 , 4 , 0 , NULL , 'Type ' , 'Intel Core 2 Duo Processor T8300 ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (85 , 7 , 5 , 0 , NULL , 'Screen Size ' , '22"' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (86 , 7 , 5 , 0 , NULL , 'XBRITE-HiColor Technology' , 'Yes ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (87 , 7 , 5 , 0 , NULL , 'Technology ' , 'WSXGA+  ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (88 , 7 , 5 , 0 , NULL , 'Resolution ' , '1680 x 1050  ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (89 , 7 , 6 , 0 , NULL , 'Interface ' , 'Serial ATA ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (90 , 7 , 6 , 0 , NULL , 'Capacity ' , '1TB (500GBx2)' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (91 , 7 , 6 , 0 , NULL , 'Speed ' , '7200rpm ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (92 , 7 , 7 , 0 , NULL , 'Maximum ' , '4GB ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (93 , 7 , 7 , 0 , NULL , 'Speed ' , '667MHz ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (94 , 7 , 7 , 0 , NULL , 'Type ' , 'DDR2 ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (95 , 7 , 7 , 0 , NULL , 'Installed ' , '4GB PC2-5300 (2GBx2)   ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (96 , 8 , 4 , 0 , NULL , 'Front Side Bus Speed' , '1333 MHz ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (97 , 8 , 4 , 0 , NULL , 'L2 Cache ' , '2MB ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (98 , 8 , 4 , 0 , NULL , 'Speed ' , '2.33 GHz' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (99 , 8 , 4 , 0 , NULL , 'Type ' , 'Intel Core 2 Duo Processor E6550 ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (104 , 8 , 6 , 0 , NULL , 'Interface ' , 'Serial ATA ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (105 , 8 , 6 , 0 , NULL , 'Capacity ' , '500GB' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (106 , 8 , 6 , 0 , NULL , 'Speed ' , '7200rpm ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (107 , 8 , 7 , 0 , NULL , 'Maximum ' , '4GB ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (108 , 8 , 7 , 0 , NULL , 'Speed ' , '667 MHz ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (109 , 8 , 7 , 0 , NULL , 'Type ' , 'DDR2 SDRAM - Non-ECC ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (110 , 8 , 7 , 0 , NULL , 'Installed ' , '2GB (1GBx2) ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (111 , 8 , 5 , 0 , NULL , 'Screen Size ' , '20' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (112 , 8 , 5 , 0 , NULL , 'Resolution' , '1680 x 1050 ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (115 , 9 , 4 , 0 , NULL , 'Speed ' , '2.1 GHz' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (116 , 9 , 4 , 0 , NULL , 'Type ' , 'Intel Core 2 Duo Processor T8100' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (117 , 9 , 6 , 0 , NULL , 'Interface ' , 'Serial ATA ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (118 , 9 , 6 , 0 , NULL , 'Capacity ' , '250GB' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (119 , 9 , 6 , 0 , NULL , 'Speed ' , '5400rpm ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (120 , 9 , 7 , 0 , NULL , 'Maximum ' , '4GB ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (121 , 9 , 7 , 0 , NULL , 'Speed ' , '667 MHz ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (122 , 9 , 7 , 0 , NULL , 'Type ' , 'DDR2 SDRAM' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (123 , 9 , 7 , 0 , NULL , 'Installed ' , '2GB' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (126 , 10 , 4 , 0 , NULL , 'Front Side Bus Speed' , '533 MHz ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (128 , 10 , 4 , 0 , NULL , 'Speed ' , '2.53GHz' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (130 , 10 , 4 , 0 , NULL , 'Type ' , 'Intel Core 2 Solo U2100 ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (131 , 10 , 5 , 0 , NULL , 'Screen Size ' , '12.1' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (133 , 10 , 5 , 0 , NULL , 'Technology ' , 'WXGA ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (134 , 10 , 5 , 0 , NULL , 'Resolution ' , '1280 x 800 ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (135 , 10 , 6 , 0 , NULL , 'Speed' , '4200' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (136 , 10 , 6 , 0 , NULL , 'Capacity ' , '40GB' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (139 , 10 , 7 , 0 , NULL , 'Type ' , 'DDR2 SDRAM ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (140 , 10 , 7 , 0 , NULL , 'Installed ' , '2GB' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (141 , 11 , 4 , 0 , NULL , 'Front Side Bus Speed' , '800 MHz ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (142 , 11 , 4 , 0 , NULL , 'Speed ' , '2.4 GHz' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (143 , 11 , 4 , 0 , NULL , 'Type ' , 'Intel Core 2 Duo T8300 ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (144 , 11 , 5 , 0 , NULL , 'Screen Size ' , '17' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (145 , 11 , 5 , 0 , NULL , 'Technology ' , 'WUXGA' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (146 , 11 , 5 , 0 , NULL , 'Resolution ' , '1920 x 1200 ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (147 , 11 , 6 , 0 , NULL , 'Speed' , '7200' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (148 , 11 , 6 , 0 , NULL , 'Capacity ' , '200GB' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (149 , 11 , 7 , 0 , NULL , 'Type ' , 'DDR2 SDRAM ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (150 , 11 , 7 , 0 , NULL , 'Installed ' , '4GB' , 1 , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_features (feature_id,item_id,group_id,google_base_attribute_id,feature_code,feature_name,feature_value,show_as_group,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (151 , 11 , 7 , 0 , NULL , 'Speed' , '667 MHz ' , 1 , NULL , NULL , NULL , NULL , NULL ) ;




INSERT INTO va_features_groups (group_id,group_order,group_name,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (4 , 1 , 'Processor' , 0 , 1 , 0 , 0 , 0 ) ;
INSERT INTO va_features_groups (group_id,group_order,group_name,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (5 , 2 , 'Display' , 0 , 1 , 0 , 0 , 0 ) ;
INSERT INTO va_features_groups (group_id,group_order,group_name,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (6 , 3 , 'Hard Drive' , 0 , 1 , 0 , 0 , 0 ) ;
INSERT INTO va_features_groups (group_id,group_order,group_name,show_on_table,show_on_details,show_on_basket,show_on_checkout,show_on_invoice) VALUES (7 , 4 , 'Memory' , 0 , 1 , 0 , 0 , 0 ) ;




INSERT INTO va_file_types (file_type_id,type_order,type_name,extension_mask) VALUES (1 , 10 , 'Other' , NULL ) ;
INSERT INTO va_file_types (file_type_id,type_order,type_name,extension_mask) VALUES (2 , 1 , 'Audio' , '*.mp3,*.mp2,*.midi,*.mid,*.wave,*.wav,*.mpa,' ) ;
INSERT INTO va_file_types (file_type_id,type_order,type_name,extension_mask) VALUES (3 , 2 , 'Video' , '*.flv,*.avi,*.asf,*.wmv,*.vma,*.mpeg,*.mpg,*.mov' ) ;
INSERT INTO va_file_types (file_type_id,type_order,type_name,extension_mask) VALUES (4 , 3 , 'Picture' , '*.gif,*.jpg,*.jpeg,*.bmp,*.tiff,*.tif,*.png,*.ico,*.psd' ) ;


INSERT INTO va_filters (filter_id,filter_type,filter_name,filter_desc) VALUES (1 , 'products' , 'Default Filter' , 'Default Filter' ) ;


INSERT INTO va_filters_properties (property_id,filter_id,property_order,property_name,property_type,property_value,filter_from_sql,filter_join_sql,filter_where_sql,list_group_fields,list_group_where,list_table,list_field_id,list_field_title,list_field_total,list_sql) VALUES (1 , 1 , 1 , 'Manufacturer:' , 'manufacturer' , NULL , NULL , NULL , ' i.manufacturer_id={value_id} ' , 'i.manufacturer_id' , NULL , 'va_manufacturers' , 'manufacturer_id' , 'manufacturer_name' , NULL , NULL ) ;
INSERT INTO va_filters_properties (property_id,filter_id,property_order,property_name,property_type,property_value,filter_from_sql,filter_join_sql,filter_where_sql,list_group_fields,list_group_where,list_table,list_field_id,list_field_title,list_field_total,list_sql) VALUES (2 , 1 , 2 , 'Price:' , 'product_price' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_filters_properties (property_id,filter_id,property_order,property_name,property_type,property_value,filter_from_sql,filter_join_sql,filter_where_sql,list_group_fields,list_group_where,list_table,list_field_id,list_field_title,list_field_total,list_sql) VALUES (3 , 1 , 3 , 'Type:' , 'product_type' , NULL , NULL , NULL , ' i.item_type_id={value_id} ' , 'i.item_type_id' , NULL , 'va_item_types' , 'item_type_id' , 'item_type_name' , NULL , NULL ) ;
INSERT INTO va_filters_properties (property_id,filter_id,property_order,property_name,property_type,property_value,filter_from_sql,filter_join_sql,filter_where_sql,list_group_fields,list_group_where,list_table,list_field_id,list_field_title,list_field_total,list_sql) VALUES (4 , 1 , 4 , 'Rating:' , 'product_price' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;


INSERT INTO va_filters_properties_values (value_id,property_id,value_order,list_value_id,list_value_title,filter_where_sql) VALUES (1 , 2 , NULL , NULL , '<10' , 'i.price<10' ) ;
INSERT INTO va_filters_properties_values (value_id,property_id,value_order,list_value_id,list_value_title,filter_where_sql) VALUES (2 , 2 , NULL , NULL , '10-50' , 'i.price>=10 and i.price<=50' ) ;
INSERT INTO va_filters_properties_values (value_id,property_id,value_order,list_value_id,list_value_title,filter_where_sql) VALUES (3 , 2 , NULL , NULL , '51-100' , 'i.price>=51 and i.price<=100' ) ;
INSERT INTO va_filters_properties_values (value_id,property_id,value_order,list_value_id,list_value_title,filter_where_sql) VALUES (4 , 2 , NULL , NULL , '101-200' , 'i.price>=101 and i.price<=200' ) ;
INSERT INTO va_filters_properties_values (value_id,property_id,value_order,list_value_id,list_value_title,filter_where_sql) VALUES (5 , 2 , NULL , NULL , '201-300' , 'i.price>=201 and i.price<=300' ) ;
INSERT INTO va_filters_properties_values (value_id,property_id,value_order,list_value_id,list_value_title,filter_where_sql) VALUES (6 , 2 , NULL , NULL , '301-400' , 'i.price>=301 and i.price<=400' ) ;
INSERT INTO va_filters_properties_values (value_id,property_id,value_order,list_value_id,list_value_title,filter_where_sql) VALUES (7 , 2 , NULL , NULL , '401-500' , 'i.price>=401 and i.price<=500' ) ;
INSERT INTO va_filters_properties_values (value_id,property_id,value_order,list_value_id,list_value_title,filter_where_sql) VALUES (8 , 2 , NULL , NULL , '501-1000' , 'i.price>=501 and i.price<=1000' ) ;
INSERT INTO va_filters_properties_values (value_id,property_id,value_order,list_value_id,list_value_title,filter_where_sql) VALUES (9 , 2 , NULL , NULL , '1001-2000' , 'i.price>=1001 and i.price<=2000' ) ;
INSERT INTO va_filters_properties_values (value_id,property_id,value_order,list_value_id,list_value_title,filter_where_sql) VALUES (10 , 2 , NULL , NULL , '2001-3000' , 'i.price>=2001 and i.price<=3000' ) ;
INSERT INTO va_filters_properties_values (value_id,property_id,value_order,list_value_id,list_value_title,filter_where_sql) VALUES (11 , 2 , NULL , NULL , '3001-5000' , 'i.price>=3001 and i.price<=5000' ) ;
INSERT INTO va_filters_properties_values (value_id,property_id,value_order,list_value_id,list_value_title,filter_where_sql) VALUES (12 , 2 , NULL , NULL , '>5000' , 'i.price>5000' ) ;
INSERT INTO va_filters_properties_values (value_id,property_id,value_order,list_value_id,list_value_title,filter_where_sql) VALUES (13 , 4 , 3 , NULL , 'Average' , 'i.rating>=2.5 and i.rating<3.5' ) ;
INSERT INTO va_filters_properties_values (value_id,property_id,value_order,list_value_id,list_value_title,filter_where_sql) VALUES (14 , 4 , 4 , NULL , 'Good' , 'i.rating>=3.5 and i.rating<4.5' ) ;
INSERT INTO va_filters_properties_values (value_id,property_id,value_order,list_value_id,list_value_title,filter_where_sql) VALUES (15 , 4 , 5 , NULL , 'Excelent' , 'i.rating>=4.5' ) ;
INSERT INTO va_filters_properties_values (value_id,property_id,value_order,list_value_id,list_value_title,filter_where_sql) VALUES (16 , 4 , 2 , NULL , 'Poor' , 'i.rating>=1.5 and i.rating<2.5' ) ;
INSERT INTO va_filters_properties_values (value_id,property_id,value_order,list_value_id,list_value_title,filter_where_sql) VALUES (17 , 4 , 1 , NULL , 'Bad' , 'i.rating<1.5' ) ;


INSERT INTO va_forum (thread_id,forum_id,priority_id,user_id,user_name,user_email,remote_address,topic,friendly_url,description,email_notification,views,replies,date_added,date_modified,thread_updated,admin_id_added_by,admin_id_modified_by,last_post_added,last_post_user_id,last_post_admin_id,last_post_message_id) VALUES (1 , 1 , NULL , 1 , 'joy' , 'jnothing@yahoo.com' , '127.0.0.1' , 'Email problem' , '' , 'Is there a problem with the email? I don\'t seem to be getting emails when they are sent to me. ' , 0 , 2 , 2 , '2006-12-26 16:59:31' , '2006-12-26 17:05:13' , '2006-12-26 17:05:13' , 0 , 0 , NULL , NULL , NULL , NULL ) ;




INSERT INTO va_forum_categories (category_id,category_name,friendly_url,category_order,short_description,full_description,allowed_view,sites_all) VALUES (1 , 'General' , NULL , 1 , 'General category' , 'General category' , 1 , 1 ) ;




INSERT INTO va_forum_list (forum_id,category_id,forum_name,friendly_url,forum_order,short_description,full_description,small_image,large_image,date_added,threads_number,messages_number,last_post_added,last_post_user_id,last_post_admin_id,last_post_thread_id,last_post_message_id,is_rss,rss_limit,rss_on_breadcrumb,rss_on_list,access_level,guest_access_level,admin_access_level) VALUES (1 , 1 , 'Forum' , '' , 1 , 'General discussions' , NULL , NULL , NULL , '2006-05-23 00:00:00' , 1 , 2 , '2006-12-26 17:05:13' , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , 63 , 31 , 7 ) ;


INSERT INTO va_forum_messages (message_id,thread_id,admin_id,user_name,user_email,remote_address,message_text,date_added,date_modified,user_id,admin_id_modified_by) VALUES (1 , 1 , 1 , 'Site Administrator' , 'admin@yoursite.com' , '127.0.0.1' , '>Is there a problem with the email? I don\'t seem to be getting emails when they are sent to me. \r\n\r\nthere has NOT been a problem with the email notification in general, but I know that there are some problems with AOL accounts as they consider the emails as spam, so please check your anti-spam settings you can always call our office to find more if there has been some servers\' issues.' , '2006-12-26 17:03:01' , NULL , 0 , NULL ) ;
INSERT INTO va_forum_messages (message_id,thread_id,admin_id,user_name,user_email,remote_address,message_text,date_added,date_modified,user_id,admin_id_modified_by) VALUES (2 , 1 , NULL , 'joy' , 'jnothing@yahoo.com' , '127.0.0.1' , 'As of today 04/03 I havent gotten any copies of email confirmations that I get after a customer places an order like I normally do and I dont have aol.' , '2006-12-26 17:05:13' , NULL , 1 , NULL ) ;




INSERT INTO va_forum_priorities (priority_id,priority_name,priority_rank,html_before_title,html_after_title,is_default) VALUES (1 , 'Normal' , 3 , NULL , NULL , 1 ) ;
INSERT INTO va_forum_priorities (priority_id,priority_name,priority_rank,html_before_title,html_after_title,is_default) VALUES (2 , 'Important' , 1 , '<b>Important:</b>' , NULL , 0 ) ;










INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_images' , 'image_large_height' , '300' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_images' , 'image_large_size' , '50000' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_images' , 'image_large_width' , '300' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_images' , 'image_small_height' , '100' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_images' , 'image_small_size' , '16000' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_images' , 'image_small_width' , '100' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_images' , 'images_number' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_notify' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_notify' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_notify' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_notify' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_notify' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_notify' , 'admin_message' , 'ID: {item_id}\r\nTitle: {item_title}\r\nURL: {site_url}ads_details.php?item_id={item_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_notify' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_notify' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_notify' , 'admin_subject' , 'Ad modifications #{item_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_notify' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_notify' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_notify' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_notify' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_notify' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_notify' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_notify' , 'user_message' , 'Dear {seller_name},\r\n\r\nOnce our editors have reviewed and approved your Ad we will post it on our web site.\r\n\r\nThank you.\r\nCustomer Services\r\n' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_notify' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_notify' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_notify' , 'user_subject' , 'Your ad \'{item_title}\' was received.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_request' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_request' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_request' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_request' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_request' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_request' , 'admin_message' , 'Title: {item_title} \r\nPrice: {offer_price}\r\nUser ID: {user_id}\r\nName: {user_name}\r\nEmail: {user_email}\r\n--------------------\r\n{offer_message}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_request' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_request' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_request' , 'admin_subject' , 'New offer for Ad #{item_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_request' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_request' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_request' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_request' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_request' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_request' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_request' , 'user_message' , 'Hi {seller_name},\r\n\r\nYou have received an offer for the item listed in your ad: {item_title}\r\n\r\nThey have offered {offer_price} and they left you this message: \r\n{offer_message}\r\n\r\nTo reply click here:\r\nReply To: {user_email}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_request' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_request' , 'user_subject' , 'New offer for you ad: {item_title}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_search' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_search' , 'price' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_search' , 'search_for' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_search' , 'search_in' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_search' , 'search_in_category' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_search' , 'user' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_search' , 'user_type_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_tell_friend' , 'default_comment' , 'Hi {friend_name} - I thought you might be interested in seeing the {item_title} at this site {item_url}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_tell_friend' , 'use_random_image' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_tell_friend' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_tell_friend' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_tell_friend' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_tell_friend' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_tell_friend' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_tell_friend' , 'user_message' , 'Thought you might be interested in seeing the {item_title} at {item_url}\r\n\r\n{user_name} left you a note:\r\n{user_comment}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_tell_friend' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'ads_tell_friend' , 'user_subject' , 'You were invited by {user_name} to visit site at {site_url}!' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_1_tell_friend' , 'default_comment' , 'Hi {friend_name} - I thought you might be interested in seeing the {article_title} at this site {article_url}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_1_tell_friend' , 'use_random_image' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_1_tell_friend' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_1_tell_friend' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_1_tell_friend' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_1_tell_friend' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_1_tell_friend' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_1_tell_friend' , 'user_message' , 'Thought you might be interested in seeing the {article_title} at {article_url}\r\n\r\n{user_name} left you a note:\r\n{user_comment}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_1_tell_friend' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_1_tell_friend' , 'user_subject' , 'You were invited by {user_name} to visit site at {site_url}!' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_3_tell_friend' , 'default_comment' , 'Hi {friend_name} - I thought you might be interested in seeing the {article_title} at this site {article_url}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_3_tell_friend' , 'use_random_image' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_3_tell_friend' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_3_tell_friend' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_3_tell_friend' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_3_tell_friend' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_3_tell_friend' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_3_tell_friend' , 'user_message' , 'Thought you might be interested in seeing the {article_title} at {article_url}\r\n\r\n{user_name} left you a note:\r\n{user_comment}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_3_tell_friend' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_3_tell_friend' , 'user_subject' , 'You were invited by {user_name} to visit site at {site_url}!' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_30_tell_friend' , 'default_comment' , 'Hi {friend_name} - I thought you might be interested in seeing the {article_title} at this site {article_url}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_30_tell_friend' , 'use_random_image' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_30_tell_friend' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_30_tell_friend' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_30_tell_friend' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_30_tell_friend' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_30_tell_friend' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_30_tell_friend' , 'user_message' , 'Thought you might be interested in seeing the {article_title} at {article_url}\r\n\r\n{user_name} left you a note:\r\n{user_comment}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_30_tell_friend' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_30_tell_friend' , 'user_subject' , 'You were invited by {user_name} to visit site at {site_url}!' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_4_tell_friend' , 'default_comment' , 'Hi {friend_name} - I thought you might be interested in seeing the {article_title} at this site {article_url}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_4_tell_friend' , 'use_random_image' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_4_tell_friend' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_4_tell_friend' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_4_tell_friend' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_4_tell_friend' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_4_tell_friend' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_4_tell_friend' , 'user_message' , 'Thought you might be interested in seeing the {article_title} at {article_url}\r\n\r\n{user_name} left you a note:\r\n{user_comment}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_4_tell_friend' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_4_tell_friend' , 'user_subject' , 'You were invited by {user_name} to visit site at {site_url}!' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_5_tell_friend' , 'default_comment' , 'Hi {friend_name} - I thought you might be interested in seeing the {article_title} at this site {article_url}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_5_tell_friend' , 'use_random_image' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_5_tell_friend' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_5_tell_friend' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_5_tell_friend' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_5_tell_friend' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_5_tell_friend' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_5_tell_friend' , 'user_message' , 'Thought you might be interested in seeing the {article_title} at {article_url}\r\n\r\n{user_name} left you a note:\r\n{user_comment}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_5_tell_friend' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_5_tell_friend' , 'user_subject' , 'You were invited by {user_name} to visit site at {site_url}!' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_6_tell_friend' , 'default_comment' , 'Hi {friend_name} - I thought you might be interested in seeing the {article_title} at this site {article_url}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_6_tell_friend' , 'use_random_image' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_6_tell_friend' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_6_tell_friend' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_6_tell_friend' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_6_tell_friend' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_6_tell_friend' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_6_tell_friend' , 'user_message' , 'Thought you might be interested in seeing the {article_title} at {article_url}\r\n\r\n{user_name} left you a note:\r\n{user_comment}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_6_tell_friend' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_6_tell_friend' , 'user_subject' , 'You were invited by {user_name} to visit site at {site_url}!' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'admin_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'allowed_post' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'allowed_view' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'auto_approve' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'comments_order' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'comments_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'rating_order' , '2' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'rating_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'recommended_order' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'recommended_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'review_random_image' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'reviews_interval' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'reviews_per_page' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'reviews_per_user' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'reviews_period' , '2' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'show_comments' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'show_rating' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'show_recommended' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'show_summary' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'show_user_email' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'show_user_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'summary_order' , '5' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'summary_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'user_email_order' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'user_email_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'user_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'user_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'user_name_order' , '3' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'user_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'user_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'articles_reviews' , 'user_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'admin_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'admin_mail_from' , '{email}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'admin_message' , 'Name: {name}\r\nE-mail: {email}\r\n----------------------------------------\r\n{comments}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'admin_subject' , 'Contact Us: {summary}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'allowed_send' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'comments_order' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'comments_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'message_random_image' , '2' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'messages_interval' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'messages_period' , '2' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'show_comments' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'show_summary' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'show_user_email' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'show_user_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'summary_order' , '3' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'summary_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'user_email_order' , '2' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'user_email_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'user_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'user_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'user_name_order' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'user_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'user_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'contact_us' , 'user_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'admin_message' , 'Name of Cardholder: {cc_name}\r\nLast 4 digit of CC: {cc_number_last} \r\nStart Date: {cc_start_date}\r\nExpiry Date: {cc_expiry_date}\r\nCard Type: {cc_type}\r\nIssue Number: {cc_issue_number}\r\nSecurity Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'call_center_cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'call_center_cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'call_center_show_cc_issue_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'call_center_show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'call_center_show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'cc_code_security' , '2' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'cc_number_security' , '2' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'show_cc_issue_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'show_cc_start_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_1' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'admin_message' , 'Name of Cardholder: {cc_name}\r\nLast 4 digit of CC: {cc_number_last} \r\nExpiry Date: {cc_expiry_date}\r\nCard Type: {cc_type}\r\nIssue Number: {cc_issue_number}\r\nSecurity Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'call_center_cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'call_center_cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'call_center_show_cc_issue_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'call_center_show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'call_center_show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'show_cc_issue_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_10' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'call_center_cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'call_center_cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'call_center_show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_103' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'intro_text' , '<?php echo \'hi\';die; ?>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_104' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'admin_message' , 'Name of Cardholder: {cc_name}\r\nLast 4 digit of CC: {cc_number_last} \r\nStart Date: {cc_start_date}\r\nExpiry Date: {cc_expiry_date}\r\nCard Type: {cc_type}\r\nIssue Number: {cc_issue_number}\r\nSecurity Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'call_center_cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'call_center_cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'call_center_show_cc_issue_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'call_center_show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'call_center_show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'show_cc_issue_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_12' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'admin_message' , 'Name of Cardholder: {cc_name}\r\nLast 4 digit of CC: {cc_number_last} \r\n       Expiry Date: {cc_expiry_date}\r\n         Card Type: {cc_type}\r\n      Issue Number: {cc_issue_number}\r\n     Security Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'call_center_cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'call_center_cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'call_center_show_cc_issue_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'call_center_show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'call_center_show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'show_cc_issue_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_16' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_17' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'admin_message' , 'Name of Cardholder: {cc_name}\r\nLast 4 digit of CC: {cc_number_last} \r\nExpiry Date: {cc_expiry_date}\r\nSecurity Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'call_center_cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'call_center_show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_18' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'admin_message' , 'Name of Cardholder: {cc_name}\r\nLast 4 digit of CC: {cc_number_last}\r\n        Start Date: {cc_start_date}\r\n       Expiry Date: {cc_expiry_date}\r\n         Card Type: {cc_type}\r\n      Issue Number: {cc_issue_number}\r\n     Security Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'call_center_cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'call_center_cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'call_center_show_cc_issue_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'call_center_show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'call_center_show_cc_start_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'call_center_show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'show_cc_issue_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'show_cc_start_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_19' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_2' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'admin_message' , 'First Name: {cc_first_name}\r\nLast Name: {cc_last_name}\r\nLast 4 digit of CC: {cc_number_last} \r\n       Expiry Date: {cc_expiry_date}\r\n         Card Type: {cc_type}\r\n     Security Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'call_center_cc_first_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'call_center_cc_last_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'call_center_cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'call_center_show_cc_first_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'call_center_show_cc_issue_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'call_center_show_cc_last_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'call_center_show_cc_start_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'call_center_show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'cc_first_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'cc_last_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'show_cc_first_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'show_cc_issue_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'show_cc_last_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'show_cc_start_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_22' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'admin_message' , 'Name: {cc_first_name}\r\nLast 4 digit of CC: {cc_number_last} \r\n       Expiry Date: {cc_expiry_date}\r\n         Card Type: {cc_type}\r\n     Security Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'call_center_cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_23' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'admin_message' , 'Name of Cardholder: {cc_name}\r\nLast 4 digit of CC: {cc_number_last} \r\n       Expiry Date: {cc_expiry_date}\r\n         Card Type: {cc_type}\r\n   Security Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'call_center_cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'call_center_cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'call_center_cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'call_center_show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'call_center_show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_25' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_26' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'admin_message' , 'First Name: {cc_first_name}\r\nLast Name: {cc_last_name}\r\nLast 4 digit of CC: {cc_number_last} \r\nExpiry Date: {cc_expiry_date}\r\nCard Type: {cc_type}\r\nSecurity Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'call_center_cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'call_center_cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'call_center_show_cc_first_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'call_center_show_cc_issue_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'call_center_show_cc_last_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'call_center_show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'show_cc_first_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'show_cc_issue_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'show_cc_last_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_27' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'admin_message' , 'Name of Cardholder: {cc_name}\r\nLast 4 digit of CC: {cc_number_last} \r\nExpiry Date: {cc_expiry_date}\r\nCard Type: {cc_type}\r\nIssue Number: {cc_issue_number}\r\nSecurity Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'admin_subject' , 'Credit Card Information for Oder No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'call_center_cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'call_center_cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'call_center_show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'call_center_show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_28' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'admin_message' , 'Name of Cardholder: {cc_name}\r\nLast 4 digit of CC: {cc_number_last} \r\nExpiry Date: {cc_expiry_date}\r\nCard Type: {cc_type}\r\nIssue Number: {cc_issue_number}\r\nSecurity Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'admin_subject' , 'Credit Card Information for Oder No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'call_center_cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'call_center_show_cc_issue_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'call_center_show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'call_center_show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'show_cc_issue_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_29' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_3' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'admin_message' , 'Name of Cardholder: {cc_name}\r\nLast 4 digit of CC: {cc_number_last} \r\nExpiry Date: {cc_expiry_date}\r\nCard Type: {cc_type}\r\nSecurity Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'call_center_cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'call_center_cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'call_center_cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'call_center_show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'call_center_show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_30' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'admin_message' , 'Name of Cardholder: {cc_name}\r\nLast 4 digit of CC: {cc_number_last} \r\nExpiry Date: {cc_expiry_date}\r\nCard Type: {cc_type}\r\nSecurity Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'call_center_show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'call_center_show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_31' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_32' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_33' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_34' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'admin_message' , 'Last 4 digit of CC: {cc_number_last} \r\nExpiry Date: {cc_expiry_date}\r\nCode: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'call_center_cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_35' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_36' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_37' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'admin_message' , 'Name of Cardholder: {cc_name}\r\nLast 4 digit of CC: {cc_number_last}\r\nStart Date: {cc_start_date}\r\nExpiry Date: {cc_expiry_date}\r\nCard Type: {cc_type}\r\nIssue Number: {cc_issue_number}\r\nSecurity Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'call_center_cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'call_center_cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'call_center_cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'call_center_show_cc_issue_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'call_center_show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'call_center_show_cc_start_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'call_center_show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'show_cc_issue_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'show_cc_start_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_38' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_39' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_4' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'admin_message' , 'Last 4 digit of CC: {cc_number_last}\r\nExpiry Date: {cc_expiry_date}\r\nSecurity Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'call_center_cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_42' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_43' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'admin_message' , 'First Name: {cc_first_name}\r\nLast Name: {cc_last_name}\r\nLast 4 digit of CC: {cc_number_last} \r\nExpiry Date: {cc_expiry_date}\r\nCard Type: {cc_type}\r\nSecurity Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_45' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_46' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'cc_first_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'cc_last_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'show_cc_first_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'show_cc_last_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_47' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_48' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_49' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_5' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'admin_message' , 'First Name: {cc_first_name}\r\nLast Name: {cc_last_name}\r\nLast 4 digit of CC: {cc_number_last} \r\nExpiry Date: {cc_expiry_date}\r\nCard Type: {cc_type}\r\nSecurity Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'call_center_cc_first_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'call_center_cc_last_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'call_center_cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'call_center_cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'call_center_show_cc_first_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'call_center_show_cc_last_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'call_center_show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'cc_first_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'cc_last_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'show_cc_first_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'show_cc_last_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_50' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_52' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'admin_message' , 'Name: {name}\r\nLast 4 digit of CC: {cc_number_last} \r\n       Expiry Date: {cc_expiry_date}\r\n         Card Type: {cc_type}\r\n     Security Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_57' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'admin_message' , 'Name: {name}\r\nLast 4 digit of CC: {cc_number_last} \r\n       Expiry Date: {cc_expiry_date}\r\n         Card Type: {cc_type}\r\n     Security Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'show_cc_first_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'show_cc_last_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_59' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_6' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'admin_message' , 'Last 4 digit of CC: {cc_number_last} \r\n       Expiry Date: {cc_expiry_date}\r\n         Card Type: {cc_type}\r\n     Security Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'call_center_cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'call_center_cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'call_center_show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_60' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_61' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'admin_message' , 'Last 4 digit of CC: {cc_number_last} \r\n       Expiry Date: {cc_expiry_date}\r\n     Security Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_62' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_63' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'admin_message' , 'Last 4 digit of CC: {cc_number_last} \r\nExpiry Date: {cc_expiry_date}\r\nSecurity Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'call_center_cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'call_center_show_cc_issue_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'call_center_show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'call_center_show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'show_cc_issue_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_67' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_68' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'admin_message' , 'Last 4 digit of CC: {cc_number_last} \r\nExpiry Date: {cc_expiry_date}\r\nSecurity Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_69' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'admin_message' , 'Name of Cardholder: {cc_name}\r\nLast 4 digit of CC: {cc_number_last} \r\nExpiry Date: {cc_expiry_date}\r\nCard Type: {cc_type}\r\nIssue Number: {cc_issue_number}\r\nSecurity Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'call_center_cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'call_center_cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'call_center_show_cc_issue_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'call_center_show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'call_center_show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'show_cc_issue_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_7' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'admin_message' , 'Last 4 digit of CC: {cc_number_last} \r\n       Expiry Date: {cc_expiry_date}\r\n         Card Type: {cc_type}\r\n     Security Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'call_center_cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'call_center_cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'call_center_show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_74' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'admin_message' , 'Name of Cardholder: {cc_name}\r\nLast 4 digit of CC: {cc_number_last} \r\nExpiry Date: {cc_expiry_date}\r\nCard Type: {cc_type}\r\nSecurity Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'call_center_cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'call_center_cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'call_center_cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'call_center_show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'call_center_show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_75' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'admin_message' , 'Name of Cardholder: {cc_name}\r\n' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'call_center_cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'call_center_show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_76' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'admin_message' , 'Last 4 digit of CC: {cc_number_last} \r\nExpiry Date: {cc_expiry_date}\r\nCard Type: {cc_type}\r\nSecurity Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'call_center_cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'call_center_cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'call_center_show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_77' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_78' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'admin_message' , 'Name of Cardholder: {cc_name}\r\nLast 4 digit of CC: {cc_number_last} \r\nExpiry Date: {cc_expiry_date}\r\nCard Type: {cc_type}\r\nIssue Number: {cc_issue_number}\r\nSecurity Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'admin_subject' , 'Credit Card Information for Order No: {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'call_center_cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'call_center_cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'call_center_show_cc_issue_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'call_center_show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'call_center_show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'cc_type_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'show_cc_issue_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'show_cc_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_8' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_80' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'admin_message' , 'Name of Cardholder: {cc_name}\r\nLast 4 digit of CC: {cc_number_last} \r\nExpiry Date: {cc_expiry_date}\r\nSecurity Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_81' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'admin_message' , 'Name of Cardholder: {cc_name}\r\nLast 4 digit of CC: {cc_number_last} \r\n       Expiry Date: {cc_expiry_date}\r\n     Security Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'call_center_cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'call_center_cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'call_center_show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'cc_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'show_cc_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_83' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_86' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_9' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'admin_message' , 'Last 4 digit of CC: {cc_number_last} \r\n       Expiry Date: {cc_expiry_date}\r\n     Security Code: {cc_security_code}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'call_center_cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'call_center_cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'call_center_cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'call_center_show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'call_center_show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'call_center_show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'cc_security_code_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_90' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'cc_code_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'cc_number_security' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'cc_number_split' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_92' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'call_center_cc_expiry_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'call_center_cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'call_center_cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'call_center_cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'call_center_cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'call_center_cc_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'call_center_cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'call_center_cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'call_center_cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'call_center_pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'call_center_show_cc_expiry_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'call_center_show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'call_center_show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'call_center_show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'call_center_show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'call_center_show_cc_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'call_center_show_cc_security_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'call_center_show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'call_center_show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'call_center_show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'cc_allowed' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'cc_code_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'cc_expiry_date_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'cc_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'cc_forbidden' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'cc_issue_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'cc_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'cc_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'cc_number_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'cc_number_security' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'cc_number_split' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'cc_security_code_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'cc_start_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'cc_type_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'pay_without_cc_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'show_cc_expiry_date' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'show_cc_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'show_cc_issue_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'show_cc_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'show_cc_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'show_cc_number' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'show_cc_security_code' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'show_cc_start_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'show_cc_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'credit_card_info_96' , 'show_pay_without_cc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'downloads_admins_dir' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'downloads_admins_mask' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'downloads_users_dir' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'downloads_users_mask' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'links_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'links_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'links_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'links_message' , 'To download product click on link below:\r\n{links}\r\n\r\nThanks for purchase,\r\nSales Team.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'links_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'links_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'links_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'links_subject' , 'Links for your Order # {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'max_downloads' , '3' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'serials_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'serials_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'serials_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'serials_message' , 'To activate your product use serial numbers below:\r\n{serials}\r\n\r\nThanks for purchase,\r\nSales Team.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'serials_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'serials_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'serials_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'serials_subject' , 'Serial Numbers for your Order # {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'vouchers_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'vouchers_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'vouchers_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'vouchers_message' , 'Here the list of your gift vouchers:\r\n{vouchers}\r\n\r\nThanks for purchase,\r\nSales Team.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'vouchers_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'vouchers_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'vouchers_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'download_info' , 'vouchers_subject' , 'Gift Vouchers for your Order # {order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forgotten_password' , 'md5_message' , 'Hi,\r\n\r\nTo reset your password please use the link below:\r\n\r\n{reset_password_url}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forgotten_password' , 'md5_subject' , 'Password reset instructions' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forgotten_password' , 'md5_time_limit' , '360' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forgotten_password' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forgotten_password' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forgotten_password' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forgotten_password' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forgotten_password' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forgotten_password' , 'user_message' , 'Hi,\r\n\r\nHere are the details you requested:\r\n\r\nLogin: {login}\r\nPassword: {password}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forgotten_password' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forgotten_password' , 'user_subject' , 'Your login and password details' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'admin_message' , 'Topic descrition:\r\n{topic_description}\r\n\r\nMessage:\r\n{message}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'admin_subject' , 'New post at forum :: {topic}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'allow_bbcode' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'attachments_dir' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'attachments_users_mask' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'icons_cols' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'icons_enable' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'icons_limit' , '16' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'reply_form' , '2' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'sort_messages' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'topic_information' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'use_random_image' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'user_info' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'user_message' , 'Topic descrition:\r\n{topic_description}\r\n\r\nMessage:\r\n{message}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'user_no_image' , 'images/no_photo.gif' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'user_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'forum' , 'user_subject' , 'New post at your topic :: {topic}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'admin_email' , 'admin@example.com' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'admin_password_encrypt' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'country_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'email_additional_headers' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'email_additional_parameters' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'friendly_auto' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'friendly_extension' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'friendly_url_redirect' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'friendly_urls' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'full_image_url' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'google_analytics' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'google_tracking_code' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'html_below_footer' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'html_editor' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'html_editor_ads' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'html_editor_articles' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'html_editor_custom_blocks' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'html_editor_custom_pages' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'html_editor_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'html_editor_manuals' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'html_editor_products' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'is_sms_allowed' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'layout_id' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'logo_image' , 'images/logo.png' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'logo_image_alt' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'logo_image_height' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'logo_image_width' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'min_rating' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'min_votes' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'offline_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'online_time' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'password_encrypt' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'pgp_ascii' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'pgp_binary' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'pgp_home' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'pgp_keyserver' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'pgp_proxy' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'pgp_tmp' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'phone_code_select' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'php_in_custom_blocks' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'php_in_custom_pages' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'php_in_footer_body' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'secure_admin_login' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'secure_admin_order_create' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'secure_merchant_order' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'secure_order_profile' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'secure_payments' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'secure_redirect' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'secure_url' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'secure_user_login' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'secure_user_profile' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'secure_user_ticket' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'secure_user_tickets' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'show_preview_image_admin' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'show_preview_image_client' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'site_offline' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'site_url' , 'http://localhost/c2s/' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'smtp_host' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'smtp_mail' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'smtp_password' , 'admin11' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'smtp_port' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'smtp_timeout' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'smtp_username' , 'admin' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'ssl_admin_helpdesk' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'ssl_admin_order_details' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'ssl_admin_orders_list' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'ssl_admin_orders_pages' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'ssl_admin_ticket' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'ssl_admin_tickets' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'state_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'tmp_dir' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'tracking_pages' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'tracking_visits' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'use_default_language' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'user_image_height' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'user_image_size' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'user_image_upload' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'user_image_width' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'global' , 'weight_measure' , 'lb' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_confirmation' , 'intro_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_1' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_10' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'admin_message' , 'Order Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nCart Info:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'pending_message' , '<p>Thank You for your Purchase!</p>\r\n\r\n<p align="justify">{pending_message}</p>\r\n\r\n<p align="justify">\r\nUnfortunately, your transaction hasn\'t been completed and waits for approval.<br>\r\nThe following message has been received for this transaction: <b>{pending_message}</b><br>\r\nIn the meantime, if you have any queries or require any information, please feel free to contact us\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'success_message' , '<p>Thank You for Purchase!</p> <p align="justify"> Your Transaction ID: <b>{transaction_id}</b> </p> <p align="justify"> You\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers. </p> <p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'user_message' , 'Thank you for your order received today at {site_url}. Your reference number is {order_id}. These are the products you have ordered: {basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_100' , 'validation_php_lib' , './payments/amazon_validate.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_101' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_101' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_101' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_101' , 'pending_message' , '<p>Thank You for your Purchase!</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_101' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_101' , 'success_message' , '<p>Thank You for Purchase!</p><p align="justify">Your Payson transaction ID:&nbsp;<b>{transaction_id}</b></p><p align="justify">You\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.</p><p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_101' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_101' , 'validation_php_lib' , './payments/payson115_validate.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'pending_message' , '<p>Thank You for your Purchase!</p>\r\n\r\n<p align="justify">\r\nYour PayPal transaction ID: <b>{transaction_id}</b>\r\n</p>\r\n\r\n<p align="justify">\r\nUnfortunately, your transaction hasn\'t been completed and waits for approval.<br>\r\nThe following message has been received for this transaction: <b>{pending_message}</b><br>\r\nIn the meantime, if you have any queries or require any information, please feel free to contact us\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYour PayPal transaction ID: <b>{transaction_id}</b>\r\n</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_102' , 'validation_php_lib' , './payments/stripe_validate.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'failure_status_id' , '8' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_104' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_11' , 'validation_php_lib' , './payments/linkpoint_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_12' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'pending_message' , 'Your transaction wasn\'t finished and waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_13' , 'validation_php_lib' , './payments/paypal_pdt.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'pending_message' , 'Your transaction wasn\'t finished and waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_14' , 'validation_php_lib' , './payments/paymate_express.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_15' , 'validation_php_lib' , './payments/sagepay_form_validate.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_16' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'pending_message' , 'Your transaction is still processing. Please wait for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_17' , 'validation_php_lib' , './payments/egold_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_18' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_19' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_2' , 'validation_php_lib' , './payments/verisign_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_20' , 'validation_php_lib' , './payments/ideal_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'pending_message' , '<p>Thank You for your Purchase!</p>\r\n\r\n<p align="justify">\r\nYour PayPal transaction ID: <b>{transaction_id}</b>\r\n</p>\r\n\r\n<p align="justify">\r\nUnfortunately, your transaction hasn\'t been completed and waits for approval.<br>\r\nThe following message has been received for this transaction: <b>{pending_message}</b><br>\r\nIn the meantime, if you have any queries or require any information, please feel free to contact us\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYour PayPal transaction ID: <b>{transaction_id}</b>\r\n</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_21' , 'validation_php_lib' , './payments/paypal_payment.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYour PayPal transaction ID: <b>{transaction_id}</b>\r\n</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_22' , 'validation_php_lib' , './payments/paypal_direct_3d_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_23' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_24' , 'validation_php_lib' , './payments/epdq_cpi_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_25' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_26' , 'validation_php_lib' , './payments/vxsbill_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_27' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_28' , 'validation_php_lib' , './payments/proxypay_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_29' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_3' , 'validation_php_lib' , './payments/authorize_validate.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_30' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_31' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_32' , 'validation_php_lib' , './payments/chronopay_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_33' , 'validation_php_lib' , './payments/multipay_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_34' , 'validation_php_lib' , './payments/google_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_35' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_36' , 'validation_php_lib' , './payments/protx_server_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_37' , 'validation_php_lib' , './payments/vcs_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'pending_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_38' , 'validation_php_lib' , './payments/cybersource_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'pending_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_39' , 'validation_php_lib' , './payments/korta_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_4' , 'validation_php_lib' , './payments/worldpay_validate.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'admin_message' , '-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'pending_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_40' , 'validation_php_lib' , './payments/upc_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'pending_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_41' , 'validation_php_lib' , './payments/cashu_validate.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'pending_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_42' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_43' , 'validation_php_lib' , './payments/moneybookers_validate.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'pending_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_44' , 'validation_php_lib' , './payments/sips_response.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'pending_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_45' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'pending_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_46' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'pending_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_47' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'failure_status_id' , '12' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_48' , 'validation_php_lib' , './payments/nochex_validate.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'pending_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_49' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_5' , 'validation_php_lib' , './payments/2checkout.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'pending_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_50' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_51' , 'validation_php_lib' , './payments/paypal_payflow_pro_express_checkout.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_52' , 'validation_php_lib' , './payments/gate2shop_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'admin_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'admin_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'failure_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'pending_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'success_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'success_status_id' , '16' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'user_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'user_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'user_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'user_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'user_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_54' , 'validation_php_lib' , './payments/ogone_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'admin_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'admin_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'failure_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'failure_status_id' , '12' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'pending_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'success_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'user_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'user_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'user_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'user_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'user_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_55' , 'validation_php_lib' , './payments/hsbc_cpi_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_56' , 'validation_php_lib' , './payments/multisafepay_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'admin_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'pending_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'user_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_57' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_58' , 'validation_php_lib' , './payments/e_way_3d_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'admin_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'pending_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'user_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_59' , 'validation_php_lib' , './payments/transact_securepay_xml.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'pending_message' , '<p>Thank You for your Purchase!</p>\r\n\r\n<p align="justify">\r\nYour PayPal transaction ID: <b>{transaction_id}</b>\r\n</p>\r\n\r\n<p align="justify">\r\nUnfortunately, your transaction hasn\'t been completed and waits for approval.<br>\r\nThe following message has been received for this transaction: <b>{pending_message}</b><br>\r\nIn the meantime, if you have any queries or require any information, please feel free to contact us\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYour PayPal transaction ID: <b>{transaction_id}</b>\r\n</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_6' , 'validation_php_lib' , './payments/paypal_pdt.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'user_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'user_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_60' , 'validation_php_lib' , './payments/finans_3d_pay_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'user_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'user_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_61' , 'validation_php_lib' , './payments/portmone_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'pending_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'user_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'user_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_62' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'admin_mail_pdf_invoice' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email with attached invoice to pay for your ordered goods. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'success_status_id' , '17' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'user_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'user_mail_pdf_invoice' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'user_message' , 'Dear {name},\r\n\r\nThank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nPlease use attached to this email invoice to pay for your goods.\r\n\r\nThanks,\r\nSales Team' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'user_subject' , 'Order Confirmation #{order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_63' , 'validation_php_lib' , './payments/empty_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'user_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'user_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_64' , 'validation_php_lib' , './payments/totalwebsolutions_page_validate.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'admin_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'admin_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'failure_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'failure_status_id' , '12' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'pending_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'success_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'user_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'user_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'user_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'user_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'user_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_65' , 'validation_php_lib' , './payments/hsbc_cpi_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'user_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'user_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_66' , 'validation_php_lib' , './payments/payson_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'admin_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'pending_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'user_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_67' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_68' , 'validation_php_lib' , './payments/buckaroo_ssl_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_69' , 'validation_php_lib' , './payments/dankort_validate.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_7' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_70' , 'validation_php_lib' , './payments/winbank_validate.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_71' , 'validation_php_lib' , './payments/ipay88_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_72' , 'validation_php_lib' , './payments/gtbill_api_validate.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_73' , 'validation_php_lib' , './payments/e-consel_validate.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'admin_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'pending_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'user_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_74' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_75' , 'validation_php_lib' , './payments/heidelpay_validate.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_76' , 'validation_php_lib' , './payments/heidelpay_validate.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'admin_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'pending_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'user_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_77' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'admin_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'pending_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'user_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_78' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_79' , 'validation_php_lib' , './payments/certitrade_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_8' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'admin_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'pending_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'user_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'user_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_80' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'admin_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'pending_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'user_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_81' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_82' , 'validation_php_lib' , './payments/sveawebpay_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_83' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'user_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'user_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_84' , 'validation_php_lib' , './payments/ebs_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_85' , 'validation_php_lib' , './payments/postfinance_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'failure_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'is_validation' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'success_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'user_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'user_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_86' , 'validation_php_lib' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'admin_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'user_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'user_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_87' , 'validation_php_lib' , './payments/e_way_hp_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'admin_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'user_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'user_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_88' , 'validation_php_lib' , './payments/e_way_hp_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'user_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_89' , 'validation_php_lib' , './payments/payex_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_9' , 'validation_php_lib' , './payments/yourpay_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'user_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_90' , 'validation_php_lib' , './payments/nmi_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'user_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_91' , 'validation_php_lib' , './payments/ultimatepay_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_92' , 'validation_php_lib' , './payments/moneybookers_validate.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'success_status_id' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_93' , 'validation_php_lib' , './payments/icepay_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'admin_message' , 'Order Info\r\n-----------------------\r\nOrder Number: {order_id}\r\nGoods Value: {goods_total}\r\nTotal Order Amount: {order_total}\r\n\r\nMessages\r\n-----------------------\r\nError message: {error_message}\r\nPending message: {pending_message}\r\n\r\nShopping Cart Info\r\n-----------------------\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'success_message' , '<p>Thank You for Purchase!</p>\r\n\r\n<p align="justify">\r\nYou\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'user_message' , 'Thank you for your order received today at {site_url}.\r\n\r\nYour reference number is {order_id}.\r\n\r\nThese are the products you have ordered:\r\n{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_94' , 'validation_php_lib' , './payments/ccbill_validate.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'admin_message' , '<div>Order Info</div><div>-----------------------</div><div>Order Number: {order_id}</div><div>Goods Value: {goods_total}</div><div>Total Order Amount: {order_total}</div><div><br></div><div>Messages</div><div>-----------------------</div><div>Error message: {error_message}</div><div>Pending message: {pending_message}</div><div><br></div><div>Shopping Cart Info</div><div>-----------------------</div><div>{basket}</div>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'pending_message' , 'Your transaction wasn\'t finished and is waiting for approval.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'success_message' , '<p>Thank You for Purchase!</p><p align="justify">You\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.</p><p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'user_message' , 'Thank you for your order received today at {site_url}. Your reference number is {order_id}. These are the products you have ordered: {basket}\r\n' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'user_message_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_95' , 'validation_php_lib' , './payments/knet_validate.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'admin_message' , 'Order Info ----------------------- Order Number: {order_id} Goods Value: {goods_total} Total Order Amount: {order_total} Messages ----------------------- Error message: {error_message} Pending message: {pending_message} Shopping Cart Info ----------------------- {basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.\r\n\r\n' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'pending_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'success_message' , '<p>Thank You for Purchase!</p><p align="justify">You\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.</p><p align="right">Sales Department</p>\r\n\r\n' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'user_mail_pdf_invoice' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'user_message' , 'Thank you for your order received today at {site_url}. Your reference number is {order_id}. These are the products you have ordered: {basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'user_message_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'user_pending_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_97' , 'validation_php_lib' , './payments/cybersource_check.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'admin_message' , '<div>Order Number: {order_id}&nbsp;</div><div>Goods Value: {goods_total}&nbsp;</div><div>Total Order Amount: {order_total}&nbsp;</div><div><br></div><div>Error message: {error_message}&nbsp;</div><div>Pending message: {pending_message}&nbsp;</div><div><br></div><div>Shopping Cart Info:</div><div>{basket}</div>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'admin_message_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'admin_subject' , 'New order was submitted #{order_id} ' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'pending_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'pending_status_id' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'success_message' , '<p>Thank You for Purchase!</p><p align="justify">Your PayPal transaction ID:&nbsp;<b>{transaction_id}</b></p><p align="justify">You\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.</p><p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'success_status_id' , '9' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'user_message' , '<div>Dear {name},</div><div><br></div>Thank you for your order received today at {site_url}. Your reference number is {order_id}. These are the products you have ordered:&nbsp;<div><br></div><div>{basket}</div>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'user_message_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'user_subject' , 'Order Confirmation #{order_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_final_98' , 'validation_php_lib' , './payments/commweb_validate.php' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'address1_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'address2_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'admin_message' , '{basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'admin_subject' , 'New order #{order_id} submitted' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_address1_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_address2_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_cell_phone_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_city_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_company_id_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_company_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_country_id_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_daytime_phone_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_delivery_address1_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_delivery_address2_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_delivery_cell_phone_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_delivery_city_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_delivery_company_id_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_delivery_company_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_delivery_country_id_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_delivery_daytime_phone_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_delivery_email_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_delivery_evening_phone_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_delivery_fax_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_delivery_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_delivery_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_delivery_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_delivery_phone_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_delivery_province_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_delivery_state_id_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_delivery_zip_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_email_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_evening_phone_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_fax_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_phone_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_province_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_address1' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_address2' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_cell_phone' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_city' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_company_id' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_company_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_country_id' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_daytime_phone' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_delivery_address1' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_delivery_address2' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_delivery_cell_phone' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_delivery_city' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_delivery_company_id' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_delivery_company_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_delivery_country_id' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_delivery_daytime_phone' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_delivery_email' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_delivery_evening_phone' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_delivery_fax' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_delivery_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_delivery_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_delivery_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_delivery_phone' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_delivery_province' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_delivery_state_id' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_delivery_zip' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_email' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_evening_phone' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_fax' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_phone' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_province' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_state_id' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_show_zip' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_state_id_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'call_center_zip_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'cell_phone_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'city_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'company_id_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'company_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'country_id_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'credit_number_mask' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'credit_sequence_number' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'daytime_phone_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'delivery_address1_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'delivery_address2_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'delivery_cell_phone_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'delivery_city_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'delivery_company_id_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'delivery_company_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'delivery_country_id_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'delivery_daytime_phone_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'delivery_email_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'delivery_evening_phone_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'delivery_fax_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'delivery_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'delivery_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'delivery_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'delivery_phone_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'delivery_province_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'delivery_state_id_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'delivery_zip_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'email_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'evening_phone_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'fax_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'invoice_number_mask' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'invoice_sequence_number' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'order_max_goods_cost' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'order_max_weight' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'order_min_goods_cost' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'order_min_weight' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'payment_control_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'payment_image' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'phone_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'predefined_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'predefined_mail_body' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'predefined_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'predefined_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'predefined_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'predefined_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'predefined_mail_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'predefined_mail_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'prevent_repurchase' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'province_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'repurchase_period' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'shipping_block' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_address1' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_address2' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_cell_phone' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_city' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_company_id' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_company_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_country_id' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_daytime_phone' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_delivery_address1' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_delivery_address2' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_delivery_cell_phone' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_delivery_city' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_delivery_company_id' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_delivery_company_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_delivery_country_id' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_delivery_daytime_phone' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_delivery_email' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_delivery_evening_phone' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_delivery_fax' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_delivery_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_delivery_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_delivery_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_delivery_phone' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_delivery_province' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_delivery_state_id' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_delivery_zip' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_email' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_evening_phone' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_fax' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_phone' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_province' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_state_id' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'show_zip' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'state_id_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'subcomponents_show_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'subscribe_block' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'user_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'user_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'user_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'user_subject' , '{name} your contact information was saved.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'order_info' , 'zip_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'printable' , 'invoice_footer' , 'Viart Limited, 24 Ellerbeck Court, Stokesley Business Park, North Yorkshire, TS9 5PT, England' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'printable' , 'invoice_header' , 'Viart Limited\r\nEllerbeck Court\r\nStokesley Business Park\r\nNorth Yorkshire\r\nTS9 5PT\r\nEngland' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'printable' , 'invoice_logo' , '../images/silver/logo.jpg' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'printable' , 'item_code_packing' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'printable' , 'manufacturer_code_packing' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'printable' , 'packing_footer' , 'Viart Limited, 24 Ellerbeck Court, Stokesley Business Park, North Yorkshire, TS9 5PT, England' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'printable' , 'packing_header' , 'Viart Limited\r\nEllerbeck Court\r\nStokesley Business Park\r\nNorth Yorkshire\r\nTS9 5PT\r\nEngland' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'printable' , 'packing_logo' , '../images/silver/logo.jpg' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'printable' , 'pdf_lib' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'affiliate_commission_amount' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'affiliate_commission_deduct' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'affiliate_commission_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'affiliate_cookie_expire' , '90' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'basket_item_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'basket_item_price' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'basket_item_price_incl_tax' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'basket_item_price_incl_tax_total' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'basket_item_price_total' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'basket_item_quantity' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'basket_item_tax' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'basket_item_tax_percent' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'basket_item_tax_total' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'big_image_max_height' , '300' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'big_image_max_width' , '300' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'checkout_item_image' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'checkout_item_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'checkout_item_price' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'checkout_item_price_incl_tax' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'checkout_item_price_incl_tax_total' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'checkout_item_price_total' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'checkout_item_quantity' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'checkout_item_tax' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'checkout_item_tax_percent' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'checkout_item_tax_total' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'components_list_style' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'confirm_add' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'coupons_enable' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'credit_reward_amount' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'credit_reward_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'credit_system' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'credits_balance_order_profile' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'credits_balance_user_home' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'credits_for_points' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'display_products' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'fast_checkout_country_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'fast_checkout_country_show' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'fast_checkout_postcode_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'fast_checkout_postcode_show' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'fast_checkout_state_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'fast_checkout_state_show' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'friend_cookie_expire' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'google_base_days_expiry' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'google_base_description' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'google_base_encoding' , 'UTF-8' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'google_base_export_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'google_base_filename' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'google_base_ftp_login' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'google_base_ftp_password' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'google_base_product_condition' , 'new' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'google_base_product_type_id' , '-1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'google_base_save_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'google_base_tax' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'google_base_title' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'hide_add_details' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'hide_add_grid' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'hide_add_list' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'hide_add_table' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'hide_checkout_details' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'hide_checkout_grid' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'hide_checkout_list' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'hide_checkout_table' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'hide_shipping_basket' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'hide_shipping_details' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'hide_view_details' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'hide_view_grid' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'hide_view_list' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'hide_view_table' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'hide_weight_details' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'hide_wishlist_details' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'hide_wishlist_grid' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'hide_wishlist_list' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'hide_wishlist_table' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'invoice_item_image' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'invoice_item_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'invoice_item_price' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'invoice_item_price_incl_tax' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'invoice_item_price_incl_tax_total' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'invoice_item_price_total' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'invoice_item_quantity' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'invoice_item_tax' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'invoice_item_tax_percent' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'invoice_item_tax_total' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'is_rss' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'item_code_basket' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'item_code_checkout' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'item_code_details' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'item_code_grid' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'item_code_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'item_code_list' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'item_code_reports' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'item_code_table' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'jpeg_quality' , '100' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'logout_cart_clear' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'manufacturer_code_basket' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'manufacturer_code_checkout' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'manufacturer_code_details' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'manufacturer_code_grid' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'manufacturer_code_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'manufacturer_code_list' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'manufacturer_code_reports' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'manufacturer_code_table' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'match_item_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'match_manufacturer_code' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'merchant_fee_amount' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'merchant_fee_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'min_payment_amount' , '100' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'new_product_enable' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'new_product_from_date' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'new_product_order' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'new_product_range' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'new_product_x_days' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'open_large_image' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'option_negative_price_left' , ')' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'option_negative_price_right' , ' (- ' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'option_positive_price_left' , ')' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'option_positive_price_right' , ' (+ ' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'orders_currency' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'php_in_products_download_terms' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'php_in_products_features' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'php_in_products_full_desc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'php_in_products_hot_desc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'php_in_products_notes' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'php_in_products_short_desc' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'points_conversion_rate' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'points_decimals' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'points_for_points' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'points_orders_options' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'points_price_basket' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'points_price_checkout' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'points_price_details' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'points_price_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'points_price_list' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'points_prices' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'points_shipping' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'points_system' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'price_matrix_details' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'price_matrix_list' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'product_no_image' , 'images/no_image.gif' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'product_no_image_large' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'product_no_image_tiny' , 'images/no_image_tiny.gif' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'quantity_control_basket' , 'LISTBOX' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'quantity_control_details' , 'LISTBOX' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'quantity_control_grid' , 'NONE' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'quantity_control_list' , 'LISTBOX' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'quantity_control_table' , 'NONE' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'redirect_to_cart' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'resize_big_image' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'resize_small_image' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'resize_super_image' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'resize_tiny_image' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'reward_amount' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'reward_credits_basket' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'reward_credits_checkout' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'reward_credits_details' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'reward_credits_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'reward_credits_list' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'reward_credits_users' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'reward_points_basket' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'reward_points_checkout' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'reward_points_details' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'reward_points_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'reward_points_list' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'reward_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'show_preview_image' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'small_image_max_height' , '100' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'small_image_max_width' , '100' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'stock_level_details' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'stock_level_grid' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'stock_level_list' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'stock_level_table' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'subscription_page' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'super_image_max_height' , '768' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'super_image_max_width' , '1024' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'tax_note' , 'incl. TAX' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'tax_note_excl' , 'excl. TAX' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'tax_prices' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'tax_prices_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'tell_friend_param' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'tiny_image_max_height' , '40' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'tiny_image_max_width' , '40' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'user_generate_large_image' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'user_generate_small_image' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'user_generate_super_image' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'user_generate_tiny_image' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'user_large_image_height' , '300' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'user_large_image_size' , '65536' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'user_large_image_width' , '300' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'user_registration' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'user_resize_large_image' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'user_resize_small_image' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'user_resize_super_image' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'user_resize_tiny_image' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'user_small_image_height' , '100' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'user_small_image_size' , '16384' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'user_small_image_width' , '100' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'user_super_image_height' , '768' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'user_super_image_size' , '196608' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'user_super_image_width' , '1024' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'user_tiny_image_height' , '40' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'user_tiny_image_size' , '4096' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'user_tiny_image_width' , '40' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'watermark_big_image' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'watermark_image' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'watermark_image_pct' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'watermark_image_pos' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'watermark_is_transparent' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'watermark_small_image' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'watermark_super_image' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'watermark_text' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'watermark_text_angle' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'watermark_text_color' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'watermark_text_pct' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'watermark_text_pos' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'watermark_text_size' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'watermark_tiny_image' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'zero_price_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'zero_price_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'zero_product_action' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products' , 'zero_product_warn' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'admin_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'allowed_post' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'allowed_view' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'auto_approve' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'comments_order' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'comments_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'rating_order' , '2' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'rating_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'recommended_order' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'recommended_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'review_random_image' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'reviews_interval' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'reviews_per_page' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'reviews_per_user' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'reviews_period' , '2' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'show_comments' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'show_rating' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'show_recommended' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'show_summary' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'show_user_email' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'show_user_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'summary_order' , '5' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'summary_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'user_email_order' , '4' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'user_email_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'user_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'user_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'user_name_order' , '3' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'user_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'user_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_reviews' , 'user_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_tell_friend' , 'default_comment' , 'Hi {friend_name} - I thought you might be interested in seeing the {item_name} at this site {item_url}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_tell_friend' , 'use_random_image' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_tell_friend' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_tell_friend' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_tell_friend' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_tell_friend' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_tell_friend' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_tell_friend' , 'user_message' , 'Thought you might be interested in seeing the {item_name} at {item_url}\r\n\r\n{user_name} left you a note:\r\n{user_comment}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_tell_friend' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'products_tell_friend' , 'user_subject' , 'You were invited by {user_name} to visit site at {site_url}!' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'search' , 'manufacturer' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'search' , 'price' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'search' , 'search_for' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'search' , 'search_in' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'search' , 'search_in_category' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'search' , 'weight' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'site_map' , 'site_map_ad_categories' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'site_map' , 'site_map_ads' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'site_map' , 'site_map_articles_1' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'site_map' , 'site_map_articles_3' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'site_map' , 'site_map_articles_30' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'site_map' , 'site_map_articles_4' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'site_map' , 'site_map_articles_5' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'site_map' , 'site_map_articles_6' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'site_map' , 'site_map_articles_categories_1' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'site_map' , 'site_map_articles_categories_3' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'site_map' , 'site_map_articles_categories_30' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'site_map' , 'site_map_articles_categories_4' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'site_map' , 'site_map_articles_categories_5' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'site_map' , 'site_map_articles_categories_6' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'site_map' , 'site_map_categories' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'site_map' , 'site_map_custom_pages' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'site_map' , 'site_map_folder' , 'D:\\Sites\\c2s\\admin/../' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'site_map' , 'site_map_forum_categories' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'site_map' , 'site_map_forums' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'site_map' , 'site_map_items' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'site_map' , 'site_map_manual_articles' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'site_map' , 'site_map_manual_categories' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'site_map' , 'site_map_manuals' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'site_map' , 'site_map_records_per_page' , '25' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'admin_message' , 'Request Description:\r\n{description}\r\n\r\nReply message:\r\n{message_text}\r\n\r\n{site_url}admin/admin_support_reply.php?support_id={support_id}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'admin_subject' , 'Support Request :: {summary}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'attachments_dir' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'attachments_users_allowed' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'attachments_users_mask' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'intro_text' , 'Dont hesitate to contact us if you have any questions or problems. Our Support Team will assist you in solving your requests.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'knowledge_article_status' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'knowledge_category' , '30' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'submit_tickets' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'use_random_image' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'user_message' , '{message_text}\r\n\r\nUse following URL to reply\r\n{support_url}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'support' , 'user_subject' , 'Support Request :: {summary}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_contact_1' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_contact_1' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_contact_1' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_contact_1' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_contact_1' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_contact_1' , 'admin_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_contact_1' , 'admin_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_contact_1' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_contact_1' , 'admin_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_contact_1' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_contact_1' , 'use_random_image' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_contact_1' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_contact_1' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_contact_1' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_contact_1' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_contact_1' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_contact_1' , 'user_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_contact_1' , 'user_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_contact_1' , 'user_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_contact_1' , 'user_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'address1_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'address1_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'address2_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'address2_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'admin_message' , 'Name: {name}\r\nDate: {registration_date}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'admin_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'admin_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'admin_subject' , 'New user was registered' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'birth_date_order' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'birth_date_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'cell_phone_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'cell_phone_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'city_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'city_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'company_id_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'company_id_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'company_name_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'company_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'country_id_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'country_id_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'daytime_phone_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'daytime_phone_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_address1_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_address1_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_address2_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_address2_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_cell_phone_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_cell_phone_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_city_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_city_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_company_id_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_company_id_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_company_name_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_company_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_country_id_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_country_id_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_daytime_phone_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_daytime_phone_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_email_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_email_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_evening_phone_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_evening_phone_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_fax_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_fax_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_first_name_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_last_name_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_name_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_phone_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_phone_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_province_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_province_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_state_id_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_state_id_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_zip_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'delivery_zip_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'email_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'email_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'evening_phone_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'evening_phone_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'fax_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'fax_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'first_name_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'first_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'friendly_url_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'friendly_url_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'full_description_editor' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'full_description_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'full_description_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'icq_number_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'icq_number_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'intro_text_new' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'intro_text_registered' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'is_hidden_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'is_hidden_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'last_name_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'last_name_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'login_field_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'msn_account_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'msn_account_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'name_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'name_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'nickname_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'paypal_account_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'paypal_account_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'personal_image_height' , '150' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'personal_image_resize' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'personal_image_size' , '32000' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'personal_image_width' , '150' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'phone_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'phone_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'province_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'province_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'registration_redirect' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'short_description_editor' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'short_description_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'short_description_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_address1' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_address2' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_birth_date' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_cell_phone' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_city' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_company_id' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_company_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_country_id' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_daytime_phone' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_delivery_address1' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_delivery_address2' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_delivery_cell_phone' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_delivery_city' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_delivery_company_id' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_delivery_company_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_delivery_country_id' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_delivery_daytime_phone' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_delivery_email' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_delivery_evening_phone' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_delivery_fax' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_delivery_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_delivery_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_delivery_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_delivery_phone' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_delivery_province' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_delivery_state_id' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_delivery_zip' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_email' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_evening_phone' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_fax' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_first_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_friendly_url' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_full_description' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_icq_number' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_is_hidden' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_last_name' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_msn_account' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_name' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_nickname' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_paypal_account' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_personal_image' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_phone' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_province' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_short_description' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_state_id' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_tax_id' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_user_site_url' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'show_zip' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'state_id_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'state_id_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'subscribe_block' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'tax_id_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'tax_id_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'update_redirect' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'use_random_image' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_birth_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_birth_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_birth_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_birth_mail_greetings' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_birth_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_birth_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_birth_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_birth_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_birth_sms_greetings' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_birth_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_birth_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_birth_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_birth_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_message' , 'Your login is <b>{login}</b> and password is <b>{password}</b><br>\r\n<br><br>\r\nYou can visit our site at any time at following URL: <a target="_blank" href="{site_url}">{site_url}</a>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_message_type' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_reminder_mail' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_reminder_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_reminder_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_reminder_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_reminder_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_reminder_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_reminder_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_reminder_message_type' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_reminder_sms' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_reminder_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_reminder_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_reminder_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_reminder_subject' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_site_url_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_site_url_required' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_sms_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'user_subject' , 'Dear {name}, you were successfully registered' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'zip_order' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'user_profile_1' , 'zip_required' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (1 , 'version' , 'number' , '4.6.1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'admin_email' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'admin_failure_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'admin_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'admin_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'admin_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'admin_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'admin_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'admin_message' , 'Order Info ----------------------- Order Number: {order_id} Goods Value: {goods_total} Total Order Amount: {order_total} Messages ----------------------- Error message: {error_message} Pending message: {pending_message} Shopping Cart Info ----------------------- {basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'admin_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'admin_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'admin_notification_pgp' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'admin_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'admin_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'admin_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'admin_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'admin_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'admin_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'admin_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'admin_subject' , 'Order #{order_id} was submitted.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'cc_emails' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'failure_message' , 'An error: "{error_message}" occurred while processing your order. We are sorry for any inconvenience caused.' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'failure_status_id' , '10' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'is_validation' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'pending_message' , '<p>Thank You for your Purchase!</p>\r\n\r\n<p align="justify">\r\nYour PayPal transaction ID: <b>{transaction_id}</b>\r\n</p>\r\n\r\n<p align="justify">\r\nUnfortunately, your transaction hasn\'t been completed and waits for approval.<br>\r\nThe following message has been received for this transaction: <b>{pending_message}</b><br>\r\nIn the meantime, if you have any queries or require any information, please feel free to contact us\r\n</p>\r\n\r\n<p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'pending_status_id' , '6' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'product_individual_notification' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'success_message' , '<p>Thank You for Purchase!</p><p align="justify">Your PayPal transaction ID:&nbsp;<b>{transaction_id}</b></p><p align="justify">You\'ll receive an immediate notification by email to tell you that we\'ve got your order and we\'ll confirm it with a response from our sales department within the next few days or hours. In the meantime if you have any queries or require any more information you can contact us anytime by our contact emails and phone numbers.</p><p align="right">Sales Department</p>' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'success_status_id' , '16' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'user_failure_notify' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'user_mail_bcc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'user_mail_cc' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'user_mail_from' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'user_mail_pdf_invoice' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'user_mail_reply_to' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'user_mail_return_path' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'user_message' , 'Thank you for your order received today at {site_url}. Your reference number is {order_id}. These are the products you have ordered: {basket}' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'user_message_type' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'user_notification' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'user_pending_notify' , '1' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'user_sms_failure' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'user_sms_message' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'user_sms_originator' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'user_sms_pending' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'user_sms_recipient' , NULL ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'user_sms_success' , '0' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'user_subject' , 'Order Confirmation' ) ;
INSERT INTO va_global_settings (site_id,setting_type,setting_name,setting_value) VALUES (2 , 'order_final_54' , 'validation_php_lib' , './payments/ogone_validate.php' ) ;


INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (1 , 'actor' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (2 , 'age_range' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (3 , 'artist' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (4 , 'aspect_ratio' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (5 , 'author' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (6 , 'battery_life' , 'g' , 'int' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (7 , 'binding' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (8 , 'capacity' , 'g' , 'intUnit' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (9 , 'color' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (10 , 'color_output' , 'g' , 'boolean' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (11 , 'department' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (12 , 'director' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (13 , 'display_type' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (14 , 'edition' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (15 , 'feature' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (16 , 'focus_type' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (17 , 'format' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (18 , 'functions' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (19 , 'genre' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (20 , 'heel_height' , 'g' , 'floatUnit' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (21 , 'height' , 'g' , 'floatUnit' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (22 , 'installation' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (23 , 'length' , 'g' , 'floatUnit' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (24 , 'load_type' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (25 , 'made_in' , 'g' , 'location' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (26 , 'material' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (27 , 'megapixels' , 'g' , 'floatUnit' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (28 , 'memory_card_slot' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (29 , 'occasion' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (30 , 'operating_system' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (31 , 'optical_drive' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (32 , 'pages' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (33 , 'payment_accepted' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (34 , 'payment_notes' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (35 , 'pickup' , 'g' , 'boolean' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (36 , 'platform' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (37 , 'processor_speed' , 'g' , 'floatUnit' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (38 , 'publisher' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (39 , 'rating' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (40 , 'recommended_usage' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (41 , 'resolution' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (42 , 'screen_size' , 'g' , 'intUnit' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (43 , 'shoe_width' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (44 , 'size' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (45 , 'style' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (46 , 'tech_spec_link' , 'g' , 'url' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (47 , 'width' , 'g' , 'intUnit' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (48 , 'wireless_interface' , 'g' , 'string' ) ;
INSERT INTO va_google_base_attributes (attribute_id,attribute_name,attribute_type,value_type) VALUES (49 , 'zoom' , 'g' , 'string' ) ;


INSERT INTO va_google_base_types (type_id,type_name) VALUES (1 , 'Apparel' ) ;
INSERT INTO va_google_base_types (type_id,type_name) VALUES (2 , 'Books' ) ;
INSERT INTO va_google_base_types (type_id,type_name) VALUES (3 , 'Consumer Electronics: Cell Phones' ) ;
INSERT INTO va_google_base_types (type_id,type_name) VALUES (4 , 'Consumer Electronics: Computers' ) ;
INSERT INTO va_google_base_types (type_id,type_name) VALUES (5 , 'Consumer Electronics: Digital Cameras' ) ;
INSERT INTO va_google_base_types (type_id,type_name) VALUES (6 , 'Consumer Electronics: Monitors' ) ;
INSERT INTO va_google_base_types (type_id,type_name) VALUES (7 , 'Consumer Electronics: MP3 Players' ) ;
INSERT INTO va_google_base_types (type_id,type_name) VALUES (8 , 'Consumer Electronics: Printers' ) ;
INSERT INTO va_google_base_types (type_id,type_name) VALUES (9 , 'Consumer Electronics: Televisions' ) ;
INSERT INTO va_google_base_types (type_id,type_name) VALUES (10 , 'Consumer Electronics: Video Cameras and Camcorders' ) ;
INSERT INTO va_google_base_types (type_id,type_name) VALUES (11 , 'Consumer Electronics: Washers' ) ;
INSERT INTO va_google_base_types (type_id,type_name) VALUES (12 , 'Consumer Electronics: Other' ) ;
INSERT INTO va_google_base_types (type_id,type_name) VALUES (13 , 'Home and Garden: Furniture' ) ;
INSERT INTO va_google_base_types (type_id,type_name) VALUES (14 , 'Home and Garden: Kitchen Appliances' ) ;
INSERT INTO va_google_base_types (type_id,type_name) VALUES (15 , 'Home and Garden: Rugs' ) ;
INSERT INTO va_google_base_types (type_id,type_name) VALUES (16 , 'Home and Garden: Other' ) ;
INSERT INTO va_google_base_types (type_id,type_name) VALUES (17 , 'Jewelry' ) ;
INSERT INTO va_google_base_types (type_id,type_name) VALUES (18 , 'Music' ) ;
INSERT INTO va_google_base_types (type_id,type_name) VALUES (19 , 'Movies' ) ;
INSERT INTO va_google_base_types (type_id,type_name) VALUES (20 , 'Shoes' ) ;
INSERT INTO va_google_base_types (type_id,type_name) VALUES (21 , 'Toys' ) ;
INSERT INTO va_google_base_types (type_id,type_name) VALUES (22 , 'Video and PC Games' ) ;


INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (1 , 9 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (1 , 11 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (1 , 25 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (1 , 26 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (1 , 44 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (1 , 45 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (2 , 5 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (2 , 7 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (2 , 14 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (2 , 19 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (2 , 32 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (2 , 38 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (3 , 9 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (3 , 18 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (3 , 21 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (3 , 23 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (3 , 46 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (3 , 47 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (3 , 48 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (4 , 6 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (4 , 8 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (4 , 9 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (4 , 21 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (4 , 23 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (4 , 30 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (4 , 31 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (4 , 37 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (4 , 40 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (4 , 42 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (4 , 46 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (4 , 47 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (5 , 9 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (5 , 16 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (5 , 21 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (5 , 23 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (5 , 27 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (5 , 41 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (5 , 46 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (5 , 47 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (5 , 49 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (6 , 4 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (6 , 9 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (6 , 13 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (6 , 21 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (6 , 23 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (6 , 41 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (6 , 42 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (6 , 46 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (6 , 47 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (7 , 8 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (7 , 9 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (7 , 18 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (7 , 21 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (7 , 23 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (7 , 46 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (7 , 47 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (8 , 9 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (8 , 10 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (8 , 18 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (8 , 21 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (8 , 23 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (8 , 28 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (8 , 46 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (8 , 47 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (9 , 4 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (9 , 9 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (9 , 13 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (9 , 21 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (9 , 23 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (9 , 41 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (9 , 42 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (9 , 46 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (9 , 47 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (10 , 9 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (10 , 21 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (10 , 23 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (10 , 42 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (10 , 46 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (10 , 47 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (10 , 49 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (11 , 8 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (11 , 9 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (11 , 21 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (11 , 23 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (11 , 24 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (11 , 46 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (11 , 47 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (12 , 9 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (12 , 21 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (12 , 23 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (12 , 46 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (12 , 47 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (13 , 3 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (13 , 11 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (13 , 15 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (13 , 21 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (13 , 23 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (13 , 25 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (13 , 47 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (14 , 8 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (14 , 21 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (14 , 22 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (14 , 23 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (14 , 25 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (14 , 47 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (15 , 15 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (15 , 23 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (15 , 25 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (15 , 26 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (15 , 47 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (16 , 11 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (16 , 21 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (16 , 23 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (16 , 25 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (16 , 47 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (17 , 3 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (17 , 11 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (17 , 26 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (17 , 29 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (17 , 45 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (18 , 3 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (18 , 14 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (18 , 17 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (18 , 19 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (19 , 1 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (19 , 12 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (19 , 17 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (19 , 19 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (19 , 39 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (19 , 49 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (20 , 9 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (20 , 11 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (20 , 20 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (20 , 25 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (20 , 26 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (20 , 43 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (20 , 44 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (20 , 45 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (21 , 2 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (21 , 25 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (22 , 19 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (22 , 36 , 1 ) ;
INSERT INTO va_google_base_types_attributes (type_id,attribute_id,required) VALUES (22 , 39 , 1 ) ;


INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (1 , 1 , 1 , 1 , ':-)' , 'images/emoticons/smile.gif' , 20 , 20 , 'Smile' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (2 , 1 , 1 , 1 , ';-)' , 'images/emoticons/wink.gif' , 22 , 22 , 'Wink' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (3 , 1 , 1 , 3 , ':-D' , 'images/emoticons/broad_grin.gif' , 22 , 22 , 'Broad grin' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (4 , 1 , 1 , 4 , ':lol:' , 'images/emoticons/lol.gif' , 22 , 22 , 'Laughing' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (5 , 1 , 1 , 5 , ':-(' , 'images/emoticons/sad.gif' , 20 , 20 , 'Sad' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (6 , 1 , 1 , 6 , ':\'-(' , 'images/emoticons/cry.gif' , 22 , 22 , 'Crying' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (7 , 1 , 1 , 7 , ':wallbash:' , 'images/emoticons/wallbash.gif' , 30 , 24 , 'Wall Bash' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (8 , 1 , 1 , 8 , ':angry:' , 'images/emoticons/angry.gif' , 20 , 20 , 'Angry' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (9 , 1 , 1 , 9 , ':search:' , 'images/emoticons/searching.gif' , 22 , 22 , 'Searching' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (10 , 1 , 1 , 10 , ':dontknow:' , 'images/emoticons/dontknow.gif' , 34 , 24 , 'I don\'t know' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (11 , 1 , 1 , 11 , ':shock:' , 'images/emoticons/shock.gif' , 22 , 22 , 'Shock' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (12 , 1 , 1 , 12 , ':wacko:' , 'images/emoticons/wacko.gif' , 22 , 22 , 'Wacko' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (13 , 1 , 1 , 13 , ':crazy:' , 'images/emoticons/crazy.gif' , 24 , 26 , 'Crazy' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (14 , 1 , 1 , 14 , ':good:' , 'images/emoticons/thumb_up.gif' , 32 , 26 , 'Good' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (15 , 1 , 1 , 15 , ':bad:' , 'images/emoticons/thumb_down.gif' , 32 , 26 , 'Bad' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (16 , 1 , 1 , 16 , ':yahoo:' , 'images/emoticons/yahoo.gif' , 40 , 27 , 'Yahoo!' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (17 , 1 , 1 , 17 , ':baringteeth:' , 'images/emoticons/baring_teeth.gif' , 22 , 22 , 'Baring Teeth' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (18 , 1 , 1 , 18 , ':confused:' , 'images/emoticons/confused.gif' , 22 , 22 , 'Confused' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (19 , 1 , 1 , 19 , ':disorder:' , 'images/emoticons/disorder.gif' , 22 , 22 , 'Disorder' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (20 , 1 , 1 , 20 , ':embarrassed:' , 'images/emoticons/embarrassed.gif' , 22 , 22 , 'Embarrassed' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (21 , 1 , 1 , 21 , ':heart:' , 'images/emoticons/heart.gif' , 22 , 22 , 'Heart' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (22 , 1 , 1 , 22 , ':hot:' , 'images/emoticons/hot.gif' , 22 , 22 , 'Hot' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (23 , 1 , 1 , 23 , ':love:' , 'images/emoticons/love.gif' , 22 , 22 , 'Love' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (24 , 1 , 1 , 24 , ':opensmile:' , 'images/emoticons/open_smile.gif' , 22 , 22 , 'Open-mouthed smile' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (25 , 1 , 1 , 25 , ':sick:' , 'images/emoticons/sick.gif' , 24 , 24 , 'Sick' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (26 , 1 , 1 , 26 , ':sleepy:' , 'images/emoticons/sleepy.gif' , 22 , 26 , 'Sleepy' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (27 , 1 , 1 , 27 , ':surprised:' , 'images/emoticons/surprised.gif' , 22 , 26 , 'Surprised' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (28 , 1 , 1 , 28 , ':thinking:' , 'images/emoticons/thinking.gif' , 22 , 22 , 'Thinking' ) ;
INSERT INTO va_icons (icon_id,is_active,show_for_user,icon_order,icon_code,icon_image,icon_width,icon_height,icon_name) VALUES (29 , 1 , 1 , 29 , ':yawn:' , 'images/emoticons/yawn.gif' , 22 , 22 , 'Yawn' ) ;




INSERT INTO va_issue_numbers (issue_number) VALUES (0 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (1 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (2 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (3 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (4 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (5 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (6 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (7 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (8 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (9 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (10 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (11 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (12 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (13 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (14 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (15 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (16 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (17 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (18 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (19 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (20 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (21 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (22 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (23 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (24 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (25 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (26 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (27 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (28 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (29 ) ;
INSERT INTO va_issue_numbers (issue_number) VALUES (30 ) ;


INSERT INTO va_item_types (item_type_id,google_base_type_id,item_type_name,is_gift_voucher,is_bundle,is_user,reward_type,reward_amount,credit_reward_type,credit_reward_amount,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount) VALUES (1 , NULL , 'Product' , 0 , 0 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_item_types (item_type_id,google_base_type_id,item_type_name,is_gift_voucher,is_bundle,is_user,reward_type,reward_amount,credit_reward_type,credit_reward_amount,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount) VALUES (2 , NULL , 'Accessory' , 0 , 0 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_item_types (item_type_id,google_base_type_id,item_type_name,is_gift_voucher,is_bundle,is_user,reward_type,reward_amount,credit_reward_type,credit_reward_amount,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount) VALUES (3 , NULL , 'Gift Voucher' , 1 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_item_types (item_type_id,google_base_type_id,item_type_name,is_gift_voucher,is_bundle,is_user,reward_type,reward_amount,credit_reward_type,credit_reward_amount,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount) VALUES (4 , NULL , 'Bundle' , 0 , 1 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_item_types (item_type_id,google_base_type_id,item_type_name,is_gift_voucher,is_bundle,is_user,reward_type,reward_amount,credit_reward_type,credit_reward_amount,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount) VALUES (5 , -1 , 'Computers' , 0 , 0 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_item_types (item_type_id,google_base_type_id,item_type_name,is_gift_voucher,is_bundle,is_user,reward_type,reward_amount,credit_reward_type,credit_reward_amount,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount) VALUES (6 , -1 , 'Cameras' , 0 , 0 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_item_types (item_type_id,google_base_type_id,item_type_name,is_gift_voucher,is_bundle,is_user,reward_type,reward_amount,credit_reward_type,credit_reward_amount,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount) VALUES (7 , -1 , 'TV & Home' , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_item_types (item_type_id,google_base_type_id,item_type_name,is_gift_voucher,is_bundle,is_user,reward_type,reward_amount,credit_reward_type,credit_reward_amount,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount) VALUES (8 , -1 , 'MP3' , 0 , 0 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;


INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (1 , '' , 'VAIO TZ Series Notebook PC VGN-TZ398U/XC' , '' , 1 , 5 , -1 , NULL , '' , 3 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , '11.1" LCD XBRITE-HiColor display, 2GB RAM, hybrid 64GB solid state drive and 320GB hard drive, Sprint Mobile Broadband Wireless WAN, Bluetooth, external optical drive, Premium Carbon Fiber casing' , 'It\'s hard to imagine just how compact and sleek the 2.7-pound, 11.1" VAIO TZ truly is. While immediately commanding the attention of anyone who lays eyes on its beautiful form, the featherweight TZ\'s allure goes far beyond aesthetics.' , 1 , 'It\'s hard to imagine just how compact and sleek the 2.7-pound, 11.1" VAIO TZ truly is. While immediately commanding the attention of anyone who lays eyes on its beautiful form, the featherweight TZ\'s allure goes far beyond aesthetics. Concealed in the durable carbon-fiber casing is an ultra-portable powerhouse boasting an Intel Core 2 Duo processor, a 64GB Solid State Drive and a 320GB hard drive for more-than-adequate storage. An external DVDR DL optical drive is also included for you DVD burning and viewing needs. Traveling with your notebook? You\'ll enjoy 4.5 to 8.5 hours of standard battery life, 802.11a/b/g/n3 LAN and integrated wireless WAN technology through Sprint Mobile Broadband for blazing-fast on-the-go connections and the built-in MOTION EYE camera with microphone for video conferencing. Add this all up and you have the VAIO TZthe ultimate ultra-portable PC. <br /><br /><b>Maximum Mobile Performance</b><br />Packed with cutting-edge mobile technology and pre-loaded with 2GB of system memory, experience the performance of an Intel Core 2 Duo U7700 processor (1.33GHz) powered by Intel Centrino Processor technology A perfect mix of power, style and design, the VAIO TZ is a slim, lightweight PC measuring less than 1 thin, wrapped in a durable yet stylish carbon-fiber casing while weighing a very feathery 2.7 lbs. (with standard battery). The VAIO TZ notebook comes pre-installed with Genuine Microsoft Windows Vista Ultimate with Service Pack 1. Get the most from your VAIO TZ notebook while on-the-go. An ultra-efficient power system and superior battery technology lets you enjoy 4.5 to 9.0 hours of standard battery life. <br /><br /><b>Wireless Freedom</b><br />Offering a massive boost in wireless connection speed and range, work and play quicker virtually anywhere within your office or caf&#1081; hotspot using next-generation 802.11n integrated wireless LAN technology. Go beyond LAN networks and caf&#1081; hotspots and find the internet virtually anywhere with integrated Wireless Wide Area Network (WWAN) technology through Sprint Mobile Broadband service. Extend the reach of your TZ notebook with Stereo A2DP Bluetooth technology that is ideal for advanced wireless communication and streaming uncompromised CD-quality stereo music to Bluetooth technology enabled speakers or headphones. <br /><br /><b>Endless Features</b><br />Watch all your favorite movies in stunning clarity on the 11.1 widescreen display using XBRITE-HiColor technology so you can see deeper blacks, brighter whites, true-to-life color and razor-sharp detail from almost any angle. The VAIO TZ includes a 64GB Solid State Drive (SSD) for rapid boot-ups, quicker application launches and greater durability. An additional 320GB Hard Disk Drive is also installed to provide ample room for storage. With the bundled external DVDR DL/ DVDRW optical drive and Click to Disc Editor, present your digital video content in a professional-looking package or archive all your important data to DVDs and CDs. <br /><br /><b>Environmentally Engineered Features</b><br />By using a mercury-free backlit LCD display, the VAIO TZ keeps one less chemical out of both your home and the landfills. The TZ notebook is ENERGY STAR qualified, which means that it uses less power and reduces greenhouse gas emissions by meeting strict efficiency guidelines set by the Environmental Protection Agency and the Department of Energy. The VAIO TZ notebook holds an environmental rating of Silver from EPEAT. ' , 1 , NULL , '11.1" LCD XBRITE-HiColor display, 2GB RAM, hybrid 64GB solid state drive and 320GB hard drive, Sprint Mobile Broadband Wireless WAN, Bluetooth, external optical drive, Premium Carbon Fiber casing' , NULL , 'images/tiny/VGNTZ398UX.jpg' , NULL , 'images/small/VGNTZ398UX.jpg' , NULL , 'images/big/VGNTZ398UX.jpg' , NULL , 'images/super/VGNTZ398UX.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 3999.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 1 , 3499.99 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 6 , 6 , 7 , 1 , NULL , 6 , 1 , 4 , 4 , NULL , 'VAIO TZ Series Notebook PC VGN-TZ398U/XC' , 'VAIO TZ Series Notebook PC VGN-TZ398U/XC' , 'VAIO TZ Series Notebook PC VGN-TZ398U/XC' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , 1 , 1 , '2008-09-08 15:53:50' , '2010-10-27 21:00:32' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (3 , NULL , 'VAIO FW Series Notebook PC VGN-FW145E/W' , NULL , 1 , 5 , -1 , NULL , NULL , 3 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , '16.4" widescreen display, 3GB RAM, 320GB hard drive, HDMI out, built-in camera/microphone and built-in Bluetooth ' , 'Multimedia and computing performance come together with the VGN-FW145E/W VAIO FW Series notebook. An entertainment powerhouse, this notebook is powered by energy-efficient, super-powerful Centrino 2 processor technology.' , 1 , 'Multimedia and computing performance come together with the VGN-FW145E/W VAIO FW Series notebook. An entertainment powerhouse, this notebook is powered by energy-efficient, super-powerful Centrino 2 processor technology. View two web sites side-by-side or fall in love with your favorite movies all over again on the extra-wide, theater inspired 16.4" display (1600x900 resolution) which uses energy-efficient XBRITE-ECO LCD technology. Watch and record DVDs and get the most out of your games with the Intel Core 2 Duo processor. The integrated MOTION EYE camera and microphone lets you stay connected to colleagues, friends and family with real-time video conferencing.\r\n<br /><br /><b>Powerful PC Performance</b><br />\r\nIntel Core 2 Duo P8400 processor (2.26GHz) with Intel Centrino 2 processor technology  the latest chipset from Intel. The VAIO FW notebook comes pre-installed with Genuine Microsoft Windows Vista Home Premium with Service Pack 1. Built-in Wireless-N technology offers a boost in wireless connection speed and range.\r\n<br /><br /><b>Inspired by theater screens. Born from Sony HD technology.</b><br />\r\nFall in love with your favorite movies all over again with an extra-wide, theater inspired 16.4 display (1600x900 resolution) using the energy-efficient XBRITE-ECO LCD technology. The 16:9 aspect ratio ensures that you wont miss an inch of the original picture from your favorite movie while also providing valuable extra workspace. Compare two documents or websites side-by-side without the hassle of having to tab back and forth. An HDMI output delivers video and audio to external sources via a single cable so you can enjoy HD entertainment on your compatible big screen TV without a myriad of extra cables (HDMI cable sold separately).\r\n<br /><br /><b>Endless Features</b><br />\r\nStore you favorite multimedia, important documents, and cherished family photos on the roomy 320GB Serial ATA hard drive. The VAIO FW notebook, designed with dedicated AV Controls, makes navigating through your favorite multimedia quick and easy. Whether its for YouTube, a class project or for sharing with friends and family, editing digital video captured by your Sony Handycam is quick and easy with VAIO Movie Story software. You choose the supplied template and professional-looking cutaways, cross-fades and soundtrack syncing are automatically done for you. Stay connected to colleagues, friends and family with real-time video conferencing using the integrated MOTION EYE camera and microphone.\r\n' , 0 , NULL , NULL , NULL , 'images/tiny/VGNFW145EW.jpg' , NULL , 'images/small/VGNFW145EW.jpg' , NULL , 'images/big/VGNFW145EW.jpg' , NULL , 'images/super/VGNFW145EW.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1199.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 1 , NULL , 2 , NULL , NULL , NULL , NULL , 'VAIO FW Series Notebook PC VGN-FW145E/W' , 'VAIO FW Series Notebook PC VGN-FW145E/W' , 'VAIO FW Series Notebook PC VGN-FW145E/W' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-11 16:54:17' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (4 , NULL , 'VAIO Z Series Notebook PC VGN-Z590UAB' , NULL , 1 , 5 , -1 , NULL , NULL , 3 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , '13.1" widescreen display, 4GB RAM, 128GB solid state drive, Hybrid Graphics System, WWAN through Sprint Mobile Broadband, NVIDIA graphics, HDMI out, Fingerprint sensor, built-in camera/microphone and Bluetooth' , 'Designed with the executive in mind, the VAIO VGN-Z590UAB notebook is built business rugged. Wrapped in carbon fiber, featuring a stunning 13.1" scratch-resistant XBRITE-DuraView LCD display and weighing in at an unbelievable 3.31 pounds.' , 1 , 'Designed with the executive in mind, the VAIO VGN-Z590UAB notebook is built business rugged. Wrapped in carbon fiber, featuring a stunning 13.1" scratch-resistant XBRITE-DuraView LCD display and weighing in at an unbelievable 3.31 pounds, this notebook is designed to withstand the rigors of the road. To complement the durability, this mobile workstation is backed by powerful yet efficient Intel Centrino 2 processor technology and a hybrid graphics system that lets you select your graphics performance on the fly, with no rebooting necessary. The Z also comes with advanced security features to protect your important data and an HDMI output to showcase your content.\r\n<br /><br /><b>Powerful Mobile Performance</b><br />\r\nIntel Core 2 Duo P9500 processor (2.53GHz) with Intel Centrino 2 processor technology  the latest chipset from Intel. The VAIO Z notebook comes pre-installed with Genuine Microsoft Windows Vista Ultimate with Service Pack 1. Uncompromising elegance weaved into a 3.4 lb go-anywhere package. XBRITE-DuraView LCD technology with LED backlight for a brighter, crisper, tougher display.\r\n<br /><br /><b>Wireless Freedom</b><br />\r\nBuilt-in Wireless-N technology offers a boost in wireless connection speed and range. Integrated Sprint Mobile Broadband technology for blazing-fast wireless access beyond LAN networks and hotspots. Extend the reach of your Z notebook with Stereo A2DP Bluetooth technology that is ideal for advanced wireless communication and streaming uncompromised CD-quality stereo music to Bluetooth technology enabled speakers or headphones.\r\n<br /><br /><b>Endless Features</b><br />\r\nHybrid graphics system lets you set graphics performance on the fly  no reboot necessary. An HDMI output delivers video and audio to external sources via a single cable so you can enjoy HD entertainment on your compatible big screen TV without a myriad of extra cables (HDMI cable sold separately). A built-in biometric fingerprint scanner makes authentication and secure access quick and easy  you only need your fingerprint. An integrated Trusted Platform Module (TPM) TCG Ver.1.2 compliant chip enhances security by allowing you to control access on a specific client/user basis.\r\n<br /><br /><b>Environmentally Engineered Features</b><br />\r\nEnergy-efficient 25 watt Intel Centrino 2 processor offers all the performance at lower power consumption. By using a mercury-free backlit LCD display, the VAIO Z keeps one less chemical out of both your home and the landfills. The Z notebook is ENERGY STAR qualified, which means that it uses less power and reduces greenhouse gas emissions by meeting strict efficiency guidelines set by the Environmental Protection Agency and the Department of Energy. The VAIO Z notebook holds an environmental rating of Silver from EPEAT.\r\n' , 0 , NULL , NULL , NULL , 'images/tiny/VGNZ590UAB.jpg' , NULL , 'images/small/VGNZ590UAB.jpg' , NULL , 'images/big/VGNZ590UAB.jpg' , NULL , 'images/super/VGNZ590UAB.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 3999.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 1 , NULL , 2 , NULL , NULL , NULL , NULL , 'VAIO Z Series Notebook PC VGN-Z590UAB' , 'VAIO Z Series Notebook PC VGN-Z590UAB' , 'VAIO Z Series Notebook PC VGN-Z590UAB' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-11 16:54:33' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (5 , NULL , 'VAIO JS Series Desktop PC VGC-JS130J/S' , NULL , 1 , 5 , -1 , NULL , NULL , 3 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , '20.1" XBRITE-HiColor LCD, 4GB pre-installed RAM, 500GB HDD storage and built-in Bluetooth, sleek silver bezel ' , 'Discover your personal style with the eye-catching VAIO VGC-JS130J/S all-in-one computer. Boasting a space-saving design, this PC/TV combo is a powerful desktop that\'s built into the back of a stunning 20.1-inch LCD with XBRITE- HiColor technology.' , 1 , 'Discover your personal style with the eye-catching VAIO VGC-JS130J/S all-in-one computer. Boasting a space-saving design, this PC/TV combo is a powerful desktop that\'s built into the back of a stunning 20.1-inch LCD with XBRITE- HiColor technology. Designed to match any d&eacute;cor, the JS is perfect for a dorm room, office, bedroom or kitchen. Easy to setup and even easier to use, it delivers a powerful computing and entertainment experience. Backed by an Intel Core 2 Duo processor, 4GB RAM and dual 3-watt speakers with Dolby Home Theater technology, the JS is the ideal solution for surfing the web, listening to music and watching movies. The VAIO JS is available in three designer colors including silver, black and pink.\r\n<br /><br /><b>Powerful PC Performance</b><br />\r\nIntel Pentium Dual-Core E5200 processor (2.5GHz) with 4GB of pre-installed system memory. The VAIO JS All-in-One Desktop comes pre-installed with Genuine Microsoft Windows Vista Home Premium 64-bit. Store your digital photos, videos and important documents on the 500GB (7200rpm) Serial ATA hard disk drive.\r\n<br /><br /><b>Endless Features</b><br />\r\nThe 20.1" widescreen display with stunning XBRITE-HiColor LCD technology delivers deeper blacks, brighter whites, true-to-life color and razor-sharp detail from almost any angle. A built-in microphone and MOTION EYE camera with face-tracking technology allows for easy communication with friends and family. Whether its for YouTube, a presentation or your website, editing digital video captured by your Sony Handycam is quick and easy with VAIO Movie Story software. You choose the supplied template and professional looking cutaways, cross-fades and soundtrack syncing are automatically done for you.\r\n<br /><br /><b>Contemporary yet Functional Design</b><br />\r\nFive high-speed USB 2.0 ports and 1 i.LINK interface are conveniently located on the back and sides for easy access and let you attach virtually all your peripherals without having to buy additional hardware. Elegant, yet simple keyboard with isolated key design can be stored below the LCD to save space.' , 0 , NULL , NULL , NULL , 'images/tiny/VGCJS130JS.jpg' , NULL , 'images/small/VGCJS130JS.jpg' , NULL , 'images/big/VGCJS130JS.jpg' , NULL , 'images/super/VGCJS130JS.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1099.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 1 , NULL , 2 , NULL , NULL , NULL , NULL , 'VAIO JS Series Desktop PC VGC-JS130J/S' , 'VAIO JS Series Desktop PC VGC-JS130J/S' , 'VAIO JS Series Desktop PC VGC-JS130J/S' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-11 14:05:58' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (6 , NULL , 'VAIO LV Series PC/TV All-in-One VGC-LV150J' , NULL , 1 , 5 , -1 , NULL , NULL , 3 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , '24" XBRITE-HiColor LCD, 4GB pre-installed RAM, 500GB HDD storage, Blu-ray Disc playback, NVIDIA Graphics with 256MB dedicated VRAM, HDMI in and TV tuners ' , 'Bring your computing and entertainment together with the VAIO VGC-LV150J. This HD PC/TV combines a TV, PC, DVR and Blu-ray Disc player into a sleek, all-in-one solution. Outfitted with a Blu-ray Disc optical drive, you can view your Blu-ray Disc movies in stunning, HD clarity on the gorgeous 24-inch widescreen display.' , 1 , 'Bring your computing and entertainment together with the VAIO VGC-LV150J. This HD PC/TV combines a TV, PC, DVR and Blu-ray Disc player into a sleek, all-in-one solution. Outfitted with a Blu-ray Disc optical drive, you can view your Blu-ray Disc movies in stunning, HD clarity on the gorgeous 24-inch widescreen display. This VAIO LV comes with an integrated DVR and NTSC/ATSC television tuners which let you record your favorite programming including HD channels. A built-in HDMI input makes it easy to hook up your PLAYSTATION3 system or set-top box. Powered by Intel Core2 Duo processor technology with 3MB of L2 cache and 500 GB Serial ATA hard disk drives, this wall-mountable PC/TV is ideal for media enthusiasts who crave brilliant design, peak performance and cutting-edge technology. A wireless mouse and keyboard are also included.\r\n<br /><br /><b>Powerful PC performance</b><br />\r\nIntel Core 2 Duo E7200 processor (2.53GHz) with 3MB of L2 cache and 4GB of pre-installed system memory. The VAIO LV HD PC / TV comes pre-installed with Genuine Microsoft Windows Vista Home Premium 64-bit. Tackle virtually any graphics requirement with a NVIDIA GeForce 9300M GS graphics card and 256MB of dedicated video RAM. Store your digital photos, videos, TV content and important documents on the roomy 500GB (7200rpm) Serial ATA hard disk drive.\r\n<br /><br /><b>Born From Sony HD Technology</b><br />\r\nThe 24.0" wall mountable display with stunning XBRITE-FullHD LCD technology delivers deeper blacks, brighter whites, true-to-life color and razor-sharp detail from almost any angle. A Blu-ray Disc optical drive lets you playback HD Blu-ray Disc movies in crystal-clear resolution. Never miss a minute of your favorite programming again. Integrated NTSC / ATSC TV tuners let you watch and record live TV including local and over-the-air HD channels. An HDMI input lets you connect to compatible cable/satellite set-top-boxes, Sony PS3 systems or other HDMI enabled devices without powering on the PC (HDMI cable sold separately).\r\n<br /><br /><b>Contemporary Yet Functional Design</b><br />\r\nFive high speed USB 2.0 ports and 1 ultra-fast i.LINK interface are conveniently located on the back and sides for easy access and let you attach virtually all of your peripherals without having to buy additional hardware.' , 0 , NULL , NULL , NULL , 'images/tiny/VGCLV150J.jpg' , NULL , 'images/small/VGCLV150J.jpg' , NULL , 'images/big/VGCLV150J.jpg' , NULL , 'images/super/VGCLV150J.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1899.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 1 , NULL , 1 , NULL , NULL , NULL , NULL , 'VAIO LV Series PC/TV All-in-One VGC-LV150J' , 'VAIO LV Series PC/TV All-in-One VGC-LV150J' , 'VAIO LV Series PC/TV All-in-One VGC-LV150J' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-11 14:06:08' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (7 , NULL , 'VAIO LT Series PC/TV All-in-One VGC-LT38E' , NULL , 1 , 5 , -1 , NULL , NULL , 3 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , '22" widescreen display, wall-mountable, NVIDIA graphics, read/write Blu-ray Disc content, wireless keyboard and mouse' , 'Take a good look at what happens when space-saving design meets premium PC performance in this beautifully crafted, all-in-one desktop masterpiece. The VAIO LT HD PC/TV incorporates a stunning floating glass form into a 22" widescreen display with HD clarity and a slim profile.' , 1 , 'Take a good look at what happens when space-saving design meets premium PC performance in this beautifully crafted, all-in-one desktop masterpiece. The VAIO LT HD PC/TV incorporates a stunning floating glass form into a 22" widescreen display with HD clarity and a slim profile. A DVR with massive 1TB hard disk drive stores up to 100 hours of HDTV while a super-powerful Intel Core 2 Duo processor combines with an incredible 4 GB14 RAM to deliver an astonishing high-definition showpiece for media enthusiasts who appreciate brilliant design, peak performance and cutting-edge technology.\r\n<br /><br /><b>Extreme PC Performance</b><br />\r\nThe VAIO LT PC/TV all-in-one is powered by Intels incredible new hafnium-based processor technology. Faster and more efficient than ever, the next-generation Intel Core 2 Duo T8300 processor (2.40GHz), with 3MB of L2 cache and an amazing 4GB of pre-installed system memory, turns up the speed on multi-tasking while curbing energy consumption. The VAIO LT PC/TV all-in-one comes pre-installed with Genuine Microsoft Windows Vista Home Premium with Service Pack 1. Offering a massive boost in wireless connection speed and range, work and play quicker almost anywhere within your office or home using next-generation 802.11n integrated wireless LAN technology. Tackle virtually any graphics requirement with a NVIDIA GeForce 8400M GT GPU graphics card with 256MB of dedicated video RAM.\r\n<br /><br /><b>Stunning HD Entertainment</b><br />\r\nWatch all of your favorite Blu-ray Disc movies in stunning HD on the 22 widescreen display using XBRITE-HiColor technology so you can see deeper blacks, brighter white, true-to-life color and razor-sharp detail from almost any angle. With integrated Blu-ray Disc technology and optical drive, you can create or record 25-50GB of home movies, music and digital photos to a single Blu-ray Disc media (sold separately). The built-in NTSC/ATSC TV tuner lets you enjoy incredibly crisp, live and recorded analog and digital TV to which you subscribe to, on the 22 widescreen flat-panel display. Record your favorite HDTV programming and store your cherished family photos on the massive 1TB hard disk drive.\r\n<br /><br /><b>Contemporary yet Functional Design</b><br />\r\nMount your 22 widescreen display on your bedroom, kitchen or office wall and store the fold-up wireless keyboard and mouse underneath for a chic, space-saving computing experience. Five hi-speed USB 2.0 ports and 1 ultra-fast i.Link interface are conveniently located on the back and sides for easy access and let you attach virtually all of your peripherals without having to buy additional hardware.' , 0 , NULL , NULL , NULL , 'images/tiny/VGCLT38E.jpg' , NULL , 'images/small/VGCLT38E.jpg' , NULL , 'images/big/VGCLT38E.jpg' , NULL , 'images/super/VGCLT38E.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 2299.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 1 , NULL , 2 , NULL , NULL , NULL , NULL , 'VAIO LT Series PC/TV All-in-One VGC-LT38E' , 'VAIO LT Series PC/TV All-in-One VGC-LT38E' , 'VAIO LT Series PC/TV All-in-One VGC-LT38E' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-11 14:06:17' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (8 , NULL , 'Dell XPS One (with Blu-ray drive)' , NULL , 1 , 5 , -1 , NULL , NULL , 4 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , 'Processor: Intel Core 2 Duo E6550 (2.33 GHz); RAM installed: 2 GB DDR2 SDRAM; Hard drive: 500 GB Standard' , 'Best HD-equipped all-in-one; 802.11n and Bluetooth for plenty of wireless flexibility; convenient walk-up controls.' , 1 , '<b>The good:</b><br /> \r\nBest HD-equipped all-in-one; 802.11n and Bluetooth for plenty of wireless flexibility; convenient walk-up controls.\r\n<br /><br /><b>The badThe bad:</b><br /> \r\nIn light of a recent HP Slimline system, PC vendor HD format allegiances now look silly; not as fast as an iMac, not as home-theater-capable as the aforementioned HP; $2,400 is a lot for a secondary home entertainment PC; no video out.\r\n<br /><br /><b>The bottom lineThe bottom line:</b><br /> \r\nIt\'s outclassed as a computer by Apple\'s iMac, and as a home entertainment system by a recent HP, but in the weird niche of high-end, digital-media-friendly all-in-ones (of which we know of one other competitor) the Dell XPS One gets our nod. If you\'re not shopping in that narrow market, move on.\r\n<br /><br /><b>Specifications:</b><br /> \r\nProcessor: Intel Core 2 Duo E6550 (2.33 GHz); RAM installed: 2 GB DDR2 SDRAM; Hard drive: 500 GB Standard ' , 0 , NULL , NULL , NULL , 'images/tiny/XPSOne.jpg' , NULL , 'images/small/XPSOne.jpg' , NULL , 'images/big/XPSOne.jpg' , NULL , 'images/super/XPSOne.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1999.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 1 , NULL , 1 , NULL , NULL , NULL , NULL , 'Dell XPS One (with Blu-ray drive)' , 'Dell XPS One (with Blu-ray drive)' , 'Dell XPS One (with Blu-ray drive)' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-09 10:26:04' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (9 , NULL , 'Dell Studio Hybrid' , NULL , 1 , 5 , -1 , NULL , NULL , 4 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , 'Processor: Intel Core 2 Duo T8100 (2.1 GHz); RAM installed: 2 GB DDR2 SDRAM; Hard drive: 250 GB Standard' , 'Flexible design lets you customize appearance and positioning; best combination of looks and specifications among supersmall desktops.' , 1 , '<b>The good:</b><br />\r\n Flexible design lets you customize appearance and positioning; best combination of looks and specifications among supersmall desktops.\r\n<br /><br /><b>The bad:</b><br />\r\n Standard budget PCs offer better price-performance and more upgradeability; laptops deliver better space economy with similar specifications for the dollar.\r\n<br /><br /><b>The bottom line:</b><br />\r\n If you\'re considering all of the computers in the $700 to $1,000 price range, the Dell Studio Hybrid is not very compelling. However, if you limit yourself to the very specific niche of small PCs, you\'ll appreciate this system\'s clever design and its relatively powerful hardware.\r\n<br /><br /><b>Specifications:</b><br />\r\n Processor: Intel Core 2 Duo T8100 (2.1 GHz); RAM installed: 2 GB DDR2 SDRAM; Hard drive: 250 GB Standard\r\n' , 0 , NULL , NULL , NULL , 'images/tiny/StudioHybrid.jpg' , NULL , 'images/small/StudioHybrid.jpg' , NULL , 'images/big/StudioHybrid.jpg' , NULL , 'images/super/StudioHybrid.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 873.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 1 , NULL , 2 , NULL , NULL , NULL , NULL , 'Dell Studio Hybrid' , 'Dell Studio Hybrid' , 'Dell Studio Hybrid' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-09 10:34:26' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (10 , NULL , 'Dell Latitude XT Laptop Computer (Intel Core 2 U2100 1.06GHz, DDR2 SDRAM 2GB, 40 GB HDD) ' , NULL , 1 , 5 , -1 , NULL , NULL , 4 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , 'Processor: Intel Core 2 Solo (1.06 GHz); RAM installed: 2.0 GB DDR2 SDRAM; Weight: 3.5 lbs' , 'This configuration of the Dell Latitude XT Laptop Computer is a 3.5-pound, ultra portable notebook computer.Compared to other ultra portable notebooks on the market, it is midpriced at around $1,990.Has Wi-Fi (wireless connectivity) built-in.' , 1 , 'This configuration of the Dell Latitude XT Laptop Computer is a 3.5-pound, ultra portable notebook computer.Compared to other ultra portable notebooks on the market, it is midpriced at around $1,990.Has Wi-Fi (wireless connectivity) built-in. \r\n<br /><br /><b>Pros:</b> <br />\r\nHas only a 3.5-pound weight, which is lighter than some similarly priced notebooks. \r\n<br /><br /><b>Cons:</b> <br />\r\nHas only a 40GB hard drive, which is somewhat smaller than some similarly priced notebooks.For example, the Sony VAIO VGN-Z540EBB, a 3.3-pound notebook at around $2, 060, has a 320GB hard drive.Also, this notebook only has a Core 2 Solo processor, which is considerably less efficient than some similarly priced notebooks.For example, the Sony VAIO SZ470N/C, a 3.7-pound notebook at around $1, 900, has a Core 2 Duo processor.The Dell Latitude XT Laptop Computer comes with no-frills word processing software ? if you often create complex documents you will likely need to upgrade to a full-fledged office suite. \r\n<br /><br /><b>Suitability:</b> <br />\r\nA ultra compact notebook, suited for traveling light.Enjoy movies while in transit with this laptop\'s DVD player.Share pictures through the mail: put them on a disc with the CD recorder in this notebook.If you have a wireless network at home, wirelessly browse the Internet from any room in your house with this Wi-Fi enabled notebook. \r\n<br /><br /><b>Value:</b> <br />\r\nIn general, this notebook has great features for the money. \r\n<br /><br /><b>Suggestions:</b> <br />\r\n(1) Given this device\'s Wi-Fi capability, you may want to get a wireless access point for your home, so that you can browse the Web more conveniently from any room.(2) Using this PC\'s Firewire (1394) port, you can connect to many digital video camcorders and other equipment to rapidly upload data to the computer hard drive. \r\n' , 0 , NULL , NULL , NULL , 'images/tiny/LatitudeXT.jpg' , NULL , 'images/small/LatitudeXT.jpg' , NULL , 'images/big/LatitudeXT.jpg' , NULL , 'images/super/LatitudeXT.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1939.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 1 , NULL , 1 , NULL , NULL , NULL , NULL , 'Dell Latitude XT Laptop Computer (Intel Core 2 U2100 1.06GHz, DDR2 SDRAM 2GB, 40 GB HDD) ' , 'Dell Latitude XT Laptop Computer (Intel Core 2 U2100 1.06GHz, DDR2 SDRAM 2GB, 40 GB HDD) ' , 'Dell Latitude XT Laptop Computer (Intel Core 2 U2100 1.06GHz, DDR2 SDRAM 2GB, 40 GB HDD) ' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-09 11:03:12' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (11 , NULL , 'Dell XPS M1730 World of Warcraft Edition Laptop Computer (Intel Core 2 Duo T8300, 200GB HDD, 4GB RAM)' , NULL , 1 , 5 , -1 , NULL , NULL , 4 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , 'Processor: Intel Core 2 Duo (2.4 GHz); RAM installed: 4 GB DDR2 SDRAM; Weight: 10.6 lbs' , 'This configuration of the Dell XPS M1730 is a 10.6-pound, mainstream consumer notebook computer.Compared to other general-purpose notebooks on the market, it is expensive at around $2,500.Has Wi-Fi (wireless connectivity) built-in.Features NVIDIA GeForce 8700M graphics with 256MB of video memory.' , 1 , 'This configuration of the Dell XPS M1730 is a 10.6-pound, mainstream consumer notebook computer.Compared to other general-purpose notebooks on the market, it is expensive at around $2,500.Has Wi-Fi (wireless connectivity) built-in.Features NVIDIA GeForce 8700M graphics with 256MB of video memory. \r\n<br /><br /><b>Pros:</b> <br />\r\nHas 4 GB RAM, which is considerably more than some similarly priced notebooks.Also, this notebook has 1920 x 1200 native screen resolution, which is higher than some similarly priced notebooks. \r\n<br /><br /><b>Cons:</b> <br />\r\nHas a 10.6-pound weight, which is considerably heavier than some similarly priced notebooks.For example, the HP Business nc6400, at around $2, 365, has only a 4.6-pound weight.The Dell XPS M1730 comes with simple word processing software ? if you often need access to special character sets you will likely need to upgrade to a full-fledged office suite. \r\n<br /><br /><b>Suitability:</b> <br />\r\nA general-purpose notebook, suited for general mobile computing.Preserve home movies for future generations by saving them onto DVD with this notebook\'s DVD burner.With 256MB of dedicated video RAM, this machine will do quite well compared to most other notebooks, when it comes to multimedia applications and games.If you have a wireless network at home, wirelessly browse the Internet from any room in your house with this Wi-Fi enabled laptop. \r\n<br /><br /><b>Value:</b> <br />\r\nDespite its expensive price, this notebook has good features for the money.However, if you were to judge strictly by how large the screen is, the average price of consumer level products with a similar screen size is quite a bit less -- in fact around $1,558. \r\n<br /><br /><b>Suggestions:</b> <br />\r\n(1) Given this device\'s Wi-Fi capability, you may want to get a wireless access point for your home, so that you can browse the Internet more conveniently from any room.(2) Using this PC\'s Firewire (1394) port, you can connect to many digital videocameras and other equipment to rapidly upload data to the computer hard drive.\r\n' , 0 , NULL , NULL , NULL , 'images/tiny/XPSM1730.jpg' , NULL , 'images/small/XPSM1730.jpg' , NULL , 'images/big/XPSM1730.jpg' , NULL , 'images/super/XPSM1730.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 2499.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 1 , NULL , 1 , NULL , NULL , NULL , NULL , 'Dell Latitude XT Laptop Computer (Intel Core 2 U2100 1.06GHz, DDR2 SDRAM 2GB, 40 GB HDD) ' , 'Dell Latitude XT Laptop Computer (Intel Core 2 U2100 1.06GHz, DDR2 SDRAM 2GB, 40 GB HDD) ' , 'Dell Latitude XT Laptop Computer (Intel Core 2 U2100 1.06GHz, DDR2 SDRAM 2GB, 40 GB HDD) ' , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-09 11:19:46' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (12 , NULL , 'EOS 5D' , NULL , 1 , 6 , -1 , NULL , NULL , 5 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'The Canon EOS 5D offers advanced photographers a lightweight, robust digital SLR that uses Canon\'s superlative EF lenses without a conversion factor. Its full-frame 12.8 Megapixel CMOS sensor combines with Canon\'s DIGIC II Image Processor, a high-precision 9-point AF system with 6 assist points, and "Picture Style" color control to deliver images of superior quality with enough resolution for any application. With its wide-angle capabilities, 2.5 inch LCD and magnesium-alloy body, the 5D is the perfect addition to the Digital EOS line. ' , NULL , 'The Canon EOS 5D offers advanced photographers a lightweight, robust digital SLR that uses Canon\'s superlative EF lenses without a conversion factor. Its full-frame 12.8 Megapixel CMOS sensor combines with Canon\'s DIGIC II Image Processor, a high-precision 9-point AF system with 6 assist points, and "Picture Style" color control to deliver images of superior quality with enough resolution for any application. With its wide-angle capabilities, 2.5 inch LCD and magnesium-alloy body, the 5D is the perfect addition to the Digital EOS line. ' , 0 , NULL , NULL , NULL , 'images/tiny/EOS5D.jpg' , NULL , 'images/small/EOS5D.jpg' , NULL , 'images/big/EOS5D.jpg' , NULL , 'images/super/EOS5D.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 2499.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , 1 , 1 , '2008-09-09 13:11:48' , '2008-09-09 13:11:48' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (13 , NULL , 'PowerShot TX1' , NULL , 1 , 6 , -1 , NULL , NULL , 5 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'With the PowerShot TX1, Canon takes a futuristic wish-list and makes it a reality. This is an ELPH-sized camera with a whopping 10x optical zoom that not only captures 7.1 megapixel images, but HD movies, too! The stylish vertical design in stainless steel houses plenty of premium advancements including Optical Image Stabilizer Technology, DIGIC III Image Processor, Face Detection Technology and Red-eye Correction. And because you\'ll want to take this compact wonder everywhere, it\'s equipped with a built-in lens cover and tough new scratch-resistant, anti-reflective coating on the LCD screen.  ' , NULL , 'With the PowerShot TX1, Canon takes a futuristic wish-list and makes it a reality. This is an ELPH-sized camera with a whopping 10x optical zoom that not only captures 7.1 megapixel images, but HD movies, too! The stylish vertical design in stainless steel houses plenty of premium advancements including Optical Image Stabilizer Technology, DIGIC III Image Processor, Face Detection Technology and Red-eye Correction. And because you\'ll want to take this compact wonder everywhere, it\'s equipped with a built-in lens cover and tough new scratch-resistant, anti-reflective coating on the LCD screen. ' , 0 , NULL , NULL , NULL , 'images/tiny/PowerShotTX1.jpg' , NULL , 'images/small/PowerShotTX1.jpg' , NULL , 'images/big/PowerShotTX1.jpg' , NULL , 'images/super/PowerShotTX1.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 499.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-09 13:13:42' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (14 , NULL , 'PowerShot SD790 IS' , NULL , 1 , 6 , -1 , NULL , NULL , 5 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'Chiseled edges with a subtle gleam give this PowerShot SD790 IS Digital ELPH distinctive sculptural appeal. Just as attractive are its high-end specifications, including 10 megapixels of resolution plus Face Detection and Motion Detection Technology that delivers the ultimate in crisp, clear, amazingly detailed images. Prepare to be transfixed by the view from the camera\'s 3.0-inch PureColor LCD II screen. With rich, accurate color and superb clarity, the screen is easy to view at wide angles too - perfect for gathering a group around to see your images.  ' , NULL , 'Chiseled edges with a subtle gleam give this PowerShot SD790 IS Digital ELPH distinctive sculptural appeal. Just as attractive are its high-end specifications, including 10 megapixels of resolution plus Face Detection and Motion Detection Technology that delivers the ultimate in crisp, clear, amazingly detailed images. Prepare to be transfixed by the view from the camera\'s 3.0-inch PureColor LCD II screen. With rich, accurate color and superb clarity, the screen is easy to view at wide angles too - perfect for gathering a group around to see your images.  ' , 0 , NULL , NULL , NULL , 'images/tiny/PowerShotSD790IS.jpg' , NULL , 'images/small/PowerShotSD790IS.jpg' , NULL , 'images/big/PowerShotSD790IS.jpg' , NULL , 'images/super/PowerShotSD790IS.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 299.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-09 13:15:22' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (15 , NULL , 'VIXIA HF10' , NULL , 1 , 6 , -1 , NULL , NULL , 5 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'From Canon\'s long history of optical excellence, advanced image processing, superb performance and technological innovation in photographic and broadcast television cameras comes the latest in high definition camcorders. ' , NULL , 'From Canon\'s long history of optical excellence, advanced image processing, superb performance and technological innovation in photographic and broadcast television cameras comes the latest in high definition camcorders.\r\n\r\nNow, with the light, compact Canon VIXIA HF10, you can have stunning AVCHD (Advanced Video Codec High Definition) format recording with the ease and numerous benefits of Flash Memory. It\'s used in some of the world\'s most innovative electronic products such as laptop computers, MP3 players, PDAs and cell phones. Canon offers the best in Flash Memory with Dual Flash Memory- record to both the camcorder\'s 16 GB internal memory and a removable SDHC card, extending your available recording space and offering added flexibility in file transfer and playback.\r\n\r\nAdd to that the VIXIA HF10\'s Canon Exclusive features such as our own 3.3 Megapixel Full HD CMOS sensor and advanced DIGIC DV II Image Processor, SuperRange Optical Image Stabilization, Instant Auto Focus, our 2.7" Widescreen Multi-Angle Vivid LCD and the Genuine Canon 12x HD video zoom lens and you have a Flash Memory camcorder that\'s hard to beat and unmistakably Canon.' , 0 , NULL , NULL , NULL , 'images/tiny/VIXIAHF10.jpg' , NULL , 'images/small/VIXIAHF10.jpg' , NULL , 'images/big/VIXIAHF10.jpg' , NULL , 'images/super/VIXIAHF10.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1099.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-09 13:19:11' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (16 , NULL , 'FS10' , NULL , 1 , 6 , -1 , NULL , NULL , 5 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'From Canon\'s long history of optical excellence, advanced image processing, superb performance and technological innovation in photographic and broadcast television cameras comes the latest in camcorders.' , NULL , 'From Canon\'s long history of optical excellence, advanced image processing, superb performance and technological innovation in photographic and broadcast television cameras comes the latest in camcorders.\r\n\r\nNow, with the light, compact Canon FS10, you can have stunning video with the ease and numerous benefits of Flash Memory, at an affordable price. Flash Memory is used in some of the world\'s most innovative electronic products such as laptop computers, MP3 players, PDAs and cell phones. Canon offers the best in Flash Memory with Dual Flash Memory- record to both the camcorder\'s 8GB internal memory and a removable SDHC card, extending your available recording space and offering added flexibility in file transfer and playback. It\'s just the thing for catching more of your life\'s most special moments.\r\n\r\nAdd to that the FS10\'s Canon Exclusive features such as Canon 48x Advanced Zoom and our DIGIC DV II Image Processor, combined with a 1.07 Megapixel CCD image sensor, Image Stabilization and a 2.7" Widescreen LCD -- and you truly have a Flash Memory camcorder that\'s both hard to beat and unmistakably Canon.' , 0 , NULL , NULL , NULL , 'images/tiny/fs10.jpg' , NULL , 'images/small/fs10.jpg' , NULL , 'images/big/fs10.jpg' , NULL , 'images/super/fs10.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 499.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-09 13:20:29' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (17 , NULL , 'DC320' , NULL , 1 , 6 , -1 , NULL , NULL , 5 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'Around the world, the name Canon means optical excellence, advanced image processing and superb performance. And Canon digital video camcorders are no exception.' , NULL , 'Around the world, the name Canon means optical excellence, advanced image processing and superb performance. And Canon digital video camcorders are no exception.\r\n\r\nIf your choice of recording media is DVD, you\'ll find a family of capable and technologically sophisticated DVD camcorders built with the Canon know how which you\'ve come to expect in all our products.\r\n\r\nAffordable, sleek and easy-to-operate, the Canon DC320 gives you the convenience of DVD technology, along with our exclusive Canon 48x Advanced Zoom- a feature only possible with our industry-leading optical technology. It also boasts our advanced DIGIC DV II image processor, a 1.07 Megapixel CCD, SD card image storage capability, Quick Start and Joystick Control. Add to that the long recording time available thanks to its high capacity battery and low power consumption and you\'ll see that the DC320 makes it simpler than ever to capture and share all the unforgettable moments of your life.' , 0 , NULL , NULL , NULL , 'images/tiny/dc320.jpg' , NULL , 'images/small/dc320.jpg' , NULL , 'images/big/dc320.jpg' , NULL , 'images/super/dc320.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 369.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-09 13:21:30' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (18 , NULL , 'Nikon D80' , NULL , 1 , 6 , -1 , NULL , NULL , 8 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , '10.2 effective megapixel Nikon DX Format CCD image sensor<br />\r\nHigh-speed continuous shooting: 3 frames per second (fps) in bursts of up to 100 consecutive JPEG (FINE M-size or smaller) or 6 RAW (NEF) images<br />\r\nAdvanced high-precision, high-performance imaging processing engine with color-independent pre-conditioning' , 1 , '<ul>\r\n<li>10.2 effective megapixel Nikon DX Format CCD image sensor</li>\r\n<li>High-speed continuous shooting: 3 frames per second (fps) in bursts of up to 100 consecutive JPEG (FINE M-size or smaller) or 6 RAW (NEF) images</li>\r\n<li>Advanced high-precision, high-performance imaging processing engine with color-independent pre-conditioning</li>\r\n<li>3D-Color Matrix Metering II with 420-pixel RGB sensor delivers consistent and dependable automatic exposure for ideal results in most lighting conditions</li>\r\n<li>Refined 11-area AF system with new Auto-area AF mode and center sensor that can be switched to wide-frame operation for broader coverage</li>\r\n<li>ISO AUTO mode automatically adjusts sensitivity between ISO 100 to 1600, maximizing available light to help achieve optimal exposure</li>\r\n<li>Seven automated Digital Vari-Programs (Auto, Portrait, Landscape, Close Up, Sports, Night Landscape and Night Portrait) optimize white balance, sharpening, tone, color, saturation and hue to match the scene.</li>\r\n<li>User-selectable choice of Normal, Softer, Vivid, More vivid, Portrait, Custom and Black-and-white image optimization options</li>\r\n<li>Near-instant response with 0.18 sec. power-up and approx. 80-millisecond shutter release time lag promotes fast handling</li>\r\n<li>Top shutter speed of 1/4,000 second and flash sync speeds up to 1/200 second</li>\r\n<li>Fast image transfer via USB 2.0 Hi-Speed interface and SD memory card </li>\r\n<li>Creative in-camera effects and editing functions consolidated under the new Retouch menu, including D-Lighting, Red-eye correction, Trim, Image Overlay, Monochrome settings (Black-and-white, Sepia, Cyanotype) and Filter Effects (Skylight, Warm filter, Color balance)</li>\r\n<li>Multiple Exposure shooting option automatically produces an effect that resembles multiple exposure techniques used with film</li>\r\n<li>Large 2.5-inch LCD monitor with ultra-wide 170-degree viewing angle for clear image preview and easy access to settings and information, including RGB Histograms</li>\r\n<li>Selectable Slideshow function (Standard or Pictmotion)</li>\r\n<li>SD memory card storage, SDHC compatible</li>\r\n<li>Lightweight, compact body</li>\r\n<li>High-energy EN-EL3e rechargeable lithium-ion battery delivers the power to shoot up to 2,700 pictures on a single charge and provides detailed battery status information. (Battery life figure determined by in-house test parameters)</li>\r\n<li>Built-in Flash with i-TTL flash control and full support for Nikon\'s Creative Lighting System</li>\r\n<li>The D80 supports more than 43 AF NIKKOR lenses in addition to the growing family of DX NIKKOR lenses</li>\r\n<li>Includes Nikons PictureProject software for easy control over image adjustment and management</li>\r\n<li>Support for Nikon\'s new Capture NX software, which provides easier access to powerful and visually intuitive enhancement tools that help tap the full potential of NEF images</li>\r\n</ul>\r\n' , 0 , NULL , NULL , NULL , 'images/tiny/D80.jpg' , NULL , 'images/small/D80.jpg' , NULL , 'images/big/D80.jpg' , NULL , 'images/super/D80.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1299.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-09 13:27:41' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (19 , NULL , 'COOLPIX S600' , NULL , 1 , 6 , -1 , NULL , NULL , 8 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'Worlds smallest body<br />\r\nWorlds fastest start-up time<br />\r\n10.0 effective megapixels for high-resolution images' , 1 , '<ul>\r\n  <li>Worlds smallest body</li>\r\n  <li>Worlds fastest start-up time</li>\r\n  <li>10.0 effective megapixels for high-resolution images</li>\r\n  <li>Optical 4x Zoom-NIKKOR lens</li>\r\n  <li>High performance optical lens shift VR image stabilization</li>\r\n  <li>High sensitivity up to ISO 3200, High-Sensitivity mode</li>\r\n  <li>2.7-inch high-resolution LCD monitor with anti-reflection coating</li>\r\n  <li>Active Child mode for automatic tracking and focusing on a subject</li>\r\n  <li>Pastel mode for soft, impressionistic images</li>\r\n  <li>Three Unique Nikon Image Innovations for high performance  Face-priority AF, In-Camera Red-Eye Fix and D-Lighting</li>\r\n</ul>' , 0 , NULL , NULL , NULL , 'images/tiny/COOLPIXS600.jpg' , NULL , 'images/small/COOLPIXS600.jpg' , NULL , 'images/big/COOLPIXS600.jpg' , NULL , 'images/super/COOLPIXS600.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 234.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-09 13:31:00' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (20 , NULL , 'High Definition Handycam Camcorder HDR-TG1' , NULL , 1 , 6 , -1 , NULL , NULL , 3 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'Shoot amazingly crisp HD footage with an unbelievably small camcorder that lives up to the Sony name. The HDR-TG1 Handycam camcorder records Full HD 1920 x 1080 resolution video from a lightweight, slim body crafted from pure titanium.' , 1 , 'Shoot amazingly crisp HD footage with an unbelievably small camcorder that lives up to the Sony name. The HDR-TG1 Handycam camcorder records Full HD 1920 x 1080 resolution video from a lightweight, slim body crafted from pure titanium. It\'s tough enough for everyday HD video shooting and perfect for travel with its compact size that won\'t take up much room in your bag. The stylish design incorporates a Premium Hard Coating for superior resistance to even the smallest scratches. It records to convenient Memory Stick PRO Duo media, and Sony\'s ClearVid CMOS Sensor (with Exmor derived technology) and BIONZ image processor provide spectacular video and stunning 4.0 megapixel still images. Face Detection technology is also included, which automatically finds and enhances faces for beautiful results.\r\n<br /><br />\r\n<b>1920x1080 Recording</b><br />\r\nRecord stunning, incredibly detailed video footage in the clarity of 1920 x 1080 Full HD resolution.\r\n<br /><br />\r\n<b>Lightweight, Pure Titanium Body</b><br />\r\nSlim and lightweight -- only 8oz without the battery -- the HDR-TG1s body is constructed of high-quality pure titanium. Additionally, a Premium Hard Coating is applied on the body surface (except for hinge and lens ring area), providing superior resistance to scratches and fingerprints.\r\n<br /><br />\r\n<b>Face Detection for Video Footage and Digital Still Photos</b><br />\r\nMade possible by the BIONZ image processing engine, Face Detection technology recognizes up to 8 faces anywhere in the frame and automatically controls focus, exposure, and color to help capture smiling faces brightly and clearly. Face Detection technology also helps make skin tones look natural without affecting other colors in the image.\r\n<br /><br />\r\n<b>2..3 Megapixel ClearVid CMOS sensor (with Exmor derived technology) and BIONZ image processor</b><br />\r\nSony\'s newly designed CMOS image sensor delivers bright, vivid, and spectacularly detailed 2.3 megapixel (effective) video footage, as well as 4.0 megapixel still images -- all with prolonged battery life. Exmor technology captures images with high resolution and high sensitivity. In addition, the advanced Sony BIONZ image processor improves camera response time and screens out noise so the signal from each pixel is as pure as possible.\r\n<br /><br />\r\n<b>Supremely Mobile Design</b><br />\r\nSony has created a camcorder with the ability to fit in a shirt pocket, yet still capture stunning Full HD resolution video and 4.0 megapixel still images. Whether for personal or business use, around town or on vacation, the HDR-TG1 is the ideal travel companion for capturing those special memories.\r\n<br /><br />\r\n<b>2..7"1 Wide (16:9) Hybrid Touch Panel Clear Photo LCD Plus Display (211K Pixels)</b><br />\r\nThe 2.7"1 Wide (16:9) LCD screen provides exceptional viewing clarity with improved resolution (211K pixels). The display rotates up to 270 degrees for multiple viewing angles, as well as provides sharp, detailed images for monitoring or playback. The widescreen format makes shooting in 16:9 mode and composing shots even easier.' , 0 , NULL , NULL , NULL , 'images/tiny/HDRTG1.jpg' , NULL , 'images/small/HDRTG1.jpg' , NULL , 'images/big/HDRTG1.jpg' , NULL , 'images/super/HDRTG1.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 899.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-09 13:46:55' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (21 , NULL , 'High Definition Handycam Camcorder HDR-CX12' , NULL , 1 , 6 , -1 , NULL , NULL , 3 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'The ultra-sleek, palm-sized HDR-CX12 Handycam camcorder is the world\'s first camcorder with Face Detection and Smile Shutter technology. Face Detection automatically adjusts focus, exposure and color for the most natural results.' , 1 , 'The ultra-sleek, palm-sized HDR-CX12 Handycam camcorder is the world\'s first camcorder with Face Detection and Smile Shutter technology. Face Detection automatically adjusts focus, exposure and color for the most natural results. With Smile Shutter, the HDR-CX12 can detect smiles and automatically capture still images, even while recording 1920 x 1080 Full HD video. You will also enjoy 10.2 megapixel still image recording to ensure your memories are preserved in stunning detail. Additionally, the CX12\'s small size and light weight make it an ideal travel companion, perfect for recording that special moment, wherever you are. Video and still images are stored directly to Memory Stick PRO Duo media (4GB supplied) for easy transfer to your computer or other compatible devices.\r\n<br /><br />\r\n<b>1920x1080 Recording</b><br />\r\nRecord stunning, incredibly detailed video footage in the clarity of 1920 x 1080 Full High Definition8 resolution.\r\n<br /><br />\r\n<b>10.2 megapixel still image capture</b><br />\r\nCarry only one compact camera for both video and still. Capture up to 10.2 megapixel still image for high resolution digital photos.\r\n<br /><br />\r\n<b>Memory Stick PRO Duo Media1</b><br />\r\nCapture still images directly onto Memory Stick PRO Duo media1 (4GB4 included) for easy transfer to PCs for emailing, printing, or sharing with other compatible Memory Stick devices (sold separately).\r\n<br /><br />\r\n<b>Smile Shutter technology</b><br />\r\nSmile Shutter technology lets the camcorder automatically take still photos when your subject smiles, even while shooting High Definition video. You can also set the Smile Shutter sensitivity and choose child, adult, or auto priority.\r\n<br /><br />\r\n<b>Face Detection for Video Footage and Digital Still Photos</b><br />\r\nMade possible by the BIONZ image processing engine, Face Detection technology recognizes up to 8 faces anywhere in the frame and automatically controls focus, exposure, and color to help capture smiling faces brightly and clearly. Face Detection also helps make skin tones look natural without affecting other colors in the image.\r\n<br /><br />\r\n<b>5 Megapixel ClearVid CMOS Sensor (with Exmor-derived technology) and BIONZ Image Processor</b><br />\r\nSonys newly designed CMOS image sensor delivers bright, vivid, and spectacularly detailed 5 megapixel (effective) video footage, as well as 10.2 megapixel still images -- all with prolonged battery life. Exmor technology captures images with high resolution and high sensitivity. In addition, the advanced Sony BIONZ image processor improves camera response time and screens out noise so the signal from each pixel is as pure as possible.\r\n\r\n' , 0 , NULL , NULL , NULL , 'images/tiny/HDRCX12.jpg' , NULL , 'images/small/HDRCX12.jpg' , NULL , 'images/big/HDRCX12.jpg' , NULL , 'images/super/HDRCX12.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 899.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-09 13:50:23' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (22 , NULL , 'Flat TV 42" LCD DVB-T  42PFL9900D/10' , NULL , 1 , 7 , -1 , NULL , NULL , 9 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , 'Let the beauty of light transform your viewing experience with the Aurea Flat TV. The Ambilight Spectra with active frame is blended in a stunning design. With Perfect Pixel HD Engine and Full HD 1080p LCD display, seeing is believing.' , 'Let the beauty of light transform your viewing experience with the Aurea Flat TV. The Ambilight Spectra with active frame is blended in a stunning design. With Perfect Pixel HD Engine and Full HD 1080p LCD display, seeing is believing.' , 0 , 'Ambilight Spectra with Active Frame and sensorial halo\r\n\r\nThe sensorial halo of Ambilight Spectra adds a magical dimension to your viewing experience. The Active Frame extends the light and atmosphere of the image far outside the screen in a soft kaleidoscope of colours. You feel fully immersed in the images, viewing becomes a more intensive experience than you could ever imagine. The compact LED technology enables a total of 126 power LEDs to be placed around the screen. This creates a multitude of segments that fuse into one soft surround of light in the Active Frame, directly enveloping the screen and flowing seamlessly into a soft halo on the wall.\r\n\r\nFull HD LCD display, with a 1920 x 1080p resolution\r\n\r\nThe Full HD screen has the widescreen resolution of 1920 x 1080p. This is the highest resolution of HD sources for the best possible picture quality. It is fully future-proof as it supports 1080p signals from all sources, including the most recent sources such as Blu-ray and advanced HD game consoles. The signal processing is extensively upgraded to support this much higher signal quality and resolution. It produces brilliant flicker-free progressive scan pictures with superb brightness and colours.\r\n\r\nPerfect Pixel HD Engine for unrivalled sharpness and clarity\r\n\r\nPerfect Pixel HD Engine offers the unique combination of ultimate sharpness, natural detail, vivid colours and smooth natural motion on all qualities of HD, standard TV signals and multimedia content, for high-definition displays. Each pixel of the incoming picture is enhanced to better match the surrounding pixels, resulting in a more natural picture. Artefacts and noise in all sources from multimedia to standard TV and also in highly compressed HD content are detected and reduced, ensuring that the picture is clean and razor sharp.' , 0 , NULL , NULL , NULL , 'images/tiny/42PFL9900D.jpg' , NULL , 'images/small/42PFL9900D.jpg' , NULL , 'images/big/42PFL9900D.jpg' , NULL , 'images/super/42PFL9900D.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 1999.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , 1 , 1 , '2008-09-09 18:05:32' , '2008-09-09 18:05:32' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (23 , NULL , 'Flat TV 52" LCD integrated digital  52PFL9632D/10' , NULL , 1 , 7 , -1 , NULL , NULL , 9 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , 'Enjoy the ultimate viewing experience of this Philips Cineos Flat TV with Perfect Pixel HD Engine, Ambilight 2 Channel, a Full HD 1080p display and 100 Hz Clear LCD technology. The USB connector gives you full access to multimedia content.' , 'Enjoy the ultimate viewing experience of this Philips Cineos Flat TV with Perfect Pixel HD Engine, Ambilight 2 Channel, a Full HD 1080p display and 100 Hz Clear LCD technology. The USB connector gives you full access to multimedia content.' , 0 , 'Full HD LCD display, with a 1920 x 1080p resolution\r\n\r\nThe Full HD screen has the widescreen resolution of 1920 x 1080p. This is the highest resolution of HD sources for the best possible picture quality. It is fully future-proof as it supports 1080p signals from all sources, including the most recent sources such as Blu-ray and advanced HD game consoles. The signal processing is extensively upgraded to support this much higher signal quality and resolution. It produces brilliant flicker-free progressive scan pictures with superb brightness and colours.\r\n\r\nIntegrated Digital Tuner for DVB-T reception\r\n\r\nThe integrated DVB-T tuner lets you receive Digital Terrestrial TV to watch or record. This means you do not need an additional set-top box or additional cables.\r\n\r\nHD Ready for the highest quality display of HD signals\r\n\r\nEnjoy the exceptional picture quality of High Definition pictures and be fully prepared for HD sources like HDTV set-top boxes and Blu-ray discs. HD Ready is a protected label that offers picture quality beyond that of progressive scan. It conforms to strict standards laid out by EICTA to offer an HD screen that displays the benefits of resolution and picture quality of a High Definition signal. It has a universal connection for both analogue YPbPr and uncompressed digital connection of DVI or HDMI, supporting HDCP. It can display 720p and 1080i signals at 50 and 60 Hz.\r\n\r\nPerfect Pixel HD Engine for unrivalled sharpness and clarity\r\n\r\nPerfect Pixel HD Engine offers the unique combination of ultimate sharpness, natural detail, vivid colours and smooth natural motion on all qualities of HD, standard TV signals and multimedia content, for high-definition displays. Each pixel of the incoming picture is enhanced to better match the surrounding pixels, resulting in a more natural picture. Artefacts and noise in all sources from multimedia to standard TV and also in highly compressed HD content are detected and reduced, ensuring that the picture is clean and razor sharp.' , 0 , NULL , NULL , NULL , 'images/tiny/52PFL9632D.jpg' , NULL , 'images/small/52PFL9632D.jpg' , NULL , 'images/big/52PFL9632D.jpg' , NULL , 'images/super/52PFL9632D.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 3999.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-09 18:09:02' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (24 , NULL , '52" Class 1920 x 1080 Flat Panel LCD LT-52X579 ' , NULL , 1 , 7 , -1 , NULL , NULL , 7 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , 'This panel provides high brightness, a wide viewing angle, and a flicker-free high resolution picture that scales all video sources to display at 1080p.' , 'For 2008 JVCs line-up of widescreen Full HD 1080p flat panel displays bring a new dimension to home theater, literally. Our line-up of panels was not only engineered to excite the most extreme audio visual enthusiasts, but also to influence the creative minds of the most discriminating home decorators.' , 0 , 'For 2008 JVCs line-up of widescreen Full HD 1080p flat panel displays bring a new dimension to home theater, literally. Our line-up of panels was not only engineered to excite the most extreme audio visual enthusiasts, but also to influence the creative minds of the most discriminating home decorators.\r\n\r\nJVC\'s LCD televisions, with stylish cosmetics, takes into consideration every feature the demanding digital customer is looking for now and for the future. It offers 176 Degree Viewing Angle. This units connectivity takes into consideration todays digital world and includes an ATSC Tuner, 3 1080p HDMI/HDCP Inputs, a side USB Photo Viewer and 2 Component Video Inputs. Ideal for any room in the house!' , 0 , NULL , NULL , NULL , 'images/tiny/LT-52X579.jpg' , NULL , 'images/small/LT-52X579.jpg' , NULL , 'images/big/LT-52X579.jpg' , NULL , 'images/super/LT-52X579.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 3499.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-09 18:12:20' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (25 , NULL , 'BRAVIA SXRD 1080p Home Cinema Front Projector VPL-VW200' , NULL , 1 , 7 , -1 , NULL , NULL , 3 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'This is it. The flagship of Sony\'s home theater lineup. This projector combines the advanced features and technology associated with the legendary QUALIA 004 and the sleek form factor made popular by the VPL-VW100 projector.' , 0 , 'This is it. The flagship of Sony\'s home theater lineup. This projector combines the advanced features and technology associated with the legendary QUALIA 004 and the sleek form factor made popular by the VPL-VW100 projector. Three newly developed high frame rate (120fps) SXRD 1080P chips work together with Sony\'s Motionflow technology and black frame insertion modes for smoother, more detailed pictures in fast motion video. The Panel Alignment function allows the precise alignment of R/G/B images with zone specific adjustment to deliver a sharper, crisper picture. An exclusive Carl Zeiss multi-element glass lens optimizes SXRD chip performance. The Anamorphic Zoom Mode works with an optional external anamorphic lens to allow 2.35:1 aspect ratio viewing. A high output Xenon cinematic lamp maintains ideal color temperature characteristics for natural and accurate color reproduction. Additional color features include RCP (Real Color Processing) and x.v.Color technology for further color refinement. The significantly improved native contrast characteristics of the three SXRD chips are dynamically enhanced by the Advanced Auto Iris 2 function for up to a 35,000:1 contrast ratio. Add 24p True Cinema connectivity, RS-232C capability and two HDMI inputs and the VPL-VW200 is destined to be the ultimate choice among videophiles everywhere.\r\n\r\nSXRD 1920 x 1080 high frame rate (120fps) SXRD chips (0.61")\r\nSXRD is a new display technology developed by the legendary television engineers at Sony to meet and exceed the demands of a High Definition image at its full 1080 line resolution. It is a 1920 x 1080p panel characterized by several key benefits -- full HD resolution, smooth film-like image with minimal screen door effects, high response times (5ms rise and fall), high contrast ratios and accurate color rendition.\r\n\r\nMotionflow 120Hz High Frame Rate Technology\r\nTaking motion performance to the next level requires innovation and expertise. Enter Sony\'s Motionflow High Frame Rate technology. Motionflow detects the incoming video signal and applies the appropriate processing for optimum motion reproduction. Taking full advantage of film sourced 24fps encoded content available on DVD and Blu-ray Disc, Motionflow eliminates the need for 3:2 pulldown and delivers smooth, judder free video, faithfully preserving the integrity of the original film. When 60fps content is detected, Motionflow doubles the amount of frames and uses real-time calculation to create a new level of natural motion reproduction. You\'ll experience movies and sports with a greater sense of realism than ever before.\r\n\r\n' , 0 , NULL , NULL , NULL , 'images/tiny/VPLVW200.jpg' , NULL , 'images/small/VPLVW200.jpg' , NULL , 'images/big/VPLVW200.jpg' , NULL , 'images/super/VPLVW200.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 14999.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-09 18:17:01' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (26 , NULL , '40" BRAVIA XBR LCD Flat Panel HDTV KDL-40XBR6' , NULL , 1 , 7 , -1 , NULL , NULL , 3 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'Performance takes center stage with the KDL-40XBR6 BRAVIA XBR LCD flat panel HDTV. Chockfull of features including HD 1080p resolution and Motionflow 120Hz technology which provide smooth, natural images. Plus, BRAVIA Engine EX fully digital video processing enhances overall image detail making every scene sharp, vibrant and life-like. ' , 0 , 'Performance takes center stage with the KDL-40XBR6 BRAVIA XBR LCD flat panel HDTV. Chockfull of features including HD 1080p resolution and Motionflow 120Hz technology which provide smooth, natural images. Plus, BRAVIA Engine EX fully digital video processing enhances overall image detail making every scene sharp, vibrant and life-like. You\'ll notice deeper blacks with Advanced Contrast Enhancer (ACE), while 24p True Cinema capability helps deliver the cinematic experience when used with 24p output devices, such as a Blu-ray Disc player. There\'s also numerous inputs including four HDMI connections, two HD component connections, Digital Media Port and a PC input that offer you a wide range of connectivity options. And with the gorgeous piano gloss black finish, you\'ll be wondering if the view is nicer with the TV on or off.\r\n\r\n16:9 Full HD 1080p Resolution Panel\r\n16:9 Full HD 1080p resolution panel (1920 x 1080) accepts any HDTV signal and renders it with optimum resolution.\r\n\r\nMotionflow 120Hz technology\r\nMotionflow 120HZ technology doubles the frame rate to help remove judder in film sources, HD content, and broadcast movies and primetime programming, creating smoother, more fluid motion that delivers a natural and realistic viewing experience.\r\n\r\n' , 0 , NULL , NULL , NULL , 'images/tiny/KDL40XBR6.jpg' , NULL , 'images/small/KDL40XBR6.jpg' , NULL , 'images/big/KDL40XBR6.jpg' , NULL , 'images/super/KDL40XBR6.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 2499.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-09 18:16:41' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (27 , NULL , 'BRAVIA Theater Micro System - black speakers DAV-IS10' , NULL , 1 , 7 , -1 , NULL , NULL , 3 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'Witness true theater-quality sound from the comfort of your home, without the need for obtrusive speakers. The DAV-IS10 5.1 micro satellite home theater system delivers big sound while complementing your dcor. Its unique, compact design and advanced audio capabilities will add a new dimension to your movie-watching experience, without the unattractive bulk of other systems.' , 0 , 'Witness true theater-quality sound from the comfort of your home, without the need for obtrusive speakers. The DAV-IS10 5.1 micro satellite home theater system delivers big sound while complementing your dcor. Its unique, compact design and advanced audio capabilities will add a new dimension to your movie-watching experience, without the unattractive bulk of other systems. And with the touch of a button on the included remote, BRAVIA Theatre Sync lets you manage your Sony HDMI components quickly and effortlessly. Speakers also available in White. \r\n\r\nYou can also expand your listening options by connecting your iPod directly to the included iPod Digital Media Port cradle (TDM-IP1). This unique accessory lets you use the DAV-IS10 remote control to browse your iPod music and video collection directly on your TV, and brings it all alive through the powerful speakers.\r\n' , 0 , NULL , NULL , NULL , 'images/tiny/DAVIS10.jpg' , NULL , 'images/small/DAVIS10.jpg' , NULL , 'images/big/DAVIS10.jpg' , NULL , 'images/super/DAVIS10.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 699.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-11 10:21:55' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (28 , NULL , 'BRAVIA Theater System DAV-HDX576WF' , NULL , 1 , 7 , -1 , NULL , NULL , 3 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'The Sony DAV-HDX576WF 5.1 channel BRAVIA Theatre System has an integrated 5-disc DVD/CD changer that will upscale your DVD\'s to 1080p through the HDMI connection. Plus, this amazing system uses S-AIR technology to eliminate speaker wires running from your system to the rear surround speakers. For easy multi-room audio, it also includes an AirStation speaker that let\'s you listen to music in another room of the house.' , 0 , 'The Sony DAV-HDX576WF 5.1 channel BRAVIA Theatre System has an integrated 5-disc DVD/CD changer that will upscale your DVD\'s to 1080p through the HDMI connection. Plus, this amazing system uses S-AIR technology to eliminate speaker wires running from your system to the rear surround speakers. For easy multi-room audio, it also includes an AirStation speaker that let\'s you listen to music in another room of the house.\r\n\r\nS-AIR Technology\r\nMulti-room listening made easy. Free your music.From room. To room. To room. AirStation automatically syncs to allow multi-room listening through simple, wireless plug-and-play technology while letting you control what you listen to.\r\n\r\nBRAVIA Sync\r\nAs a leader in products with HDMI connectivity, Sony has created BRAVIA Theatre Sync to go beyond simple audio and video transmission. When combined with a BRAVIA Theatre Sync-ready BRAVIA Televisions and other Sony audio and video products, in addition to audio and video transmission, control functions will be transmitted via the HDMI cable. With One Touch Play, operations that took several steps have been reduced to one. Press the PLAY button on the remote, watch the audio and/or the video component(s) turn on then the BRAVIA television turn on and switch to the correct video inputSimpleEasyBRAVIA Theatre Sync.\r\n\r\n\r\n' , 0 , NULL , NULL , NULL , 'images/tiny/DAVHDX576WF.jpg' , NULL , 'images/small/DAVHDX576WF.jpg' , NULL , 'images/big/DAVHDX576WF.jpg' , NULL , 'images/super/DAVHDX576WF.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 499.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-11 10:32:24' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (29 , NULL , 'Multi-Media Sound Solution TH-L1' , NULL , 1 , 7 , -1 , NULL , NULL , 7 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , '5.1ch Surround Media System for Multi Sources with 5 Cubic Front/Center/Surround Speakers and 6 5/6 (16cm) Subwoofer - featuring HDMI Input (1080p Pass - Through)/Output, Connection for iPod, and USB Host' , 0 , '5.1ch Surround Media System for Multi Sources with 5 Cubic Front/Center/Surround Speakers and 6 5/6 (16cm) Subwoofer - featuring HDMI Input (1080p Pass - Through)/Output, Connection for iPod, and USB Host\r\n\r\n    * Dolby Digital, DTS and Dolby Pro Logic II Decoders Built-in\r\n    * Plays MP3/WMA/WAV/JPEG AND MPEG-4 (ASF)/-2/-1 (USB)\r\n    * Progressive Scan Output\r\n    * HDMI Input (X2; 1080p Pass-Through) and Output\r\n    * USB Host\r\n' , 0 , NULL , NULL , NULL , 'images/tiny/THL1.jpg' , NULL , 'images/small/THL1.jpg' , NULL , 'images/big/THL1.jpg' , NULL , 'images/super/THL1.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 299.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-11 10:35:19' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (30 , NULL , 'DVD Home Theatre System  HTS6600/05' , NULL , 1 , 7 , -1 , NULL , NULL , 9 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , 'Featuring Ambisound, the HTS6600 delivers an embracing multi-channel surround experience from just two speakers, in any sort of room. Providing excellent sound and picture quality, the easy-setup wall-mountable system fits into any home.' , 'Featuring Ambisound, the HTS6600 delivers an embracing multi-channel surround experience from just two speakers, in any sort of room. Providing excellent sound and picture quality, the easy-setup wall-mountable system fits into any home.' , 0 , 'Ambisound for full no-fuss multi-channel surround sound\r\n\r\nAmbisound technology produces truly embracing multi-channel surround sound from fewer speakers. It delivers an immersive 5.1 sound experience without the constraints of sitting position, room shape or size, and without the clutter of multiple speakers and cables. Through the combined effect of psychoacoustic phenomena, array processing and precisely angled driver positioning, Ambisound delivers more sound from fewer speakers.\r\n\r\nHDMI 1080i with high definition video upscaling\r\n\r\nHDMI stands for High Definition Multimedia Interface. It is a direct digital connection that can carry digital HD video as well as digital multi-channel audio. By eliminating the conversion to analogue signals it delivers perfect picture and sound quality, completely free from noise. HDMI is fully backwards-compatible with DVI (Digital Video Interface).\r\n\r\nSmart Surround optimises surround settings automatically\r\n\r\nEnjoy the best possible listening experience without having to make surround setting changes every time you change a disc or switch between movies and music. Unlike normal 5.1 systems that feature default multi-channel surround settings, Smart Surround automatically changes the surround settings to suit your music encoded in stereo or your movie with multi-channel content.' , 1 , NULL , 'Featuring Ambisound, the HTS6600 delivers an embracing multi-channel surround experience from just two speakers, in any sort of room. Providing excellent sound and picture quality, the easy-setup wall-mountable system fits into any home.' , NULL , 'images/tiny/HTS6600.jpg' , NULL , 'images/small/HTS6600.jpg' , NULL , 'images/big/HTS6600.jpg' , NULL , 'images/super/HTS6600.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 549.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 1 , 449.99 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-11 17:11:19' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (31 , NULL , 'DVD Micro Theatre  MCD289/05' , NULL , 1 , 7 , -1 , NULL , NULL , 9 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , 'Enhance movie and music playback with the MCD289 and experience easy and clutter-free installation via a digital wireless subwoofer. It plays DVD, DivX and USB direct playback. Comes with table-top and wall-mount options.' , 'Enhance movie and music playback with the MCD289 and experience easy and clutter-free installation via a digital wireless subwoofer. It plays DVD, DivX and USB direct playback. Comes with table-top and wall-mount options.' , 0 , 'Play DVD, DivX, (S)VCD, MP3-CD, WMA-CD, CD(RW) & Picture CD\r\nDolby Digital for ultimate movie experience \t\r\n\r\nDolby Digital for ultimate movie experience\r\n\r\nBecause Dolby Digital standards, the world\'s leading digital multi-channel audio standards, make use of the way the human ear naturally processes sound, you experience superb quality surround sound audio with realistic spatial cues.\r\n\r\n12-bit/108 MHz video processing for sharp, natural images\r\n\r\n12-bit Video DAC is a superior Digital-to-Analogue Converter that preserves every detail of the authentic picture quality. It shows subtle shades and smoother graduation of colours, resulting in a more vibrant and natural picture. The limitations of the standard 10-bit DAC become particularly apparent when it is used with large screens and projectors.' , 0 , NULL , NULL , NULL , 'images/tiny/MCD289.jpg' , NULL , 'images/small/MCD289.jpg' , NULL , 'images/big/MCD289.jpg' , NULL , 'images/super/MCD289.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 199.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-11 12:43:31' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (32 , NULL , ' Flash audio player SA2645/02' , NULL , 1 , 8 , -1 , NULL , NULL , 9 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'Show off your Philips GoGear digital audio player SA2645, featuring a stylish interchangeable cover! Easily take your favorite tunes on the go with Direct USB and enjoy 20 hours of playtime with just one AAA battery.' , 0 , 'Show off your Philips GoGear digital audio player SA2645, featuring a stylish interchangeable cover! Easily take your favorite tunes on the go with Direct USB and enjoy 20 hours of playtime with just one AAA battery.\r\n\r\nYour music, your data - on the go, everyday \r\n\r\n Enjoy MP3 and WMA playback\r\n Make voice recordings\r\n Fast downloads from your PC via USB 2.0*\r\n Preset up to 20 FM radio stations for more music\r\n Plug and play data storage\r\n \r\nReady to wear, ready to go \r\n\r\n Up to 20-hour music playback with one AAA battery\r\n Small and wearable for ultimate portability\r\n' , 1 , NULL , 'Show off your Philips GoGear digital audio player SA2645, featuring a stylish interchangeable cover! Easily take your favorite tunes on the go with Direct USB and enjoy 20 hours of playtime with just one AAA battery.' , NULL , 'images/tiny/SA2645.jpg' , NULL , 'images/small/SA2645.jpg' , NULL , 'images/big/SA2645.jpg' , NULL , 'images/super/SA2645.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 49.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 1 , 39.99 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-11 17:37:23' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (33 , NULL , '8GB Flash audio video player SA6085/02 ' , NULL , 1 , 8 , -1 , NULL , NULL , 9 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'Show off your Philips GoGear digital audio player SA2645, featuring a stylish interchangeable cover! Easily take your favorite tunes on the go with Direct USB and enjoy 20 hours of playtime with just one AAA battery.' , 0 , 'Experience music and video - anytime, anyplace \r\n \r\n Enjoy your favourite videos on the move\r\n Enjoy MP3 and WMA (DRM) music plus FM radio\r\n Enjoy WMV videos on the move\r\n\r\nEnjoy straight away \r\n \r\n 2.2 colour LCD for a great video viewing experience\r\n SuperScroll finds what you want in a fast, simple way\r\n Small and wearable - goes where you go\r\n' , 0 , NULL , NULL , NULL , 'images/tiny/SA6085.jpg' , NULL , 'images/small/SA6085.jpg' , NULL , 'images/big/SA6085.jpg' , NULL , 'images/super/SA6085.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 149.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-11 13:07:00' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (34 , NULL , '8GB Walkman Video MP3 Player NWZ-S738FBNC' , NULL , 1 , 8 , -1 , NULL , NULL , 3 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , 'Black, integrated noise canceling technology, up to 40 hours of music and 10 hours of video, FM tuner, 2.0" LCD (measured diagonally) ' , 'Enjoy your multimedia entertainment from virtually anywhere with the NWZ-S738FBNC 8GB Walkman video MP3 player. Boasting an impressive battery life, you can listen up to 40 hours of your favorite music, podcasts and FM stations or watch up to 10 hours of video on a single charge.' , 0 , 'Enjoy your multimedia entertainment from virtually anywhere with the NWZ-S738FBNC 8GB Walkman video MP3 player. Boasting an impressive battery life, you can listen up to 40 hours of your favorite music, podcasts and FM stations or watch up to 10 hours of video on a single charge. Enhance your listening experience with integrated noise cancellation and five clear audio technologies. Ideal for the gym, travel or daily commute, this MP3 player comes with noise canceling EX headphones which reduce ambient sound and deliver superior audio. Additionally, this Walkman comes with an innovative feature called SensMe, which uses beats-per-minute and an algorithm to analyze portions of your music and automatically create music channels.\r\n\r\nExceptional sound quality\r\nWith five clear audio technologies: The NWZ-S738FBNC houses five integrated clear audio technologies, allowing you to enjoy all of your music and videos with exceptional crisp sound.\r\n\r\nIntegrated noise canceling system\r\nThe NWZ-S738FBNC features integrated noise canceling technology that cancels up to 75 percent of ambient noise2. An in-box adaptor can also turn your Walkman player and EX headphones combination into noise canceling headphones for other devices. For example, simply connect the included adapter to your Walkman player and the AV system in an airplane via an audio jack, and you\'re ready to enjoy the benefit of noise canceling technology while watching an in-flight movie.\r\n\r\n' , 0 , NULL , NULL , NULL , 'images/tiny/NWZS738FBNC.jpg' , NULL , 'images/small/NWZS738FBNC.jpg' , NULL , 'images/big/NWZS738FBNC.jpg' , NULL , 'images/super/NWZS738FBNC.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 179.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-11 13:06:52' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (35 , NULL , '8GB Walkman Video MP3 Player NWZ-E438FRED' , NULL , 1 , 8 , -1 , NULL , NULL , 3 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , 'Red, support for multiple audio and video formats, FM tuner, 2.0" LCD (measured diagonally)' , 'Make a statement with the slim and fashionable NWZ-E438FRED 8GB video Walkman MP3 player. View pictures or watch your favorite videos in vivid detail on the 2.0" QVGA high-brightness, color LCD (measured diagonally).' , 0 , 'Make a statement with the slim and fashionable NWZ-E438FRED 8GB video Walkman MP3 player. View pictures or watch your favorite videos in vivid detail on the 2.0" QVGA high-brightness, color LCD (measured diagonally). A long battery life give you up to 45 hours of continuous music and up to 8 hours of uninterrupted video while offering support of multiple audio codecs and video formats. You can even listen to your favorite FM programming thanks to a built-in FM tuner. Loading your music, pictures and video is a snap, simply drag and drop them into the video MP3 player on your computer. This MP3 player is available in red, pink and black.\r\n\r\nLong battery life\r\nEquipped with a long-lasting battery that enables you to listen to up to 45 hours of music and watch up to 8 hours of video without needing to recharge.\r\n\r\nCompact and slim design \r\nMakes carrying your video MP3 player wherever you go a breeze.\r\n\r\n2" QVGA high-brightness screen (measured diagonally)\r\nColor LCD provides you with a rich viewing experience on your video MP3 player.' , 0 , NULL , NULL , NULL , 'images/tiny/NWZE438FRED.jpg' , NULL , 'images/small/NWZE438FRED.jpg' , NULL , 'images/big/NWZE438FRED.jpg' , NULL , 'images/super/NWZE438FRED.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 139.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-11 13:09:18' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (36 , NULL , '16GB Walkman Video MP3 Player NWZ-A829BLK' , NULL , 1 , 8 , -1 , NULL , NULL , 3 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , 'Black, store up to 3,800 songs, up to 62 hours of video, 2.4" LCD (measured diagonally), Bluetooth connectivity ' , 'Now you have the option of listening to pristine audio without wires. Featuring Bluetooth connectivity, this 16GB Walkman Video MP3 player offers you wireless freedom when paired Bluetooth-enabled headphones. And if you want to watch video, a high-resolution 2.4" QVGA LCD beautifully presents them in portrait or landscape mode.' , 0 , 'Now you have the option of listening to pristine audio without wires. Featuring Bluetooth connectivity, this 16GB Walkman Video MP3 player offers you wireless freedom when paired Bluetooth-enabled headphones. And if you want to watch video, a high-resolution 2.4" QVGA LCD beautifully presents them in portrait or landscape mode. The sleek, slim-line player also supports multiple file types so you\'re not locked into one format. Small enough to slip into your pocket, the 16GB built-in memory can store up to 3,800 songs, up to 62 hours of video or hundreds of photographs, offering the widest variety of multimedia entertainment wherever you go.' , 0 , NULL , NULL , NULL , 'images/tiny/NWZA829BLK.jpg' , NULL , 'images/small/NWZA829BLK.jpg' , NULL , 'images/big/NWZA829BLK.jpg' , NULL , 'images/super/NWZA829BLK.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 319.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-11 13:11:01' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (37 , NULL , 'Stereo Bluetooth Headset DR-BT50' , NULL , 1 , 2 , -1 , NULL , NULL , 3 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'The freedom to roam. There is nothing like strapping on a pair of high-fidelity wireless headphones. The DR-BT50 stereo Bluetooth headset features 40mm driver units that deliver crisp treble and dramatic bass for superior audio accuracy.' , 0 , 'The freedom to roam. There is nothing like strapping on a pair of high-fidelity wireless headphones. The DR-BT50 stereo Bluetooth headset features 40mm driver units that deliver crisp treble and dramatic bass for superior audio accuracy. And an auranomic inner earcup design creates an excellent fit and allows for enhanced audio quality. This rechargeable set of headphones also provides you with approximately 17 hours of music playback time1 on a single charge, so you can enjoy your favorite music even longer. Remote control of play, stop, fast forward and rewind on the earpiece is included for your convenience.' , 1 , NULL , 'The freedom to roam. There is nothing like strapping on a pair of high-fidelity wireless headphones. The DR-BT50 stereo Bluetooth headset features 40mm driver units that deliver crisp treble and dramatic bass for superior audio accuracy.' , NULL , 'images/tiny/DRBT50.jpg' , NULL , 'images/small/DRBT50.jpg' , NULL , 'images/big/DRBT50.jpg' , NULL , 'images/super/DRBT50.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 299.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 1 , 199.99 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-23 12:19:38' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (38 , NULL , 'HA-SX500' , NULL , 1 , 2 , -1 , NULL , NULL , 7 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'The new JVC HA-SX500 achieves high quality sound with a large 16mm neodymium driver unit and new Bi-METAL structure. The lightweight headphones fit securely and comfortably with a cushioned metallic headband and 3 sizes of ergonomic earpieces.' , 0 , 'The new JVC HA-SX500 achieves high quality sound with a large 16mm neodymium driver unit and new Bi-METAL structure. The lightweight headphones fit securely and comfortably with a cushioned metallic headband and 3 sizes of ergonomic earpieces. The HA-SX500 delivers enhanced dynamic sound by employing a large 16mm neodymium driver unit, offering greater power handling and sensitivity. The Bi-METAL structure, meanwhile, designed to holds a stainless steel base unit with a ring made of brass, high specific heavy metal, and controls its movement for superior sound reproduction across the frequency range. The vast reduction in friction noise is another innovation to please audiophiles. The HA-SX500 features flexible rubber joints, reducing friction noise while increasing comfort, and uses OFC (oxygen-free copper) to minimize transmission loss in the 0.8 meter friction noise reduction cord. The HA-SX500 also offers up-to-date connection possibilities with an iPhone compatible plug. The lightweight HA-SX500 headphones also feature an ear-conscious design, making them comfortable and practical, fitting securely in the ear while at the same time delivering superior isolation from external sounds and minimizing sound leakage. The ergonomic earpieces come in three sizes to provide snug, customized sizing, supported by a cushioned metallic headband for a secure fit and hours of comfort. A number of convenient accessories complete the package, such as a 0.7m extension cord and a handy carrying case. ' , 0 , NULL , NULL , NULL , 'images/tiny/HASX500.jpg' , NULL , 'images/small/HASX500.jpg' , NULL , 'images/big/HASX500.jpg' , NULL , 'images/super/HASX500.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 79.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-23 12:19:45' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (39 , NULL , 'Noise Canceling Headphones MDR-NC6' , NULL , 1 , 2 , -1 , NULL , NULL , 3 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'Settle down with MDR-NC6 noise canceling headphones that can reduce ambient sound by over 10dB at 300Hz. ' , 0 , 'Shut off the outside world with MDR-NC6 noise canceling headphones. Ambient sound can be reduced over 10dB at 300Hz, which means you\'ll hear more music, and less plane, train, or bus noise. Noise canceling headphones are great for work, traveling, or even when you\'re trying to sleep in a loud place. The MDR-NC6 headphones have a slim swivel folding design, making them more compact for storage. Includes plug adaptor to connect directly to a stereo or dual jack of in-flight music services.' , 0 , NULL , NULL , NULL , 'images/tiny/MDRNC6.jpg' , NULL , 'images/small/MDRNC6.jpg' , NULL , 'images/big/MDRNC6.jpg' , NULL , 'images/super/MDRNC6.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 49.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-23 12:19:51' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (40 , NULL , 'Active Style Headphones MDR-AS50G' , NULL , 1 , 2 , -1 , NULL , NULL , 3 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'Neck-band headphones like these make it easy to add music to your workout without the fuss. The lightweight construction and unobtrusive form make them perfect for almost any activity.' , 0 , 'The MDR-AS50G earbud-style headphones don\'t get in the way during your favorite activities. Whether you\'re biking, climbing, or just jogging down the street, you\'ll appreciate the lightweight construction and freedom of the behind-the neck design. The short 2\' cord (with 2\' extension) is perfect for arm band use and the three-size ear buds allow for a snug fit. Includes a convenient hard carrying case for traveling.' , 0 , NULL , NULL , NULL , 'images/tiny/MDRAS50G.jpg' , NULL , 'images/small/MDRAS50G.jpg' , NULL , 'images/big/MDRAS50G.jpg' , NULL , 'images/super/MDRAS50G.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 49.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-23 12:20:02' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (41 , NULL , 'S2 Sports Street Style Headphones MDR-G57G' , NULL , 1 , 2 , -1 , NULL , NULL , 3 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'Tune into the latest style on the street with MDR-G57G Street Style stereo headphones. Featuring an exclusive behind-the-neck headband that won\'t interfere with your style. ' , 0 , 'Ultra light and comfortable, Street Style headphones set the standard for portable audio. Featuring an exclusive behind-the-neck headband that won\'t interfere with your hairstyle or hat, a reflective ear piece that is perfect for jogging at night and a non-slip design that stays on your ears during active sports like jogging, dancing, roller-blading and other activities that are enhanced by music.\r\n\r\nExclusive behind-the-neck headband\r\nwon\'t interfere with your hairstyle; let\'s you wear a hat; establishes a stylish look on the street\r\n\r\nStable\r\neven while playing sports, dancing, and walking; matches new Sports Walkman  design\r\n\r\nReflective ear piece\r\nis perfect when jogging at night' , 0 , NULL , NULL , NULL , 'images/tiny/MDRG57G.jpg' , NULL , 'images/small/MDRG57G.jpg' , NULL , 'images/big/MDRG57G.jpg' , NULL , 'images/super/MDRG57G.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 29.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-23 12:20:09' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (42 , NULL , 'Nikon 200-400mm f/4G ED-IF AF-S VR Zoom Nikkor Lens for Nikon Digital SLR Cameras' , NULL , 1 , 2 , -1 , NULL , NULL , 8 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , 'Vibration Reduction (VR) operation offers the equivalent of using a shutter speed 3 stops faster. Two VR Modes - normal or active for taking pictures with extreme movement - Automatically detects when photographer is panning and compensates.\r\n' , 'This is the world\'s first AF-S VR super-telephoto 200-400mm zoom lens. The Nikon 200-400mm f4G ED-IF AF-S VR Zoom Nikkor telephoto lens features Nikon\'s remarkable Vibration Reduction (VR) technology and is designed to work seamlessly with Nikon\'s digital SLR cameras.' , 1 , '<b>From the Manufacturer</b><br />\r\n<p>This is the world\'s first AF-S VR super-telephoto 200-400mm zoom lens. \r\nThe Nikon 200-400mm f4G ED-IF AF-S VR Zoom Nikkor telephoto lens features \r\nNikon\'s remarkable Vibration Reduction (VR) technology and is designed to work \r\nseamlessly with Nikon\'s digital SLR cameras. The lens offers sports, action, \r\nbird, and wildlife photographers a high level of quality, precision, and \r\nperformance. With VR (Vibration Reduction), photographers to take high-quality \r\nhandheld shots indoors and in dark locations at shutter speeds three stops \r\nslower without compromising the sharpness of the picture.</p>\r\n<b>Features: </b>\r\n<ul>\r\n\t<li>Combines four fixed focal length super-telephoto lenses into a single \r\n\tcompact super-telephoto AF Zoom Nikkor lens. </li>\r\n\t<li>VR operation offers the equivalent of using a shutter speed 3 stops \r\n\tfaster. </li>\r\n\t<li>Two VR Modes: Normal mode or Active Mode for when taking pictures with \r\n\textreme movement. </li>\r\n\t<li>Automatically detects when photographer is panning and compensates. \r\n\t</li>\r\n\t<li>Compatible with Nikon TC-14E II, TC-17E II, and TC-20E II \r\n\tTeleconverters, AF operation only with TC14E II. </li>\r\n\t<li>Exclusive Silent Wave Motor enables ultra-high-speed autofocusing with \r\n\texceptional accuracy and powerful super-quiet operation. </li>\r\n\t<li>4 Extra-Low Dispersion (ED) glass elements minimizes chromatic \r\n\taberration and provides higher resolution and superior contrast. </li>\r\n\t<li>Minimum focus distance of 6.2 feet at all focal lengths. </li>\r\n\t<li>Internal Focusing (IF) design for smoother focusing and great body \r\n\tbalance. </li>\r\n\t<li>M/A mode enables instant switching from autofocus to manual with \r\n\tvirtually no time lag even during AF servo operation. </li>\r\n\t<li>No power consumption during manual focus operation. </li>\r\n\t<li>Rounded diaphragm (9 blades) for natural blur. </li>\r\n\t<li>Nikon D-type design provides precise distance information for flash and \r\n\tambient light exposure processes. </li>\r\n\t<li>Detachable tripod mounting collar provided. </li>\r\n\t<li>G design enables apertures to be selected from select Nikon AF SLR \r\n\tcamera bodies.</li>\r\n</ul>\r\n<br />\r\n<b>Product Description</b><br />\r\nVibration Reduction / Combines four fixed focal length super-Telephoto lenses \r\ninto a single compact super-Telephoto / For Nikon SLR Internal Focusing (IF) \r\ndesign for smoother focusing and great body balance. M/A mode enables instant \r\nswitching from auto focus to manual with virtually no time lag even during AF \r\nservo operation. No power consumption during manual focus operation. 9 blade \r\nrounded diaphragm for natural blur. Nikon D-type design provides precise \r\ndistance information for flash and ambient light exposure processes. Detachable \r\ntripod mounting collar provided. G-design enables apertures to be selected from \r\nselect Nikon AF SLR camera bodies.\r\n' , 0 , NULL , NULL , NULL , 'images/tiny/Nikon200-400mm.jpg' , NULL , 'images/small/Nikon200-400mm.jpg' , NULL , 'images/big/Nikon200-400mm.jpg' , NULL , 'images/super/Nikon200-400mm.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 6890 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 1 , 5469 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 2 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , 1 , 1 , '2008-09-12 08:44:13' , '2008-09-15 19:26:32' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (43 , NULL , 'Sanus Systems Low Profile Universal Flat Panel Mount 32" to 60" Displays' , NULL , 1 , 2 , -1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , '\r\n' , 'The Sanus VisionMount is the ideal solution for mounting a flat panel TV or computer monitor to a wall, ceiling or even under a cabinet. Extruded aluminum construction perfectly complements the high-tech look and feel of today\'s flat panels.' , 0 , 'The Sanus VisionMount is the ideal solution for mounting a flat panel TV or computer monitor to a wall, ceiling or even under a cabinet. Extruded aluminum construction perfectly complements the high-tech look and feel of today\'s flat panels. The VisionMount is designed to fit every mounting and adjustment need. Every model uses VESA mounting standards. Plus, the patent pending SmoothLock adjustment mechanism provides effortless tilt and swivel adjustment with no tools required.' , 0 , NULL , NULL , NULL , 'images/tiny/SanusSystems.jpg' , NULL , 'images/small/SanusSystems.jpg' , NULL , 'images/big/SanusSystems.jpg' , NULL , 'images/super/SanusSystems.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 169.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 1 , 93 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-12 08:47:36' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (44 , NULL , 'diNovo Media Desktop Laser' , NULL , 1 , 2 , -1 , NULL , NULL , 6 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'Add some sophistication to your surroundings with this award-winning design, and experience a performance revolution with the rechargeable MX Laser Mouse. Enjoy Bluetooth Wireless Technology compatibility to bring all your favorite components together.' , 0 , 'Add some sophistication to your surroundings with this award-winning design, and experience a performance revolution with the rechargeable MX Laser Mouse. Enjoy Bluetooth Wireless Technology compatibility to bring all your favorite components together.' , 0 , NULL , NULL , NULL , 'images/tiny/diNovo.jpg' , NULL , 'images/small/diNovo.jpg' , NULL , 'images/big/diNovo.jpg' , NULL , 'images/super/diNovo.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 319.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-12 08:50:41' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (45 , NULL , 'QuickCam Pro for Notebooks' , NULL , 1 , 2 , -1 , NULL , NULL , 6 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'Enjoy image-perfect detail and clarity on your notebook. Carl Zeiss optics and autofocus keep images razor-sharp, even in extreme close-ups.' , 0 , 'Enjoy image-perfect detail and clarity on your notebook. Carl Zeiss optics and autofocus keep images razor-sharp, even in extreme close-ups.' , 0 , NULL , NULL , NULL , 'images/tiny/QuickCam.jpg' , NULL , 'images/small/QuickCam.jpg' , NULL , 'images/big/QuickCam.jpg' , NULL , 'images/super/QuickCam.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 139.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-12 09:05:05' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (46 , NULL , 'Digital Precision PC Gaming Headset' , NULL , 1 , 2 , -1 , NULL , NULL , 6 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'Stay cool with Logitech\'s Digital Precision PC Gaming Headset. The cushioned headset fits behind the head and features a unique open-air design that keeps your ears cool and comfortable over extended gaming sessions.' , 0 , 'Stay cool with Logitech\'s Digital Precision PC Gaming Headset. The cushioned headset fits behind the head and features a unique open-air design that keeps your ears cool and comfortable over extended gaming sessions.\r\n\r\nTake advantage of pure digital audio with the included plug-and-play USB adapter or connect the dual analogue 3.5 mm jacks to your favourite soundcard. A noise-cancelling microphone puts the focus on your voice by reducing background chatter and the in-line volume and mute controls give you instant access to audio adjustments in the heat of the action.' , 0 , NULL , NULL , NULL , 'images/tiny/PCGamingHeadset.jpg' , NULL , 'images/small/PCGamingHeadset.jpg' , NULL , 'images/big/PCGamingHeadset.jpg' , NULL , 'images/super/PCGamingHeadset.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 69.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-12 09:06:28' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (47 , NULL , 'AudioHub' , NULL , 1 , 2 , -1 , NULL , NULL , 6 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'Premium audio in a compact, one-piece speaker system.' , 0 , 'Premium audio in a compact, one-piece speaker system.' , 0 , NULL , NULL , NULL , 'images/tiny/AudioHub.jpg' , NULL , 'images/small/AudioHub.jpg' , NULL , 'images/big/AudioHub.jpg' , NULL , 'images/super/AudioHub.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 139.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-12 09:07:57' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (48 , NULL , 'Harmony 1000 Advanced Universal Remote  ' , NULL , 1 , 2 , -1 , NULL , NULL , 6 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'Step up to the ultimate in universal remotes. Experience a new standard in simplicity with even the most complex components. With just one touch you can start watching, listening or playing. And youll be proud to display the Harmony 1000s sleek brushed aluminum and iconic colour LCD.' , 0 , 'Step up to the ultimate in universal remotes. Experience a new standard in simplicity with even the most complex components. With just one touch you can start watching, listening or playing. And youll be proud to display the Harmony 1000s sleek brushed aluminum and iconic colour LCD.' , 0 , NULL , NULL , NULL , 'images/tiny/Harmony1000.jpg' , NULL , 'images/small/Harmony1000.jpg' , NULL , 'images/big/Harmony1000.jpg' , NULL , 'images/super/Harmony1000.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 559.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-12 09:09:24' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (49 , NULL , 'Harmony One Advanced Universal Remote' , NULL , 1 , 2 , -1 , NULL , NULL , 6 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'We know who you are. You have a half-dozen remotes on the coffee table. And you\'re the only one in the family who knows how to use them.' , 0 , 'We know who you are. You have a half-dozen remotes on the coffee table. And you\'re the only one in the family who knows how to use them.\r\n\r\nWe can help. Enjoy a full-colour touch screen. Intuitive button layout. And an ergonomic design. So easy to use, the whole family will love it.' , 0 , NULL , NULL , NULL , 'images/tiny/HarmonyOne.jpg' , NULL , 'images/small/HarmonyOne.jpg' , NULL , 'images/big/HarmonyOne.jpg' , NULL , 'images/super/HarmonyOne.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 259.99 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-12 09:10:58' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (50 , NULL , 'Canon 430EX Speedlite Flash for Canon Pro1, Pro 90, G Series and all EOS SLR Cameras' , NULL , 1 , 2 , -1 , NULL , NULL , 5 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , 'This exciting new addition to the EOS System combines compact size and affordability with an added dose of features and sophistication. Improving on the previous model, Speedlite 420EX, the new 430EX can now be set into manual flash mode (full power~1/64).' , 0 , 'This exciting new addition to the EOS System combines compact size and affordability with an added dose of features and sophistication. Improving on the previous model, Speedlite 420EX, the new 430EX can now be set into manual flash mode (full power~1/64). Flash compensation can be set on the flash unit, and manual control of the flash zoom head is now possible. Improvements in its circuitry make recycle time 40% faster, and since it is compatible with Canon\'s wireless E-TTL, it can be used as an affordable "slave unit."' , 0 , NULL , NULL , NULL , 'images/tiny/Canon430EX.jpg' , NULL , 'images/small/Canon430EX.jpg' , NULL , 'images/big/Canon430EX.jpg' , NULL , 'images/super/Canon430EX.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 400 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 1 , 239 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-12 09:12:58' ) ;
INSERT INTO va_items (item_id,item_code,item_name,friendly_url,item_order,item_type_id,google_base_type_id,issue_date,language_code,manufacturer_id,manufacturer_code,supplier_id,author_id,template_name,hide_add_list,hide_add_table,hide_add_grid,hide_add_details,features,highlights,short_description,full_desc_type,full_description,is_special_offer,special_order,special_offer,notes,tiny_image,tiny_image_alt,small_image,small_image_alt,big_image,big_image_alt,super_image,super_image_alt,preview_url,preview_width,preview_height,specification,buying_price,buying_price_id,price,price_id,trade_price,trade_price_id,is_price_edit,properties_price,properties_price_id,trade_properties_price,trade_properties_price_id,tax_id,tax_free,width,height,length,weight,packages_number,min_quantity,max_quantity,quantity_increment,stock_level,use_stock_level,hide_out_of_stock,disable_out_of_stock,is_shipping_free,is_separate_shipping,is_shipping_required,shipping_in_stock,shipping_out_stock,shipping_rule_id,shipping_cost,shipping_trade_cost,shipping_modules_default,shipping_modules_ids,is_sales,sales_price,sales_price_id,trade_sales,trade_sales_id,discount_percent,is_points_price,points_price,reward_type,reward_amount,credit_reward_type,credit_reward_amount,is_recurring,recurring_price,recurring_period,recurring_interval,recurring_payments_total,recurring_start_date,recurring_end_date,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,downloadable,download_period,download_path,download_show_terms,download_terms_text,generate_serial,serial_period,activations_number,is_showing,is_approved,is_new,is_keywords,sites_all,access_level,guest_access_level,admin_access_level,is_compared,buy_link,total_views,votes,points,rating,a_title,meta_title,meta_keywords,meta_description,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,user_id,is_draft,draft_parent_id,admin_id_added_by,admin_id_modified_by,date_added,date_modified) VALUES (52 , NULL , 'Nikon 85mm f/1.8D AF Nikkor Lens for Nikon Digital SLR Cameras' , NULL , 1 , 2 , -1 , NULL , NULL , 8 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 0 , NULL , '\r\n' , '85mm lens for Nikon SLR cameras ' , 0 , 'Portable medium telephoto lens that is ideal for indoor and outdoor portraits\r\nRear Focusing system for fast and smooth optical performance\r\nSubtle blurring of background for beautifully natural and evocative portraits\r\nHigh-speed f1.8 aperture\r\nCompact and lightweight at 13.2 ounces\r\n' , 0 , NULL , NULL , NULL , 'images/tiny/Nikon85mm.jpg' , NULL , 'images/small/Nikon85mm.jpg' , NULL , 'images/big/Nikon85mm.jpg' , NULL , 'images/super/Nikon85mm.jpg' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 565 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 1 , 385 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 1 , 1 , NULL , NULL , 1 , 7 , 7 , 7 , 0 , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , 0 , NULL , NULL , 1 , NULL , '2008-09-12 09:15:56' ) ;


INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (1 , 37 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (1 , 47 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (3 , 44 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (3 , 45 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (4 , 44 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (4 , 46 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (5 , 37 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (5 , 46 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (6 , 37 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (6 , 46 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (7 , 37 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (7 , 46 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (8 , 37 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (8 , 46 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (9 , 37 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (9 , 46 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (10 , 45 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (10 , 46 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (11 , 44 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (11 , 47 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (12 , 50 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (18 , 42 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (18 , 52 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (22 , 43 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (22 , 49 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (23 , 43 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (23 , 48 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (24 , 43 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (24 , 49 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (25 , 48 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (25 , 49 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (26 , 43 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (26 , 48 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (27 , 48 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (27 , 49 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (28 , 48 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (28 , 49 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (29 , 48 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (29 , 49 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (30 , 48 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (30 , 49 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (31 , 48 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (31 , 49 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (32 , 39 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (32 , 41 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (33 , 38 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (33 , 40 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (34 , 38 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (34 , 39 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (35 , 40 , 1 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (35 , 41 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (36 , 37 , 2 ) ;
INSERT INTO va_items_accessories (item_id,accessory_id,accessory_order) VALUES (36 , 41 , 1 ) ;


INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (1 , 0 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (1 , 53 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (3 , 53 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (4 , 53 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (5 , 54 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (6 , 54 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (7 , 0 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (7 , 54 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (8 , 54 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (9 , 0 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (9 , 54 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (10 , 0 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (10 , 53 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (11 , 53 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (12 , 0 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (12 , 55 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (13 , 55 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (14 , 0 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (14 , 55 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (15 , 56 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (16 , 56 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (17 , 0 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (17 , 56 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (18 , 0 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (18 , 55 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (19 , 55 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (20 , 0 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (20 , 56 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (21 , 56 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (22 , 57 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (23 , 0 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (23 , 57 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (24 , 57 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (25 , 0 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (25 , 57 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (26 , 57 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (27 , 58 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (28 , 58 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (29 , 0 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (29 , 58 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (30 , 0 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (30 , 58 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (31 , 0 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (31 , 58 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (32 , 0 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (32 , 59 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (33 , 59 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (34 , 59 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (35 , 0 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (35 , 59 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (36 , 59 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (37 , 60 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (38 , 0 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (38 , 60 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (39 , 60 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (40 , 0 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (40 , 60 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (41 , 60 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (42 , 52 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (43 , 0 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (43 , 52 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (44 , 0 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (44 , 52 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (45 , 0 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (45 , 52 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (46 , 0 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (46 , 52 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (47 , 52 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (48 , 52 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (49 , 0 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (49 , 52 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (50 , 52 , 1 ) ;
INSERT INTO va_items_categories (item_id,category_id,item_order) VALUES (52 , 52 , 1 ) ;










INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (1 , 1 , NULL , NULL , 2 , 'VGNTZ398UX' , NULL , NULL , 'images/small/VGNTZ398UX_0.jpg' , NULL , NULL , NULL , 'images/big/VGNTZ398UX.jpg' , NULL , 'images/super/VGNTZ398UX.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (2 , 1 , NULL , NULL , 2 , 'VGNTZ398UX_1' , NULL , NULL , 'images/small/VGNTZ398UX_1.jpg' , NULL , NULL , NULL , 'images/big/VGNTZ398UX_1.jpg' , NULL , 'images/super/VGNTZ398UX_1.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (3 , 1 , NULL , NULL , 2 , 'VGNTZ398UX_2' , NULL , NULL , 'images/small/VGNTZ398UX_2.jpg' , NULL , NULL , NULL , 'images/big/VGNTZ398UX_2.jpg' , NULL , 'images/super/VGNTZ398UX_2.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (4 , 1 , NULL , NULL , 2 , 'VGNTZ398UX_3' , NULL , NULL , 'images/small/VGNTZ398UX_3.jpg' , NULL , NULL , NULL , 'images/big/VGNTZ398UX_3.jpg' , NULL , 'images/super/VGNTZ398UX_3.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (5 , 3 , NULL , NULL , 2 , 'VGNFW145EW' , NULL , NULL , 'images/small/VGNFW145EW_0.jpg' , NULL , NULL , NULL , 'images/big/VGNFW145EW.jpg' , NULL , 'images/super/VGNFW145EW.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (6 , 3 , NULL , NULL , 2 , 'VGNFW145EW_1' , NULL , NULL , 'images/small/VGNFW145EW_1.jpg' , NULL , NULL , NULL , 'images/big/VGNFW145EW_1.jpg' , NULL , 'images/super/VGNFW145EW_1.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (7 , 3 , NULL , NULL , 2 , 'VGNFW145EW_2' , NULL , NULL , 'images/small/VGNFW145EW_2.jpg' , NULL , NULL , NULL , 'images/big/VGNFW145EW_2.jpg' , NULL , 'images/super/VGNFW145EW_2.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (8 , 3 , NULL , NULL , 2 , 'VGNFW145EW_3' , NULL , NULL , 'images/small/VGNFW145EW_3.jpg' , NULL , NULL , NULL , 'images/big/VGNFW145EW_3.jpg' , NULL , 'images/super/VGNFW145EW_3.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (9 , 4 , NULL , NULL , 2 , 'VGNZ590UAB' , NULL , NULL , 'images/small/VGNZ590UAB_0.jpg' , NULL , NULL , NULL , 'images/big/VGNZ590UAB.jpg' , NULL , 'images/super/VGNZ590UAB.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (10 , 4 , NULL , NULL , 2 , 'VGNZ590UAB_1' , NULL , NULL , 'images/small/VGNZ590UAB_1.jpg' , NULL , NULL , NULL , 'images/big/VGNZ590UAB_1.jpg' , NULL , 'images/super/VGNZ590UAB_1.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (11 , 4 , NULL , NULL , 2 , 'VGNZ590UAB_2' , NULL , NULL , 'images/small/VGNZ590UAB_2.jpg' , NULL , NULL , NULL , 'images/big/VGNZ590UAB_2.jpg' , NULL , 'images/super/VGNZ590UAB_2.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (12 , 4 , NULL , NULL , 2 , 'VGNZ590UAB_3' , NULL , NULL , 'images/small/VGNZ590UAB_3.jpg' , NULL , NULL , NULL , 'images/big/VGNZ590UAB_3.jpg' , NULL , 'images/super/VGNZ590UAB_3.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (13 , 5 , NULL , NULL , 2 , 'VGCJS130JS' , NULL , NULL , 'images/small/VGCJS130JS_0.jpg' , NULL , NULL , NULL , 'images/big/VGCJS130JS.jpg' , NULL , 'images/super/VGCJS130JS.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (14 , 5 , NULL , NULL , 2 , 'VGCJS130JS_1' , NULL , NULL , 'images/small/VGCJS130JS_1.jpg' , NULL , NULL , NULL , 'images/big/VGCJS130JS_1.jpg' , NULL , 'images/super/VGCJS130JS_1.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (15 , 5 , NULL , NULL , 2 , 'VGCJS130JS_2' , NULL , NULL , 'images/small/VGCJS130JS_2.jpg' , NULL , NULL , NULL , 'images/big/VGCJS130JS_2.jpg' , NULL , 'images/big/VGCJS130JS_2.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (16 , 5 , NULL , NULL , 2 , 'VGCJS130JS_3' , NULL , NULL , 'images/small/VGCJS130JS_3.jpg' , NULL , NULL , NULL , 'images/big/VGCJS130JS_3.jpg' , NULL , 'images/super/VGCJS130JS_3.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (17 , 6 , NULL , NULL , 2 , 'VGCLV150J' , NULL , NULL , 'images/small/VGCLV150J_0.jpg' , NULL , NULL , NULL , 'images/big/VGCLV150J.jpg' , NULL , 'images/super/VGCLV150J.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (18 , 6 , NULL , NULL , 2 , 'VGCLV150J_1' , NULL , NULL , 'images/small/VGCLV150J_1.jpg' , NULL , NULL , NULL , 'images/big/VGCLV150J_1.jpg' , NULL , 'images/big/VGCLV150J_1.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (19 , 6 , NULL , NULL , 2 , 'VGCLV150J_2' , NULL , NULL , 'images/small/VGCLV150J_2.jpg' , NULL , NULL , NULL , 'images/big/VGCLV150J_2.jpg' , NULL , 'images/super/VGCLV150J_2.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (20 , 6 , NULL , NULL , 2 , 'VGCLV150J_3' , NULL , NULL , 'images/small/VGCLV150J_3.jpg' , NULL , NULL , NULL , 'images/big/VGCLV150J_3.jpg' , NULL , 'images/super/VGCLV150J_3.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (21 , 7 , NULL , NULL , 2 , 'VGCLT38E' , NULL , NULL , 'images/small/VGCLT38E_0.jpg' , NULL , NULL , NULL , 'images/big/VGCLT38E.jpg' , NULL , 'images/super/VGCLT38E.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (22 , 7 , NULL , NULL , 2 , 'VGCLT38E_1' , NULL , NULL , 'images/small/VGCLT38E_1.jpg' , NULL , NULL , NULL , 'images/big/VGCLT38E_1.jpg' , NULL , 'images/super/VGCLT38E_1.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (23 , 7 , NULL , NULL , 2 , 'VGCLT38E_2' , NULL , NULL , 'images/small/VGCLT38E_2.jpg' , NULL , NULL , NULL , 'images/big/VGCLT38E_2.jpg' , NULL , 'images/super/VGCLT38E_2.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (25 , 8 , NULL , NULL , 2 , 'XPSOne' , NULL , NULL , 'images/small/XPSOne_0.jpg' , NULL , NULL , NULL , 'images/big/XPSOne.jpg' , NULL , 'images/super/XPSOne.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (26 , 8 , NULL , NULL , 2 , 'XPSOne_1' , NULL , NULL , 'images/small/XPSOne_1.jpg' , NULL , NULL , NULL , 'images/big/XPSOne_1.jpg' , NULL , 'images/super/XPSOne_1.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (27 , 8 , NULL , NULL , 2 , 'XPSOne_2' , NULL , NULL , 'images/small/XPSOne_2.jpg' , NULL , NULL , NULL , 'images/big/XPSOne_2.jpg' , NULL , 'images/super/XPSOne_2.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (28 , 8 , NULL , NULL , 2 , 'XPSOne_3' , NULL , NULL , 'images/small/XPSOne_3.jpg' , NULL , NULL , NULL , 'images/big/XPSOne_3.jpg' , NULL , 'images/super/XPSOne_3.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (29 , 9 , NULL , NULL , 2 , 'StudioHybrid' , NULL , NULL , 'images/small/StudioHybrid_0.jpg' , NULL , NULL , NULL , 'images/big/StudioHybrid.jpg' , NULL , 'images/super/StudioHybrid.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (30 , 9 , NULL , NULL , 2 , 'StudioHybrid_1' , NULL , NULL , 'images/small/StudioHybrid_1.jpg' , NULL , NULL , NULL , 'images/big/StudioHybrid_1.jpg' , NULL , 'images/big/StudioHybrid_1.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (31 , 9 , NULL , NULL , 2 , 'StudioHybrid_2' , NULL , NULL , 'images/small/StudioHybrid_2.jpg' , NULL , NULL , NULL , 'images/big/StudioHybrid_2.jpg' , NULL , 'images/super/StudioHybrid_2.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (32 , 9 , NULL , NULL , 2 , 'StudioHybrid_3' , NULL , NULL , 'images/small/StudioHybrid_3.jpg' , NULL , NULL , NULL , 'images/big/StudioHybrid_3.jpg' , NULL , 'images/super/StudioHybrid_3.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (33 , 10 , NULL , NULL , 2 , 'LatitudeXT' , NULL , NULL , 'images/small/LatitudeXT_0.jpg' , NULL , NULL , NULL , 'images/big/LatitudeXT.jpg' , NULL , 'images/super/LatitudeXT.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (34 , 10 , NULL , NULL , 2 , 'LatitudeXT_1' , NULL , NULL , 'images/small/LatitudeXT_1.jpg' , NULL , NULL , NULL , 'images/big/LatitudeXT_1.jpg' , NULL , 'images/super/LatitudeXT_1.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (37 , 11 , NULL , NULL , 2 , 'XPSM1730' , NULL , NULL , 'images/small/XPSM1730_0.jpg' , NULL , NULL , NULL , 'images/big/XPSM1730.jpg' , NULL , 'images/super/XPSM1730.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (38 , 11 , NULL , NULL , 2 , 'XPSM1730_1' , NULL , NULL , 'images/small/XPSM1730_1.jpg' , NULL , NULL , NULL , 'images/big/XPSM1730_1.jpg' , NULL , 'images/super/XPSM1730_1.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (39 , 1 , 1 , 1 , 2 , 'VGNTZ398UX.jpg' , 'images/tiny/VGNTZ398UX.jpg' , NULL , 'images/small/VGNTZ398UX.jpg' , NULL , NULL , NULL , 'images/big/VGNTZ398UX.jpg' , NULL , 'images/super/VGNTZ398UX.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (40 , 27 , 1 , 1 , 2 , 'DAVIS10.jpg' , 'images/tiny/DAVIS10.jpg' , NULL , 'images/small/DAVIS10.jpg' , NULL , NULL , NULL , 'images/big/DAVIS10.jpg' , NULL , 'images/super/DAVIS10.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (41 , 26 , 1 , 1 , 2 , 'KDL40XBR6.jpg' , 'images/tiny/KDL40XBR6.jpg' , NULL , 'images/small/KDL40XBR6.jpg' , NULL , NULL , NULL , 'images/big/KDL40XBR6.jpg' , NULL , 'images/super/KDL40XBR6.jpg' , NULL , NULL ) ;
INSERT INTO va_items_images (image_id,item_id,is_default,image_order,image_position,image_title,image_tiny,image_tiny_alt,image_small,image_small_alt,small_width,small_height,image_large,image_large_alt,image_super,image_super_alt,image_description) VALUES (42 , 25 , 1 , 1 , 2 , 'VPLVW200.jpg' , 'images/tiny/VPLVW200.jpg' , NULL , 'images/small/VPLVW200.jpg' , NULL , NULL , NULL , 'images/big/VPLVW200.jpg' , NULL , 'images/super/VPLVW200.jpg' , NULL , NULL ) ;


INSERT INTO va_items_prices (price_id,item_id,is_active,min_quantity,max_quantity,price,user_type_id,site_id,discount_action,properties_discount) VALUES (1 , 1 , 1 , 1 , 10 , 3999.99 , 0 , 0 , 2 , NULL ) ;
INSERT INTO va_items_prices (price_id,item_id,is_active,min_quantity,max_quantity,price,user_type_id,site_id,discount_action,properties_discount) VALUES (2 , 1 , 1 , 11 , 100 , 3799.99 , 0 , 0 , 2 , NULL ) ;
INSERT INTO va_items_prices (price_id,item_id,is_active,min_quantity,max_quantity,price,user_type_id,site_id,discount_action,properties_discount) VALUES (3 , 3 , 1 , 1 , 10 , 1199.99 , 0 , 0 , 2 , NULL ) ;
INSERT INTO va_items_prices (price_id,item_id,is_active,min_quantity,max_quantity,price,user_type_id,site_id,discount_action,properties_discount) VALUES (4 , 3 , 1 , 11 , 100 , 999.99 , 0 , 0 , 2 , NULL ) ;
INSERT INTO va_items_prices (price_id,item_id,is_active,min_quantity,max_quantity,price,user_type_id,site_id,discount_action,properties_discount) VALUES (5 , 4 , 1 , 1 , 10 , 3999.99 , 0 , 0 , 2 , NULL ) ;
INSERT INTO va_items_prices (price_id,item_id,is_active,min_quantity,max_quantity,price,user_type_id,site_id,discount_action,properties_discount) VALUES (6 , 4 , 1 , 11 , 100 , 3799.99 , 0 , 0 , 2 , NULL ) ;
INSERT INTO va_items_prices (price_id,item_id,is_active,min_quantity,max_quantity,price,user_type_id,site_id,discount_action,properties_discount) VALUES (7 , 10 , 1 , 1 , 10 , 1939.99 , 0 , 0 , 2 , NULL ) ;
INSERT INTO va_items_prices (price_id,item_id,is_active,min_quantity,max_quantity,price,user_type_id,site_id,discount_action,properties_discount) VALUES (8 , 10 , 1 , 11 , 100 , 1739.99 , 0 , 0 , 2 , NULL ) ;
INSERT INTO va_items_prices (price_id,item_id,is_active,min_quantity,max_quantity,price,user_type_id,site_id,discount_action,properties_discount) VALUES (9 , 11 , 1 , 1 , 10 , 2499.99 , 0 , 0 , 2 , NULL ) ;
INSERT INTO va_items_prices (price_id,item_id,is_active,min_quantity,max_quantity,price,user_type_id,site_id,discount_action,properties_discount) VALUES (10 , 11 , 1 , 11 , 100 , 2299.99 , 0 , 0 , 2 , NULL ) ;


INSERT INTO va_items_properties (property_id,property_type_id,parent_property_id,parent_value_id,show_for_user,usage_type,item_type_id,item_id,sub_item_id,additional_price,trade_additional_price,percentage_price_type,percentage_property_id,quantity,quantity_action,property_order,property_step,property_name,property_hint,hide_name,property_code,property_description,property_class,property_style,property_price_type,free_price_type,free_price_amount,max_limit_type,max_limit_length,length_units,use_on_list,use_on_table,use_on_grid,use_on_details,use_on_checkout,use_on_second,control_type,control_style,control_code,onchange_code,onclick_code,required,start_html,middle_html,before_control_html,after_control_html,end_html,sites_all) VALUES (1 , 3 , NULL , NULL , 1 , 1 , 0 , 22 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 'Profile Universal' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 1 , 0 , 0 , 'CHECKBOXLIST' , NULL , NULL , NULL , NULL , 0 , '<div style="margin: 0px 0px 10px 0px;"><div style="display: none;">' , '</div>' , NULL , NULL , '</div>' , NULL ) ;
INSERT INTO va_items_properties (property_id,property_type_id,parent_property_id,parent_value_id,show_for_user,usage_type,item_type_id,item_id,sub_item_id,additional_price,trade_additional_price,percentage_price_type,percentage_property_id,quantity,quantity_action,property_order,property_step,property_name,property_hint,hide_name,property_code,property_description,property_class,property_style,property_price_type,free_price_type,free_price_amount,max_limit_type,max_limit_length,length_units,use_on_list,use_on_table,use_on_grid,use_on_details,use_on_checkout,use_on_second,control_type,control_style,control_code,onchange_code,onclick_code,required,start_html,middle_html,before_control_html,after_control_html,end_html,sites_all) VALUES (3 , 3 , NULL , NULL , 1 , 1 , 0 , 23 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 'Profile Universal' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 1 , 0 , 0 , 'CHECKBOXLIST' , NULL , NULL , NULL , NULL , 0 , '<div style="margin: 0px 0px 10px 0px;"><div style="display: none;">' , '</div>' , NULL , NULL , '</div>' , NULL ) ;
INSERT INTO va_items_properties (property_id,property_type_id,parent_property_id,parent_value_id,show_for_user,usage_type,item_type_id,item_id,sub_item_id,additional_price,trade_additional_price,percentage_price_type,percentage_property_id,quantity,quantity_action,property_order,property_step,property_name,property_hint,hide_name,property_code,property_description,property_class,property_style,property_price_type,free_price_type,free_price_amount,max_limit_type,max_limit_length,length_units,use_on_list,use_on_table,use_on_grid,use_on_details,use_on_checkout,use_on_second,control_type,control_style,control_code,onchange_code,onclick_code,required,start_html,middle_html,before_control_html,after_control_html,end_html,sites_all) VALUES (4 , 3 , NULL , NULL , 1 , 1 , 0 , 24 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 'Profile Universal' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 1 , 0 , 0 , 'CHECKBOXLIST' , NULL , NULL , NULL , NULL , 0 , '<div style="margin: 0px 0px 10px 0px;"><div style="display: none;">' , '</div>' , NULL , NULL , '</div>' , NULL ) ;
INSERT INTO va_items_properties (property_id,property_type_id,parent_property_id,parent_value_id,show_for_user,usage_type,item_type_id,item_id,sub_item_id,additional_price,trade_additional_price,percentage_price_type,percentage_property_id,quantity,quantity_action,property_order,property_step,property_name,property_hint,hide_name,property_code,property_description,property_class,property_style,property_price_type,free_price_type,free_price_amount,max_limit_type,max_limit_length,length_units,use_on_list,use_on_table,use_on_grid,use_on_details,use_on_checkout,use_on_second,control_type,control_style,control_code,onchange_code,onclick_code,required,start_html,middle_html,before_control_html,after_control_html,end_html,sites_all) VALUES (5 , 3 , NULL , NULL , 1 , 1 , 0 , 26 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 'Profile Universal' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 1 , 0 , 0 , 'CHECKBOXLIST' , NULL , NULL , NULL , NULL , 0 , '<div style="margin: 0px 0px 10px 0px;"><div style="display: none;">' , '</div>' , NULL , NULL , '</div>' , NULL ) ;
INSERT INTO va_items_properties (property_id,property_type_id,parent_property_id,parent_value_id,show_for_user,usage_type,item_type_id,item_id,sub_item_id,additional_price,trade_additional_price,percentage_price_type,percentage_property_id,quantity,quantity_action,property_order,property_step,property_name,property_hint,hide_name,property_code,property_description,property_class,property_style,property_price_type,free_price_type,free_price_amount,max_limit_type,max_limit_length,length_units,use_on_list,use_on_table,use_on_grid,use_on_details,use_on_checkout,use_on_second,control_type,control_style,control_code,onchange_code,onclick_code,required,start_html,middle_html,before_control_html,after_control_html,end_html,sites_all) VALUES (6 , 3 , NULL , NULL , 1 , 1 , 5 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 , NULL , 'Accessories' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 1 , 0 , 0 , 'CHECKBOXLIST' , NULL , NULL , NULL , NULL , 0 , '<div style="font-weight: bold; margin: 2px 2px 6px 6px;">' , '</div>' , '<div style="margin: 0px 2px 6px 2px;">' , '</div>' , NULL , NULL ) ;
INSERT INTO va_items_properties (property_id,property_type_id,parent_property_id,parent_value_id,show_for_user,usage_type,item_type_id,item_id,sub_item_id,additional_price,trade_additional_price,percentage_price_type,percentage_property_id,quantity,quantity_action,property_order,property_step,property_name,property_hint,hide_name,property_code,property_description,property_class,property_style,property_price_type,free_price_type,free_price_amount,max_limit_type,max_limit_length,length_units,use_on_list,use_on_table,use_on_grid,use_on_details,use_on_checkout,use_on_second,control_type,control_style,control_code,onchange_code,onclick_code,required,start_html,middle_html,before_control_html,after_control_html,end_html,sites_all) VALUES (7 , 3 , NULL , NULL , 1 , 1 , 8 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 2 , NULL , 'Accessories' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 1 , 0 , 0 , 'CHECKBOXLIST' , NULL , NULL , NULL , NULL , 0 , '<div style="font-weight: bold; margin: 2px 2px 6px 6px;">' , '</div>' , '<div style="margin: 0px 2px 6px 2px;">' , '</div>' , NULL , NULL ) ;
INSERT INTO va_items_properties (property_id,property_type_id,parent_property_id,parent_value_id,show_for_user,usage_type,item_type_id,item_id,sub_item_id,additional_price,trade_additional_price,percentage_price_type,percentage_property_id,quantity,quantity_action,property_order,property_step,property_name,property_hint,hide_name,property_code,property_description,property_class,property_style,property_price_type,free_price_type,free_price_amount,max_limit_type,max_limit_length,length_units,use_on_list,use_on_table,use_on_grid,use_on_details,use_on_checkout,use_on_second,control_type,control_style,control_code,onchange_code,onclick_code,required,start_html,middle_html,before_control_html,after_control_html,end_html,sites_all) VALUES (8 , 1 , NULL , NULL , 1 , 1 , 0 , 32 , 0 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 'Color' , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , NULL , 0 , NULL , NULL , 0 , 0 , 0 , 1 , 0 , 0 , 'LISTBOX' , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , NULL , NULL , 1 ) ;


INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (1 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (3 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (4 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (5 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (6 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (7 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (8 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (9 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (10 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (11 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (12 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (13 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (14 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (15 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (16 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (17 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (18 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (19 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (20 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (21 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (22 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (23 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (24 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (25 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (26 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (27 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (28 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (29 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (30 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (31 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (32 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (33 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (34 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (35 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (36 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (37 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (38 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (39 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (40 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (41 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (42 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (43 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (44 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (45 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (46 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (47 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (48 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (49 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (50 , 0 , NULL ) ;
INSERT INTO va_items_properties_assigned (item_id,property_id,property_description) VALUES (52 , 0 , NULL ) ;






INSERT INTO va_items_properties_values (item_property_id,property_id,sub_item_id,property_value,value_order,item_code,manufacturer_code,download_period,download_path,download_files_ids,buying_price,additional_price,trade_additional_price,quantity,percentage_price,additional_weight,use_stock_level,stock_level,hide_out_of_stock,hide_value,is_default_value,image_tiny,image_small,image_large,image_super) VALUES (1 , 1 , 43 , 'Flat Panel Mount 32" to 60" Displays' , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_items_properties_values (item_property_id,property_id,sub_item_id,property_value,value_order,item_code,manufacturer_code,download_period,download_path,download_files_ids,buying_price,additional_price,trade_additional_price,quantity,percentage_price,additional_weight,use_stock_level,stock_level,hide_out_of_stock,hide_value,is_default_value,image_tiny,image_small,image_large,image_super) VALUES (3 , 3 , 43 , 'Flat Panel Mount 32" to 60" Displays' , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_items_properties_values (item_property_id,property_id,sub_item_id,property_value,value_order,item_code,manufacturer_code,download_period,download_path,download_files_ids,buying_price,additional_price,trade_additional_price,quantity,percentage_price,additional_weight,use_stock_level,stock_level,hide_out_of_stock,hide_value,is_default_value,image_tiny,image_small,image_large,image_super) VALUES (4 , 4 , 43 , 'Flat Panel Mount 32" to 60" Displays' , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_items_properties_values (item_property_id,property_id,sub_item_id,property_value,value_order,item_code,manufacturer_code,download_period,download_path,download_files_ids,buying_price,additional_price,trade_additional_price,quantity,percentage_price,additional_weight,use_stock_level,stock_level,hide_out_of_stock,hide_value,is_default_value,image_tiny,image_small,image_large,image_super) VALUES (5 , 5 , 43 , 'Flat Panel Mount 32" to 60" Displays' , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_items_properties_values (item_property_id,property_id,sub_item_id,property_value,value_order,item_code,manufacturer_code,download_period,download_path,download_files_ids,buying_price,additional_price,trade_additional_price,quantity,percentage_price,additional_weight,use_stock_level,stock_level,hide_out_of_stock,hide_value,is_default_value,image_tiny,image_small,image_large,image_super) VALUES (6 , 6 , 44 , 'diNovo Media Desktop Laser' , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_items_properties_values (item_property_id,property_id,sub_item_id,property_value,value_order,item_code,manufacturer_code,download_period,download_path,download_files_ids,buying_price,additional_price,trade_additional_price,quantity,percentage_price,additional_weight,use_stock_level,stock_level,hide_out_of_stock,hide_value,is_default_value,image_tiny,image_small,image_large,image_super) VALUES (7 , 6 , 45 , 'QuickCam Pro for Notebooks' , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_items_properties_values (item_property_id,property_id,sub_item_id,property_value,value_order,item_code,manufacturer_code,download_period,download_path,download_files_ids,buying_price,additional_price,trade_additional_price,quantity,percentage_price,additional_weight,use_stock_level,stock_level,hide_out_of_stock,hide_value,is_default_value,image_tiny,image_small,image_large,image_super) VALUES (8 , 6 , 46 , 'Digital Precision PC Gaming Headset' , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_items_properties_values (item_property_id,property_id,sub_item_id,property_value,value_order,item_code,manufacturer_code,download_period,download_path,download_files_ids,buying_price,additional_price,trade_additional_price,quantity,percentage_price,additional_weight,use_stock_level,stock_level,hide_out_of_stock,hide_value,is_default_value,image_tiny,image_small,image_large,image_super) VALUES (9 , 6 , 47 , 'AudioHub' , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_items_properties_values (item_property_id,property_id,sub_item_id,property_value,value_order,item_code,manufacturer_code,download_period,download_path,download_files_ids,buying_price,additional_price,trade_additional_price,quantity,percentage_price,additional_weight,use_stock_level,stock_level,hide_out_of_stock,hide_value,is_default_value,image_tiny,image_small,image_large,image_super) VALUES (10 , 7 , 38 , 'HA-SX500' , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_items_properties_values (item_property_id,property_id,sub_item_id,property_value,value_order,item_code,manufacturer_code,download_period,download_path,download_files_ids,buying_price,additional_price,trade_additional_price,quantity,percentage_price,additional_weight,use_stock_level,stock_level,hide_out_of_stock,hide_value,is_default_value,image_tiny,image_small,image_large,image_super) VALUES (11 , 7 , 39 , 'Noise Canceling Headphones MDR-NC6' , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_items_properties_values (item_property_id,property_id,sub_item_id,property_value,value_order,item_code,manufacturer_code,download_period,download_path,download_files_ids,buying_price,additional_price,trade_additional_price,quantity,percentage_price,additional_weight,use_stock_level,stock_level,hide_out_of_stock,hide_value,is_default_value,image_tiny,image_small,image_large,image_super) VALUES (12 , 7 , 40 , 'Active Style Headphones MDR-AS50G' , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_items_properties_values (item_property_id,property_id,sub_item_id,property_value,value_order,item_code,manufacturer_code,download_period,download_path,download_files_ids,buying_price,additional_price,trade_additional_price,quantity,percentage_price,additional_weight,use_stock_level,stock_level,hide_out_of_stock,hide_value,is_default_value,image_tiny,image_small,image_large,image_super) VALUES (13 , 7 , 41 , 'S2 Sports Street Style Headphones MDR-G57G' , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_items_properties_values (item_property_id,property_id,sub_item_id,property_value,value_order,item_code,manufacturer_code,download_period,download_path,download_files_ids,buying_price,additional_price,trade_additional_price,quantity,percentage_price,additional_weight,use_stock_level,stock_level,hide_out_of_stock,hide_value,is_default_value,image_tiny,image_small,image_large,image_super) VALUES (14 , 8 , 0 , 'Black' , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_items_properties_values (item_property_id,property_id,sub_item_id,property_value,value_order,item_code,manufacturer_code,download_period,download_path,download_files_ids,buying_price,additional_price,trade_additional_price,quantity,percentage_price,additional_weight,use_stock_level,stock_level,hide_out_of_stock,hide_value,is_default_value,image_tiny,image_small,image_large,image_super) VALUES (15 , 8 , 0 , 'White' , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , 0 , NULL , NULL , NULL , NULL ) ;


INSERT INTO va_items_related (item_id,related_id,related_order) VALUES (1 , 5 , 1 ) ;
INSERT INTO va_items_related (item_id,related_id,related_order) VALUES (32 , 36 , 1 ) ;
INSERT INTO va_items_related (item_id,related_id,related_order) VALUES (33 , 34 , 1 ) ;
INSERT INTO va_items_related (item_id,related_id,related_order) VALUES (34 , 36 , 1 ) ;
INSERT INTO va_items_related (item_id,related_id,related_order) VALUES (35 , 36 , 1 ) ;
INSERT INTO va_items_related (item_id,related_id,related_order) VALUES (36 , 35 , 1 ) ;












INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (1 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (3 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (4 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (5 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (6 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (7 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (8 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (9 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (10 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (11 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (12 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (13 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (14 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (15 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (16 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (17 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (18 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (19 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (20 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (21 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (22 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (23 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (24 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (25 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (26 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (27 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (28 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (29 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (30 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (31 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (32 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (33 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (34 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (35 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (36 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (37 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (38 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (39 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (40 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (41 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (42 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (43 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (44 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (45 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (46 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (47 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (48 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (49 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (50 , 0 , 1 ) ;
INSERT INTO va_items_values_assigned (item_id,property_value_id,is_default_value) VALUES (52 , 0 , 1 ) ;






INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('ar' , 1 , 'Arabic' , 1 , 'images/flags/sa.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('bg' , 1 , 'Bulgarian' , 1 , 'images/flags/ba.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('br' , 1 , 'Brazilian' , 1 , 'images/flags/br.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('cs' , 1 , 'Czech' , 1 , 'images/flags/cz.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('da' , 1 , 'Danish' , 1 , 'images/flags/da.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('de' , 1 , 'German' , 1 , 'images/flags/de.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('el' , 1 , 'Greek' , 1 , 'images/flags/gr.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('en' , 1 , 'English' , 1 , 'images/flags/gb.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('es' , 1 , 'Spanish' , 1 , 'images/flags/es.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('et' , 1 , 'Estonian' , 1 , 'images/flags/et.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('fa' , 1 , 'Persian' , 1 , 'images/flags/ir.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('fi' , 1 , 'Finnish' , 1 , 'images/flags/fi.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('fr' , 1 , 'French' , 1 , 'images/flags/fr.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('he' , 1 , 'Hebrew' , 1 , 'images/flags/he.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('hk' , 1 , 'Cantonese' , 1 , 'images/flags/hk.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('hr' , 1 , 'Croatian' , 1 , 'images/flags/hr.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('hu' , 1 , 'Hungarian' , 1 , 'images/flags/hu.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('is' , 1 , 'Icelandic' , 1 , 'images/flags/is.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('it' , 1 , 'Italian' , 1 , 'images/flags/it.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('ja' , 1 , 'Japanese' , 1 , 'images/flags/jp.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('lt' , 1 , 'Lithuanian' , 1 , 'images/flags/lt.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('lv' , 1 , 'Latvian' , 1 , 'images/flags/lv.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('mk' , 1 , 'Macedonian' , 1 , 'images/flags/mk.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('nl' , 1 , 'Dutch' , 1 , 'images/flags/nl.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('no' , 1 , 'Norweigian' , 1 , 'images/flags/no.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('pl' , 1 , 'Polish' , 1 , 'images/flags/pl.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('pt' , 1 , 'Portuguese' , 1 , 'images/flags/pt.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('ro' , 1 , 'Romanian' , 1 , 'images/flags/ro.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('ru' , 1 , 'Russian' , 1 , 'images/flags/ru.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('sk' , 1 , 'Slovak' , 1 , 'images/flags/sk.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('sv' , 1 , 'Swedish' , 1 , 'images/flags/se.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('tr' , 1 , 'Turkish' , 1 , 'images/flags/tr.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('uk' , 1 , 'Ukrainian' , 1 , 'images/flags/ua.gif' , NULL , NULL ) ;
INSERT INTO va_languages (language_code,language_order,language_name,show_for_user,language_image,language_image_active,currency_code) VALUES ('vi' , 1 , 'Vietnamese' , 1 , 'images/flags/vi.gif' , NULL , NULL ) ;


INSERT INTO va_layouts (layout_id,sites_all,show_for_user,layout_name,user_layout_name,top_menu_type,style_name,scheme_name,templates_dir,admin_templates_dir,block_default_template,block_area_template,block_breadcrumb_template) VALUES (1 , 1 , 0 , 'Default' , NULL , 1 , 'default' , NULL , './templates/user' , '../templates/admin' , NULL , NULL , NULL ) ;
INSERT INTO va_layouts (layout_id,sites_all,show_for_user,layout_name,user_layout_name,top_menu_type,style_name,scheme_name,templates_dir,admin_templates_dir,block_default_template,block_area_template,block_breadcrumb_template) VALUES (2 , 1 , 0 , 'Curved' , NULL , 3 , 'curved' , NULL , './templates/user' , '../templates/admin' , NULL , NULL , NULL ) ;
INSERT INTO va_layouts (layout_id,sites_all,show_for_user,layout_name,user_layout_name,top_menu_type,style_name,scheme_name,templates_dir,admin_templates_dir,block_default_template,block_area_template,block_breadcrumb_template) VALUES (3 , 1 , 0 , 'Silver' , NULL , 1 , 'silver' , NULL , './templates/user' , '../templates/admin' , NULL , NULL , NULL ) ;
INSERT INTO va_layouts (layout_id,sites_all,show_for_user,layout_name,user_layout_name,top_menu_type,style_name,scheme_name,templates_dir,admin_templates_dir,block_default_template,block_area_template,block_breadcrumb_template) VALUES (4 , 1 , 0 , 'Convex' , NULL , 1 , 'convex' , NULL , './templates/user' , '../templates/admin' , NULL , NULL , NULL ) ;
INSERT INTO va_layouts (layout_id,sites_all,show_for_user,layout_name,user_layout_name,top_menu_type,style_name,scheme_name,templates_dir,admin_templates_dir,block_default_template,block_area_template,block_breadcrumb_template) VALUES (5 , 1 , 0 , 'Fashion' , NULL , 1 , 'fashion' , NULL , './templates/user' , '../templates/admin' , NULL , NULL , NULL ) ;
INSERT INTO va_layouts (layout_id,sites_all,show_for_user,layout_name,user_layout_name,top_menu_type,style_name,scheme_name,templates_dir,admin_templates_dir,block_default_template,block_area_template,block_breadcrumb_template) VALUES (6 , 1 , 0 , 'Rainbow' , NULL , 1 , 'rainbow' , NULL , './templates/user' , '../templates/admin' , NULL , NULL , NULL ) ;
INSERT INTO va_layouts (layout_id,sites_all,show_for_user,layout_name,user_layout_name,top_menu_type,style_name,scheme_name,templates_dir,admin_templates_dir,block_default_template,block_area_template,block_breadcrumb_template) VALUES (7 , 1 , 0 , 'Classic' , NULL , 1 , 'classic' , NULL , './templates/user' , '../templates/admin' , NULL , NULL , NULL ) ;
INSERT INTO va_layouts (layout_id,sites_all,show_for_user,layout_name,user_layout_name,top_menu_type,style_name,scheme_name,templates_dir,admin_templates_dir,block_default_template,block_area_template,block_breadcrumb_template) VALUES (8 , 1 , 0 , 'Marine' , NULL , 1 , 'marine' , NULL , './templates/user' , '../templates/admin' , NULL , NULL , NULL ) ;
INSERT INTO va_layouts (layout_id,sites_all,show_for_user,layout_name,user_layout_name,top_menu_type,style_name,scheme_name,templates_dir,admin_templates_dir,block_default_template,block_area_template,block_breadcrumb_template) VALUES (9 , 1 , 0 , 'Ecommerce' , NULL , 1 , 'ecommerce' , NULL , './templates/user' , '../templates/admin' , NULL , NULL , NULL ) ;
INSERT INTO va_layouts (layout_id,sites_all,show_for_user,layout_name,user_layout_name,top_menu_type,style_name,scheme_name,templates_dir,admin_templates_dir,block_default_template,block_area_template,block_breadcrumb_template) VALUES (10 , 1 , 0 , 'Double' , NULL , 1 , 'double' , NULL , './templates/user' , '../templates/admin' , NULL , NULL , NULL ) ;
INSERT INTO va_layouts (layout_id,sites_all,show_for_user,layout_name,user_layout_name,top_menu_type,style_name,scheme_name,templates_dir,admin_templates_dir,block_default_template,block_area_template,block_breadcrumb_template) VALUES (11 , 1 , 0 , 'Demo' , NULL , 1 , 'demo' , NULL , './templates/user' , '../templates/admin' , NULL , NULL , NULL ) ;
INSERT INTO va_layouts (layout_id,sites_all,show_for_user,layout_name,user_layout_name,top_menu_type,style_name,scheme_name,templates_dir,admin_templates_dir,block_default_template,block_area_template,block_breadcrumb_template) VALUES (12 , 1 , 0 , 'Mobile' , NULL , 1 , 'mobile' , NULL , './templates/mobile' , '../templates/admin' , NULL , NULL , NULL ) ;
















INSERT INTO va_manufacturers (manufacturer_id,manufacturer_name,manufacturer_order,friendly_url,affiliate_code,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt) VALUES (3 , 'Sony' , 1 , NULL , NULL , NULL , NULL , 'images/manufacturers/small/sony.gif' , 'sony' , 'images/manufacturers/large/sony.gif' , 'sony' ) ;
INSERT INTO va_manufacturers (manufacturer_id,manufacturer_name,manufacturer_order,friendly_url,affiliate_code,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt) VALUES (4 , 'Dell' , 2 , NULL , NULL , NULL , NULL , 'images/manufacturers/small/dell.gif' , 'Dell' , 'images/manufacturers/large/dell.gif' , 'Dell' ) ;
INSERT INTO va_manufacturers (manufacturer_id,manufacturer_name,manufacturer_order,friendly_url,affiliate_code,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt) VALUES (5 , 'Canon' , 3 , NULL , NULL , NULL , NULL , 'images/manufacturers/small/canon.gif' , 'Canon' , 'images/manufacturers/large/canon.gif' , 'Canon' ) ;
INSERT INTO va_manufacturers (manufacturer_id,manufacturer_name,manufacturer_order,friendly_url,affiliate_code,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt) VALUES (6 , 'Logitech' , 4 , NULL , NULL , NULL , NULL , 'images/manufacturers/small/logitech.gif' , 'Logitech' , 'images/manufacturers/large/logitech.gif' , 'Logitech' ) ;
INSERT INTO va_manufacturers (manufacturer_id,manufacturer_name,manufacturer_order,friendly_url,affiliate_code,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt) VALUES (7 , 'JVC' , 5 , NULL , NULL , NULL , NULL , 'images/manufacturers/small/jvc.gif' , 'JVC' , 'images/manufacturers/large/jvc.gif' , 'JVC' ) ;
INSERT INTO va_manufacturers (manufacturer_id,manufacturer_name,manufacturer_order,friendly_url,affiliate_code,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt) VALUES (8 , 'Nikon' , 6 , NULL , NULL , NULL , NULL , 'images/manufacturers/small/nikon.gif' , 'Nikon' , 'images/manufacturers/large/nikon.gif' , 'Nikon' ) ;
INSERT INTO va_manufacturers (manufacturer_id,manufacturer_name,manufacturer_order,friendly_url,affiliate_code,short_description,full_description,image_small,image_small_alt,image_large,image_large_alt) VALUES (9 , 'Philips' , 7 , NULL , NULL , NULL , NULL , 'images/manufacturers/small/philips.gif' , 'Philips' , 'images/manufacturers/large/philips.gif' , 'Philips' ) ;


INSERT INTO va_menus (menu_id,menu_type,menu_title,menu_name,block_class,menu_class,menu_notes,show_title,sites_all) VALUES (1 , 1 , NULL , 'NAVIGATION_BAR_MSG' , NULL , 'nav-bar' , NULL , 0 , 1 ) ;
INSERT INTO va_menus (menu_id,menu_type,menu_title,menu_name,block_class,menu_class,menu_notes,show_title,sites_all) VALUES (2 , 2 , NULL , 'HEADER_MENU_MSG' , NULL , 'nav-header' , NULL , 0 , 1 ) ;
INSERT INTO va_menus (menu_id,menu_type,menu_title,menu_name,block_class,menu_class,menu_notes,show_title,sites_all) VALUES (3 , 4 , NULL , 'FOOTER_MENU_MSG' , NULL , 'nav-footer' , NULL , 0 , 1 ) ;
INSERT INTO va_menus (menu_id,menu_type,menu_title,menu_name,block_class,menu_class,menu_notes,show_title,sites_all) VALUES (4 , 5 , NULL , 'ADMINISTRATIVE_MENU_MSG' , NULL , 'nav-admin' , NULL , 0 , 1 ) ;


INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (1 , 2 , 0 , NULL , 'MENU_HOME' , 'index.php' , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 1 , '1' , 'all' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (31 , 3 , 0 , NULL , 'NEWS_TITLE' , 'articles.php?category_id=1' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , '1' , 'all' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (32 , 3 , 0 , NULL , 'MENU_FAQ' , 'articles.php?category_id=4' , NULL , NULL , NULL , 2 , NULL , NULL , NULL , 1 , '1' , 'all' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (33 , 3 , 0 , NULL , 'Lyrics' , 'articles.php?category_id=3' , NULL , NULL , NULL , 3 , NULL , NULL , NULL , 1 , '1' , 'all' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (34 , 3 , 0 , NULL , 'IMAGES_MSG' , 'articles.php?category_id=5' , NULL , NULL , NULL , 4 , NULL , NULL , NULL , 0 , '0' , NULL ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (35 , 3 , 0 , NULL , 'EVENTS_TITLE' , 'articles.php?category_id=6' , NULL , NULL , NULL , 5 , NULL , NULL , NULL , 0 , '0' , NULL ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (36 , 3 , 0 , NULL , 'SITE_MAP_TITLE' , 'site_map.php' , NULL , NULL , NULL , 6 , NULL , NULL , NULL , 1 , '1' , 'all' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (37 , 3 , 0 , NULL , 'MENU_FORUM' , 'forum.php' , NULL , NULL , NULL , 7 , NULL , NULL , NULL , 1 , '1' , 'all' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (38 , 3 , 0 , NULL , 'MENU_SUPPORT' , 'support.php' , NULL , NULL , NULL , 8 , NULL , NULL , NULL , 1 , '1' , 'all' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (39 , 3 , 0 , NULL , 'MENU_HOW' , 'page.php?page=help' , NULL , NULL , NULL , 9 , NULL , NULL , NULL , 1 , '1' , 'all' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (40 , 3 , 0 , NULL , 'MENU_ABOUT' , 'page.php?page=about' , NULL , NULL , NULL , 10 , NULL , NULL , NULL , 1 , '1' , 'all' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (41 , 3 , 0 , NULL , 'MENU_SUPPORT' , 'support.php' , NULL , NULL , NULL , 11 , NULL , NULL , NULL , 1 , '1' , 'all' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (42 , 3 , 0 , NULL , 'PROFILES_TITLE' , 'profiles.php' , NULL , NULL , NULL , 12 , NULL , NULL , NULL , 1 , '1' , 'all' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (43 , 4 , 0 , 'dashboard' , 'DASHBOARD_MSG' , 'admin_menu.php?code=dashboard' , NULL , NULL , NULL , 1 , 'dashboard' , NULL , NULL , 1 , NULL , 'all' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (44 , 4 , 43 , 'products' , 'PRODUCTS_MSG' , 'admin_items_list.php' , NULL , NULL , NULL , 1 , 'section js-no' , NULL , NULL , 1 , NULL , 'products_categories,products_reviews,products_report' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (45 , 4 , 44 , 'products-categories' , 'PRODUCTS_CATEGORIES_MSG' , 'admin_items_list.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'products_categories' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (46 , 4 , 44 , NULL , 'PRODUCTS_REVIEWS_MSG' , 'admin_reviews.php' , NULL , NULL , NULL , 5 , NULL , NULL , NULL , 1 , NULL , 'products_reviews' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (47 , 4 , 44 , NULL , 'PRODUCTS_REPORT' , 'admin_products_report.php' , NULL , NULL , NULL , 8 , NULL , NULL , NULL , 1 , NULL , 'products_report' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (48 , 4 , 43 , 'orders' , 'ORDERS_MSG' , 'admin_orders.php' , NULL , NULL , NULL , 2 , 'section js-no' , NULL , NULL , 1 , NULL , 'sales_orders,orders_recover,orders_stats' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (49 , 4 , 48 , NULL , 'SALES_ORDERS_MSG' , 'admin_orders.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'sales_orders' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (50 , 4 , 48 , NULL , 'ORDERS_RECOVER_MSG' , 'admin_orders_recover.php' , NULL , NULL , NULL , 2 , NULL , NULL , NULL , 1 , NULL , 'orders_recover' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (51 , 4 , 48 , NULL , 'ORDERS_REPORTS_MSG' , 'admin_orders_report.php' , NULL , NULL , NULL , 3 , NULL , NULL , NULL , 1 , NULL , 'orders_stats' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (52 , 4 , 48 , NULL , 'CARTS_REPORT_MSG' , 'admin_carts_report.php' , NULL , NULL , NULL , 4 , NULL , NULL , NULL , 1 , NULL , 'orders_stats' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (53 , 4 , 43 , 'articles' , 'ARTICLES_TITLE' , 'admin_articles_all.php' , NULL , NULL , NULL , 3 , 'section js-no' , NULL , NULL , 1 , NULL , 'articles,articles_reviews' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (54 , 4 , 53 , 'articles-categories' , 'ARTICLES_TITLE' , 'admin_articles_all.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'articles' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (55 , 4 , 53 , NULL , 'ARTICLES_REVIEWS_MSG' , 'admin_articles_reviews.php' , NULL , NULL , NULL , 2 , NULL , NULL , NULL , 1 , NULL , 'articles_reviews' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (56 , 4 , 43 , 'helpdesk' , 'HELPDESK_MSG' , 'admin_support.php' , NULL , NULL , NULL , 4 , 'section js-no' , NULL , NULL , 1 , NULL , 'support,support_users_stats' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (57 , 4 , 56 , NULL , 'SUPPORT_TICKETS_MSG' , 'admin_support.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'support' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (58 , 4 , 56 , NULL , 'CHATS_MSG' , 'admin_support_chats.php' , NULL , NULL , NULL , 2 , NULL , NULL , NULL , 1 , NULL , 'support' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (59 , 4 , 56 , NULL , 'OPERATORS_REPORT_MSG' , 'admin_support_users_report.php' , NULL , NULL , NULL , 3 , NULL , NULL , NULL , 1 , NULL , 'support_users_stats' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (61 , 4 , 43 , 'forum' , 'ADMIN_FORUM_TITLE' , 'admin_forum.php' , NULL , NULL , NULL , 5 , 'section js-no' , NULL , NULL , 1 , NULL , 'forum' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (62 , 4 , 61 , 'forum-categories' , 'CATEGORIES_TITLE' , 'admin_forum.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'forum' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (63 , 4 , 43 , 'manual' , 'MANUAL_MSG' , 'admin_manual.php' , NULL , NULL , NULL , 6 , 'section js-no' , NULL , NULL , 1 , NULL , 'manual' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (64 , 4 , 63 , 'manual-categories' , 'MANUAL_MSG' , 'admin_manual.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'manual' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (65 , 4 , 43 , 'ads' , 'ADS_TITLE' , 'admin_ads.php' , NULL , NULL , NULL , 7 , 'section js-no' , NULL , NULL , 1 , NULL , 'ads' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (66 , 4 , 65 , 'ads-categories' , 'ADS_TITLE' , 'admin_ads.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'ads' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (67 , 4 , 43 , 'registration' , 'PRODUCT_REGISTRATION_MSG' , 'admin_registrations.php' , NULL , NULL , NULL , 8 , 'section js-no' , NULL , NULL , 1 , NULL , 'admin_registration' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (68 , 4 , 67 , NULL , 'PRODUCT_REGISTRATION_MSG' , 'admin_registrations.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'admin_registration' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (69 , 4 , 67 , NULL , 'REGISTRATION_PRODUCTS_MSG' , 'admin_registration_products.php' , NULL , NULL , NULL , 2 , NULL , NULL , NULL , 1 , NULL , 'admin_registration' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (70 , 4 , 43 , 'users' , 'CUSTOMERS_MSG' , 'admin_users.php' , NULL , NULL , NULL , 9 , 'section js-no' , NULL , NULL , 1 , NULL , 'site_users,newsletter' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (71 , 4 , 70 , NULL , 'ACCOUNTS_MSG' , 'admin_users.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'site_users' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (72 , 4 , 70 , NULL , 'NEWSLETTER_USERS_MSG' , 'admin_newsletter_users.php' , NULL , NULL , NULL , 2 , NULL , NULL , NULL , 1 , NULL , 'newsletter' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (73 , 4 , 43 , 'profiles' , 'PROFILES_TITLE' , 'admin_profiles.php' , NULL , NULL , NULL , 10 , 'section js-no' , NULL , NULL , 1 , NULL , 'profiles' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (74 , 4 , 0 , 'cms' , 'CMS_MSG' , 'admin_cms.php' , NULL , NULL , NULL , 2 , 'cms' , NULL , NULL , 1 , NULL , 'all' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (75 , 4 , 74 , 'layouts' , 'CMS_MSG' , 'admin_cms.php' , NULL , NULL , NULL , 1 , 'section js-no' , NULL , NULL , 1 , NULL , 'cms_settings' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (76 , 4 , 75 , NULL , 'PAGES_LAYOUTS_MSG' , 'admin_cms.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'cms_settings' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (77 , 4 , 75 , NULL , 'MULTI_EDIT_MSG' , 'admin_cms_multi_edit.php' , NULL , NULL , NULL , 2 , NULL , NULL , NULL , 1 , NULL , 'cms_settings' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (78 , 4 , 75 , NULL , 'CMS_LAYOUTS_MSG' , 'admin_cms_layouts.php' , NULL , NULL , NULL , 3 , NULL , NULL , NULL , 1 , NULL , 'cms_settings' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (79 , 4 , 75 , NULL , 'CMS_MODULES_MSG' , 'admin_cms_modules.php' , NULL , NULL , NULL , 4 , NULL , NULL , NULL , 1 , NULL , 'cms_settings' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (80 , 4 , 75 , NULL , 'CMS_PAGES_MSG' , 'admin_cms_pages.php' , NULL , NULL , NULL , 5 , NULL , NULL , NULL , 1 , NULL , 'cms_settings' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (81 , 4 , 75 , NULL , 'CMS_BLOCKS_MSG' , 'admin_cms_blocks.php' , NULL , NULL , NULL , 6 , NULL , NULL , NULL , 1 , NULL , 'cms_settings' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (82 , 4 , 75 , NULL , 'DESIGNS_MSG' , 'admin_designs.php' , NULL , NULL , NULL , 7 , NULL , NULL , NULL , 1 , NULL , 'cms_settings' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (83 , 4 , 74 , 'custom-modules' , 'CUSTOM_MODULES_MSG' , 'admin_menu.php?code=custom-modules' , NULL , NULL , NULL , 2 , 'section js-no' , NULL , NULL , 1 , NULL , 'site_navigation,custom_blocks,web_pages,custom_friendly_urls,polls,filters,sliders' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (84 , 4 , 83 , NULL , 'SITE_NAVIGATION_MSG' , 'admin_menu_list.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'site_navigation' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (85 , 4 , 83 , NULL , 'CUSTOM_BLOCKS_MSG' , 'admin_custom_blocks.php' , NULL , NULL , NULL , 2 , NULL , NULL , NULL , 1 , NULL , 'custom_blocks' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (86 , 4 , 83 , NULL , 'CUSTOM_PAGES_MSG' , 'admin_pages.php' , NULL , NULL , NULL , 3 , NULL , NULL , NULL , 1 , NULL , 'web_pages' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (87 , 4 , 83 , NULL , 'CUSTOM_FRIENDLY_URLS_MSG' , 'admin_friendly_urls.php' , NULL , NULL , NULL , 4 , NULL , NULL , NULL , 1 , NULL , 'custom_friendly_urls' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (88 , 4 , 99 , NULL , 'BANNERS_MSG' , 'admin_banners.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'banners' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (89 , 4 , 99 , NULL , 'BANNERS_GROUPS_MSG' , 'admin_banners_groups.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'banners' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (90 , 4 , 83 , NULL , 'OPINION_POLLS_MSG' , 'admin_polls.php' , NULL , NULL , NULL , 7 , NULL , NULL , NULL , 1 , NULL , 'polls' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (91 , 4 , 83 , NULL , 'FILTERS_MSG' , 'admin_filters.php' , NULL , NULL , NULL , 8 , NULL , NULL , NULL , 1 , NULL , 'filters' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (92 , 4 , 83 , NULL , 'SLIDERS_MSG' , 'admin_sliders.php' , NULL , NULL , NULL , 9 , NULL , NULL , NULL , 1 , NULL , 'sliders' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (94 , 4 , 0 , 'settings' , 'SETTINGS_MSG' , 'admin_menu.php?code=settings' , NULL , NULL , NULL , 4 , 'settings' , NULL , NULL , 1 , NULL , 'all' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (96 , 4 , 94 , 'articles-forum-settings' , '{ARTICLES_TITLE} & {ADMIN_FORUM_TITLE}' , 'admin_articles_top.php' , NULL , NULL , NULL , 4 , 'section-group js-no' , NULL , NULL , 1 , NULL , 'articles,articles_statuses,articles_reviews_settings,forum' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (97 , 4 , 94 , 'users-profiles-settings' , '{CUSTOMERS_MSG} & {PROFILES_TITLE}' , 'admin_user_types.php' , NULL , NULL , NULL , 7 , 'section-group js-no' , NULL , NULL , 1 , NULL , 'users_groups,static_tables,subscriptions_groups,subscriptions,users_forgot,users_payments,profiles' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (99 , 4 , 74 , 'banners' , 'BANNERS_MSG' , 'admin_banners.php' , NULL , NULL , NULL , 3 , 'section js-no' , NULL , NULL , 1 , NULL , 'banners' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (104 , 4 , 0 , 'tools' , 'TOOLS_MSG' , 'admin_menu.php?code=tools' , NULL , NULL , NULL , 3 , 'tools' , NULL , NULL , 1 , NULL , 'all' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (105 , 4 , 104 , NULL , 'DATABASE_MANAGEMENT_MSG' , 'admin_dump.php' , NULL , NULL , NULL , 1 , 'section js-no' , NULL , NULL , 1 , NULL , 'db_management,system_upgrade' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (106 , 4 , 105 , NULL , 'APPLY_DUMP_MSG' , 'admin_dump.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'db_management' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (107 , 4 , 105 , NULL , 'CREATE_NEW_DUMP_MSG' , 'admin_dump_create.php' , NULL , NULL , NULL , 2 , NULL , NULL , NULL , 1 , NULL , 'db_management' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (108 , 4 , 105 , NULL , 'RUN_SQL_QUERY_MSG' , 'admin_db_query.php' , NULL , NULL , NULL , 3 , NULL , NULL , NULL , 1 , NULL , 'db_management' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (109 , 4 , 105 , NULL , 'SYSTEM_UPGRADE_MSG' , 'admin_upgrade.php' , NULL , NULL , NULL , 4 , NULL , NULL , NULL , 1 , NULL , 'system_upgrade' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (110 , 4 , 104 , 'export-tools' , 'EXPORT_MSG' , 'admin_export_templates.php' , NULL , NULL , NULL , 5 , 'section js-no' , NULL , NULL , 1 , NULL , 'products_export,export_users,sales_orders' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (111 , 4 , 110 , NULL , 'EXPORT_TEMPLATES_MSG' , 'admin_export_templates.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'products_export,export_users,sales_orders' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (112 , 4 , 104 , 'email-campaigns' , 'EMAIL_CAMPAIGNS_MSG' , 'admin_newsletter_campaigns.php' , NULL , NULL , NULL , 4 , 'section js-no' , NULL , NULL , 1 , NULL , 'newsletter' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (113 , 4 , 112 , NULL , 'EMAIL_CAMPAIGNS_MSG' , 'admin_newsletter_campaigns.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'newsletter' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (114 , 4 , 112 , NULL , 'NEWSLETTER_USERS_MSG' , 'admin_newsletter_users.php' , NULL , NULL , NULL , 2 , NULL , NULL , NULL , 1 , NULL , 'newsletter' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (115 , 4 , 104 , 'content-tools' , 'TOOLS_MSG' , 'admin_fm.php' , NULL , NULL , NULL , 2 , 'section js-no' , NULL , NULL , 1 , NULL , 'filemanager,black_ips,static_tables,banned_contents,visits_report,all' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (116 , 4 , 115 , NULL , 'FILE_MANAGER_MSG' , 'admin_fm.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'filemanager' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (117 , 4 , 115 , NULL , 'FILE_TRANSFERS_MSG' , 'admin_file_transfers.php' , NULL , NULL , NULL , 2 , NULL , NULL , NULL , 1 , NULL , 'filemanager' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (118 , 4 , 115 , NULL , 'RESIZE_IMAGES_MSG' , 'admin_images_resize.php' , NULL , NULL , NULL , 3 , NULL , NULL , NULL , 1 , NULL , 'filemanager' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (119 , 4 , 115 , NULL , 'BLACK_IPS_MSG' , 'admin_black_ips.php' , NULL , NULL , NULL , 4 , NULL , NULL , NULL , 1 , NULL , 'black_ips' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (120 , 4 , 115 , NULL , 'COUNTRIES_IPS_MSG' , 'admin_ips_countries.php' , NULL , NULL , NULL , 5 , NULL , NULL , NULL , 1 , NULL , 'static_tables' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (121 , 4 , 115 , NULL , 'BANNED_CONTENT' , 'admin_banned_contents.php' , NULL , NULL , NULL , 6 , NULL , NULL , NULL , 1 , NULL , 'banned_contents' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (123 , 4 , 115 , NULL , 'BOOKMARKS_MSG' , 'admin_bookmarks.php' , NULL , NULL , NULL , 8 , NULL , NULL , NULL , 1 , NULL , 'all' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (125 , 4 , 94 , 'system-settings' , 'SYSTEM_MSG' , 'admin_global_settings.php' , NULL , NULL , NULL , 1 , 'section js-no' , NULL , NULL , 1 , NULL , 'site_settings,admin_sites,admin_users,admins_groups,static_tables,static_messages' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (126 , 4 , 125 , NULL , 'GLOBAL_SETTINGS_MSG' , 'admin_global_settings.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'site_settings' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (127 , 4 , 125 , NULL , 'ADMIN_SITES_MSG' , 'admin_sites.php' , NULL , NULL , NULL , 2 , NULL , NULL , NULL , 1 , NULL , 'admin_sites' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (128 , 4 , 125 , NULL , 'ADMINISTRATORS_MSG' , 'admin_admins.php' , NULL , NULL , NULL , 3 , NULL , NULL , NULL , 1 , NULL , 'admin_users' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (129 , 4 , 125 , NULL , 'PRIVILEGE_GROUPS_MSG' , 'admin_privileges.php' , NULL , NULL , NULL , 4 , NULL , NULL , NULL , 1 , NULL , 'admins_groups' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (130 , 4 , 125 , NULL , 'TWO_FACTOR_AUTH_MSG' , 'settings_two_factor.php' , NULL , NULL , NULL , 5 , NULL , NULL , NULL , 1 , NULL , 'admins_groups' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (131 , 4 , 125 , NULL , 'STATIC_TABLES_MSG' , 'admin_static_tables.php' , NULL , NULL , NULL , 6 , NULL , NULL , NULL , 1 , NULL , 'static_tables' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (132 , 4 , 125 , NULL , 'SYSTEM_STATIC_MESSAGES_MSG' , 'admin_messages.php' , NULL , NULL , NULL , 7 , NULL , NULL , NULL , 1 , NULL , 'static_messages' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (133 , 4 , 125 , NULL , 'CONTACT_US_MSG' , 'admin_contact_us.php' , NULL , NULL , NULL , 8 , NULL , NULL , NULL , 1 , NULL , 'site_settings' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (134 , 4 , 125 , NULL , 'INTERNAL_MESSAGES_MSG' , 'admin_messages_settings.php' , NULL , NULL , NULL , 9 , NULL , NULL , NULL , 1 , NULL , 'site_settings' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (135 , 4 , 94 , 'products-settings' , 'PRODUCTS_MSG' , 'admin_products_settings.php' , NULL , NULL , NULL , 1 , 'section js-no' , NULL , NULL , 1 , NULL , 'products_settings,product_types,manufacturers,suppliers,features_groups,coupons,update_products,downloadable_products,products_reviews_settings,tell_friend' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (136 , 4 , 135 , NULL , 'PRODUCTS_SETTINGS_MSG' , 'admin_products_settings.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'products_settings' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (137 , 4 , 135 , NULL , 'PRODUCTS_TYPES_MSG' , 'admin_item_types.php' , NULL , NULL , NULL , 2 , NULL , NULL , NULL , 1 , NULL , 'product_types' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (138 , 4 , 135 , NULL , 'MANUFACTURERS_TITLE' , 'admin_manufacturers.php' , NULL , NULL , NULL , 3 , NULL , NULL , NULL , 1 , NULL , 'manufacturers' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (139 , 4 , 135 , NULL , 'SUPPLIERS_MSG' , 'admin_suppliers.php' , NULL , NULL , NULL , 4 , NULL , NULL , NULL , 1 , NULL , 'suppliers' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (140 , 4 , 135 , NULL , 'SPECIFICATION_GROUPS_MSG' , 'admin_features_groups.php' , NULL , NULL , NULL , 5 , NULL , NULL , NULL , 1 , NULL , 'features_groups' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (141 , 4 , 135 , NULL , 'WISHLIST_TYPES_MSG' , 'admin_saved_types.php' , NULL , NULL , NULL , 6 , NULL , NULL , NULL , 1 , NULL , 'saved_types' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (142 , 4 , 135 , NULL , 'COUPONS_MSG' , 'admin_coupons.php' , NULL , NULL , NULL , 7 , NULL , NULL , NULL , 1 , NULL , 'coupons' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (143 , 4 , 135 , NULL , 'KEYWORDS_MSG' , 'admin_keywords.php' , NULL , NULL , NULL , 8 , NULL , NULL , NULL , 1 , NULL , 'update_products' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (144 , 4 , 135 , NULL , 'DOWNLOADABLE_PRODUCTS_MSG' , 'admin_download_info.php' , NULL , NULL , NULL , 9 , NULL , NULL , NULL , 1 , NULL , 'downloadable_products' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (145 , 4 , 135 , NULL , 'ADVANCED_SEARCH_TITLE' , 'admin_search.php' , NULL , NULL , NULL , 10 , NULL , NULL , NULL , 1 , NULL , 'advanced_search' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (146 , 4 , 135 , NULL , 'REVIEWS_SETTINGS_MSG' , 'admin_products_reviews_sets.php' , NULL , NULL , NULL , 11 , NULL , NULL , NULL , 1 , NULL , 'products_reviews_settings' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (147 , 4 , 135 , NULL , 'TELL_FRIEND' , 'admin_tell_friend.php?type=products' , NULL , NULL , NULL , 12 , NULL , NULL , NULL , 1 , NULL , 'tell_friend' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (148 , 4 , 135 , NULL , 'SAVED_CART_NOTIFICATION_MSG' , 'admin_saved_cart_notify.php' , NULL , NULL , NULL , 13 , NULL , NULL , NULL , 1 , NULL , 'products_categories' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (149 , 4 , 94 , 'orders-settings' , 'ORDERS_MSG' , 'admin_menu.php?code=orders-settings' , NULL , NULL , NULL , 3 , 'section js-no' , NULL , NULL , 1 , NULL , 'sales_orders,order_profile,shipping_methods,shipping_times,shipping_rules,payment_systems,order_statuses,tax_rates' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (150 , 4 , 149 , NULL , 'ORDER_PROFILE_PAGE_MSG' , 'admin_order_info.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'order_profile' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (151 , 4 , 149 , NULL , 'SHIPPING_METHODS_MSG' , 'admin_shipping_modules.php' , NULL , NULL , NULL , 2 , NULL , NULL , NULL , 1 , NULL , 'shipping_methods' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (152 , 4 , 149 , NULL , 'SHIPPING_TIMES_MSG' , 'admin_shipping_times.php' , NULL , NULL , NULL , 3 , NULL , NULL , NULL , 1 , NULL , 'shipping_times' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (153 , 4 , 149 , NULL , 'SHIPPING_RULES_MSG' , 'admin_shipping_rules.php' , NULL , NULL , NULL , 4 , NULL , NULL , NULL , 1 , NULL , 'shipping_rules' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (154 , 4 , 149 , NULL , 'PAYMENT_SYSTEMS_MSG' , 'admin_payment_systems.php' , NULL , NULL , NULL , 5 , NULL , NULL , NULL , 1 , NULL , 'payment_systems' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (156 , 4 , 149 , NULL , 'ORDERS_STATUSES_MSG' , 'admin_order_statuses.php' , NULL , NULL , NULL , 6 , NULL , NULL , NULL , 1 , NULL , 'order_statuses' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (157 , 4 , 149 , NULL , 'TAX_RATES_MSG' , 'admin_tax_rates.php' , NULL , NULL , NULL , 7 , NULL , NULL , NULL , 1 , NULL , 'tax_rates' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (158 , 4 , 149 , NULL , 'CURRENCIES_MSG' , 'admin_currencies.php' , NULL , NULL , NULL , 8 , NULL , NULL , NULL , 1 , NULL , 'static_tables' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (159 , 4 , 149 , NULL , 'PRINTABLE_PAGE_SETTINGS_MSG' , 'admin_order_printable.php' , NULL , NULL , NULL , 9 , NULL , NULL , NULL , 1 , NULL , 'order_profile' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (160 , 4 , 149 , NULL , 'BOM_SETTINGS_MSG' , 'admin_orders_bom_settings.php' , NULL , NULL , NULL , 10 , NULL , NULL , NULL , 1 , NULL , 'order_profile' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (161 , 4 , 149 , NULL , 'ORDERS_RECOVER_MSG' , 'admin_orders_recover_settings.php' , NULL , NULL , NULL , 11 , NULL , NULL , NULL , 1 , NULL , 'orders_recover' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (162 , 4 , 96 , 'articles-settings' , 'ARTICLES_TITLE' , 'admin_articles_top.php' , NULL , NULL , NULL , 1 , 'section js-no' , NULL , NULL , 1 , NULL , 'articles,articles_statuses,articles_reviews_settings' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (163 , 4 , 162 , NULL , 'ARTICLES_SETTINGS_MSG' , 'admin_articles_top.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'articles' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (164 , 4 , 162 , NULL , 'ARTICLES_STATUSES_MSG' , 'admin_articles_statuses.php' , NULL , NULL , NULL , 2 , NULL , NULL , NULL , 1 , NULL , 'articles_statuses' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (165 , 4 , 162 , NULL , 'REVIEWS_SETTINGS_MSG' , 'admin_articles_reviews_sets.php' , NULL , NULL , NULL , 3 , NULL , NULL , NULL , 1 , NULL , 'articles_reviews_settings' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (166 , 4 , 162 , NULL , 'ARTICLES_LOST_MSG' , 'admin_articles_lost.php' , NULL , NULL , NULL , 4 , NULL , NULL , NULL , 1 , NULL , 'articles' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (167 , 4 , 94 , 'helpdesk-settings' , 'HELPDESK_MSG' , 'admin_support_settings.php' , NULL , NULL , NULL , 5 , 'section js-no' , NULL , NULL , 1 , NULL , 'support_settings,support_users,support_users_priorities,support_departments,support_static_data,support_predefined_reply' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (168 , 4 , 167 , NULL , 'HELPDESK_SETTINGS_MSG' , 'admin_support_settings.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'support_settings' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (169 , 4 , 167 , NULL , 'SUPPORT_USERS_MSG' , 'admin_support_admins.php' , NULL , NULL , NULL , 2 , NULL , NULL , NULL , 1 , NULL , 'support_users' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (170 , 4 , 167 , NULL , 'CUSTOMERS_RANKS_MSG' , 'admin_support_ranks.php' , NULL , NULL , NULL , 3 , NULL , NULL , NULL , 1 , NULL , 'support_users_priorities' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (171 , 4 , 167 , NULL , 'DEPARTMENTS_MSG' , 'admin_support_departments.php' , NULL , NULL , NULL , 4 , NULL , NULL , NULL , 1 , NULL , 'support_departments' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (172 , 4 , 167 , NULL , 'SUPPORT_TYPES_MSG' , 'admin_support_types.php' , NULL , NULL , NULL , 5 , NULL , NULL , NULL , 1 , NULL , 'support_static_data' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (173 , 4 , 167 , NULL , 'SUPPORT_PRODUCTS_MSG' , 'admin_support_products.php' , NULL , NULL , NULL , 6 , NULL , NULL , NULL , 1 , NULL , 'support_static_data' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (174 , 4 , 167 , NULL , 'SUPPORT_PRIORITIES_MSG' , 'admin_support_priorities.php' , NULL , NULL , NULL , 7 , NULL , NULL , NULL , 1 , NULL , 'support_static_data' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (175 , 4 , 167 , NULL , 'SUPPORT_STATUSES_MSG' , 'admin_support_statuses.php' , NULL , NULL , NULL , 8 , NULL , NULL , NULL , 1 , NULL , 'support_static_data' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (176 , 4 , 167 , NULL , 'PREDEFINED_REPLIES_MSG' , 'admin_support_prereplies.php' , NULL , NULL , NULL , 9 , NULL , NULL , NULL , 1 , NULL , 'support_predefined_reply' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (177 , 4 , 167 , NULL , 'PREDEFINED_TYPES_MSG' , 'admin_support_pretypes.php' , NULL , NULL , NULL , 10 , NULL , NULL , NULL , 1 , NULL , 'support_predefined_reply' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (178 , 4 , 96 , 'forum-settings' , 'ADMIN_FORUM_TITLE' , 'admin_forum_settings.php' , NULL , NULL , NULL , 2 , 'section js-no' , NULL , NULL , 1 , NULL , 'forum,static_tables' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (179 , 4 , 178 , NULL , 'FORUM_SETTINGS_MSG' , 'admin_forum_settings.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'forum' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (180 , 4 , 178 , NULL , 'FORUM_PRIORITIES_MSG' , 'admin_forum_priorities.php' , NULL , NULL , NULL , 2 , NULL , NULL , NULL , 1 , NULL , 'forum' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (181 , 4 , 178 , NULL , 'EMOTION_ICONS_MSG' , 'admin_icons.php' , NULL , NULL , NULL , 3 , NULL , NULL , NULL , 1 , NULL , 'static_tables' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (182 , 4 , 94 , 'ads-settings' , 'ADS_TITLE' , 'admin_ads_settings.php' , NULL , NULL , NULL , 6 , 'section js-no' , NULL , NULL , 1 , NULL , 'ads' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (183 , 4 , 182 , NULL , 'ADS_SETTINGS_MSG' , 'admin_ads_settings.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'ads' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (184 , 4 , 182 , NULL , 'ADS_TYPES_MSG' , 'admin_ads_types.php' , NULL , NULL , NULL , 2 , NULL , NULL , NULL , 1 , NULL , 'ads' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (185 , 4 , 182 , NULL , 'ADVERT_NOTIFICATION_MSG' , 'admin_ads_notify.php' , NULL , NULL , NULL , 3 , NULL , NULL , NULL , 1 , NULL , 'ads' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (186 , 4 , 182 , NULL , 'ADVERT_REQUEST_MSG' , 'admin_ads_request.php' , NULL , NULL , NULL , 4 , NULL , NULL , NULL , 1 , NULL , 'ads' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (187 , 4 , 182 , NULL , 'ADVANCED_SEARCH_TITLE' , 'admin_ads_search.php' , NULL , NULL , NULL , 5 , NULL , NULL , NULL , 1 , NULL , 'ads' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (188 , 4 , 182 , NULL , 'TELL_FRIEND' , 'admin_tell_friend.php?type=ads' , NULL , NULL , NULL , 6 , NULL , NULL , NULL , 1 , NULL , 'ads' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (189 , 4 , 182 , NULL , 'SPECIFICATION_GROUPS_MSG' , 'admin_ads_features_groups.php' , NULL , NULL , NULL , 7 , NULL , NULL , NULL , 1 , NULL , 'ads' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (190 , 4 , 182 , NULL , 'ADS_DAYS_MSG' , 'admin_ads_days.php' , NULL , NULL , NULL , 8 , NULL , NULL , NULL , 1 , NULL , 'ads' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (191 , 4 , 182 , NULL , 'ADS_HOT_DAYS_MSG' , 'admin_ads_hot_days.php' , NULL , NULL , NULL , 9 , NULL , NULL , NULL , 1 , NULL , 'ads' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (192 , 4 , 182 , NULL , 'ADS_SPECIAL_DAYS_MSG' , 'admin_ads_special_days.php' , NULL , NULL , NULL , 10 , NULL , NULL , NULL , 1 , NULL , 'ads' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (193 , 4 , 97 , 'registration-settings' , 'PRODUCT_REGISTRATION_MSG' , 'admin_registration_settings.php' , NULL , NULL , NULL , 3 , 'section js-no' , NULL , NULL , 1 , NULL , 'admin_registration' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (194 , 4 , 193 , NULL , 'SETTINGS_MSG' , 'admin_registration_settings.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'admin_registration' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (195 , 4 , 97 , 'users-settings' , 'CUSTOMERS_MSG' , 'admin_user_types.php' , NULL , NULL , NULL , 1 , 'section js-no' , NULL , NULL , 1 , NULL , 'users_groups,static_tables,subscriptions_groups,subscriptions,users_forgot,users_payments' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (196 , 4 , 195 , NULL , 'USERS_TYPES_MSG' , 'admin_user_types.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'users_groups' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (197 , 4 , 195 , NULL , 'FORM_SECTIONS_MSG' , 'admin_user_sections.php' , NULL , NULL , NULL , 2 , NULL , NULL , NULL , 1 , NULL , 'static_tables' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (198 , 4 , 195 , NULL , 'SUBSCRIPTIONS_GROUPS_MSG' , 'admin_subscriptions_groups.php' , NULL , NULL , NULL , 3 , NULL , NULL , NULL , 1 , NULL , 'subscriptions_groups' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (199 , 4 , 195 , NULL , 'SUBSCRIPTIONS_MSG' , 'admin_subscriptions.php' , NULL , NULL , NULL , 4 , NULL , NULL , NULL , 1 , NULL , 'subscriptions' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (200 , 4 , 195 , NULL , 'FORGOTTEN_PASSWORD_MSG' , 'admin_forgotten_password.php' , NULL , NULL , NULL , 5 , NULL , NULL , NULL , 1 , NULL , 'users_forgot' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (201 , 4 , 195 , NULL , 'CUSTOMERS_COMISSIONS_MSG' , 'admin_user_commissions.php' , NULL , NULL , NULL , 6 , NULL , NULL , NULL , 1 , NULL , 'users_payments' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (202 , 4 , 195 , NULL , 'COMMISSION_PAYMENTS' , 'admin_user_payments.php' , NULL , NULL , NULL , 7 , NULL , NULL , NULL , 1 , NULL , 'users_payments' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (203 , 4 , 97 , 'profiles-settings' , 'PROFILES_TITLE' , 'admin_profiles_settings.php' , NULL , NULL , NULL , 2 , 'section js-no' , NULL , NULL , 1 , NULL , 'profiles' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (204 , 4 , 203 , NULL , 'SETTINGS_MSG' , 'admin_profiles_settings.php' , NULL , NULL , NULL , 1 , NULL , NULL , NULL , 1 , NULL , 'profiles' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (212 , 4 , 104 , 'reports' , 'REPORTS_MSG' , 'admin_menu.php?code=reports' , NULL , NULL , NULL , 3 , 'section js-no' , NULL , NULL , 1 , NULL , 'products_report,visits_report,support_users_stats' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (213 , 4 , 212 , NULL , 'PRODUCTS_REPORT' , 'admin_products_report.php' , NULL , NULL , NULL , 8 , NULL , NULL , NULL , 1 , NULL , 'products_report' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (214 , 4 , 212 , NULL , 'ORDERS_REPORTS_MSG' , 'admin_orders_report.php' , NULL , NULL , NULL , 3 , NULL , NULL , NULL , 1 , NULL , 'orders_stats' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (215 , 4 , 212 , NULL , 'CARTS_REPORT_MSG' , 'admin_carts_report.php' , NULL , NULL , NULL , 4 , NULL , NULL , NULL , 1 , NULL , 'orders_stats' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (216 , 4 , 212 , NULL , '{HELPDESK_MSG} {OPERATORS_REPORT_MSG}' , 'admin_support_users_report.php' , NULL , NULL , NULL , 3 , NULL , NULL , NULL , 1 , NULL , 'support_users_stats' ) ;
INSERT INTO va_menus_items (menu_item_id,menu_id,parent_menu_item_id,menu_code,menu_title,menu_url,menu_target,menu_image,menu_image_active,menu_order,menu_class,head_content,foot_content,guest_access,user_access,admin_access) VALUES (217 , 4 , 212 , NULL , 'TRACKING_VISITS_REPORT_MSG' , 'admin_visits_report.php' , NULL , NULL , NULL , 7 , NULL , NULL , NULL , 1 , NULL , 'visits_report' ) ;






















INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (1 , 1 , 45 , 1 , NULL , 'Password (2 digits, Korean cards ONLY)' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'TEXTBOX' , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , NULL , ' /^\\d{2}$/' , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (2 , 1 , 46 , 2 , NULL , 'Account Type' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'LISTBOX' , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (3 , 1 , 46 , 3 , NULL , 'Bank Routing Number' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'TEXTBOX' , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , NULL , '/^\\d+$/' , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (4 , 1 , 46 , 4 , NULL , 'Bank Account Number' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'TEXTBOX' , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , NULL , '/^\\d+$/' , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (5 , 1 , 46 , 5 , NULL , 'Check Number' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'TEXTBOX' , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , NULL , '/^\\d+$/' , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (6 , 1 , 46 , 6 , NULL , 'ID Issuing State' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'LISTBOX' , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (7 , 1 , 46 , 7 , NULL , 'State ID  License' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'TEXTBOX' , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (8 , 1 , 46 , 8 , NULL , 'Last 4 digits of SSN' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'TEXTBOX' , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , NULL , '/^\\d{4}$/' , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (9 , 1 , 46 , 9 , NULL , 'Bank name' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'TEXTBOX' , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (10 , 1 , 46 , 10 , NULL , 'City of the bank' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'TEXTBOX' , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (11 , 1 , 46 , 11 , NULL , 'State of the bank' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'LISTBOX' , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (12 , 1 , 46 , 12 , NULL , 'Date of Birth (mmddyyyy)' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'TEXTBOX' , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , NULL , '/^\\d{8}$/' , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (13 , 1 , 46 , 13 , NULL , 'Name' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'TEXTBOX' , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (14 , 1 , 46 , 14 , NULL , 'Password' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'TEXTBOX' , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (15 , 1 , 47 , 15 , NULL , 'First 11 Characters of the ID Card (10 digits and 1 letter)' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'TEXTBOX' , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , '/^\\d{10}\\w{1}$/' , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (16 , 1 , 47 , 16 , NULL , 'Second set of digits of the ID Card (7 digits long)' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'TEXTBOX' , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , '/^\\d{7}$/' , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (17 , 1 , 47 , 17 , NULL , 'Third set of digits of the ID Card (7 digits long)' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'TEXTBOX' , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , '/^\\d{7}$/' , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (18 , 1 , 47 , 18 , NULL , 'Final digit of the ID card' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'TEXTBOX' , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , '/^\\d{1}$/' , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (19 , 1 , 47 , 19 , NULL , 'Name' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'TEXTBOX' , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (20 , 1 , 47 , 20 , NULL , 'Password' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'TEXTBOX' , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (21 , 1 , 76 , 21 , NULL , 'domestic bank account' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'TEXTBOX' , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (22 , 1 , 76 , 22 , NULL , 'The domestic code of the bank' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'TEXTBOX' , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (23 , 1 , 76 , 23 , NULL , 'Country code' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'TEXTBOX' , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (24 , 1 , 78 , 24 , NULL , 'accounting number' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'TEXTBOX' , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (25 , 1 , 78 , 25 , NULL , 'bankcode' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'TEXTBOX' , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (26 , 1 , 80 , 26 , NULL , 'social security number/company registration number' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'TEXTBOX' , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (27 , 1 , 86 , 27 , NULL , 'User Name' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'TEXTBOX' , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_order_custom_properties (property_id,site_id,payment_id,property_order,property_code,property_name,property_description,default_value,property_class,property_style,property_type,property_show,tax_free,control_type,control_style,control_code,onchange_code,onclick_code,required,before_name_html,after_name_html,before_control_html,after_control_html,validation_regexp,regexp_error,options_values_sql) VALUES (28 , 1 , 86 , 28 , NULL , 'User Password' , NULL , NULL , NULL , NULL , 4 , 0 , 0 , 'TEXTBOX' , NULL , NULL , NULL , NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;


INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (1 , 2 , 'PC' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (2 , 2 , 'PS' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (3 , 6 , 'AK' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (4 , 6 , 'AL' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (5 , 6 , 'AR' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (6 , 6 , 'AS' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (7 , 6 , 'AZ' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (8 , 6 , 'CA' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (9 , 6 , 'CO' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (10 , 6 , 'CT' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (11 , 6 , 'DC' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (12 , 6 , 'DE' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (13 , 6 , 'FL' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (14 , 6 , 'FM' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (15 , 6 , 'GA' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (16 , 6 , 'GU' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (17 , 6 , 'HI' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (18 , 6 , 'IA' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (19 , 6 , 'ID' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (20 , 6 , 'IL' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (21 , 6 , 'IN' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (22 , 6 , 'KS' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (23 , 6 , 'KY' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (24 , 6 , 'LA' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (25 , 6 , 'MA' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (26 , 6 , 'MD' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (27 , 6 , 'ME' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (28 , 6 , 'MH' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (29 , 6 , 'MI' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (30 , 6 , 'MN' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (31 , 6 , 'MP' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (32 , 6 , 'MS' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (33 , 6 , 'MT' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (34 , 6 , 'NC' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (35 , 6 , 'ND' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (36 , 6 , 'NE' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (37 , 6 , 'NH' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (38 , 6 , 'NJ' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (39 , 6 , 'NM' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (40 , 6 , 'NV' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (41 , 6 , 'NY' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (42 , 6 , 'OH' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (43 , 6 , 'OK' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (44 , 6 , 'OR' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (45 , 6 , 'PA' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (46 , 6 , 'PR' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (47 , 6 , 'PW' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (48 , 6 , 'RI' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (49 , 6 , 'SC' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (50 , 6 , 'SD' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (51 , 6 , 'TN' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (52 , 6 , 'TX' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (53 , 6 , 'UT' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (54 , 6 , 'VA' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (55 , 6 , 'VI' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (56 , 6 , 'VT' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (57 , 6 , 'WA' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (58 , 6 , 'WV' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (59 , 6 , 'WY' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (60 , 6 , 'AB' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (61 , 6 , 'BC' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (62 , 6 , 'MB' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (63 , 6 , 'NB' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (64 , 6 , 'NF' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (65 , 6 , 'NS' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (66 , 6 , 'NT' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (67 , 6 , 'NU' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (68 , 6 , 'ON' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (69 , 6 , 'PE' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (70 , 6 , 'QC' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (71 , 6 , 'SK' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (72 , 6 , 'YT' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (73 , 6 , 'XX' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (74 , 11 , 'AK' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (75 , 11 , 'AL' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (76 , 11 , 'AR' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (77 , 11 , 'AS' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (78 , 11 , 'AZ' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (79 , 11 , 'CA' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (80 , 11 , 'CO' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (81 , 11 , 'CT' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (82 , 11 , 'DC' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (83 , 11 , 'DE' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (84 , 11 , 'FL' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (85 , 11 , 'FM' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (86 , 11 , 'GA' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (87 , 11 , 'GU' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (88 , 11 , 'HI' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (89 , 11 , 'IA' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (90 , 11 , 'ID' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (91 , 11 , 'IL' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (92 , 11 , 'IN' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (93 , 11 , 'KS' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (94 , 11 , 'KY' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (95 , 11 , 'LA' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (96 , 11 , 'MA' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (97 , 11 , 'MD' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (98 , 11 , 'ME' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (99 , 11 , 'MH' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (100 , 11 , 'MI' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (101 , 11 , 'MN' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (102 , 11 , 'MP' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (103 , 11 , 'MS' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (104 , 11 , 'MT' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (105 , 11 , 'NC' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (106 , 11 , 'ND' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (107 , 11 , 'NE' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (108 , 11 , 'NH' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (109 , 11 , 'NJ' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (110 , 11 , 'NM' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (111 , 11 , 'NV' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (112 , 11 , 'NY' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (113 , 11 , 'OH' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (114 , 11 , 'OK' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (115 , 11 , 'OR' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (116 , 11 , 'PA' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (117 , 11 , 'PR' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (118 , 11 , 'PW' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (119 , 11 , 'RI' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (120 , 11 , 'SC' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (121 , 11 , 'SD' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (122 , 11 , 'TN' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (123 , 11 , 'TX' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (124 , 11 , 'UT' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (125 , 11 , 'VA' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (126 , 11 , 'VI' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (127 , 11 , 'VT' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (128 , 11 , 'WA' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (129 , 11 , 'WV' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (130 , 11 , 'WY' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (131 , 11 , 'AB' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (132 , 11 , 'BC' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (133 , 11 , 'MB' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (134 , 11 , 'NB' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (135 , 11 , 'NF' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (136 , 11 , 'NS' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (137 , 11 , 'NT' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (138 , 11 , 'NU' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (139 , 11 , 'ON' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (140 , 11 , 'PE' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (141 , 11 , 'QC' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (142 , 11 , 'SK' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (143 , 11 , 'YT' , NULL , NULL , 0 , 0 ) ;
INSERT INTO va_order_custom_values (property_value_id,property_id,property_value,property_price,property_weight,hide_value,is_default_value) VALUES (144 , 11 , 'XX' , NULL , NULL , 0 , 0 ) ;


INSERT INTO va_order_statuses (status_id,status_order,status_name,status_type,admin_order_class,user_order_class,is_user_cancel,allow_user_cancel,generate_invoice,user_invoice_activation,is_dispatch,is_list,show_for_user,paid_status,credit_note_action,stock_level_action,points_action,credit_action,commission_action,download_activation,download_notify,item_notify,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_pdf_invoice,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,merchant_notify,merchant_to,merchant_from,merchant_cc,merchant_bcc,merchant_reply_to,merchant_return_path,merchant_mail_type,merchant_subject,merchant_body,merchant_sms_notify,merchant_sms_recipient,merchant_sms_originator,merchant_sms_message,supplier_notify,supplier_to,supplier_from,supplier_cc,supplier_bcc,supplier_reply_to,supplier_return_path,supplier_mail_type,supplier_subject,supplier_body,supplier_sms_notify,supplier_sms_recipient,supplier_sms_originator,supplier_sms_message,affiliate_notify,affiliate_to,affiliate_from,affiliate_cc,affiliate_bcc,affiliate_reply_to,affiliate_return_path,affiliate_mail_type,affiliate_subject,affiliate_body,affiliate_sms_notify,affiliate_sms_recipient,affiliate_sms_originator,affiliate_sms_message,admin_notify,admin_to,admin_to_groups_ids,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_pdf_invoice,admin_subject,admin_body,admin_sms_notify,admin_sms_recipient,admin_sms_originator,admin_sms_message,final_title,final_message,is_active,view_order_groups_all,view_order_groups_ids,set_status_groups_all,set_status_groups_ids,update_order_groups_all,update_order_groups_ids) VALUES (1 , 1 , 'STATUS_NEW_ORDER_MSG' , 'NEW' , NULL , NULL , 0 , 0 , 1 , 0 , 0 , 1 , 1 , 0 , 0 , 1 , -1 , -1 , -1 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , NULL , 1 , NULL ) ;
INSERT INTO va_order_statuses (status_id,status_order,status_name,status_type,admin_order_class,user_order_class,is_user_cancel,allow_user_cancel,generate_invoice,user_invoice_activation,is_dispatch,is_list,show_for_user,paid_status,credit_note_action,stock_level_action,points_action,credit_action,commission_action,download_activation,download_notify,item_notify,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_pdf_invoice,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,merchant_notify,merchant_to,merchant_from,merchant_cc,merchant_bcc,merchant_reply_to,merchant_return_path,merchant_mail_type,merchant_subject,merchant_body,merchant_sms_notify,merchant_sms_recipient,merchant_sms_originator,merchant_sms_message,supplier_notify,supplier_to,supplier_from,supplier_cc,supplier_bcc,supplier_reply_to,supplier_return_path,supplier_mail_type,supplier_subject,supplier_body,supplier_sms_notify,supplier_sms_recipient,supplier_sms_originator,supplier_sms_message,affiliate_notify,affiliate_to,affiliate_from,affiliate_cc,affiliate_bcc,affiliate_reply_to,affiliate_return_path,affiliate_mail_type,affiliate_subject,affiliate_body,affiliate_sms_notify,affiliate_sms_recipient,affiliate_sms_originator,affiliate_sms_message,admin_notify,admin_to,admin_to_groups_ids,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_pdf_invoice,admin_subject,admin_body,admin_sms_notify,admin_sms_recipient,admin_sms_originator,admin_sms_message,final_title,final_message,is_active,view_order_groups_all,view_order_groups_ids,set_status_groups_all,set_status_groups_ids,update_order_groups_all,update_order_groups_ids) VALUES (2 , 2 , 'STATUS_PAYMENT_DATA_MSG' , 'PAYMENT_INFO' , NULL , NULL , 0 , 0 , 1 , 0 , 0 , 1 , 1 , 0 , 0 , 1 , -1 , -1 , -1 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , NULL , 1 , NULL ) ;
INSERT INTO va_order_statuses (status_id,status_order,status_name,status_type,admin_order_class,user_order_class,is_user_cancel,allow_user_cancel,generate_invoice,user_invoice_activation,is_dispatch,is_list,show_for_user,paid_status,credit_note_action,stock_level_action,points_action,credit_action,commission_action,download_activation,download_notify,item_notify,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_pdf_invoice,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,merchant_notify,merchant_to,merchant_from,merchant_cc,merchant_bcc,merchant_reply_to,merchant_return_path,merchant_mail_type,merchant_subject,merchant_body,merchant_sms_notify,merchant_sms_recipient,merchant_sms_originator,merchant_sms_message,supplier_notify,supplier_to,supplier_from,supplier_cc,supplier_bcc,supplier_reply_to,supplier_return_path,supplier_mail_type,supplier_subject,supplier_body,supplier_sms_notify,supplier_sms_recipient,supplier_sms_originator,supplier_sms_message,affiliate_notify,affiliate_to,affiliate_from,affiliate_cc,affiliate_bcc,affiliate_reply_to,affiliate_return_path,affiliate_mail_type,affiliate_subject,affiliate_body,affiliate_sms_notify,affiliate_sms_recipient,affiliate_sms_originator,affiliate_sms_message,admin_notify,admin_to,admin_to_groups_ids,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_pdf_invoice,admin_subject,admin_body,admin_sms_notify,admin_sms_recipient,admin_sms_originator,admin_sms_message,final_title,final_message,is_active,view_order_groups_all,view_order_groups_ids,set_status_groups_all,set_status_groups_ids,update_order_groups_all,update_order_groups_ids) VALUES (3 , 3 , 'STATUS_CONFIRMED_MSG' , 'CONFIRMED' , NULL , NULL , 0 , 0 , 1 , 0 , 0 , 1 , 1 , 0 , 0 , 1 , -1 , -1 , -1 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , NULL , 1 , NULL ) ;
INSERT INTO va_order_statuses (status_id,status_order,status_name,status_type,admin_order_class,user_order_class,is_user_cancel,allow_user_cancel,generate_invoice,user_invoice_activation,is_dispatch,is_list,show_for_user,paid_status,credit_note_action,stock_level_action,points_action,credit_action,commission_action,download_activation,download_notify,item_notify,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_pdf_invoice,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,merchant_notify,merchant_to,merchant_from,merchant_cc,merchant_bcc,merchant_reply_to,merchant_return_path,merchant_mail_type,merchant_subject,merchant_body,merchant_sms_notify,merchant_sms_recipient,merchant_sms_originator,merchant_sms_message,supplier_notify,supplier_to,supplier_from,supplier_cc,supplier_bcc,supplier_reply_to,supplier_return_path,supplier_mail_type,supplier_subject,supplier_body,supplier_sms_notify,supplier_sms_recipient,supplier_sms_originator,supplier_sms_message,affiliate_notify,affiliate_to,affiliate_from,affiliate_cc,affiliate_bcc,affiliate_reply_to,affiliate_return_path,affiliate_mail_type,affiliate_subject,affiliate_body,affiliate_sms_notify,affiliate_sms_recipient,affiliate_sms_originator,affiliate_sms_message,admin_notify,admin_to,admin_to_groups_ids,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_pdf_invoice,admin_subject,admin_body,admin_sms_notify,admin_sms_recipient,admin_sms_originator,admin_sms_message,final_title,final_message,is_active,view_order_groups_all,view_order_groups_ids,set_status_groups_all,set_status_groups_ids,update_order_groups_all,update_order_groups_ids) VALUES (4 , 4 , 'STATUS_PAID_MSG' , 'PAID' , NULL , NULL , 0 , 0 , 1 , 1 , 0 , 1 , 1 , 1 , 0 , 1 , -1 , -1 , 1 , 1 , 1 , 1 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , NULL , 1 , NULL ) ;
INSERT INTO va_order_statuses (status_id,status_order,status_name,status_type,admin_order_class,user_order_class,is_user_cancel,allow_user_cancel,generate_invoice,user_invoice_activation,is_dispatch,is_list,show_for_user,paid_status,credit_note_action,stock_level_action,points_action,credit_action,commission_action,download_activation,download_notify,item_notify,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_pdf_invoice,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,merchant_notify,merchant_to,merchant_from,merchant_cc,merchant_bcc,merchant_reply_to,merchant_return_path,merchant_mail_type,merchant_subject,merchant_body,merchant_sms_notify,merchant_sms_recipient,merchant_sms_originator,merchant_sms_message,supplier_notify,supplier_to,supplier_from,supplier_cc,supplier_bcc,supplier_reply_to,supplier_return_path,supplier_mail_type,supplier_subject,supplier_body,supplier_sms_notify,supplier_sms_recipient,supplier_sms_originator,supplier_sms_message,affiliate_notify,affiliate_to,affiliate_from,affiliate_cc,affiliate_bcc,affiliate_reply_to,affiliate_return_path,affiliate_mail_type,affiliate_subject,affiliate_body,affiliate_sms_notify,affiliate_sms_recipient,affiliate_sms_originator,affiliate_sms_message,admin_notify,admin_to,admin_to_groups_ids,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_pdf_invoice,admin_subject,admin_body,admin_sms_notify,admin_sms_recipient,admin_sms_originator,admin_sms_message,final_title,final_message,is_active,view_order_groups_all,view_order_groups_ids,set_status_groups_all,set_status_groups_ids,update_order_groups_all,update_order_groups_ids) VALUES (5 , 5 , 'STATUS_SHIPPED_MSG' , 'SHIPPED' , NULL , NULL , 0 , 0 , 1 , 1 , 0 , 1 , 1 , 1 , 0 , 1 , -1 , -1 , 1 , 1 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , NULL , 1 , NULL ) ;
INSERT INTO va_order_statuses (status_id,status_order,status_name,status_type,admin_order_class,user_order_class,is_user_cancel,allow_user_cancel,generate_invoice,user_invoice_activation,is_dispatch,is_list,show_for_user,paid_status,credit_note_action,stock_level_action,points_action,credit_action,commission_action,download_activation,download_notify,item_notify,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_pdf_invoice,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,merchant_notify,merchant_to,merchant_from,merchant_cc,merchant_bcc,merchant_reply_to,merchant_return_path,merchant_mail_type,merchant_subject,merchant_body,merchant_sms_notify,merchant_sms_recipient,merchant_sms_originator,merchant_sms_message,supplier_notify,supplier_to,supplier_from,supplier_cc,supplier_bcc,supplier_reply_to,supplier_return_path,supplier_mail_type,supplier_subject,supplier_body,supplier_sms_notify,supplier_sms_recipient,supplier_sms_originator,supplier_sms_message,affiliate_notify,affiliate_to,affiliate_from,affiliate_cc,affiliate_bcc,affiliate_reply_to,affiliate_return_path,affiliate_mail_type,affiliate_subject,affiliate_body,affiliate_sms_notify,affiliate_sms_recipient,affiliate_sms_originator,affiliate_sms_message,admin_notify,admin_to,admin_to_groups_ids,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_pdf_invoice,admin_subject,admin_body,admin_sms_notify,admin_sms_recipient,admin_sms_originator,admin_sms_message,final_title,final_message,is_active,view_order_groups_all,view_order_groups_ids,set_status_groups_all,set_status_groups_ids,update_order_groups_all,update_order_groups_ids) VALUES (6 , 6 , 'STATUS_PENDING_MSG' , 'PENDING' , NULL , NULL , 0 , 0 , 1 , 0 , 0 , 1 , 0 , 0 , 0 , 1 , -1 , -1 , -1 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , NULL , 1 , NULL ) ;
INSERT INTO va_order_statuses (status_id,status_order,status_name,status_type,admin_order_class,user_order_class,is_user_cancel,allow_user_cancel,generate_invoice,user_invoice_activation,is_dispatch,is_list,show_for_user,paid_status,credit_note_action,stock_level_action,points_action,credit_action,commission_action,download_activation,download_notify,item_notify,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_pdf_invoice,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,merchant_notify,merchant_to,merchant_from,merchant_cc,merchant_bcc,merchant_reply_to,merchant_return_path,merchant_mail_type,merchant_subject,merchant_body,merchant_sms_notify,merchant_sms_recipient,merchant_sms_originator,merchant_sms_message,supplier_notify,supplier_to,supplier_from,supplier_cc,supplier_bcc,supplier_reply_to,supplier_return_path,supplier_mail_type,supplier_subject,supplier_body,supplier_sms_notify,supplier_sms_recipient,supplier_sms_originator,supplier_sms_message,affiliate_notify,affiliate_to,affiliate_from,affiliate_cc,affiliate_bcc,affiliate_reply_to,affiliate_return_path,affiliate_mail_type,affiliate_subject,affiliate_body,affiliate_sms_notify,affiliate_sms_recipient,affiliate_sms_originator,affiliate_sms_message,admin_notify,admin_to,admin_to_groups_ids,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_pdf_invoice,admin_subject,admin_body,admin_sms_notify,admin_sms_recipient,admin_sms_originator,admin_sms_message,final_title,final_message,is_active,view_order_groups_all,view_order_groups_ids,set_status_groups_all,set_status_groups_ids,update_order_groups_all,update_order_groups_ids) VALUES (7 , 7 , 'STATUS_CLOSED_MSG' , 'OTHER' , NULL , NULL , 0 , 0 , 1 , 1 , 0 , 1 , 1 , 1 , 0 , 1 , -1 , -1 , -1 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , NULL , 1 , NULL ) ;
INSERT INTO va_order_statuses (status_id,status_order,status_name,status_type,admin_order_class,user_order_class,is_user_cancel,allow_user_cancel,generate_invoice,user_invoice_activation,is_dispatch,is_list,show_for_user,paid_status,credit_note_action,stock_level_action,points_action,credit_action,commission_action,download_activation,download_notify,item_notify,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_pdf_invoice,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,merchant_notify,merchant_to,merchant_from,merchant_cc,merchant_bcc,merchant_reply_to,merchant_return_path,merchant_mail_type,merchant_subject,merchant_body,merchant_sms_notify,merchant_sms_recipient,merchant_sms_originator,merchant_sms_message,supplier_notify,supplier_to,supplier_from,supplier_cc,supplier_bcc,supplier_reply_to,supplier_return_path,supplier_mail_type,supplier_subject,supplier_body,supplier_sms_notify,supplier_sms_recipient,supplier_sms_originator,supplier_sms_message,affiliate_notify,affiliate_to,affiliate_from,affiliate_cc,affiliate_bcc,affiliate_reply_to,affiliate_return_path,affiliate_mail_type,affiliate_subject,affiliate_body,affiliate_sms_notify,affiliate_sms_recipient,affiliate_sms_originator,affiliate_sms_message,admin_notify,admin_to,admin_to_groups_ids,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_pdf_invoice,admin_subject,admin_body,admin_sms_notify,admin_sms_recipient,admin_sms_originator,admin_sms_message,final_title,final_message,is_active,view_order_groups_all,view_order_groups_ids,set_status_groups_all,set_status_groups_ids,update_order_groups_all,update_order_groups_ids) VALUES (8 , 8 , 'STATUS_DECLINED_MSG' , 'DECLINED' , NULL , NULL , 0 , 0 , 1 , 0 , 0 , 1 , 0 , 0 , 0 , -1 , 1 , 1 , -1 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , NULL , 1 , NULL ) ;
INSERT INTO va_order_statuses (status_id,status_order,status_name,status_type,admin_order_class,user_order_class,is_user_cancel,allow_user_cancel,generate_invoice,user_invoice_activation,is_dispatch,is_list,show_for_user,paid_status,credit_note_action,stock_level_action,points_action,credit_action,commission_action,download_activation,download_notify,item_notify,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_pdf_invoice,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,merchant_notify,merchant_to,merchant_from,merchant_cc,merchant_bcc,merchant_reply_to,merchant_return_path,merchant_mail_type,merchant_subject,merchant_body,merchant_sms_notify,merchant_sms_recipient,merchant_sms_originator,merchant_sms_message,supplier_notify,supplier_to,supplier_from,supplier_cc,supplier_bcc,supplier_reply_to,supplier_return_path,supplier_mail_type,supplier_subject,supplier_body,supplier_sms_notify,supplier_sms_recipient,supplier_sms_originator,supplier_sms_message,affiliate_notify,affiliate_to,affiliate_from,affiliate_cc,affiliate_bcc,affiliate_reply_to,affiliate_return_path,affiliate_mail_type,affiliate_subject,affiliate_body,affiliate_sms_notify,affiliate_sms_recipient,affiliate_sms_originator,affiliate_sms_message,admin_notify,admin_to,admin_to_groups_ids,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_pdf_invoice,admin_subject,admin_body,admin_sms_notify,admin_sms_recipient,admin_sms_originator,admin_sms_message,final_title,final_message,is_active,view_order_groups_all,view_order_groups_ids,set_status_groups_all,set_status_groups_ids,update_order_groups_all,update_order_groups_ids) VALUES (9 , 9 , 'STATUS_VALIDATED_MSG' , 'VALIDATED' , NULL , NULL , 0 , 0 , 1 , 1 , 0 , 1 , 1 , 1 , 0 , 1 , -1 , -1 , 1 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , NULL , 1 , NULL ) ;
INSERT INTO va_order_statuses (status_id,status_order,status_name,status_type,admin_order_class,user_order_class,is_user_cancel,allow_user_cancel,generate_invoice,user_invoice_activation,is_dispatch,is_list,show_for_user,paid_status,credit_note_action,stock_level_action,points_action,credit_action,commission_action,download_activation,download_notify,item_notify,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_pdf_invoice,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,merchant_notify,merchant_to,merchant_from,merchant_cc,merchant_bcc,merchant_reply_to,merchant_return_path,merchant_mail_type,merchant_subject,merchant_body,merchant_sms_notify,merchant_sms_recipient,merchant_sms_originator,merchant_sms_message,supplier_notify,supplier_to,supplier_from,supplier_cc,supplier_bcc,supplier_reply_to,supplier_return_path,supplier_mail_type,supplier_subject,supplier_body,supplier_sms_notify,supplier_sms_recipient,supplier_sms_originator,supplier_sms_message,affiliate_notify,affiliate_to,affiliate_from,affiliate_cc,affiliate_bcc,affiliate_reply_to,affiliate_return_path,affiliate_mail_type,affiliate_subject,affiliate_body,affiliate_sms_notify,affiliate_sms_recipient,affiliate_sms_originator,affiliate_sms_message,admin_notify,admin_to,admin_to_groups_ids,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_pdf_invoice,admin_subject,admin_body,admin_sms_notify,admin_sms_recipient,admin_sms_originator,admin_sms_message,final_title,final_message,is_active,view_order_groups_all,view_order_groups_ids,set_status_groups_all,set_status_groups_ids,update_order_groups_all,update_order_groups_ids) VALUES (10 , 10 , 'STATUS_FAILED_MSG' , 'FAILED' , NULL , NULL , 0 , 0 , 1 , 0 , 0 , 1 , 0 , 0 , 0 , -1 , 1 , 1 , -1 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , NULL , 1 , NULL ) ;
INSERT INTO va_order_statuses (status_id,status_order,status_name,status_type,admin_order_class,user_order_class,is_user_cancel,allow_user_cancel,generate_invoice,user_invoice_activation,is_dispatch,is_list,show_for_user,paid_status,credit_note_action,stock_level_action,points_action,credit_action,commission_action,download_activation,download_notify,item_notify,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_pdf_invoice,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,merchant_notify,merchant_to,merchant_from,merchant_cc,merchant_bcc,merchant_reply_to,merchant_return_path,merchant_mail_type,merchant_subject,merchant_body,merchant_sms_notify,merchant_sms_recipient,merchant_sms_originator,merchant_sms_message,supplier_notify,supplier_to,supplier_from,supplier_cc,supplier_bcc,supplier_reply_to,supplier_return_path,supplier_mail_type,supplier_subject,supplier_body,supplier_sms_notify,supplier_sms_recipient,supplier_sms_originator,supplier_sms_message,affiliate_notify,affiliate_to,affiliate_from,affiliate_cc,affiliate_bcc,affiliate_reply_to,affiliate_return_path,affiliate_mail_type,affiliate_subject,affiliate_body,affiliate_sms_notify,affiliate_sms_recipient,affiliate_sms_originator,affiliate_sms_message,admin_notify,admin_to,admin_to_groups_ids,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_pdf_invoice,admin_subject,admin_body,admin_sms_notify,admin_sms_recipient,admin_sms_originator,admin_sms_message,final_title,final_message,is_active,view_order_groups_all,view_order_groups_ids,set_status_groups_all,set_status_groups_ids,update_order_groups_all,update_order_groups_ids) VALUES (11 , 11 , 'STATUS_DISPATCHED_MSG' , 'DISPATCHED' , NULL , NULL , 0 , 0 , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , -1 , -1 , 1 , 1 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , NULL , 1 , NULL ) ;
INSERT INTO va_order_statuses (status_id,status_order,status_name,status_type,admin_order_class,user_order_class,is_user_cancel,allow_user_cancel,generate_invoice,user_invoice_activation,is_dispatch,is_list,show_for_user,paid_status,credit_note_action,stock_level_action,points_action,credit_action,commission_action,download_activation,download_notify,item_notify,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_pdf_invoice,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,merchant_notify,merchant_to,merchant_from,merchant_cc,merchant_bcc,merchant_reply_to,merchant_return_path,merchant_mail_type,merchant_subject,merchant_body,merchant_sms_notify,merchant_sms_recipient,merchant_sms_originator,merchant_sms_message,supplier_notify,supplier_to,supplier_from,supplier_cc,supplier_bcc,supplier_reply_to,supplier_return_path,supplier_mail_type,supplier_subject,supplier_body,supplier_sms_notify,supplier_sms_recipient,supplier_sms_originator,supplier_sms_message,affiliate_notify,affiliate_to,affiliate_from,affiliate_cc,affiliate_bcc,affiliate_reply_to,affiliate_return_path,affiliate_mail_type,affiliate_subject,affiliate_body,affiliate_sms_notify,affiliate_sms_recipient,affiliate_sms_originator,affiliate_sms_message,admin_notify,admin_to,admin_to_groups_ids,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_pdf_invoice,admin_subject,admin_body,admin_sms_notify,admin_sms_recipient,admin_sms_originator,admin_sms_message,final_title,final_message,is_active,view_order_groups_all,view_order_groups_ids,set_status_groups_all,set_status_groups_ids,update_order_groups_all,update_order_groups_ids) VALUES (12 , 12 , 'STATUS_CANCELLED_MSG' , 'CANCELLED' , NULL , NULL , 1 , 0 , 1 , 0 , 0 , 1 , 1 , 0 , 0 , -1 , 1 , 1 , -1 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , NULL , 1 , NULL ) ;
INSERT INTO va_order_statuses (status_id,status_order,status_name,status_type,admin_order_class,user_order_class,is_user_cancel,allow_user_cancel,generate_invoice,user_invoice_activation,is_dispatch,is_list,show_for_user,paid_status,credit_note_action,stock_level_action,points_action,credit_action,commission_action,download_activation,download_notify,item_notify,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_pdf_invoice,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,merchant_notify,merchant_to,merchant_from,merchant_cc,merchant_bcc,merchant_reply_to,merchant_return_path,merchant_mail_type,merchant_subject,merchant_body,merchant_sms_notify,merchant_sms_recipient,merchant_sms_originator,merchant_sms_message,supplier_notify,supplier_to,supplier_from,supplier_cc,supplier_bcc,supplier_reply_to,supplier_return_path,supplier_mail_type,supplier_subject,supplier_body,supplier_sms_notify,supplier_sms_recipient,supplier_sms_originator,supplier_sms_message,affiliate_notify,affiliate_to,affiliate_from,affiliate_cc,affiliate_bcc,affiliate_reply_to,affiliate_return_path,affiliate_mail_type,affiliate_subject,affiliate_body,affiliate_sms_notify,affiliate_sms_recipient,affiliate_sms_originator,affiliate_sms_message,admin_notify,admin_to,admin_to_groups_ids,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_pdf_invoice,admin_subject,admin_body,admin_sms_notify,admin_sms_recipient,admin_sms_originator,admin_sms_message,final_title,final_message,is_active,view_order_groups_all,view_order_groups_ids,set_status_groups_all,set_status_groups_ids,update_order_groups_all,update_order_groups_ids) VALUES (13 , 13 , 'STATUS_REFUNDED_MSG' , 'REFUNDED' , NULL , NULL , 0 , 0 , 1 , 0 , 0 , 1 , 1 , 0 , 0 , -1 , 1 , 1 , -1 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , NULL , 1 , NULL ) ;
INSERT INTO va_order_statuses (status_id,status_order,status_name,status_type,admin_order_class,user_order_class,is_user_cancel,allow_user_cancel,generate_invoice,user_invoice_activation,is_dispatch,is_list,show_for_user,paid_status,credit_note_action,stock_level_action,points_action,credit_action,commission_action,download_activation,download_notify,item_notify,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_pdf_invoice,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,merchant_notify,merchant_to,merchant_from,merchant_cc,merchant_bcc,merchant_reply_to,merchant_return_path,merchant_mail_type,merchant_subject,merchant_body,merchant_sms_notify,merchant_sms_recipient,merchant_sms_originator,merchant_sms_message,supplier_notify,supplier_to,supplier_from,supplier_cc,supplier_bcc,supplier_reply_to,supplier_return_path,supplier_mail_type,supplier_subject,supplier_body,supplier_sms_notify,supplier_sms_recipient,supplier_sms_originator,supplier_sms_message,affiliate_notify,affiliate_to,affiliate_from,affiliate_cc,affiliate_bcc,affiliate_reply_to,affiliate_return_path,affiliate_mail_type,affiliate_subject,affiliate_body,affiliate_sms_notify,affiliate_sms_recipient,affiliate_sms_originator,affiliate_sms_message,admin_notify,admin_to,admin_to_groups_ids,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_pdf_invoice,admin_subject,admin_body,admin_sms_notify,admin_sms_recipient,admin_sms_originator,admin_sms_message,final_title,final_message,is_active,view_order_groups_all,view_order_groups_ids,set_status_groups_all,set_status_groups_ids,update_order_groups_all,update_order_groups_ids) VALUES (14 , 14 , 'STATUS_CAPTURED_MSG' , 'CAPTURED' , NULL , NULL , 0 , 0 , 1 , 1 , 0 , 1 , 1 , 1 , 0 , 1 , -1 , -1 , 1 , 1 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , NULL , 1 , NULL ) ;
INSERT INTO va_order_statuses (status_id,status_order,status_name,status_type,admin_order_class,user_order_class,is_user_cancel,allow_user_cancel,generate_invoice,user_invoice_activation,is_dispatch,is_list,show_for_user,paid_status,credit_note_action,stock_level_action,points_action,credit_action,commission_action,download_activation,download_notify,item_notify,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_pdf_invoice,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,merchant_notify,merchant_to,merchant_from,merchant_cc,merchant_bcc,merchant_reply_to,merchant_return_path,merchant_mail_type,merchant_subject,merchant_body,merchant_sms_notify,merchant_sms_recipient,merchant_sms_originator,merchant_sms_message,supplier_notify,supplier_to,supplier_from,supplier_cc,supplier_bcc,supplier_reply_to,supplier_return_path,supplier_mail_type,supplier_subject,supplier_body,supplier_sms_notify,supplier_sms_recipient,supplier_sms_originator,supplier_sms_message,affiliate_notify,affiliate_to,affiliate_from,affiliate_cc,affiliate_bcc,affiliate_reply_to,affiliate_return_path,affiliate_mail_type,affiliate_subject,affiliate_body,affiliate_sms_notify,affiliate_sms_recipient,affiliate_sms_originator,affiliate_sms_message,admin_notify,admin_to,admin_to_groups_ids,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_pdf_invoice,admin_subject,admin_body,admin_sms_notify,admin_sms_recipient,admin_sms_originator,admin_sms_message,final_title,final_message,is_active,view_order_groups_all,view_order_groups_ids,set_status_groups_all,set_status_groups_ids,update_order_groups_all,update_order_groups_ids) VALUES (15 , 15 , 'STATUS_VOIDED_MSG' , 'VOIDED' , NULL , NULL , 0 , 0 , 1 , 0 , 0 , 1 , 0 , 0 , 0 , -1 , 1 , 1 , -1 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , NULL , 1 , NULL ) ;
INSERT INTO va_order_statuses (status_id,status_order,status_name,status_type,admin_order_class,user_order_class,is_user_cancel,allow_user_cancel,generate_invoice,user_invoice_activation,is_dispatch,is_list,show_for_user,paid_status,credit_note_action,stock_level_action,points_action,credit_action,commission_action,download_activation,download_notify,item_notify,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_pdf_invoice,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,merchant_notify,merchant_to,merchant_from,merchant_cc,merchant_bcc,merchant_reply_to,merchant_return_path,merchant_mail_type,merchant_subject,merchant_body,merchant_sms_notify,merchant_sms_recipient,merchant_sms_originator,merchant_sms_message,supplier_notify,supplier_to,supplier_from,supplier_cc,supplier_bcc,supplier_reply_to,supplier_return_path,supplier_mail_type,supplier_subject,supplier_body,supplier_sms_notify,supplier_sms_recipient,supplier_sms_originator,supplier_sms_message,affiliate_notify,affiliate_to,affiliate_from,affiliate_cc,affiliate_bcc,affiliate_reply_to,affiliate_return_path,affiliate_mail_type,affiliate_subject,affiliate_body,affiliate_sms_notify,affiliate_sms_recipient,affiliate_sms_originator,affiliate_sms_message,admin_notify,admin_to,admin_to_groups_ids,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_pdf_invoice,admin_subject,admin_body,admin_sms_notify,admin_sms_recipient,admin_sms_originator,admin_sms_message,final_title,final_message,is_active,view_order_groups_all,view_order_groups_ids,set_status_groups_all,set_status_groups_ids,update_order_groups_all,update_order_groups_ids) VALUES (16 , 16 , 'STATUS_AUTHORIZED_MSG' , 'AUTHORIZED' , NULL , NULL , 0 , 0 , 1 , 1 , 0 , 1 , 0 , 0 , 0 , 1 , -1 , -1 , 1 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , NULL , 1 , NULL ) ;
INSERT INTO va_order_statuses (status_id,status_order,status_name,status_type,admin_order_class,user_order_class,is_user_cancel,allow_user_cancel,generate_invoice,user_invoice_activation,is_dispatch,is_list,show_for_user,paid_status,credit_note_action,stock_level_action,points_action,credit_action,commission_action,download_activation,download_notify,item_notify,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_pdf_invoice,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,merchant_notify,merchant_to,merchant_from,merchant_cc,merchant_bcc,merchant_reply_to,merchant_return_path,merchant_mail_type,merchant_subject,merchant_body,merchant_sms_notify,merchant_sms_recipient,merchant_sms_originator,merchant_sms_message,supplier_notify,supplier_to,supplier_from,supplier_cc,supplier_bcc,supplier_reply_to,supplier_return_path,supplier_mail_type,supplier_subject,supplier_body,supplier_sms_notify,supplier_sms_recipient,supplier_sms_originator,supplier_sms_message,affiliate_notify,affiliate_to,affiliate_from,affiliate_cc,affiliate_bcc,affiliate_reply_to,affiliate_return_path,affiliate_mail_type,affiliate_subject,affiliate_body,affiliate_sms_notify,affiliate_sms_recipient,affiliate_sms_originator,affiliate_sms_message,admin_notify,admin_to,admin_to_groups_ids,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_pdf_invoice,admin_subject,admin_body,admin_sms_notify,admin_sms_recipient,admin_sms_originator,admin_sms_message,final_title,final_message,is_active,view_order_groups_all,view_order_groups_ids,set_status_groups_all,set_status_groups_ids,update_order_groups_all,update_order_groups_ids) VALUES (17 , 17 , 'STATUS_INVOICE_SENT_MSG' , 'OTHER' , NULL , NULL , 0 , 0 , 1 , 1 , 0 , 1 , 0 , 0 , 0 , 1 , -1 , -1 , -1 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , NULL , 1 , NULL ) ;
INSERT INTO va_order_statuses (status_id,status_order,status_name,status_type,admin_order_class,user_order_class,is_user_cancel,allow_user_cancel,generate_invoice,user_invoice_activation,is_dispatch,is_list,show_for_user,paid_status,credit_note_action,stock_level_action,points_action,credit_action,commission_action,download_activation,download_notify,item_notify,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_pdf_invoice,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,merchant_notify,merchant_to,merchant_from,merchant_cc,merchant_bcc,merchant_reply_to,merchant_return_path,merchant_mail_type,merchant_subject,merchant_body,merchant_sms_notify,merchant_sms_recipient,merchant_sms_originator,merchant_sms_message,supplier_notify,supplier_to,supplier_from,supplier_cc,supplier_bcc,supplier_reply_to,supplier_return_path,supplier_mail_type,supplier_subject,supplier_body,supplier_sms_notify,supplier_sms_recipient,supplier_sms_originator,supplier_sms_message,affiliate_notify,affiliate_to,affiliate_from,affiliate_cc,affiliate_bcc,affiliate_reply_to,affiliate_return_path,affiliate_mail_type,affiliate_subject,affiliate_body,affiliate_sms_notify,affiliate_sms_recipient,affiliate_sms_originator,affiliate_sms_message,admin_notify,admin_to,admin_to_groups_ids,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_pdf_invoice,admin_subject,admin_body,admin_sms_notify,admin_sms_recipient,admin_sms_originator,admin_sms_message,final_title,final_message,is_active,view_order_groups_all,view_order_groups_ids,set_status_groups_all,set_status_groups_ids,update_order_groups_all,update_order_groups_ids) VALUES (18 , 1 , 'Quote' , 'QUOTE' , NULL , NULL , 0 , 0 , 0 , 0 , 0 , 1 , 1 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , NULL , 1 , NULL ) ;
INSERT INTO va_order_statuses (status_id,status_order,status_name,status_type,admin_order_class,user_order_class,is_user_cancel,allow_user_cancel,generate_invoice,user_invoice_activation,is_dispatch,is_list,show_for_user,paid_status,credit_note_action,stock_level_action,points_action,credit_action,commission_action,download_activation,download_notify,item_notify,mail_notify,mail_to,mail_from,mail_cc,mail_bcc,mail_reply_to,mail_return_path,mail_type,mail_pdf_invoice,mail_subject,mail_body,sms_notify,sms_recipient,sms_originator,sms_message,merchant_notify,merchant_to,merchant_from,merchant_cc,merchant_bcc,merchant_reply_to,merchant_return_path,merchant_mail_type,merchant_subject,merchant_body,merchant_sms_notify,merchant_sms_recipient,merchant_sms_originator,merchant_sms_message,supplier_notify,supplier_to,supplier_from,supplier_cc,supplier_bcc,supplier_reply_to,supplier_return_path,supplier_mail_type,supplier_subject,supplier_body,supplier_sms_notify,supplier_sms_recipient,supplier_sms_originator,supplier_sms_message,affiliate_notify,affiliate_to,affiliate_from,affiliate_cc,affiliate_bcc,affiliate_reply_to,affiliate_return_path,affiliate_mail_type,affiliate_subject,affiliate_body,affiliate_sms_notify,affiliate_sms_recipient,affiliate_sms_originator,affiliate_sms_message,admin_notify,admin_to,admin_to_groups_ids,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_pdf_invoice,admin_subject,admin_body,admin_sms_notify,admin_sms_recipient,admin_sms_originator,admin_sms_message,final_title,final_message,is_active,view_order_groups_all,view_order_groups_ids,set_status_groups_all,set_status_groups_ids,update_order_groups_all,update_order_groups_ids) VALUES (19 , 1 , 'Partially Paid' , 'PARTIALLY_PAID' , NULL , NULL , 0 , 0 , 1 , 1 , 0 , 1 , 1 , 0 , 0 , 1 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , NULL , 1 , NULL ) ;




























INSERT INTO va_pages (page_id,page_code,page_title,friendly_url,page_url,page_order,is_site_map,sites_all,user_types_all,is_showing,link_in_footer,is_html,page_type,page_path,page_body,meta_title,meta_keywords,meta_description) VALUES (7 , 'help' , 'How to Shop' , '' , NULL , 8 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL , '<table align="center" border="0" cellpadding="5" cellspacing="5" width="75%"><tbody><tr><td class="middle"><font size="4"><b>How to Shop at Our Online Store</b></font></td></tr><tr><td><font size="2"></font><p align="justify"> Shopping online with us isa lot like shopping at a retail store you browse our catalogue of products for the items you\'dlike, put them in a shopping basket, and complete your purchase at the check-outarea. The only difference is, we\'re open 24 hours a day - seven days a week,...and you don\'t have to put on shoes and leave the comfort of your home to shop!</p><p align="justify">So read on and find out how you can enjoy the shopping experience with Our Online Store.</p></td></tr><tr><td class="middle"><font size="3"><b>Step 1. Make Your Product Selections</b></font></td></tr><tr><td><font size="2"></font><p align="justify"> The first thing to do is browse our shop for the items you\'d like. As you browse you\'ll see \'Add/Buy\' link.Once you\'ve found the&nbsp;&nbsp;&nbsp; first product you\'d like, simply add it to your online shopping basket by clicking on the \'Add/Buy\' link.</p><p align="justify">Just like in a retail store, the shopping basket is a holding place for your items until you are ready to purchase them. You can view thecontents of your basket at any time by clicking on the \'View Basket\' link on yourshopping page. And, just like a retail store, you can always remove items from your basket and put them back on the shelves.</p></td></tr><tr><td class="middle"><font size="3"><b>Step 2.Proceed to the Check Out</b></font></td></tr><tr><td><font size="2"></font><p align="justify"> Once you\'ve decided upon your selection and checked your basket, you proceed to the Ordering Stage by clicking "Check Out". At this point you must decide whether you want to register with us or just place your order without registration. Registration means that you don\'t need to complete your details again if you order from us in the future. This action takes you to page where we need some extra details to complete the order.You enter your personal information, your name and address so we know where to send the goods.</p></td></tr><tr><td class="middle"><font size="3"><b>Step 3.Submit your Order</b></font></td></tr><tr><td><font size="2"></font><p align="justify"> At this point you must decide upon the payment method.You can pay online by entering your credit card information.Alternatively you can print out the form and fax it to us, <b>or</b> you can submit the form without credit card details and call us later with this information.</p><p align="justify"> We appreciate that ordering online the first time can be a little confusing so if you have any problems or are unsure of any aspects of the ordering process please contact us and we\'ll try to help you out.</p> </td></tr></tbody></table>' , NULL , NULL , NULL ) ;
INSERT INTO va_pages (page_id,page_code,page_title,friendly_url,page_url,page_order,is_site_map,sites_all,user_types_all,is_showing,link_in_footer,is_html,page_type,page_path,page_body,meta_title,meta_keywords,meta_description) VALUES (8 , 'about' , 'About Us' , NULL , NULL , 9 , NULL , 1 , 1 , 1 , 0 , 1 , 1 , NULL , '<b>Contact Us</b>\r\n<p>\r\nWondering when your order will arrive? Need to change something about an order you\'ve placed? You can review and revise your order so that it\'s just the way you want it by visiting Your Account.\r\n<p>\r\nIf you were unable to find your answer using our Web site\'s self-service features, click one of the links to the left to contact our customer service department. We\'ll be happy to assist you. Please note: \r\n<p>\r\nFor security reasons, we can only modify orders or send account information when the request comes from the e-mail address that is associated with your Amazon.com account. \r\nWe can only read and respond to messages in English. \r\n<p>\r\n<b>Not Receiving Replies?</b>\r\n<p>\r\nIf you have used our forms to contact us in the past but have not received a reply, your ISP or e-mail client may be blocking e-mail from Amazon.com. If you suspect this is happening, please consult the help section of your e-mail program to see how to relax or remove the settings that filter or block mail. Once you\'ve established that you should be able to receive mail from us, please contact us again with your question.\r\n<p>\r\nWant to check if your ISP or e-mail client is blocking mail from Amazon.com? Try subscribing for a Special Occasion Reminder. If you don\'t receive it on the date you specify, you will know that messages from Amazon.com are being blocked or deleted automatically.\r\n \r\n' , NULL , NULL , NULL ) ;
INSERT INTO va_pages (page_id,page_code,page_title,friendly_url,page_url,page_order,is_site_map,sites_all,user_types_all,is_showing,link_in_footer,is_html,page_type,page_path,page_body,meta_title,meta_keywords,meta_description) VALUES (19 , 'use_of_cookies' , 'Use of cookies' , 'use_of_cookies' , NULL , 16 , NULL , 1 , 1 , 1 , 0 , 1 , 1 , NULL , '<p>\r\n\tCookie is usually a small piece of data sent from a website and stored in a web browser. Cookies were designed to be a reliable mechanism for websites to remember the state of the website or activity the user had taken in the past.</p>\r\n<p>\r\n\tAny cookies that may be used by this website are used either solely on a per session basis (for example, to add product in the cart, track site visits) or to maintain user preferences (for example save login data, language preference etc.). Please read below a more detailed information about the cookies we use.</p>\r\n<table border="1" cellpadding="1" cellspacing="1">\r\n\t<thead>\r\n\t\t<tr>\r\n\t\t\t<th scope="col">\r\n\t\t\t\tCookies Name</th>\r\n\t\t\t<th scope="col">\r\n\t\t\t\tDescription</th>\r\n\t\t</tr>\r\n\t</thead>\r\n\t<tbody>\r\n\t\t<tr>\r\n\t\t\t<td>\r\n\t\t\t\tPHPSESSID</td>\r\n\t\t\t<td>\r\n\t\t\t\tThis is a cookie designed to store and identify your unique session ID on the website. It does not contain any personal information and usually looks something like 1234567890abcdef. PHPSESSID is deleted as soon as you close all browser windows. This cookie is essential to the store because it allows to login, add products to cart and make purchases. It would be impossible to implement a convenient shopping mechanism without cookies or something like them <b>that&#39;s why PHPSESSID is always turned on</b>.</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<td>\r\n\t\t\t\tcookie_lang</td>\r\n\t\t\t<td>\r\n\t\t\t\tThis cookie saves the language selection on the site so the next time you visit it will be automatically selected. The cookie expires in a year.</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<td>\r\n\t\t\t\tcookie_visit</td>\r\n\t\t\t<td>\r\n\t\t\t\tThis is a cookie that records your IP address during the first visit and the number of the visit. It is used for various analytical studies and for anti-fraud purposes.</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<td>\r\n\t\t\t\tcookie_user_login<br />\r\n\t\t\t\tcookie_user_password</td>\r\n\t\t\t<td>\r\n\t\t\t\tThese two cookies store login and password information for a year if you choose an option &quot;remember me&quot; in the login form.</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<td>\r\n\t\t\t\tcookie_af</td>\r\n\t\t\t<td>\r\n\t\t\t\tThis cookie is used to save the data about affiliate visits and calculate appropriate commissions. Usually this cookie expires in 90 days.</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<td>\r\n\t\t\t\tcookie_friend</td>\r\n\t\t\t<td>\r\n\t\t\t\tThis is a cookie used in coupons&#39; Friendly Visits functionality. It saves the data of a friendly visit and calculates possible bonuses.</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<td>\r\n\t\t\t\t__utma<br />\r\n\t\t\t\t__utmb<br />\r\n\t\t\t\t__utmc<br />\r\n\t\t\t\t__utm</td>\r\n\t\t\t<td>\r\n\t\t\t\tOur website uses Google Analytics cookies to monitor traffic sources and make different reports. These cookies enable Google to determine your IP address (which is stored anonymously on their servers), whether you are a returning visitor to the site and to track the pages that you visit during your session. See <a href="https://developers.google.com/analytics/resources/concepts/gaConceptsCookies" target="_blank">Google Analytics documentation</a> to learn more details. These cookies will be disabled if you turn off cookies.</td>\r\n\t\t</tr>\r\n\t</tbody>\r\n</table>\r\n<p>\r\n\t&nbsp;</p>' , NULL , NULL , NULL ) ;






INSERT INTO va_payment_countries (payment_id,country_id) VALUES (NULL , NULL ) ;






INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1 , 1 , 'order_id' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2 , 1 , 'vc' , 'VARIABLE' , 'vc' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (8 , 2 , 'PARTNER' , 'CONSTANT' , '!Your partner ID!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (9 , 2 , 'TYPE' , 'CONSTANT' , 'S' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (10 , 2 , 'LOGIN' , 'CONSTANT' , '!Your customer login!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (11 , 2 , 'DESCRIPTION' , 'VARIABLE' , 'description' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (12 , 2 , 'AMOUNT' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (13 , 2 , 'INVOICE' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (14 , 3 , 'x_login' , 'CONSTANT' , '!Your customer login!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (15 , 3 , 'x_tran_key' , 'CONSTANT' , '!Transaction key obtained from the merchant interface!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (16 , 3 , 'x_secret' , 'CONSTANT' , '!Type here your Autorize MD5 Hash Secret!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (17 , 3 , 'x_invoice_num' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (18 , 3 , 'x_first_name' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (19 , 3 , 'x_last_name' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (20 , 3 , 'x_address' , 'VARIABLE' , 'address' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (21 , 3 , 'x_city' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (22 , 3 , 'x_state' , 'VARIABLE' , 'state' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (23 , 3 , 'x_zip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (25 , 3 , 'x_country' , 'VARIABLE' , 'country' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (26 , 3 , 'x_phone' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (27 , 3 , 'x_fax' , 'VARIABLE' , 'fax' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (28 , 4 , 'instId' , 'CONSTANT' , '!Your installation ID!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (29 , 4 , 'cartId' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (30 , 4 , 'amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (31 , 4 , 'currency' , 'CONSTANT' , 'USD' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (32 , 4 , 'desc' , 'VARIABLE' , 'description' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (33 , 4 , 'testMode' , 'CONSTANT' , '100' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (34 , 4 , 'email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (35 , 4 , 'name' , 'VARIABLE' , 'name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (36 , 4 , 'address' , 'VARIABLE' , 'address' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (37 , 4 , 'postcode' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (38 , 4 , 'country' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (39 , 4 , 'tel' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (40 , 5 , 'sid' , 'CONSTANT' , '!Your vendor number!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (41 , 5 , 'total' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (42 , 5 , 'cart_order_id' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (43 , 5 , 'card_holder_name' , 'VARIABLE' , 'name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (44 , 5 , 'street_address' , 'VARIABLE' , 'address' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (45 , 5 , 'city' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (46 , 5 , 'state' , 'VARIABLE' , 'state' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (47 , 5 , 'zip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (48 , 5 , 'country' , 'VARIABLE' , 'country' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (49 , 5 , 'email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (50 , 5 , 'phone' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (51 , 6 , 'business' , 'CONSTANT' , 'your.paypal@example.com' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (52 , 6 , 'at' , 'CONSTANT' , '!Please specify your identity token!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (53 , 6 , 'cmd' , 'CONSTANT' , '_xclick' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (54 , 6 , 'invoice' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (55 , 6 , 'item_name' , 'VARIABLE' , 'basket' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (56 , 6 , 'amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (57 , 6 , 'email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (58 , 6 , 'first_name' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (59 , 6 , 'last_name' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (60 , 6 , 'city' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (61 , 6 , 'zip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (62 , 6 , 'state' , 'VARIABLE' , 'state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (63 , 3 , 'x_company' , 'VARIABLE' , 'company_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (64 , 3 , 'x_fp_timestamp' , 'VARIABLE' , 'timestamp' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (65 , 3 , 'x_fp_hash' , 'VARIABLE' , 'fingerprint' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (66 , 3 , 'x_fp_sequence' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (67 , 3 , 'x_relay_response' , 'CONSTANT' , 'TRUE' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (68 , 3 , 'x_show_form' , 'CONSTANT' , 'PAYMENT_FORM' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (69 , 5 , 'id_type' , 'CONSTANT' , '1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (70 , 5 , 'c_prod_{digit}' , 'VARIABLE' , '{item_id},{quantity}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (71 , 5 , 'c_name_{digit}' , 'VARIABLE' , 'item_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (72 , 5 , 'c_description_{digit}' , 'VARIABLE' , 'short_description' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (73 , 5 , 'c_price_{digit}' , 'VARIABLE' , 'price' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (74 , 5 , 'c_tangible_{digit}' , 'CONSTANT' , 'Y' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (75 , 7 , 'x_version' , 'CONSTANT' , '3.1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (76 , 7 , 'x_Invoice_Num' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (77 , 7 , 'x_relay_response' , 'CONSTANT' , 'False' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (78 , 7 , 'x_login' , 'VARIABLE' , '!Your Login ID!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (79 , 7 , 'x_tran_key' , 'VARIABLE' , '!Transaction key obtained from the Merchant Interface!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (80 , 7 , 'x_amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (81 , 7 , 'x_card_num' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (82 , 7 , 'x_exp_date' , 'VARIABLE' , '{cc_expiry_month}{cc_expiry_year}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (83 , 7 , 'x_type' , 'CONSTANT' , 'AUTH_CAPTURE' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (84 , 7 , 'x_description' , 'VARIABLE' , 'description' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (85 , 7 , 'x_first_name' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (86 , 7 , 'x_last_name' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (87 , 7 , 'x_address' , 'VARIABLE' , 'address' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (88 , 7 , 'x_city' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (89 , 7 , 'x_state' , 'VARIABLE' , 'state' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (90 , 7 , 'x_zip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (91 , 7 , 'x_country' , 'VARIABLE' , 'country' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (92 , 7 , 'x_phone' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (93 , 8 , 'MerchantID' , 'CONSTANT' , '!Your Assigned Number!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (94 , 8 , 'RegKey' , 'CONSTANT' , '!Security Key!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (95 , 8 , 'Amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (96 , 8 , 'REFID' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (97 , 8 , 'TaxAmount' , 'VARIABLE' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (98 , 8 , 'AccountNo' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (99 , 8 , 'CCMonth' , 'VARIABLE' , 'cc_expiry_month' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (100 , 8 , 'CCYear' , 'VARIABLE' , 'cc_expiry_year' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (101 , 8 , 'NameonAccount' , 'VARIABLE' , 'cc_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (102 , 8 , 'AVSADDR' , 'VARIABLE' , 'address' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (103 , 8 , 'AVSZIP' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (104 , 8 , 'CCRURL' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (105 , 8 , 'CVV2' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (106 , 7 , 'x_Ship_To_First_Name' , 'VARIABLE' , 'delivery_first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (110 , 9 , 'storename' , 'CONSTANT' , '!Fill with your store number!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (111 , 9 , 'chargetotal' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (112 , 9 , 'txnorg' , 'CONSTANT' , 'eci' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (113 , 9 , 'mode' , 'CONSTANT' , 'payplus' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (114 , 9 , 'txntype' , 'CONSTANT' , 'sale' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (115 , 9 , 'baddr1' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (116 , 9 , 'baddr2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (117 , 9 , 'bcity' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (118 , 9 , 'bcompany' , 'VARIABLE' , 'company_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (119 , 9 , 'bcountry' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (120 , 9 , 'bname' , 'VARIABLE' , 'name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (121 , 9 , 'bstate' , 'VARIABLE' , 'state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (122 , 9 , 'bstate2' , 'VARIABLE' , 'province' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (123 , 9 , 'bzip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (124 , 10 , 'configfile' , 'CONSTANT' , '!Fill with your store id!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (125 , 10 , 'keyfile' , 'CONSTANT' , '!Absolute path to your SSL certificate file!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (126 , 10 , 'ordertype' , 'CONSTANT' , 'SALE' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (127 , 10 , 'result' , 'CONSTANT' , 'LIVE' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (128 , 10 , 'oid' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (129 , 10 , 'chargetotal' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (130 , 10 , 'cardnumber' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (131 , 10 , 'cardexpmonth' , 'VARIABLE' , 'cc_expiry_month' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (132 , 10 , 'cardexpyear' , 'VARIABLE' , 'cc_expiry_year' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (134 , 10 , 'cvmvalue' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (135 , 11 , 'storename' , 'CONSTANT' , '!Fill with your store number!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (136 , 11 , 'chargetotal' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (137 , 11 , 'txnorg' , 'CONSTANT' , 'eci' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (138 , 11 , 'mode' , 'CONSTANT' , 'payplus' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (139 , 11 , 'txntype' , 'CONSTANT' , 'sale' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (140 , 11 , 'baddr1' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (141 , 11 , 'baddr2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (142 , 11 , 'bcity' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (143 , 11 , 'bcompany' , 'VARIABLE' , 'company_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (144 , 11 , 'bcountry' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (145 , 11 , 'bname' , 'VARIABLE' , 'name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (146 , 11 , 'bstate' , 'VARIABLE' , 'state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (147 , 11 , 'bstate2' , 'VARIABLE' , 'province' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (148 , 11 , 'bzip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (149 , 12 , 'configfile' , 'CONSTANT' , '!Fill with your store id!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (150 , 12 , 'keyfile' , 'CONSTANT' , '!Absolute path to your SSL certificate file!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (151 , 12 , 'ordertype' , 'CONSTANT' , 'SALE' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (152 , 12 , 'result' , 'CONSTANT' , 'LIVE' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (153 , 12 , 'oid' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (154 , 12 , 'chargetotal' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (155 , 12 , 'cardnumber' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (156 , 12 , 'cardexpmonth' , 'VARIABLE' , 'cc_expiry_month' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (157 , 12 , 'cardexpyear' , 'VARIABLE' , 'cc_expiry_year' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (159 , 12 , 'cvmvalue' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (160 , 6 , 'country' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (176 , 5 , 'secret_word' , 'CONSTANT' , '!Your secret word!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (177 , 14 , 'mid' , 'CONSTANT' , '!Enter the username you use to log into the Paymate website!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (178 , 14 , 'amt' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (179 , 14 , 'amt_editable' , 'CONSTANT' , 'N' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (180 , 14 , 'currency' , 'CONSTANT' , 'AUD' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (181 , 14 , 'ref' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (182 , 14 , 'pmt_sender_email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (183 , 14 , 'pmt_contact_firstname' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (184 , 14 , 'pmt_contact_surname' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (185 , 14 , 'pmt_contact_phone' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (186 , 14 , 'pmt_country' , 'VARIABLE' , '2country' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (187 , 14 , 'regindi_address1' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (188 , 14 , 'regindi_address2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (189 , 14 , 'regindi_sub' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (190 , 14 , 'regindi_pcode' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (191 , 15 , 'VPSProtocol' , 'CONSTANT' , '3.00' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (192 , 15 , 'TxType' , 'CONSTANT' , 'PAYMENT' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (193 , 15 , 'Vendor' , 'CONSTANT' , '!Type your vendor name here!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (194 , 15 , 'EncryptionPassword' , 'CONSTANT' , '!Type your encryption password here!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (195 , 15 , 'VendorTxCode' , 'VARIABLE' , 'order_id' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (196 , 15 , 'Amount' , 'VARIABLE' , 'order_total' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (197 , 15 , 'Currency' , 'VARIABLE' , 'currency_code' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (198 , 15 , 'Description' , 'VARIABLE' , '{total_quantity} products; Goods Total: {goods_total}; Order Total: {order_total}' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (199 , 15 , 'SuccessURL' , 'VARIABLE' , '{site_url}order_final.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (200 , 15 , 'FailureURL' , 'VARIABLE' , '{site_url}order_final.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (201 , 15 , 'CustomerName' , 'VARIABLE' , 'name' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (202 , 15 , 'CustomerEmail' , 'VARIABLE' , 'email' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (203 , 15 , 'eMailMessage' , 'CONSTANT' , NULL , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (204 , 15 , 'BillingAddress1' , 'VARIABLE' , '{address1} {address2} {city} {province} {state}  {country}' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (205 , 15 , 'BillingPostCode' , 'VARIABLE' , 'zip' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (206 , 15 , 'DeliveryAddress1' , 'VARIABLE' , '{delivery_address1} {delivery_address2} {delivery_city} {delivery_province} {delivery_state}  {delivery_country}' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (207 , 15 , 'DeliveryPostCode' , 'VARIABLE' , 'delivery_zip' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (208 , 15 , 'BillingPhone' , 'VARIABLE' , 'daytime_phone' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (209 , 15 , 'Basket' , 'VARIABLE' , '1:Total:{total_quantity}:---:---:---:{order_total}' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (211 , 16 , 'VPSProtocol' , 'CONSTANT' , '2.23' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (212 , 16 , 'TxType' , 'CONSTANT' , 'PAYMENT' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (213 , 16 , 'Vendor' , 'CONSTANT' , '!Type your vendor name here!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (214 , 16 , 'VendorTxCode' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (215 , 16 , 'Amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (216 , 16 , 'Currency' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (217 , 16 , 'Description' , 'VARIABLE' , '{total_quantity} products; Goods Total: {goods_total}; Order Total: {order_total}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (218 , 16 , 'CardHolder' , 'VARIABLE' , 'cc_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (219 , 16 , 'CardNumber' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (220 , 16 , 'ExpiryDate' , 'VARIABLE' , '{cc_expiry_month}{cc_expiry_year}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (221 , 16 , 'CardType' , 'VARIABLE' , 'cc_type' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (222 , 16 , 'CV2' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (223 , 17 , 'PAYEE_ACCOUNT' , 'CONSTANT' , '!Enter your account!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (224 , 17 , 'PAYEE_NAME' , 'CONSTANT' , '!Enter your name!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (225 , 17 , 'PAYEE_PASSPHRASE' , 'CONSTANT' , '!Enter your alternate passphrase!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (226 , 17 , 'PAYMENT_AMOUNT' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (227 , 17 , 'PAYMENT_UNITS' , 'CONSTANT' , '1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (228 , 17 , 'PAYMENT_METAL_ID' , 'CONSTANT' , '1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (229 , 17 , 'PAYMENT_ID' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (230 , 17 , 'STATUS_URL' , 'VARIABLE' , '{site_url}payments/egold_response.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (231 , 17 , 'PAYMENT_URL' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (232 , 17 , 'PAYMENT_URL_METHOD' , 'CONSTANT' , 'POST' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (233 , 17 , 'NOPAYMENT_URL' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (234 , 17 , 'NOPAYMENT_URL_METHOD' , 'CONSTANT' , 'POST' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (235 , 17 , 'BAGGAGE_FIELDS' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (236 , 18 , 'requestType' , 'CONSTANT' , 'BACKEND' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (237 , 18 , 'merchant_id' , 'CONSTANT' , '!Type your merchant_id here!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (238 , 18 , 'trnOrderNumber' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (239 , 18 , 'trnAmount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (240 , 18 , 'trnCardOwner' , 'VARIABLE' , 'cc_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (241 , 18 , 'trnCardNumber' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (242 , 18 , 'trnExpMonth' , 'VARIABLE' , 'cc_expiry_month' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (243 , 18 , 'trnExpYear' , 'VARIABLE' , 'cc_expiry_year' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (244 , 18 , 'ordName' , 'VARIABLE' , 'name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (245 , 18 , 'ordEmailAddress' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (246 , 18 , 'ordAddress1' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (247 , 18 , 'ordAddress2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (248 , 18 , 'ordCity' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (249 , 18 , 'ordProvince' , 'VARIABLE' , 'state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (250 , 18 , 'ordPostalCode' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (251 , 18 , 'ordCountry' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (252 , 18 , 'ordPhoneNumber' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (253 , 19 , 'merchantid' , 'CONSTANT' , '!Type your merchantid here!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (254 , 19 , 'cardnumber' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (255 , 19 , 'cardholder' , 'VARIABLE' , 'cc_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (256 , 19 , 'amount' , 'VARIABLE' , 'order_total_100' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (257 , 19 , 'expmonth' , 'VARIABLE' , 'cc_expiry_month' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (258 , 19 , 'expyear' , 'VARIABLE' , 'cc_expiry_year' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (259 , 19 , 'premonth' , 'VARIABLE' , 'cc_start_month' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (260 , 19 , 'preyear' , 'VARIABLE' , 'cc_start_year' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (261 , 19 , 'issuenumber' , 'VARIABLE' , 'cc_issue_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (262 , 19 , 'identifier' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (263 , 19 , 'cv2' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (264 , 19 , 'debug' , 'CONSTANT' , 'true' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (265 , 19 , 'test' , 'CONSTANT' , 'false' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (266 , 19 , 'avspostcode' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (267 , 20 , 'MerchantID' , 'CONSTANT' , '!Your Merchant ID!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (268 , 20 , 'SubID' , 'CONSTANT' , '0' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (269 , 20 , 'IssuerID' , 'CONSTANT' , NULL , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (270 , 20 , 'Privatekey' , 'CONSTANT' , '!Absolute path to your private key!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (271 , 20 , 'Privatecert' , 'CONSTANT' , '!Absolute path to your certificate!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (272 , 20 , 'Certificate0' , 'CONSTANT' , '!Absolute path to default public certificate!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (273 , 20 , 'PrivatekeyPass' , 'CONSTANT' , '!Your Private key Password!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (274 , 20 , 'EntranceCode' , 'CONSTANT' , '!Your Entrance Code!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (275 , 20 , 'AcquirerURL' , 'CONSTANT' , 'ssl://ideal.secure-ing.com:443/ideal/iDeal' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (276 , 20 , 'AcquirerTimeout' , 'CONSTANT' , '10' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (277 , 20 , 'AuthenticationType' , 'CONSTANT' , 'SHA1_RSA' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (278 , 20 , 'MerchantReturnURL' , 'VARIABLE' , '{site_url}order_final.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (279 , 20 , 'Currency' , 'CONSTANT' , 'EUR' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (280 , 20 , 'ExpirationPeriod' , 'CONSTANT' , 'PT120S' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (281 , 20 , 'Language' , 'CONSTANT' , 'en' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (282 , 20 , 'DESCRIPTION' , 'CONSTANT' , 'Default description' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (283 , 4 , 'callbackPW' , 'CONSTANT' , '!Your Callback password!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (285 , 6 , 'address1' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (286 , 10 , 'name' , 'VARIABLE' , 'name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (287 , 10 , 'company' , 'VARIABLE' , 'company_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (288 , 10 , 'address1' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (289 , 10 , 'address2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (290 , 10 , 'city' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (291 , 10 , 'state' , 'VARIABLE' , 'state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (292 , 10 , 'zip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (293 , 10 , 'country' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (294 , 10 , 'phone' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (295 , 10 , 'fax' , 'VARIABLE' , 'fax' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (296 , 10 , 'email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (297 , 10 , 'addrnum' , 'VARIABLE' , 'address_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (298 , 12 , 'name' , 'VARIABLE' , 'name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (299 , 12 , 'company' , 'VARIABLE' , 'company_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (300 , 12 , 'address1' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (301 , 12 , 'address2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (302 , 12 , 'city' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (303 , 12 , 'state' , 'VARIABLE' , 'state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (304 , 12 , 'zip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (305 , 12 , 'country' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (306 , 12 , 'phone' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (307 , 12 , 'fax' , 'VARIABLE' , 'fax' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (308 , 12 , 'email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (309 , 12 , 'addrnum' , 'VARIABLE' , 'address_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (310 , 16 , 'StartDate' , 'VARIABLE' , '{cc_start_month}{cc_start_year}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (311 , 16 , 'BillingAddress1' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (312 , 16 , 'BillingPostCode' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (313 , 16 , 'DeliveryAddress1' , 'VARIABLE' , 'delivery_address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (314 , 16 , 'DeliveryPostCode' , 'VARIABLE' , 'delivery_zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (315 , 16 , 'BillingFirstnames' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (316 , 16 , 'CustomerEmail' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (317 , 16 , 'BillingPhone' , 'VARIABLE' , 'daytime_phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (318 , 21 , 'Username' , 'CONSTANT' , '!Your API Account Name' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (319 , 21 , 'Password' , 'CONSTANT' , '!Your API Account Password' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (320 , 21 , 'Subject' , 'CONSTANT' , NULL , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (321 , 21 , 'SSLCert' , 'CONSTANT' , '!Absolute path to your API certificate' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (322 , 21 , 'OrderTotal' , 'VARIABLE' , 'order_total' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (323 , 21 , 'ReturnURL' , 'VARIABLE' , '{site_url}payments/paypal_details.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (324 , 21 , 'CancelURL' , 'VARIABLE' , '{site_url}order_final.php?va_status=cancel' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (325 , 22 , 'Username' , 'CONSTANT' , '!Your API Account Name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (326 , 22 , 'Password' , 'CONSTANT' , '!Your API Account Password' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (327 , 22 , 'Subject' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (328 , 22 , 'SSLCert' , 'CONSTANT' , '!Absolute path to your API certificate' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (329 , 22 , 'InvoiceID' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (330 , 22 , 'OrderTotal' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (331 , 22 , 'CreditCardType' , 'VARIABLE' , 'cc_type' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (332 , 22 , 'CreditCardNumber' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (333 , 22 , 'ExpMonth' , 'VARIABLE' , 'cc_expiry_month' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (334 , 22 , 'ExpYear' , 'VARIABLE' , 'cc_expiry_yyyy' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (335 , 22 , 'Payer' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (336 , 22 , 'CCFirstName' , 'VARIABLE' , 'cc_first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (337 , 22 , 'CCLastName' , 'VARIABLE' , 'cc_last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (338 , 22 , 'CVV2' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (339 , 22 , 'Name' , 'VARIABLE' , 'name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (340 , 22 , 'Street1' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (341 , 22 , 'Street2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (342 , 22 , 'CityName' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (343 , 22 , 'StateOrProvince' , 'VARIABLE' , 'state_code_or_province' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (344 , 22 , 'Country' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (345 , 22 , 'PostalCode' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (346 , 22 , 'ShipName' , 'VARIABLE' , 'delivery_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (347 , 22 , 'ShipStreet1' , 'VARIABLE' , 'delivery_address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (348 , 22 , 'ShipStreet2' , 'VARIABLE' , 'delivery_address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (349 , 22 , 'ShipCityName' , 'VARIABLE' , 'delivery_city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (350 , 22 , 'ShipStateOrProvince' , 'VARIABLE' , 'delivery_state_code_or_province' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (351 , 22 , 'ShipCountry' , 'VARIABLE' , 'delivery_country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (352 , 22 , 'ShipPostalCode' , 'VARIABLE' , 'delivery_zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (353 , 21 , 'ShipName' , 'VARIABLE' , 'delivery_name' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (354 , 21 , 'ShipStreet1' , 'VARIABLE' , 'delivery_address1' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (355 , 21 , 'ShipStreet2' , 'VARIABLE' , 'delivery_address2' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (356 , 21 , 'ShipCityName' , 'VARIABLE' , 'delivery_city' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (357 , 21 , 'ShipStateOrProvince' , 'VARIABLE' , 'delivery_state_code_or_province' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (358 , 21 , 'ShipCountry' , 'VARIABLE' , 'delivery_country_code' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (359 , 21 , 'ShipPostalCode' , 'VARIABLE' , 'delivery_zip' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (360 , 23 , 'req_username' , 'CONSTANT' , '!Your username' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (361 , 23 , 'req_password' , 'CONSTANT' , '!Your password' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (362 , 23 , 'request_action' , 'CONSTANT' , 'CCAUTHCAP' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (363 , 23 , 'request_api_version' , 'CONSTANT' , '3.4' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (364 , 23 , 'request_response_format' , 'CONSTANT' , 'XML' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (365 , 23 , 'site_id' , 'CONSTANT' , '!Your site ID' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (366 , 23 , 'merch_acct_id' , 'VARIABLE' , '!Your merchant account UD' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (367 , 23 , 'li_prod_id_1' , 'CONSTANT' , '!Dynamic Amount Product ID' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (368 , 23 , 'li_count_1' , 'CONSTANT' , '1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (369 , 23 , 'li_value_1' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (370 , 23 , 'cust_email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (371 , 23 , 'cust_fname' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (372 , 23 , 'cust_lname' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (373 , 23 , 'cust_phone' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (374 , 23 , 'bill_addr' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (375 , 23 , 'bill_addr_city' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (376 , 23 , 'bill_addr_country' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (377 , 3 , 'x_description' , 'VARIABLE' , 'description' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (378 , 3 , 'x_version' , 'CONSTANT' , '3.1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (379 , 3 , 'x_ship_to_first_name' , 'VARIABLE' , 'delivery_first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (380 , 3 , 'x_ship_to_last_name' , 'VARIABLE' , 'delivery_last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (381 , 3 , 'x_ship_to_company' , 'VARIABLE' , 'delivery_company_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (382 , 3 , 'x_ship_to_address' , 'VARIABLE' , 'delivery_address' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (383 , 3 , 'x_ship_to_city' , 'VARIABLE' , 'delivery_city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (384 , 3 , 'x_ship_to_state' , 'VARIABLE' , 'delivery_state' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (385 , 3 , 'x_ship_to_zip' , 'VARIABLE' , 'delivery_zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (386 , 3 , 'x_ship_to_country' , 'VARIABLE' , 'delivery_country' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (387 , 3 , 'x_amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (388 , 3 , 'x_currency_code' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (389 , 3 , 'x_method' , 'CONSTANT' , 'CC' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (390 , 3 , 'x_type' , 'CONSTANT' , 'AUTH_CAPTURE' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (391 , 6 , 'address2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (392 , 6 , 'bn' , 'CONSTANT' , 'ViArt_ShoppingCart_ST' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (393 , 24 , 'clientid' , 'CONSTANT' , '!Specify your client id!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (394 , 24 , 'password' , 'CONSTANT' , '!Specify your pass-phrase!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (395 , 24 , 'oid' , 'VARIABLE' , 'order_id' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (396 , 24 , 'chargetype' , 'CONSTANT' , 'Auth' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (397 , 24 , 'currencycode' , 'VARIABLE' , 'currency_value' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (398 , 24 , 'total' , 'VARIABLE' , 'order_total' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (399 , 24 , 'epdqdata' , 'VARIABLE' , 'epdqdata' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (400 , 24 , 'returnurl' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (401 , 24 , 'merchantdisplayname' , 'CONSTANT' , '!Specify you company name!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (402 , 24 , 'baddr1' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (403 , 24 , 'baddr2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (404 , 24 , 'bcity' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (405 , 24 , 'bcountry' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (406 , 24 , 'bpostalcode' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (407 , 24 , 'bstate' , 'VARIABLE' , 'state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (408 , 24 , 'btelephonenumber' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (409 , 24 , 'email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (410 , 24 , 'bcountyprovince' , 'VARIABLE' , 'province' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (411 , 24 , 'saddr1' , 'VARIABLE' , 'delivery_address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (412 , 24 , 'saddr2' , 'VARIABLE' , 'delivery_address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (413 , 24 , 'scity' , 'VARIABLE' , 'delivery_city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (414 , 24 , 'scountyprovince' , 'VARIABLE' , 'delivery_province' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (415 , 24 , 'scountry' , 'VARIABLE' , 'delivery_country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (416 , 24 , 'spostalcode' , 'VARIABLE' , 'delivery_zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (417 , 24 , 'sstate' , 'VARIABLE' , 'delivery_state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (418 , 24 , 'stelephonenumber' , 'VARIABLE' , 'delivery_phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (419 , 21 , 'ButtonSource' , 'CONSTANT' , 'ViArt_ShoppingCart_EC' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (420 , 22 , 'StartMonth' , 'VARIABLE' , 'cc_start_month' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (421 , 25 , 'ClientId' , 'CONSTANT' , '!Enter your ClientId here!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (422 , 25 , 'Name' , 'CONSTANT' , '!Enter your user name here!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (423 , 25 , 'Password' , 'CONSTANT' , '!Enter your password here!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (424 , 25 , 'Email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (425 , 25 , 'Mode' , 'CONSTANT' , 'P' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (426 , 25 , 'OrderId' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (427 , 25 , 'GroupId' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (428 , 25 , 'TransId' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (429 , 25 , 'UserId' , 'VARIABLE' , 'user_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (430 , 25 , 'Type' , 'CONSTANT' , 'Auth' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (431 , 25 , 'Number' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (432 , 25 , 'Expires' , 'VARIABLE' , 'cc_expiry_date' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (433 , 25 , 'Cvv2Val' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (434 , 25 , 'Total' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (435 , 25 , 'Currency' , 'CONSTANT' , '792' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (436 , 25 , 'Taksit' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (437 , 25 , 'BillToName' , 'VARIABLE' , 'name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (438 , 25 , 'BillToStreet1' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (439 , 25 , 'BillToStreet2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (440 , 25 , 'BillToStreet3' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (441 , 25 , 'BillToCity' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (442 , 25 , 'BillToStateProv' , 'VARIABLE' , 'province' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (443 , 25 , 'BillToPostalCode' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (444 , 25 , 'BillToCountry' , 'VARIABLE' , 'country' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (445 , 25 , 'BillToCompany' , 'VARIABLE' , 'company_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (446 , 25 , 'BillToTelVoice' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (447 , 25 , 'ShipToName' , 'VARIABLE' , 'delivery_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (448 , 25 , 'ShipToStreet1' , 'VARIABLE' , 'delivery_address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (449 , 25 , 'ShipToStreet2' , 'VARIABLE' , 'delivery_address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (450 , 25 , 'ShipToStreet3' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (451 , 25 , 'ShipToCity' , 'VARIABLE' , 'delivery_city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (452 , 25 , 'ShipToStateProv' , 'VARIABLE' , 'delivery_province' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (453 , 25 , 'ShipToPostalCode' , 'VARIABLE' , 'delivery_zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (454 , 25 , 'ShipToCountry' , 'VARIABLE' , 'delivery_country' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (455 , 26 , 'username' , 'CONSTANT' , '!Enter your User Name here!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (456 , 26 , 'password' , 'CONSTANT' , '!Enter your Password here!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (457 , 26 , 'site' , 'CONSTANT' , '!Enter your Site ID here!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (458 , 26 , 'package' , 'CONSTANT' , '!Enter your Package ID here!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (460 , 26 , 'reference' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (461 , 26 , 'price' , 'VARIABLE' , 'order_total_100' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (462 , 26 , 'description' , 'VARIABLE' , 'basket' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (463 , 26 , 'payment_url' , 'CONSTANT' , 'http://secure.vxsbill.com/ezbill.php3' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (464 , 26 , 'check_url' , 'CONSTANT' , 'http://secure.vxsbill.com/check_order.php3' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (465 , 26 , 'create_order_url' , 'CONSTANT' , 'http://secure.vxsbill.com/create_order.php3' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (473 , 27 , 'UserName' , 'CONSTANT' , '!Specify your username!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (474 , 27 , 'Password' , 'CONSTANT' , '!Specify your password!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (475 , 27 , 'ClientId' , 'CONSTANT' , '!Specify your client id!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (476 , 27 , 'Mode' , 'CONSTANT' , 'P' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (477 , 27 , 'Type' , 'CONSTANT' , 'Auth' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (478 , 27 , 'PoNumber' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (479 , 27 , 'Currency' , 'VARIABLE' , 'currency_value' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (480 , 27 , 'OrderTotal' , 'VARIABLE' , 'order_total_100' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (481 , 27 , 'CCNumber' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (482 , 27 , 'CCExpires' , 'VARIABLE' , '{cc_expiry_month}/{cc_expiry_year}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (483 , 27 , 'Cvv2Val' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (484 , 27 , 'CCStartDate' , 'VARIABLE' , '{cc_start_month}/{cc_start_year}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (485 , 27 , 'CCType' , 'VARIABLE' , 'cc_type' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (486 , 27 , 'CCIssueNum' , 'VARIABLE' , 'cc_issue_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (487 , 27 , 'BillEmail' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (488 , 27 , 'BillName' , 'VARIABLE' , 'name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (489 , 27 , 'BillStreet1' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (490 , 27 , 'BillStreet2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (491 , 27 , 'BillCity' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (492 , 27 , 'BillStateProv' , 'VARIABLE' , 'state_code_or_province' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (493 , 27 , 'BillPostalCode' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (494 , 27 , 'BillCountry' , 'VARIABLE' , 'country_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (495 , 27 , 'BillTelVoice' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (496 , 27 , 'BillTelFax' , 'VARIABLE' , 'fax' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (497 , 27 , 'ShipName' , 'VARIABLE' , 'delivery_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (498 , 27 , 'ShipStreet1' , 'VARIABLE' , 'delivery_address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (499 , 27 , 'ShipStreet2' , 'VARIABLE' , 'delivery_address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (500 , 27 , 'ShipCity' , 'VARIABLE' , 'delivery_city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (501 , 27 , 'ShipStateProv' , 'VARIABLE' , 'delivery_state_code_or_province' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (502 , 27 , 'ShipCountry' , 'VARIABLE' , 'delivery_country_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (503 , 27 , 'ShipTelVoice' , 'VARIABLE' , 'delivery_phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (504 , 27 , 'ShipTelFax' , 'VARIABLE' , 'delivery_fax' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (505 , 27 , 'ShipEmail' , 'VARIABLE' , 'delivery_email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (506 , 28 , 'merchantID' , 'CONSTANT' , '!Enter your merchantID here!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (507 , 28 , 'APACScommand' , 'CONSTANT' , 'NewPayment' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (508 , 28 , 'amount' , 'VARIABLE' , 'order_total_100' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (509 , 28 , 'merchantRef' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (510 , 28 , 'merchantDesc' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (511 , 28 , 'currency' , 'CONSTANT' , '0978' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (512 , 28 , 'lang' , 'CONSTANT' , 'EN' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (513 , 28 , 'CustomerEmail' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (514 , 28 , 'Offset' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (515 , 28 , 'Period' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (516 , 28 , 'var1' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (517 , 28 , 'var2' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (518 , 28 , 'var3' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (519 , 28 , 'var4' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (520 , 28 , 'var5' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (521 , 28 , 'var6' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (522 , 28 , 'var7' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (523 , 28 , 'var8' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (524 , 28 , 'var9' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (525 , 29 , 'username' , 'CONSTANT' , '!Enter your username!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (526 , 29 , 'password' , 'CONSTANT' , '!Enter your password!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (527 , 29 , 'mid' , 'CONSTANT' , '!Enter your mid!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (528 , 29 , 'tid' , 'CONSTANT' , '!Enter your tid!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (529 , 29 , 'currencyCode' , 'CONSTANT' , 'YT' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (530 , 29 , 'installment' , 'CONSTANT' , '03' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (531 , 29 , 'extraPoint' , 'CONSTANT' , '000000' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (532 , 29 , 'multiplePoint' , 'CONSTANT' , '00' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (533 , 29 , 'amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (534 , 29 , 'orderID' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (535 , 29 , 'ccno' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (536 , 29 , 'cardexpyear' , 'VARIABLE' , 'cc_expiry_year' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (537 , 29 , 'cardexpmonth' , 'VARIABLE' , 'cc_expiry_month' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (538 , 29 , 'cvc' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (539 , 30 , 'ClientId' , 'CONSTANT' , '!Enter your ClientId here!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (540 , 30 , 'Name' , 'CONSTANT' , '!Enter your user name here!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (541 , 30 , 'Password' , 'CONSTANT' , '!Enter you password here!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (542 , 30 , 'Mode' , 'CONSTANT' , 'P' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (543 , 30 , 'GroupId' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (544 , 30 , 'TransId' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (545 , 30 , 'Type' , 'CONSTANT' , 'Auth' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (546 , 30 , 'Currency' , 'CONSTANT' , '949' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (547 , 30 , 'Taskit' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (548 , 30 , 'IPAddress' , 'VARIABLE' , 'user_ip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (549 , 30 , 'Email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (550 , 30 , 'OrderId' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (551 , 30 , 'UserId' , 'VARIABLE' , 'user_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (552 , 30 , 'Number' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (553 , 30 , 'Expires' , 'VARIABLE' , 'cc_expiry_date' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (554 , 30 , 'Cvv2Val' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (555 , 30 , 'Total' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (556 , 30 , 'BillToName' , 'VARIABLE' , 'name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (557 , 30 , 'BillToStreet1' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (558 , 30 , 'BillToStreet2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (559 , 30 , 'BillToStreet3' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (560 , 30 , 'BillToCity' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (561 , 30 , 'BillToStateProv' , 'VARIABLE' , 'province' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (562 , 30 , 'BillToPostalCode' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (563 , 30 , 'BillToCountry' , 'VARIABLE' , 'country' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (564 , 30 , 'BillToCompany' , 'VARIABLE' , 'company_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (565 , 30 , 'BillToTelVoice' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (566 , 30 , 'ShipToName' , 'VARIABLE' , 'delivery_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (567 , 30 , 'ShipToStreet1' , 'VARIABLE' , 'delivery_address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (568 , 30 , 'ShipToStreet2' , 'VARIABLE' , 'delivery_address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (569 , 30 , 'ShipToStreet3' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (570 , 30 , 'ShipToCity' , 'VARIABLE' , 'delivery_city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (571 , 30 , 'ShipToStateProv' , 'VARIABLE' , 'delivery_province' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (572 , 30 , 'ShipToPostalCode' , 'VARIABLE' , 'delivery_zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (573 , 30 , 'ShipToCountry' , 'VARIABLE' , 'delivery_country' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (574 , 31 , 'account_id' , 'CONSTANT' , '!Enter your Account ID here!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (575 , 31 , 'tran_type' , 'CONSTANT' , 'S' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (576 , 31 , 'pay_type' , 'CONSTANT' , 'C' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (577 , 31 , 'amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (578 , 31 , 'card_number' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (579 , 31 , 'cardexpyear' , 'VARIABLE' , 'cc_expiry_year' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (580 , 31 , 'cardexpmonth' , 'VARIABLE' , 'cc_expiry_month' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (581 , 32 , 'product_id' , 'CONSTANT' , '!Enter your product_id!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (582 , 32 , 'product_name' , 'CONSTANT' , '!Enter your product_name!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (583 , 32 , 'language' , 'CONSTANT' , 'En' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (584 , 32 , 'cb_url' , 'CONSTANT' , '{site_url}payments/chronopay_confirm.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (585 , 32 , 'cb_type' , 'CONSTANT' , 'P' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (586 , 32 , 'product_price' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (587 , 32 , 'street' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (588 , 32 , 'city' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (589 , 32 , 'state' , 'VARIABLE' , 'state' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (590 , 32 , 'zip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (591 , 32 , 'country' , 'VARIABLE' , 'country' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (592 , 32 , 'phone' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (593 , 32 , 'email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (594 , 32 , 'f_name' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (595 , 32 , 's_name' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (596 , 32 , 'cs1' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (597 , 32 , 'cs2' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (598 , 32 , 'cs3' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (599 , 3 , 'x_email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (600 , 33 , 'mpSeller_ID' , 'CONSTANT' , '!Enter your Seller ID!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (601 , 33 , 'mpType' , 'CONSTANT' , '1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (602 , 33 , 'mpAdministration_ID' , 'CONSTANT' , '1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (603 , 33 , 'mpAllow' , 'CONSTANT' , 'TEVI' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (604 , 33 , 'mpCountry' , 'CONSTANT' , 'NL' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (605 , 33 , 'mpCurrency' , 'CONSTANT' , 'NLG' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (606 , 33 , 'mpDescription' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (607 , 33 , 'mpItems' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (608 , 33 , 'mpVar1' , 'VARIABLE' , 'response' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (609 , 33 , 'mpVar2' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (610 , 33 , 'mpAmount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (611 , 33 , 'mpOrder_ID' , 'VARIABLE' , 'test_{order_id}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (612 , 33 , 'mpSuccessURL' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (613 , 33 , 'mpnaw_first' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (614 , 33 , 'mpnaw_last' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (615 , 33 , 'mpnaw_email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (616 , 33 , 'mpnaw_telno' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (617 , 33 , 'mpnaw_country' , 'VARIABLE' , 'country' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (618 , 33 , 'mpnaw_city' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (619 , 33 , 'mpnaw_street' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (620 , 33 , 'mpnaw_zipcode' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (621 , 33 , 'mpnaw_number' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (622 , 3 , 'x_test_request' , 'CONSTANT' , 'FALSE' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (623 , 3 , 'x_relay_url' , 'VARIABLE' , '{site_url}payments/authorize_confirm.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (624 , 34 , 'merchant_id' , 'CONSTANT' , '!Enter here your merchant_id!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (625 , 34 , 'merchant_key' , 'CONSTANT' , '!Enter here your merchant_key!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (626 , 34 , 'action_url' , 'CONSTANT' , 'https://sandbox.google.com/checkout/cws/v2/Merchant/[[Your-Merchant-ID]]/checkout' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (627 , 34 , 'currency' , 'CONSTANT' , 'USD' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (628 , 34 , 'item-name' , 'VARIABLE' , NULL , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (629 , 35 , 'Clientid' , 'CONSTANT' , '!Enter here your Clientid!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (630 , 35 , 'pan' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (631 , 35 , 'cv2' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (632 , 35 , 'Ecom_Payment_Card_ExpDate_Year' , 'VARIABLE' , 'cc_expiry_year' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (633 , 35 , 'Ecom_Payment_Card_ExpDate_Month' , 'VARIABLE' , 'cc_expiry_month' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (634 , 35 , 'Amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (635 , 35 , 'Description' , 'VARIABLE' , 'basket' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (636 , 35 , 'OkUrl' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (637 , 35 , 'FailUrl' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (639 , 6 , 'currency_code' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (640 , 6 , 'charset' , 'VARIABLE' , 'charset' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (641 , 21 , 'sandbox' , 'CONSTANT' , '0' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (642 , 22 , 'StartYear' , 'VARIABLE' , 'cc_start_yyyy' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (643 , 22 , 'IssueNumber' , 'VARIABLE' , 'cc_issue_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (644 , 22 , 'ButtonSource' , 'CONSTANT' , 'ViArt_ShoppingCart_DP' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (645 , 22 , 'IPAddress' , 'VARIABLE' , 'order_ip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (646 , 22 , 'sandbox' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (647 , 7 , 'x_Ship_To_Last_Name' , 'VARIABLE' , 'delivery_last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (648 , 7 , 'x_Ship_To_Company' , 'VARIABLE' , 'delivery_company_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (649 , 7 , 'x_Ship_To_Address' , 'VARIABLE' , 'delivery_address' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (650 , 7 , 'x_Ship_To_City' , 'VARIABLE' , 'delivery_city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (651 , 7 , 'x_Ship_To_State' , 'VARIABLE' , 'delivery_state' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (652 , 7 , 'x_Ship_To_Zip' , 'VARIABLE' , 'delivery_zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (653 , 7 , 'x_Ship_To_Country' , 'VARIABLE' , 'delivery_country' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (654 , 7 , 'x_Email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (655 , 7 , 'x_Delim_Data' , 'CONSTANT' , 'TRUE' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (656 , 7 , 'x_Delim_Char' , 'CONSTANT' , '|' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (657 , 7 , 'x_Cust_ID' , 'VARIABLE' , 'user_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (658 , 7 , 'x_Customer_IP' , 'VARIABLE' , 'user_ip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (659 , 7 , 'x_Currency_Code' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (660 , 3 , 'x_Cust_ID' , 'VARIABLE' , 'user_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (661 , 3 , 'x_Customer_IP' , 'VARIABLE' , 'user_ip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (662 , 14 , 'return' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (663 , 14 , 'popup' , 'CONSTANT' , 'false' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (664 , 36 , 'TxType' , 'CONSTANT' , 'PAYMENT' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (665 , 36 , 'Vendor' , 'CONSTANT' , '!Enter your Vendor login name here!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (666 , 36 , 'VPSProtocol' , 'CONSTANT' , '2.23' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (667 , 36 , 'Currency' , 'CONSTANT' , 'GBP' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (668 , 36 , 'Description' , 'CONSTANT' , 'Description' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (669 , 36 , 'VendorTxCode' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (670 , 36 , 'Amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (671 , 36 , 'NotificationURL' , 'VARIABLE' , '{site_url}payments/protx_server_checkout.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (672 , 36 , 'Basket' , 'VARIABLE' , '1:Total:{total_quantity}:---:---:---:{order_total}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (673 , 36 , 'BillingAddress1' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (674 , 36 , 'BillingPostCode' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (675 , 36 , 'DeliveryAddress1' , 'VARIABLE' , 'delivery_address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (676 , 36 , 'DeliveryPostCode' , 'VARIABLE' , 'delivery_zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (677 , 36 , 'BillingFirstnames' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (678 , 36 , 'BillingPhone' , 'VARIABLE' , 'daytime_phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (679 , 36 , 'BillingSurname' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (680 , 36 , 'CustomerEMail' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (681 , 36 , 'AllowGiftAid' , 'VARIABLE' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (682 , 36 , 'ApplyAVSCV2' , 'VARIABLE' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (683 , 36 , 'Apply3DSecure' , 'VARIABLE' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (684 , 36 , 'RedirectURL' , 'VARIABLE' , '{site_url}order_final.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (685 , 37 , 'p1' , 'CONSTANT' , '!Enter your VCS Terminal ID here!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (686 , 37 , 'p2' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (687 , 37 , 'p3' , 'VARIABLE' , 'basket' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (688 , 37 , 'p4' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (689 , 37 , 'p5' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (690 , 37 , 'p6' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (691 , 37 , 'p7' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (692 , 37 , 'p8' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (693 , 37 , 'p9' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (694 , 37 , 'p10' , 'VARIABLE' , '{site_url}order_info.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (695 , 37 , 'p11' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (696 , 37 , 'p12' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (697 , 37 , 'p13' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (698 , 37 , 'Budget' , 'CONSTANT' , 'N' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (699 , 37 , 'NextOccurDate' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (700 , 37 , 'pam' , 'CONSTANT' , '!Enter your pam here!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (701 , 38 , 'merchantID' , 'CONSTANT' , '!Enter here your merchantID here!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (702 , 38 , 'PublicKey' , 'CONSTANT' , '!Enter here your PublicKey!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (703 , 38 , 'PrivateKey' , 'CONSTANT' , '!Enter here your PrivateKey!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (704 , 38 , 'orderPage_serialNumber' , 'CONSTANT' , '!Enter here your Serial Number!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (705 , 38 , 'orderPage_version' , 'CONSTANT' , '4' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (706 , 38 , 'orderPage_transactionType' , 'CONSTANT' , 'authorization' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (707 , 38 , 'orderNumber' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (708 , 38 , 'amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (709 , 38 , 'currency' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (710 , 38 , 'taxAmount' , 'VARIABLE' , 'tax_cost' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (711 , 38 , 'billTo_firstName' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (712 , 38 , 'billTo_lastName' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (713 , 38 , 'billTo_country' , 'VARIABLE' , 'country' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (714 , 38 , 'billTo_city' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (715 , 38 , 'billTo_postalCode' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (716 , 38 , 'billTo_state' , 'VARIABLE' , 'state' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (717 , 38 , 'billTo_street1' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (718 , 38 , 'billTo_street2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (719 , 38 , 'billTo_email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (720 , 38 , 'billTo_customerID' , 'VARIABLE' , 'user_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (721 , 38 , 'billTo_company' , 'VARIABLE' , 'company_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (722 , 38 , 'billTo_phoneNumber' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (723 , 38 , 'shipTo_firstName' , 'VARIABLE' , 'delivery_first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (724 , 38 , 'shipTo_lastName' , 'VARIABLE' , 'delivery_last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (725 , 38 , 'shipTo_country' , 'VARIABLE' , 'delivery_country' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (726 , 38 , 'shipTo_city' , 'VARIABLE' , 'delivery_city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (727 , 38 , 'shipTo_postalCode' , 'VARIABLE' , 'delivery_zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (728 , 38 , 'shipTo_state' , 'VARIABLE' , 'delivery_state' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (729 , 38 , 'shipTo_street1' , 'VARIABLE' , 'delivery_address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (730 , 38 , 'shipTo_street2' , 'VARIABLE' , 'delivery_address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (731 , 38 , 'shipTo_company' , 'VARIABLE' , 'delivery_company_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (732 , 38 , 'card_accountNumber' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (733 , 38 , 'card_cardType' , 'VARIABLE' , 'cc_type' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (734 , 38 , 'card_cvNumber' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (735 , 38 , 'card_startMonth' , 'VARIABLE' , 'cc_start_month' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (736 , 38 , 'card_startYear' , 'VARIABLE' , 'cc_start_yyyy' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (737 , 38 , 'card_expirationMonth' , 'VARIABLE' , 'cc_expiry_month' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (738 , 38 , 'card_expirationYear' , 'VARIABLE' , 'cc_expiry_yyyy' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (739 , 38 , 'card_issueNumber' , 'VARIABLE' , 'cc_issue_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (740 , 39 , 'merchant' , 'CONSTANT' , '!Enter here your Merchant name!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (741 , 39 , 'terminal' , 'CONSTANT' , '!Enter here your Terminal name!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (742 , 39 , 'secretcode' , 'CONSTANT' , '!Enter here your Secret Code!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (743 , 39 , 'lang' , 'CONSTANT' , 'en' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (744 , 39 , 'refermethod' , 'CONSTANT' , 'POST' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (745 , 39 , 'refertarget' , 'CONSTANT' , '_top' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (746 , 39 , 'terms' , 'CONSTANT' , 'Y' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (747 , 39 , 'readonly' , 'CONSTANT' , 'Y' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (748 , 39 , 'description' , 'VARIABLE' , 'basket' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (749 , 39 , 'amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (750 , 39 , 'currency' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (751 , 39 , 'reference' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (752 , 39 , 'name' , 'VARIABLE' , 'name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (753 , 39 , 'address' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (754 , 39 , 'address2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (755 , 39 , 'email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (756 , 39 , 'zip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (757 , 39 , 'city' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (758 , 39 , 'country' , 'VARIABLE' , 'country' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (759 , 39 , 'phone' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (760 , 39 , 'downloadurl' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (761 , 39 , 'continueurl' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (762 , 39 , 'checkvaluemd5' , 'VARIABLE' , 'korta_checkvaluemd5' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (763 , 39 , 'email2' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (764 , 39 , 'korta_payment_url' , 'CONSTANT' , 'https://netgreidslur.korta.is/testing/' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (765 , 39 , 'is_test' , 'CONSTANT' , '1' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (766 , 39 , 'korta_submit_method' , 'CONSTANT' , 'POST' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (767 , 27 , '3DAuthentication' , 'CONSTANT' , 'false' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (768 , 27 , 'JavaClasspath' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (769 , 27 , 'VISAMerchantID' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (770 , 27 , 'VISADSPassword' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (771 , 27 , 'VISAAcquirerBIN' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (772 , 27 , 'MCMerchantID' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (773 , 27 , 'MCAcquirerBIN' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (774 , 27 , 'MerchantName' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (775 , 21 , 'InvoiceID' , 'VARIABLE' , 'order_id' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (776 , 21 , 'CurrencyID' , 'VARIABLE' , 'currency_code' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (777 , 22 , 'CurrencyID' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (778 , 32 , 'response_ip' , 'CONSTANT' , '69.20.58.35' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (779 , 40 , 'Version' , 'CONSTANT' , '1' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (780 , 40 , 'action_url' , 'CONSTANT' , 'https://secure.upc.ua/go/enter' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (781 , 40 , 'MerchantID' , 'CONSTANT' , '!Your Merchant ID!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (782 , 40 , 'TerminalID' , 'CONSTANT' , '!Your Terminal ID!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (783 , 40 , 'Privatekey' , 'CONSTANT' , '!full path to your private key!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (784 , 40 , 'PublicServerCert' , 'CONSTANT' , '!full path to your certificate!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (785 , 40 , 'TotalAmount' , 'VARIABLE' , 'order_total' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (786 , 40 , 'Currency' , 'CONSTANT' , '980' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (787 , 40 , 'locale' , 'CONSTANT' , 'en' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (788 , 40 , 'OrderID' , 'VARIABLE' , 'order_id' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (789 , 40 , 'Delay' , 'CONSTANT' , '0' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (790 , 40 , 'SD' , 'VARIABLE' , NULL , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (791 , 40 , 'PurchaseDesc' , 'VARIABLE' , NULL , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (792 , 40 , 'forwardUrl' , 'VARIABLE' , '{site_url}order_final.php?oid={order_id}' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (793 , 41 , 'action_url' , 'CONSTANT' , 'https://www.cashu.com/cgi-bin/pcashu.cgi' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (794 , 41 , 'merchant_id' , 'CONSTANT' , '!Your Merchant ID!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (795 , 41 , 'display_text' , 'VARIABLE' , 'Order No: {order_id}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (796 , 41 , 'currency' , 'CONSTANT' , 'USD' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (797 , 41 , 'amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (798 , 41 , 'language' , 'CONSTANT' , 'en' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (799 , 41 , 'email' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (800 , 41 , 'session_id' , 'VARIABLE' , 'session_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (801 , 41 , 'txt1' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (802 , 41 , 'test_mode' , 'CONSTANT' , '1' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (803 , 41 , 'servicesName' , 'CONSTANT' , NULL , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (804 , 41 , 'keyword' , 'CONSTANT' , '!Your secure keyword!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (805 , 41 , 'response_ip' , 'VARIABLE' , '75.125.36.194' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (806 , 42 , 'merchant_echo_id' , 'CONSTANT' , '!Enter here your Merchant ID!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (807 , 42 , 'merchant_pin' , 'CONSTANT' , '!Enter here your Merchant PIN!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (808 , 42 , 'counter' , 'CONSTANT' , '1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (809 , 42 , 'test_mode' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (810 , 42 , 'order_number' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (811 , 42 , 'grand_total' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (812 , 42 , 'sales_tax' , 'VARIABLE' , 'tax_cost' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (813 , 42 , 'billing_first_name' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (814 , 42 , 'billing_last_name' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (815 , 42 , 'billing_country' , 'VARIABLE' , 'country' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (816 , 42 , 'billing_city' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (817 , 42 , 'billing_zip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (818 , 42 , 'billing_state' , 'VARIABLE' , 'state' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (819 , 42 , 'billing_address1' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (820 , 42 , 'billing_address2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (821 , 42 , 'billing_email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (822 , 42 , 'billing_company_name' , 'VARIABLE' , 'company_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (823 , 42 , 'billing_phone' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (824 , 42 , 'cc_number' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (825 , 42 , 'cnp_security' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (826 , 42 , 'ccexp_month' , 'VARIABLE' , 'cc_expiry_month' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (827 , 42 , 'ccexp_year' , 'VARIABLE' , 'cc_expiry_yyyy' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (828 , 16 , 'TermUrl' , 'VARIABLE' , '{site_url}payments/protx_direct_3d_check.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (829 , 16 , 'CallbackURL' , 'CONSTANT' , 'https://live.sagepay.com/gateway/service/direct3dcallback.vsp' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (830 , 43 , 'pay_to_email' , 'CONSTANT' , '!enter your email!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (831 , 43 , 'return_url' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (832 , 43 , 'cancel_url' , 'VARIABLE' , '{site_url}order_info.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (833 , 43 , 'status_url' , 'VARIABLE' , '{site_url}payments/moneybookers_status.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (834 , 43 , 'secret_word' , 'CONSTANT' , '!Enter your secret word!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (835 , 43 , 'transaction_id' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (836 , 43 , 'pay_from_email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (837 , 43 , 'amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (838 , 43 , 'currency' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (839 , 43 , 'firstname' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (840 , 43 , 'lastname' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (841 , 43 , 'address' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (842 , 43 , 'address2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (843 , 43 , 'phone_number' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (844 , 43 , 'postal_code' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (845 , 43 , 'city' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (846 , 43 , 'state' , 'VARIABLE' , 'state' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (847 , 43 , 'country' , 'VARIABLE' , 'country_code_alpha3' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (848 , 44 , 'merchant_id' , 'CONSTANT' , '!enter your SIPS Merchant ID!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (849 , 44 , 'merchant_country' , 'CONSTANT' , '!enter your country code!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (852 , 44 , 'pathfile' , 'VARIABLE' , '!enter your pathfile!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (853 , 44 , 'path_bin' , 'VARIABLE' , '!enter your SIPS request binary path!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (854 , 44 , 'path_bin_resp' , 'VARIABLE' , '!enter your  SIPS responce binary path!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (855 , 44 , 'normal_return_url' , 'VARIABLE' , '{site_url}order_final.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (856 , 44 , 'cancel_return_url' , 'VARIABLE' , '{site_url}order_info.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (857 , 44 , 'automatic_response_url' , 'VARIABLE' , '{site_url}payments/sips_autoresponse.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (858 , 44 , 'payment_means' , 'CONSTANT' , 'BOURBON,2,CB,2,VISA,2,MASTERCARD,2' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (863 , 45 , 'Account' , 'CONSTANT' , '!Enter your Account number!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (864 , 45 , 'CCName' , 'VARIABLE' , 'cc_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (865 , 45 , 'CCNum' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (866 , 45 , 'CCType' , 'VARIABLE' , 'cc_type' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (867 , 45 , 'CVV2' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (868 , 45 , 'CCExpireMonth' , 'VARIABLE' , 'cc_expiry_month' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (869 , 45 , 'CCExpireYear' , 'VARIABLE' , 'cc_expiry_yyyy' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (870 , 45 , 'Address1' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (871 , 45 , 'Address2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (872 , 45 , 'Country' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (873 , 45 , 'City' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (874 , 45 , 'State' , 'VARIABLE' , 'state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (875 , 45 , 'Zip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (876 , 45 , 'Email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (877 , 45 , 'Phone' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (878 , 45 , 'CCPassword' , 'VARIABLE' , '{field_1}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (879 , 45 , 'Amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (880 , 45 , 'RemoteHost' , 'VARIABLE' , 'user_ip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (881 , 45 , 'AddUser' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (882 , 46 , 'Account' , 'CONSTANT' , '!Enter your Account number!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (883 , 46 , 'Currency' , 'CONSTANT' , 'USD' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (884 , 46 , 'Amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (885 , 46 , 'CCName' , 'VARIABLE' , 'cc_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (886 , 46 , 'City' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (887 , 46 , 'Address' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (888 , 46 , 'State' , 'VARIABLE' , 'state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (889 , 46 , 'Zip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (890 , 46 , 'Country' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (891 , 46 , 'Email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (892 , 46 , 'Phone' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (893 , 46 , 'RemoteHost' , 'VARIABLE' , 'user_ip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (894 , 46 , 'AccountType' , 'VARIABLE' , '{field_2}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (895 , 46 , 'Routing' , 'VARIABLE' , '{field_3}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (896 , 46 , 'BankAccount' , 'VARIABLE' , '{field_4}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (897 , 46 , 'CheckNumber' , 'VARIABLE' , '{field_5}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (898 , 46 , 'ST' , 'VARIABLE' , '{field_6}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (899 , 46 , 'StateID' , 'VARIABLE' , '{field_7}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (900 , 46 , 'DOB' , 'VARIABLE' , '{field_12}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (901 , 46 , 'SSN4' , 'VARIABLE' , '{field_8}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (902 , 46 , 'Bank' , 'VARIABLE' , '{field_9}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (903 , 46 , 'BanklCity' , 'VARIABLE' , '{field_10}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (904 , 46 , 'BankState' , 'VARIABLE' , '{field_11}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (905 , 46 , 'AddUser' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (906 , 46 , 'Rebill' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (907 , 46 , 'TrialFlag' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (908 , 46 , 'TEST' , 'CONSTANT' , 'TRUE' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (909 , 47 , 'PaymentType' , 'CONSTANT' , 'EuroDebit' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (910 , 47 , 'Account' , 'CONSTANT' , '!Enter your Account number!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (911 , 47 , 'Amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (912 , 47 , 'BankNameCode' , 'CONSTANT' , '10' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (913 , 47 , 'FirstName' , 'VARIABLE' , 'cc_first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (914 , 47 , 'LastName' , 'VARIABLE' , 'cc_last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (915 , 47 , 'Address1' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (916 , 47 , 'City' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (917 , 47 , 'State' , 'VARIABLE' , 'state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (918 , 47 , 'Zip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (919 , 47 , 'Country' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (920 , 47 , 'Email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (921 , 47 , 'Phone' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (922 , 47 , 'RemoteHost' , 'VARIABLE' , 'user_ip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (923 , 47 , 'CKABA' , 'VARIABLE' , '!Enter your CKABA!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (924 , 47 , 'CKAcct' , 'VARIABLE' , '!Enter your CKAcct!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (925 , 47 , 'UserName' , 'VARIABLE' , '{field_19}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (926 , 47 , 'UserPassword' , 'VARIABLE' , '{field_20}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (927 , 47 , 'IDField1' , 'VARIABLE' , '{field_15}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (928 , 47 , 'IDField2' , 'VARIABLE' , '{field_16}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (929 , 47 , 'IDField3' , 'VARIABLE' , '{field_17}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (930 , 47 , 'IDField4' , 'VARIABLE' , '{field_18}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (931 , 47 , 'AddUser' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (932 , 47 , 'Rebill' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (933 , 47 , 'TrialFlag' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (934 , 47 , 'UserAgent' , 'VARIABLE' , 'user_agent' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (935 , 47 , 'TEST' , 'CONSTANT' , 'TRUE' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (936 , 6 , 'sandbox' , 'CONSTANT' , '0' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (937 , 11 , 'oid' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (938 , 11 , 'phone' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (939 , 11 , 'fax' , 'VARIABLE' , 'fax' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (940 , 11 , 'email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (941 , 11 , 'sname' , 'VARIABLE' , 'delivery_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (942 , 11 , 'saddr1' , 'VARIABLE' , 'delivery_address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (943 , 11 , 'saddr2' , 'VARIABLE' , 'delivery_address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (944 , 11 , 'scity' , 'VARIABLE' , 'delivery_city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (945 , 11 , 'sstate' , 'VARIABLE' , 'delivery_state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (946 , 11 , 'szip' , 'VARIABLE' , 'delivery_zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (947 , 11 , 'scountry' , 'VARIABLE' , 'delivery_country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (948 , 11 , 'sstate2' , 'VARIABLE' , 'delivery_province' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (949 , 9 , 'oid' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (950 , 9 , 'phone' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (951 , 9 , 'fax' , 'VARIABLE' , 'fax' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (952 , 9 , 'email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (953 , 9 , 'sname' , 'VARIABLE' , 'delivery_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (954 , 9 , 'saddr1' , 'VARIABLE' , 'delivery_address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (955 , 9 , 'saddr2' , 'VARIABLE' , 'delivery_address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (956 , 9 , 'scity' , 'VARIABLE' , 'delivery_city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (957 , 9 , 'sstate' , 'VARIABLE' , 'delivery_state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (958 , 9 , 'szip' , 'VARIABLE' , 'delivery_zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (959 , 9 , 'scountry' , 'VARIABLE' , 'delivery_country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (960 , 9 , 'sstate2' , 'VARIABLE' , 'delivery_province' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (961 , 48 , 'merchant_id' , 'CONSTANT' , '!Enter here your Merchant ID!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (962 , 48 , 'amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (963 , 48 , 'order_id' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (964 , 48 , 'description' , 'VARIABLE' , 'basket' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (965 , 48 , 'billing_fullname' , 'VARIABLE' , '{first_name} {last_name}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (966 , 48 , 'billing_address' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (967 , 48 , 'billing_postcode' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (968 , 48 , 'delivery_fullname' , 'VARIABLE' , '{delivery_first_name} {delivery_last_name}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (969 , 48 , 'delivery_address' , 'VARIABLE' , 'delivery_address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (970 , 48 , 'delivery_postcode' , 'VARIABLE' , 'delivery_zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (971 , 48 , 'email_address' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (972 , 48 , 'customer_phone_number' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (973 , 48 , 'test_transaction' , 'CONSTANT' , '100' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (974 , 48 , 'test_success_url' , 'VARIABLE' , '{site_url}order_final.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (975 , 48 , 'failed_url' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (976 , 48 , 'success_url' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (977 , 48 , 'cancel_url' , 'VARIABLE' , '{site_url}order_final.php?return_action=cancel' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (978 , 48 , 'callback_url' , 'VARIABLE' , '{site_url}payments/nochex_response.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (979 , 48 , 'hide_billing_details' , 'CONSTANT' , 'true' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (980 , 49 , 'PSPID' , 'CONSTANT' , '!Enter here your PSPID!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (981 , 49 , 'REFERER' , 'VARIABLE' , '{site_url}order_info.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (982 , 49 , 'PM' , 'CONSTANT' , 'iDEAL' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (983 , 49 , 'name' , 'VARIABLE' , 'name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (984 , 49 , 'email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (985 , 49 , 'owneraddress' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (986 , 49 , 'ownertown' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (987 , 49 , 'ownerzip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (988 , 49 , 'ownercty' , 'VARIABLE' , 'country' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (989 , 49 , 'currency' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (990 , 49 , 'order_total' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (991 , 49 , 'description' , 'VARIABLE' , 'description' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (992 , 50 , 'USER' , 'CONSTANT' , '!Your Account User!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (993 , 50 , 'VENDOR' , 'CONSTANT' , '!Your Account Vendor!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (994 , 50 , 'PARTNER' , 'CONSTANT' , '!Your Account Partner!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (995 , 50 , 'PWD' , 'CONSTANT' , '!Your Account Password!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (996 , 50 , 'TENDER' , 'CONSTANT' , 'C' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (997 , 50 , 'TRXTYPE' , 'CONSTANT' , 'A' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (998 , 50 , 'ACCT' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (999 , 50 , 'CVV2' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1000 , 50 , 'EXPDATE' , 'VARIABLE' , '{cc_expiry_month}{cc_expiry_year}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1001 , 50 , 'ACCTTYPE' , 'VARIABLE' , 'cc_type' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1002 , 50 , 'AMT' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1003 , 50 , 'CURRENCY' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1004 , 50 , 'FIRSTNAME' , 'VARIABLE' , 'cc_first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1005 , 50 , 'LASTNAME' , 'VARIABLE' , 'cc_last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1006 , 50 , 'STREET' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1007 , 50 , 'CITY' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1008 , 50 , 'STATE' , 'VARIABLE' , 'state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1009 , 50 , 'ZIP' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1010 , 50 , 'COUNTRY' , 'VARIABLE' , 'country' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1011 , 50 , 'EMAIL' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1012 , 50 , 'CUSTIP' , 'VARIABLE' , 'user_ip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1013 , 50 , 'COMMENT1' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1014 , 50 , 'INVNUM' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1015 , 50 , 'ORDERDESC' , 'VARIABLE' , 'basket' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1016 , 50 , 'VERBOSITY' , 'CONSTANT' , 'MEDIUM' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1017 , 51 , 'USER' , 'CONSTANT' , '!Your Account User!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1018 , 51 , 'VENDOR' , 'CONSTANT' , '!Your Account Vendor!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1019 , 51 , 'PARTNER' , 'CONSTANT' , '!Your Account Partner!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1020 , 51 , 'PWD' , 'CONSTANT' , '!Your Account Password!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1021 , 51 , 'TENDER' , 'CONSTANT' , 'P' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1022 , 51 , 'TRXTYPE' , 'CONSTANT' , 'S' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1023 , 51 , 'ACTION' , 'CONSTANT' , 'S' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1024 , 51 , 'AMT' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1025 , 51 , 'CURRENCY' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1026 , 51 , 'INVNUM' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1027 , 51 , 'ORDERDESC' , 'VARIABLE' , 'basket' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1028 , 51 , 'EMAIL' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1029 , 51 , 'CANCELURL' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1030 , 51 , 'RETURNURL' , 'VARIABLE' , '{site_url}order_final.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1031 , 51 , 'Advanced_URL' , 'CONSTANT' , 'https://payflowpro.paypal.com' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1032 , 6 , 'return' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1033 , 6 , 'cancel_return' , 'VARIABLE' , '{site_url}order_final.php?return_action=cancel' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1034 , 52 , 'payment_url' , 'CONSTANT' , 'https://secure.gate2shop.com/ppp/purchase.do' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1035 , 52 , 'merchant_site_id' , 'CONSTANT' , '!Enter your site id!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1036 , 52 , 'merchant_id' , 'CONSTANT' , '!Enter your ip!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1037 , 52 , 'secret' , 'CONSTANT' , '!Enter your secret code!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1038 , 52 , 'merchant_unique_id' , 'CONSTANT' , '!Enter your unique id!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1039 , 52 , 'version' , 'CONSTANT' , '1.0.0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1040 , 52 , 'currency' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1041 , 52 , 'total_amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1042 , 52 , 'time_stamp' , 'VARIABLE' , 'order_placed_timestamp' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1043 , 52 , 'invoice_id' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1044 , 52 , 'first_name' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1045 , 52 , 'last_name' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1046 , 52 , 'email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1047 , 52 , 'address1' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1048 , 52 , 'address2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1049 , 52 , 'city' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1050 , 52 , 'country' , 'VARIABLE' , 'country' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1051 , 52 , 'state' , 'VARIABLE' , 'state' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1052 , 52 , 'zip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1053 , 53 , 'MerchantID' , 'CONSTANT' , '!Please specify your merchant ID!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1054 , 53 , 'URL' , 'CONSTANT' , 'https://idealtest.rabobank.nl/ideal/mpiPayInitRabo.do' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1055 , 53 , 'Key' , 'CONSTANT' , '!Please specify your secure key!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1056 , 53 , 'urlCancel' , 'VARIABLE' , '{site_url}order_info.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1057 , 53 , 'urlError' , 'VARIABLE' , '{site_url}order_info.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1058 , 53 , 'urlSuccess' , 'VARIABLE' , '{site_url}order_final.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1059 , 54 , 'PSPID' , 'CONSTANT' , '!Please specify your merchant ID!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1060 , 54 , 'SHA-IN' , 'CONSTANT' , '!Please specify your SHA-IN pass phrase' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1061 , 54 , 'SHA-OUT' , 'CONSTANT' , '!Please specify your SHA-OUT pass phrase' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1062 , 54 , 'PAYMENTURL' , 'CONSTANT' , 'https://secure.ogone.com/ncol/test/orderstandard.asp' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1063 , 54 , 'ACCEPTURL' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1064 , 54 , 'DECLINEURL' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1065 , 54 , 'CN' , 'VARIABLE' , 'name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1066 , 55 , 'storefront_id' , 'CONSTANT' , '!Please specify your Storefront ID!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1067 , 55 , 'cpi_hash_key' , 'CONSTANT' , '!Please specify your hash code!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1068 , 55 , 'hsbc_cpi_dir' , 'CONSTANT' , '!Please specify path to hsbc_cpi executable!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1069 , 55 , 'URL' , 'CONSTANT' , 'https://www.cpi.hsbc.com/servlet' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1070 , 55 , 'Mode' , 'CONSTANT' , 'T' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1071 , 55 , 'CpiDirectResultUrlMode' , 'VARIABLE' , '{site_url}/payments/hsbc_cpi_direct.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1072 , 55 , 'CpiReturnUrl' , 'VARIABLE' , '{site_url}/order_final.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1073 , 7 , 'x_Card_Code' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1074 , 56 , 'action_url' , 'CONSTANT' , 'https://api.multisafepay.com/ewx/' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1075 , 56 , 'notification_url' , 'VARIABLE' , '{site_url}order_final.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1076 , 56 , 'account' , 'CONSTANT' , '!Enter your Account ID!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1077 , 56 , 'site_id' , 'CONSTANT' , '!Enter your Site ID!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1078 , 56 , 'site_secure_code' , 'CONSTANT' , '!Enter your Site Secure Code!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1079 , 56 , 'locale' , 'CONSTANT' , 'EN' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1080 , 56 , 'ipaddress' , 'VARIABLE' , 'user_ip' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1081 , 56 , 'firstname' , 'VARIABLE' , 'first_name' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1082 , 56 , 'lastname' , 'VARIABLE' , 'last_name' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1083 , 56 , 'address1' , 'VARIABLE' , 'address1' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1084 , 56 , 'address2' , 'VARIABLE' , 'address2' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1085 , 56 , 'zipcode' , 'VARIABLE' , 'zip' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1086 , 56 , 'city' , 'VARIABLE' , 'city' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1087 , 56 , 'state' , 'VARIABLE' , 'state' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1088 , 56 , 'country' , 'VARIABLE' , 'country_code' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1089 , 56 , 'phone' , 'VARIABLE' , 'phone' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1090 , 56 , 'email' , 'VARIABLE' , 'email' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1091 , 56 , 'id' , 'VARIABLE' , 'order_id' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1092 , 56 , 'currency' , 'VARIABLE' , 'currency_code' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1093 , 56 , 'amount' , 'VARIABLE' , 'order_total' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1094 , 56 , 'description' , 'VARIABLE' , 'basket' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1095 , 56 , 'var1' , 'VARIABLE' , NULL , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1096 , 56 , 'var2' , 'VARIABLE' , NULL , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1097 , 56 , 'var3' , 'VARIABLE' , NULL , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1098 , 56 , 'items' , 'VARIABLE' , NULL , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1099 , 56 , 'manual' , 'CONSTANT' , 'false' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1100 , 56 , 'gateway' , 'CONSTANT' , NULL , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1101 , 57 , 'ewayCustomerID' , 'CONSTANT' , '!Enter your Customer ID!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1102 , 57 , 'ewayTrxnNumber' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1103 , 57 , 'ewayTotalAmount' , 'VARIABLE' , 'order_total_100' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1104 , 57 , 'ewayCardHoldersName' , 'VARIABLE' , 'cc_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1105 , 57 , 'ewayCardNumber' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1106 , 57 , 'ewayCardExpiryMonth' , 'VARIABLE' , 'cc_expiry_month' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1107 , 57 , 'ewayCardExpiryYear' , 'VARIABLE' , 'cc_expiry_year' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1108 , 57 , 'ewayCustomerFirstName' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1109 , 57 , 'ewayCustomerLastName' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1110 , 57 , 'ewayCustomerEmail' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1111 , 57 , 'ewayCustomerAddress' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1112 , 57 , 'ewayCustomerPostcode' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1113 , 57 , 'ewayCustomerInvoiceDescription' , 'VARIABLE' , 'basket' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1114 , 57 , 'ewayCVN' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1115 , 57 , 'ewayCustomerInvoiceRef' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1116 , 57 , 'ewayOption1' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1117 , 57 , 'ewayOption2' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1118 , 57 , 'ewayOption3' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1119 , 58 , 'ewayCustomerID' , 'CONSTANT' , '!Enter your Customer ID!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1120 , 58 , 'ewayTotalAmount' , 'VARIABLE' , 'order_total_100' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1121 , 58 , 'eWAYTrxnNumber' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1122 , 58 , 'eWAYURL' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1123 , 58 , 'eWAYSiteTitle' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1124 , 58 , 'ewayCustomerFirstName' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1125 , 58 , 'ewayCustomerLastName' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1126 , 58 , 'ewayCustomerEmail' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1127 , 58 , 'ewayCustomerAddress' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1128 , 58 , 'ewayCustomerPostcode' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1129 , 58 , 'ewayCustomerInvoiceDescription' , 'VARIABLE' , 'basket' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1130 , 58 , 'ewayCustomerInvoiceRef' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1131 , 58 , 'ewayOption1' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1132 , 58 , 'ewayOption2' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1133 , 58 , 'ewayOption3' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1134 , 58 , 'ewayAutoRedirect' , 'CONSTANT' , '1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1135 , 59 , 'TID' , 'CONSTANT' , '!Enter your TID!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1136 , 59 , 'Userid' , 'CONSTANT' , '!Enter your Userid!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1137 , 59 , 'Password' , 'CONSTANT' , '!Enter your Password!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1138 , 59 , 'advenced_url' , 'CONSTANT' , 'https://caesar-ips.transact-gmbh.de/xml' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1139 , 59 , 'Currency' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1140 , 59 , 'Amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1141 , 59 , 'Paymode' , 'VARIABLE' , 'direct' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1142 , 59 , 'TransID' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1143 , 59 , 'card_number' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1144 , 59 , 'card_type' , 'VARIABLE' , 'cc_type' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1145 , 59 , 'VALIDMONTH' , 'VARIABLE' , 'cc_expiry_month' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1146 , 59 , 'VALIDYEAR' , 'VARIABLE' , '20{cc_expiry_year}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1147 , 59 , 'card_security_code' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1148 , 59 , 'card_name' , 'VARIABLE' , 'cc_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1149 , 59 , 'function' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1150 , 60 , 'clientid' , 'CONSTANT' , '!Enter your clientid!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1151 , 60 , 'storekey' , 'CONSTANT' , '!Enter your storekey!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1152 , 60 , 'islemtipi' , 'CONSTANT' , 'Auth' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1153 , 60 , 'rnd' , 'VARIABLE' , 'session_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1154 , 60 , 'storetype' , 'CONSTANT' , '3d_pay' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1155 , 60 , 'lang' , 'CONSTANT' , 'en' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1156 , 60 , 'oid' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1157 , 60 , 'amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1158 , 60 , 'pan' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1159 , 60 , 'cv2' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1160 , 60 , 'Ecom_Payment_Card_ExpDate_Year' , 'VARIABLE' , 'cc_expiry_year' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1161 , 60 , 'Ecom_Payment_Card_ExpDate_Month' , 'VARIABLE' , 'cc_expiry_month' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1162 , 60 , 'cardType' , 'VARIABLE' , 'cc_type' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1163 , 60 , 'okUrl' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1164 , 60 , 'failUrl' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1165 , 60 , 'firmaadi' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1166 , 60 , 'Fismi' , 'VARIABLE' , 'name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1167 , 60 , 'Fadres' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1168 , 60 , 'Fadres2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1169 , 60 , 'Fil' , 'VARIABLE' , 'province' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1170 , 60 , 'Filce' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1171 , 60 , 'Fpostakodu' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1172 , 60 , 'tel' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1173 , 60 , 'fulkekod' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1174 , 60 , 'nakliyeFirma' , 'VARIABLE' , 'company_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1175 , 60 , 'tismi' , 'VARIABLE' , 'delivery_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1176 , 60 , 'tadres' , 'VARIABLE' , 'delivery_address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1177 , 60 , 'tadres2' , 'VARIABLE' , 'delivery_address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1178 , 60 , 'til' , 'VARIABLE' , 'delivery_province' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1179 , 60 , 'tilce' , 'VARIABLE' , 'delivery_city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1180 , 60 , 'tpostakodu' , 'VARIABLE' , 'delivery_zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1181 , 60 , 'tulkekod' , 'VARIABLE' , 'delivery_country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1182 , 61 , 'PAYEE_ID' , 'CONSTANT' , '!Enter your PAYEE_ID!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1183 , 61 , 'SHOPORDERNUMBER' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1184 , 61 , 'BILL_AMOUNT' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1185 , 61 , 'DESCRIPTION' , 'VARIABLE' , 'basket' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1186 , 61 , 'SUCCESS_URL' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1187 , 61 , 'FAILURE_URL' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1188 , 61 , 'LANG' , 'VARIABLE' , 'en' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1189 , 61 , 'LOG' , 'CONSTANT' , '!Enter your LOG!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1190 , 61 , 'PASS' , 'CONSTANT' , '!Enter your PASS!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1191 , 61 , 'CHECK_URL' , 'CONSTANT' , 'https://www.portmone.com.ua/secure/bills/results.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1192 , 62 , 'ePNAccount' , 'CONSTANT' , '!Enter your ePNAccount!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1193 , 62 , 'HTML' , 'CONSTANT' , 'No' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1194 , 62 , 'Total' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1195 , 62 , 'ID' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1196 , 62 , 'CardNo' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1197 , 62 , 'ExpMonth' , 'VARIABLE' , 'cc_expiry_month' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1198 , 62 , 'ExpYear' , 'VARIABLE' , 'cc_expiry_year' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1199 , 62 , 'CVV2Type' , 'CONSTANT' , '0' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1200 , 62 , 'CVV2' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1201 , 62 , 'FirstName' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1202 , 62 , 'LastName' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1203 , 62 , 'Address' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1204 , 62 , 'Zip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1205 , 62 , 'EMail' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1206 , 21 , 'business' , 'CONSTANT' , NULL , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1207 , 22 , 'business' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1208 , 63 , 'order_id' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1209 , 63 , 'vc' , 'VARIABLE' , 'vc' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1210 , 64 , 'CustomerID' , 'CONSTANT' , '!Enter your CustomerID!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1211 , 64 , 'TransactionCurrency' , 'VARIABLE' , 'currency_value' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1212 , 64 , 'TransactionAmount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1213 , 64 , 'Notes' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1214 , 64 , 'CustomerEmail' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1215 , 64 , 'HouseNumber' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1216 , 64 , 'PostCode' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1217 , 64 , 'Country' , 'VARIABLE' , 'country' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1218 , 64 , 'RedirectorSuccess' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1219 , 64 , 'RedirectorFailed' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1220 , 64 , 'confirm_url' , 'CONSTANT' , 'https://secure.totalwebsecure.com/paypage/confirm.asp' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1221 , 65 , 'storefront_id' , 'CONSTANT' , '!Your Storefront ID!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1222 , 65 , 'cpi_hash_key' , 'CONSTANT' , '!Your Hash Key ID!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1223 , 65 , 'Mode' , 'CONSTANT' , 'T' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1224 , 65 , 'CpiDirectResultUrl' , 'VARIABLE' , '{secure_url}payments/hsbc_cpi_direct.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1225 , 65 , 'CpiReturnUrl' , 'VARIABLE' , '{secure_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1226 , 65 , 'URL' , 'CONSTANT' , 'https://www.cpi.hsbc.com/servlet' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1227 , 65 , 'hsbc_cpi_filepath' , 'CONSTANT' , '!Your path to the hsbc_cpi binary!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1228 , 66 , 'action_url' , 'CONSTANT' , 'https://www.payson.se/merchant/default.aspx' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1229 , 66 , 'agentid' , 'CONSTANT' , '!Enter your agentid!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1230 , 66 , 'secretkey' , 'CONSTANT' , '!Enter your storekey!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1231 , 66 , 'selleremail' , 'CONSTANT' , '!Enter your email!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1232 , 66 , 'buyeremail' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1233 , 66 , 'buyerfirstname' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1234 , 66 , 'buyerlastname' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1235 , 66 , 'description' , 'VARIABLE' , 'basket' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1236 , 66 , 'cost' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1237 , 66 , 'extracost' , 'VARIABLE' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1238 , 66 , 'refnr' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1239 , 66 , 'guaranteeoffered' , 'CONSTANT' , '1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1240 , 66 , 'customreceipt' , 'CONSTANT' , 'true' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1241 , 66 , 'paymentmethod' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1242 , 66 , 'okurl' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1243 , 66 , 'cancelurl' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1244 , 67 , 'version' , 'CONSTANT' , '1.0.0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1245 , 67 , 'merid' , 'CONSTANT' , '!Enter your merchant id!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1246 , 67 , 'merrespurl' , 'CONSTANT' , 'thhps://www.acme.com' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1247 , 67 , 'acqid' , 'CONSTANT' , '402971' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1248 , 67 , 'purchaseamt' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1249 , 67 , 'purchasecurrency' , 'VARIABLE' , 'currency_value' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1250 , 67 , 'purchasecurrencyexponent' , 'CONSTANT' , '2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1251 , 67 , 'orderid' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1252 , 67 , 'cardno' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1253 , 67 , 'cardexpdate' , 'VARIABLE' , '{cc_expiry_month}{cc_expiry_year}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1254 , 67 , 'cardcvv2' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1255 , 67 , 'signaturemethod' , 'CONSTANT' , 'SHA1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1256 , 67 , 'testflag' , 'CONSTANT' , 'False' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1257 , 67 , 'captureflag' , 'CONSTANT' , 'A' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1258 , 67 , 'password' , 'CONSTANT' , '!Enter your merchant password!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1259 , 68 , 'BPE_Merchant' , 'CONSTANT' , '!Enter here your merchant code!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1260 , 68 , 'BPE_Amount' , 'VARIABLE' , 'order_total_100' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1261 , 68 , 'BPE_Currency' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1262 , 68 , 'Currency' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1263 , 68 , 'BPE_Description' , 'VARIABLE' , 'basket' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1264 , 68 , 'BPE_Reference' , 'VARIABLE' , 'session_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1265 , 68 , 'BPE_Language' , 'VARIABLE' , 'EN' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1266 , 68 , 'BPE_Mode' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1267 , 68 , 'BPE_Invoice' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1268 , 68 , 'BPE_Return_Success' , 'VARIABLE' , '{site_url}order_final.php?status=Success' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1269 , 68 , 'BPE_Return_Reject' , 'VARIABLE' , '{site_url}order_final.php?status=Reject' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1270 , 68 , 'BPE_Return_Error' , 'VARIABLE' , '{site_url}order_final.php?status=Error' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1271 , 68 , 'BPE_Return_Method' , 'CONSTANT' , 'POST' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1272 , 68 , 'BPE_Return_Target' , 'CONSTANT' , NULL , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1273 , 68 , 'BPE_Style' , 'CONSTANT' , 'FRAME' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1274 , 68 , 'BPE_Autoclose_Popup' , 'CONSTANT' , '0' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1275 , 68 , 'action_url' , 'CONSTANT' , 'https://payment.buckaroo.nl/sslplus/request_for_authorization.asp' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1276 , 68 , 'secretkey' , 'CONSTANT' , '!Enter here your secret key!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1277 , 68 , 'BPE_Issuer' , 'CONSTANT' , 'RABOBANK' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1278 , 69 , 'MerchantNumber' , 'CONSTANT' , '!Enter here your merchant number!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1279 , 69 , 'Amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1280 , 69 , 'OrderID' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1281 , 69 , 'CurrencyID' , 'VARIABLE' , 'currency_value' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1286 , 69 , 'OKURL' , 'VARIABLE' , '{site_url}order_final.php?va_status=success' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1287 , 69 , 'FAILURL' , 'VARIABLE' , '{site_url}order_final.php?va_status=failed' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1288 , 69 , 'SessionID' , 'VARIABLE' , 'session_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1289 , 69 , 'ReferenceText' , 'VARIABLE' , 'basket' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1290 , 69 , 'PayType' , 'VARIABLE' , '1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1291 , 69 , 'TestMode' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1292 , 56 , 'redirect_url' , 'VARIABLE' , '{site_url}order_final.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1293 , 56 , 'close_window' , 'CONSTANT' , 'true' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1294 , 69 , 'Secure_Capture' , 'CONSTANT' , 'https://pay.dandomain.dk/securecapture.asp' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1295 , 69 , 'Port' , 'CONSTANT' , '80' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1296 , 69 , 'AddFormPostVars' , 'CONSTANT' , '1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1297 , 69 , 'TunnelURL' , 'VARIABLE' , '{site_url}credit_card_info.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1298 , 69 , 'Secure_Tunnel' , 'VARIABLE' , 'https://pay.dandomain.dk/securetunnel.asp?merchantnumber={merchantnumber}&SessionID={session_id}&tunnelurl={site_url}payments/dankort_credit_card_info.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1299 , 69 , 'InstantCapture' , 'CONSTANT' , '0' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1300 , 70 , 'merchantid' , 'CONSTANT' , '!Your merchantid!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1301 , 70 , 'posid' , 'CONSTANT' , '!Your posid!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1302 , 70 , 'user' , 'CONSTANT' , '!Your username!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1303 , 70 , 'merchantreference' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1304 , 70 , 'currency' , 'VARIABLE' , 'currency_value' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1305 , 70 , 'amount' , 'VARIABLE' , 'order_total_100' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1306 , 70 , 'Installments' , 'CONSTANT' , NULL , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1307 , 70 , 'preorderdate' , 'CONSTANT' , NULL , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1308 , 70 , 'langid' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1309 , 70 , 'backlink' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1310 , 70 , 'parameters' , 'VARIABLE' , NULL , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1311 , 71 , 'action_url' , 'CONSTANT' , 'https://www.mobile88.com/epayment/entry.asp' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1312 , 71 , 'MerchantCode' , 'CONSTANT' , '!Your MerchantCode!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1313 , 71 , 'MerchantKey' , 'CONSTANT' , '!Your MerchantKey!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1314 , 71 , 'PaymentId' , 'CONSTANT' , '2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1315 , 71 , 'RefNo' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1316 , 71 , 'Amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1317 , 71 , 'Currency' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1318 , 71 , 'ProdDesc' , 'VARIABLE' , 'basket' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1319 , 71 , 'UserName' , 'VARIABLE' , 'name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1320 , 71 , 'UserEmail' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1321 , 71 , 'UserContact' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1322 , 71 , 'Remark' , 'CONSTANT' , NULL , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1323 , 71 , 'Lang' , 'CONSTANT' , NULL , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1324 , 72 , 'action_url' , 'CONSTANT' , 'https://sale.GTBill.com/quickpay.aspx' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1325 , 72 , 'confirm_ip_list_url' , 'CONSTANT' , 'https://sale.gtbill.com/ip_list.txt' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1326 , 72 , 'MerchantID' , 'CONSTANT' , '!Your MerchantID!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1327 , 72 , 'SiteID' , 'CONSTANT' , '!Your SiteID!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1328 , 72 , 'AmountTotal' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1329 , 72 , 'CurrencyID' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1330 , 72 , 'AmountShipping' , 'VARIABLE' , 'shipping_cost_incl_tax' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1331 , 72 , 'ReturnURL' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1332 , 72 , 'FirstName' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1333 , 72 , 'LastName' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1334 , 72 , 'Address1' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1335 , 72 , 'Address2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1336 , 72 , 'City' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1337 , 72 , 'State' , 'VARIABLE' , 'state' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1338 , 72 , 'PostalCode' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1339 , 72 , 'Country' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1340 , 72 , 'Email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1341 , 72 , 'PhoneNumber' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1342 , 72 , 'RequireShipping' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1343 , 72 , 'ConfirmUrl' , 'VARIABLE' , '{site_url}payments/gtbill_api_confirm.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1344 , 72 , 'MerchantReference' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1345 , 73 , 'action_url' , 'CONSTANT' , 'https://reserved.e-consel.it/ecom/ins2acq.mac/inizio' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1346 , 73 , 'tipoesec' , 'CONSTANT' , 'T' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1347 , 73 , 'tabfin' , 'CONSTANT' , '!Enter here your tabfin!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1348 , 73 , 'convenz' , 'CONSTANT' , '!Enter here your convenz!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1349 , 73 , 'nome' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1350 , 73 , 'cognome' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1351 , 73 , 'indirizzo' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1352 , 73 , 'impdafin' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1353 , 73 , 'impspe' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1354 , 73 , 'descr1' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1355 , 73 , 'parz1' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1356 , 74 , 'mid' , 'CONSTANT' , '!Enter here your mid!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1357 , 74 , 'user' , 'CONSTANT' , '!Enter here your user!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1358 , 74 , 'password' , 'CONSTANT' , '!Enter here your password!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1359 , 74 , 'responsetype' , 'CONSTANT' , 'direct' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1360 , 74 , 'type' , 'CONSTANT' , 'charge' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1361 , 74 , 'etan' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1362 , 74 , 'cctype' , 'VARIABLE' , 'cc_type' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1363 , 74 , 'cc' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1364 , 74 , 'expire' , 'VARIABLE' , '{cc_expiry_month}/{cc_expiry_year}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1365 , 74 , 'cvv' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1366 , 74 , 'firstname' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1367 , 74 , 'lastname' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1368 , 74 , 'street' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1369 , 74 , 'housenumber' , 'VARIABLE' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1370 , 74 , 'zip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1371 , 74 , 'zip4' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1372 , 74 , 'city' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1373 , 74 , 'country' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1374 , 74 , 'state' , 'VARIABLE' , 'state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1375 , 74 , 'telephone' , 'VARIABLE' , 'daytime_phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1376 , 74 , 'email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1377 , 74 , 'ip' , 'VARIABLE' , 'user_ip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1378 , 74 , 'currency' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1379 , 74 , 'exponent' , 'CONSTANT' , '2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1380 , 74 , 'amount' , 'VARIABLE' , 'order_total_100' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1381 , 75 , 'REQUEST.VERSION' , 'CONSTANT' , '1.0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1382 , 75 , 'SECURITY.SENDER' , 'CONSTANT' , '!Enter your SECURITY.SENDER!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1383 , 75 , 'TRANSACTION.MODE' , 'CONSTANT' , 'LIVE' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1384 , 75 , 'TRANSACTION.RESPONSE' , 'CONSTANT' , 'SYNC' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1385 , 75 , 'TRANSACTION.CHANNEL' , 'CONSTANT' , '!Enter your TRANSACTION.CHANNEL!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1386 , 75 , 'USER.LOGIN' , 'CONSTANT' , '!Enter your USER.LOGIN!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1387 , 75 , 'USER.PWD' , 'CONSTANT' , '!Enter your USER.PWD!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1388 , 75 , 'IDENTIFICATION.UNIQUEID' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1389 , 75 , 'IDENTIFICATION.SHORTID' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1390 , 75 , 'PAYMENT.CODE' , 'CONSTANT' , 'CC.DB' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1391 , 75 , 'PRESENTATION.AMOUNT' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1392 , 75 , 'PRESENTATION.CURRENCY' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1393 , 75 , 'CURRENCY' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1394 , 75 , 'PRESENTATION.USAGE' , 'VARIABLE' , 'basket' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1395 , 75 , 'ACCOUNT.HOLDER' , 'VARIABLE' , 'cc_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1396 , 75 , 'ACCOUNT.NUMBER' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1397 , 75 , 'ACCOUNT.BRAND' , 'VARIABLE' , 'cc_type' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1398 , 75 , 'ACCOUNT.EXPIRY_MONTH' , 'VARIABLE' , 'cc_expiry_month' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1399 , 75 , 'ACCOUNT.EXPIRY_YEAR' , 'VARIABLE' , 'cc_expiry_yyyy' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1400 , 75 , 'ACCOUNT.VERIFICATION' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1401 , 75 , 'NAME.GIVEN' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1402 , 75 , 'NAME.FAMILY' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1403 , 75 , 'ADDRESS.STREET' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1404 , 75 , 'ADDRESS.ZIP' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1405 , 75 , 'ADDRESS.CITY' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1406 , 75 , 'ADDRESS.STATE' , 'VARIABLE' , 'state' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1407 , 75 , 'ADDRESS.COUNTRY' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1408 , 75 , 'CONTACT.EMAIL' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1409 , 75 , 'CONTACT.IP' , 'VARIABLE' , 'user_ip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1410 , 75 , 'FRONTEND.ENABLED' , 'CONSTANT' , 'true' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1411 , 75 , 'FRONTEND.POPUP' , 'CONSTANT' , 'true' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1412 , 75 , 'FRONTEND.MODE' , 'CONSTANT' , 'DEFAULT' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1413 , 75 , 'FRONTEND.LANGUAGE' , 'CONSTANT' , 'en' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1414 , 75 , 'FRONTEND.RESPONSE_URL' , 'VARIABLE' , '{site_url}order_final.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1415 , 76 , 'REQUEST.VERSION' , 'CONSTANT' , '1.0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1416 , 76 , 'SECURITY.SENDER' , 'CONSTANT' , '!Enter your SECURITY.SENDER!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1417 , 76 , 'TRANSACTION.MODE' , 'CONSTANT' , 'LIVE' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1418 , 76 , 'TRANSACTION.RESPONSE' , 'CONSTANT' , 'SYNC' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1419 , 76 , 'TRANSACTION.CHANNEL' , 'CONSTANT' , '!Enter your TRANSACTION.CHANNEL!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1420 , 76 , 'USER.LOGIN' , 'CONSTANT' , '!Enter your USER.LOGIN!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1421 , 76 , 'USER.PWD' , 'CONSTANT' , '!Enter your USER.PWD!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1422 , 76 , 'IDENTIFICATION.UNIQUEID' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1423 , 76 , 'IDENTIFICATION.TRANSACTIONID' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1424 , 76 , 'PAYMENT.CODE' , 'VARIABLE' , 'DD.DB' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1425 , 76 , 'PRESENTATION.AMOUNT' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1426 , 76 , 'PRESENTATION.CURRENCY' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1427 , 76 , 'CURRENCY' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1428 , 76 , 'PRESENTATION.USAGE' , 'VARIABLE' , 'basket' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1429 , 76 , 'ACCOUNT.HOLDER' , 'VARIABLE' , 'cc_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1430 , 76 , 'ACCOUNT.NUMBER' , 'VARIABLE' , '{field_21}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1431 , 76 , 'ACCOUNT.BANK' , 'VARIABLE' , '{field_22}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1432 , 76 , 'ACCOUNT.COUNTRY' , 'VARIABLE' , '{field_23}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1433 , 76 , 'NAME.GIVEN' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1434 , 76 , 'NAME.FAMILY' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1435 , 76 , 'ADDRESS.STREET' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1436 , 76 , 'ADDRESS.ZIP' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1437 , 76 , 'ADDRESS.CITY' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1438 , 76 , 'ADDRESS.STATE' , 'VARIABLE' , 'state' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1439 , 76 , 'ADDRESS.COUNTRY' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1440 , 76 , 'CONTACT.EMAIL' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1441 , 76 , 'CONTACT.IP' , 'VARIABLE' , 'user_ip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1442 , 76 , 'FRONTEND.ENABLED' , 'CONSTANT' , 'true' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1443 , 76 , 'FRONTEND.POPUP' , 'CONSTANT' , 'true' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1444 , 76 , 'FRONTEND.MODE' , 'CONSTANT' , 'DEFAULT' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1445 , 76 , 'FRONTEND.LANGUAGE' , 'CONSTANT' , 'en' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1446 , 76 , 'FRONTEND.RESPONSE_URL' , 'VARIABLE' , '{site_url}order_final.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1447 , 77 , 'merchid' , 'CONSTANT' , '!Enter your merchid!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1448 , 77 , 'merchpwd' , 'CONSTANT' , '!Enter your merchpwd!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1449 , 77 , 'msgnr' , 'CONSTANT' , '200' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1450 , 77 , 'pcode' , 'CONSTANT' , '00' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1451 , 77 , 'creditc' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1452 , 77 , 'cctyp' , 'VARIABLE' , 'cc_type' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1453 , 77 , 'cvc2' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1454 , 77 , 'expdat' , 'VARIABLE' , '{cc_expiry_year}{cc_expiry_month}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1455 , 77 , 'timestamp' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1456 , 77 , 'trefnum' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1457 , 77 , 'amount' , 'VARIABLE' , 'order_total_100' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1458 , 77 , 'currency' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1459 , 78 , 'merchid' , 'CONSTANT' , '!Enter your merchid!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1460 , 78 , 'merchpwd' , 'CONSTANT' , '!Enter your merchpwd!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1461 , 78 , 'msgnr' , 'CONSTANT' , '300' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1462 , 78 , 'pcode' , 'CONSTANT' , '00' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1463 , 78 , 'timestamp' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1464 , 78 , 'bankcode' , 'VARIABLE' , '{field_25}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1465 , 78 , 'account' , 'VARIABLE' , '{field_24}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1466 , 78 , 'trefnum' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1467 , 78 , 'amount' , 'VARIABLE' , 'order_total_100' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1468 , 78 , 'currency' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1469 , 78 , 'cctyp' , 'CONSTANT' , '99' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1470 , 79 , 'action_url' , 'CONSTANT' , 'https://payment.certitrade.net/webshophtml/e/auth.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1471 , 79 , 'merchantid' , 'CONSTANT' , '!Enter here your merchant id!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1472 , 79 , 'md5key' , 'CONSTANT' , '!Enter here your key!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1473 , 79 , 'rev' , 'CONSTANT' , 'E' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1474 , 79 , 'orderid' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1475 , 79 , 'amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1476 , 79 , 'currency' , 'VARIABLE' , 'currency_value' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1477 , 79 , 'retururl' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1478 , 79 , 'approveurl' , 'VARIABLE' , '{site_url}payments/certitrade_confirm.php?va_status=approve' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1479 , 79 , 'declineurl' , 'VARIABLE' , '{site_url}payments/certitrade_confirm.php?va_status=decline' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1480 , 79 , 'cancelurl' , 'VARIABLE' , '{site_url}payments/certitrade_confirm.php?va_status=cancel' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1481 , 79 , 'lang' , 'CONSTANT' , 'en' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1482 , 79 , 'cust_id' , 'VARIABLE' , 'user_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1483 , 79 , 'cust_name' , 'VARIABLE' , 'name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1484 , 79 , 'cust_address1' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1485 , 79 , 'cust_address2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1486 , 79 , 'cust_address3' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1487 , 79 , 'cust_zip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1488 , 79 , 'cust_city' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1489 , 79 , 'cust_phone' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1490 , 79 , 'cust_email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1491 , 79 , 'cust_country' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1492 , 79 , 'connection' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1493 , 79 , 'acquirer' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1494 , 79 , 'returwindow' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1495 , 79 , 'DEBUG' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1496 , 79 , 'HTTPDEBUG' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1497 , 79 , 'timeout' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1498 , 79 , 'delayed_capture' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1499 , 80 , 'eid' , 'CONSTANT' , '!Enter here your eid!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1500 , 80 , 'estoreUser' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1501 , 80 , 'secret' , 'CONSTANT' , '!Enter here your secret!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1502 , 80 , 'timeout' , 'CONSTANT' , '30' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1503 , 80 , 'PROTO_VSN' , 'CONSTANT' , '3.5' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1504 , 80 , 'CLIENT_VSN' , 'CONSTANT' , 'php:1.3' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1505 , 80 , 'estoreOrderNo' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1506 , 80 , 'pno' , 'VARIABLE' , '{field_26}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1507 , 80 , 'fname' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1508 , 80 , 'lname' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1509 , 80 , 'careof' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1510 , 80 , 'street' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1511 , 80 , 'postno' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1512 , 80 , 'city' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1513 , 80 , 'country' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1514 , 80 , 'telno' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1515 , 80 , 'cellno' , 'VARIABLE' , 'daytime_phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1516 , 80 , 'email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1517 , 80 , 'clientIp' , 'VARIABLE' , 'user_ip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1518 , 80 , 'passwd' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1519 , 80 , 'newPasswd' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1520 , 80 , 'flags' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1521 , 80 , 'comment' , 'VARIABLE' , 'basket' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1522 , 80 , 'ready_date' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1523 , 80 , 'rand_string' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1524 , 80 , 'currency' , 'VARIABLE' , 'SEK' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1525 , 80 , 'ecountry' , 'CONSTANT' , '209' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1526 , 80 , 'language' , 'CONSTANT' , '138' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1527 , 80 , 'pno_encoding' , 'CONSTANT' , '2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1528 , 80 , 'pclass' , 'CONSTANT' , '-1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1529 , 80 , 'ysalary' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1530 , 80 , 'shipmenttype' , 'CONSTANT' , '1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1531 , 81 , 'UserName' , 'CONSTANT' , '!Enter here your User Name!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1532 , 81 , 'Password' , 'CONSTANT' , '!Enter here your Password!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1533 , 81 , 'TransType' , 'CONSTANT' , 'Sale' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1534 , 81 , 'Amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1535 , 81 , 'InvNum' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1536 , 81 , 'PNRef' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1537 , 81 , 'NameOnCard' , 'VARIABLE' , 'cc_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1538 , 81 , 'CardNum' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1539 , 81 , 'ExpDate' , 'VARIABLE' , '{cc_expiry_month}{cc_expiry_year}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1540 , 81 , 'CVNum' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1541 , 81 , 'Zip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1542 , 81 , 'Street' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1543 , 81 , 'MagData' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1544 , 81 , 'ExtData' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1545 , 82 , 'payment_url' , 'CONSTANT' , 'https://partnerweb.sveaekonomi.se/webpayhosted2/InitiatePayment.aspx' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1546 , 82 , 'password' , 'CONSTANT' , '!Enter your password!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1547 , 82 , 'Username' , 'CONSTANT' , '!Enter your Username!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1548 , 82 , 'OrderId' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1549 , 82 , 'SuccessURL' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1550 , 82 , 'CancelURL' , 'VARIABLE' , '{site_url}payments/sveawebpay_cancel.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1551 , 82 , 'Testmode' , 'CONSTANT' , 'False' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1552 , 82 , 'Language' , 'CONSTANT' , 'en' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1553 , 82 , 'Currency' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1554 , 52 , 'phone1' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1555 , 52 , 'phone2' , 'VARIABLE' , 'daytime_phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1556 , 52 , 'phone3' , 'VARIABLE' , 'evening_phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1557 , 83 , 'PSPID' , 'CONSTANT' , '!Enter your PSPID!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1558 , 83 , 'USERID' , 'CONSTANT' , '!Enter your USERID!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1559 , 83 , 'PSWD' , 'CONSTANT' , '!Enter your PSWD!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1560 , 83 , 'Signature' , 'CONSTANT' , '!Enter your Signature!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1561 , 83 , 'orderID' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1562 , 83 , 'amount' , 'VARIABLE' , 'order_total_100' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1563 , 83 , 'currency' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1564 , 83 , 'CN' , 'VARIABLE' , 'cc_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1565 , 83 , 'CARDNO' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1566 , 83 , 'CVC' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1567 , 83 , 'ED' , 'VARIABLE' , '{cc_expiry_year}/{cc_expiry_month}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1568 , 83 , 'EMAIL' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1569 , 83 , 'OwnerZip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1570 , 83 , 'Owneraddress' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1571 , 83 , 'REMOTE_ADDR' , 'VARIABLE' , 'user_ip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1572 , 83 , 'ECI' , 'CONSTANT' , '7' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1573 , 83 , 'operation' , 'CONSTANT' , 'SAL' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1574 , 84 , 'account_id' , 'CONSTANT' , '!Enter here your Account ID!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1575 , 84 , 'secret_key' , 'CONSTANT' , '!Enter here your secret key!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1576 , 84 , 'mode' , 'CONSTANT' , 'LIVE' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1577 , 84 , 'return_url' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1578 , 84 , 'reference_no' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1579 , 84 , 'amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1580 , 84 , 'currency' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1581 , 84 , 'description' , 'VARIABLE' , 'basket' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1582 , 84 , 'name' , 'VARIABLE' , 'name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1583 , 84 , 'address' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1584 , 84 , 'city' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1585 , 84 , 'state' , 'VARIABLE' , 'state' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1586 , 84 , 'postal_code' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1587 , 84 , 'country' , 'VARIABLE' , 'country_code_alpha3' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1588 , 84 , 'email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1589 , 84 , 'phone' , 'VARIABLE' , 'daytime_phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1590 , 84 , 'ship_name' , 'VARIABLE' , 'delivery_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1591 , 84 , 'ship_address' , 'VARIABLE' , 'delivery_address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1592 , 84 , 'ship_city' , 'VARIABLE' , 'delivery_city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1593 , 84 , 'ship_state' , 'VARIABLE' , 'delivery_state' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1594 , 84 , 'ship_postal_code' , 'VARIABLE' , 'delivery_zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1595 , 84 , 'ship_country' , 'VARIABLE' , 'delivery_country_code_alpha3' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1596 , 84 , 'ship_phone' , 'VARIABLE' , 'delivery_phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1597 , 22 , 'Amount' , 'VARIABLE' , 'order_total_100' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1598 , 22 , 'CurrencyCode' , 'VARIABLE' , 'currency_value' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1599 , 22 , 'ProcessorId' , 'CONSTANT' , '!Your ProcessorId!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1600 , 22 , 'MerchantId' , 'CONSTANT' , '!Your MerchantId!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1601 , 22 , 'TransactionPwd' , 'CONSTANT' , '!Your TransactionPwd!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1602 , 22 , 'MAPS_URL' , 'CONSTANT' , 'https://paypal.cardinalcommerce.com/maps/txns.asp' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1603 , 85 , 'payment_url' , 'CONSTANT' , 'https://e-payment.postfinance.ch/ncol/prod/orderstandard.asp' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1604 , 85 , 'PSPID' , 'CONSTANT' , '!Enter here your PSPID!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1605 , 85 , 'secret_in' , 'CONSTANT' , '!Enter here your additional string!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1606 , 85 , 'secret_out' , 'CONSTANT' , '!Enter here your additional string!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1607 , 85 , 'orderID' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1608 , 85 , 'amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1609 , 85 , 'currency' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1610 , 85 , 'language' , 'CONSTANT' , 'en_US' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1611 , 85 , 'CN' , 'VARIABLE' , 'name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1612 , 85 , 'EMAIL' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1613 , 85 , 'ownerZIP' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1614 , 85 , 'owneraddress' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1615 , 85 , 'ownercty' , 'VARIABLE' , 'country' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1616 , 85 , 'ownertown' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1617 , 85 , 'ownertelno' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1618 , 85 , 'COM' , 'VARIABLE' , 'basket' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1619 , 85 , 'accepturl' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1620 , 85 , 'declineurl' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1621 , 85 , 'exceptionurl' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1622 , 85 , 'cancelurl' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1623 , 86 , 'secret' , 'CONSTANT' , '!Enter here your secret!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1624 , 86 , 'inv_id_ext' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1625 , 86 , 'inv_value_requested' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1626 , 86 , 'curr_id' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1627 , 86 , 'curr_id_requested' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1628 , 86 , 'sub_username' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1629 , 86 , 'sub_password' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1630 , 86 , 'cust_fname' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1631 , 86 , 'cust_lname' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1632 , 86 , 'cust_email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1633 , 86 , 'bill_addr_address' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1634 , 86 , 'bill_addr_city' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1635 , 86 , 'bill_addr_state' , 'VARIABLE' , 'state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1636 , 86 , 'bill_addr_zip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1637 , 86 , 'bill_addr_country' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1638 , 86 , 'WebsiteHomePage' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1639 , 86 , 'cascade_target' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1640 , 86 , 'httl' , 'VARIABLE' , 'order_placed_timestamp' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1641 , 86 , 'hver' , 'CONSTANT' , '1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1642 , 86 , 'cust_id_ext' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1643 , 86 , 'disable_emailbox' , 'CONSTANT' , '0' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1644 , 86 , 'afid_ext' , 'CONSTANT' , NULL , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1645 , 86 , 'udf_01' , 'VARIABLE' , NULL , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1646 , 86 , 'udf_02' , 'VARIABLE' , NULL , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1647 , 87 , 'request_url' , 'CONSTANT' , 'https://payment.ewaygateway.com/Request/' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1648 , 87 , 'CustomerID' , 'CONSTANT' , '!Enter here your eWAY Customer ID!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1649 , 87 , 'UserName' , 'CONSTANT' , '!Enter here your eWAY Customer User Name!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1650 , 87 , 'Amount' , 'VARIABLE' , 'order_total' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1651 , 87 , 'Currency' , 'VARIABLE' , 'currency_code' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1652 , 87 , 'ReturnURL' , 'VARIABLE' , '{site_url}order_final.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1653 , 87 , 'CancelURL' , 'VARIABLE' , '{site_url}order_final.php?return_action=cancel' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1654 , 87 , 'response_url' , 'CONSTANT' , 'https://payment.ewaygateway.com/Result/' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1655 , 87 , 'CustomerFirstName' , 'VARIABLE' , 'first_name' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1656 , 87 , 'CustomerLastName' , 'VARIABLE' , 'last_name' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1657 , 87 , 'CustomerAddress' , 'VARIABLE' , 'address1' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1658 , 87 , 'CustomerCity' , 'VARIABLE' , 'city' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1659 , 87 , 'CustomerState' , 'VARIABLE' , 'state' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1660 , 87 , 'CustomerPostCode' , 'VARIABLE' , 'zip' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1661 , 87 , 'CustomerCountry' , 'VARIABLE' , 'country' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1662 , 87 , 'CustomerPhone' , 'VARIABLE' , 'daytime_phone' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1663 , 87 , 'CustomerEmail' , 'VARIABLE' , 'email' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1664 , 87 , 'InvoiceDescription' , 'VARIABLE' , 'basket' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1665 , 87 , 'MerchantReference' , 'VARIABLE' , 'order_id' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1666 , 87 , 'MerchantInvoice' , 'VARIABLE' , 'order_id' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1667 , 88 , 'request_url' , 'CONSTANT' , 'https://nz.ewaygateway.com/Request/' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1668 , 88 , 'CustomerID' , 'CONSTANT' , '!Enter here your eWAY Customer ID!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1669 , 88 , 'UserName' , 'CONSTANT' , '!Enter here your eWAY Customer User Name!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1670 , 88 , 'Amount' , 'VARIABLE' , 'order_total' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1671 , 88 , 'Currency' , 'VARIABLE' , 'currency_code' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1672 , 88 , 'ReturnURL' , 'VARIABLE' , '{site_url}order_final.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1673 , 88 , 'CancelURL' , 'VARIABLE' , '{site_url}order_final.php?return_action=cancel' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1674 , 88 , 'response_url' , 'CONSTANT' , 'https://nz.ewaygateway.com/Result/' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1675 , 88 , 'CustomerFirstName' , 'VARIABLE' , 'first_name' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1676 , 88 , 'CustomerLastName' , 'VARIABLE' , 'last_name' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1677 , 88 , 'CustomerAddress' , 'VARIABLE' , 'address1' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1678 , 88 , 'CustomerCity' , 'VARIABLE' , 'city' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1679 , 88 , 'CustomerState' , 'VARIABLE' , 'state' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1680 , 88 , 'CustomerPostCode' , 'VARIABLE' , 'zip' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1681 , 88 , 'CustomerCountry' , 'VARIABLE' , 'country' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1682 , 88 , 'CustomerPhone' , 'VARIABLE' , 'daytime_phone' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1683 , 88 , 'CustomerEmail' , 'VARIABLE' , 'email' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1684 , 88 , 'InvoiceDescription' , 'VARIABLE' , 'basket' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1685 , 88 , 'MerchantReference' , 'VARIABLE' , 'order_id' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1686 , 88 , 'MerchantInvoice' , 'VARIABLE' , 'order_id' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1687 , 89 , 'payment_url' , 'CONSTANT' , 'https://external.payex.com/pxorder/pxorder.asmx?wsdl' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1688 , 89 , 'accountNumber' , 'CONSTANT' , '!Enter here your account Number!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1689 , 89 , 'encryptionkey' , 'CONSTANT' , '!Enter here your encryption key!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1690 , 89 , 'purchaseOperation' , 'CONSTANT' , 'SALE' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1691 , 89 , 'price' , 'VARIABLE' , 'order_total_100' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1692 , 89 , 'priceArgList' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1693 , 89 , 'currency' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1694 , 89 , 'vat' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1695 , 89 , 'orderID' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1696 , 89 , 'productNumber' , 'CONSTANT' , '1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1697 , 89 , 'description' , 'CONSTANT' , 'description' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1698 , 89 , 'clientIPAddress' , 'VARIABLE' , 'user_ip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1699 , 89 , 'additionalValues' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1700 , 89 , 'externalID' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1701 , 89 , 'view' , 'CONSTANT' , 'CC' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1702 , 89 , 'agreementRef' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1703 , 89 , 'clientLanguage' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1704 , 89 , 'returnUrl' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1705 , 89 , 'cancelUrl' , 'VARIABLE' , '{site_url}order_final.php?return_action=cancel' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1706 , 90 , 'api-key' , 'CONSTANT' , '!Enter here your api-key!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1707 , 90 , 'redirect-url' , 'VARIABLE' , '{site_url}order_final.php' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1708 , 90 , 'amount' , 'VARIABLE' , 'order_total' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1709 , 90 , 'currency' , 'VARIABLE' , 'currency_code' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1710 , 90 , 'order-id' , 'VARIABLE' , 'order_id' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1711 , 90 , 'order-description' , 'VARIABLE' , 'basket' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1712 , 90 , 'ip-address' , 'VARIABLE' , 'user_ip' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1713 , 90 , 'billing-cc-number' , 'VARIABLE' , 'cc_number' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1714 , 90 , 'billing-cc-exp' , 'VARIABLE' , '{cc_expiry_month}{cc_expiry_year}' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1715 , 90 , 'cvv' , 'VARIABLE' , 'cc_security_code' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1716 , 90 , 'first-name' , 'VARIABLE' , 'first_name' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1717 , 90 , 'last-name' , 'VARIABLE' , 'last_name' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1718 , 90 , 'address1' , 'VARIABLE' , 'address1' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1719 , 90 , 'address2' , 'VARIABLE' , 'address2' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1720 , 90 , 'city' , 'VARIABLE' , 'city' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1721 , 90 , 'state' , 'VARIABLE' , 'state_code' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1722 , 90 , 'postal' , 'VARIABLE' , 'zip' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1723 , 90 , 'country' , 'VARIABLE' , 'country_code' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1724 , 90 , 'email' , 'VARIABLE' , 'email' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1725 , 90 , 'phone' , 'VARIABLE' , 'phone' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1726 , 90 , 'company' , 'VARIABLE' , 'company_name' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1727 , 90 , 'fax' , 'VARIABLE' , 'fax' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1728 , 90 , 'shipping_first-name' , 'VARIABLE' , 'delivery_first_name' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1729 , 90 , 'shipping_last-name' , 'VARIABLE' , 'delivery_last_name' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1730 , 90 , 'shipping_address1' , 'VARIABLE' , 'delivery_address1' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1731 , 90 , 'shipping_address2' , 'VARIABLE' , 'delivery_address2' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1732 , 90 , 'shipping_city' , 'VARIABLE' , 'delivery_city' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1733 , 90 , 'shipping_state' , 'VARIABLE' , 'delivery_state_code' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1734 , 90 , 'shipping_postal' , 'VARIABLE' , 'delivery_zip' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1735 , 90 , 'shipping_country' , 'VARIABLE' , 'delivery_country_code' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1736 , 90 , 'shipping_email' , 'VARIABLE' , 'delivery_email' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1737 , 90 , 'shipping_phone' , 'VARIABLE' , 'delivery_phone' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1738 , 90 , 'shipping_company' , 'VARIABLE' , 'delivery_company_name' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1739 , 90 , 'shipping_fax' , 'VARIABLE' , 'delivery_fax' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1740 , 91 , 'action_url' , 'CONSTANT' , 'https://www.UltimatePay.com/app/api/live/' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1741 , 91 , 'notify_url' , 'CONSTANT' , 'https://www.UltimatePay.com/app/api/live/' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1742 , 91 , 'login' , 'CONSTANT' , '!Enter here your login!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1743 , 91 , 'adminpwd' , 'CONSTANT' , '!Enter here your adminpwd!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1744 , 91 , 'secret' , 'CONSTANT' , '!Enter here your secret hash pass phrase!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1745 , 91 , 'sn' , 'CONSTANT' , '!Enter here your Service Code!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1746 , 91 , 'display' , 'CONSTANT' , 'Lightbox' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1747 , 91 , 'method' , 'CONSTANT' , 'StartOrderFrontEnd' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1748 , 91 , 'riskmode' , 'CONSTANT' , 'normal' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1749 , 91 , 'responseFormat' , 'CONSTANT' , 'nvp' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1750 , 91 , 'completeurl' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1751 , 91 , 'cancelurl' , 'VARIABLE' , '{site_url}order_final.php?return_action=cancel' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1752 , 91 , 'xdurl' , 'VARIABLE' , '{site_url}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1753 , 91 , 'amount' , 'CONSTANT' , '0.00' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1754 , 91 , 'currency' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1755 , 91 , 'amountdesc' , 'VARIABLE' , 'basket' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1756 , 91 , 'mirror' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1757 , 91 , 'userid' , 'VARIABLE' , 'user_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1758 , 91 , 'accountname' , 'VARIABLE' , 'name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1759 , 36 , 'BillingAddress2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1760 , 36 , 'BillingCity' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1761 , 36 , 'BillingCountry' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1762 , 36 , 'BillingState' , 'VARIABLE' , 'state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1763 , 36 , 'DeliveryFirstnames' , 'VARIABLE' , 'delivery_first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1764 , 36 , 'DeliverySurname' , 'VARIABLE' , 'delivery_last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1765 , 36 , 'DeliveryAddress2' , 'VARIABLE' , 'delivery_address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1766 , 36 , 'DeliveryCity' , 'VARIABLE' , 'delivery_city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1767 , 36 , 'DeliveryCountry' , 'VARIABLE' , 'delivery_country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1768 , 36 , 'DeliveryState' , 'VARIABLE' , 'delivery_state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1769 , 36 , 'DeliveryPhone' , 'VARIABLE' , 'delivery_daytime_phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1770 , 15 , 'BillingFirstnames' , 'VARIABLE' , 'first_name' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1771 , 15 , 'BillingSurname' , 'VARIABLE' , 'last_name' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1772 , 15 , 'BillingAddress2' , 'VARIABLE' , 'BillingAddress2' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1773 , 15 , 'BillingCity' , 'VARIABLE' , 'BillingCity' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1774 , 15 , 'BillingCountry' , 'VARIABLE' , 'country_code' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1775 , 15 , 'BillingState' , 'VARIABLE' , 'state_code' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1776 , 15 , 'DeliveryFirstnames' , 'VARIABLE' , 'delivery_first_name' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1777 , 15 , 'DeliverySurname' , 'VARIABLE' , 'delivery_last_name' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1778 , 15 , 'DeliveryAddress2' , 'VARIABLE' , 'delivery_address2' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1779 , 15 , 'DeliveryCity' , 'VARIABLE' , 'delivery_city' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1780 , 15 , 'DeliveryCountry' , 'VARIABLE' , 'delivery_country_code' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1781 , 15 , 'DeliveryState' , 'VARIABLE' , 'delivery_state_code' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1782 , 15 , 'DeliveryPhone' , 'VARIABLE' , 'delivery_daytime_phone' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1783 , 16 , 'BillingSurname' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1784 , 16 , 'BillingAddress2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1785 , 16 , 'BillingCity' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1786 , 16 , 'BillingCountry' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1787 , 16 , 'BillingState' , 'VARIABLE' , 'state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1788 , 16 , 'DeliveryFirstnames' , 'VARIABLE' , 'delivery_first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1789 , 16 , 'DeliverySurname' , 'VARIABLE' , 'delivery_last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1790 , 16 , 'DeliveryAddress2' , 'VARIABLE' , 'delivery_address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1791 , 16 , 'DeliveryCity' , 'VARIABLE' , 'delivery_city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1792 , 16 , 'DeliveryCountry' , 'VARIABLE' , 'delivery_country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1793 , 16 , 'DeliveryState' , 'VARIABLE' , 'delivery_state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1794 , 16 , 'DeliveryPhone' , 'VARIABLE' , 'delivery_daytime_phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1795 , 15 , 'payment_url' , 'CONSTANT' , 'https://live.sagepay.com/gateway/service/vspform-register.vsp' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1796 , 43 , 'payment_methods' , 'CONSTANT' , 'WLT' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1797 , 43 , 'recipient_description' , 'CONSTANT' , '!Enter your Merchant Name description!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1798 , 43 , 'hide_login' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1799 , 43 , 'logo_url' , 'VARIABLE' , '{site_url}images/logo.png' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1800 , 43 , 'merchant_fields' , 'CONSTANT' , 'platform' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1801 , 43 , 'platform' , 'CONSTANT' , 'viart' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1802 , 92 , 'pay_to_email' , 'CONSTANT' , '!enter your email!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1803 , 92 , 'return_url' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1804 , 92 , 'cancel_url' , 'VARIABLE' , '{site_url}order_info.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1805 , 92 , 'status_url' , 'VARIABLE' , '{site_url}payments/moneybookers_status.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1806 , 92 , 'secret_word' , 'CONSTANT' , '!Enter your secret word!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1807 , 92 , 'transaction_id' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1808 , 92 , 'pay_from_email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1809 , 92 , 'amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1810 , 92 , 'currency' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1811 , 92 , 'firstname' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1812 , 92 , 'lastname' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1813 , 92 , 'address' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1814 , 92 , 'address2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1815 , 92 , 'phone_number' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1816 , 92 , 'postal_code' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1817 , 92 , 'city' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1818 , 92 , 'state' , 'VARIABLE' , 'state' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1819 , 92 , 'country' , 'VARIABLE' , 'country_code_alpha3' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1820 , 92 , 'payment_methods' , 'CONSTANT' , 'ACC' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1821 , 92 , 'recipient_description' , 'CONSTANT' , '!Enter your Merchant Name description!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1822 , 92 , 'hide_login' , 'CONSTANT' , '1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1823 , 92 , 'logo_url' , 'CONSTANT' , '{site_url}images/logo.png' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1824 , 92 , 'merchant_fields' , 'CONSTANT' , 'platform' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1825 , 92 , 'platform' , 'CONSTANT' , 'viart' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1826 , 91 , 'merchtrans' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1827 , 93 , 'IC_Merchant' , 'CONSTANT' , '!Enter your merchant ID!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1828 , 93 , 'Encryptioncode' , 'CONSTANT' , '!Enter your Encryption code!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1829 , 93 , 'IC_Amount' , 'VARIABLE' , 'order_total_100' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1830 , 93 , 'IC_Currency' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1831 , 93 , 'IC_OrderID' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1832 , 93 , 'IC_PaymentMethod' , 'CONSTANT' , 'WIRE' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1833 , 93 , 'IC_Issuer' , 'VARIABLE' , 'DEFAULT' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1834 , 93 , 'IC_ResponseType' , 'CONSTANT' , 'REDIRECT' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1835 , 93 , 'IC_URLCompleted' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1836 , 93 , 'IC_URLError' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1837 , 93 , 'IC_Language' , 'CONSTANT' , 'EN' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1838 , 93 , 'IC_Country' , 'CONSTANT' , 'UK' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1839 , 22 , 'apply3dsecure' , 'CONSTANT' , '1' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1840 , 94 , 'clientAccnum' , 'CONSTANT' , '!Enter your clientAccnum!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1841 , 94 , 'clientSubacc' , 'CONSTANT' , '!Enter your clientSubacc!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1842 , 94 , 'formName' , 'CONSTANT' , '!Enter your formName!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1843 , 94 , 'salt' , 'CONSTANT' , '!Enter your salt!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1844 , 94 , 'payment_url' , 'CONSTANT' , 'https://bill.ccbill.com/jpost/signup.cgi' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1845 , 94 , 'formPrice' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1846 , 94 , 'currencyCode' , 'VARIABLE' , 'currency_value' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1847 , 94 , 'formPeriod' , 'CONSTANT' , '2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1848 , 94 , 'formRecurringPrice' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1849 , 94 , 'formRecurringPeriod' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1850 , 94 , 'formRebills' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1851 , 6 , 'ssl' , 'CONSTANT' , '0' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1852 , 95 , 'payment_url' , 'CONSTANT' , 'https://www.knetpay.com.kw/CGW/servlet/PaymentInitHTTPServlet' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1853 , 95 , 'id' , 'CONSTANT' , '!specify id!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1854 , 95 , 'password' , 'CONSTANT' , '!specify password!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1855 , 95 , 'action' , 'CONSTANT' , '1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1856 , 95 , 'currencycode' , 'VARIABLE' , '414' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1857 , 95 , 'langid' , 'CONSTANT' , 'ENG' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1858 , 95 , 'responseURL' , 'VARIABLE' , 'https://aagtco.com/payments/knet_response.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1859 , 95 , 'errorURL' , 'VARIABLE' , 'https://aagtco.com/order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1860 , 95 , 'trackid' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1861 , 95 , 'amt' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1862 , 95 , 'udf1' , 'VARIABLE' , 'name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1863 , 95 , 'udf2' , 'VARIABLE' , 'daytime_phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1864 , 95 , 'udf3' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1865 , 95 , 'udf4' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1866 , 95 , 'udf5' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1867 , 96 , 'type' , 'CONSTANT' , 'sale' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1868 , 96 , 'username' , 'CONSTANT' , 'Specify your username' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1869 , 96 , 'password' , 'CONSTANT' , 'Specify your password' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1870 , 96 , 'ccnumber' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1871 , 96 , 'ccexp' , 'VARIABLE' , '{cc_expiry_month}{cc_expiry_year}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1872 , 96 , 'amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1873 , 96 , 'cvv' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1874 , 96 , 'payment' , 'CONSTANT' , 'creditcard' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1875 , 96 , 'orderid' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1876 , 96 , 'ipaddress' , 'VARIABLE' , 'user_ip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1877 , 96 , 'firstname' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1878 , 96 , 'lastname' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1879 , 96 , 'company' , 'VARIABLE' , 'company_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1880 , 96 , 'address1' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1881 , 96 , 'address2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1882 , 96 , 'city' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1883 , 96 , 'state' , 'VARIABLE' , 'state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1884 , 96 , 'zip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1885 , 96 , 'country' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1886 , 96 , 'phone' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1887 , 96 , 'fax' , 'VARIABLE' , 'fax' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1888 , 96 , 'email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1889 , 96 , 'shipping_firstname' , 'VARIABLE' , 'delivery_first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1890 , 96 , 'shipping_lastname' , 'VARIABLE' , 'delivery_last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1891 , 96 , 'shipping_company' , 'VARIABLE' , 'delivery_company_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1892 , 96 , 'shipping_address1' , 'VARIABLE' , 'delivery_address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1893 , 96 , 'shipping_address2' , 'VARIABLE' , 'delivery_address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1894 , 96 , 'shipping_city' , 'VARIABLE' , 'delivery_city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1895 , 96 , 'shipping_state' , 'VARIABLE' , 'delivery_state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1896 , 96 , 'shipping_zip' , 'VARIABLE' , 'delivery_zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1897 , 96 , 'shipping_country' , 'VARIABLE' , 'delivery_country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1898 , 96 , 'shipping_email' , 'VARIABLE' , 'delivery_email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1899 , 51 , 'PayPal_URL' , 'CONSTANT' , 'https://www.paypal.com/cgi-bin/webscr?cmd=_express-checkout&token=' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1900 , 91 , 'sepamount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1901 , 91 , 'fname' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1902 , 91 , 'lname' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1903 , 97 , 'billTo_country' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1904 , 97 , 'taxAmount' , 'VARIABLE' , 'tax_cost' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1905 , 97 , 'orderNumber' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1906 , 97 , 'shipTo_street1' , 'VARIABLE' , 'delivery_address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1907 , 97 , 'shipTo_street2' , 'VARIABLE' , 'delivery_address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1908 , 97 , 'shipTo_state' , 'VARIABLE' , 'delivery_state_code_or_province' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1910 , 97 , 'shipTo_postalCode' , 'VARIABLE' , 'delivery_zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1911 , 97 , 'shipTo_phoneNumber' , 'VARIABLE' , 'delivery_phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1912 , 97 , 'shipTo_lastName' , 'VARIABLE' , 'delivery_last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1913 , 97 , 'shipTo_firstName' , 'VARIABLE' , 'delivery_first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1914 , 97 , 'shipTo_country' , 'VARIABLE' , 'delivery_country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1915 , 97 , 'shipTo_company' , 'VARIABLE' , 'delivery_company_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1916 , 97 , 'shipTo_city' , 'VARIABLE' , 'delivery_city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1917 , 97 , 'billTo_street2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1918 , 97 , 'billTo_street1' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1919 , 97 , 'billTo_state' , 'VARIABLE' , 'state_code_or_province' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1920 , 97 , 'billTo_postalCode' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1921 , 97 , 'billTo_phoneNumber' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1922 , 97 , 'merchantID' , 'CONSTANT' , '!Enter here your merchantID here!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1923 , 97 , 'PublicKey' , 'CONSTANT' , '!Enter here your PublicKey!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1924 , 97 , 'orderPage_serialNumber' , 'CONSTANT' , '!Enter here your Serial Number!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1925 , 97 , 'payment_url' , 'CONSTANT' , 'https://orderpage.ic3.com/hop/orderform.jsp' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1926 , 97 , 'orderPage_transactionType' , 'CONSTANT' , 'authorization' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1927 , 97 , 'currency' , 'CONSTANT' , 'USD' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1928 , 97 , 'orderPage_version' , 'CONSTANT' , '7' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1929 , 97 , 'amount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1930 , 97 , 'billTo_firstName' , 'CONSTANT' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1931 , 97 , 'billTo_city' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1932 , 97 , 'billTo_lastName' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1933 , 97 , 'billTo_email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1934 , 97 , 'billTo_company' , 'VARIABLE' , 'company_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1935 , 98 , 'vpc_Version' , 'CONSTANT' , '1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1936 , 98 , 'vpc_Command' , 'CONSTANT' , 'pay' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1937 , 98 , 'vpc_MerchTxnRef' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1938 , 98 , 'vpc_Merchant' , 'CONSTANT' , '!Specify your Merchant ID!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1939 , 98 , 'vpc_OrderInfo' , 'VARIABLE' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1940 , 98 , 'vpc_Amount' , 'VARIABLE' , 'order_total_100' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1941 , 98 , 'vpc_Locale' , 'CONSTANT' , 'en' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1942 , 98 , 'vpc_ReturnURL' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1943 , 98 , 'vpc_AccessCode' , 'CONSTANT' , '!Specify your Access Code!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1944 , 98 , 'SecureHash' , 'CONSTANT' , '!Specify your Secure Hash Secret!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1945 , 54 , 'AMOUNT' , 'VARIABLE' , 'order_total_100' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1946 , 54 , 'CURRENCY' , 'CONSTANT' , 'EUR' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1947 , 54 , 'LANGUAGE' , 'CONSTANT' , 'nl_NL' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1948 , 54 , 'EMAIL' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1949 , 54 , 'ORDERID' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1950 , 54 , 'OWNERADDRESS' , 'VARIABLE' , '{address1} {address2}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1951 , 54 , 'OWNERZIP' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1952 , 54 , 'OWNERTOWN' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1953 , 54 , 'OWNERCTY' , 'VARIABLE' , 'country' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1954 , 54 , 'OWNERTELNO' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1955 , 99 , 'ok_receiver' , 'CONSTANT' , '!your wallet id!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1956 , 99 , 'ok_currency' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1957 , 99 , 'ok_invoice' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1958 , 99 , 'ok_return_success' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1959 , 99 , 'ok_return_fail' , 'VARIABLE' , '{site_url}order_final.php?return_action=fail' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1960 , 99 , 'ok_payer_state' , 'VARIABLE' , 'state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1961 , 99 , 'ok_payer_first_name' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1962 , 99 , 'ok_payer_last_name' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1963 , 99 , 'ok_payer_email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1964 , 99 , 'ok_payer_street' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1965 , 99 , 'ok_payer_country_code' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1966 , 99 , 'ok_payer_zip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1967 , 99 , 'ok_payer_city' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1968 , 99 , 'okpay_account_email' , 'CONSTANT' , '!your okpay account email!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1969 , 54 , 'CANCELURL' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1970 , 100 , 'merchant_id' , 'CONSTANT' , '!Specify your Merchant ID!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1971 , 100 , 'aws_access_key_id' , 'CONSTANT' , '!Specify Access Key ID!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1972 , 100 , 'aws_secret_key_id' , 'CONSTANT' , '!Specify Secret Access Key!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1973 , 100 , 'currency_code' , 'CONSTANT' , 'USD' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1974 , 100 , 'sandbox' , 'CONSTANT' , '0' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1975 , 100 , 'CancelUrl' , 'VARIABLE' , '{site_url}order_final.php?operation=cancel' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1976 , 100 , 'ReturnUrl' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1977 , 101 , 'api_user_id' , 'CONSTANT' , '!payson application id!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1978 , 101 , 'api_pwd' , 'CONSTANT' , '!payson md5 key!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1979 , 101 , 'memo' , 'VARIABLE' , 'Description of items the customer is purchasing.' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1980 , 101 , 'cancelUrl' , 'VARIABLE' , '{site_url}order_final.php?operation=cancel' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1981 , 101 , 'returnUrl' , 'VARIABLE' , '{site_url}order_final.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1982 , 101 , 'forward_url' , 'VARIABLE' , 'https://www.payson.se/paysecure/' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1983 , 101 , 'reciever_email' , 'VARIABLE' , 'your Payson account email' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1984 , 101 , 'ipnNotificationUrl' , 'VARIABLE' , '{site_url}payments/payson115_ipn.php' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1985 , 101 , 'localeCode' , 'VARIABLE' , 'EN' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1986 , 101 , 'currencyCode' , 'VARIABLE' , 'SEK' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1987 , 101 , 'trackingId' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1988 , 101 , 'senderEmail' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1990 , 101 , 'senderFirstName' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1991 , 101 , 'senderLastName' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1992 , 94 , 'va_order_id' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1993 , 94 , 'customer_fname' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1994 , 94 , 'customer_lname' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1995 , 94 , 'address1' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1996 , 94 , 'email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1997 , 94 , 'city' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1998 , 94 , 'state' , 'VARIABLE' , 'state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (1999 , 94 , 'zipcode' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2000 , 94 , 'country' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2001 , 94 , 'phone_number' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2002 , 102 , 'data-amount' , 'VARIABLE' , 'order_total_100' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2003 , 102 , 'data-secret-key' , 'CONSTANT' , '!Your secret key!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2004 , 102 , 'data-key' , 'CONSTANT' , '!You publishable key!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2005 , 102 , 'data-email' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2006 , 102 , 'data-shipping-address' , 'CONSTANT' , 'true' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2007 , 102 , 'data-billing-address' , 'CONSTANT' , 'true' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2008 , 102 , 'data-currency' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2009 , 102 , 'data-description' , 'VARIABLE' , '{total_quantity} product(s) ({order_total})' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2010 , 102 , 'data-name' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2011 , 102 , 'data-image' , 'CONSTANT' , NULL , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2012 , 103 , 'secret_key' , 'CONSTANT' , '!Your secret key!' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2013 , 103 , 'amount' , 'VARIABLE' , 'order_total_100' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2014 , 103 , 'currency' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2015 , 103 , 'description' , 'VARIABLE' , '{total_quantity} product(s) ({order_total})' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2016 , 103 , 'card_number' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2017 , 103 , 'card_exp_month' , 'VARIABLE' , 'cc_expiry_month' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2018 , 103 , 'card_exp_year' , 'VARIABLE' , 'cc_expiry_year' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2019 , 103 , 'card_cvc' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2020 , 103 , 'card_name' , 'VARIABLE' , 'cc_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2021 , 103 , 'address_line1' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2022 , 103 , 'address_line2' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2023 , 103 , 'address_city' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2024 , 103 , 'address_zip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2025 , 103 , 'address_state' , 'VARIABLE' , 'state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2026 , 103 , 'address_country' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2027 , 104 , 'sitereference' , 'CONSTANT' , '!specify sitereference!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2028 , 104 , 'orderreference' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2029 , 104 , 'mainamount' , 'VARIABLE' , 'order_total' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2030 , 104 , 'version' , 'CONSTANT' , '1' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2031 , 104 , 'customertelephone' , 'VARIABLE' , 'delivery_phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2032 , 104 , 'customercountryiso2a' , 'VARIABLE' , 'delivery_country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2033 , 104 , 'customerpostcode' , 'VARIABLE' , 'delivery_zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2034 , 104 , 'customercounty' , 'VARIABLE' , 'delivery_state' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2035 , 104 , 'customertown' , 'VARIABLE' , 'delivery_city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2036 , 104 , 'customerstreet' , 'VARIABLE' , 'delivery_address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2037 , 104 , 'customerpremise' , 'VARIABLE' , 'delivery_address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2038 , 104 , 'currencyiso3a' , 'VARIABLE' , 'currency_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2039 , 104 , 'customeremail' , 'VARIABLE' , 'delivery_email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2040 , 104 , 'customerlastname' , 'VARIABLE' , 'delivery_last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2041 , 104 , 'customerfirstname' , 'VARIABLE' , 'delivery_first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2042 , 104 , 'billingtelephone' , 'VARIABLE' , 'phone' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2043 , 104 , 'billingcountryiso2a' , 'VARIABLE' , 'country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2044 , 104 , 'billingpostcode' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2045 , 104 , 'billingcounty' , 'VARIABLE' , 'state' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2046 , 104 , 'billingtown' , 'VARIABLE' , 'city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2047 , 104 , 'billingstreet' , 'VARIABLE' , 'address2' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2048 , 104 , 'billingpremise' , 'VARIABLE' , 'address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2049 , 104 , 'billingemail' , 'VARIABLE' , 'email' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2050 , 104 , 'billinglastname' , 'VARIABLE' , 'last_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2051 , 104 , 'billingfirstname' , 'VARIABLE' , 'first_name' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2052 , 104 , 'usesitesecurity' , 'CONSTANT' , '1' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2053 , 104 , 'sitesecurity' , 'CONSTANT' , '!specify sitesecurity!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2054 , 104 , 'usenotificationhash' , 'CONSTANT' , '1' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2055 , 104 , 'notificationhash' , 'CONSTANT' , '!specify notificationhash!' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2056 , 104 , 'settlestatus' , 'CONSTANT' , '0' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2057 , 104 , 'settleduedate' , 'CONSTANT' , '0' , 1 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2058 , 23 , 'bill_addr_state' , 'VARIABLE' , 'state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2059 , 23 , 'bill_addr_zip' , 'VARIABLE' , 'zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2060 , 23 , 'ship_addr' , 'VARIABLE' , 'delivery_address1' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2061 , 23 , 'ship_addr_city' , 'VARIABLE' , 'delivery_city' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2062 , 23 , 'ship_addr_country' , 'VARIABLE' , 'delivery_country_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2063 , 23 , 'ship_addr_state' , 'VARIABLE' , 'delivery_state_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2064 , 23 , 'ship_addr_zip' , 'VARIABLE' , 'delivery_zip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2065 , 23 , 'pmt_numb' , 'VARIABLE' , 'cc_number' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2066 , 23 , 'pmt_expiry' , 'VARIABLE' , '{cc_expiry_month}/{cc_expiry_yyyy}' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2067 , 23 , 'pmt_key' , 'VARIABLE' , 'cc_security_code' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2068 , 23 , 'xtl_ip' , 'VARIABLE' , 'user_ip' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2069 , 23 , 'xtl_cust_id' , 'VARIABLE' , 'user_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2070 , 23 , 'xtl_order_id' , 'VARIABLE' , 'order_id' , 0 ) ;
INSERT INTO va_payment_parameters (parameter_id,payment_id,parameter_name,parameter_type,parameter_source,not_passed) VALUES (2071 , 23 , 'request_currency' , 'VARIABLE' , 'currency_code' , 0 ) ;


INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (1 , 1 , 'Personal' , NULL , NULL , 'An example of payment system that does not process payments automatically but may collect credit cards data for offline charging.' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (2 , 1 , 'VeriSign PayFlow Link' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 'https://payflowlink.paypal.com' , NULL , NULL , 0 , NULL , NULL , NULL , 'GET' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (3 , 1 , 'Authorize.Net SIM' , NULL , NULL , 'Please specify values for parameters \'x_login\' and \'x_tran_key\'.\r\n\r\nPlease also set parameter \'x_test_request\' to \'FALSE\' if you are ready to run live transactions or leave it as \'TRUE\' for test your site before going live.\r\n\r\nIf you want receive response from Authorize.net about submiited transactions please also specify \'x_secret\' parameter to the same value as in your Authorize settings.\r\n\r\nTo choose your MD5 Hash Secret in the Authorize.net account please do the following:\r\n- Log into your merchant menu at https://secure.authorize.net/ \r\n- Click "Settings"\r\n- Select "MD5 Hash" under the "Security" portion of your account settings\r\n- Enter the MD5 Hash Secret value that you would like to use\r\n- Confirm the MD5 Hash value\r\n- Click "Submit" to save the changes\r\n\r\nAlso you need specify response URL in your Authorize.net account:\r\n- Select "Response/Receipt URLs" under the "Transaction Response" portion of your account settings\r\n- Click on "Edit" link for "Default Relay Response URL"\r\n- Enter your response URL (for this url you can use following page \'http://www.your_site_url/payments/authorize_confirm.php\')\r\n- Click "Submit" to save the changes' , NULL , NULL , NULL , NULL , 'https://secure.authorize.net/gateway/transact.dll' , NULL , NULL , NULL , NULL , NULL , NULL , 'POST' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (4 , 1 , 'WorldPay' , NULL , NULL , 'Please specify your installation ID for parameter \'instId\'.\r\n\r\nAs well you can specify callbackPW parameter if you set Callback password for your installation on the WorldPay Customer Management System. As the value of this is only known to this and your callback script it can be used as a basic security check within your script.\r\nAlways leave \'Not Passed\' checkbox checked for callbackPW parameter.\r\n\r\nA quote from WorldPay FAQ.\r\n\r\nQ: How can I configure WorldPay to use callback for an installation ID? \r\nA: You will initially need to log in to the WorldPay Customer Management System (via http://www.worldpay.com/admin). In the new window that opens you will need to scroll down to the section headed Installations and select the Configuration options button corresponding to the instId you are using. \r\nThe minimum settings you need to complete are: \r\nCallback URL - This should be set to the complete address to your callback URL, hosted on your server. \r\nCallback enabled? - This determines whether WorldPay will attempt to use the callback URL when needed. \r\nIn order to use callback you must selected at least these two settings. \r\nIf you have developed your callback script to output an HTML response and would like WorldPay to capture this for display to the shopper after the callback has completed successfully, you should also ensure Use callback response? is selected. \r\nTo commit the changes you have made, select SAVE CHANGES at the bottom of the page.\r\n\r\nthe callback URL should have the value like: http://www.yoursite.com/payments/worldpay_confirm.php\r\n\r\nPayment URL: https://secure.worldpay.com/wcc/purchase\r\nTest Payment URL: https://secure-test.worldpay.com/wcc/purchase' , NULL , NULL , NULL , NULL , 'https://secure.worldpay.com/wcc/purchase' , NULL , NULL , NULL , NULL , NULL , NULL , 'POST' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (5 , 1 , '2CheckOut' , NULL , NULL , 'Please specify values for parameters \'sid\' and \'secret_word\'.\r\n\'sid\' - is your vendor number in the 2CO system like 340000, \r\n\'secret_word\' - must be the same as in your 2CO settings.\r\nFor \'Return URL\' use the following address \'http://www.example.com/order_final.php\'.\r\n\r\nTo check the system in test mode please add a new parameter \'demo\' with parameter type Constant and value \'Y\'. Once you finished testing change it to \'N\' or delete it.' , NULL , NULL , NULL , NULL , 'https://www2.2checkout.com/2co/buyer/purchase' , NULL , NULL , NULL , NULL , NULL , NULL , 'POST' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (6 , 1 , 'PayPal Website Payments Standard' , NULL , NULL , 'Please specify your business email address where it says \'your.paypal@example.com\' and specify value for \'at\' (identity token) parameter from your PayPal settings.\r\n------------------------------------------------------------------------------------------------\r\n\r\nActivating Payment Data Transfer\r\nYou must turn on PDT and Auto Return in your PayPal account profile. And you must also acquire a PDT identity token, which is used in all communications sent to PayPal. Please follow these steps:\r\n\r\n1. Log in to your PayPal account. \r\n2. Click the "Profile" sub tab. \r\n3. Go to "My selling tools" section and select "Website Preferences".  \r\n4. Under "Auto Return" for Website Payments, click the "On" radio button. \r\n5. For the "Return URL", enter the URL that will receive the transaction ID posted by PayPal after customer\'s payment as \r\n"http://www.site.com/order_final.php"\r\n6. Under Payment Data Transfer, click the "On" radio button. \r\n7. Click Save. \r\n8. Click "My selling tools" section and select "Website Preferences" again. \r\n9. Scroll down to the Payment Data Transfer section of the page to view your PDT identity token, copy it and paste as the \'at\' parameter value in ViArt Shop admin.\r\n\r\nAlso it\'s recommended to turn on PayPal IPN notifications. For this go to Profile > My selling tools > Instant Payment Notification where turn on IPN setting and set "Notification URL" as "http://www.site.com/payments/paypal_ipn.php"\r\nNote: replace "www.site.com" in the above example with your real domain.\r\n------------------------------------------------------------------------------------------------\r\n\r\nIf you don\'t want to pass data to PayPal in multi currencies please replace the value for "currency_code" parameter in ViArt admin with your preferred currency code like \'USD\'.\r\n------------------------------------------------------------------------------------------------\r\n\r\nFor testing with PayPal SandBox please change \'sandbox\' parameter value to \'1\' and specify \'Payment URL\' as \r\nhttps://www.sandbox.paypal.com/cgi-bin/webscr\r\nand use your sandbox account\r\n\r\nAfter testing with Sandbox please return \'sandbox\' value to \'0\' and specify production \'Payment URL\': \r\nhttps://www.paypal.com/cgi-bin/webscr\r\nand use your live account' , NULL , NULL , NULL , NULL , 'https://www.paypal.com/cgi-bin/webscr' , NULL , NULL , NULL , NULL , NULL , NULL , 'POST' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (7 , 1 , 'Authorize.Net AIM' , NULL , NULL , 'Please specify values for parameters \'x_login\' and \'x_tran_key\' - transaction key obtained from the Merchant Interface.\r\n\r\nFor sending test transaction add x_Test_Request parameter with TRUE value.' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 1 , 'https://secure.authorize.net/gateway/transact.dll' , './payments/authorize_aim.php' , NULL , NULL , NULL , 9 , 6 , 10 , 1 , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (8 , 1 , 'PRI Merchants Advanced' , NULL , NULL , 'Please specify values for parameters \'MerchantID\' and \'RegKey\'.\r\n\r\nFor security purposes you also need to change Payment URL to the absolute secure path, e.g. https://www.example.com/credit_card_info.php' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 1 , 'https://webservices.primerchants.com/billing/TransactionCentral/processCC.asp' , './payments/pri_merchants.php' , NULL , NULL , NULL , 9 , 6 , 10 , 1 , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (9 , 1 , 'YourPay Connect' , NULL , NULL , 'Change the value of the storename field from "!Your store number!" to your merchant store number specified in your Welcome Email.\r\n\r\nIf you are using a test store on the staging server, change the Payment URL to:\r\nhttps://staging.linkpt.net/lpc/servlet/lppay\r\nfor live transaction live:\r\nhttps://secure.linkpt.net/lpcentral/servlet/lppay\r\n\r\nCustomize the payment form in the Customization section in  YourPay Central before you attempt any transactions:\r\n1. In YourPay Central, click Customization on the Main Menu Bar, and then click Settings on the Side Menu Bar to customize YourPay Connect settings.\r\n2. Specify for Submission Form URL your submission page like http://www.example.com/payment.php or https://www.example.com/payment.php but with your actual site url\r\n3. Specify for Confirmation URL and Failure Page your final page like\r\nhttp://www.example.com/order_final.php\r\n4. Checked all available options on this page for Confirmation Page and Failure Page:\r\n[x] URL is a CGI script \r\n[x] Automatically display specified URL after the YourPay HTML receipt \r\n5. Hit the \'Submit\' button to save your changes' , NULL , NULL , NULL , NULL , 'https://secure.linkpt.net/lpcentral/servlet/lppay' , NULL , NULL , NULL , NULL , NULL , NULL , 'POST' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (10 , 1 , 'YourPay API' , NULL , NULL , 'Please specify values for parameters \'configfile\' and \'keyfile\'.\r\n\r\nParameters description:\r\n\r\n\'configfile\' - it is you store number in system, \r\n\r\n\'keyfile\' - local path to your SSL certificate provided by YourPay.\r\n\r\n\'result\' - this field puts the account in live mode or test mode. \r\nSet to LIVE for live mode, GOOD for an approved response in test mode, DECLINE for a declined reponse in test mode, or DUPLICATE for a duplicate response.\r\n\r\nFor security purposes you also need to change Payment URL to the absolute secure path, e.g. https://www.example.com/credit_card_info.php' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 1 , 'https://secure.linkpt.net:1129/LSGSXML' , './payments/linkpoint_api.php' , NULL , NULL , NULL , 4 , 6 , 10 , 1 , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (11 , 1 , 'LinkPoint Connect' , NULL , NULL , 'Change the value of the storename field from "!your store number!" to your merchant store number specified in your Welcome Email.\r\n\r\nIf you are using a test store on the staging server, change the Payment URL to:\r\nhttps://staging.linkpt.net/lpc/servlet/lppay\r\nfor live transaction live:\r\nhttps://www.linkpointcentral.com/lpc/servlet/lppay\r\n\r\nCustomize the payment form in the Customization section in LinkPoint Central before you attempt any transactions:\r\n1. In LinkPoint Central, click Customization on the Main Menu Bar, and then click Settings on the Side Menu Bar to customize LinkPoint Connect settings.\r\n2. Specify for Submission Form URL your submission page like http://www.example.com/payment.php or https://www.example.com/payment.php but with your actual site url\r\n3. Specify for Confirmation URL and Failure Page your final page like\r\nhttp://www.example.com/order_final.php\r\n4. Checked all available options on this page for Confirmation Page and Failure Page:\r\n[x] URL is a CGI script \r\n[x] Automatically display specified URL after the LinkPoint Central HTML receipt \r\n5. Hit the \'Submit\' button to save your changes' , NULL , NULL , NULL , NULL , 'https://www.linkpointcentral.com/lpc/servlet/lppay' , NULL , NULL , NULL , NULL , NULL , NULL , 'POST' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (12 , 1 , 'LinkPoint API' , NULL , NULL , 'Please specify values for parameters \'configfile\' and \'keyfile\'.\r\n\r\nParameters description:\r\n\r\n\'configfile\' - it is you store number in system, \r\n\r\n\'keyfile\' - local path to your SSL certificate provided by Linkpoint.\r\n\r\n\'result\' - this field puts the account in live mode or test mode. \r\nSet to LIVE for live mode, GOOD for an approved response in test mode, DECLINE for a declined reponse in test mode, or DUPLICATE for a duplicate response.\r\n\r\nFor security purposes you also need to change Payment URL to the absolute secure path, e.g. https://www.example.com/credit_card_info.php' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 1 , 'https://secure.linkpt.net:1129/LSGSXML' , './payments/linkpoint_api.php' , NULL , NULL , NULL , 4 , 6 , 10 , 1 , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (14 , 1 , 'Paymate Express' , NULL , NULL , 'Payment URL:\r\n\r\nProduction - https://www.paymate.com/PayMate/ExpressPayment\r\nTesting - https://www.paymate.com/PayMate/TestExpressPayment\r\n\r\n\r\nParameters description:\r\n\r\n\'mid\' - Your Paymate account name\r\n\'amt\' -\tThe payment amount\r\n\'amt_editable\' - Indicate if payment amount is editable or not\r\n\'currency\' - The currency code of the payment (\'AUD\', \'USD\',\'EUR\',\'GBP\',\'NZD\')\r\n\'ref\' - A seller reference number for a transaction\r\n\'pmt_sender_email\' - The buyer\'s email address\r\n\'pmt_contact_firstname\' - The buyer\'s first name\r\n\'pmt_contact_surname\' - The buyer\'s surname\r\n\'pmt_contact_phone\' - The buyer\'s phone number\r\n\'pmt_country\' - The buyer\'s country.\r\n\'regindi_address1\' - The first address line of the buyer\'s address\r\n\'regindi_address2\' - The second address line of the buyer\'s address\r\n\'regindi_sub\' - The suburb of the buyer\'s address\r\n\'regindi_pcode\' - The postcode of the buyer\'s address\r\n\'return\' - A fully-qualified (eg: http://www.example.com/order_final.php) URL to which Paymate will redirect the buyer. This redirect will occur when they click the continue button on the receipt page of the Express Payment wizard.\r\n\'popup\' - Indicate if your web site has opened Paymate Express in a new window. Must be either "true" or "false", default is "true". If this value is true, the Paymate Express window will close before displaying the \'return\' URL in the browser window which opened Paymate Express. If this value is false, the \'return\' URL will be displayed in the same window as Paymate Express.\r\n\r\n\r\nThis will allow you to process test payments to a dummy account.\r\n\r\n    * mid: testuser\r\n    * The card number : 4564456445644564 will always return an approved transaction.\r\n    * The card number : 5424000000000015 will always return an declined transaction.\r\n    * Other valid card number will always return that the payment is processing.\r\n' , NULL , NULL , NULL , NULL , 'https://www.paymate.com/PayMate/ExpressPayment' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (15 , 1 , 'Sage Pay Form' , NULL , NULL , 'Please specify your Vendor name and your EncryptionPassword. \r\nAs well you can change currency_code parameter to some default value like \'USD\'.\r\n\r\n-- Simulator site\r\nhttps://test.sagepay.com/Simulator/VSPFormGateway.asp\r\n--  Test site\r\nhttps://test.sagepay.com/gateway/service/vspform-register.vsp\r\n-- Live site URL\r\nhttps://live.sagepay.com/gateway/service/vspform-register.vsp' , NULL , NULL , NULL , NULL , './payments/sagepay_form_checkout.php' , NULL , NULL , 0 , NULL , NULL , NULL , 'GET' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (16 , 1 , 'Sage Pay Direct' , NULL , NULL , 'Please specify your Vendor name. \r\nAs well you can change currency_code parameter to some default value like \'USD\'.\r\n\r\nFor security purposes you also need to change Payment URL to the absolute secure path, e.g. https://www.example.com/credit_card_info.php\r\n\r\nAdvanced URL:\r\n-- Simulator site\r\nhttps://test.sagepay.com/Simulator/VSPDirectGateway.asp\r\n-- Test site\r\nhttps://test.sagepay.com/gateway/service/vspdirect-register.vsp\r\n-- Live site URL\r\nhttps://live.sagepay.com/gateway/service/vspdirect-register.vsp\r\n\r\nCallbackURL:\r\n-- Simulator site\r\nhttps://test.sagepay.com/Simulator/VSPDirectCallback.asp\r\n-- Test site\r\nhttps://test.sagepay.com/gateway/service/direct3dcallback.vsp\r\n-- Live site URL\r\nhttps://live.sagepay.com/gateway/service/direct3dcallback.vsp' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 1 , 'https://live.sagepay.com/gateway/service/vspdirect-register.vsp' , './payments/protx_direct.php' , NULL , NULL , NULL , 4 , 6 , 10 , 1 , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (17 , 1 , 'eGold SCI' , NULL , NULL , 'Please specify your account number, name and passpharse in fields below. \r\nIf you want to change currency please use PAYMENT_UNITS parameter (possible values are 1 - USD, 2 - CAD, 85 - EUR, 81 - JPY).' , NULL , NULL , NULL , NULL , 'https://www.e-gold.com/sci_asp/payments.asp' , NULL , NULL , NULL , NULL , NULL , NULL , 'POST' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (18 , 1 , 'Beanstream API' , NULL , NULL , 'Please specify your merchant_id.\r\nFor security purposes you also need to change Payment URL to the absolute secure path, e.g. https://www.example.com/credit_card_info.php' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 1 , 'https://www.beanstream.com/scripts/process_transaction.asp' , './payments/beanstream_api.php' , NULL , NULL , NULL , 4 , 6 , 10 , 1 , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (19 , 1 , 'Payoffshore API' , NULL , NULL , 'Please specify your merchantid.\r\nIf you want test system before go live you can change \'test\' parameter value to \'true\'.\r\nFor security purposes you also need to change Payment URL to the absolute secure path, e.g. https://www.example.com/credit_card_info.php' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 1 , 'https://secure.payoffshore.com/ecom/api.xml' , './payments/payoffshore_api.php' , NULL , NULL , NULL , 4 , 6 , 10 , 1 , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (20 , 1 , 'iDEAL Advanced for Ing Bank' , NULL , NULL , 'Please specify your MerchanID and your certificates information.\r\nIf you will using only one Issuer you can specify it for IssuerID parameter as well.\r\n\r\nLive site: ssl://ideal.secure-ing.com:443/ideal/iDeal\r\nTest site: ssl://idealtest.secure-ing.com:443/ideal/iDeal' , NULL , NULL , NULL , NULL , './payments/ideal_process.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (21 , 1 , 'PayPal Express Checkout' , NULL , NULL , 'Before proceed with this method you need to request an API Certificate withing your PayPal account from \'My Account > Profile > API Access\'.\r\n\r\nOn \'API Access\' page click the link \'Request API Credentials\' and on the new page select \'API SSL client-side certificate\' option and tick off the checkbox to agree to PayPal terms. On the next page you will see API Username and Password details for your account and here you can download your certificate by pressing \'Download Certificate\' button, this SSL certificate should be uploaded to your web server.\r\n\r\nThen please specify you API information like \'Username\', \'Password\' and \'SSLCert\' in the fields below.\r\n\r\nIf you don\'t want pass data to PayPal in multi currencies please specify for CurrencyID parameter preferred currency code like \'USD\'.\r\n\r\nFor testing with PayPal SandBox please change \'sandbox\' parameter value to \'1\' and specify your developer account user name, password and API certificate. For testing with your real account the sandbox must always be set to 0 to avoid authentication errors.\r\n\r\nIn case you are intend to use PayPal IPN, you will need to specify your business email address for \'business\' parameter.' , NULL , NULL , NULL , NULL , 'payments/paypal_checkout.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 1 , 'https://www.paypal.com/en_US/i/btn/btn_xpressCheckout.gif' , 145 , 42 , 'PayPal is the safer, easier way to pay.' , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (22 , 1 , 'PayPal Website Payments Pro' , NULL , NULL , 'Before proceeding with this method or \'PayPal Express Checkout\' you need to request an API Certificate within your PayPal account at \'My Account > Profile > API Access\'.\r\n\r\nOn \'API Access\' page click the link \'Request API Credentials\' and on the next page select option \'API SSL client-side certificate\' and tick off the checkbox to agree with PayPal terms. Next you will see API Username and Password details for your account and here you can download your certificate by clicking on \'Download Certificate\' button - this SSL certificate should be uploaded to your web server.\r\n\r\nAfterwords you need to click \'Submit Application\' from \'Merchant Tools > Business Solutions > Website Payment Pro\'. (Works only for USA and UK customers).\r\n\r\nWhen you have finished all the steps above please specify your API information i.e. \'Username\', \'Password\' and \'SSLCert\' in the fields below.\r\n\r\nPlease note that Direct Payment API is not a standalone product. You are required to use Direct Payment API and Express Checkout together as part of the Website Payments Pro solution according to PayPal documentation.\r\n\r\nFor testing with PayPal SandBox change \'sandbox\' parameter value from \'0\' to \'1\'.\r\n\r\nIn case you intend to use PayPal IPN, you will need to specify your business email address in \'business\' parameter.\r\n\r\nTo use 3d secure check (required for UK merchants to process Maestro credit cards) please fill in these parameters:\r\nProcessorId - the Processor ID you received in the email notification\r\nMerchantId - the Merchant ID you received in the email notification\r\nTransactionPwd - the TransactionPwd you have created in your Profile section.\r\n\r\nTo create a TransactionPwd please do the following steps:\r\nLogin into that profile <https://centineltest.cardinalcommerce.com>\r\n- Select Manage Profile\r\n- Select Update Profile information\r\n- Create a password in the field provided. \r\n- Hit \'Enter\' to save your settings.\r\n\r\nFor test mode use:\r\nMAPS_URL - https://centineltest.cardinalcommerce.com/maps/txns.asp\r\n\r\nfor live mode:\r\nMAPS_URL - https://paypal.cardinalcommerce.com/maps/txns.asp\r\n\r\nIf you are from USA or any other country and don\'t want to use 3d secure check please change \'apply3dsecure\' parameter value from \'1\' to \'0\'.\r\n\r\nIf you don\'t want to pass data to PayPal in multicurrencies, please specify for \'CurrencyID\' parameter preferred currency code like \'USD\', \'EUR\'.\r\nNote: if you use 3d secure check please specify for \'CurrencyCode\' parameter preferred currency iso code like \'840\', \'978\'.' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 1 , 'https://api-aa.paypal.com/2.0/' , './payments/paypal_direct.php' , NULL , NULL , NULL , 4 , 6 , 10 , 1 , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (23 , 1 , 'Argus Payments' , NULL , NULL , 'Please specify all parameters starting with exclamation mark - !' , NULL , NULL , NULL , NULL , 'order_confirmation.php' , NULL , NULL , 0 , NULL , NULL , NULL , 'GET' , 0 , 1 , 'https://svc.arguspayments.com/payment/pmt_service.cfm' , './payments/argus_payment.php' , NULL , NULL , NULL , 9 , 6 , 10 , 1 , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (24 , 1 , 'ePDQ CPI' , NULL , NULL , 'Please specify values for parameters \'clientid\', \'password\' and \'merchantdisplayname\'.\r\nPlease use for the \'password\' parameter the same value as you specified it for \'Pass-phrase\' field in your ePDQ settings.\r\n\r\nWithin the CPI account, define the \'Allowed URL\' as \'http://www.your_site.com/payment.php\'\r\n\r\nTo receive order results please specify in your ePDQ settings \'POST URL\' as \'http://www.your_site.com/payments/epqd_cpi_response.php\' and set parameters \'POST Order result\' to \'Yes\'.\r\nPlease note that payment system can\'t callback to an https address.\r\n\r\nIf you don\'t want pass data to ePDQ in multi currencies please specify for currencycode parameter preferred currency value like 840 for USD, 826 for GBP.' , NULL , NULL , NULL , NULL , 'https://secure2.epdq.co.uk/cgi-bin/CcxBarclaysEpdq.e' , NULL , NULL , NULL , NULL , NULL , NULL , 'POST' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (25 , 1 , 'Garanti' , NULL , NULL , 'Please specify the following parameters:\r\n\r\nClientId - Shop Code sent you via e-mail\r\nName - Payment pages user id\r\nPassword - User password\r\nCurrency - Currency code' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 1 , 'https://ccpos.garanti.com.tr/servlet/cc5ApiServer' , './payments/garanti.php' , NULL , NULL , NULL , 4 , 6 , 10 , 1 , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (26 , 1 , 'VXSBill.com' , NULL , NULL , 'Please specify values for the following parameters:\r\n \r\nusername - isashop\r\npassword - Your password\r\nsite - Site ID, integer value\r\npackage - Package ID, integer value\r\n \r\nThese values are sent to you by email once you\'ve registered at vxsbill.com\r\n \r\nTo update the status please run the below script in the browser\'s bar:\r\n\'http://www.example.com/payments/vxsbill_confirm.php\'\r\n \r\nTo set the script by cron please use this command:\r\n \r\n0 * * * * /path/to/php -f /home/rg1/public_html/payments/vxsbill_confirm.php -l ted -p readfast\r\n \r\nwhere 0 * * * *  updates the status each hour\r\n \r\nand -l ted -p readfast are the login/password for an administrator.' , NULL , NULL , NULL , NULL , './payments/vxsbill_process.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , 4 , 6 , 10 , 0 , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (27 , 1 , 'ePDQ MPI' , NULL , NULL , 'Please specify values for parameters \'UserName\', \'Password\' and \'ClientId\'.\r\nAs well you can change currency to some default value\r\n840 - US Dollar\r\n826 - British Pound\r\n\r\nIf you want to enable 3D Authentication service please compile epdq_3d_check.java, epdq_3d_verify.java libraries and specify the following parameters in the parameters list below. Please note, to run 3D service your server need to has support of JDK 1.3 or greater.\r\n\r\n3DAuthentication - 1\r\nJavaClasspath - path to Merchant Service Provider classes like \'.;j_api\\XFMS_Java_API.jar;j_api\\jcert.jar;j_api\\jnet.jar;j_api\\jsse.jar\'\r\n\r\nFor Visa checks you need specify following parameters: VISAMerchantID, VISADSPassword, VISAAcquirerBIN\r\n\r\nFor MasterCard you need specify following parameters: MCMerchantID, MCAcquirerBIN, MerchantName' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 1 , 'https://secure2.epdq.co.uk:11500/' , './payments/epdq_mpi.php' , NULL , NULL , NULL , 4 , 6 , 10 , 1 , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (28 , 1 , 'ProxyPay' , NULL , NULL , 'Please specify the following parameters:\r\n\r\nmerchantID - your merchant ID' , NULL , NULL , NULL , NULL , 'https://eptest.eurocommerce.gr/proxypay/apacs' , NULL , NULL , NULL , NULL , NULL , NULL , 'POST' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (29 , 1 , 'Posnet' , NULL , NULL , 'Please specify the following parameters:\r\nusername - your username\r\npassword - your password\r\nmid - your mid parameter\r\ntid - your tid parameter\r\n\r\nPosnet XML service lets posnet merchants send posnet transactions by sending XML documents. Merchants send XML documents to a URL like https://www.posnet.ykb.com/PosnetWebService/XML?xmldata=XML \r\n(for test environment: \r\nhttp://setmpos.ykb.com/PosnetWebService/XML?xmldata=XML). \r\nResponse is returned to the merchant also as an XML document.\r\n\r\nDon\'t forget to enter your account details in appropriate fields: username, password, mid, tid.\r\n\r\nProduction URL:\r\nhttps://www.posnet.ykb.com/PosnetWebService/XML\r\n\r\ntesting URL:\r\nhttp://setmpos.ykb.com/PosnetWebService/XML\r\n\r\nSite for more information https://www.posnet.ykb.com' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 1 , 'http://setmpos.ykb.com/PosnetWebService/XML' , './payments/posnet.php' , NULL , NULL , NULL , 4 , 6 , 10 , 1 , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (30 , 1 , 'Akbank' , NULL , NULL , 'Please specify the following parameters:\r\n\r\nClientId - Shop Code sent you via e-mail\r\nName - Payment pages user id\r\nPassword - User password\r\nCurrency - Currency code' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 1 , 'https://vpos.est.com.tr/servlet/cc5ApiServer' , './payments/akbank.php' , NULL , NULL , NULL , 4 , 6 , 10 , 1 , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (31 , 1 , 'Netbilling' , NULL , NULL , 'Please specify this parameter:\r\naccount_id - set it to your account ID\r\n\r\nInsecure testing URL:\r\nhttp://secure.netbilling.com:1401/gw/sas/direct3.1\r\n\r\nSecure production URL:\r\nhttps://secure.netbilling.com:1402/gw/sas/direct3.1' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 1 , 'http://secure.netbilling.com:1401/gw/sas/direct3.1' , './payments/netbilling.php' , NULL , NULL , NULL , 4 , 6 , 10 , 1 , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (32 , 1 , 'Chronopay' , NULL , NULL , 'Please specify values for the following parameters:\r\n\r\n- product_id\r\n- product_name\r\n- cb_url = "{your_site_url}payments/chronopay_confirm.php"\r\n\r\non Chronopay in \'Check Product Info\'\r\n(https://clients.chronopay.com/?cat=product&act=view&id=!Enter your product_id!)\r\nplease set \'Access URL\' like \'http://www.your_site.com/order_final.php\'\r\n' , NULL , NULL , NULL , NULL , 'https://secure.chronopay.com/index_shop.cgi' , NULL , NULL , NULL , NULL , NULL , NULL , 'POST' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (33 , 1 , 'Multipay' , NULL , NULL , 'Parameters description:\r\n\r\nmpSeller_ID - Fill in your Seller ID\r\nmpDescription - some description\r\nmpItems - some order info\r\nmpAdministration_ID - Free admin id for yourself\r\nmpVar1 - Free (session id or something else)\r\nmpVar2 - Free (session id or something else)\r\nmpAllow V = Visa\r\n              E = Mastercard\r\n              T = Teletik safepay\r\n              O = Overboeking\r\n              I = Incasso\r\n              C = Acceptgiro\r\n              R = Rabodirect\r\n              L = iDeal\r\nmpCountry - NL, US ..\r\nmpCurrency - NLG = Nederlandse Gulden\r\n                      USD = US Dollar\r\n                      GBP = United Kingdom Pound\r\n                      EUR = Euro' , NULL , NULL , NULL , NULL , 'https://multipay.net/transaction/mpmain.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'POST' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (34 , 1 , 'Google Checkout API' , NULL , NULL , 'Parameters description:\r\n\r\n\'action_url\' must be\r\nFor testing purposes: https://sandbox.google.com/checkout/cws/v2/Merchant/[[Your-Merchant-ID]]/checkout\r\nFor production site: https://checkout.google.com/cws/v2/Merchant/[[Your-Merchant-ID]]/checkout\r\nProduction URL example: https://checkout.google.com/cws/v2/Merchant/1234567890/checkout\r\n\r\nFor full Integration you must go to \'Settings, Profile\' in your Google Checkout account\r\nTesting account\r\nhttps://sandbox.google.com/checkout/sell/settings?section=Profile\r\nor live one\r\nhttps://checkout.google.com/sell/settings?section=Profile\r\nand set in field \'Public business website\' your URL to order_final.php \r\nFor example: http://www.your_site.com/order_final.php.\r\n\r\nAlso you have to go to \'Settings, Integration\' and set in field \'API callback URL\' your URL to google_checkout.php\r\nFor example: http://www.your_site.com/payments/google_checkout.php\r\nand set \'Callback method\' to \'XML\'.\r\n\r\nTo receive status \'Charged\' for ViArt Shop orders, it is necessary to perform the following step:\r\n1) Specify [./payments/google_charge.php] in the \'Library to Capture Order\';\r\n2) Add the request URL parameter \'request_url\' in the Parameters List.\r\nFor a test account use - https://sandbox.google.com/checkout/api/checkout/v2/request/Merchant/!Your merchant_id!\r\nFor a live account use - https://checkout.google.com/api/checkout/v2/request/Merchant/!Your merchant_id!' , NULL , NULL , NULL , NULL , './payments/google_process.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , 4 , 6 , 10 , 1 , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 1 , 'http://checkout.google.com/buttons/checkout.gif?merchant_id={merchant_id}&w=160&h=43&style=white&variant=text&loc=en_US' , 160 , 43 , 'Google Checkout' , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (35 , 1 , 'Akbank 3D secure' , NULL , NULL , 'Parameters description.\r\n\r\nPlease fill in filed \'Clientid\' by th Client ID issued for you by Akbank 3D secure system.' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 1 , 'https://vpos.est.com.tr/servlet/ThreeDGate' , './payments/akbank_3d.php' , NULL , NULL , NULL , 4 , 6 , 10 , 1 , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (36 , 1 , 'Sage Pay Server' , NULL , NULL , 'TxType - \'PAYMENT\', \'DEFERRED\' or \'PREAUTH\'\r\nCurrency - \'GBP\', \'EUR\' and \'USD\'\r\n\r\nAdvanced URL -\r\nsimulator site: https://test.sagepay.com/Simulator/VSPServerGateway.asp?Service=VendorRegisterTx\r\ntest site: https://test.sagepay.com/gateway/service/vspserver-register.vsp\r\nlive site: https://live.sagepay.com/gateway/service/vspserver-register.vsp\r\n\r\nAllowGiftAid:\r\n0 = No Gift Aid Box displayed (default)\r\n1 = Display Gift Aid Box on payment screen.\r\n\r\nApplyAVSCV2:\r\n0 = If AVC/CV2 enabled then check them. If rules apply, use rules. (default)\r\n1 = Force NO AVC/CV2 checks even if not enabled for the account. If rules apply, use rules.\r\n2 = Force NO AVC/CV2 checks even if enabled on account.\r\n3 = Force NO AVC/CV2 checks even if not enabled for the account but DON\'T apply any rules.\r\n\r\nApply3DSecure:\r\n0 = If 3D-Secure checks are possible and rules allow, perform the checks and apply the authorisation rules. (default) \r\n1 = Force 3D-Secure checks for this transaction if possible and apply rules for authorisation.\r\n2 = Do not perform 3D-Secure checks for this transaction and always authorise.\r\n3 = Forse 3D-Secure checks for this transaction if possible but ALWAYS obtain an auth code, irrespective of rule base.\r\n\r\nTo use a release library, please add the following path ./payments/protx_server_release.php\' in the field \'Library to Capture Order\'. Further, insert a parameter \'releaseURL\' with the value\'https://live.sagepay.com/gateway/service/release.vsp\' and select\'Notpassed\' option. In addition, it is necessary tochange the value of the \'TxType\' to \'DEFERRED\'.\r\n\r\nRelease url:\r\nsimulator site: https://test.sagepay.com/Simulator/VSPServerGateway.asp?Service=VendorReleaseTx\r\ntest site: https://test.sagepay.com/gateway/service/release.vsp\r\nlive site: https://live.sagepay.com/gateway/service/release.vsp' , NULL , NULL , NULL , NULL , './payments/protx_server_process.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 0 , 'https://live.sagepay.com/gateway/service/vspserver-register.vsp' , NULL , NULL , NULL , NULL , 4 , 6 , 10 , 1 , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (37 , 1 , 'VCS' , NULL , NULL , 'Parameters description\r\n\r\np1 - VCS Terminal ID allocated by VCS.\r\np2 - Unique Transaction Reference Number with no spaces, generated by you. This reference number MUST ONLY BE USED ONCE. If for some reason you do not get a response, then re-try the transaction with the same reference number, our system will return the response that we got from the bank the first time the transaction was presented. We will not re-try the authorisation request if it has already been presented to the bank and we have received a response. We will merely return the response we already have. If you did get a response but it was declined and now you wish to try again then you must change the reference number. Refer p2 logic diagram.\r\np3 - Description of Goods.\r\np4 - Transaction Amount.\r\np5 - Currency, ZAR is the only valid currency.\r\np6 - Occur Count, 1  99 or U for an unlimited number of occurrences. Include only if transaction should re-occur.\r\np7 - Occur Frequency. Include only if transaction should re-occur. \r\nD Daily \r\nW Weekly \r\nM Monthly \r\nQ Quarterly (3-monthly) \r\n6 Bi-annually (6-monthly)  \r\nY Annually\r\np8 - Cell phone Number for SMS message. We can send a message to a cell phone on any network. Request registration from support@vcs.co.za. Include only if registered.\r\np9 -Message for SMS. This message will be sent to the above cell phone. We will add the response we get from the bank after this message. Include only if registered.\r\np10 - URL for Cancelled Transactions. If your customer gets to our credit card details entry page and then presses the cancel button we will return him to this URL. If this URL is omitted the customer will be returned to the previous page by default.\r\np11 - Occurrence E-mail Address. If you include an e-mail address VCS will send a transaction receipt to the cardholder after each occurrence. Include only if transaction should re-occur.\r\np12- Delayed Settlement. \r\nY Settlement delayed until manually released by merchant.\r\nN Settle today - automatic process. The default option.\r\np13 - Occur Amount, with a decimal point. If you do not include the decimal point we will assume one at the end of the amount, i.e. 10 is 10.00. If this amount is omitted then the actual amount will be used by default. Include only if the transaction should re-occur.\r\nBudget - Budget Allowed. \r\nY Budget is allowed. The default option.\r\nN Budget is not allowed.\r\nNextOccurDate - Next Occurrence Date. This is the date that the next occurrence of this transaction will be presented irrespective of what the occurrence frequency is. This date must be greater than the date that the transaction is presented. Format: ccyy/mm/dd Include only if transaction should re-occur.\r\npam - Personal Authentication Message, a security feature to confirm that the response is from VCS. The merchant enters the Merchant PAM in his Merchant Settings and VCS returns that PAM with the response.\r\n\r\nHow to submit the approved & declined URLs, call-back URLs and PAM:\r\n1. Go to https://www.vcs.co.za > Admin Login > Virtual Terminal\r\n2. Enter the user login name and password\r\n3. Click the Merchant Administration link\r\n4. Select 3. VCS Interfacing (page1)\r\n4.1. Enter merchants website as http://www.your_site.com/\r\n, approved & declined URLs as http://www.your_site.com/order_final.php\r\n4.2. Select the Http method \'POST\'.\r\n5. Select 4. VCS Interfacing (page2)\r\n5.1. Enter Merchant PAM\r\n6. Select 6. Callback Settings\r\n6.1. Select the Do Auth CallBack \'Yes\'\r\n6.2. Enter approved & declined URLs as http://www.your_site.com/payments/vcs_response.php\r\n6.3. Select the CallBack Protocol (HTTP or HTTPS)\r\n6.4. Select the CallBack Method \'POST\'\r\n6.5. Select the Response Format \'Name Value Pairs (ccForm - p1. p2..)\'\r\n\r\nFor the statuses to be updated properly we stronglyrecommend to turn on VCS Call-back option.' , NULL , NULL , NULL , NULL , 'https://www.vcs.co.za/vvonline/ccform.asp' , NULL , NULL , NULL , NULL , NULL , NULL , 'POST' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , 4 , 6 , 10 , NULL , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (38 , 1 , 'Cybersource SOP' , NULL , NULL , 'Setting up your Cybersource Account:\r\n\r\nSecurity settings:\r\n1. First of all after logging into Cybersource Business Center visit menu Tools & Settings > Hosted Order Page > Security.\r\n2. In section \'Generate Security Script\' select option \'PHP\' and press button \'Submit\'. You\'ll get a proposal to download a security script HOP.php with your Public and Private Keys.\r\n3. Open file HOP.php and find function getPublicKey(). Copy all text between \' return " \' and \' " \' into the parameter PublicKey.\r\n4. Find function getPrivateKey(). Copy all text between \' return " \' and \' " \' into the parameter PrivateKey.\r\n5. Copy your Serial Number from section \'Existing Security Keys\' on the page Tools & Settings >  Hosted Order Page > Security and paste it into parameter orderPage_serialNumber.\r\n\r\nCommon settings: \r\n1. Open menu Tools & Settings > Hosted Order Page > Settings.\r\n2. In section Data Entry Fields > Payment Types select \'Credit Card\' and check all preferred types.\r\n3. In section Appearance > Receipt Page check option \'This URL is my custom receipt page.\' and fill in \'Receipt Response URL\' as http://www.example.com/order_final.php\r\n4. In section Appearance > Decline Page check option \'This URL is my custom decline page.\' and fill in \'Decline Response URL\' as http://www.example.com/order_final.php\r\n5. In section Notifications > Addresses and Options we recommend to check option \'Merchant POST URL\' and set it to http://www.example.com/payments/cybersource_notify.php\r\n\r\n------------------------------------------------------------------------------------------------------\r\nFill in the following parameters:\r\n\r\nmerchantID - Cybersource Merchant ID\r\nPublicKey - Public Key that can be acquired from file HOP.php\r\nPrivateKey - Private Key that can be acquired from file HOP.php\r\norderPage_serialNumber - Serial Number that can be acquired from file HOP.php or from Tools & Settings > Hosted Order Page > Security page in Business Center.\r\n\r\n------------------------------------------------------------------------------------------------------\r\nAdvanced URL:\r\n\r\nDebug Test - https://orderpagetest.ic3.com/hop/CheckOrderData.do\r\nTest - https://orderpagetest.ic3.com/hop/ProcessOrder.do\r\nDebug Production - https://orderpage.ic3.com/hop/CheckOrderData.do\r\nProduction - https://orderpage.ic3.com/hop/ProcessOrder.do' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'POST' , 0 , 1 , 'https://orderpage.ic3.com/hop/ProcessOrder.do' , './payments/cybersource_process.php' , NULL , NULL , NULL , 4 , 6 , 10 , 1 , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (39 , 1 , 'KORTA in Iceland' , NULL , NULL , 'Please specify values for parameters \'merchant\', \'terminal\' and \'secretcode\'.\r\n\r\nFor testing purposes use the following parameter values:\r\n1. korta_payment_url = https://netgreidslur.korta.is/testing/\r\n2. is_test = 1\r\n\r\nFor real transactions use the following parameter values:\r\n1. korta_payment_url = https://netgreidslur.korta.is/\r\n2. is_test = 0\r\n\r\nPlease, note, that the following fields on the checkout page should be mandatory:\r\n- phone (Or you can use \'daytime_phone\' or \'evening_phone\' as the value of the \'phone\' parameter. In this case, \'daytime phone\' or \'evening phone\' field should be mandatory).\r\n- email\r\n- name\r\n- address1\r\n- zip\r\n- city\r\n- country' , NULL , NULL , NULL , NULL , 'https://netgreidslur.korta.is/' , NULL , NULL , NULL , NULL , NULL , NULL , 'POST' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (40 , 1 , 'eCommerceConnect' , NULL , NULL , 'Please specify values for the following parameters:\r\n- action_url\r\nlive - https://secure.upc.ua/go/enter\r\ntest - https://secure.upc.ua/ecgtest/enter\r\n- MerchantID\r\n- TerminalID\r\n- Privatekey\r\n- PublicServerCert\r\n\r\nOn eCommerceConnect site (https://secure.upc.ua/ecgtest/merchant)\r\nplease set \'Return URL\' like \'http://www.your_site.com/order_final.php\'\r\nplease set \'Notify URL\' like \'http://www.your_site.com/payments/upc_confirm.php\'' , NULL , NULL , NULL , NULL , './payments/upc_process.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , 4 , NULL , 10 , NULL , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (41 , 1 , 'Cashu' , NULL , NULL , 'Please specify values for the following parameters:\r\n- merchant_id\r\n- keyword\r\n\r\nOn your Cashu merchant account (https://www.cashu.com/Merchants/index?lang=en) in \'Encryption Information\'\r\nPlease set \'thnx_url\' like \'http://www.your_site.com/payments/cashu_confirm.php\'\r\n\r\nUse \'servicesName\' if you use multiple services.\r\nAnd on your Cashu merchant account in \'Encryption Information\' please set \'thanks_url\' like \'http://www.your_site.com/payments/cashu_confirm.php\'\r\n\r\n\'email\' - Email address for the user if this is needed by the merchant.\r\n\r\n\'test_mode\' - This parameter is used only for testing purposes.\r\n\r\nParameters \'action_url\', \'keyword\' and \'response_ip\' always is \'Not Passed\'!' , NULL , NULL , NULL , NULL , './payments/cashu_process.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (42 , 1 , 'ECHO' , NULL , NULL , 'Please note: The system supports only USD currency.\r\n\r\nFill in the following parameters:\r\n\r\nmerchant_echo_id - ECHO Merchant ID\r\nmerchant_pin -  ECHO Merchant PIN\r\ncounter - how many times the same transaction can be submitted (once in 8 hours), default value: 1\r\ntest_mode - maybe 1 or 0, accordingly turns on/off Echo test mode' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'POST' , 0 , 1 , 'https://wwws.echo-inc.com/scripts/INR200.EXE' , './payments/echo_process.php' , NULL , NULL , NULL , 4 , 6 , 10 , 1 , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (43 , 1 , 'Moneybookers e-wallet' , NULL , NULL , 'Required fields:\r\n\r\npay_to_email - Email address of the merchants moneybookers.com account\r\n\r\nsecret_word (required for moneybookers_status.php)\r\nThe secret word MUST be submitted in the Merchant Tools section in lowercase before the md5sig can be used. If you insert any uppercase symbols, they will automatically be converted to lower case. The only restriction on your secret word is the length which must not exceed 10 characters. Non-alphanumeric symbols can be used. If the Merchant Tools section is not shown in your account, please contact merchantservices@moneybookers.com\r\n------------------------------------------------------------------------------------------------------\r\nOther fields:\r\n\r\nconfirmation_note\r\nMerchant may show to the customer on the confirmation screen - the end step of the\r\nprocess - a note, confirmation number, PIN or any other message. Line breaks <br> may be used for longer messages.\r\n\r\nreturn_url\r\nURL to which the customer will be returned when the payment is made. If this field is not filled, the merchant gateway window will simply close automatically at the end of the transaction, so that the customer will be returned to the last page on the merchant\'s website where he has been before. A secure return_url functionality is available.\r\n\r\ncancel_url\r\nURL to which the customer will be returned if the payment process is cancelled. If this field is not filled, the merchant gateway window will simply close automatically upon clicking the cancellation button, so the customer will be\r\nreturned to the last page on the merchant\'s website where the customer has been before.\r\n\r\nstatus_url\r\nURL to which the transaction details will posted after the payment process is complete.\r\nAlternatively, you may specify an email to which you would like to receive the If the status_url is omitted, no transaction details will be sent to the merchant.\r\nCould be email like  EMAIL_TO_RECEIVE_PAYMENT_NOTIFICATION@MERCHANT.COM\r\nrecipient_description\r\nMerchant Name this replaces merchant email address on the gateway\r\n\r\nlogo_url\r\nprovides a co-branded gateway ideally the image should be stored on a secure server to avoid a security warning which comes up on some browsers\r\n\r\nhide_login\r\nthis hides the Moneybookers login screen on the right hand side, should be done for all options apart from e-wallet\r\n\r\npayment_methods\r\nACC - for credit & debit cards\r\nWLT - for the e-wallet\r\nread more information in documentation' , NULL , NULL , NULL , NULL , 'https://www.moneybookers.com/app/payment.pl' , NULL , NULL , NULL , NULL , NULL , NULL , 'POST' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , 4 , 6 , 10 , NULL , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , 'http://www.moneybookers.com/images/logos/checkout_logos/checkout_120x40px.gif' , NULL , 'http://www.moneybookers.com/images/logos/checkout_logos/checkout_180x60px.gif' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (44 , 1 , 'SIPS' , NULL , NULL , 'This payment system requires executable files and certificates you could retrieve from Webaffaires. Also, it is highly recommended to grant appropriate executable permissions to these binaries.\r\n\r\n------------------------------------------------------------------------------------------------------\r\nFill in the following parameters:\r\n\r\nmerchant_id - Your SIPS Merchant ID\r\npathfile         - a path to the file with the required parameters for sips executable script\r\n\tWindows   C:\\webaffaires\\payment\\param\\pathfile\r\n\tUnix          /home/Webaffaires/payment/param/pathfile\r\npath_bin      - path to a SIPS request binary file\r\n\tWindows   C:\\webaffaires\\payment\\bin\\request.exe\r\n\tUnix          /home/Webaffaires/payment/bin/request\r\npath_bin_resp - path to a SIPS response binary file \r\n\tWindows   C:\\webaffaires\\payment\\bin\\response.exe\r\n\tUnix          /home/Webaffaires/payment/bin/response\r\n\r\n------------------------------------------------------------------------------------------------------\r\nAdditional parameters:\r\n\r\npayment_means - credit cards types from the following list, written inline and separated by ",2," (for example  CB,2,VISA,2,MASTERCARD,2)\r\nCB, HYPERMEDIA, NORAUTO, TOYSRUS, AMEX, CLEF, INTERVAL, NOUVFRONT, \r\nAURORE, CONNEXION, JCB, PLURIEL, VISA, BOURBON, DELATOUR, MASTERCARD,  SERAP\r\nNOTE: Do not forget to add the selected types to the Credit Cards Types table' , NULL , NULL , NULL , NULL , './payments/sips_request.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'POST' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , 9 , NULL , 10 , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (45 , 1 , 'DGL (Credit Card)' , NULL , NULL , 'Please specify values for the following parameter:\r\n- Account\r\n\r\nParameters description:\r\n\r\nAddUser - User Creation Control Flag :\r\n\'0\' - Do nothing on the User creation\r\n\'1\' - Log Username and password and add user account in merchant web site.\r\n\'2\' - Log Username and password ONLY' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 1 , 'https://secure.2000Charge.com/secure/CCRemoteProcess.asp' , './payments/dgl_ccrp.php' , NULL , NULL , NULL , 4 , 6 , 10 , 1 , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (46 , 1 , 'DGL (ACH Interface)' , NULL , NULL , 'Please specify values for the following parameter:\r\n- Account\r\n\r\nParameters description:\r\n\r\nTEST - \'TRUE\' or \'FALSE\' Optional\r\n\r\nAddUser - \r\n0 - Do nothing but run the transaction\r\n1 - Log username and password and add user account in the merchant web site\r\n2-Just log username/password in 2000Charge system\r\n\r\nTrialFlag - \'1\' or \'0\' Is this a trial? If yes, then Rebill must be 1and all rebill values must be set.\r\n\r\nRebill - \'1\' or \'0\' Should we automatically rebill this customer after a set amount of time? Must be \'1\' if TrialFlag is \'1\', otherwise optional\r\n\r\nRebillAmount - Amount in USD of the automatic rebill. Mandatory if Rebill is \'1\', otherwise ignored.\r\nRebillDateType - \r\n\'d\' (day), \'m\' (month), \'y\' (year) The units expressed in RebillDateValue.Mandatory if Rebill is \'1\' otherwise ignored.\r\n\r\nRebillDateValue - Number of units specified in RebillDateType to wait between automatic rebilling. Mandatory if Rebill is \'1\' otherwise ignored.\r\n\r\nUserExpire - (7/7/2004 12:29:02 PM) When should this users access expire?\r\nMust be valid date format. Mandatory if AddUser=1 and Rebill=0' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 1 , 'https://secure.2000charge.com/Secure/ChkRemoteProcess.asp' , './payments/dgl_ach.php' , NULL , NULL , NULL , 4 , NULL , 10 , 1 , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (47 , 1 , 'DGL (EUDEBIT Interface)' , NULL , NULL , 'Please specify values for the following parameter:\r\n- Account\r\n- CKABA\r\n- CKAcct\r\n\r\nParameters description:\r\n\r\nBankNameCode -\r\n10 - All bank\r\n19 - BBBank\r\n13 - Bezirkssparkasse\r\n2   - BfG Bank\r\n18 - CC-Bank\r\n3   - Citibank\r\n4   - Commerzbank\r\n5   - Deutsche Bank\r\n1   - Deutsche Bank 24\r\n6   - Dresdner Bank\r\n20 - Hypo- und Vereinsbank\r\n14 - Kreissparkasse\r\n7   - Landesbank\r\n16 - Landessparkasse\r\n11 - Norisbank\r\n8   - Postbank\r\n23 - Raiffeisenbank\r\n12 - Sonstige Sparkasse\r\n9   - Sparda-Bank\r\n15 - Stadtsparkasse\r\n17 - Vereinigte Sparkasse\r\n21 - Vereins- und Westbank\r\n22 - Volks- und Raiffeisenbank (VR-Bank)\r\n24 - Volksbank\r\n0   - Andere Bank\r\n\r\nCKABA - Bankleitzahl (Routing number) Mandatory if Country=DE or AT or CH (not needed for NL)\r\n\r\nCKAcct - Kontonummber (Account number) It varies from 6 digits to 11 digits depending on the countries. There is no length check on the account number.\r\n\r\nAddUser - \r\n0 - Do nothing but run the transaction\r\n1 - Log username and password and add user account in the merchant web site\r\n2-Just log username/password in 2000Charge system\r\n\r\nTrialFlag - \'1\' or \'0\' Is this a trial? If yes, then Rebill must be 1and all rebill values must be set.\r\n\r\nRebill - \'1\' or \'0\' Should we automatically rebill this customer after a set amount of time? Must be \'1\' if TrialFlag is \'1\', otherwise optional\r\n\r\nRebillAmount - Amount in USD of the automatic rebill. Mandatory if Rebill is \'1\', otherwise ignored.\r\nRebillDateType - \r\n\'d\' (day), \'m\' (month), \'y\' (year) The units expressed in RebillDateValue.Mandatory if Rebill is \'1\' otherwise ignored.\r\n\r\nRebillDateValue - Number of units specified in RebillDateType to wait between automatic rebilling. Mandatory if Rebill is \'1\' otherwise ignored.\r\n\r\nUserExpire - (7/7/2004 12:29:02 PM) When should this users access expire?\r\nMust be valid date format. Mandatory if AddUser=1 and Rebill=0\r\n\r\nTEST - \'TRUE\' or \'FALSE\' Optional' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 1 , 'https://secure.2000charge.com/Secure/MasterRemoteProcess.asp' , './payments/dgl_eudebit.php' , NULL , NULL , NULL , 4 , 6 , 10 , 1 , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (48 , 1 , 'Nochex' , NULL , NULL , 'Fill in \'merchant_id\' parameter by your Merchant ID which system provides you during registration process.\r\n\r\nIn order to turn on test mode, remove Not Passed marks for \'test_transaction\' and \'test_success_url\' parameters.\r\n\r\nUsually Nochex permits to process orders only from British IP space. If you see any error regarding incorrect IP address please contact Nochex support for details.' , NULL , NULL , NULL , NULL , 'https://secure.nochex.com/' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , 4 , 6 , 10 , NULL , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (49 , 1 , 'iDEAL Easy for ABN AMRO' , NULL , NULL , 'Please fill in \'PSPID\' parameter.\r\n\r\nTest parameters:\r\nPSPID: TESTiDEALEASY' , NULL , NULL , NULL , NULL , './payments/ideal_easy_advanced.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'POST' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , 'https://internetkassa.abnamro.nl/images/iDEAL_easy.gif' , 'iDEAL Easy' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (50 , 1 , 'PayFlow Pro Direct' , NULL , NULL , 'Please specify values for parameters \'USER\', \'VENDOR\', \'PARTNER\' and \'PWD\'.\r\n\r\nAdvanced URL:\r\nlive: \'https://payflowpro.paypal.com\'\r\ntest: \'https://pilot-payflowpro.paypal.com\'\r\n\r\n\'TENDER\': C - Direct Payment using credit card\r\n\'TRXTYPE\': A - Authorization, S - Sale\r\n' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , 0 , NULL , NULL , NULL , 'GET' , 0 , 1 , 'https://payflowpro.paypal.com' , './payments/paypal_payflow_pro_direct.php' , NULL , NULL , NULL , 4 , 6 , 10 , 0 , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (51 , 1 , 'PayFlow Pro Express' , NULL , NULL , 'Please specify values for parameters \'USER\', \'VENDOR\', \'PARTNER\' and \'PWD\'.\r\n\r\nAdvanced_URL:\r\nlive: \'https://payflowpro.paypal.com\'\r\ntest: \'https://pilot-payflowpro.paypal.com\'\r\n\r\nPayPal_URL:\r\nlive: \'https://www.paypal.com/cgi-bin/webscr?cmd=_express-checkout&token=\'\r\ntest: \'https://www.sandbox.paypal.com/cgi-bin/webscr?cmd=_express-checkout&token=\'\r\n\r\n\'TENDER\': P - Express Checkout using PayPal account\r\n\'TRXTYPE\': S - Sale\r\n\'ACTION\': S - Sale, D\r\n\r\n' , NULL , NULL , NULL , NULL , './payments/paypal_payflow_pro_express.php' , NULL , NULL , 0 , NULL , NULL , NULL , 'GET' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (52 , 1 , 'Gate2Shop' , NULL , NULL , 'Please specify the following parameters that should be acquired from Gate2Shop account:\r\n\r\n- merchant_site_id,\r\n- merchant_id,\r\n- secret. \r\n\r\nAlso please fill in merchantIp and merchant_unique_id parameters.\r\n\r\nTo use Direct Merchant Notification (DMN) please set URL of DMN as \'http://www.example.com/payments/gate2shop_response.php\'.\r\n\r\n' , NULL , NULL , NULL , NULL , 'payments/gate2shop_process.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , 4 , 6 , 10 , NULL , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (53 , 1 , 'Rabo iDEAL Lite' , NULL , NULL , 'test environment\r\nURL = https://idealtest.rabobank.nl/ideal/mpiPayInitRabo.do\r\nproduction environment \r\nURL =https://ideal.rabobank.nl/ideal/mpiPayInitRabo.do\r\n\r\nThe only way to obtain a guaranteed transaction status is to check the status of a transaction in the Rabo iDEAL Dashboard. Rabobank recommends checking the status of every transaction in the Rabo iDEAL Dashboard (manually) before delivery of the order. Another option is to deliver the purchased goods only after the payment has been reported on your bank account statement.\r\n' , NULL , NULL , NULL , NULL , './payments/ideal_lite_process.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (54 , 1 , 'Ogone e-Commerce' , NULL , NULL , 'Please specify your merchant PSPID parameter and two pass phrases SHA-IN and SHA-OUT from your Ogone settings which you can find under\r\nSHA-IN: Technical information > Data and origin verification\r\nSHA-OUT: Technical information > Transaction feedback\r\n\r\nSubmit your merchant page URL at Technical information > Data and origin verification as:\r\nhttp://www.yoursitename.com/payments/ogone_checkout.php\r\nReplace www.yoursitename.com with your real site URL.\r\n\r\nUse different PAYMENTURL for test and production modes:\r\nTest: https://secure.ogone.com/ncol/test/orderstandard.asp\r\nProduction: https://secure.ogone.com/ncol/prod/orderstandard.asp' , NULL , NULL , NULL , NULL , './payments/ogone_checkout.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (55 , 1 , 'HSBC' , NULL , NULL , 'The site MUST have a valid SSL cert, shared or dedicated to the site, (otherwise the gateway will fail every time). Also you need hsbc_cpi executable installed and availiable for php scripts or use special java class.' , NULL , NULL , NULL , NULL , './payments/hsbc_cpi.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , 9 , 6 , 12 , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (56 , 1 , 'MultiSafePay' , NULL , NULL , 'Please specify this parameters:\r\naccount - set it to your account ID\r\nsite_id - set it to your site ID\r\nsite_secure_code - set it to your site secure code\r\n\r\naction_url:\r\ntest - https://testapi.multisafepay.com/ewx/\r\nlive - https://api.multisafepay.com/ewx/\r\n' , NULL , NULL , NULL , NULL , './payments/multisafepay_process.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , 9 , 6 , 10 , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (57 , 1 , 'eWay' , NULL , NULL , 'Please specify this parameter:\r\newayCustomerID - set it to your Customer ID\r\n\r\nAdvanced URL:\r\nTest gateway - https://www.eway.com.au/gateway_cvn/xmltest/testpage.asp\r\nLive gateway - https://www.eway.com.au/gateway_cvn/xmlpayment.asp\r\n' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 1 , 'https://www.eway.com.au/gateway_cvn/xmlpayment.asp' , './payments/e_way.php' , NULL , NULL , NULL , 4 , NULL , 10 , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (58 , 1 , 'eWay 3D-Secure' , NULL , NULL , 'Please specify this parameter:\r\newayCustomerID - set it to your Customer ID\r\n\r\nPayment URL:\r\nhttps://www.eway.com.au/gateway_3d/payment.asp\r\nhttps://www.eway.com.au/gateway/payment.asp\r\n' , NULL , NULL , NULL , NULL , 'https://www.eway.com.au/gateway/payment.asp' , NULL , NULL , NULL , NULL , NULL , NULL , 'POST' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (59 , 1 , 'Transact SecurePay XML' , NULL , NULL , 'Please specify this parameter:\r\nTID - set it to your TID\r\nLoginID - set it to your LoginID\r\nPassword - set it to your Password\r\n\r\nDemo:\r\nTID:93999999\r\nLoginID:123456789012345678901234567890\r\nPassword:123456789012345678901234567890\r\nURL:https://caesar-ips.transact-gmbh.de/xml' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (60 , 1 , 'Garanti 3D Secure PAY' , NULL , NULL , 'Please specify this parameters:\r\nclientid - set it to your client ID\r\nstorekey - set it to your storekey\r\n' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 1 , 'https://ccpos.garanti.com.tr/servlet/gar3Dgate' , './payments/finans_3d_pay.php' , NULL , NULL , NULL , 3 , NULL , 10 , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (61 , 1 , 'Portmone' , NULL , NULL , 'Please specify this parameters:\r\nPAYEE_ID:  Id shop. Assigns Portmone.com each store individually \r\nLOG:  Login for online store. Assigns Portmone.com each store individually \r\nPASS: The password for Internet shop. Assigns Portmone.com each store individually \r\nLANG: en - English, ru - Russian, uk - Ukrainian\r\n\r\nPayment URL:\r\nlive -\r\nhttps://www.portmone.com.ua/secure/gate/pay.php\r\ntest -\r\nhttps://www.portmone.com.ua/3dsecure/index.php\r\nhttps://www.portmone.com.ua/3dtest/index.php\r\n\r\nCHECK_URL:\r\nlive -\r\nhttps://www.portmone.com.ua/secure/bills/results.php \r\ntest -\r\nhttps://www.portmone.com.ua/3dsecure/result.php\r\nhttps://www.portmone.com.ua/3dtest/result.php' , NULL , NULL , NULL , NULL , 'https://www.portmone.com.ua/secure/gate/pay.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'POST' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (62 , 1 , 'eProcessingNetwork' , NULL , NULL , 'Please specify this parameter:\r\nePNAccount - set it to your eProcessingNetwork ccount\r\n\r\nCVV2Type - is optional have a following value\r\n1  I have entered my CVV2 above \r\n2  My cards\' CVV2 is illegible \r\n9  My card has no CVV2 imprint \r\n0 Not passing CVV2 \r\n' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 1 , 'https://www.eProcessingNetwork.Com/cgi-bin/tdbe/transact.pl' , './payments/eprocessingnetwork.php' , NULL , NULL , NULL , 4 , NULL , 10 , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (63 , 1 , 'Payment on Invoice' , NULL , NULL , 'An example of payment system that does not process payments automatically but send PDF invoice to customer and set special status Invoice Sent.' , NULL , NULL , NULL , NULL , 'order_confirmation.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (64 , 1 , 'Total Web Solutions' , NULL , NULL , 'Please specify this parameter:\r\nCustomerID - set it to your Customer ID\r\n\r\nLive System:\r\nPay Page - https://secure.totalwebsecure.com/paypage/clear.asp\r\nConfirmation Page - https://secure.totalwebsecure.com/paypage/confirm.asp\r\n\r\nTest System:\r\nPay Page - https://testsecure.totalwebsecure.com/paypage/clear.asp\r\nConfirmation Page - https://testsecure.totalwebsecure.com/paypage/confirm.asp' , NULL , NULL , NULL , NULL , 'https://secure.totalwebsecure.com/paypage/clear.asp' , NULL , NULL , NULL , NULL , NULL , NULL , 'POST' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (65 , 1 , 'HSBC' , NULL , NULL , 'Please specify values for the following parameters:\r\n- Mode\r\nlive - P\r\ntest - T\r\n- storefront_id\r\n- cpi_hash_key\r\n- hsbc_cpi_filepath\r\n\r\nThe site MUST have a valid SSL cert, shared or dedicated to the site, otherwise the gateway will fail every time\r\nMode for the live site - \'P\' mode, if test - \'T\' mode' , NULL , NULL , NULL , NULL , './payments/hsbc_cpi.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'POST' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , 9 , 6 , 12 , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (66 , 1 , 'Payson' , NULL , NULL , 'Please specify this parameters:\r\nagentid - set it to your agent ID\r\nstorekey - set it to your storekey\r\nselleremail - set it to your email\r\n\r\naction_url:\r\nhttps://www.payson.se/testagent/default.aspx\r\nhttps://www.payson.se/merchant/default.aspx\r\n\r\nguaranteeoffered: Set this parameter to 1 to not offer the Payson Guarantee and 2 to offer the Payson Guarantee.\r\n\r\ncustomreceipt: This parameter decides whether or not a buyer is directly sent to the page specified by the parameter OKUrl after a purchase is completed without the receipt first being displayed. Using this parameter and setting it to true requires the Agent to display the receipt.\r\n\r\npaymentmethod: \r\n0 (zero) = All payment methods (Cards, Internet banks and Payson) 1 = Only cards (VISA/MasterCard) 2 = Only Internet banks (SEB, Handelsbanken, Nordea and Swedbank) 3 = Funds in Payson accounts 4 = Internet banks + Funds on Payson accounts' , NULL , NULL , NULL , NULL , './payments/payson_process.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (67 , 1 , 'JCC API' , NULL , NULL , 'Please specify this parameters:\r\n\'merid\' - set it to your JCC merchant id\r\n\'password\' - set it to your JCC merchant password' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 1 , 'https://www.jccsecure.com/SENTRY/PaymentGateway/Application/DirectAuthzLink.aspx' , './payments/jcc_api.php' , NULL , NULL , NULL , 4 , NULL , 10 , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (68 , 1 , 'Buckaroo SSL page' , NULL , NULL , 'Please specify the following parameters:\r\nBPE_Merchant - your merchant code\r\nsecretkey - your secret key\r\n\r\nNote: The parameters \'action_url\' and \'secretkey\' should be always set as Not Passed!\r\n\r\nBPE_Merchant - The Merchant key you received from us. This code makes sure the transaction will be added to your account. You will receive a unique Merchant key for each web site.\r\nBPE_Currency - The code of the currency in which the transaction needs to occur. Use the currency codes for the currency you are authorized to:\r\n- EUR = Euro\r\n- GBP = British pound\r\n- USD = US dollar\r\n- CAD = Canadian dollar\r\n- SHR = Swiss franc\r\n- NOK = Norwegian krone\r\n- SEK = Swedish krone\r\n- DKK = Danish krone\r\nBPE_Language - Language code for the user interface:\r\n- NL = dutch\r\n- EN = english\r\n- DE = german\r\n- FR = french\r\nBPE_Mode - Mode value:\r\n- 0 = complete processing\r\n- 1 = test mode. Transactions\r\nBPE_Return_Method - (Optional) Set in which way the result parameters should be returned to you response pages.\r\ndefault=POST\r\n- GET = http get-method\r\n- POST = http post-method\r\nBPE_Return_Target - (Optional) Set in this field the name of the target window (window.name) in which the return page should be called.\r\nBPE_Style - (Optional) Set in this page whether you want the SSL pages to be used in a popup screen or in a frameset/iframe.\r\n- FRAME = pages in iframe/frame\r\n- POPUP = pages in popup screen\r\nBPE_Autoclose_Popup - (Optional) You have to set in this field if the screen needs to be closed automatic when you use a popup screen. The popup will be closed directly when loading the response. Your own return page informs the customer about the results of the transaction. The possible values are:\r\n- 1, ja, yes, true = JA\r\n- 0, nee, no, false = NEE\r\n\r\nFor Buckaroo action_url:\r\nhttps://payment.buckaroo.nl/sslplus/request_for_authorization.asp\r\n\r\nFor iDeal action_url:\r\nhttps://payment.buckaroo.nl/gateway/ideal_payment.asp\r\nBPE_Currency and Currency always - EUR = Euro' , NULL , NULL , NULL , NULL , './payments/buckaroo_ssl_process.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (69 , 1 , 'DanDomain PayNet' , NULL , NULL , 'Please specify the following parameter:\r\nMerchantNumber - your merchant number\r\n\r\nTestMode - If this parameter is declared and set to = 1, the system will be in test mode.\r\n\r\nPayType - Setup to reflect whether it is a normal credit card transaction or an eDankort transaction.\r\n0 = Normal credit card (Default)\r\n1 = eDankort\r\n' , NULL , NULL , NULL , NULL , './payments/dankort_process.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (70 , 1 , 'WINBANK' , NULL , NULL , 'The only way to obtain a guaranteed transaction status is to check the status of a transaction in the WINBANK.\r\n\r\nPlease specify the following parameter:\r\nmerchantid - your Merchants code (issued by winbank)\r\nposid - Merchants virtual terminal id (issued by winbank)\r\nuser - Username of the merchants user that is authorized to execute transactions on behalf of the given terminal id of the merchant (issued by winbank)\r\n\r\nlangid - \r\n0:Greek (default)\r\n1:English\r\n2: German' , NULL , NULL , NULL , NULL , 'https://paycenter.winbank.gr/ePos2003/winpay.asp' , NULL , NULL , NULL , NULL , NULL , NULL , 'POST' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (71 , 1 , 'iPay88' , NULL , NULL , 'Please specify the following parameter:\r\nMerchantCode - your Merchant code assigned by iPay88\r\nMerchantKey - your MerchantKey (This is the key that supply by iPay88 and share between iPay88 and the merchant only)\r\n\r\nPaymentId: Payment Method Id\r\n"2" - Credit Card\r\n"6" - Maybank2U\r\n"8" - Alliance Online Transfer\r\n"10" - AM Bank\r\n"14" - RHB\r\n"15" - Hong Leong Bank Transfer\r\n"16" - FPX\r\n"17" - Mobile Money\r\n"20" - CIMB Click\r\n"21" - China Union Pay\r\n"22" - Web Cash\r\n"23" - Meps Cash\r\n"25" - Credit Card (USD)\r\n"33" - Paypal\r\nRemark: Merchant remarks\r\nLang: Encoding type\r\n"ISO-8859-1" - English\r\n"UTF-8" - Unicode\r\n"GB2312" - Chinese Simplified\r\n"GD18030" - Chinese Simplified\r\n"BIG5" - Chinese Traditional' , NULL , NULL , NULL , NULL , './payments/ipay88_process.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (72 , 1 , 'GTBill QuickPay API' , NULL , NULL , 'Please specify the following parameter:\r\nMerchantID - Merchant reference ID supplied by GTBill\r\nSiteID - A site reference ID supplied by GTBill for each of your websites\r\nRequireShipping - \r\n\'0\' default - hides shipping option\r\n\'1\' - requires shipping' , NULL , NULL , NULL , NULL , './payments/gtbill_api_process.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , 9 , NULL , 10 , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (73 , 1 , 'Consel' , NULL , NULL , 'Please specify the following parameters:\r\ntabfin - a table code provided by Consel.\r\nconvenz - The field is 7 numeric characters long (es. 0987654) and is provided by Consel\r\n\r\nThe return URLs should be:\r\nURL OK: \'http://www.example.com/order_final.php?va_status_return=success\'\r\nURL KO: \'http://www.example.com/order_final.php?va_status_return=decline\'\r\n' , NULL , NULL , NULL , NULL , './payments/e-consel_process.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (74 , 1 , 'Check Savers' , NULL , NULL , 'Please specify the following parameters:\r\nmid - Your merchant ID\r\nuser - API User ID\r\npassword - API password\r\n\r\ntype: charge,check\r\n\r\ntype - charge:\r\ncctype - Type of card\r\ncc - Card number\r\nexpire - Expire Date for card Must be formatted mm/yy\r\ncvv - CVV2 code\r\n\r\ntype - check:\r\nbankname - Customer Bank name, note only first 50 chard will be used on the check\r\nchecktype - check\r\nroutingnumber - Account routing number. Must pass ABA routing number validation\r\naccountnumber - Check account number\r\naccounttype - P for personal account, B for business account (businessname - business name of the entity making the payment)\r\nrisknmodifier - Extended risk check of the check routing number and account number can be modified by this field. 0 disables risk checks. 1 forces risk checks. If not specified, the contract default will be used, according to a threshold. *This feature might not be available, ask your account manager.\r\nauthenticationmodifier - Authentication of the payer name and address can be modified by this field. 0 disables payer authentication checks. 1 forces payer authentication check. If not specified, the contract default will be used, according to a threshold. *This feature might not be available, ask your account manager.\r\ninsurancemodifier - Insurance that the amount will clear can be modified by this field. 0 disables insurance. 1 forces insurance. If not specified, the contract default will be used, according to a threshold. *This feature might not be available, ask your account manager.' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 1 , 'https://secure.check-21.org/api/interface.aspx' , './payments/check_savers.php' , NULL , NULL , NULL , 4 , NULL , 10 , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (75 , 1 , 'Heidelpay' , NULL , NULL , 'Please specify the following parameters:\r\nSECURITY.SENDER - Each Server which sends requests to the system has an own sender unique ID. The sender UID is no logical business orientated subdivision like the channel ID, but refers to physical installations of software. Please provide here the value you have received from the customer support department.\r\nTRANSACTION.CHANNEL - The channel ID is a unique key for the identification of the unit which sends transactions into the system. Every merchant can have multiple channels for different purposes. Possible division criteria are for example different shops, organizational units, customer groups or countries. The channel ID doesnt refer to physical installations of the software like the sender ID but is a logical business orientated subdivision. Different channels help to analyze the entirety of transactions and to provide different system configurations for a nonuniform transaction base. The channel IDs are assigned by the account management.\r\nUSER.LOGIN - User Id of the sending user. This user must be configured with SEND rights.\r\nUSER.PWD - Password of the sending user\r\n\r\nTRANSACTION.MODE:\r\ntest - INTEGRATOR_TEST or CONNECTOR_TEST\r\nlive - LIVE\r\nAdvanced URL:\r\ntest - https://test.ctpe.net/frontend/payment.prc\r\nlive - https://ctpe.net/frontend/payment.prc' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 1 , 'https://ctpe.net/frontend/payment.prc' , './payments/heidelpay_call_wpf.php' , NULL , NULL , NULL , 4 , 6 , 10 , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (76 , 1 , 'Heidelpay Bank' , NULL , NULL , 'Please specify the following parameters:\r\nSECURITY.SENDER - Each Server which sends requests to the system has an own sender unique ID. The sender UID is no logical business orientated subdivision like the channel ID, but refers to physical installations of software. Please provide here the value you have received from the customer support department.\r\nTRANSACTION.CHANNEL - The channel ID is a unique key for the identification of the unit which sends transactions into the system. Every merchant can have multiple channels for different purposes. Possible division criteria are for example different shops, organizational units, customer groups or countries. The channel ID doesnt refer to physical installations of the software like the sender ID but is a logical business orientated subdivision. Different channels help to analyze the entirety of transactions and to provide different system configurations for a nonuniform transaction base. The channel IDs are assigned by the account management.\r\nUSER.LOGIN - User Id of the sending user. This user must be configured with SEND rights.\r\nUSER.PWD - Password of the sending user\r\n\r\nTRANSACTION.MODE:\r\ntest - INTEGRATOR_TEST or CONNECTOR_TEST\r\nlive - LIVE\r\nAdvanced URL:\r\ntest - https://test.ctpe.net/frontend/payment.prc\r\nlive - https://ctpe.net/frontend/payment.prc' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 1 , 'https://ctpe.net/frontend/payment.prc' , './payments/heidelpay_call_wpf.php' , NULL , NULL , NULL , 4 , 6 , 10 , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (77 , 1 , 'easypay' , NULL , NULL , 'Please specify the following parameters:\r\nmerchid - your merchant id\r\nmerchpwd - your merchant password\r\n\r\nAdvanced URL:\r\ntest - https://www.kartenverwaltung.de/ips/pgi/_test_free/auth.php\r\ntest - https://www.kartenverwaltung.de/ips/test_profi.php\r\nlive - https://www.easypayhost.de/skript.php' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 1 , 'https://www.easypayhost.de/skript.php' , './payments/easypay.php' , NULL , NULL , NULL , 4 , NULL , 10 , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (78 , 1 , 'easypay ELV' , NULL , NULL , 'Please specify the following parameters:\r\nmerchid - your merchant id\r\nmerchpwd - your merchant password\r\n\r\nAdvanced URL:\r\ntest - https://www.kartenverwaltung.de/ips/pgi/_test_free/auth.php\r\ntest - https://www.kartenverwaltung.de/ips/test_profi.php\r\nlive - https://www.easypayhost.de/skript.php' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 1 , 'https://www.easypayhost.de/skript.php' , './payments/easypay.php' , NULL , NULL , NULL , 4 , NULL , 10 , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (79 , 1 , 'CertiTrade' , NULL , NULL , 'Please specify the following parameters:\r\nmerchantid - your Merchants ID with CertiTrade\r\nmd5key - key that is provided by CertiTrade\r\n\r\naction_url:\r\nhttps://payment.certitrade.net/webshophtml/e/auth.php - live\r\nhttps://www.certitrade.net/webshophtml/e/auth.php - demo\r\nhttps://www.certitrade.net/webshophtml/e/eko.php - eko\r\n\r\noptional parameters\r\nlang:\r\nsv - Swedish\r\nen - English\r\nno - Norwegian\r\nda - Danish\r\nfi - Finnish\r\nde - German\r\nfr - French\r\nit - Italian\r\nes - Spanish\r\n\r\nconnection - The supplier of the communication that is used\r\nacquirer - Bank making payment\r\nreturwindow - Incase of need to name framset upon retururl\r\nDEBUG - Printout of the parameters to CertiTrade is shown\r\nHTTPDEBUG - Tracking of calls to the shops URL is shown\r\ntimeout - Possibility to timeout after 15-45 minutes\r\ndelayed_capture - Possibility to manually approve capture' , NULL , NULL , NULL , NULL , './payments/certitrade_process.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (80 , 1 , 'Kreditor' , NULL , NULL , 'Please specify the following parameters:\r\neid - A store ID which refers to your store in Kreditor\'s database\r\nestoreUser - The customers name in your store. Good to pass if you want to perform searches with it at https://online.kreditor.se/\r\nsecret - A shared secret used to secure all traffic exchanged by Kreditor and your store\r\n\r\nLive Advanced URL: payment.kreditor.se:80/\r\nTest Advanced URL: beta-test.kreditor.se:4567/\r\n' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 1 , 'beta-test.kreditor.se:4567/' , './payments/kreditor.php' , NULL , NULL , NULL , 4 , NULL , 10 , NULL , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (81 , 1 , 'PayLeap Credit' , NULL , NULL , 'Please specify the following parameters:\r\nUserName - User name assigned in the payment server\r\nPassword - Password for the user name assigned in the payment server\r\n\r\nTest Advanced URL: http://test.payleap.com/SmartPayments/transact.asmx/ProcessCreditCard' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 1 , 'http://test.payleap.com/SmartPayments/transact.asmx/ProcessCreditCard' , './payments/payleap.php' , NULL , NULL , NULL , 4 , NULL , 10 , 1 , 0 , 0 , 0 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 0 , 0 , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (82 , 1 , 'SveaWebPay' , NULL , NULL , 'Please specify values for parameters \'Username\', \'password\'.\r\n\r\nLanguage:\r\nsv - Swedish\r\nen - English\r\nno - Norwegian\r\nda - Danish\r\nfi - Finnish\r\nde - German\r\nfr - French\r\nit - Italian\r\nes - Spanish\r\n\r\nFor use test mode:\r\nTestmode - True\r\nFor use live mode:\r\nTestmode - False' , NULL , NULL , NULL , NULL , './payments/sveawebpay_process.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 0 , 0 , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (83 , 1 , 'Ogone DireckLink' , NULL , NULL , 'Please specify this parameters:\r\nPSPID - set it to your PSPID\r\nUSERID - set it to your USERID \r\nPSWD - set it to your PSWD\r\nSignature - set it to your additional string\r\n\r\nAdvanced URL:\r\ntest - https://secure.ogone.com/ncol/test/orderdirect.asp\r\nlive - https://secure.ogone.com/ncol/prod/orderdirect.asp\r\n\r\nOperation:\r\nRES - an authorisation is requested, independently of the merchant\'s account configuration.\r\nSAL - a direct sale is requested, independently of the merchant\'s account configuration.\r\nRFD - a negative payment not linked to a previous payment\r\n\r\nECI: \r\n0 - Swiped\r\n1 - Manually keyed: Mail Orde^elephone Order (MOTO) (card not present)\r\n2 - Recurring payments, originating from MOTO\r\n3 - Installment payments\r\n4 - Manually keyed, card present\r\n7 - E-commerce with SSL encryption\r\n9 - Recurring after first e-commerce transaction\r\n\r\nlanguage: client\'s language (en_US, fr_FR, nl_NL, es_ES, it_IT, de_DE, no_NO)' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 1 , 'https://secure.ogone.com/ncol/test/orderdirect.asp' , './payments/ogone_direct_l.php' , NULL , NULL , NULL , 4 , 6 , 10 , NULL , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 0 , 0 , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (84 , 1 , 'EBS' , NULL , NULL , 'Please specify the following parameters:\r\naccount_id - Your Account ID\r\nsecret_key - Your secret key\r\n\r\nTo get the secret Key,\r\na. Login to the EBS Merchant Account.\r\nb. Go to Settings. In the Account settings you can view the secret key.\r\n\r\nmode:\r\nlive - for LIVE mode\r\ntest - for TEST mode' , NULL , NULL , NULL , NULL , 'https://secure.ebs.in/pg/ma/sale/pay/' , 0 , NULL , NULL , 0 , NULL , NULL , 'POST' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 0 , 0 , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (85 , 1 , 'PostFinance' , NULL , NULL , 'Please specify values for the following parameters:\r\nPSPID - Your affiliation name in PostFinance system\r\nsecret_in - Additional string in the SHA-1-IN Signature field in the Data and origin verification tab, Checks for e-Commerce section of the Technical Information\r\nsecret_out - Additional string in the SHA-1-OUT Signature field in the Transaction feedback tab, Security for request parameters section of the Technical Information\r\n\r\nlanguage:\r\nen_US, nl_NL, fr_FR, de_CH ...\r\n\r\npayment_url:\r\nPRODUCTION mode - https://e-payment.postfinance.ch/ncol/prod/orderstandard.asp\r\nTEST mode - https://e-payment.postfinance.ch/ncol/test/orderstandard.asp' , NULL , NULL , NULL , NULL , './payments/postfinance_process.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (86 , 1 , 'Segpay DHD' , NULL , NULL , 'Please specify values for the following parameters:\r\nMERCH_ID - this is a string, up to 9 characters, which uniquely identifies the merchant web site.\r\nPURCHASE_PAGE_ID - this is a unique identifier for the merchant\'s customized purchase page/template.\r\nPlease, ensure that an advanced URL is as follows:\r\n\'https://payment.dhdmedia.com/merchants/p.000000/m.000/buy.cfm\'\r\nsecret - this is a string, up to 2048 characters, which is used to secure data passed between the shopping cart software and the purchase pages hosted on the DHD Media platform.\r\n \r\nSpecify a Postback URL in DHD Media as follows:\r\n\'http://www.example.com/payments/dhd_nusoap_stubs.php?wsdl\'\r\n \r\ndisable_emailbox - This will control the display of a checkbox used for joining an email list [0:False; 1:True]\r\n \r\nafid_ext - This is the parameter for the unique identifier of the merchants affiliate/reseller\r\n \r\nudf_01; udf_02 - These parameters can be applied to any information which may be sent and stored with the order. NOTE that the cart order id should not be populated in these parameters -- the order id should be in the inv_id_ext parameter described above' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 1 , 'https://payment.dhdmedia.com/merchants/p.[PURCHASE_PAGE_ID]/m.[MERCH_ID]/buy.cfm' , './payments/dhd_process.php' , NULL , NULL , NULL , 9 , 6 , 10 , NULL , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (87 , 1 , 'eWAY UK "Hosted Payment" Page' , NULL , NULL , 'Please specify the following parameters:\r\nCustomerID - Your eWAY Customer ID\r\nUserName - Your eWAY Customer User Name\r\n\r\nLanguage: The two letter acronym of the language code. Default: English\r\nEN - English\r\nES - Spanish\r\nFR - French\r\nDE -German\r\nNL - Dutch\r\n\r\nPageTitle: This is value will be displayed as the title of the browser.\r\nDefault: eWAY Hosted Payment Page\r\n\r\nPageDescription: This value will be displayed above the Transaction Details.\r\nDefault: Blank\r\n\r\nPageFooter: This value will be displayed below the Transaction Details.\r\n\r\nCompanyName: This will be displayed as the company the customer is purchasing from, including this is highly recommended.\r\n\r\nMerchantOption1, MerchantOption2, MerchantOption3: This field is not displayed to the customer but is returned in the result string. Anything can be used here, useful for tracking transactions\r\n\r\nUseAVS: This field marks that Address Verification should be used (FUTURE USE ONLY) Values: True or False\r\n\r\nUseZIP: This field marks that ZIP Verification should be used (FUTURE USE ONLY)\r\nValues: True or False' , NULL , NULL , NULL , NULL , './payments/e_way_hp_process.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (88 , 1 , 'eWAY NZ "Hosted Payment" Page' , NULL , NULL , 'Please specify the following parameters:\r\nCustomerID - Your eWAY Customer ID\r\nUserName - Your eWAY Customer User Name\r\n\r\nLanguage: The two letter acronym of the language code. Default: English\r\nEN - English\r\nES - Spanish\r\nFR - French\r\nDE -German\r\nNL - Dutch\r\n\r\nCompanyLogo: The url of the image can be hosted on the merchants website and pass the secure https:// path of the image to be displayed at the top of the website. This is the top image block on the webpage and is restricted to 960px X 65px. A default secure image is used if none is supplied.\r\n\r\nPagebanner: The url of the image can be hosted on the merchants website and pass the secure https:// path of the image to be displayed at the top of the website. This is the second image block on the webpage and is restricted to 960px X 65px. A default secure image is used if none is supplied.\r\n\r\nModifiableCustomerDetails: This field specifies if the customer can change the contact details on the payment page This is useful if the merchant is not collecting details on their site. Values: True or False\r\n\r\nCompanyName: This will be displayed as the company the customer is purchasing from, including this is highly recommended.\r\n\r\nMerchantOption1, MerchantOption2, MerchantOption3: This field is not displayed to the customer but is returned in the result string. Anything can be used here, useful for tracking transactions\r\n\r\nUseAVS: This field marks that Address Verification should be used (FUTURE USE ONLY) Values: True or False\r\n\r\nUseZIP: This field marks that ZIP Verification should be used (FUTURE USE ONLY)\r\nValues: True or False' , NULL , NULL , NULL , NULL , './payments/e_way_hp_process.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (89 , 1 , 'PayEx' , NULL , NULL , 'Please specify the following parameters:\r\naccountNumber - Merchants PayEx account number\r\nencryptionkey - Merchants PayEx encryption key\r\n\r\npayment_url:\r\nTest - https://test-external.payex.com/pxorder/pxorder.asmx?wsdl\r\nLive - https://external.payex.com/pxorder/pxorder.asmx?wsdl\r\n\r\npurchaseOperation - SALE | AUTHORIZATION \r\nIf AUTHORIZATION is submitted, this indicates that the order will be a 2-phased transaction if the payment method supports it.\r\n\r\npriceArgList:\r\nPrice must be submitted in the lowest monetary unit of the selected currency. Example: 1200 = 12.00 NOK. Set this parameter to [blank] if Price is submitted.\r\n\r\nComma separated field with name=value data for specifying prices for several payment methods. (Example: PX=2000,CPA=2500,VISA=5000,MC=5000)\r\n\r\nWhen using Direct Debit the bank name can be specified in priceArgList.\r\nCurrently supported banks are\r\n\r\nSweden (SEK):\r\nFSPA (Swedbank)\r\nNB (Nordea Bank)\r\nSEB (Svenska Enskilda Bank)\r\nSHB (Handelsbanken)\r\n\r\nDenmark (DKK):\r\nNB:DK (Nordea Bank DK)\r\nDDB (Den Danske Bank)\r\n\r\nNorway (NOK):\r\nBAX (BankAxess)\r\n\r\nFinland (EUR):\r\nSAMPO\r\nAKTIA (Aktia, Sstpankki)\r\nOP (Osuuspanki, Pohjola, Oko)\r\nNB:FI (Nordea Bank Finland)\r\nSHB:FI\r\nSPANKKI\r\nTAPIOLA\r\nAALAND (landsbanken)\r\n\r\nValid price types are:\r\nPX(PayEx account)\r\nCPA(norwegian and swedish overcharged SMS)\r\nVISA\r\nMC(mastercard)\r\nEUROCARD (Eurocard)\r\nMAESTRO\r\nDINERS(Diners Club)\r\nAMEX(American Express)\r\nJCB\r\nFINAX\r\nDANKORT\r\nIVR(Overcharged Call)\r\nEVC (Value Code)\r\nGC (Gift card / Generic card)\r\nINVOICE (Invoice)\r\nLOAN (Loan)\r\nPayPal (PayPal)\r\n\r\nview:\r\nDefault payment method.\r\n\r\nValid price types are:\r\n\r\nCC (Payment method PayEx)\r\nPX (PayEx account)\r\nCREDITCARD (Credit Card)\r\nDIRECTDEBIT (Direct bank)\r\nCPA (Norwegian and Swedish overcharged SMS)\r\nIVR (Overcharged call)\r\nEVC (Value code)\r\nInvoice\r\nLoan\r\nGC (Gift card / generic card)\r\nCA (Credit account)\r\nPAYPAL(PayPal transactions)\r\n\r\nclientLanguage:\r\nThe language used in the redirect purchase dialog with the client. Available languages depend on the merchant configuration. Supported languages: nb-NO,da-DK,en-US,sv-SE,es-ES,de-DE,fi-FI,fr-FR,pl-PL,cs-CZ,hu-HU\r\nIf no language is specified, the default language for client UI is used.' , NULL , NULL , NULL , NULL , './payments/payex_process.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (90 , 1 , 'NMI' , NULL , NULL , 'Please specify the following parameter:\r\napi-key - is obtained in the Security Keys section of the Control Panel options\r\n' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 1 , 'https://secure.nmi.com/api/v2/three-step' , './payments/nmi_call.php' , NULL , NULL , NULL , 3 , 6 , 10 , NULL , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (91 , 1 , 'UltimatePay' , NULL , NULL , 'Please specify the following parameters:\r\nlogin - can be found in your UltimatePay Merchant Configuration document\r\nadminpwd - can be found in your UltimatePay Merchant Configuration document\r\nsecret - secret hash pass phrase can be found in your UltimatePay Merchant Configuration document\r\nsn - Your Service Code (SN) can be found in your UltimatePay Merchant Configuration document\r\n\r\nIn UltimatePay Configuration specify PostBackURL like it  http://www.example.com/payments/ultimatepay_postback.php\r\n\r\nTest mode\r\naction_url - https://www.UltimatePay.com/app/api/test/\r\nLive mode\r\naction_url - https://www.UltimatePay.com/app/api/live/\r\nnotify_url - https://www.UltimatePay.com/app/api/live/\r\n\r\ndisplay - UltimatePay Order Flow presentation type\r\nvalues:\r\nLightbox\r\nLightboxLauncher\r\nLargePanel\r\nSmallPanel\r\ndirect\r\nfullpage\r\n\r\nriskmode - Enables to override their default risk mode settings on a per-transaction basis, either to relax their restrictions or reduce their risk of reversals. \r\nvalues:\r\nnormal allows all payment methods, including customer reversible (CR) payment methods like debit, credit cards, and PayPal.\r\nlow restricts the payment methods available to the customer to only those that are cash-equivalent (CE) or non-reversible (NR).\r\nnone restricts the customer to only being able to pay via payment methods that are non-reversible (NR) like cash, Western Union, etc. This severely restricts the options available to the customer and should normally only be used for transactions where the merchant has zero risk tolerance.' , NULL , NULL , NULL , NULL , './payments/ultimatepay_process.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , 4 , 6 , 10 , NULL , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (92 , 1 , 'Moneybookers Quick Checkout' , 'Pay securely by credit / debit card' , NULL , 'Required fields:\r\n\r\npay_to_email - Email address of the merchants moneybookers.com account\r\n\r\nsecret_word (required for moneybookers_status.php)\r\nThe secret word MUST be submitted in the Merchant Tools section in lowercase before the md5sig can be used. If you insert any uppercase symbols, they will automatically be converted to lower case. The only restriction on your secret word is the length which must not exceed 10 characters. Non-alphanumeric symbols can be used. If the Merchant Tools section is not shown in your account, please contact merchantservices@moneybookers.com\r\n------------------------------------------------------------------------------------------------------\r\nOther fields:\r\n\r\nconfirmation_note\r\nMerchant may show to the customer on the confirmation screen - the end step of the\r\nprocess - a note, confirmation number, PIN or any other message. Line breaks <br> may be used for longer messages.\r\n\r\nreturn_url\r\nURL to which the customer will be returned when the payment is made. If this field is not filled, the merchant gateway window will simply close automatically at the end of the transaction, so that the customer will be returned to the last page on the merchant\'s website where he has been before. A secure return_url functionality is available.\r\n\r\ncancel_url\r\nURL to which the customer will be returned if the payment process is cancelled. If this field is not filled, the merchant gateway window will simply close automatically upon clicking the cancellation button, so the customer will be\r\nreturned to the last page on the merchant\'s website where the customer has been before.\r\n\r\nstatus_url\r\nURL to which the transaction details will posted after the payment process is complete.\r\nAlternatively, you may specify an email to which you would like to receive the If the status_url is omitted, no transaction details will be sent to the merchant.\r\nCould be email like  EMAIL_TO_RECEIVE_PAYMENT_NOTIFICATION@MERCHANT.COM\r\n\r\nrecipient_description\r\nMerchant Name this replaces merchant email address on the gateway\r\n\r\nlogo_url\r\nprovides a co-branded gateway ideally the image should be stored on a secure server to avoid a security warning which comes up on some browsers\r\n\r\nhide_login\r\nthis hides the Moneybookers login screen on the right hand side, should be done for all options apart from e-wallet\r\n\r\npayment_methods\r\nACC - for credit & debit cards\r\nWLT - for the e-wallet\r\nread more information in documentation\r\n\r\nPay securely by credit / debit card' , NULL , NULL , NULL , NULL , 'https://www.moneybookers.com/app/payment.pl' , 0 , NULL , NULL , NULL , NULL , NULL , 'POST' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , 4 , 6 , 10 , 0 , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , 'http://www.moneybookers.com/images/logos/checkout_logos/checkout_120x40px.gif' , NULL , 'http://www.moneybookers.com/images/logos/checkout_logos/checkout_180x60px.gif' , NULL , 1 , NULL , NULL , NULL , NULL , 0 , 0 , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (93 , 1 , 'IcePay WIRE' , NULL , NULL , 'Please specify the following parameters:\r\nIC_Merchant - your merchant ID\r\nEncryptioncode - your Encryption code\r\n\r\nIC_PaymentMethod available values:\r\nWIRE IC_Issuer: DEFAULT\r\nCREDITCARD IC_Issuer: AMEX, MASTER, VISA\r\nDDEBIT IC_Issuer: INCASSO\r\nIDEAL IC_Issuer: ABNAMRO, ASNBANK, VANLANSCHOT, FRIESLAND, ING, RABOBANK, SNSBANK, SNSREGIOBANK, TRIODOSBANK\r\nFor more information please read the documentation.\r\n\r\nAlso you shoul specify the return URLS:\r\nSuccess URL - This is the page shown to the final user after a successful payment.\r\nError URL - This is the page shown to the final user after a failed payment.\r\nPostback URL - This is the page to which payment information will be sent.\r\nsubmit it as:\r\nSuccess URL - http://www.your_site.com/order_final.php\r\nError URL - http://www.your_site.com/order_final.php\r\nPostback URL - http://www.your_site.com/payments/icepay_postback.php' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 1 , 'https://pay.icepay.eu/checkout.aspx' , './payments/icepay_call.php' , NULL , NULL , NULL , 4 , NULL , 10 , 1 , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (94 , 1 , 'CCBill' , NULL , NULL , 'Please specify values for parameters below:\r\nclientAccnum - CCBill client main account number.\r\nclientSubacc - CCBill client subaccount Number.\r\nformName - Three or more character code for the form.\r\nsalt - Enter up to 24 alphanumeric characters into the field. This is the TripleDES encryption key that the client must obtain from CCBill. \r\n\r\nIn your CCBill account please specify the following URLs:\r\nApproval URL: http://www.your_site.com/order_final.php\r\nDenial URL:   http://www.your_site.com/order_final.php\r\nApproval Post URL: http://www.yoursite.com/payments/ccbill_check.php\r\nDenial Post URL: http://www.yoursite.com/payments/ccbill_check.php\r\n\r\nNote: replace "www.yoursite.com" in the above example with your real domain.' , NULL , NULL , NULL , NULL , './payments/ccbill_process.php' , NULL , NULL , 0 , NULL , NULL , NULL , 'GET' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , 9 , 6 , 10 , 0 , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (95 , 1 , 'Knet' , NULL , NULL , 'Please specify values for parameters \'id\' and \'password\'.\r\n\r\nUse the following URLs as your payment_url\r\ntest - https://www.knetpaytest.com.kw/CGW302/servlet/PaymentInitHTTPServlet\r\nlive - https://www.knetpay.com.kw/CGW/servlet/PaymentInitHTTPServlet' , NULL , NULL , NULL , NULL , './payments/knet_process.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (96 , 1 , 'Renaissance Associates (Direct Post API)' , 'Renaissance Associates' , NULL , 'Please specify your values for parameters \'username\' and \'password\'.\r\n\r\nTesting:\r\nThe Payment Gateway demo account can also be used for testing at any\r\ntime.  Please use the following username and password for testing with\r\nthis account:\r\n \r\nUsername: demo\r\nPassword: password \r\n\r\nTest transactions can be submitted with the following information:\r\nVisa: 4111111111111111 \r\nMasterCard: 5431111111111111 \r\nDiscoverCard: 6011601160116611 \r\nAmerican Express: 341111111111111 \r\nCredit Card Expiration: 10/10\r\nAmount > 1.00 \r\n\r\nTo cause a declined message, pass an amount less than 1.00.' , NULL , NULL , NULL , NULL , 'credit_card_info.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , NULL , 1 , 'https://secure.rabankcardgateway.com/api/transact.php' , './payments/renaissance_direct.php' , NULL , NULL , NULL , 4 , 6 , 10 , 1 , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (97 , 1 , 'Cybersource HOP' , NULL , NULL , 'Fill in the following parameters:\r\n\r\nmerchantID - Cybersource Merchant ID\r\nPublicKey - Public Key that can be acquired from file HOP.php\r\nPrivateKey - Private Key that can be acquired from file HOP.php\r\norderPage_serialNumber - Serial Number that can be acquired from file HOP.php or from Tools & Settings > Hosted Order Page > Security page in Business Center.\r\n\r\n------------------------------------------------------------------------------------------------\r\nSetting up your Cybersource Account:\r\n\r\nSecurity settings:\r\n1. First of all after logging into Cybersource Business Center visit menu Tools & Settings > Hosted Order Page > Security.\r\n2. In section \'Generate Security Script\' select option \'PHP\' and press button \'Submit\'. You\'ll get a proposal to download a security script HOP.php with your Public and Private Keys.\r\n3. Open file HOP.php and find function getPublicKey(). Copy all text between \' return " \' and \' " \' into the parameter PublicKey.\r\n4. Find function getPrivateKey(). Copy all text between \' return " \' and \' " \' into the parameter PrivateKey.\r\n5. Copy your Serial Number from section \'Existing Security Keys\' on the page Tools & Settings >  Hosted Order Page > Security and paste it into parameter orderPage_serialNumber.\r\n\r\nCommon settings: \r\n1. Open menu Tools & Settings > Hosted Order Page > Settings.\r\n2. In section Data Entry Fields > Payment Types select \'Credit Card\' and check all preferred types.\r\n3. In section Appearance > Receipt Page check option \'This URL is my custom receipt page.\' and fill in \'Receipt Response URL\' as http://www.example.com/order_final.php\r\n4. In section Appearance > Decline Page check option \'This URL is my custom decline page.\' and fill in \'Decline Response URL\' as http://www.example.com/order_final.php\r\n5. In section Notifications > Addresses and Options we recommend to check option \'Merchant POST URL\' and set it to http://www.example.com/payments/cybersource_notify.php\r\n\r\n------------------------------------------------------------------------------------------------------\r\nPayment URL:\r\n\r\nTest payment_url - https://orderpagetest.ic3.com/hop/orderform.jsp\r\nProduction payment_url - https://orderpage.ic3.com/hop/orderform.jsp' , NULL , NULL , NULL , NULL , './payments/cybersource_hop_process.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'GET' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , 4 , 6 , 10 , 1 , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (98 , 1 , 'CommWeb (Commonwealth Bank)' , NULL , NULL , 'To use the VPC you will need to know the Access Code and Secure Hash Secret. To obtain these details:\r\n- Log in to Merchant Administration using the details provided in your welcome email from CommWeb Software Support.\r\n- From the Admin menu click on the Operators link.\r\n- Create a new operator and assign them the privilege to \'Modify the Merchant Configuration\'.\r\n- Once you have created this operator, you can then logout from Administrator and login using the newly created operator.\r\n- From there, select Admin, followed by Configuration Details and then note the Access Code and Secure Hash Secret details.\r\n' , NULL , NULL , NULL , NULL , './payments/commweb_checkout.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'POST' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (99 , 1 , 'Okpay' , NULL , NULL , 'Please specify your &apos;okpay_account_email&apos; address where it says &apos;your.okpay@example.com&apos; and specify value for &apos;ok_receiver&apos; (your wallet id) parameter from your OKPAY account.\r\n\r\nActivating IPN.\r\nTo use IPN, you must activate IPN in your OKPAY account profile.\r\n \r\nFollow these steps to configure your account for IPN:\r\n1. Log in to your OKPAY account. \r\n2. Click the &apos;Option&apos; icon in &apos;Wallets and Currencies&apos; section. \r\n3. In &apos;Wallet Properties&apos; click &apos;Integration&apos; tab. \r\n4. Under &apos;Payment Notification (IPN)&apos;, click the &apos;Enable IPN&apos; checkbox. \r\n5. For the &apos;Notification URL&apos; use http://www.your-site.com/payments/okpay_ipn.php where &apos;your-site.com&apos; is your domain.\r\n6. Click Save.' , NULL , NULL , NULL , NULL , './payments/okpay_process.php' , 0 , NULL , NULL , 0 , NULL , NULL , 'POST' , 0 , 1 , 'https://www.okpay.com/process.html' , './payments/okpay_process.php' , NULL , NULL , NULL , 4 , 6 , 10 , 0 , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (100 , 1 , 'Amazon Checkout' , NULL , NULL , 'Please specify your access keys:\r\n1. In Seller Central, go to Integration > Access Key.\r\nThe Access Key page appears, showing your Access Key ID and a placeholder for your Secret Access Key.\r\n2. To view your Secret Access Key, click Show. Your Secret Access Key appears.\r\n\r\nSet Instant Order Processing Notifications:\r\n1. In Seller Central, click Settings > Checkout Pipeline Settings.\r\nThe Checkout Pipeline Settings page appears.\r\n2. Click the first Edit button on the page (the one immediately under the Instant Order Processing Notification Settings heading).\r\n3. In the Merchant URL box, type the URL for your website \r\nhttp://www.yoursite.com/payments/amazon_ipn.php\r\n(NOTE: replace www.yoursite.com with your real site URL and shop path)' , NULL , NULL , NULL , NULL , './payments/amazon_checkout.php' , NULL , NULL , NULL , NULL , NULL , NULL , 'POST' , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (101 , 1 , 'Payson Direct v1.15' , 'Payson Direct' , NULL , 'Required parameters\r\n\'reciever_email\' - your Payson account email\r\n\'api_user_id\' - your application ID\r\n\'api_pwd\' - your application md5 key\r\n\'memo\' - message shown in \'Description\' field on Payson payment page (max 128 characters).\r\n----------------------------------------------------------------------------------------------\r\nLive or Test account\r\n1) For test environment specify in \'forward_url\' \r\nhttps://test-www.payson.se/paysecure/\r\nIn \'Advanced URL\'\r\nhttps://test-api.payson.se/1.0/Pay/\r\n\r\n2) For live environment specify in \'forward_url\' \r\nhttps://www.payson.se/paySecure/\r\nIn \'Advanced URL\'\r\nhttps://api.payson.se/1.0/Pay/\r\n----------------------------------------------------------------------------------------------\r\nOptional parameters\r\n\'localeCode\' - language on Payson payment page, could be SV, EN or FI\r\n\'currencyCode\' - the currency used for the payment, could be SEK for Swedish krona or EUR for Euro (default currency)\r\n----------------------------------------------------------------------------------------------\r\nNotes\r\nRemember to activate First Name, Last Name and Email as required fields on www.yoursite/admin/admin_order_info.php\r\n\r\nSingle payment transaction (including VAT) must not exceed $10,000 USD in any currency.' , NULL , NULL , NULL , NULL , './payments/payson115_process.php' , 0 , NULL , NULL , 0 , NULL , NULL , 'POST' , 0 , 1 , 'https://api.payson.se/1.0/Pay/' , './payments/payson115_process.php' , NULL , NULL , NULL , 4 , 6 , 10 , 0 , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , 0 , 0 , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (102 , 1 , 'Stripe Checkout' , NULL , NULL , 'Download PHP Stripe library from https://stripe.com/docs/libraries\r\nunzip files and copy them to your viart \'payments/stripe/\' subfolder.\r\n\r\nPlease specify parameters below:\r\ndata-key\tYour publishable key (test or live).\r\ndata-secret-key\tYour secret key (test or live).\r\ndata-image - A relative URL pointing to a square image of your brand or product. The recommended minimum size is 128x128px.\r\ndata-name\t- The name of your company or website.\r\ndata-description\t- A description of the product or service being purchased.' , NULL , NULL , NULL , NULL , './payments/stripe_checkout.php' , NULL , NULL , 0 , NULL , NULL , NULL , 'GET' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (103 , 1 , 'Stripe Direct' , NULL , NULL , 'Download PHP Stripe library from https://stripe.com/docs/libraries\r\nunzip files and copy them to your viart \'payments/stripe/\' subfolder.\r\n\r\nPlease specify parameters below:\r\nsecret_key - Your secret key (test or live)' , NULL , NULL , NULL , NULL , 'order_confirmation.php' , NULL , NULL , 0 , NULL , NULL , NULL , 'GET' , 0 , 1 , 'https://api.stripe.com/v1/charges' , './payments/stripe_direct.php' , NULL , NULL , NULL , 4 , 6 , 10 , 1 , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;
INSERT INTO va_payment_systems (payment_id,payment_order,payment_name,user_payment_name,payment_info,payment_notes,payment_type,payment_code,payment_php_lib,validation_php_lib,payment_url,processing_time,processing_fee,processing_tax_free,fee_type,fee_min_goods,fee_max_goods,submit_method,recurring_method,is_advanced,advanced_url,advanced_php_lib,capture_php_lib,refund_php_lib,void_php_lib,success_status_id,pending_status_id,failure_status_id,failure_action,is_active,is_default,is_call_center,allowed_user_edit,sites_all,non_logged_users,user_types_all,item_types_all,currencies_all,image_small,image_small_alt,image_large,image_large_alt,fast_checkout_active,fast_checkout_image,fast_checkout_width,fast_checkout_height,fast_checkout_alt,order_total_min,order_total_max,countries_all) VALUES (104 , 1 , 'SecureTrading' , NULL , NULL , 'SecureTrading STPP PaymentPages' , NULL , NULL , NULL , NULL , 'payments/securetrading_format.php' , NULL , NULL , 0 , NULL , NULL , NULL , 'POST' , 0 , 0 , NULL , NULL , NULL , NULL , NULL , 4 , 6 , 8 , 0 , 0 , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 1 , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , NULL , NULL , NULL , 1 ) ;






INSERT INTO va_polls (poll_id,poll_type,question,is_active,date_added,total_votes) VALUES (1 , 1 , 'As gas prices continue to escalate, how close are you to purchasing an electric car or a gas/electric hybrid car?' , 1 , '2005-09-06 20:00:00' , 0 ) ;


INSERT INTO va_polls_options (poll_option_id,poll_id,option_description,is_default_value) VALUES (1 , 1 , 'An electric or hybrid car will definitely be my next auto purchase. ' , 0 ) ;
INSERT INTO va_polls_options (poll_option_id,poll_id,option_description,is_default_value) VALUES (2 , 1 , 'I will purchase one provided they are comparably priced to gasoline cars. ' , 0 ) ;
INSERT INTO va_polls_options (poll_option_id,poll_id,option_description,is_default_value) VALUES (3 , 1 , 'I doubt this will be a viable technology for some time to come. ' , 0 ) ;
INSERT INTO va_polls_options (poll_option_id,poll_id,option_description,is_default_value) VALUES (4 , 1 , 'I like the size, style and horsepower of gas-powered cars; I don\'t intend to ever go electric.  ' , 0 ) ;
INSERT INTO va_polls_options (poll_option_id,poll_id,option_description,is_default_value) VALUES (5 , 1 , 'I already own one of these cars. ' , 0 ) ;
INSERT INTO va_polls_options (poll_option_id,poll_id,option_description,is_default_value) VALUES (6 , 1 , 'Not sure. ' , 0 ) ;








INSERT INTO va_profiles_types (profile_type_id,profile_type_name,show_for_user,is_profile_default,is_looking_default) VALUES (1 , 'Male' , 1 , 1 , 0 ) ;
INSERT INTO va_profiles_types (profile_type_id,profile_type_name,show_for_user,is_profile_default,is_looking_default) VALUES (2 , 'Female' , 1 , 0 , 1 ) ;










INSERT INTO va_quotes_statuses (status_id,status_name,status_icon,html_start,html_end,notes) VALUES (1 , 'New' , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_quotes_statuses (status_id,status_name,status_icon,html_start,html_end,notes) VALUES (2 , 'Quoted' , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_quotes_statuses (status_id,status_name,status_icon,html_start,html_end,notes) VALUES (3 , 'Paid' , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_quotes_statuses (status_id,status_name,status_icon,html_start,html_end,notes) VALUES (4 , 'In progress' , NULL , NULL , NULL , NULL ) ;
















INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (1 , 1 , 1 , '2003-05-23 00:00:00' , 'Fixed bug when FAQ articles has some redundant spaces in HTML \\r\\n' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (2 , 1 , 1 , '2003-05-23 00:00:00' , 'Fixed bug when News articles maintenance didn\'t allow entering large title containing HTML tags' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (3 , 1 , 2 , '2003-01-10 00:00:00' , 'Added Postgre SQL and ODBC support ' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (4 , 1 , 2 , '2003-01-10 00:00:00' , 'Added search box to Sales Orders maintenance ' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (5 , 1 , 2 , '2003-01-10 00:00:00' , 'Added \'Page sources\' options for Web pages maintenance \\r\\n' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (6 , 1 , 2 , '2003-01-10 00:00:00' , 'Added \'Remember me\' option to Login page ' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (7 , 1 , 2 , '2003-01-10 00:00:00' , 'Added search box to \'Countries maintenance\' ' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (8 , 1 , 1 , '2003-01-10 00:00:00' , 'Fixed bug when countries listbox wasn\'t sorted by \'Country Order\' ' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (9 , 1 , 1 , '2003-01-10 00:00:00' , 'Fixed typo in Web pages section of Administration \\r\\n' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (10 , 1 , 1 , '2003-01-10 00:00:00' , 'Fixed bug when two Administrators could not access Administration simultaneously ' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (11 , 1 , 1 , '2003-01-10 00:00:00' , 'Fixed bug when Final Checkout page sent two Administrator notifications instead of one \\r\\n' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (12 , 1 , 2 , '2003-01-23 00:00:00' , 'Added Forum system \\r\\n' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (13 , 1 , 2 , '2003-01-24 00:00:00' , 'Added Customizable Layouts support' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (14 , 1 , 2 , '2003-01-24 00:00:00' , 'Added \'Downloadable\' products support \\r\\n' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (15 , 1 , 2 , '2003-01-25 00:00:00' , 'Added ability to assign one product to multiple categories ' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (16 , 1 , 2 , '2003-01-25 00:00:00' , 'Added ability to reassign cases within Support System (only Enterprise Edition) ' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (17 , 1 , 2 , '2003-01-25 00:00:00' , 'Added customizable payment systems support ' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (18 , 1 , 2 , '2003-01-26 00:00:00' , 'Added ability to submit POST payment systems parameters ' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (19 , 1 , 2 , '2003-01-26 00:00:00' , 'Added option that allows hiding Reviews section at Product details page ' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (20 , 1 , 2 , '2003-01-27 00:00:00' , 'Added custom tags support to email notifications ' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (21 , 1 , 2 , '2003-01-27 00:00:00' , 'Added \'free shipping\' rules to Shipping settings ' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (22 , 1 , 1 , '2003-01-28 00:00:00' , 'Fixed bug when footer menu didn\'t hide when there were no menu items ' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (23 , 1 , 1 , '2003-01-28 00:00:00' , 'Fixed bug when sorting order wasn\'t preserved in \'Countries\' maintenance ' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (24 , 1 , 1 , '2003-01-28 00:00:00' , 'Fixed bug with complex date formats masks processing ' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (25 , 1 , 1 , '2003-01-28 00:00:00' , 'Fixed bug when product description had illegal characters ' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (26 , 1 , 1 , '2003-01-28 00:00:00' , 'Fixed bug when sending emails from some domains took more time then from others ' , 1 ) ;
INSERT INTO va_release_changes (change_id,release_id,change_type_id,change_date,change_desc,is_showing) VALUES (27 , 1 , 1 , '2003-01-29 00:00:00' , 'Fixed bug when some phone numbers were not considered to be valid during user registration ' , 1 ) ;


INSERT INTO va_release_types (type_id,type_name) VALUES (1 , 'Upgrade' ) ;
INSERT INTO va_release_types (type_id,type_name) VALUES (2 , 'Update' ) ;
INSERT INTO va_release_types (type_id,type_name) VALUES (3 , 'New Version' ) ;


INSERT INTO va_releases (release_id,item_id,release_type_id,release_date,release_title,version,release_desc,download_type,path_to_file,is_showing,show_on_index) VALUES (1 , 1 , 1 , '2003-12-24 00:00:00' , 'New Upgrade for C2S' , '2.0.1.11' , NULL , 1 , '/etc/c2s/c2s.tgz' , 0 , 0 ) ;












INSERT INTO va_saved_types (type_id,type_name,type_desc,is_active,allowed_search) VALUES (1 , 'Wish List' , NULL , 1 , 0 ) ;


INSERT INTO va_search_engines (engine_id,engine_name,keywords_parameter,referer_regexp,user_agent_regexp,ip_regexp) VALUES (1 , 'Google' , 'q' , '/http:\\/\\/www\\.google\\.\\w{2,4}(\\.\\w{2})?\\//i' , '/Googlebot.*google\\.com/i' , NULL ) ;
INSERT INTO va_search_engines (engine_id,engine_name,keywords_parameter,referer_regexp,user_agent_regexp,ip_regexp) VALUES (2 , 'Yahoo' , 'p' , '/search\\.yahoo\\.com\\//i' , '/Yahoo.*Slurp.*yahoo\\.com/i' , NULL ) ;
INSERT INTO va_search_engines (engine_id,engine_name,keywords_parameter,referer_regexp,user_agent_regexp,ip_regexp) VALUES (3 , 'MSN' , 'q' , '/search(\\.\\w+)?\\.msn\\.\\w{2,4}(\\.\\w{2})?\\//i' , '/msnbot.*msn\\.com/i' , NULL ) ;
INSERT INTO va_search_engines (engine_id,engine_name,keywords_parameter,referer_regexp,user_agent_regexp,ip_regexp) VALUES (4 , 'Ask.com' , 'q' , '/\\.ask\\.com\\//i' , '/Ask.*Jeeves\\/Teoma.*ask\\.com/i' , NULL ) ;
INSERT INTO va_search_engines (engine_id,engine_name,keywords_parameter,referer_regexp,user_agent_regexp,ip_regexp) VALUES (5 , 'Gigablast' , 'q' , '/gigablast\\.com\\//i' , '/Gigabot.*gigablast\\.com/i' , NULL ) ;
INSERT INTO va_search_engines (engine_id,engine_name,keywords_parameter,referer_regexp,user_agent_regexp,ip_regexp) VALUES (6 , 'Exalead' , 'q' , '/exalead\\.com\\//i' , '/Exabot/i' , NULL ) ;
INSERT INTO va_search_engines (engine_id,engine_name,keywords_parameter,referer_regexp,user_agent_regexp,ip_regexp) VALUES (7 , 'LookSmart' , 'qt' , '/looksmart\\.(com|net)\\//i' , '/ZyBorg.*looksmart.*WISEnutbot\\.com/i' , NULL ) ;
INSERT INTO va_search_engines (engine_id,engine_name,keywords_parameter,referer_regexp,user_agent_regexp,ip_regexp) VALUES (8 , 'Zibb' , 'q' , '/zibb\\.com\\//i' , '/Zibber.*zibb\\.com/i' , NULL ) ;
INSERT INTO va_search_engines (engine_id,engine_name,keywords_parameter,referer_regexp,user_agent_regexp,ip_regexp) VALUES (9 , 'Entireweb' , 'q' , '/entireweb\\.com\\//i' , '/Speedy.*Spider.*entireweb\\.com/i' , NULL ) ;
INSERT INTO va_search_engines (engine_id,engine_name,keywords_parameter,referer_regexp,user_agent_regexp,ip_regexp) VALUES (10 , 'Seekport' , 'query' , '/seekport\\.\\w{2,4}(\\.\\w{2})?\\//i' , '/Seekbot.*Seekbot\\.net/i' , NULL ) ;
INSERT INTO va_search_engines (engine_id,engine_name,keywords_parameter,referer_regexp,user_agent_regexp,ip_regexp) VALUES (11 , 'Become' , 'q' , '/become.com\\//i' , '/BecomeBot.*become\\.com/i' , NULL ) ;


INSERT INTO va_shipping_modules (shipping_module_id,shipping_module_name,user_module_name,module_notes,admin_order_class,user_order_class,is_external,php_external_lib,external_url,tracking_url,is_active,is_default,is_call_center,cost_add_percent) VALUES (1 , 'Custom Shipping' , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 1 , 1 , 1 , NULL ) ;
INSERT INTO va_shipping_modules (shipping_module_id,shipping_module_name,user_module_name,module_notes,admin_order_class,user_order_class,is_external,php_external_lib,external_url,tracking_url,is_active,is_default,is_call_center,cost_add_percent) VALUES (2 , 'UPS' , NULL , 'United Parcel Service (UPS)\r\n\r\nObtain UserID, Password and Developer\'s Key from your country\'s UPS E-Business web site. (See the\r\n\'Security Elements\' section in the Product Overview chapter.)\r\n\r\nObtain Access Key. (See \'Security Elements\' in the Product Overview section.)\r\n\r\nFor Signature Tracking and Shipping, follow the product certification process identified in the Customer\r\nIntegration Environment section.\r\n\r\nFill in the parameters list below and set UPS system as active and begin receive you online shipping rates.\r\n\r\nShipperPostalCode - The shipper\'s postal code. Required if country utilizes postal codes (i.e. US & PR)\r\n\r\nShipperCountryCode - The UPS billing code for the shipper\'s country. See Origin Country Information\r\n table. Default = US.\r\n\r\nShipFromPostalCode - The sender\'s postal code. Required if Ship To country code is US. Must supply\r\n either the sender\'s city or postal code.\r\n\r\nShipFromCountryCode - The UPS billing code representing the sender\'s country. See Appendix C for\r\n UPS Country Code table. Default = US.\r\n\r\nPackageWeightUnitOfMeasurement - The code representing the unit of measure associated with the \r\n package weight. LBS = Pounds (default) KGS = Kilograms\r\n\r\nPickupType:\r\n 01 - Daily Pickup\r\n 03 - Customer Counter\r\n 06 - One Time Pickup\r\n 07 - On Call Air Pickup\r\n 11 - Suggested Retail Rates (UPS Store)\r\n 19 - Letter Center\r\n 20 - Air Service Center\r\n\r\nPackagingType:\r\n 01 - UPS letter/ UPS Express Envelope\r\n 02 - Package\r\n 03 - UPS Tube\r\n 04 - UPS Pak\r\n 21 - UPS Express Box\r\n 24 - UPS 25Kg Box\r\n 25 - UPS 10Kg Box\r\n\r\nFor PackagingType 02 or 21 you must fill PackagingTypeUnitOfMeasurement, PackageLength,\r\n PackageWidth and PackageHeight. \r\n\r\nPackagingTypeUnitOfMeasurement - The code for the unit of measure associated with the package\r\n dimensions. IN = Inches (default)\r\n\r\nPackageLength - The length of the package used to determine dimensional weight. Valid values are\r\n between 1 and 108.00.\r\n\r\nPackageWidth - The width of the package used to determine dimensional weight. Valid values are\r\n between 1 and 108.00.\r\n\r\nPackageHeight - The height of the package used to determine dimensional weight. Valid values are\r\n between 1and 108.00.\r\n\r\nYou may visit your country\'s UPS E-Business web site for more information about UPS Rates & Service Selection Tool. http://www.ups.com' , NULL , NULL , 1 , './shipping/ups.php' , 'https://www.ups.com/ups.app/xml/Rate' , 'http://wwwapps.ups.com/WebTracking/processInputRequest?HTMLVersion=5.0&error_carried=true&tracknums_displayed=1&TypeOfInquiryNumber=T&loc=en_US&AgreeToTermsAndConditions=yes&InquiryNumber1=' , 0 , NULL , NULL , NULL ) ;
INSERT INTO va_shipping_modules (shipping_module_id,shipping_module_name,user_module_name,module_notes,admin_order_class,user_order_class,is_external,php_external_lib,external_url,tracking_url,is_active,is_default,is_call_center,cost_add_percent) VALUES (3 , 'USPS V4' , NULL , 'United States Postal Service (USPS).\r\n\r\nTo use the USPS Web Tools you must be a registered user. Completing the registration process resulted in the receipt of your user ID, password, and test server URL. This test URL is already filled in.\r\n\r\nTo gain access to Live server, all users, including those registered for previous Web Tools use, must contact the Internet Customer Care Center (ICCC) for the URL to make Live calls. The ICCC will send the Live URL via e-mail. Fill live url in the field  External URL instead of present one.\r\n\r\nThis office is manned from 7:00 AM to 11:00 PM EST daily. \r\nE-mail address: icustomercare@usps.com \r\nTelephone: 1-800-344-7779 (7:00 AM to 11:00 PM EST daily)\r\n\r\nExternal URL:\r\nProduction - http://production.shippingapis.com/ShippingAPI.dll\r\nTesting - http://testing.shippingapis.com/ShippingAPI.dll\r\n\r\nCommon parameters:\r\n  USERID - required, use user ID provided with registration.\r\n  PASSWORD - is not required now, use password provided with registration if any.\r\n\r\nDomestic parameters:\r\n  ZipOrigination - required. Your ZIP code where dispath will be made from. ZIP code must be valid. Maximum characters allowed: 5.\r\n  Container - required, is only applicable for Express Mail and Priority Mail. Must contain one of the following valid packaging type names: "VARIABLE", "RECTANGULAR", "NONRECTANGULAR.\r\n  Size - required, must be one of the following: "REGULAR", "LARGE".\r\n  Machinable - optional. Applies only to Parcel Post. For Parcel Post, the value entered must be either "True" or "False".\r\n\r\nInternational parameters:\r\n  MailType - required, the following are valid international mail types: "package", "postcards or aerogrammes", "matter for the blind", "envelope".\r\n\r\nDimensions:\r\nTo capture the dimensional weight for Large Priority Mail pieces, RateV3 will require three new dimension tags for rectangular Priority Mail pieces: Length, Width, and Height; \r\nand four new dimension tags for non-rectangular pieces: Length, Width, Height, and Girth. \r\nShippers will specify in the existing Container tag whether a Large Priority Mail piece is rectangular or non-rectangular.\r\n\r\n\r\nPLEASE NOTE.\r\n\r\nUSPS service does not work for orders with weight more than 70 pounds. If you may receive orders that exceed this limit and you want to charge some shipping fee for them, don\'t forget to set up alternative Shipping Module with Shipping Types that will handle over-weighted orders.\r\n\r\nIf you use USPS shipping module as the only shipping module of the shop please use functionality to restrict order placing if the weight is not in the following bounds 0 -70. And if the order exceeds these bounds appropriate error message will be shown. This restriction can be set on Orders > Order Profile Page, Order Restrictions section, Weight setting.\r\n\r\n\r\nTesting:\r\nPlease note that for test purposes USPS permits only particular parameters\' values to be used for the query. So in the Test mode it expects to receive the following information about the package for domestic shipping:\r\n \r\nZipOrigination: 10022\r\nZipDestination: 20008 (is set during Checkout process by user)\r\nPounds: 10\r\nOunces: 5 (weight can be set for every product on  Administration > Products)\r\nSize: LARGE\r\nMachinable: TRUE\r\n\r\nTo test international shipping use the following values:\r\n\r\nPounds: 2\r\nOunces: 0\r\nMailType: Package\r\nCountry: Albania\r\n\r\nor\r\n\r\nPounds: 0\r\nOunces: 1\r\nMailType: Postcards or Aerogrammes\r\nCountry: Algeria\r\n\r\nAll other parameters\' values won\'t work in the test environment.' , NULL , NULL , 1 , './shipping/usps_v4.php' , 'http://production.shippingapis.com/ShippingAPI.dll' , 'http://trkcnfrm1.smi.usps.com/PTSInternetWeb/InterLabelInquiry.do?formattedLabel=' , 0 , NULL , NULL , NULL ) ;
INSERT INTO va_shipping_modules (shipping_module_id,shipping_module_name,user_module_name,module_notes,admin_order_class,user_order_class,is_external,php_external_lib,external_url,tracking_url,is_active,is_default,is_call_center,cost_add_percent) VALUES (5 , 'CanadaPost' , NULL , 'Canada Post.\r\n\r\nTo use the CanadaPost Web Tools you must be a registered user. Completing the registration process resulted in the receipt of your CPCID.\r\n\r\nThis office is manned from Monday to Friday 8:00 AM to 6:00 PM. \r\nTelephone: 1-866-607-6301; (Outside of Canada\r\n1-416-979-8822)\r\nGeneral Information\r\n\r\nExternal URL:\r\nProduction - http://sellonline.canadapost.ca:30000\r\nTesting - http://sellonline.canadapost.ca:30000\r\n\r\nCommon parameters:\r\n  CPCID - required, use CPCID provided with registration.\r\n  For testing CPCID = CPC_DEMO_XML.\r\n\r\nDomestic parameters:\r\n  fromPostalCode - required. Your ZIP code where dispath will be made from. ZIP code must be valid. \r\n  price - total cost.\r\n  quantity - quantity package\r\n  heiht, width, length - size of package in cm\r\n  weight - size of kg\r\n  description - description in package\r\n\r\nTesting:\r\nPlease note that for test purposes CanadaPost permits only particular parameters\' values to be used for the query. So in the Test mode it expects to receive the following information about the package for domestic shipping:' , NULL , NULL , 0 , './shipping/canadapost.php' , 'http://sellonline.canadapost.ca:30000' , 'https://em.canadapost.ca/emo/basicPin.do?language=en' , 0 , NULL , NULL , NULL ) ;
INSERT INTO va_shipping_modules (shipping_module_id,shipping_module_name,user_module_name,module_notes,admin_order_class,user_order_class,is_external,php_external_lib,external_url,tracking_url,is_active,is_default,is_call_center,cost_add_percent) VALUES (7 , 'Australia Post' , NULL , 'Pickup_Postcode - Postcode of the pick-up address.\r\nWeight - Packed weight of each parcel in grams (kg).\r\nLength - Packed length of parcel in millimetres (cm).\r\nWidth - Packed width of parcel in millimetres (cm).\r\nHeight - Packed height of parcel in millimetres (cm).\r\n\r\nShipping Code - The type of delivery service\r\nonly for Australia (country code "AU"):\r\n"STANDARD"= Regular Parcels\r\n"EXPRESS" = Express Parcels\r\nOutside Australia (country code rather than "AU"):\r\n"AIR" = Air Mail\r\n"SEA" = Sea Mail\r\n"ECI_D" = Express Courier International Document\r\n"ECI_M" = Express Courier International Merchandise\r\n"EPI" = Express Post International' , NULL , NULL , 1 , './shipping/australia_post.php' , 'http://drc.edeliver.com.au/ratecalc.asp' , NULL , 0 , 0 , 0 , NULL ) ;
INSERT INTO va_shipping_modules (shipping_module_id,shipping_module_name,user_module_name,module_notes,admin_order_class,user_order_class,is_external,php_external_lib,external_url,tracking_url,is_active,is_default,is_call_center,cost_add_percent) VALUES (8 , 'FedEx V10' , NULL , 'The FedEx sign-up process requires some interaction with the company. FedEx requires that developers and their applications be certified. The first step is to register for a FedEx Ship Manager Direct account by visiting https://www.fedex.com/cgi-bin/shipapiDownload.cgi?link=4&first=y and completing the form. In addition to the standard contact information, you must indicate how you intend to use their online tools. If you\'re developing e-commerce solutions for multiple clients, choose the Consultant option, otherwise select Corporate Developer. Select the communication path used - FSM Direct. Finally, you\'ll need to select the Data Format desired - XML Tools. After you\'ve registered, you\'ll be given an address to a shipping server to be used in your testing phases, please apply it to \'External URL\' field below.\r\n\r\nYou\'ll need to next contact FedEx by e-mail (websupport@fedex.com) to request to have you or your client\'s account set up in a test environment. You\'ll need to specify whether you\'re using the Express, Ground, or both services. As well you can include source code from the shipping folder to contact the FedEx Direct. Although the company\'s documentation indicates a 24 hour turn-around for approval, representatives indicate the wait can go up to 72 hours.\r\n\r\nAfter you\'ve been approved, the next task is to subscribe to FedEx to get a meter number. Meter numbers are programmatically generated. Before sending subscription request please change the parameters below. \r\nAccountNumber - you FedEx account number.\r\nPersonName, CompanyName, Department, PhoneNumber, PagerNumber, FaxNumber, EmailAddress - your contact information\r\nLine1, Line2, City, StateOrProvinceCode, PostalCode, CountryCode - your address information\r\nWhen all the data are ready you can go to the page at http://your_site_url/shipping/fedex_subscription.php and send subscription request.\r\nAfter successfully running the FedEx subscription request a meter number will be returned, add this value to the MeterNumber field below.\r\n\r\nPlease update the rest of fields below.\r\nStateOrProvinceCode - Represents the state or province from which the shipment will be originating. (Required only if Sender Country Code is US or CA)\r\nPostalCode - Represents the postal code from which the shipment will be originating. (Required only if Sender Country Code is US or CA. May be required for other postal-aware countries when rating an internationally shipment.)\r\nCountryCode - Represents the country from which the shipment will be originating. (Required)\r\nReturnShipmentIndicator - NONRETURN, PRINTRETURNLABEL, EMAILLABEL\r\nDropoffType - REGULARPICKUP, REQUESTCOURIER, DROPBOX, BUSINESSSERVICECENTER, STATION\r\nPackaging - FEDEXENVELOPE, FEDEXPAK, FEDEXBOX, FEDEXTUBE, FEDEX10KGBOX, FEDEX25KGBOX, YOURPACKAGING\r\nFor YOURPACKAGING you also need specify package sizes: Length, Width, Height, Units (IN, CM)\r\nWeightUnits - LBS, KGS\r\nSet FedEx system as active and receive you online shipping rates.\r\n\r\nAfter your application testing is done, contact FedEx again by e-mail (websupport@fedex.com) or phone (800.810.9073) to initiate the certification process. For retrieving rates, you\'ll be asked to state exactly how you will be using their API; you can also request to be moved to the production environment in a specific timeframe. Once certified, FedEx will provide you with a FedEx server address to be used when you re-subscribe to receive your production meter number.' , NULL , NULL , 1 , './shipping/fedex_v10.php' , 'https://ws.fedex.com:443/web-services' , 'http://www.fedex.com/Tracking?ascend_header=1&clienttype=dotcom&cntry_code=us&language=english&tracknumbers=' , 0 , 0 , 0 , NULL ) ;
INSERT INTO va_shipping_modules (shipping_module_id,shipping_module_name,user_module_name,module_notes,admin_order_class,user_order_class,is_external,php_external_lib,external_url,tracking_url,is_active,is_default,is_call_center,cost_add_percent) VALUES (9 , 'FedEx' , NULL , 'The FedEx sign-up process requires some interaction with the company. FedEx requires that developers and their applications be certified. The first step is to register for a FedEx Ship Manager Direct account by visiting https://www.fedex.com/cgi-bin/shipapiDownload.cgi?link=4&first=y and completing the form. In addition to the standard contact information, you must indicate how you intend to use their online tools. If you\'re developing e-commerce solutions for multiple clients, choose the Consultant option, otherwise select Corporate Developer. Select the communication path used - FSM Direct. Finally, you\'ll need to select the Data Format desired - XML Tools. After you\'ve registered, you\'ll be given an address to a shipping server to be used in your testing phases, please apply it to \'External URL\' field below.\r\n\r\nYou\'ll need to next contact FedEx by e-mail (websupport@fedex.com) to request to have you or your client\'s account set up in a test environment. You\'ll need to specify whether you\'re using the Express, Ground, or both services. As well you can include source code from the shipping folder to contact the FedEx Direct. Although the company\'s documentation indicates a 24 hour turn-around for approval, representatives indicate the wait can go up to 72 hours.\r\n\r\nAfter you\'ve been approved, the next task is to subscribe to FedEx to get a meter number. Meter numbers are programmatically generated. Before sending subscription request please change the parameters below. \r\nAccountNumber - you FedEx account number.\r\nPersonName, CompanyName, Department, PhoneNumber, PagerNumber, FaxNumber, EmailAddress - your contact information\r\nLine1, Line2, City, StateOrProvinceCode, PostalCode, CountryCode - your address information\r\nWhen all the data are ready you can go to the page at http://your_site_url/shipping/fedex_subscription.php and send subscription request.\r\nAfter successfully running the FedEx subscription request a meter number will be returned, add this value to the MeterNumber field below.\r\n\r\nPlease update the rest of fields below.\r\nStateOrProvinceCode - Represents the state or province from which the shipment will be originating. (Required only if Sender Country Code is US or CA)\r\nPostalCode - Represents the postal code from which the shipment will be originating. (Required only if Sender Country Code is US or CA. May be required for other postal-aware countries when rating an internationally shipment.)\r\nCountryCode - Represents the country from which the shipment will be originating. (Required)\r\nReturnShipmentIndicator - NONRETURN, PRINTRETURNLABEL, EMAILLABEL\r\nDropoffType - REGULARPICKUP, REQUESTCOURIER, DROPBOX, BUSINESSSERVICECENTER, STATION\r\nPackaging - FEDEXENVELOPE, FEDEXPAK, FEDEXBOX, FEDEXTUBE, FEDEX10KGBOX, FEDEX25KGBOX, YOURPACKAGING\r\nFor YOURPACKAGING you also need to specify package sizes: Length, Width, Height dimensions and Units (IN, CM), BUT they only need to be 1, 1, 1 (Length, Width, Height). Fedex does not require the exact dimensions for every box using YOURPACKAGING.\r\nWeightUnits - LBS, KGS\r\nSet FedEx system as active and receive you online shipping rates.\r\n\r\nAfter your application testing is done, contact FedEx again by e-mail (websupport@fedex.com) or phone (800.810.9073) to initiate the certification process. For retrieving rates, you\'ll be asked to state exactly how you will be using their API; you can also request to be moved to the production environment in a specific timeframe. Once certified, FedEx will provide you with a FedEx server address to be used when you re-subscribe to receive your production meter number.' , NULL , NULL , 1 , './shipping/fedex.php' , 'https://sslserver.fedex.com:443/GatewayDC' , 'http://www.fedex.com/Tracking?ascend_header=1&clienttype=dotcom&cntry_code=us&language=english&tracknumbers=' , 0 , 0 , 0 , NULL ) ;


INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (8 , 2 , 'user_id' , '!Enter your USERID here!' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (9 , 2 , 'password' , '!Enter your PASSWORD here!' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (10 , 2 , 'access_license_number' , '!Enter your Access License Number here!' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (11 , 2 , 'ShipperPostalCode' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (12 , 2 , 'ShipperCountryCode' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (13 , 2 , 'ShipFromPostalCode' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (14 , 2 , 'ShipFromCountryCode' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (15 , 2 , 'PickupType' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (16 , 2 , 'PackagingType' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (21 , 2 , 'PackageWeightUnitOfMeasurement' , 'LBS' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (50 , 2 , 'PackagingTypeUnitOfMeasurement' , 'IN' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (51 , 2 , 'PackageLength' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (52 , 2 , 'PackageWidth' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (53 , 2 , 'PackageHeight' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (54 , 3 , 'USERID' , '!Enter your USERID here!' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (55 , 3 , 'PASSWORD' , '!Enter your PASSWORD here!' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (56 , 3 , 'ZipOrigination' , '!Enter your ZipOrigination here!' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (57 , 3 , 'Container' , 'VARIABLE' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (58 , 3 , 'Size' , 'REGULAR' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (59 , 3 , 'Machinable' , 'true' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (60 , 3 , 'MailType' , 'Package' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (61 , 3 , 'Width' , '15' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (62 , 3 , 'Length' , '30' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (63 , 3 , 'Height' , '15' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (64 , 3 , 'Girth' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (65 , 5 , 'language' , 'en' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (66 , 5 , 'CPCID' , 'CPC_DEMO_XML' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (67 , 5 , 'fromPostalCode' , 'G1P2H5' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (68 , 5 , 'AroundTime' , '240' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (69 , 5 , 'price' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (70 , 5 , 'quantity' , '1' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (71 , 5 , 'weight' , '10' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (72 , 5 , 'length' , '10' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (73 , 5 , 'width' , '10' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (74 , 5 , 'height' , '10' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (75 , 5 , 'description' , 'Canada Post' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (76 , 5 , 'city' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (77 , 5 , 'State' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (78 , 5 , 'country' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (79 , 5 , 'postalCode' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (103 , 7 , 'Pickup_Postcode' , '!Enter here your Pickup_Postcode!' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (104 , 7 , 'Weight' , '1' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (105 , 7 , 'Length' , '10' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (106 , 7 , 'Width' , '10' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (107 , 7 , 'Height' , '10' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (108 , 8 , 'Key' , '!Type here your key!' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (109 , 8 , 'Password' , '!Type here your password!' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (110 , 8 , 'AccountNumber' , '!Type here your account number!' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (111 , 8 , 'MeterNumber' , '!Type here your metter number!' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (112 , 8 , 'StreetLines' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (113 , 8 , 'City' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (114 , 8 , 'StateOrProvinceCode' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (115 , 8 , 'PostalCode' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (116 , 8 , 'CountryCode' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (117 , 8 , 'ServiceId' , 'crs' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (118 , 8 , 'Major' , '10' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (119 , 8 , 'Intermediate' , '0' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (120 , 8 , 'Minor' , '0' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (121 , 8 , 'PackagingType' , 'YOUR_PACKAGING' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (122 , 8 , 'DropoffType' , 'REGULAR_PICKUP' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (123 , 8 , 'Residential' , 'true' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (124 , 8 , 'RateType' , 'RATED_ACCOUNT' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (125 , 8 , 'DimensionsUnit' , 'IN' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (126 , 8 , 'Length' , '10' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (127 , 8 , 'Width' , '10' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (128 , 8 , 'Height' , '10' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (129 , 8 , 'WeightUnits' , 'LB' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (130 , 8 , 'WeightValue' , '20' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (131 , 9 , 'CustomerTransactionIdentifier' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (132 , 9 , 'AccountNumber' , '!Type here your account number!' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (133 , 9 , 'MeterNumber' , '!Type here your subscription number!' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (134 , 9 , 'CarrierCode' , 'FDXE' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (135 , 9 , 'PersonName' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (136 , 9 , 'CompanyName' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (137 , 9 , 'Department' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (138 , 9 , 'PhoneNumber' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (139 , 9 , 'PagerNumber' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (140 , 9 , 'FaxNumber' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (141 , 9 , 'EMailAddress' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (142 , 9 , 'Line1' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (143 , 9 , 'Line2' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (144 , 9 , 'City' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (145 , 9 , 'StateOrProvinceCode' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (146 , 9 , 'PostalCode' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (147 , 9 , 'CountryCode' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (148 , 9 , 'ReturnShipmentIndicator' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (149 , 9 , 'DropoffType' , NULL , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (150 , 9 , 'Packaging' , 'YOURPACKAGING' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (151 , 9 , 'WeightUnits' , 'LBS' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (152 , 9 , 'PayorType' , 'SENDER' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (153 , 9 , 'Length' , '1' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (154 , 9 , 'Width' , '1' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (155 , 9 , 'Height' , '1' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (156 , 9 , 'Units' , 'IN' , 0 ) ;
INSERT INTO va_shipping_modules_parameters (parameter_id,shipping_module_id,parameter_name,parameter_source,not_passed) VALUES (157 , 9 , 'PackageCount' , '1' , 0 ) ;


INSERT INTO va_shipping_rules (shipping_rule_id,shipping_rule_desc,is_country_restriction) VALUES (1 , 'Can ship anywhere' , NULL ) ;
INSERT INTO va_shipping_rules (shipping_rule_id,shipping_rule_desc,is_country_restriction) VALUES (2 , 'Can ship only within USA' , 1 ) ;
INSERT INTO va_shipping_rules (shipping_rule_id,shipping_rule_desc,is_country_restriction) VALUES (3 , 'Can ship only within United Kingdom' , 1 ) ;


INSERT INTO va_shipping_rules_countries (shipping_rule_id,country_id) VALUES (2 , 228 ) ;
INSERT INTO va_shipping_rules_countries (shipping_rule_id,country_id) VALUES (3 , 77 ) ;


INSERT INTO va_shipping_times (shipping_time_id,shipping_time_desc,availability_time) VALUES (1 , 'Usually ships in 2 to 3 days' , NULL ) ;
INSERT INTO va_shipping_times (shipping_time_id,shipping_time_desc,availability_time) VALUES (2 , 'Usually ships in 24 hours' , NULL ) ;


INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (28 , 2 , 1 , '01' , 'UPS Next Day Air' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (29 , 2 , 1 , '02' , 'UPS 2nd Day Air' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (30 , 2 , 1 , '03' , 'UPS Ground' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (31 , 2 , 1 , '07' , 'UPS Worldwide ExpressSM' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (32 , 2 , 1 , '08' , 'UPS Worldwide ExpeditedSM' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (33 , 2 , 1 , '11' , 'UPS Standard' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (34 , 2 , 1 , '12' , 'UPS 3 Day Select' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (35 , 2 , 1 , '13' , 'UPS Next Day Air Saver' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (36 , 2 , 1 , '14' , 'UPS Next Day Air Early A.M.' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (37 , 2 , 1 , '54' , 'UPS Worldwide Express PlusSM' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (38 , 2 , 1 , '01' , 'UPS Express' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (39 , 2 , 1 , '02' , 'UPS Expedited' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (40 , 2 , 1 , '13' , 'UPS Express Saver' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (41 , 2 , 1 , '14' , 'UPS Express Early A.M.' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (42 , 2 , 1 , '07' , 'UPS Express' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (43 , 2 , 1 , '65' , 'UPS Express Saver' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (44 , 2 , 1 , '59' , 'UPS 2nd Day Air A.M.' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (45 , 2 , 1 , '08' , 'UPS Expedited' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (46 , 2 , 1 , '54' , 'UPS Express Plus' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (47 , 3 , 1 , '/^Priority Mail Express \\d+-Day$/is' , 'USPS Priority Mail Express' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (48 , 3 , 1 , '/^Priority Mail Express \\d+-Day Hold For Pickup$/is' , 'USPS Priority Mail Express Hold For Pickup' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (49 , 3 , 1 , '/^Priority Mail Express \\d+-Day Flat Rate Boxes$/is' , 'USPS Priority Mail Express Flat Rate Boxes' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (50 , 3 , 1 , '/^Priority Mail Express \\d+-Day Flat Rate Boxes Hold For Pickup$/is' , 'USPS Priority Mail Express Flat Rate Boxes Hold For Pickup' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (51 , 3 , 1 , '/^Priority Mail Express \\d+-Day Flat Rate Envelope$/is' , 'USPS Priority Mail Express Flat Rate Envelope' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (52 , 3 , 1 , '/^Priority Mail Express \\d+-Day Flat Rate Envelop Hold For Pickup$/is' , 'USPS Priority Mail Express Flat Rate Envelope Hold For Pickup' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (53 , 3 , 1 , '/^Priority Mail Express \\d+-Day Legal Flat Rate Envelope$/is' , 'USPS Priority Mail Express Legal Flat Rate Envelope' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (54 , 3 , 1 , '/^Priority Mail Express \\d+-Day Legal Flat Rate Envelope Hold For Pickup$/is' , 'USPS Priority Mail Express Legal Flat Rate Envelope Hold For Pickup' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (55 , 3 , 1 , '/^Priority Mail Express \\d+-Day Padded Flat Rate Envelope$/is' , 'USPS Priority Mail Express Padded Flat Rate Envelope' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (56 , 3 , 1 , '/^Priority Mail \\d+-Day$/is' , 'USPS Priority Mail' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (57 , 3 , 1 , '/^Priority Mail \\d+-Day Large Flat Rate Box$/is' , 'USPS Priority Mail Large Flat Rate Box' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (58 , 3 , 1 , '/^Priority Mail \\d+-Day Medium Flat Rate Box$/is' , 'USPS Priority Mail Medium Flat Rate Box' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (59 , 3 , 1 , '/^Priority Mail \\d+-Day Small Flat Rate Box$/is' , 'USPS Priority Mail Small Flat Rate Box' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (60 , 3 , 1 , '/^Priority Mail \\d+-Day Flat Rate Envelope$/is' , 'USPS Priority Mail Flat Rate Envelope' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (61 , 3 , 1 , '/^Priority Mail \\d+-Day Legal Flat Rate Envelope$/is' , 'USPS Priority Mail Legal Flat Rate Envelope' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (62 , 3 , 1 , '/^Priority Mail \\d+-Day Padded Flat Rate Envelope$/is' , 'USPS Priority Mail Padded Flat Rate Envelope' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (63 , 3 , 1 , '/^Priority Mail \\d+-Day Gift Card Flat Rate Envelope$/is' , 'USPS Priority Mail Gift Card Flat Rate Envelope' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (64 , 3 , 1 , '/^Priority Mail \\d+-Day Small Flat Rate Envelope$/is' , 'USPS Priority Mail Small Flat Rate Envelope' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (65 , 3 , 1 , '/^Priority Mail \\d+-Day Window Flat Rate Envelope$/is' , 'USPS Priority Mail Window Flat Rate Envelope' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (66 , 3 , 1 , 'First-Class Mail Package' , 'USPS First-Class Mail Package' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (67 , 3 , 1 , 'First-Class Mail Large Envelope' , 'USPS First-Class Mail Large Envelope' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (68 , 3 , 1 , 'First-Class Mail Letter' , 'USPS First-Class Mail Letter' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (69 , 5 , 1 , '1020' , 'Canada Post Expedited' , NULL , NULL , NULL , NULL , 0 , 20000 , 0 , 50 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (70 , 5 , 1 , '1040' , 'Canada Post Priority Courier' , NULL , NULL , NULL , NULL , 0 , 20000 , 0 , 50 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (71 , 5 , 1 , '1010' , 'Canada Post Regular' , NULL , NULL , NULL , NULL , 0 , 20000 , 0 , 50 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (72 , 5 , 1 , '1030' , 'Canada Post Xpresspost' , NULL , NULL , NULL , NULL , 0 , 20000 , 0 , 50 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (73 , 5 , 2 , '2005' , 'Canada Post Small Packets Surface' , NULL , NULL , NULL , NULL , 0 , 20000 , 0 , 50 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (74 , 5 , 2 , '2015' , 'Canada Post Small Packets Air' , NULL , NULL , NULL , NULL , 0 , 20000 , 0 , 50 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (75 , 5 , 2 , '2020' , 'Canada Post Expedited US Business Contract' , NULL , NULL , NULL , NULL , 0 , 20000 , 0 , 50 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (76 , 5 , 2 , '2025' , 'Canada Post Expedited US Commercial' , NULL , NULL , NULL , NULL , 0 , 20000 , 0 , 50 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (77 , 5 , 2 , '2030' , 'Canada Post Xpresspost USA' , NULL , NULL , NULL , NULL , 0 , 20000 , 0 , 50 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (78 , 5 , 2 , '2040' , 'Canada Post Purolator International' , NULL , NULL , NULL , NULL , 0 , 20000 , 0 , 50 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (79 , 5 , 2 , '2050' , 'Canada Post Puropak International' , NULL , NULL , NULL , NULL , 0 , 20000 , 0 , 50 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (80 , 5 , 3 , '3005' , 'Canada Post Small Packets Surface for World' , NULL , NULL , NULL , NULL , 0 , 20000 , 0 , 50 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (81 , 5 , 3 , '3010' , 'Canada Post Surface International fo World' , NULL , NULL , NULL , NULL , 0 , 20000 , 0 , 50 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (82 , 5 , 3 , '3015' , 'Canada Post Small Packets Air for World' , NULL , NULL , NULL , NULL , 0 , 20000 , 0 , 50 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (83 , 5 , 3 , '3020' , 'Canada Post Air International for World' , NULL , NULL , NULL , NULL , 0 , 20000 , 0 , 50 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (84 , 5 , 3 , '3025' , 'Canada Post Xpresspost International for World' , NULL , NULL , NULL , NULL , 0 , 20000 , 0 , 50 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (85 , 5 , 3 , '3040' , 'Canada Post Purolator International for World' , NULL , NULL , NULL , NULL , 0 , 20000 , 0 , 50 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (86 , 5 , 3 , '3050' , 'Canada Post Puropak International for World' , NULL , NULL , NULL , NULL , 0 , 20000 , 0 , 50 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (87 , 3 , 4 , 'First-Class Mail Postcards' , 'USPS First-Class Mail Postcards' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (88 , 3 , 5 , 'First-Class Mail Large Postcards' , 'USPS First-Class Mail Large Postcards' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , NULL , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (108 , 7 , 25 , 'STANDARD' , 'Regular Parcels' , NULL , NULL , NULL , NULL , 0 , 999999 , 0 , 999999 , 0 , 999999 , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (109 , 7 , 26 , 'EXPRESS' , 'Express Parcels' , NULL , NULL , NULL , NULL , 0 , 999999 , 0 , 999999 , 0 , 999999 , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (110 , 7 , 27 , 'AIR' , 'Air Mail' , NULL , NULL , NULL , NULL , 0 , 999999 , 0 , 999999 , 0 , 999999 , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (111 , 7 , 28 , 'SEA' , 'Sea Mail' , NULL , NULL , NULL , NULL , 0 , 999999 , 0 , 999999 , 0 , 999999 , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (112 , 7 , 29 , 'ECI_D' , 'Express Courier International Document' , NULL , NULL , NULL , NULL , 0 , 999999 , 0 , 999999 , 0 , 999999 , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (113 , 7 , 30 , 'ECI_M' , 'Express Courier International Merchandise' , NULL , NULL , NULL , NULL , 0 , 999999 , 0 , 999999 , 0 , 999999 , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (114 , 7 , 31 , 'EPI' , 'Express Post International' , NULL , NULL , NULL , NULL , 0 , 999999 , 0 , 999999 , 0 , 999999 , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (115 , 8 , 6 , 'STANDARD_OVERNIGHT' , 'STANDARD OVERNIGHT' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (116 , 8 , 7 , 'PRIORITY_OVERNIGHT' , 'PRIORITY OVERNIGHT' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (117 , 8 , 8 , 'FEDEX_1_DAY_FREIGHT' , 'FEDEX 1 DAY FREIGHT' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (118 , 8 , 9 , 'FEDEX_2_DAY' , 'FEDEX 2 DAY' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (119 , 8 , 10 , 'FEDEX_2_DAY_FREIGHT' , 'FEDEX 2 DAY FREIGHT' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (120 , 8 , 11 , 'FEDEX_3_DAY_FREIGHT' , 'FEDEX 3 DAY FREIGHT' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (121 , 8 , 12 , 'FEDEX_EXPRESS_SAVER' , 'FEDEX EXPRESS SAVER' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (122 , 8 , 13 , 'FEDEX_GROUND' , 'FEDEX GROUND' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (123 , 8 , 14 , 'FIRST_OVERNIGHT' , 'FIRST OVERNIGHT' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (124 , 8 , 15 , 'GROUND_HOME_DELIVERY' , 'GROUND HOME DELIVERY' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (125 , 8 , 16 , 'EUROPE_FIRST_INTERNATIONAL_PRIORITY' , 'EUROPE FIRST INTERNATIONAL PRIORITY' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (126 , 8 , 17 , 'INTERNATIONAL_DISTRIBUTION_FREIGHT' , 'INTERNATIONAL DISTRIBUTION FREIGHT' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (127 , 8 , 18 , 'INTERNATIONAL_ECONOMY' , 'INTERNATIONAL ECONOMY' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (128 , 8 , 19 , 'INTERNATIONAL_ECONOMY_DISTRIBUTION' , 'INTERNATIONAL ECONOMY DISTRIBUTION' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (129 , 8 , 20 , 'INTERNATIONAL_ECONOMY_FREIGHT ' , 'INTERNATIONAL ECONOMY FREIGHT ' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (130 , 8 , 21 , 'INTERNATIONAL_FIRST' , 'INTERNATIONAL FIRST' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (131 , 8 , 22 , 'INTERNATIONAL_PRIORITY' , 'INTERNATIONAL PRIORITY' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (132 , 8 , 23 , 'INTERNATIONAL_PRIORITY_DISTRIBUTION' , 'INTERNATIONAL PRIORITY DISTRIBUTION' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (133 , 8 , 24 , 'INTERNATIONAL_PRIORITY_FREIGHT' , 'INTERNATIONAL PRIORITY FREIGHT' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (134 , 3 , 32 , 'Parcel Post' , 'USPS Parcel Post' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (135 , 3 , 33 , 'Media Mail Parcel' , 'USPS Media Mail' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (136 , 3 , 34 , 'Library Mail Parcel' , 'USPS Library Mail' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (137 , 3 , 35 , 'USPS GXG Envelopes**' , 'USPS GXG Envelopes**' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (138 , 3 , 36 , 'Priority Mail Express International ' , 'USPS Priority Mail Express International' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (139 , 3 , 37 , 'Priority Mail International' , 'USPS Priority Mail International' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (140 , 3 , 38 , 'Priority Mail International Large Flat Rate Box' , 'USPS Priority Mail International Large Flat Rate Box' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (141 , 3 , 39 , 'Priority Mail International Medium Flat Rate Box' , 'USPS Priority Mail International Medium Flat Rate Box' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (142 , 3 , 40 , 'Priority Mail International Small Flat Rate Box**' , 'USPS Priority Mail International Small Flat Rate Box**' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (143 , 3 , 41 , 'Priority Mail International DVD Flat Rate Box**' , 'USPS Priority Mail International DVD Flat Rate Box**' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (144 , 3 , 42 , 'Priority Mail International Large Video Flat Rate Box**' , 'USPS Priority Mail International Large Video Flat Rate Box**' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (145 , 3 , 43 , 'First-Class Mail International Package**' , 'USPS First-Class Mail International Package**' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 70 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (146 , 9 , 1 , 'PRIORITYOVERNIGHT' , 'FedEx Priority Overnight' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (147 , 9 , 1 , 'STANDARDOVERNIGHT' , 'FedEx Standard Overnight' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (148 , 9 , 1 , 'FIRSTOVERNIGHT' , 'FedEx First Overnight' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (149 , 9 , 1 , 'FEDEX2DAY' , 'FedEx 2Day' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (150 , 9 , 1 , 'FEDEXEXPRESSSAVER' , 'FedEx Express Saver' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (151 , 9 , 1 , 'FEDEX1DAYFREIGHT' , 'FedEx 1Day Freight' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (152 , 9 , 1 , 'FEDEX2DAYFREIGHT' , 'FedEx 2Day Freight' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (153 , 9 , 1 , 'FEDEX3DAYFREIGHT' , 'FedEx 3Day Freight' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (154 , 9 , 1 , 'FEDEXGROUND' , 'FedEx Ground' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (155 , 9 , 1 , 'GROUNDHOMEDELIVERY' , 'FedEx Home Delivery' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (156 , 9 , 1 , 'INTERNATIONALPRIORITY' , 'FedEx International Priority' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (157 , 9 , 1 , 'INTERNATIONALECONOMY' , 'FedEx International Economy' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (158 , 9 , 1 , 'INTERNATIONALFIRST' , 'FedEx International First' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (159 , 9 , 1 , 'INTERNATIONALPRIORITYFREIGHT' , 'FedEx International Priority Freight' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (160 , 9 , 1 , 'INTERNATIONALECONOMYFREIGHT' , 'FedEx International Economy Freight' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (161 , 9 , 1 , 'EUROPEFIRSTINTERNATIONALPRIORITY' , 'FedEx Europe First' , NULL , NULL , NULL , NULL , 0 , 10000 , 0 , 10000 , NULL , NULL , NULL , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;
INSERT INTO va_shipping_types (shipping_type_id,shipping_module_id,shipping_order,shipping_type_code,shipping_type_desc,shipping_time,admin_order_class,user_order_class,tare_weight,min_goods_cost,max_goods_cost,min_weight,max_weight,min_quantity,max_quantity,cost_per_order,cost_per_product,cost_per_weight,is_taxable,is_active,is_call_center,sites_all,guest_access,user_types_all,countries_all,states_all,postal_match_type,postal_codes) VALUES (162 , 1 , 44 , NULL , 'Shipping' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 10 , NULL , NULL , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , NULL , NULL ) ;


INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (28 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (28 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (29 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (29 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (30 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (30 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 1 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 2 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 3 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 4 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 5 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 6 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 7 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 9 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 10 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 11 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 12 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 14 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 15 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 16 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 17 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 18 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 19 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 20 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 22 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 23 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 24 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 25 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 26 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 27 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 28 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 29 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 30 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 31 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 32 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 33 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 34 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 35 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 36 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 38 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 39 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 40 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 41 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 42 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 43 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 44 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 45 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 46 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 47 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 48 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 49 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 50 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 51 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 52 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 53 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 57 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 59 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 60 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 61 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 62 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 64 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 65 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 66 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 68 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 70 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 71 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 72 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 73 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 75 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 76 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 78 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 79 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 80 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 81 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 82 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 83 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 84 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 85 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 86 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 87 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 88 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 90 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 91 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 92 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 93 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 94 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 95 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 96 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 97 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 98 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 99 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 101 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 103 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 104 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 105 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 106 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 107 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 108 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 109 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 111 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 112 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 113 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 114 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 115 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 116 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 117 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 118 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 119 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 120 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 121 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 122 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 123 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 124 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 125 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 126 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 127 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 128 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 129 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 130 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 131 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 132 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 136 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 137 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 139 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 140 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 141 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 142 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 143 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 144 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 145 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 146 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 147 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 148 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 149 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 150 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 152 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 153 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 154 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 155 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 156 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 157 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 158 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 159 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 160 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 161 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 162 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 163 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 165 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 166 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 167 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 168 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 169 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 170 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 171 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 172 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 173 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 174 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 175 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 176 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 178 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 179 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 181 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 183 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 184 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 185 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 186 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 187 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 188 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 189 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 190 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 191 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 192 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 193 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 195 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 196 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 198 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 200 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 201 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 202 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 203 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 204 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 205 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 206 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 207 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 208 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 209 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 210 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 211 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 212 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 213 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 214 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 215 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 216 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 217 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 218 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 219 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 220 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 221 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 222 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 223 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 224 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 225 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 226 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 229 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 230 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 231 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 232 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 233 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 234 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 236 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 237 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 238 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 239 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 240 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 241 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 242 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 243 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (31 , 244 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 1 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 2 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 3 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 4 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 5 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 6 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 7 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 9 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 10 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 11 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 12 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 14 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 15 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 16 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 17 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 18 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 19 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 20 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 22 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 23 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 24 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 25 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 26 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 27 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 28 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 29 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 30 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 31 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 32 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 33 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 34 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 35 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 36 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 38 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 39 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 40 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 41 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 42 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 43 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 44 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 45 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 46 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 47 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 48 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 49 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 50 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 51 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 52 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 53 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 57 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 59 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 60 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 61 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 62 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 64 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 65 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 66 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 68 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 70 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 71 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 72 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 73 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 75 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 76 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 78 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 79 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 80 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 81 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 82 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 83 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 84 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 85 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 86 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 87 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 88 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 90 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 91 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 92 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 93 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 94 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 95 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 96 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 97 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 98 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 99 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 101 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 103 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 104 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 105 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 106 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 107 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 108 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 109 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 111 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 112 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 113 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 114 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 115 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 116 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 117 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 118 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 119 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 120 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 121 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 122 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 123 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 124 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 125 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 126 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 127 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 128 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 129 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 130 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 131 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 132 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 136 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 137 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 139 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 140 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 141 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 142 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 143 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 144 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 145 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 146 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 147 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 148 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 149 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 150 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 152 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 153 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 154 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 155 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 156 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 157 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 158 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 159 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 160 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 161 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 162 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 163 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 165 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 166 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 167 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 168 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 169 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 170 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 171 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 172 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 173 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 174 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 175 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 176 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 178 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 179 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 181 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 183 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 184 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 185 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 186 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 187 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 188 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 189 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 190 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 191 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 192 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 193 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 195 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 196 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 198 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 200 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 201 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 202 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 203 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 204 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 205 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 206 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 207 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 208 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 209 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 210 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 211 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 212 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 213 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 214 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 215 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 216 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 217 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 218 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 219 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 220 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 221 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 222 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 223 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 224 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 225 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 226 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 229 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 230 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 231 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 232 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 233 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 234 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 236 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 237 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 238 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 239 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 240 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 241 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 242 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 243 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (32 , 244 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 8 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 13 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 21 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 54 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 55 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 56 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 58 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 63 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 67 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 69 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 74 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 77 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 89 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 100 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 102 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 110 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 133 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 134 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 135 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 138 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 151 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 164 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 177 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 182 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 194 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 197 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 199 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (33 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (34 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (34 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (35 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (35 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (36 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (36 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (38 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (39 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (40 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (41 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 8 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 13 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 21 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 54 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 55 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 56 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 58 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 63 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 67 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 69 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 74 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 77 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 89 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 100 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 102 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 110 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 133 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 134 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 135 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 138 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 151 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 164 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 177 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 182 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 194 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 197 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (42 , 199 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 8 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 13 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 21 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 54 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 55 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 56 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 58 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 63 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 67 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 69 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 74 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 77 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 89 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 100 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 102 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 110 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 133 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 134 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 135 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 138 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 151 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 164 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 177 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 182 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 194 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 197 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (43 , 199 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (44 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (45 , 155 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (46 , 155 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (47 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (47 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (47 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (47 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (48 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (48 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (48 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (48 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (49 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (49 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (49 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (49 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (50 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (50 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (50 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (50 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (51 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (51 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (51 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (51 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (52 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (52 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (52 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (52 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (53 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (53 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (53 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (53 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (54 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (54 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (54 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (54 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (55 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (55 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (55 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (55 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (56 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (56 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (56 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (56 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (57 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (57 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (57 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (57 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (58 , 179 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (58 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (58 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (58 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (59 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (59 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (59 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (59 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (60 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (60 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (60 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (60 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (61 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (61 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (61 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (61 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (62 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (62 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (62 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (62 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (63 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (63 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (63 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (63 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (64 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (64 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (64 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (64 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (65 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (65 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (65 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (65 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (66 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (66 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (66 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (66 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (67 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (67 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (67 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (67 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (68 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (68 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (68 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (68 , 246 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (69 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (70 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (71 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (72 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (73 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (74 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (75 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (76 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (77 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (87 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (87 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (87 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (87 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (88 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (88 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (88 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (88 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (108 , 14 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (109 , 14 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 1 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 2 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 3 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 4 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 5 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 6 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 7 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 8 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 9 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 10 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 11 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 12 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 13 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 15 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 16 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 17 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 18 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 19 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 20 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 21 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 22 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 23 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 24 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 25 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 26 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 27 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 28 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 29 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 30 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 31 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 32 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 33 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 34 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 35 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 36 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 38 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 39 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 40 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 41 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 42 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 43 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 44 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 45 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 46 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 47 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 48 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 49 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 50 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 51 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 52 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 53 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 54 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 55 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 56 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 57 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 58 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 59 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 60 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 61 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 62 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 63 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 64 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 65 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 66 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 67 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 68 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 69 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 70 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 71 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 72 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 73 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 74 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 76 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 77 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 78 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 79 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 80 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 81 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 82 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 83 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 84 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 85 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 86 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 87 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 88 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 89 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 90 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 91 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 92 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 93 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 94 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 95 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 96 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 97 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 98 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 99 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 100 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 101 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 102 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 103 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 104 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 105 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 106 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 107 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 108 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 109 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 110 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 111 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 112 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 113 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 114 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 115 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 116 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 117 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 118 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 119 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 120 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 121 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 122 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 123 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 124 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 125 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 126 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 127 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 128 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 129 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 130 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 131 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 132 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 133 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 134 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 135 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 136 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 137 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 138 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 139 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 140 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 141 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 142 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 143 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 144 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 145 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 146 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 147 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 148 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 149 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 150 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 151 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 152 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 153 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 154 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 155 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 156 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 157 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 158 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 159 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 160 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 161 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 162 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 163 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 164 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 165 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 166 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 167 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 168 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 169 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 170 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 171 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 172 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 173 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 174 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 175 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 176 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 177 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 178 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 179 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 181 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 182 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 183 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 184 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 185 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 186 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 187 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 188 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 189 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 190 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 191 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 192 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 193 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 194 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 195 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 196 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 197 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 198 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 199 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 200 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 201 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 202 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 203 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 204 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 205 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 206 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 207 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 208 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 209 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 210 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 211 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 212 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 213 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 214 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 215 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 216 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 217 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 218 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 219 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 220 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 221 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 222 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 223 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 224 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 225 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 226 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 229 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 230 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 231 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 232 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 233 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 234 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 236 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 237 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 238 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 239 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 240 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 241 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 242 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 243 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 244 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 245 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 246 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (110 , 247 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 1 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 2 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 3 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 4 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 5 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 6 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 7 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 8 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 9 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 10 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 11 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 12 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 13 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 15 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 16 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 17 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 18 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 19 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 20 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 21 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 22 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 23 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 24 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 25 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 26 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 27 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 28 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 29 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 30 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 31 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 32 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 33 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 34 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 35 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 36 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 38 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 39 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 40 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 41 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 42 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 43 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 44 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 45 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 46 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 47 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 48 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 49 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 50 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 51 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 52 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 53 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 54 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 55 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 56 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 57 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 58 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 59 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 60 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 61 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 62 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 63 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 64 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 65 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 66 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 67 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 68 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 69 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 70 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 71 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 72 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 73 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 74 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 76 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 77 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 78 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 79 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 80 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 81 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 82 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 83 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 84 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 85 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 86 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 87 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 88 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 89 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 90 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 91 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 92 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 93 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 94 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 95 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 96 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 97 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 98 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 99 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 100 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 101 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 102 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 103 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 104 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 105 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 106 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 107 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 108 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 109 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 110 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 111 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 112 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 113 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 114 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 115 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 116 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 117 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 118 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 119 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 120 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 121 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 122 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 123 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 124 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 125 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 126 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 127 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 128 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 129 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 130 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 131 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 132 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 133 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 134 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 135 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 136 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 137 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 138 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 139 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 140 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 141 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 142 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 143 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 144 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 145 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 146 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 147 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 148 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 149 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 150 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 151 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 152 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 153 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 154 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 155 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 156 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 157 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 158 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 159 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 160 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 161 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 162 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 163 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 164 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 165 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 166 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 167 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 168 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 169 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 170 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 171 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 172 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 173 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 174 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 175 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 176 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 177 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 178 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 179 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 181 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 182 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 183 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 184 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 185 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 186 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 187 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 188 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 189 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 190 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 191 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 192 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 193 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 194 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 195 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 196 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 197 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 198 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 199 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 200 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 201 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 202 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 203 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 204 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 205 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 206 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 207 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 208 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 209 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 210 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 211 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 212 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 213 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 214 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 215 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 216 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 217 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 218 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 219 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 220 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 221 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 222 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 223 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 224 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 225 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 226 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 229 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 230 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 231 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 232 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 233 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 234 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 236 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 237 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 238 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 239 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 240 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 241 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 242 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 243 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 244 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 245 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 246 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (111 , 247 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 1 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 2 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 3 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 4 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 5 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 6 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 7 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 8 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 9 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 10 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 11 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 12 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 13 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 15 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 16 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 17 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 18 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 19 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 20 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 21 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 22 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 23 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 24 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 25 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 26 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 27 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 28 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 29 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 30 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 31 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 32 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 33 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 34 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 35 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 36 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 38 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 39 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 40 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 41 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 42 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 43 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 44 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 45 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 46 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 47 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 48 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 49 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 50 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 51 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 52 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 53 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 54 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 55 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 56 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 57 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 58 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 59 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 60 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 61 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 62 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 63 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 64 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 65 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 66 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 67 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 68 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 69 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 70 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 71 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 72 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 73 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 74 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 76 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 77 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 78 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 79 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 80 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 81 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 82 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 83 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 84 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 85 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 86 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 87 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 88 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 89 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 90 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 91 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 92 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 93 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 94 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 95 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 96 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 97 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 98 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 99 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 100 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 101 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 102 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 103 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 104 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 105 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 106 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 107 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 108 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 109 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 110 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 111 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 112 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 113 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 114 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 115 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 116 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 117 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 118 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 119 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 120 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 121 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 122 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 123 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 124 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 125 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 126 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 127 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 128 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 129 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 130 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 131 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 132 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 133 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 134 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 135 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 136 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 137 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 138 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 139 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 140 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 141 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 142 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 143 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 144 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 145 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 146 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 147 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 148 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 149 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 150 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 151 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 152 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 153 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 154 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 155 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 156 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 157 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 158 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 159 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 160 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 161 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 162 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 163 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 164 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 165 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 166 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 167 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 168 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 169 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 170 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 171 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 172 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 173 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 174 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 175 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 176 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 177 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 178 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 179 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 181 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 182 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 183 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 184 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 185 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 186 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 187 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 188 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 189 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 190 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 191 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 192 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 193 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 194 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 195 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 196 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 197 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 198 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 199 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 200 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 201 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 202 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 203 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 204 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 205 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 206 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 207 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 208 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 209 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 210 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 211 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 212 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 213 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 214 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 215 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 216 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 217 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 218 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 219 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 220 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 221 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 222 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 223 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 224 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 225 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 226 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 229 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 230 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 231 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 232 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 233 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 234 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 236 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 237 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 238 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 239 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 240 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 241 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 242 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 243 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 244 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 245 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 246 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (112 , 247 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 1 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 2 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 3 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 4 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 5 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 6 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 7 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 8 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 9 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 10 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 11 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 12 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 13 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 15 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 16 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 17 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 18 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 19 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 20 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 21 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 22 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 23 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 24 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 25 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 26 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 27 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 28 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 29 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 30 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 31 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 32 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 33 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 34 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 35 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 36 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 38 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 39 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 40 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 41 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 42 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 43 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 44 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 45 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 46 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 47 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 48 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 49 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 50 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 51 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 52 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 53 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 54 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 55 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 56 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 57 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 58 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 59 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 60 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 61 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 62 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 63 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 64 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 65 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 66 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 67 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 68 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 69 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 70 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 71 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 72 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 73 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 74 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 76 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 77 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 78 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 79 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 80 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 81 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 82 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 83 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 84 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 85 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 86 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 87 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 88 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 89 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 90 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 91 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 92 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 93 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 94 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 95 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 96 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 97 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 98 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 99 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 100 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 101 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 102 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 103 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 104 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 105 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 106 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 107 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 108 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 109 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 110 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 111 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 112 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 113 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 114 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 115 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 116 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 117 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 118 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 119 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 120 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 121 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 122 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 123 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 124 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 125 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 126 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 127 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 128 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 129 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 130 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 131 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 132 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 133 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 134 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 135 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 136 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 137 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 138 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 139 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 140 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 141 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 142 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 143 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 144 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 145 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 146 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 147 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 148 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 149 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 150 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 151 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 152 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 153 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 154 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 155 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 156 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 157 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 158 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 159 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 160 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 161 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 162 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 163 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 164 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 165 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 166 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 167 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 168 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 169 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 170 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 171 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 172 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 173 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 174 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 175 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 176 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 177 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 178 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 179 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 181 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 182 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 183 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 184 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 185 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 186 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 187 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 188 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 189 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 190 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 191 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 192 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 193 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 194 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 195 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 196 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 197 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 198 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 199 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 200 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 201 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 202 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 203 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 204 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 205 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 206 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 207 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 208 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 209 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 210 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 211 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 212 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 213 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 214 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 215 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 216 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 217 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 218 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 219 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 220 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 221 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 222 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 223 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 224 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 225 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 226 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 229 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 230 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 231 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 232 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 233 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 234 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 236 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 237 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 238 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 239 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 240 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 241 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 242 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 243 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 244 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 245 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 246 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (113 , 247 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 1 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 2 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 3 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 4 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 5 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 6 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 7 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 8 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 9 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 10 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 11 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 12 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 13 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 15 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 16 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 17 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 18 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 19 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 20 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 21 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 22 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 23 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 24 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 25 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 26 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 27 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 28 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 29 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 30 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 31 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 32 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 33 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 34 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 35 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 36 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 38 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 39 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 40 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 41 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 42 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 43 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 44 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 45 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 46 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 47 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 48 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 49 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 50 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 51 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 52 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 53 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 54 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 55 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 56 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 57 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 58 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 59 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 60 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 61 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 62 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 63 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 64 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 65 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 66 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 67 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 68 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 69 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 70 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 71 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 72 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 73 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 74 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 76 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 77 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 78 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 79 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 80 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 81 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 82 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 83 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 84 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 85 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 86 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 87 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 88 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 89 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 90 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 91 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 92 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 93 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 94 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 95 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 96 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 97 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 98 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 99 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 100 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 101 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 102 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 103 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 104 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 105 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 106 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 107 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 108 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 109 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 110 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 111 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 112 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 113 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 114 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 115 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 116 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 117 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 118 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 119 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 120 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 121 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 122 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 123 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 124 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 125 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 126 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 127 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 128 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 129 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 130 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 131 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 132 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 133 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 134 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 135 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 136 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 137 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 138 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 139 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 140 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 141 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 142 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 143 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 144 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 145 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 146 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 147 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 148 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 149 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 150 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 151 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 152 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 153 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 154 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 155 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 156 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 157 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 158 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 159 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 160 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 161 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 162 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 163 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 164 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 165 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 166 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 167 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 168 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 169 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 170 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 171 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 172 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 173 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 174 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 175 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 176 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 177 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 178 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 179 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 181 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 182 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 183 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 184 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 185 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 186 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 187 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 188 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 189 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 190 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 191 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 192 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 193 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 194 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 195 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 196 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 197 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 198 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 199 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 200 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 201 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 202 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 203 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 204 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 205 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 206 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 207 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 208 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 209 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 210 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 211 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 212 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 213 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 214 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 215 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 216 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 217 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 218 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 219 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 220 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 221 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 222 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 223 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 224 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 225 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 226 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 229 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 230 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 231 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 232 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 233 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 234 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 236 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 237 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 238 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 239 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 240 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 241 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 242 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 243 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 244 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 245 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 246 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (114 , 247 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (134 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (134 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (134 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (134 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (135 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (135 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (135 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (135 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (136 , 180 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (136 , 227 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (136 , 228 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (136 , 235 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 1 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 2 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 3 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 4 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 5 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 6 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 7 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 8 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 9 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 10 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 11 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 12 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 13 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 14 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 15 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 16 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 17 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 18 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 19 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 20 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 21 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 22 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 23 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 24 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 25 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 26 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 27 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 28 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 29 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 30 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 31 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 32 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 33 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 34 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 35 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 36 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 38 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 39 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 40 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 41 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 42 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 43 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 44 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 45 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 46 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 47 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 48 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 49 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 50 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 51 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 52 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 53 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 54 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 55 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 56 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 57 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 58 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 59 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 60 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 61 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 62 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 63 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 64 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 65 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 66 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 67 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 68 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 69 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 70 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 71 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 72 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 73 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 74 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 76 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 77 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 78 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 79 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 80 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 81 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 82 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 83 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 84 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 85 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 86 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 87 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 88 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 89 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 90 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 91 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 92 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 93 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 94 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 95 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 96 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 97 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 98 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 99 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 100 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 101 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 102 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 103 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 104 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 105 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 106 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 107 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 108 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 109 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 110 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 111 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 112 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 113 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 114 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 115 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 116 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 117 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 118 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 119 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 120 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 121 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 122 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 123 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 124 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 125 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 126 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 127 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 128 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 129 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 130 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 131 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 132 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 133 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 134 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 135 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 136 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 137 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 138 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 139 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 140 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 141 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 142 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 143 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 144 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 145 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 146 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 147 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 148 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 149 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 150 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 151 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 152 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 153 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 154 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 155 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 156 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 157 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 158 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 159 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 160 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 161 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 162 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 163 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 164 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 165 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 166 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 167 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 168 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 169 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 170 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 171 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 172 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 173 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 174 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 175 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 176 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 177 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 178 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 179 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 181 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 182 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 183 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 184 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 185 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 186 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 187 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 188 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 189 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 190 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 191 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 192 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 193 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 194 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 195 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 196 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 197 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 198 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 199 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 200 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 201 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 202 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 203 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 204 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 205 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 206 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 207 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 208 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 209 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 210 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 211 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 212 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 213 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 214 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 215 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 216 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 217 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 218 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 219 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 220 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 221 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 222 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 223 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 224 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 225 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 226 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 229 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 230 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 231 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 232 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 233 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 234 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 236 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 237 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 238 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 239 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 240 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 241 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 242 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 243 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 244 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 245 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 246 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (137 , 247 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 1 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 2 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 3 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 4 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 5 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 6 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 7 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 8 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 9 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 10 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 11 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 12 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 13 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 14 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 15 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 16 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 17 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 18 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 19 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 20 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 21 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 22 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 23 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 24 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 25 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 26 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 27 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 28 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 29 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 30 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 31 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 32 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 33 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 34 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 35 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 36 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 38 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 39 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 40 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 41 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 42 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 43 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 44 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 45 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 46 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 47 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 48 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 49 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 50 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 51 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 52 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 53 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 54 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 55 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 56 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 57 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 58 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 59 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 60 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 61 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 62 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 63 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 64 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 65 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 66 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 67 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 68 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 69 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 70 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 71 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 72 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 73 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 74 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 76 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 77 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 78 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 79 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 80 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 81 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 82 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 83 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 84 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 85 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 86 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 87 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 88 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 89 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 90 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 91 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 92 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 93 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 94 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 95 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 96 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 97 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 98 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 99 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 100 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 101 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 102 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 103 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 104 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 105 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 106 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 107 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 108 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 109 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 110 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 111 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 112 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 113 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 114 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 115 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 116 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 117 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 118 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 119 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 120 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 121 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 122 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 123 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 124 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 125 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 126 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 127 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 128 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 129 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 130 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 131 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 132 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 133 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 134 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 135 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 136 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 137 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 138 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 139 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 140 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 141 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 142 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 143 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 144 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 145 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 146 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 147 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 148 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 149 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 150 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 151 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 152 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 153 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 154 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 155 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 156 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 157 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 158 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 159 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 160 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 161 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 162 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 163 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 164 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 165 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 166 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 167 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 168 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 169 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 170 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 171 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 172 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 173 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 174 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 175 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 176 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 177 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 178 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 179 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 181 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 182 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 183 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 184 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 185 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 186 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 187 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 188 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 189 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 190 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 191 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 192 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 193 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 194 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 195 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 196 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 197 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 198 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 199 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 200 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 201 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 202 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 203 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 204 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 205 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 206 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 207 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 208 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 209 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 210 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 211 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 212 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 213 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 214 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 215 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 216 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 217 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 218 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 219 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 220 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 221 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 222 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 223 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 224 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 225 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 226 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 229 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 230 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 231 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 232 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 233 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 234 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 236 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 237 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 238 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 239 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 240 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 241 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 242 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 243 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 244 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 245 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 246 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (138 , 247 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 1 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 2 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 3 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 4 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 5 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 6 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 7 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 8 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 9 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 10 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 11 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 12 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 13 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 14 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 15 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 16 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 17 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 18 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 19 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 20 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 21 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 22 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 23 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 24 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 25 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 26 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 27 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 28 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 29 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 30 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 31 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 32 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 33 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 34 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 35 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 36 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 38 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 39 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 40 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 41 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 42 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 43 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 44 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 45 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 46 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 47 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 48 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 49 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 50 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 51 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 52 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 53 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 54 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 55 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 56 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 57 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 58 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 59 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 60 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 61 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 62 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 63 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 64 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 65 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 66 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 67 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 68 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 69 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 70 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 71 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 72 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 73 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 74 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 76 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 77 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 78 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 79 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 80 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 81 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 82 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 83 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 84 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 85 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 86 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 87 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 88 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 89 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 90 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 91 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 92 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 93 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 94 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 95 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 96 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 97 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 98 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 99 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 100 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 101 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 102 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 103 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 104 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 105 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 106 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 107 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 108 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 109 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 110 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 111 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 112 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 113 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 114 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 115 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 116 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 117 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 118 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 119 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 120 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 121 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 122 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 123 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 124 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 125 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 126 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 127 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 128 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 129 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 130 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 131 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 132 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 133 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 134 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 135 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 136 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 137 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 138 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 139 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 140 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 141 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 142 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 143 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 144 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 145 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 146 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 147 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 148 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 149 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 150 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 151 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 152 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 153 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 154 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 155 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 156 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 157 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 158 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 159 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 160 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 161 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 162 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 163 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 164 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 165 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 166 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 167 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 168 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 169 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 170 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 171 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 172 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 173 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 174 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 175 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 176 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 177 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 178 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 179 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 181 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 182 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 183 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 184 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 185 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 186 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 187 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 188 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 189 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 190 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 191 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 192 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 193 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 194 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 195 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 196 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 197 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 198 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 199 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 200 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 201 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 202 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 203 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 204 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 205 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 206 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 207 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 208 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 209 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 210 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 211 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 212 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 213 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 214 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 215 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 216 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 217 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 218 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 219 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 220 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 221 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 222 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 223 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 224 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 225 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 226 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 229 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 230 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 231 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 232 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 233 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 234 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 236 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 237 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 238 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 239 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 240 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 241 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 242 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 243 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 244 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 245 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 246 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (139 , 247 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 1 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 2 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 3 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 4 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 5 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 6 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 7 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 8 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 9 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 10 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 11 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 12 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 13 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 14 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 15 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 16 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 17 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 18 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 19 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 20 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 21 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 22 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 23 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 24 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 25 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 26 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 27 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 28 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 29 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 30 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 31 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 32 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 33 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 34 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 35 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 36 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 38 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 39 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 40 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 41 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 42 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 43 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 44 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 45 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 46 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 47 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 48 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 49 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 50 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 51 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 52 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 53 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 54 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 55 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 56 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 57 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 58 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 59 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 60 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 61 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 62 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 63 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 64 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 65 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 66 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 67 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 68 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 69 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 70 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 71 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 72 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 73 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 74 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 76 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 77 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 78 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 79 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 80 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 81 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 82 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 83 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 84 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 85 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 86 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 87 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 88 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 89 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 90 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 91 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 92 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 93 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 94 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 95 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 96 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 97 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 98 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 99 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 100 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 101 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 102 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 103 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 104 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 105 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 106 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 107 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 108 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 109 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 110 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 111 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 112 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 113 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 114 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 115 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 116 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 117 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 118 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 119 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 120 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 121 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 122 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 123 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 124 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 125 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 126 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 127 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 128 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 129 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 130 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 131 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 132 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 133 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 134 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 135 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 136 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 137 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 138 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 139 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 140 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 141 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 142 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 143 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 144 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 145 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 146 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 147 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 148 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 149 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 150 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 151 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 152 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 153 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 154 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 155 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 156 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 157 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 158 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 159 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 160 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 161 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 162 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 163 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 164 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 165 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 166 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 167 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 168 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 169 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 170 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 171 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 172 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 173 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 174 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 175 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 176 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 177 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 178 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 179 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 181 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 182 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 183 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 184 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 185 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 186 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 187 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 188 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 189 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 190 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 191 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 192 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 193 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 194 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 195 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 196 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 197 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 198 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 199 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 200 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 201 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 202 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 203 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 204 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 205 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 206 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 207 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 208 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 209 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 210 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 211 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 212 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 213 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 214 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 215 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 216 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 217 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 218 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 219 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 220 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 221 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 222 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 223 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 224 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 225 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 226 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 229 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 230 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 231 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 232 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 233 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 234 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 236 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 237 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 238 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 239 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 240 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 241 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 242 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 243 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 244 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 245 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 246 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (140 , 247 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 1 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 2 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 3 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 4 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 5 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 6 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 7 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 8 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 9 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 10 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 11 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 12 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 13 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 14 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 15 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 16 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 17 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 18 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 19 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 20 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 21 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 22 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 23 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 24 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 25 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 26 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 27 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 28 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 29 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 30 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 31 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 32 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 33 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 34 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 35 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 36 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 38 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 39 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 40 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 41 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 42 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 43 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 44 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 45 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 46 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 47 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 48 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 49 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 50 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 51 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 52 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 53 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 54 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 55 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 56 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 57 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 58 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 59 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 60 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 61 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 62 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 63 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 64 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 65 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 66 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 67 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 68 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 69 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 70 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 71 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 72 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 73 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 74 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 76 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 77 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 78 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 79 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 80 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 81 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 82 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 83 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 84 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 85 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 86 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 87 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 88 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 89 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 90 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 91 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 92 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 93 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 94 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 95 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 96 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 97 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 98 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 99 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 100 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 101 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 102 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 103 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 104 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 105 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 106 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 107 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 108 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 109 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 110 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 111 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 112 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 113 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 114 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 115 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 116 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 117 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 118 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 119 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 120 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 121 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 122 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 123 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 124 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 125 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 126 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 127 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 128 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 129 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 130 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 131 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 132 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 133 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 134 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 135 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 136 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 137 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 138 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 139 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 140 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 141 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 142 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 143 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 144 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 145 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 146 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 147 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 148 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 149 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 150 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 151 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 152 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 153 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 154 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 155 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 156 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 157 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 158 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 159 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 160 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 161 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 162 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 163 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 164 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 165 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 166 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 167 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 168 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 169 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 170 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 171 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 172 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 173 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 174 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 175 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 176 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 177 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 178 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 179 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 181 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 182 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 183 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 184 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 185 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 186 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 187 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 188 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 189 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 190 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 191 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 192 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 193 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 194 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 195 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 196 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 197 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 198 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 199 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 200 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 201 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 202 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 203 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 204 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 205 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 206 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 207 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 208 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 209 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 210 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 211 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 212 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 213 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 214 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 215 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 216 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 217 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 218 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 219 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 220 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 221 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 222 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 223 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 224 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 225 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 226 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 229 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 230 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 231 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 232 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 233 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 234 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 236 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 237 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 238 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 239 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 240 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 241 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 242 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 243 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 244 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 245 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 246 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (141 , 247 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 1 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 2 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 3 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 4 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 5 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 6 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 7 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 8 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 9 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 10 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 11 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 12 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 13 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 14 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 15 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 16 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 17 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 18 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 19 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 20 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 21 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 22 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 23 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 24 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 25 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 26 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 27 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 28 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 29 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 30 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 31 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 32 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 33 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 34 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 35 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 36 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 38 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 39 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 40 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 41 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 42 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 43 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 44 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 45 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 46 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 47 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 48 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 49 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 50 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 51 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 52 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 53 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 54 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 55 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 56 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 57 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 58 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 59 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 60 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 61 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 62 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 63 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 64 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 65 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 66 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 67 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 68 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 69 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 70 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 71 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 72 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 73 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 74 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 76 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 77 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 78 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 79 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 80 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 81 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 82 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 83 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 84 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 85 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 86 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 87 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 88 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 89 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 90 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 91 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 92 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 93 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 94 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 95 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 96 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 97 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 98 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 99 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 100 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 101 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 102 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 103 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 104 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 105 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 106 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 107 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 108 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 109 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 110 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 111 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 112 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 113 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 114 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 115 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 116 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 117 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 118 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 119 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 120 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 121 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 122 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 123 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 124 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 125 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 126 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 127 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 128 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 129 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 130 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 131 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 132 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 133 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 134 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 135 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 136 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 137 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 138 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 139 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 140 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 141 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 142 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 143 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 144 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 145 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 146 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 147 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 148 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 149 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 150 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 151 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 152 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 153 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 154 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 155 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 156 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 157 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 158 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 159 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 160 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 161 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 162 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 163 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 164 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 165 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 166 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 167 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 168 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 169 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 170 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 171 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 172 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 173 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 174 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 175 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 176 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 177 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 178 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 179 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 181 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 182 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 183 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 184 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 185 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 186 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 187 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 188 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 189 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 190 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 191 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 192 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 193 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 194 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 195 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 196 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 197 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 198 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 199 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 200 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 201 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 202 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 203 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 204 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 205 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 206 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 207 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 208 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 209 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 210 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 211 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 212 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 213 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 214 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 215 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 216 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 217 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 218 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 219 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 220 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 221 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 222 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 223 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 224 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 225 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 226 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 229 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 230 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 231 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 232 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 233 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 234 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 236 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 237 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 238 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 239 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 240 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 241 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 242 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 243 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 244 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 245 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 246 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (142 , 247 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 1 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 2 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 3 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 4 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 5 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 6 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 7 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 8 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 9 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 10 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 11 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 12 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 13 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 14 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 15 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 16 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 17 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 18 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 19 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 20 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 21 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 22 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 23 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 24 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 25 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 26 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 27 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 28 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 29 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 30 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 31 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 32 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 33 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 34 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 35 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 36 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 38 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 39 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 40 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 41 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 42 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 43 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 44 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 45 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 46 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 47 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 48 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 49 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 50 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 51 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 52 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 53 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 54 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 55 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 56 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 57 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 58 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 59 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 60 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 61 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 62 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 63 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 64 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 65 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 66 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 67 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 68 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 69 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 70 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 71 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 72 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 73 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 74 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 76 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 77 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 78 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 79 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 80 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 81 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 82 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 83 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 84 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 85 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 86 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 87 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 88 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 89 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 90 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 91 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 92 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 93 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 94 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 95 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 96 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 97 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 98 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 99 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 100 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 101 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 102 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 103 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 104 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 105 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 106 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 107 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 108 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 109 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 110 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 111 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 112 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 113 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 114 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 115 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 116 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 117 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 118 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 119 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 120 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 121 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 122 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 123 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 124 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 125 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 126 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 127 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 128 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 129 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 130 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 131 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 132 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 133 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 134 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 135 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 136 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 137 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 138 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 139 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 140 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 141 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 142 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 143 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 144 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 145 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 146 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 147 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 148 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 149 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 150 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 151 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 152 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 153 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 154 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 155 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 156 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 157 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 158 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 159 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 160 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 161 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 162 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 163 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 164 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 165 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 166 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 167 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 168 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 169 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 170 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 171 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 172 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 173 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 174 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 175 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 176 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 177 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 178 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 179 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 181 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 182 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 183 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 184 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 185 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 186 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 187 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 188 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 189 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 190 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 191 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 192 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 193 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 194 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 195 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 196 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 197 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 198 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 199 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 200 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 201 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 202 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 203 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 204 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 205 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 206 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 207 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 208 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 209 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 210 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 211 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 212 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 213 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 214 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 215 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 216 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 217 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 218 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 219 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 220 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 221 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 222 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 223 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 224 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 225 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 226 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 229 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 230 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 231 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 232 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 233 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 234 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 236 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 237 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 238 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 239 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 240 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 241 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 242 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 243 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 244 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 245 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 246 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (143 , 247 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 1 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 2 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 3 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 4 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 5 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 6 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 7 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 8 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 9 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 10 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 11 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 12 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 13 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 14 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 15 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 16 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 17 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 18 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 19 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 20 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 21 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 22 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 23 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 24 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 25 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 26 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 27 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 28 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 29 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 30 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 31 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 32 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 33 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 34 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 35 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 36 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 38 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 39 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 40 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 41 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 42 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 43 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 44 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 45 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 46 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 47 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 48 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 49 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 50 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 51 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 52 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 53 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 54 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 55 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 56 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 57 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 58 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 59 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 60 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 61 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 62 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 63 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 64 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 65 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 66 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 67 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 68 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 69 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 70 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 71 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 72 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 73 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 74 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 76 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 77 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 78 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 79 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 80 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 81 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 82 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 83 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 84 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 85 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 86 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 87 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 88 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 89 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 90 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 91 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 92 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 93 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 94 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 95 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 96 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 97 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 98 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 99 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 100 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 101 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 102 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 103 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 104 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 105 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 106 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 107 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 108 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 109 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 110 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 111 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 112 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 113 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 114 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 115 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 116 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 117 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 118 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 119 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 120 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 121 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 122 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 123 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 124 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 125 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 126 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 127 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 128 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 129 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 130 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 131 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 132 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 133 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 134 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 135 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 136 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 137 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 138 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 139 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 140 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 141 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 142 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 143 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 144 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 145 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 146 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 147 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 148 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 149 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 150 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 151 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 152 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 153 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 154 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 155 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 156 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 157 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 158 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 159 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 160 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 161 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 162 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 163 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 164 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 165 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 166 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 167 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 168 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 169 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 170 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 171 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 172 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 173 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 174 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 175 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 176 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 177 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 178 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 179 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 181 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 182 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 183 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 184 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 185 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 186 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 187 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 188 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 189 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 190 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 191 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 192 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 193 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 194 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 195 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 196 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 197 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 198 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 199 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 200 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 201 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 202 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 203 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 204 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 205 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 206 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 207 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 208 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 209 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 210 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 211 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 212 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 213 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 214 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 215 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 216 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 217 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 218 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 219 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 220 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 221 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 222 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 223 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 224 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 225 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 226 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 229 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 230 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 231 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 232 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 233 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 234 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 236 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 237 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 238 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 239 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 240 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 241 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 242 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 243 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 244 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 245 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 246 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (144 , 247 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 1 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 2 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 3 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 4 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 5 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 6 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 7 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 8 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 9 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 10 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 11 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 12 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 13 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 14 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 15 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 16 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 17 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 18 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 19 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 20 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 21 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 22 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 23 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 24 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 25 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 26 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 27 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 28 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 29 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 30 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 31 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 32 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 33 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 34 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 35 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 36 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 37 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 38 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 39 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 40 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 41 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 42 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 43 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 44 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 45 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 46 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 47 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 48 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 49 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 50 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 51 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 52 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 53 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 54 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 55 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 56 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 57 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 58 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 59 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 60 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 61 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 62 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 63 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 64 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 65 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 66 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 67 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 68 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 69 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 70 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 71 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 72 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 73 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 74 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 76 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 77 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 78 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 79 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 80 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 81 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 82 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 83 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 84 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 85 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 86 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 87 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 88 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 89 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 90 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 91 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 92 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 93 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 94 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 95 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 96 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 97 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 98 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 99 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 100 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 101 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 102 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 103 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 104 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 105 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 106 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 107 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 108 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 109 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 110 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 111 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 112 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 113 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 114 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 115 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 116 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 117 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 118 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 119 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 120 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 121 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 122 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 123 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 124 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 125 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 126 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 127 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 128 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 129 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 130 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 131 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 132 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 133 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 134 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 135 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 136 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 137 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 138 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 139 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 140 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 141 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 142 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 143 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 144 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 145 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 146 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 147 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 148 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 149 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 150 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 151 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 152 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 153 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 154 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 155 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 156 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 157 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 158 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 159 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 160 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 161 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 162 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 163 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 164 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 165 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 166 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 167 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 168 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 169 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 170 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 171 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 172 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 173 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 174 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 175 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 176 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 177 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 178 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 179 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 181 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 182 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 183 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 184 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 185 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 186 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 187 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 188 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 189 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 190 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 191 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 192 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 193 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 194 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 195 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 196 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 197 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 198 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 199 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 200 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 201 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 202 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 203 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 204 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 205 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 206 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 207 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 208 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 209 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 210 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 211 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 212 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 213 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 214 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 215 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 216 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 217 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 218 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 219 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 220 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 221 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 222 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 223 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 224 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 225 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 226 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 229 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 230 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 231 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 232 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 233 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 234 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 236 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 237 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 238 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 239 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 240 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 241 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 242 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 243 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 244 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 245 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 246 ) ;
INSERT INTO va_shipping_types_countries (shipping_type_id,country_id) VALUES (145 , 247 ) ;








INSERT INTO va_sites (site_id,parent_site_id,is_mobile,is_mobile_redirect,short_name,site_name,site_description) VALUES (1 , NULL , NULL , NULL , NULL , 'Default Site' , NULL ) ;






INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (1 , 'AK' , 228 , 'Alaska' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (2 , 'AL' , 228 , 'Alabama' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (3 , 'AR' , 228 , 'Arkansas' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (4 , 'AZ' , 228 , 'Arizona' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (5 , 'CA' , 228 , 'California' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (6 , 'CO' , 228 , 'Colorado' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (7 , 'CT' , 228 , 'Connecticut' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (8 , 'DC' , 228 , 'District of Columbia' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (9 , 'DE' , 228 , 'Delaware' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (10 , 'FL' , 228 , 'Florida' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (11 , 'GA' , 228 , 'Georgia' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (12 , 'HI' , 228 , 'Hawaii' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (13 , 'IA' , 228 , 'Iowa' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (14 , 'ID' , 228 , 'Idaho' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (15 , 'IL' , 228 , 'Illinois' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (16 , 'IN' , 228 , 'Indiana' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (17 , 'KS' , 228 , 'Kansas' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (18 , 'KY' , 228 , 'Kentucky' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (19 , 'LA' , 228 , 'Louisiana' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (20 , 'MA' , 228 , 'Massachusetts' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (21 , 'MD' , 228 , 'Maryland' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (22 , 'ME' , 228 , 'Maine' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (23 , 'MI' , 228 , 'Michigan' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (24 , 'MN' , 228 , 'Minnesota' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (25 , 'MO' , 228 , 'Missouri' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (26 , 'MS' , 228 , 'Mississippi' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (27 , 'MT' , 228 , 'Montana' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (28 , 'NC' , 228 , 'North Carolina' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (29 , 'ND' , 228 , 'North Dakota' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (30 , 'NE' , 228 , 'Nebraska' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (31 , 'NH' , 228 , 'New Hampshire' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (32 , 'NJ' , 228 , 'New Jersey' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (33 , 'NM' , 228 , 'New Mexico' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (34 , 'NV' , 228 , 'Nevada' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (35 , 'NY' , 228 , 'New York' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (36 , 'OH' , 228 , 'Ohio' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (37 , 'OK' , 228 , 'Oklahoma' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (38 , 'OR' , 228 , 'Oregon' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (39 , 'PA' , 228 , 'Pennsylvania' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (40 , 'RI' , 228 , 'Rhode Island' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (41 , 'SC' , 228 , 'South Carolina' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (42 , 'SD' , 228 , 'South Dakota' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (43 , 'TN' , 228 , 'Tennessee' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (44 , 'TX' , 228 , 'Texas' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (45 , 'UT' , 228 , 'Utah' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (46 , 'VA' , 228 , 'Virginia' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (47 , 'VT' , 228 , 'Vermont' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (48 , 'WA' , 228 , 'Washington' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (49 , 'WI' , 228 , 'Wisconsin' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (50 , 'WV' , 228 , 'West Virginia' , 1 ) ;
INSERT INTO va_states (state_id,state_code,country_id,state_name,show_for_user) VALUES (51 , 'WY' , 228 , 'Wyoming' , 1 ) ;
















INSERT INTO va_support_departments (dep_id,short_title,full_title,signature,attachments_dir,attachments_mask,incoming_account,incoming_type_id,incoming_product_id,outgoing_account,show_for_user,sites_all) VALUES (1 , 'SUPPORT_SHORT_TITLE' , 'SUPPORT_TITLE' , NULL , NULL , NULL , 'support@your_site.com' , 2 , NULL , 'support@your_site.com' , 1 , 1 ) ;










INSERT INTO va_support_priorities (priority_id,priority_name,priority_rank,admin_html,is_default) VALUES (1 , 'PRIORITY_HIGH_MSG' , 1 , '<img src="../images/high.gif" width="8" height="12" border="0">' , 0 ) ;
INSERT INTO va_support_priorities (priority_id,priority_name,priority_rank,admin_html,is_default) VALUES (2 , 'PRIORITY_NORMAL_MSG' , 2 , NULL , 1 ) ;
INSERT INTO va_support_priorities (priority_id,priority_name,priority_rank,admin_html,is_default) VALUES (3 , 'PRIORITY_LOW_MSG' , 3 , '<img src="../images/low.gif" width="8" height="12" border="0">' , 0 ) ;








INSERT INTO va_support_statuses (status_id,status_name,show_for_user,status_caption,status_icon,admin_ticket_class,user_ticket_class,html_start,html_end,is_user_new,is_user_reply,is_admin_reply,is_operation,is_reassign,is_internal,is_list,is_closed,is_add_knowledge,admin_notify,admin_to,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_subject,admin_body,manager_by_notify,manager_by_to,manager_by_from,manager_by_cc,manager_by_bcc,manager_by_reply_to,manager_by_return_path,manager_by_mail_type,manager_by_subject,manager_by_body,manager_to_notify,manager_to_to,manager_to_from,manager_to_cc,manager_to_bcc,manager_to_reply_to,manager_to_return_path,manager_to_mail_type,manager_to_subject,manager_to_body,user_notify,user_to,user_from,user_cc,user_bcc,user_reply_to,user_return_path,user_mail_type,user_subject,user_body) VALUES (1 , 'TICKET_NEW_MSG' , 0 , 'TICKET_NEW_MSG' , NULL , NULL , NULL , NULL , NULL , 1 , 0 , 0 , 0 , 0 , 0 , 1 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_support_statuses (status_id,status_name,show_for_user,status_caption,status_icon,admin_ticket_class,user_ticket_class,html_start,html_end,is_user_new,is_user_reply,is_admin_reply,is_operation,is_reassign,is_internal,is_list,is_closed,is_add_knowledge,admin_notify,admin_to,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_subject,admin_body,manager_by_notify,manager_by_to,manager_by_from,manager_by_cc,manager_by_bcc,manager_by_reply_to,manager_by_return_path,manager_by_mail_type,manager_by_subject,manager_by_body,manager_to_notify,manager_to_to,manager_to_from,manager_to_cc,manager_to_bcc,manager_to_reply_to,manager_to_return_path,manager_to_mail_type,manager_to_subject,manager_to_body,user_notify,user_to,user_from,user_cc,user_bcc,user_reply_to,user_return_path,user_mail_type,user_subject,user_body) VALUES (2 , 'TICKET_ANSWERED_MSG' , 0 , 'TICKET_ANSWERED_MSG' , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 1 , 1 , 0 , 0 , 1 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_support_statuses (status_id,status_name,show_for_user,status_caption,status_icon,admin_ticket_class,user_ticket_class,html_start,html_end,is_user_new,is_user_reply,is_admin_reply,is_operation,is_reassign,is_internal,is_list,is_closed,is_add_knowledge,admin_notify,admin_to,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_subject,admin_body,manager_by_notify,manager_by_to,manager_by_from,manager_by_cc,manager_by_bcc,manager_by_reply_to,manager_by_return_path,manager_by_mail_type,manager_by_subject,manager_by_body,manager_to_notify,manager_to_to,manager_to_from,manager_to_cc,manager_to_bcc,manager_to_reply_to,manager_to_return_path,manager_to_mail_type,manager_to_subject,manager_to_body,user_notify,user_to,user_from,user_cc,user_bcc,user_reply_to,user_return_path,user_mail_type,user_subject,user_body) VALUES (4 , 'TICKET_REQUEST_INFO_MSG' , 0 , 'TICKET_REQUEST_INFO_MSG' , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 1 , 0 , 0 , 1 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_support_statuses (status_id,status_name,show_for_user,status_caption,status_icon,admin_ticket_class,user_ticket_class,html_start,html_end,is_user_new,is_user_reply,is_admin_reply,is_operation,is_reassign,is_internal,is_list,is_closed,is_add_knowledge,admin_notify,admin_to,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_subject,admin_body,manager_by_notify,manager_by_to,manager_by_from,manager_by_cc,manager_by_bcc,manager_by_reply_to,manager_by_return_path,manager_by_mail_type,manager_by_subject,manager_by_body,manager_to_notify,manager_to_to,manager_to_from,manager_to_cc,manager_to_bcc,manager_to_reply_to,manager_to_return_path,manager_to_mail_type,manager_to_subject,manager_to_body,user_notify,user_to,user_from,user_cc,user_bcc,user_reply_to,user_return_path,user_mail_type,user_subject,user_body) VALUES (5 , 'TICKET_INVESTIGATING_MSG' , 0 , 'TICKET_INVESTIGATING_MSG' , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 1 , 0 , 0 , 1 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_support_statuses (status_id,status_name,show_for_user,status_caption,status_icon,admin_ticket_class,user_ticket_class,html_start,html_end,is_user_new,is_user_reply,is_admin_reply,is_operation,is_reassign,is_internal,is_list,is_closed,is_add_knowledge,admin_notify,admin_to,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_subject,admin_body,manager_by_notify,manager_by_to,manager_by_from,manager_by_cc,manager_by_bcc,manager_by_reply_to,manager_by_return_path,manager_by_mail_type,manager_by_subject,manager_by_body,manager_to_notify,manager_to_to,manager_to_from,manager_to_cc,manager_to_bcc,manager_to_reply_to,manager_to_return_path,manager_to_mail_type,manager_to_subject,manager_to_body,user_notify,user_to,user_from,user_cc,user_bcc,user_reply_to,user_return_path,user_mail_type,user_subject,user_body) VALUES (6 , 'TICKET_CLOSED_MSG' , 0 , 'TICKET_CLOSED_MSG' , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 1 , 0 , 0 , 0 , 1 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_support_statuses (status_id,status_name,show_for_user,status_caption,status_icon,admin_ticket_class,user_ticket_class,html_start,html_end,is_user_new,is_user_reply,is_admin_reply,is_operation,is_reassign,is_internal,is_list,is_closed,is_add_knowledge,admin_notify,admin_to,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_subject,admin_body,manager_by_notify,manager_by_to,manager_by_from,manager_by_cc,manager_by_bcc,manager_by_reply_to,manager_by_return_path,manager_by_mail_type,manager_by_subject,manager_by_body,manager_to_notify,manager_to_to,manager_to_from,manager_to_cc,manager_to_bcc,manager_to_reply_to,manager_to_return_path,manager_to_mail_type,manager_to_subject,manager_to_body,user_notify,user_to,user_from,user_cc,user_bcc,user_reply_to,user_return_path,user_mail_type,user_subject,user_body) VALUES (7 , 'TICKET_ASSIGNED_MANAGER_MSG' , 0 , 'TICKET_ASSIGNED_MANAGER_MSG' , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 1 , 1 , 1 , 1 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_support_statuses (status_id,status_name,show_for_user,status_caption,status_icon,admin_ticket_class,user_ticket_class,html_start,html_end,is_user_new,is_user_reply,is_admin_reply,is_operation,is_reassign,is_internal,is_list,is_closed,is_add_knowledge,admin_notify,admin_to,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_subject,admin_body,manager_by_notify,manager_by_to,manager_by_from,manager_by_cc,manager_by_bcc,manager_by_reply_to,manager_by_return_path,manager_by_mail_type,manager_by_subject,manager_by_body,manager_to_notify,manager_to_to,manager_to_from,manager_to_cc,manager_to_bcc,manager_to_reply_to,manager_to_return_path,manager_to_mail_type,manager_to_subject,manager_to_body,user_notify,user_to,user_from,user_cc,user_bcc,user_reply_to,user_return_path,user_mail_type,user_subject,user_body) VALUES (8 , 'TICKET_CUSOMER_REPLY_MSG' , 0 , 'TICKET_CUSOMER_REPLY_MSG' , NULL , NULL , NULL , NULL , NULL , 0 , 1 , 0 , 0 , 0 , 0 , 1 , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;
INSERT INTO va_support_statuses (status_id,status_name,show_for_user,status_caption,status_icon,admin_ticket_class,user_ticket_class,html_start,html_end,is_user_new,is_user_reply,is_admin_reply,is_operation,is_reassign,is_internal,is_list,is_closed,is_add_knowledge,admin_notify,admin_to,admin_from,admin_cc,admin_bcc,admin_reply_to,admin_return_path,admin_mail_type,admin_subject,admin_body,manager_by_notify,manager_by_to,manager_by_from,manager_by_cc,manager_by_bcc,manager_by_reply_to,manager_by_return_path,manager_by_mail_type,manager_by_subject,manager_by_body,manager_to_notify,manager_to_to,manager_to_from,manager_to_cc,manager_to_bcc,manager_to_reply_to,manager_to_return_path,manager_to_mail_type,manager_to_subject,manager_to_body,user_notify,user_to,user_from,user_cc,user_bcc,user_reply_to,user_return_path,user_mail_type,user_subject,user_body) VALUES (9 , 'TICKET_ADDED_KB_MSG' , 0 , 'TICKET_ADDED_KB_MSG' , NULL , NULL , NULL , NULL , NULL , 0 , 0 , 0 , 0 , 0 , 1 , 0 , 0 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;




INSERT INTO va_support_types (type_id,type_name,show_for_user) VALUES (1 , 'TICKET_BUG_ISSUE_MSG' , 1 ) ;
INSERT INTO va_support_types (type_id,type_name,show_for_user) VALUES (2 , 'TICKET_QUESTION_MSG' , 1 ) ;
INSERT INTO va_support_types (type_id,type_name,show_for_user) VALUES (3 , 'TICKET_WISH_REQUEST_MSG' , 1 ) ;
INSERT INTO va_support_types (type_id,type_name,show_for_user) VALUES (4 , 'TICKET_OTHER_MSG' , 1 ) ;
INSERT INTO va_support_types (type_id,type_name,show_for_user) VALUES (5 , 'TICKET_QUOTE_MSG' , 1 ) ;


INSERT INTO va_support_users_departments (admin_id,dep_id,is_default_dep) VALUES (1 , 1 , 1 ) ;
















INSERT INTO va_user_profile_sections (section_id,is_active,step_number,section_order,section_code,section_name,user_types_all) VALUES (1 , 1 , 1 , 1 , 'login' , 'LOGIN_INFO_MSG' , 1 ) ;
INSERT INTO va_user_profile_sections (section_id,is_active,step_number,section_order,section_code,section_name,user_types_all) VALUES (2 , 1 , 1 , 2 , 'personal' , 'PERSONAL_DETAILS_MSG' , 1 ) ;
INSERT INTO va_user_profile_sections (section_id,is_active,step_number,section_order,section_code,section_name,user_types_all) VALUES (3 , 1 , 1 , 3 , 'delivery' , 'DELIVERY_DETAILS_MSG' , 1 ) ;
INSERT INTO va_user_profile_sections (section_id,is_active,step_number,section_order,section_code,section_name,user_types_all) VALUES (4 , 1 , 1 , 4 , 'additional' , 'ADDITIONAL_DETAILS_MSG' , 1 ) ;






INSERT INTO va_user_types (type_id,type_name,is_default,is_active,is_sms_allowed,sites_all,countries_all,show_for_user,price_type,discount_type,discount_amount,reward_type,reward_amount,credit_reward_type,credit_reward_amount,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,order_min_goods_cost,order_max_goods_cost,tax_free,coupons_ids,is_subscription,is_subscription_recurring,subscription_name,subscription_fee,subscription_period,subscription_interval,subscription_suspend,subscription_points_type,subscription_points_amount,subscription_credits_type,subscription_credits_amount,subscription_affiliate_type,subscription_affiliate_amount) VALUES (1 , 'CUSTOMER_TYPE_MSG' , 1 , 1 , 1 , 1 , 1 , 1 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , 0 , 0 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL ) ;




INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'access_product_registration' , '0' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'access_products' , '0' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'add_ad' , '1' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'add_product' , '0' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'affiliate_join' , '0' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'affiliate_sales' , '0' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'approve_ad' , '0' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'approve_product' , '0' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'approve_product_registration' , '0' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'approve_profile' , '1' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'cancel_subscription' , NULL ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'delete_ad' , '1' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'delete_product' , '0' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'edit_ad' , '1' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'edit_pd' , '0' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'edit_product' , '0' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'edit_profile' , '1' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'merchant_order_cc_cvv2' , '0' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'merchant_order_cc_number' , '0' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'merchant_order_payment_details' , '0' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'merchant_orders' , '0' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'merchant_sales' , '0' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'my_carts' , '1' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'my_details' , '1' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'my_forum' , '1' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'my_orders' , '1' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'my_payments' , '0' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'my_product_registrations' , '0' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'my_support' , '1' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'my_wishlist' , '1' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'new_profile' , '1' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'reminder_service' , '1' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'upgrade_downgrade' , '0' ) ;
INSERT INTO va_user_types_settings (type_id,setting_name,setting_value) VALUES (1 , 'user_order_update_pd' , '0' ) ;




INSERT INTO va_users (user_id,user_type_id,subscription_id,layout_id,is_approved,is_hidden,is_unsubscribed,expiry_date,suspend_date,registration_last_step,registration_total_steps,is_sms_allowed,credit_balance,total_points,discount_type,discount_amount,reward_type,reward_amount,credit_reward_type,credit_reward_amount,affiliate_commission_type,affiliate_commission_amount,merchant_fee_type,merchant_fee_amount,tax_free,order_min_goods_cost,order_max_goods_cost,coupons_ids,login,password,nickname,affiliate_code,security_question,security_answer,personal_image,short_description,full_description,friendly_url,name,first_name,last_name,company_id,company_name,email,address1,address2,city,province,state_id,state_code,zip,country_id,country_code,phone,daytime_phone,evening_phone,cell_phone,fax,delivery_name,delivery_first_name,delivery_last_name,delivery_company_id,delivery_company_name,delivery_email,delivery_address1,delivery_address2,delivery_city,delivery_province,delivery_state_id,delivery_state_code,delivery_zip,delivery_country_id,delivery_country_code,delivery_phone,delivery_daytime_phone,delivery_evening_phone,delivery_cell_phone,delivery_fax,bill_encrypt_type,bill_name,bill_first_name,bill_last_name,bill_company_id,bill_company_name,bill_email,bill_address1,bill_address2,bill_city,bill_province,bill_state_id,bill_zip,bill_country_id,bill_phone,bill_daytime_phone,bill_evening_phone,bill_cell_phone,bill_fax,bill_cc_number,bill_cc_start_date,bill_cc_expiry_date,bill_cc_type,bill_cc_issue_number,bill_cc_cvv2,paypal_account,msn_account,icq_number,user_site_url,tax_id,birth_year,birth_month,birth_day,user_notes,admin_notes,registration_ip,registration_date,modified_ip,modified_date,admin_modified_ip,admin_modified_date,last_logged_ip,last_logged_date,last_visit_ip,last_visit_page,last_visit_date,reset_password_code,reset_password_date) VALUES (1 , 1 , NULL , NULL , 1 , NULL , NULL , NULL , NULL , 1 , 1 , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , 0 , NULL , NULL , NULL , 'joy' , 'joy' , '' , '' , '' , '' , NULL , NULL , NULL , '' , 'joy' , '' , '' , NULL , NULL , 'jnothing@example.com' , '5-th Avenue' , NULL , 'New York' , NULL , 35 , 'NY' , '10002' , 228 , 'US' , NULL , '1234567890' , '1234567890' , NULL , NULL , 'joy' , NULL , NULL , NULL , NULL , NULL , '5-th Avenue' , NULL , 'New York' , NULL , 35 , 'NY' , '10002' , 228 , 'US' , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , '2004-06-22 20:03:44' , '127.0.0.1' , '2008-06-03 19:14:24' , NULL , NULL , '127.0.0.1' , '2015-11-24 03:14:10' , '127.0.0.1' , '/c2s/admin/admin_articles_top.php' , '2015-11-24 04:11:38' , NULL , NULL ) ;





















